#ifndef APV25I2CCHANNELDUMMY_H
#define APV25I2CCHANNELDUMMY_H
#include "fecdescription.h"
#include "ccudescription.h"
#include "apv25description.h"
#include "apv25i2cchannelinterface.h"
#include "simulationexchange.h"
/**
   \class Apv25I2cChannelDummy
    \author V.Z
    \date 10/2001
    \version 1.0
    \brief Dummy channel implementations

    Set some of the i2c values forthe simulatorexchange class
    see setCsel for special remark
    so the cal prof can be restored by the fedchannel dummy class
   

 */
class Apv25I2cChannelDummy : public Apv25I2cChannelInterface
{
private:
  pid_t fPid;
  bool fLoadAll;
  bool fInitialized;
  SimulationExchange* SimExch;
  Apv25Description* fApvDescription;
  FecDescription* fFecDescription;
  int AttachChannel();
  bool fReadOnly;
public:  
  /**
     Constructor. Correct initialisation should be checked with isInitialized()
     @param tdname  Name of the TDeviceControl, controlling the Daq
     @param board   Board id as registered in TDeviceControl
     @param channel Channel number as registered in TDeviceControl
  */
  
  Apv25I2cChannelDummy(char *tdname,int board,int channel);
  virtual ~Apv25I2cChannelDummy();///< Destructor. Release channel
  /** True if the channel was found in the Daq*/
  inline bool isInitialized() { return fInitialized;}
  /** Apv25Description callbacks */
  virtual void setStatus(short i) {;}
  virtual void setAddress(short i) {;}
  virtual void setMode(short i) {SimExch->Ad.setMode(i);}
  virtual void setLatency(short i) {  SimExch->Ad.setLatency(i);}
  virtual void setMuxgain(short i) {;}
  virtual void setIpre(short i) {;}
  virtual void setIpcasc(short i) {;}
  virtual void setIpsf(short i) {;}
  virtual void setIsha(short i) {;}
  virtual void setIssf(short i) {;}
  virtual void setIpsp(short i) {;}
  virtual void setImuxin(short i) {;}
  virtual void setIcal(short i) {SimExch->Ad.setIcal(i);}
  virtual void setVfp(short i) {;}
  virtual void setVfs(short i) {;}
  virtual void setVpsp(short i) {;}
  virtual void setCdrv(short i) {SimExch->Ad.setCdrv(i);}
  
/** 
    the csel store the delay number and not as bit representation
    (that is normally send to the apv.)
    This is done because the conversion from bit representation
    to delay number takes a while. 
    @param i  the setting to Csel (bit representation)
*/
  virtual void setCsel(short i) {
    short delay;
    unsigned char a=~(unsigned char) i;
    switch (a){
      case 128 : delay= 7;break;
      case 64  : delay= 6;break;
      case 32  : delay= 5;break;
      case 16  : delay= 4;break;
      case 8   : delay= 3;break;
      case 4   : delay= 2;break;
      case 2   : delay= 1;break;
      case 1   : delay= 0;break;
      default : delay=8; break; // not possible 
    };
    SimExch->Ad.setCsel(delay);}
  virtual void setLoadAll(bool t) {fLoadAll=t ;}
  virtual short getStatus() {return 1;}
  virtual short getAddress() {return 1;}
  virtual short getMode() {return SimExch->Ad.getMode() ;}
  virtual short getLatency() {return SimExch->Ad.getLatency();}
  virtual short getMuxgain() {return 1;}
  virtual short getIpre() {return 1;}
  virtual short getIpcasc() {return 1;}
  virtual short getIpsf() {return 1;}
  virtual short getIsha() {return 1;}
  virtual short getIssf() {return 1;}
  virtual short getIpsp() {return 1;}
  virtual short getImuxin() {return 1;}
  virtual short getIcal() {return 1;}
  virtual short getVfp() {return 1;}
  virtual short getVfs() {return 1;}
  virtual short getVpsp() {return 1;}
  virtual short getCdrv() {return SimExch->Ad.getCdrv();}
  virtual short getCsel() {
    unsigned char a=0 ;
    switch (SimExch->Ad.getCsel()){
      case 7: a=128;break;
      case 6: a=64;break;
      case 5: a=32;break;
      case 4: a=16;break;
      case 3: a=8;break;
      case 2: a=4;break;
      case 1: a=2;break;
      case 0: a=1;break;
      default : a=0; break; // not possible
    };
    return ~a;
}
   virtual short getErr(){ return 0;}
  virtual bool  getLoadAll(){return fLoadAll;};
  virtual bool getReadOnly(){ return fReadOnly;}
  /** Share memory lock */
  virtual void  lock() {;}
  /** Share memory unlock */
  virtual void  unlock() {;}
  /** Share memory detach, Daq release of the channel */
  virtual void  release();
  /** DaqAction callback to requestAction + signal of server */
  virtual void  justDoIt();
  
 virtual int  wait();
};
#endif
