#ifndef APV25I2CCHANNELLT_H
#define APV25I2CCHANNELLT_H

#include "apv25i2cchannelinterface.h"
/**
   \class Apv25I2cChannelLt
    \author W. Beaumont 
    \date September  2001
    \version 1.0
    \brief Lt  implementation of Apv25I2cChannelInterface

    <h2> Usage </h2>
    It implements Apv25I2cChannelInterface class for the lt application. 
    It takes care about the correct instantiation
    of the Apv25Channel (ccu address, Fec board, Ccu channel ) according the connection
    to the bench position as described in the lt.xml file. This file should be available.
    
   

 */
class Apv25I2cChannelLt : public Apv25I2cChannelInterface
{
private:
  //pid_t fPid;

  Apv25I2cChannelInterface* fCd;
  bool fInitialised; 
  int fChannel;
  int fBoard;
public:  
  /**
     Constructor. Correct initialisation should be checked with isInitialized()
     @param  benchposition   the benchposition where the module is .
    */
  
  Apv25I2cChannelLt(int benchposition );
  virtual ~Apv25I2cChannelLt();///< Destructor. Release channel
  /** True if the channel was found in the Daq*/
  inline bool isInitialized() { return fInitialised ;}
  /** Apv25Description callbacks */
  virtual void setReadOnly(bool t){fCd->setReadOnly(t);}
  virtual void setStatus(short i) {fCd->setStatus(i);}
  virtual void setAddress(short i) {fCd->setAddress(i);}
  virtual void setMode(short i) {fCd->setMode(i);}
  virtual void setLatency(short i) {fCd->setLatency(i);}
  virtual void setMuxgain(short i) {fCd->setMuxgain(i);}
  virtual void setIpre(short i) {fCd->setIpre(i);}
  virtual void setIpcasc(short i) {fCd->setIpcasc(i);}
  virtual void setIpsf(short i) {fCd->setIpsf(i);}
  virtual void setIsha(short i) {fCd->setIsha(i);}
  virtual void setIssf(short i) {fCd->setIssf(i);}
  virtual void setIpsp(short i) {fCd->setIpsp(i);}
  virtual void setImuxin(short i) {fCd->setImuxin(i);}
  virtual void setIcal(short i) {fCd->setIcal(i);}
  virtual void setVfp(short i) {fCd->setVfp(i);}
  virtual void setVfs(short i) {fCd->setVfs(i);}
  virtual void setVpsp(short i) {fCd->setVpsp(i);}
  virtual void setCdrv(short i) {fCd->setCdrv(i);}
  virtual void setCsel(short i) {fCd->setCsel(i);}
  virtual void setLoadAll(bool t) {fCd->setLoadAll(t) ;}
  virtual short getStatus() {return fCd->getStatus();}
  virtual short getAddress() {return fCd->getAddress();}
  virtual short getMode() {return fCd->getMode();}
  virtual short getLatency() {return fCd->getLatency();}
  virtual short getMuxgain() {return fCd->getMuxgain();}
  virtual short getIpre() {return fCd->getIpre();}
  virtual short getIpcasc() {return fCd->getIpcasc();}
  virtual short getIpsf() {return fCd->getIpsf();}
  virtual short getIsha() {return fCd->getIsha();}
  virtual short getIssf() {return fCd->getIssf();}
  virtual short getIpsp() {return fCd->getIpsp();}
  virtual short getImuxin() {return fCd->getImuxin();}
  virtual short getIcal() {return fCd->getIcal();}
  virtual short getVfp() {return fCd->getVfp();}
  virtual short getVfs() {return fCd->getVfs();}
  virtual short getVpsp() {return fCd->getVpsp();}
  virtual short getCdrv() {return fCd->getCdrv();}
  virtual short getCsel() {return fCd->getCsel();}
  virtual short getErr(){ return fCd->getErr() ;}

  virtual bool getReadOnly(){return fCd-> getReadOnly();}

  virtual bool  getLoadAll(){return  fCd->getLoadAll();}
  /** Share memory lock */
  virtual void  lock() {fCd->lock();}
  /** Share memory unlock */
  virtual void  unlock() {fCd->unlock();}
  /** Share memory detach, Daq release of the channel */
  virtual void  release(){fCd->release();}
  /** DaqAction callback to requestAction + signal of server */
  virtual void  justDoIt(){fCd->justDoIt();}
  /** poll routine */
  virtual int wait(){return fCd->wait();}
  
  

};
#endif
