#ifndef Apv_Smulator_h_
#define  Apv_Simulator_h_

#include <vector>
#include <algorithm>
#include <valarray>
#include "simulationexchange.h"
/**
   \class 
   \authors: V.Z
   \date 10/08/2001
   \version 1.0
   \brief Simulate APV frame 
   <p> provide the apv 128 ch frame with rms,cmn, noisy/disconnected 
       channels and hits


*/


typedef std::valarray<double>  Fvec;
typedef std::valarray<int>    Ivec;

class Apv_Simulator{
  protected :
    //  SimulationExchage* Se;

public:

  Apv_Simulator(float avped, float avrms, float avcmn, float cmnslope, bool bbadch, bool bbclus);
  ~Apv_Simulator();
  Ivec getframe() {return theFrame;}  // frame 512
  Ivec getpedestal() {return thePedestal;} // pedestal 128 
  Ivec  getdata() {return theData;}
  Ivec  getdatancmn() {return theDatancmn;}
  void setnoisych(int n) {theNoisyList.push_back(n);}  //set noisy channel
  void setquietch(int n) {theQuietList.push_back(n);}  // set quiet channel
  void clearbadch();  // clear noisy and quiet channels
  void setdefaultbadch();  // set default 5 noisy and 5 quiet
   void setclustersOn()  {bclusters=true;}  // 
  void setclustersOff()  {bclusters=false;}  // 
  void Simulate();
private:
  Ivec addrms();  // add rms
  Ivec addcmn(const Ivec& data);  //add cmn
  Ivec addclus(const Ivec& data);  //add clusters
  Ivec addframe(const Ivec& data); //add frame
private:
  float theavPed;
  float theavRms;
  float theavCmn;
  float theCmnslope;
  float theNoisyFactor;
  float theQuietFactor;

  Ivec thePedestal;
  Ivec theData; 
  Ivec theDatancmn;
  Ivec theFrame;
  float  theCmn;
  std::vector<int> theNoisyList;
  std::vector<int> theQuietList;
  bool bdefaultbadch;
  bool bclusters;


};
#endif
