#ifndef DCUCHANNELLT
#define DCUCHANNELLT
#include <time.h>
#include "dcuchannelinterface.h"
/**
   \class DcuChannelLt
    \author W. Beaumont V.Z
    \date 10/ 2001
    \version 1.0
    \brief Lt  implementation of DcuChannelInterface

    <h2> Usage </h2>
    It implements DcuChannelInterface class for the lt application. 
    It takes care about the correct instantiation
    of the dcuhannel (ccu address, Fec board, Ccu channel ) according the connection
    to the bench position as described in the lt.xml file
    
    

 */
class DcuChannelLt : public DcuChannelInterface
{
 
private:
 

   DcuChannelInterface* fMci;
 
  bool fInitialized;
  int fChannel;
  int fBoard;
  
  int AttachChannel();



public:
  /**
     Constructor. Correct initialisation should be checked with isInitialized()
     @param  benchposition   the benchposition where the module is .
     
  */
  
  DcuChannelLt(int benchposition );
  virtual ~DcuChannelLt();
  /** True if the channel was found in the Daq*/
  bool isInitialized() { return fInitialized;}

  /** MuxDescription callbacks */

  virtual void update(){ fMci->update();}
  virtual void setAddress(short i) {fMci->setAddress(i);}
  virtual short getAddress() { return fMci->getAddress();}
  virtual void setActiveChannel(unsigned char ch) {fMci->setActiveChannel(ch);}
  virtual unsigned char getActiveChannel(){return fMci->getActiveChannel();}
  virtual time_t getTime() {return  fMci->getTime(); }
  virtual unsigned short getDcuADC(unsigned char ch){ return fMci->getDcuADC(ch);}
  virtual unsigned int getDcuHardId(){ return fMci->getDcuHardId();}
  /** Share memory lock */
  virtual void  lock() {fMci->lock();}
  /** Share memory unlock */
  virtual void  unlock() {fMci->unlock();}
  /** Share memory and Daq release */
  virtual void  release(){ fMci->release();}
  /** DaqAction callback to requestAction + signal of server */
  // virtual void  justDoIt() {fMci->justDoIt();};
 /** poll routine */
   virtual int wait(){return fMci->wait();}
  
  // additional functions not defined in dcuchannelinterface 

  
  float getDcuV125( );
  float getDcuV250();
  float getDcuTempSi(); 
  float getDcuTempHyb();
  float getDcuTempInt();
  float getDcuTemp();
  float getDcuBiasCurrent();


  
};



#endif
