#ifndef FED_CHANNELDIMDEV
#define FED_CHANNELDIMDEV

#include "fedchannelinterface.h"
#include "feddevicedim.h"
#include "channelid.h"
#include "daqerrormsg.h"

/** \class FedChannelDimDev
    \author W.Beaumont
    \date December 2002 
    \date March 2002
    \version 1.0
    

    \brief FedDimDev implementation of FedChannelInterface

    It implements FedChannelInterface class and make the Daq 
    transparent to the user   
    this connects the dimdev to the channel and selects the
    correct data. Only supports Fed in frame finding  
*/
class FedChannelDimDev  :protected FedDeviceDim ,
			  public FedChannelInterface {

  private :
    int fFrameSize;
    int sizeNeeded;
    int fOdd ;// if 1 the odd sampel (second ) else even sample
  PROCESS_MODE fProcessMode;
  int fChannel;
  int fApvChannelIdMode;
  
 public:
  /** Constructor
      @param fed_ser_nr the fed serial number determins which fed will be used.
      @param channel_arg wich channel of the fed will be used
      @param process_mode_arg PROCESS_MODE see FedChannel interface
      @param mux_channel_arg the mux channel (even /odd ) the APV will be connected to, -1 = no Mux
      @param time_channel_arg  the petal multiplexer time slot the channel is connected to (not used) 
  */
  FedChannelDimDev(long fed_ser_nr, long channel_arg, PROCESS_MODE process_mode_arg, 
		  long  mux_channel_arg , long  time_channel_arg);
 
  
  virtual ~FedChannelDimDev();
protected:
 


public:
  /** Copy Fed Data to user buffer 
      @param event_nr  Event number in the spill
      @param out_buf Output buffer
      @param size_of_buf Size of output buffer
   */     
  virtual int getApvEventData(int event_nr,short *out_buf, int size_of_buf );
    
  /** Return Sample i from Event event_nr */
  virtual short getApvEventSample(int i,int  event_nr) ;
   
  /** Return Sample i from the whole buffer 
      NOT IMPLEMENTED FOR THE MOMENT
   */
  virtual short getSampleFromBuffer(int i);
  /** Unused */	  
  int  setBufferReadmodeSetup( PROCESS_MODE  process_mode_arg){
    //fProcessMode= process_mode_arg;
    DT_WARN("NOT SUPPORTED");
    return fProcessMode;  
  }
  /** Get Number of event in Spill (buffer) */
  virtual int getNumberOfEvents(){return 
   fDimFedDev->getNumberOfEvents();
  }
  /** Get Time of last event*/
  virtual time_t getTime(){ return
    fDimFedDev->getTime(fDimFedDev->getNumberOfEvents());
  }
  /** Get Milli Time of last event */
  virtual short getMilliTime(){return 
    fDimFedDev->getMilliTime(fDimFedDev->getNumberOfEvents());
  }


  /** Get Spill number (buffer counts) */
  virtual int getSpillNumber(){
   return fDimFedDev->getSpillNumber() ;
  }
  /** Lock share memory access */
  virtual void  lockDataBuffer(){fDimFedDev->lockDataBuffer() ;}
  /** Unlock share memory access */
  virtual void  unlockDataBuffer(){fDimFedDev->unlockDataBuffer();}
  /** detach share memory access and release daq channel */
  virtual void  release(){fDimFedDev->release();}

};
#endif





