#ifndef FED_CHANNELDUMMY
#define FED_CHANNELDUMMY
#include "fedchannelinterface.h"
#include "channelid.h"
#include "apvsimulator.h"
#include "simulationexchange.h"


/** \class FedChannelDummy
    \author W.Beaumont
    \date March 2001
    \date March 2002
    \version 1.0
    \version 1.1

    \brief FedDummy implementation of FedChannelInterface

    <h2> Usage </h2>
    It implements FedChannelInterface class and make the Daq transparent to 
    the user:
    <ul>
    <li>this is a dummy chanel to test higher level programs without the (HW) interface
    <li> it use the apvsim class to generate apv frames (since version 1.1)
        
    
    </ul>
 
*/
class FedChannelDummy  : public FedChannelInterface {
 protected:
  SimulationExchange* Se;
private :

  Apv_Simulator *ssim;

  float avped;
  float avrms;
  float avcmn;
  float avcmsl;
  float bbad;

  int sizeNeeded;
  PROCESS_MODE fProcessMode;
 
  int fSpill;
  /** this generates the apv channel nr into the data so 
      the channel nr can be indentiyfy. 
      Beocmes active in the environment variable 
      APVCHANNELID = strip with "hit (+80)
  */
  int fApvChannelIdMode;
public:
  /** Constructor
      @param fed_ser_nr the fed serial number determins which fed will be used.
      @param channel_arg wich channel of the fed will be used
      @param process_mode_arg PROCESS_MODE see FedChannel interface
      @param mux_channel_arg the mux channel (even /odd ) the APV will be connected to, -1 = no Mux
      @param time_channel_arg  the petal multiplexer time slot the channel is connected to (not used) 
  */
  FedChannelDummy(long fed_ser_nr, long channel_arg, PROCESS_MODE process_mode_arg, 
		  long  mux_channel_arg , long  time_channel_arg);
 
  
  virtual ~FedChannelDummy();
protected:
 


public:
  /** Copy Fed Data to user buffer 
      @param event_nr  Event number in the spill
      @param out_buf Output buffer
      @param size_of_buf Size of output buffer
   */     
  virtual int getApvEventData(int event_nr,short *out_buf, int size_of_buf );
    
  /** Return Sample i from Event event_nr */
  virtual short getApvEventSample(int i,int  event_nr) 
    {fSpill++; 
                   return (short int) (i+ 1024*event_nr);
    }
  /** Return Sample i from the whole buffer */
  virtual short getSampleFromBuffer(int i){fSpill++; return (short int) i;}
  /** Unused */	  
  int  setBufferReadmodeSetup( PROCESS_MODE  process_mode_arg){return fProcessMode;  }
  /** Get Number of event in Spill (buffer) */
  virtual int getNumberOfEvents(){ return 50; }
  /** Get Time */
  virtual time_t getTime(){ return  0;}
  /** Get Milli Time */
  virtual short getMilliTime(){return 0;}  


  /** Get Spill number (buffer counts) */
  virtual int getSpillNumber(){fSpill++ ;return fSpill;} 
  /** Lock share memory access */
  virtual void  lockDataBuffer() {std::cout<<" locked "<<std::endl;}
  /** Unlock share memory access */
  virtual void  unlockDataBuffer() {;} 
  /** detach share memory access and release daq channel */
  virtual void  release(){;}

};
#endif





