#ifndef __FEDCHANNELLT
#define  __FEDCHANNELLT

#include <fedchannelinterface.h>
#include <iostream>
#include "benchposconnection.h"
/** 
    \class FedChannelLt
    \author W.Beaumont
    \date May  2001
    \version 1.0
     \date  20010923
    \version 1.1

    \brief class to get fed channel related on slot position from the lt config file
    The connection between the APV and the FED depends on the position of th APV on the hybrid
    and the cabling between the VUTRI board and the FED. This is described in the xml file.
    The Fed channel is defined by the serial number of the FED and the channel number.
    The I2C address defines if the APV ouputs the frame in the "first" clock pulse or in the second one.
    Due fact that the I2C address are fixed this information is supperflusious. 
    If the serial number of the FED is greater than 100 the  FedChannelDummy  will be used.
    This dummy fedchannel doesn't use the server-client mechanism. 
    The FedChannelLt doesn't inherit the FedChannelInterface because this is not a abstract  /pure virtual class.
    Also some functions for the mux are not provided by the FedChannelInterface
    
*/

class FedChannelLt //: public FedChannelInterface 
{
private:
  FedChannelInterface*  fFed;
   bool fInitialized;
 
  int fChannel;
  int fSerialNumber; 
  BenchPosConnection::MuxChId fMuxId;
  static int  fLockStatus; // works for the moment only for one FED 
  void FedChannelLtInit (  int hybrpos , int slotpos , int apvI2Caddr , FedChannelInterface::PROCESS_MODE processmode   
                                      //will be defined by the application
  );
 public:
/** Constructor
      \brief contstructor to use in the long term test applicatoin
      The information for the connection is retrieved from the XML file
      The process mode is at the moment hard coded. Later on it will be read
      from a test setting file. The I2C address is not passed to the fedchannel
      instead the hybrid position is given because for the FedChannel it only matters 
      if the I2C address is even or odd. 
      @param  hybrpos the position of the APV on the hybrid
      @param slotpos the slot in which the module is located  
      
*/
  FedChannelLt::FedChannelLt (  int hybrpos , int slotpos );

   /** Constructor
      @param  hybrpos the position of the APV on the hybrid
      @param slotpos the slot in which the module is located  
      @param apvI2caddr the I2C address of the APV to detect if it is odd or even
      @param processmode  the process mode in which the FED is expected to work 
  */   
  FedChannelLt(  int hybrpos , int slotpos , int apvI2caddr , FedChannelInterface::PROCESS_MODE processmode );

  virtual ~FedChannelLt();
   /** Copy Fed Data to user buffer 
      @param event_nr  Event number in the spill
      @param out_buf Output buffer
      @param size_of_buf Size of output buffer
   */   
   virtual int getApvEventData(int event_nr,short *out_buf, int size_of_buf )
     { return fFed->getApvEventData( event_nr,out_buf, size_of_buf );}
 /** Return Sample i from Event event_nr */
  virtual short getApvEventSample(int i,int  event_nr)
    {return   fFed->getApvEventSample( i, event_nr);}
  /** Return Sample i from the whole buffer */
  virtual short getSampleFromBuffer(int i)
  { return  fFed->getSampleFromBuffer( i);}
  /** Unused == not tested */	  
  virtual int  setBufferReadmodeSetup( FedChannelInterface::PROCESS_MODE  process_mode_arglt)
  {return  fFed->setBufferReadmodeSetup(process_mode_arglt);}
  /** Get Number of event in Spill (buffer) */
  virtual int getNumberOfEvents()
    { return fFed-> getNumberOfEvents();}
  /** Get Spill number (buffer counts) */
  virtual int getSpillNumber()
  { return  fFed->getSpillNumber();}

 /** Get Time */
  virtual time_t getTime(){ return  fFed-> getTime();}
  /** Get Milli Time */
  virtual short getMilliTime(){return fFed->getMilliTime();}  

  /** Lock access */
  virtual void  lockDataBuffer();
 
  /** UnLock access */
  virtual void  unlockDataBuffer();

  /** utility (internal) */
   int getSkip() { return fFed->getSkip();}

   /** returns PROCESS_MODE */ 
    FedChannelInterface::PROCESS_MODE  getProcessMode() { return fFed->getProcessMode() ;}
  /** Release access */
  virtual void  release()
 { fFed->release();}
  
  virtual void  info() {std::cout<<" ===works "<<std::endl;}
 
  FedChannelInterface* getFedPtr(){ return fFed;}
 
  bool isInitialized() {return fInitialized;}

  int getGroupNr() { return fMuxId.fMuxGroup;}



};


#endif 
