#ifndef FED_PROCESSCTRL
#define FED_PROCESSCTRL 
#include "TDeviceControl.h"
#include "TShare.h"
#include "fedprocessctrlinterface.h"
#include "feddescription.h"

/** \class FedChannel
    \author W.Beaumont,
    \date September 2001
    \version 1.0

    \brief Fed implementation of FedProcessCtrlInterface

    <h2> Usage </h2>
    It implements FedProcessCtrlInterface class and make the Daq transparent to 
    the user:
    <ul>
    <li> it connects to the Main TDeviceControl and request device 
    <li> it maps the Fed Share memory corresponding part to a FedDescription
    <li> it controls access to this FedDescrition thru methods lockDataBuffer,
    unlockDataBuffer  and release.
    </ul>
    For the moment it will attach the first FED description it will find in the 
    TControl data base. Lateron it should be a control for all the Fed's in the system
    <pre>
    // Request 8 channel 
     
    FedProcessCtrlInterface* fedctrl;
   
    { 
      
    fedctrl= new FedProcessCtrl() ; 
    if (!fedctrl->isInitialized()) exit(-1);
    }
    // Loop
    
    while (event_to_get)
    { while ( !fedctrl->CheckForNewSpill();) {usleep(1000)}
      fedctrl->lockDataBuffer();

      for (int i=0;i<fedctrl-> getNumberOfEvents;i++)
       {
       for (int j=0;j<8;j++)
         {
         fedc[i]->getAPvEventData(i,mybuf,1024);
         ....
         // do what you want with mybuf
         ....
         }
       }
     fedctrl->unlockDataBuffer();
    }

    // Release
        fedctrl->release();
    </pre>
*/
class FedProcessCtrl : public FedProcessCtrlInterface 
{

private :

  int sizeNeeded;
 int fBoardnr; 
  ChannelDescription* fCd;
  pid_t fPid;
  TShare *fMemoryClient;
  FedDescription *fFed;
  char fShmName[256];
  unsigned int fSpillnr;
  int fShmSize;
  int fNrEvents;
  bool fLocked;
 
public:
  /** Constructor
     
  */
  FedProcessCtrl();
  ~FedProcessCtrl();
protected:
  /** Connect to the Daq and check if the device is aviable,
      it turns isInitialized() to true if Ok
  */
  virtual int AttachFEDDevice(char* shname, pid_t cp );


public:
  /** check for new spill return true if spill is DIFFERENT in respect to the last request */
  virtual bool NewSpill();
    /** check for new spill and wait for skip spills 
     @param skip the number of spills that should be skipped 
     */
  virtual bool NewSpill(int skip );

  /** Get Spill number (buffer counts) */
  virtual int getSpillNumber(){return fFed->getSpill();} 

 /** return the board Id of the FED device in use if not known returns -1*/
  virtual int getFedId(){ return fBoardnr;}

 
  /** Lock share memory access */
  virtual void  lockDataBuffer() {fMemoryClient->Lock();}
  /** Unlock share memory access */
  virtual void  unlockDataBuffer() {fMemoryClient->Unlock();} 
  /** detach share memory access and release daq channel */
  virtual void  release();
  /** Get Number of event in Spill (buffer) */
  virtual int getNumberOfEvents(){ return fNrEvents; }
  
  virtual void NextSpill();
};
#endif





