#ifndef __FedProcessCtrlDimDev
#define  __FedProcessCtrlDimDev
#include "fedprocessctrlinterface.h"
#include "feddevicedim.h"

/** 
    \class FedProcessCtrlDimDev
    \author W.Beaumont
    \date September  2001
    \version 1.0

    \brief Abstract all functionnalities of a  FedProcessCtrl
*/

class FedProcessCtrlDimDev : protected FedDeviceDim ,
			     public FedProcessCtrlInterface { 
public:
       /** Constructor
      
  */   
  FedProcessCtrlDimDev  (int fedsrn, string host,vector<int> fbd );

  virtual ~FedProcessCtrlDimDev(){};

 
  /** Get Number of event in Spill (buffer) */
  virtual int getNumberOfEvents(){
    return fDimFedDev->getNinSpill();
  } 
  /** Get Spill number (buffer counts) */
  virtual int getSpillNumber(){
    return fDimFedDev-> getSpillNumber();
  } 
/** check for new spill return true if spill is DIFFERENT in respect to the last request
    will lock the buffer if there is a new spill */  
  virtual bool NewSpill();
 /** check for new spill and wait for skip spills 
     @param skip the number of spills that should be skipped 
     */
  virtual bool NewSpill(int skip );

  /** purge all buffers and returns true if the new data
      is provided. In case of purge is not possible it 
      calls NewSpill(3)
  */
  virtual bool clearFed();
  virtual bool stopFedDaqLoop();

  /** return the board Id of the FED device in use if not implemented returns -1*/  
  virtual int getFedId(){ return getFedSerialNr();}
  /** returns the data pointer to the apv buffer */
  virtual unsigned short
    getApvEventData(int event,int ch,int apv, int nr_sample,int fednr=0);


  /** Lock access */
  virtual void  lockDataBuffer(){;}
  /** UnLock access */
  virtual void  unlockDataBuffer(){;}
  /** Release access 
      doesn't do anything in the dimfeddevice channel
      to be sure for the futer 
   */
  virtual void  release() {fDimFedDev->release();}
  
  /** if the buffer is locked it will unlock the buffer */
  virtual void NextSpill();
  /** will tune the fed timing 
       if delay = -1 the fine tune program will be activated
       else the fed clock delay is set to delay 
   */
  virtual bool TuneFedSampleTime(int& delay ){return setFedDelay(delay);}
  virtual bool AdjustThresholds(int& low, int& high);
  virtual bool AdjustThresholds();
  virtual void  setScopeMode(int j);
  virtual void setSpillSize(int spillsize);
  private:
  // the numbers in the spill
  int fActSpillSize;
  int fSpill;
};

#endif 
	

