#ifndef __FedProcessCtrlInterface
#define  __FedProcessCtrlInterface

/** 
    \class FedProcessCtrlInterface
    \author W.Beaumont
    \date September  2001
    \version 1.0

    \brief Abstract all functionnalities of a  FedProcessCtrl
*/


class FedProcessCtrlInterface 
{
  public:
  enum FedProcessCtrlTypes {DUMMY,DIMFEDDEV,
				   SHAREDMEM,DIRECT};
   //variable 
protected :
  int fSpillSize;
  bool fInitialized;

public:
       /** Constructor
      
  */   
  FedProcessCtrlInterface  (   ){ fInitialized=false;
  fSpillSize=200;
  } 

  virtual ~FedProcessCtrlInterface(){};

 
  /** Get Number of event in Spill (buffer) */
  virtual int getNumberOfEvents()=0; 
  /** Get Spill number (buffer counts) */
  virtual int getSpillNumber()=0; 
/** check for new spill return true if spill is DIFFERENT in respect to the last request
    will lock the buffer if there is a new spill */  
  virtual bool NewSpill()=0;
 /** check for new spill and wait for skip spills 
     @param skip the number of spills that should be skipped 
     */
  virtual bool NewSpill(int skip )=0;

  /** return the board Id of the FED device in use if not implemented returns -1*/
  virtual int getFedId(){ return -1;}

/** purge all buffers and returns true if the new data
      is provided. In case of purge is not possible it 
      calls NewSpill(2)
  */
  virtual bool clearFed(){ return NewSpill(2);}
  /**  stopst the data acquisition loop 
       returns true if successfull
  */
  virtual bool stopFedDaqLoop(){return true; }

  /** will tune the fed timing 
      if delay = -1 the fine tune program will be activated
      else the fed clock delay is set to delay 
      will tune the fed timing  return true if succesfull
      not all implementation can do this for these where 
      it can not be done return false 
  */
  virtual bool TuneFedSampleTime(int& delay ){return false;}
  /** adjust the threshold of the fed this should be used 
      with care because not all implementation can do this
  */
  virtual bool AdjustThresholds(){return false;}
  /** adjust the threshold of the FED header finding algorithm
      if low and high are 0 the the threshold finding algorithm is activated
      else the thresholds are set to the given value 
      this should be used 
      with care because not all implementation can do this
  */
  virtual bool AdjustThresholds(int& low, int& high){return false;}
  virtual void setScopeMode(int j){;}
  virtual void setSpillSize(int spillsize){fSpillSize=spillsize;}
  virtual int  getSpillSize(){ return fSpillSize;}
  /** returns the data pointer to the apv buffer */
  virtual unsigned short 
    getApvEventData(int event,int ch,int apv, int nr_sample,int fednr=0){
    return 0;}

  /** Lock access */
  virtual void  lockDataBuffer(){;}
  /** UnLock access */
  virtual void  unlockDataBuffer(){;}
  /** Release access */
  virtual void  release() {;}
  /** True if channel correctly initialized */
  virtual  bool isInitialized() { return fInitialized;}
  /** if the buffer is locked it will unlock the buffer */
  virtual void NextSpill()=0;
 
};

#endif 
	

