#ifndef MUXDEVINTERFACE
#define MUXDEVINTERFACE

#include "pmap.h"
#include "muxchannelinterface.h"


class MuxDevInterface {
 protected:
  bool fInitialized;
  int fGroupnr;
  pmap <int ,MuxChannelInterface*> fCards;
 public:
  MuxDevInterface( ){ fGroupnr=-2; /* invalid*/ }
  virtual ~MuxDevInterface(){;}
  virtual int  getNrMuxCards(){return fCards.size();} 
  virtual MuxChannelInterface* getMuxCard(int card){ 
    if( fCards.find(card) != fCards.end()) {
      return fCards[card];
    }
    else {return 0;}
  }
  /** sets the group nr for all cards connected to this 
      multiplexer device
      @param group : the group nr that should be set
      @return 0   for the moment no checking implemented
  */
  virtual int  setGroupNr(int group);
  /** sets the group nr for an individual card. 
      This function should only be used for debug purposes
      because all cards should set to the same group normmaly
      @param group : the group nr that should be set
      @param cardnr: the card number that should be set 
  */
  virtual int  setGroupNr(int group, int cardnr);
  virtual bool isInitialized(){return fInitialized;}
  /** returns the group nr that is now active for the device
      this has not to be necessary the group nr for each card
      if a groupnr is not defined for a certain card then the
      old group nr will be kept for that card. 
      The group nr is a variable in this class that is set
      everytime setGroupNr(int group) is called 
      @return the active group nr of the device ;
  */
  virtual int getGroupNr( ){ return fGroupnr;} 
  /** returns the group nr of  a certain card 
      @param card the cardnumber of this multiplexer device
      @return the actual groupnr that is active   or -3 if the card doesn't exist
   */
  virtual int getGroupNr(int cardnr ); 
  
};
#endif 
