#ifndef PLLI2CCHANNELLT_H
#define PLLI2CCHANNELLT_H

#include "plli2cchannelinterface.h"
/**
   \class PllI2cChannel
   \author W. Beaumont V.Z
   \date 10/  2001
   \version 1.0
   \brief Lt  implementation of PllChannelInterface

    <h2> Usage </h2>
    It implements PllChannelInterface class for the lt application. 
    It takes care about the correct instantiation
    of the PllChannel (ccu address, Fec board, Ccu channel ) according the connection
    to the bench position as described in the lt.xml file
    
   
    \

 */

class PllI2cChannelLt : public PllI2cChannelInterface
{
private:
   PllI2cChannelInterface* fCd;
   bool fInitialized; 
   int fChannel;
   int fBoard;
   unsigned int fLatencyOffset;
public:
  /**
     Constructor. Correct initialisation should be checked with isInitialized()
     @param  benchposition   the benchposition where the module is .
  */  
  PllI2cChannelLt(int benchpostion);
 virtual ~PllI2cChannelLt();
  /** True if the channel was found in the Daq*/
  inline bool isInitialized() { return fInitialized;}
  /** PllDescription callbacks */
  virtual void setAddress(short i) {fCd->setAddress(i);}
  virtual void setLatency(short i) {fCd->setLatency(i);}
  virtual void setInit(short i) {fCd->setInit(i);}
  virtual void setReadOnly(bool i) {fCd->setReadOnly(i);}
  virtual void setClock(short i){fCd->setClock(i);}
  virtual short getClock(){return fCd->getClock();}
  virtual short getAddress() {return fCd->getAddress();}
  virtual short getLatency() {return fCd->getLatency();}
  virtual  bool initRequested() { return fCd->initRequested();}
 /** Share memory lock */
  virtual void  lock() {fCd->lock();}
  /** Share memory unlock */
  virtual void  unlock() {fCd->unlock();}
  /** Share memory detach, Daq release of the channel */
  virtual void  release(){fCd->release();}
  /** DaqAction callback to requestAction + signal of server */
  virtual void  justDoIt(){fCd->justDoIt();}
 
 /** poll routine */ 
  virtual int wait(){return fCd->wait();}
  //lt specific
  /** returns the latency offset as given in the configuration file
   */
  unsigned int getLatencyOffset(){return fLatencyOffset;}
};
#endif
