#ifndef TSCCHANNELDUMMY_H
#define TSCCHANNELDUMMY_H
//#include "TDeviceControl.h"
//#include "TShare.h"
#include "tscdescription.h"
#include "tscchannelinterface.h"
/**
   \class Tscchanneldummy
    \author W. Beaumont  
    \date February  2002
    \version 1.0
    \brief Implementation of TscChannelInterface

    <h2> Usage </h2>
    copied from tscchannel
    It implements TscChannelInterface class:
    <ul>
    <li> For having the application working if not TSC device is present
    <li> also emulates  wait    unlock,justDoIt and release. functions
    </ul>
    <pre>
    // Request one channel
     tsc = 
     new Tscchanneldummy("ControlePrincipal",tscnb,0);
     none of these are used but is nice for checking correct passing in case of debug
     if (!tsc->isInitialized()) exit(0);


     // Use it 
     tsc->lock();
     tsc->setExternalTrigger();
     tsc->setApvLatency(SBLatency->text().toInt());
     ...
     tsc->justDoIt();
     tsc->unlock();  

     // Release
     tsc->release();
    </pre>

 */
class TscChannelDummy : public TscChannelInterface
{
private:
  pid_t fPid;

  ChannelDescription* fCd;
  //TShare *fMemoryClient;
  bool fInitialized;
  TscDescription* fTscDescription;
  int AttachChannel();
public:  
  /**
     Constructor. Correct initialisation should be checked with isInitialized()
     no real function done during construction
     @param tdname  Name of the TDeviceControl, controlling the Daq
     @param board   Board id as registered in TDeviceControl
     @param channel Channel number as registered in TDeviceControl
  */
  
  TscChannelDummy(char *tdname,int board,int channel);
  ~TscChannelDummy();///< Destructor. Release channel
  /** True if the channel was found in the Daq*/
  inline bool isInitialized() { return fInitialized;}
  /** TscDescription callbacks */
  /// Set latency after trigger before reset 
  virtual void setResetLatency(int  a){fTscDescription->setResetLatency(a);} 
  /// Get latency after trigger before reset 
  virtual int getResetLatency(){return fTscDescription->getResetLatency();}


  /// Set minimum number of clock between trigger
  virtual void setClockMin(int  a){fTscDescription->setClockMin(a);}  
  /// Get minimum number of clock between trigger
  virtual int getClockMin(){return fTscDescription->getClockMin();}

  /// Set Trigger Maximum Count
  virtual void setTriggerCount(int  a){fTscDescription->setTriggerCount(a);}  
  /// Get Trigger Maximum Count
  virtual int getTriggerCount(){return fTscDescription->getTriggerCount();}


  /// Set Trigger Register
  virtual void setTriggerRegister(int  a){fTscDescription->setTriggerRegister(a);}  
  /// Get Trigger Register
  virtual int getTriggerRegister(){return fTscDescription->getTriggerRegister();}


  /// Enable Interrupt
  virtual void enableInterrupt() {fTscDescription->enableInterrupt();}
  
  /// disable Interrupt
  virtual void disableInterrupt() {fTscDescription->disableInterrupt();}
  
  /// Interrupt status
  virtual bool getInterrupt(){return fTscDescription->getInterrupt();}

  /// Inhibit trigger
  virtual void enableInhibitTrigger() {fTscDescription->enableInhibitTrigger();}
  
  /// disableInhibitTrigger
  virtual void disableInhibitTrigger() {fTscDescription->disableInhibitTrigger();}
  
  /// InhibitTrigger status
  virtual bool getInhibitTrigger(){return fTscDescription->getInhibitTrigger();}

  /// enable calibration
  virtual void enableCalibration() {fTscDescription->enableCalibration();}
  
  /// disableCalibration
  virtual void disableCalibration() {fTscDescription->disableCalibration();}
  
  /// Calibration status
  virtual bool getCalibration(){return fTscDescription->getCalibration();}


  /// enable Reset
  virtual void enableReset() {fTscDescription->enableReset();}
  
  /// disable Reset
  virtual void disableReset() {fTscDescription->disableReset();}
  
  /// Reset status
  virtual bool getReset(){return fTscDescription->getReset();}



  /// set External trigger
  virtual void setExternalTrigger() { fTscDescription->setExternalTrigger();}
  
  /// External Trigger status
  virtual bool getExternalTrigger(){return fTscDescription->getExternalTrigger();}

  /// set Internal trigger
  virtual void setInternalTrigger(){fTscDescription->setInternalTrigger();}
  
  /// Internal Trigger status
  virtual bool getInternalTrigger(){return fTscDescription->getInternalTrigger();}

  /// set Soft trigger
  virtual void setSoftTrigger() {fTscDescription->setSoftTrigger();}
  
  /// Soft Trigger status
  virtual bool getSoftTrigger(){return fTscDescription->getSoftTrigger();}


  /// Set Internal Trigger Frequency in 25 micro sec unit
  virtual void setTriggerFrequency(int  a){fTscDescription->setTriggerFrequency(  a);}  
  /// Get Internal Trigger Frequency in 25 micro sec unit
  virtual int getTriggerFrequency(){return fTscDescription->getTriggerFrequency();}


  /// Set latency after trigger before Fed
  virtual void setFedLatency(int  a){fTscDescription->setFedLatency(  a);}  
  /// Get latency after trigger before Fed
  virtual int getFedLatency(){return fTscDescription->getFedLatency();}

  /// Set Apv latency 
  virtual void setApvLatency(int  a){fTscDescription->setApvLatency(  a);}  
  /// Get Apv latency 
  virtual int getApvLatency(){return fTscDescription->getApvLatency();}


  /// Set Fed Delay trigger
  virtual void setFedDelayTrigger(int  a){fTscDescription->setFedDelayTrigger( a);}  
  /// Get Fed Delay Trigger
  virtual int getFedDelayTrigger(){return fTscDescription->getFedDelayTrigger();}


  /// Set Fed Delay Clock
  virtual void setFedDelayClock(int  a){fTscDescription->setFedDelayClock(  a);}  
  /// Get Fed Delay Clock
  virtual int getFedDelayClock(){return fTscDescription->getFedDelayClock();}

  /// Set Pulser Delay
  virtual void setPulserDelay(int  a){fTscDescription->setPulserDelay(  a);}  
  /// Get Pulser Delay
  virtual int getPulserDelay(){return fTscDescription->getPulserDelay();}


  /// Set Calibration latency 
  virtual void setCalibrationLatency(int  a){fTscDescription->setCalibrationLatency(  a);}  
  /// Get Calibration latency 
  virtual int getCalibrationLatency(){return fTscDescription->getCalibrationLatency();}

  /// Reset Apv
  virtual void setResetApv(bool a) {fTscDescription->setResetApv(a);}
  virtual bool getResetApv(){return fTscDescription-> getResetApv();}

  /// Reenable trigger
  virtual void setReenableTrigger( bool a ){fTscDescription->setReenableTrigger(  a );}
  virtual bool getReenableTrigger(){return fTscDescription->getReenableTrigger();}

  /// Reset Pll
  virtual void setResetPll(bool a){fTscDescription->setResetPll( a);}
  virtual bool getResetPll(){return fTscDescription->getResetPll();}

  /// Soft Trigger
  virtual void setSendSoftTrigger(bool a){fTscDescription->setSendSoftTrigger( a);}
  virtual bool getSendSoftTrigger(){return fTscDescription->getSendSoftTrigger();}

  /// Reset Fed
  virtual void setResetFed(bool a){fTscDescription->setResetFed( a);}
  virtual bool getResetFed(){return fTscDescription->getResetFed();}


  /// Set gate position calibration delay
  virtual void setGatePositionCalibrationDelay(unsigned char del){fTscDescription->setGatePositionCalibrationDelay( del);}
  /// Get gate position calibration delay
  virtual unsigned char getGatePositionCalibrationDelay(){return fTscDescription->getGatePositionCalibrationDelay();}
  
  /// Set gate position user delay
  virtual void setGatePositionDelay(unsigned char del){fTscDescription->setGatePositionDelay( del);}
  /// Get gate position user delay
  virtual unsigned char getGatePositionDelay(){return fTscDescription->getGatePositionDelay();}

  /// Set gate width calibration delay
  virtual void setGateWidthCalibrationDelay(unsigned char del) {fTscDescription->setGateWidthCalibrationDelay( del);}
  /// Get gate width calibration delay
  virtual unsigned char getGateWidthCalibrationDelay(){return fTscDescription->getGateWidthCalibrationDelay();}
  
 
  /// Set gate width user delay
  virtual void setGateWidthDelay(unsigned char del){fTscDescription->setGateWidthDelay( del);}
  /// Set gate width user delay
  virtual unsigned char getGateWidthDelay(){return fTscDescription->getGateWidthDelay();}
 
  /// Get Gate Delay register
  virtual long getGateDelayRegister(){return fTscDescription->getGateDelayRegister();}

  /// set Gate Delay register
  virtual void setGateDelayRegister(long a) {fTscDescription->setGateDelayRegister( a);}




  /** Share memory lock */
  virtual void  lock() {;} // noting to do for a dummy
  /** Share memory unlock */
  virtual void  unlock() {;}
  /** Share memory detach, Daq release of the channel */
  virtual void  release();
  /** DaqAction callback to requestAction + signal of server */
  virtual void  justDoIt();
  /** Polling loop */
  int wait();
  
  };
#endif
