#include "daqerrormsg.h"
#include "apvsimulator.h"
#include "TRandom.h"

#define MAXAPVCHAN  128
#define SAMPLELEN  512


using  namespace std;
Apv_Simulator::Apv_Simulator(float avped, float avrms, float avcmn, float cmnslope, bool bbadch, bool bbclus):
  theavPed(avped),
  theavRms(avrms),
  theavCmn(avcmn),
  theCmnslope(cmnslope),
  bdefaultbadch(bbadch),
  bclusters(bbclus)
{
  thePedestal.resize(MAXAPVCHAN);
  theData.resize(MAXAPVCHAN);
  theDatancmn.resize(MAXAPVCHAN);
  theFrame.resize(SAMPLELEN);
  //Se = new SimulationExchange();
  
  if(bdefaultbadch) setdefaultbadch(); 
  float  varped=avrms; //variance of pedestal is equal the av rms
 

  for(int i=0;i<MAXAPVCHAN;i++)
    {
         thePedestal[i]=int(theavPed+varped*((gRandom->Rndm(1))-0.5));
         //    thePedestal[i]=int(theavPed);
    }
  DT_DEBUG_LTC(" constructed simulator");
}

Apv_Simulator::~Apv_Simulator()
{
}

void Apv_Simulator::setdefaultbadch()
{
 theNoisyList.clear();
  theQuietList.clear();  
    //default abnormal channels
   theNoisyList.push_back(10);
  theNoisyList.push_back(30);
  theNoisyList.push_back(50); 
  theNoisyList.push_back(70); 
  theNoisyList.push_back(90);
 
 theQuietList.push_back(20); 
 theQuietList.push_back(40);  
 theQuietList.push_back(60);  
 theQuietList.push_back(80); 
 theQuietList.push_back(100);
 bdefaultbadch=true;  
}

void Apv_Simulator::clearbadch()
{
 theNoisyList.clear();
 theQuietList.clear();  
}


Ivec Apv_Simulator::addrms()
{
  Ivec frame(0,MAXAPVCHAN);
  for(int i=0;i<MAXAPVCHAN;i++)
    {
      float rms=theavRms;
    if(find(theNoisyList.begin(),theNoisyList.end(),i)!=theNoisyList.end()) 
{rms=theavRms*theNoisyFactor;theNoisyFactor+=0.5;}
      //{a=a*theNoisyFactor;theNoisyFactor+=0.5;}
    if(find(theQuietList.begin(),theQuietList.end(),i)!=theQuietList.end())
{rms=theavRms*theQuietFactor;theQuietFactor-=0.2;}
// {a=a*theQuietFactor;theQuietFactor-=0.2;}
    //    frame[i]=int(thePedestal[i]+theavRms*a+0.5);
    frame[i]=int(gRandom->Gaus(thePedestal[i],rms));   
    if(frame[i]>512) frame[i]=1024;
    if(frame[i]<0)   frame[i]=0;
    }
  return frame;
}

Ivec Apv_Simulator::addcmn(const Ivec& frame)
{
  Ivec frameout(0,MAXAPVCHAN);
   theCmn=gRandom->Gaus(0,theavCmn);
   int sgn=1;
   if((gRandom->Rndm(1))>0.5)  sgn=-1; 
    for(int i=0;i<MAXAPVCHAN;i++)
    {
 if(find(theQuietList.begin(),theQuietList.end(),i)!=theQuietList.end())  frameout[i]=frame[i];//no cmn for quiet ch.
      frameout[i]=int(frame[i]+theCmn+sgn*(i*theCmnslope/MAXAPVCHAN-theCmnslope/2)+0.5);
    if(frameout[i]>1024) frameout[i]=1024;
    if(frameout[i]<0)   frameout[i]=0;
    }
  return frameout;
}

Ivec Apv_Simulator::addclus(const Ivec& frame)
{
  //default clusters settings
  double avclussize=2.;      // av cluster size 2
  int clsize=gRandom->Poisson(avclussize); //clsize  by Poisson
  int xpos=int(gRandom->Uniform(MAXAPVCHAN-clsize));  // position from 0:MAXAPVCHAN-clsize
  double signal=theavRms*15./clsize;  //SNR
  double sigma=signal;       
  float q[clsize];
  // xpos=55;
 for(int i=0;i<clsize;i++)
    {
  q[i]=(gRandom->Landau(signal,sigma))/clsize;
    }
  //
  Ivec frameout(0,MAXAPVCHAN);
       int k=0;
  for(int i=0;i<MAXAPVCHAN;i++)
  {
    if(i>=xpos&&k<clsize)
      { frameout[i]=int(frame[i]+q[k]);k++;
      if(frameout[i]>1024) frameout[i]=1024;
     } else {
       frameout[i]=frame[i];}
    }
 return frameout;
}


Ivec Apv_Simulator::addframe(const Ivec& framein)
{
  Ivec frameout(0,SAMPLELEN);
  int baseline=10; 
  int tickmarks=100;
  bool bframe=false;
  int start=300;
  for(int i=0;i<SAMPLELEN;i++)
    {
     if(i>start+12&&i<=(start+12+MAXAPVCHAN))
      bframe=true;

      if(!(i%35)&&!bframe) 
      frameout[i]=baseline+tickmarks;
      else if(!bframe)
        frameout[i]=baseline;
  
      if(i>start&&i<=(start+3))
        frameout[i]=baseline+tickmarks;
        else if(i>(start+3)&&i<=(start+10))
        frameout[i]=baseline;
        else if(i==start+11)
        frameout[i]=baseline+tickmarks;
        else if(i==start+12)
         frameout[i]=baseline+tickmarks;
        else if(i>start+12&&i<=(start+12+MAXAPVCHAN))
        frameout[i]=baseline+framein[i-start-12];
        
      }

  return frameout;
}

void Apv_Simulator::Simulate()
{
  theNoisyFactor=1.5; // for noisy channels factor
  theQuietFactor=0.8; // for quiet channels

  theDatancmn=addrms();
  theData=addcmn(theDatancmn);
  if(bclusters) theData=addclus(theData);
}
