#include "dcuconversion.h"
#include "CalibrationValues.h"

// a number of helper functions for the DCU1 
/** returns the voltage at the input of the DCU when the DCU returns the dcuvalue 
    Assumes the following converstion
    DCU     input voltage
    0  ==>  0.25V
    2048  ==> 1.25 V
    4096  ==> 2.25 V
*/
#ifdef TCZERO 
#undef TCZERO
#endif 
const float TCZERO=-237;
    
float  DcuConversion::Dcu2Voltage( unsigned short dcuvalue ) 
{  
  return   (dcuvalue-fAdcOffset)/fAdcGain;
}

float  DcuConversion::Dcu125Voltage( unsigned short dcuvalue  ) 
{ return (dcuvalue-fAdcOffset)/fAdcGain/fK1_25;
}

float  DcuConversion::Dcu250Voltage( unsigned short dcuvalue ) 
{
 return  (dcuvalue-fAdcOffset)/fAdcGain/fK2_5;
}

float DcuConversion::DcuExternTemperature ( unsigned short dcuvalue  )
{ 
 float Ractual = Dcu2Voltage( dcuvalue) / fIBias0;
 return  NtcResistor2temperature(Ractual,fSiR0,fRefTemp,
						fSiNtcBeta);
}
 
float DcuConversion::DcuHybridTemperature ( unsigned short dcuvalue  )
{ 
 float Ractual = Dcu2Voltage( dcuvalue) / fIBias4;
 return  NtcResistor2temperature(Ractual,fHybR0,
						fRefTemp,fHybNtcBeta);
}
 
float DcuConversion::DcuLeakageCurrent( unsigned short dcuvalue ){
  return Dcu2Voltage( dcuvalue)/fBiasRes;
}

float DcuConversion::DcuTsTemperature( unsigned short dcuvalue ){
  return  fRefTemp+ (dcuvalue-fTempRefValue) /fTempGain;
}


float DcuConversion::NtcResistor2temperature
 ( float R , float R0 , float T0, float beta){  
  // only first order  so Rac = R0 exp { Beta ( 1/T -1/To)
  float temperature ; // resistor NTC at calibration temperature 
  temperature     =-TCZERO +  1/ 
    ( (1/(T0  +TCZERO) ) + 
      (1/beta) * log(R / R0) );
  return temperature ;
}     

float DcuConversion::Temperature2NtcResitor
 ( float T , float R0 , float T0, float beta){  
  // only first order  so Rac = R0 exp { Beta ( 1/T -1/To)
  return R0* exp( beta/((T+TCZERO)- 1/(T0+TCZERO))) ;
}  

