#include "daqerrormsg.h"
#include "fedchanneldummy.h"
#include <TROOT.h>
#include <valarray>
#include <cstdlib>
using namespace std;
typedef valarray<int> Ivec;

/** \class FedChannelDummy implementation
    \author W.Beaumont
    \date March 2001
    \version 1.0

    \brief FedDummy implementation of FedChannelInterface

    <h2> Usage </h2>
    It implements FedChannelInterface class and make the Daq transparent to 
    the user:
    <ul>
    <li>this is a dummy chanel to test higher level programs without the (HW) interface
    <li> it doesn'tgenerate usefull data 
    
    </ul>
 
*/

 
  FedChannelDummy::FedChannelDummy(long fed_ser_nr, long channel_arg, PROCESS_MODE process_mode_arg, 
             long  mux_channel_arg , long  time_channel_arg)
    : FedChannelInterface(fed_ser_nr,channel_arg,process_mode_arg,mux_channel_arg,time_channel_arg) 
    { fInitialized=true ; fSpill=0;
    fProcessMode = process_mode_arg;
    Se = new SimulationExchange();
    TROOT root("apvsim","apv simulation");
    char envvar[40];
    if ( getenv( "APVSIMNOISE"))     {     
      strcpy(envvar,   getenv("APVSIMNOISE") );
      avrms =(float)strtod( envvar,0);
    }
    else { avrms=5;
    }
    avped=200.;
    avcmn=(3/5)*avrms ;
    avcmsl=0;
    bbad=true; // flag to simulate bad channels

    ssim=new Apv_Simulator(avped,avrms,avcmn,avcmsl,bbad,true);
    ssim->setclustersOff();  
    //check if apv channel id is requested 
    
    if ( getenv( "APVCHANNELID"))     {     
      strcpy(envvar,   getenv("APVCHANNELID") );
      fApvChannelIdMode =( strtol( envvar,0,0))% 128;
    }
    else {fApvChannelIdMode = -1 ; // default value}
    }

    DT_DEBUG_LTC(" dummy constructed with ApvChannelIdMode is "
		  <<fApvChannelIdMode);
    }


int  FedChannelDummy::getApvEventData(int event_nr,short *out_buf, int size_of_buf ) { 

  Ivec rdata,dt;
  int ndat;
  int i,j;
  short int *p=out_buf;
  if(fApvChannelIdMode < 0){
    ssim->Simulate();
    i=(int)(fProcessMode);
    //       if (i>=0 && i<=5)
    //	   {	     
    ndat=140;
    dt.resize(128);
    dt=ssim->getdata();
    rdata.resize(ndat);
    rdata[0]=380;rdata[1]=380;rdata[2]=380;rdata[3]=170;
    rdata[4]=380;rdata[5]=170;rdata[6]=380;rdata[7]=170;
    rdata[8]=380;rdata[9]=380;rdata[10]=380;rdata[11]=380;
    for (j=0;j<=127;j++){ rdata[12+j]=dt[j]; }
    //	   }
    //       else
    //	 if (i>=6 && i<=10) 
    //	   {
    //	     ndat=512;
    //	     rdata.resize(ndat);
    //	     rdata=ssim->getframe();
    //	   }
    
    // add cal pulse if needed
    if (Se->CheckCalMode()){
      for (int j=0;j<128;j++) {
	
	int k= (int)( j/32) + 4*(int)((j%32)/8) +16*(j%8);  
	rdata[12+k]+=Se->getCalPulseHeight(j) ;
      }
    }
    
    //      cout<<" FedChannelDummy "<<ndat<<endl;
    for ( i=0;i<ndat;i++)    { 
      //    cout<<" "<<rdata[i];
      *p++ = rdata[i];
    }
  }
  else {//0    1/7     2/8     3/9      4/10         5/11
    *p++=380;*p++=380;*p++=380;*p++=170;*p++=380;*p++=170;
    *p++=170;*p++=170;*p++=380;*p++=170;*p++=380;*p++=170;
    ndat=140;
    switch (fProcessMode){
     case SCOPE_SECOND_SAMPLE:
    case FRAME_FIRST_SAMPLE:{
       for(i=12;i<ndat;i++){
	 *p++=243+ (ndat-i) % 64;
       }
     }
     break;
      default:{
       for(i=12;i<ndat;i++){
	 *p++=243+ (i-11) % 64;
       }
     }     
    }
    p=out_buf;
    p+=12; //first strip
    p+=fApvChannelIdMode ;
    *p+=80;
  }
  return  1;
}

  
 
FedChannelDummy::~FedChannelDummy(){ delete ssim ;}





