#include "fedchannellt.h"
#include "fedchanneldummy.h"
#include "fedchanneldimdev.h"
#include "fedchannel.h"
#include "ltparser.h"
#include "daqerrormsg.h"
#include "muxexchange.h"
extern LtParser* ltparser; 

FedChannelLt::FedChannelLt (  int hybrpos , int slotpos )
{ FedChannelInterface::PROCESS_MODE processmode ;
 processmode= !(hybrpos % 2) ?  FedChannelInterface::FRAME_FIRST_SAMPLE :  
                          FedChannelInterface::FRAME_SECOND_SAMPLE;
 FedChannelLtInit( hybrpos,slotpos,hybrpos, processmode);  
 /* for the fedchannel the only thing that matters if the I2C addres 
    is odd or even   so we can fill in the hybrid pos 
 */
}
 
FedChannelLt::FedChannelLt(  int hybrpos , int slotpos , int apvI2Caddr , FedChannelInterface::PROCESS_MODE processmode   
                                      //will be defined by the application
) {
  FedChannelLtInit( hybrpos,slotpos,apvI2Caddr, processmode);
}


void FedChannelLt::FedChannelLtInit (  int hybrpos , int slotpos , int apvI2Caddr , FedChannelInterface::PROCESS_MODE processmode   
                         //will be defined by the application
){  
  fFed=0; 
  int utrichannel = hybrpos /2 ; 
  // the output of the utri channel is the hybrid pos /2  
  BenchPosConnection* BenchP
    = new BenchPosConnection(*ltparser->getBenchPosConnections(slotpos));
  if ( BenchP->getId() == -1) { 
    DAQEXIT("Benchpos" << slotpos <<" doesn't exist \n"<< endl); 
  }
  //  BenchP->info();
    
  // asssume fixed relation between hybrid position and UTRI card or if 
  // depening on hybrid type fixed by table   
    //

    /// these are protected in the FED channel so can be overwritten 
     fSerialNumber= BenchP-> getFedNr(utrichannel);     
     if (fSerialNumber < 0){
       fMuxId  = BenchP->getMuxChId(utrichannel);
       if (fMuxId.fMuxDevice  < 0 || 
	   fMuxId.fMuxCard < 0    || 
	   fMuxId.fMuxChannel < 0 ){ 
	 DAQEXIT("Benchpos" << slotpos <<"mux  doesn't exist \n"<< endl); 
       }
       DT_DEBUG_LTC("Mux info : " <<fMuxId.fMuxDevice <<","<<
		    fMuxId.fMuxCard <<" "<<fMuxId.fMuxChannel);
       MuxExchange  muxdev( *ltparser->getMuxDevInfo(fMuxId.fMuxDevice));
      //   if( muxdev == 0 not possible ) {
//  	 DAQEXIT("Benchpos" << slotpos <<"mux "<< fMuxId.fMuxDevice <<
//  		 "doesn't exist \n"<< endl);
//         }
       fChannel=muxdev.getFedCh(fMuxId.fMuxCard);
       fSerialNumber= muxdev.getFedNr(fMuxId.fMuxCard);
       fMuxId.fMuxGroup= muxdev.getGroupNr(fMuxId.fMuxCard,fMuxId.fMuxChannel);
     }
     else  { // getUtriChannel should be called getFedChannel 
       fChannel=BenchP->getUtriChannel(utrichannel);	 
       fMuxId.fMuxGroup=-1; //direct FED connection
     }
     
    if (BenchP->getFedSimulation(utrichannel) ) {	
      DT_DEBUG_LTC("4");
	fFed= new FedChannelDummy(fSerialNumber , fChannel,processmode ,
					      apvI2Caddr,fMuxId.fMuxGroup);
	DT_DEBUG_LTC(" dummy fed ch constructed ");
      }
    else{ 
      DT_DEBUG_LTC("constr fedch: type: "<<BenchP->getFedType(utrichannel) 
		   << " srnr "<< fSerialNumber 
		   << "channel " << fChannel
		   << "mux group " <<fMuxId.fMuxGroup);
      if (fChannel >= 0  && fChannel < 8) { 
	switch( BenchP->getFedType(utrichannel)){
	  case BenchPosConnection::SHAREDMEM :{
	    fFed= new FedChannel((long)fSerialNumber , 
				 (long)fChannel,processmode ,
				 (long) apvI2Caddr,
				 (long)fMuxId.fMuxGroup);
	    DT_DEBUG_LTC(" shared mem constructed ");
	  }
	  break;
	  case BenchPosConnection::DIMDEV : {
	     fFed=new FedChannelDimDev ((long)fSerialNumber , 
				  (long)fChannel,processmode ,
				  (long) apvI2Caddr,
				  (long)fMuxId.fMuxGroup);
	     DT_DEBUG_LTC(" dimdev srnr:"<< fSerialNumber<< 
			  "chan "<<fChannel <<"  constructed ");
	  }
	  break;
	  default: fFed=0;
	    DT_WARN("Fed connection type not supproted");
	}
      }
      else { fFed=0; }
    }
    if(fFed ){ 
      if (fFed->isInitialized()) { fInitialized=true;}
    }
    else fInitialized=false ;  
    delete BenchP;
}

FedChannelLt::~FedChannelLt(){
  delete fFed;
} 

void  FedChannelLt::lockDataBuffer()  { 
  if( ! fLockStatus)     {
    fFed->lockDataBuffer();
    fLockStatus = fSerialNumber;
  }
}

void  FedChannelLt::unlockDataBuffer()  {
  if(  fLockStatus)     {  
    fFed-> unlockDataBuffer();
    fLockStatus =0;
  }
}

int FedChannelLt::fLockStatus=0;


