/*****************************************************************************/
/*                     The Karlsruhe CMS readout project                     */
/*                                                                           */
/* Copyright (C) 2000/2001  by Guido Dirkes, Manuel Fahrer,                  */
/*			       Stefan Heier, Wolfgang Scherdtfeger,          */
/*			       Thomas Weiler and Siegfried Weseler           */
/* All rights reserved.                                                      */
/*                                                                           */
/* This file is part of the ANALYSIS tool.                                   */
/* Corresponding file are:                                                   */
/*   motherboard.h  definition of Motherboard class                          */
/*   motherboard.cc declaration of motherboard methodes                      */
/*   testmb.cc      simple test programm for motherbaord hardware            */
/*   Makefile                                                                */
/*                                                                           */
/* First release  19/07/2000 by Stefan Heier                                 */
/* Second release 15/08/2001 by Guido Dirkes make shared libraries           */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/*   This program is free software; you can redistribute it and/or modify    */
/*   it under the terms of the GNU General Public License as published by    */
/*   the Free Software Foundation; either version 2 of the License, or       */
/*   (at your option) any later version.                                     */
/*                                                                           */
/*   This program is distributed in the hope that it will be useful,         */
/*   but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*   GNU General Public License for more details.                            */
/*                                                                           */
/*   You should have received a copy of the GNU General Public License       */
/*   along with this program; if not, write to the Free Software             */
/*   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.               */
/*                                                                           */
/*****************************************************************************/
#include <motherboardi2c.h>
#include <unistd.h>
#include "daqerrormsg.h"
#include <normi2c.h>


MotherboardI2C::MotherboardI2C(NormI2c *i, int baseaddr ){  
  i2c = i; 
 DATAADDR= baseaddr;
 CTRLADDR=baseaddr+2;
 ACKADDR=baseaddr+4;
 // set DS = 0 and RESET = 1;
 unsigned char dummy = 0x02;
 i2c->SingleByteWrite(CTRLADDR, dummy);
 
 // set RESET = 0;
 dummy = 0x00;
 i2c->SingleByteWrite(CTRLADDR, dummy);
 
 i2c->SingleByteRead(ACKADDR, dummy);
 ctrl = 0;
}

int MotherboardI2C::testbyte(int s){
  unsigned char dummy = (s & 0xFF);
  return i2c->SingleByteWrite(DATAADDR, dummy);
}

int MotherboardI2C::getSlot(int s, int b )
{
    unsigned char dummy;
    int timeout = 0;
    
    if (ctrl & 0x08) {
	DT_WARN( "Board already selected !" );;
	return -1;
    }
    
    //apply data byte
    ctrl = ((b & 0x0f) << 4) | 0x08 | (s & 0x07);
    i2c->SingleByteWrite(DATAADDR, ctrl);

    // SEL = 1;
    dummy = 0x04;
    i2c->SingleByteWrite(CTRLADDR,dummy);

    // DS = 1;
    dummy |= 0x08;
    i2c->SingleByteWrite(CTRLADDR,dummy);

    do 
    {
	i2c->SingleByteRead(ACKADDR, dummy);
	i2c->SingleByteRead(ACKADDR, dummy);
//	usleep(1000);
	timeout++;
	if (timeout > TIMEOUT) {
	    DT_ERROR("<Motherboard I2C> getSlot: "
		     << "tired of waiting for acknowledge");		 
	    ctrl = 0;
	    return -1;
	}
    }
    while ( !(dummy & 0x1) );
    // cout << "Waittime was: " << timeout << endl;
        
    // DS = 0
    dummy = 0x04;
    i2c->SingleByteWrite(CTRLADDR, dummy);

    // SEL = 0
    dummy = 0;
    i2c->SingleByteWrite(CTRLADDR, dummy);

    do 
    {
	i2c->SingleByteRead(ACKADDR, dummy);
	i2c->SingleByteRead(ACKADDR, dummy);
//	usleep(1000);
	timeout++;
	if (timeout > TIMEOUT) {
	    DT_ERROR("<Motherboard I2C> getSlot: "
		     << "tired of waiting for acknowledge");		 
	    ctrl = 0;
	    return -1;
	}
    }
    while ( (dummy & 0x1) );
    // cout << "Waittime was: " << timeout << endl;

    return 0;
}

int MotherboardI2C::releaseSlot()
{
    unsigned char dummy;
    int timeout = 0;

    if ( !(ctrl & 0x08)) {
      DT_ERROR( "mb::releaseSlot : no board selected !" );
	return -1;
    }
    ctrl &= 0xf7;
    i2c->SingleByteWrite(DATAADDR, ctrl);
        
    // SEL = 1;
    dummy = 0x04;
    i2c->SingleByteWrite(CTRLADDR, dummy);

    // DS = 1;
    dummy |= 0x08;
    i2c->SingleByteWrite(CTRLADDR, dummy);

    do 
    {
	i2c->SingleByteRead(ACKADDR, dummy);
	i2c->SingleByteRead(ACKADDR, dummy);
//	usleep(1000);
	timeout++;
	if (timeout > TIMEOUT) {
	    DT_ERROR("<Motherboard I2C> getSlot: "
		     << "tired of waiting for acknowledge");	 
	    ctrl = 0;
	    return -1;
	}
    }
    while ( !(dummy & 0x1) );
    // cout << "Waittime was: " << timeout << endl;        

    // DS = 0
    dummy = 0x04;
    i2c->SingleByteWrite(CTRLADDR, dummy);

    // SEL = 0
    dummy = 0;
    i2c->SingleByteWrite(CTRLADDR, dummy);

    do 
    {
	i2c->SingleByteRead(ACKADDR, dummy);
	i2c->SingleByteRead(ACKADDR, dummy);
//	usleep(1000);
	timeout++;
	if (timeout > TIMEOUT) {
	    DT_ERROR("<Motherboard I2C> getSlot: "
		     << "tired of waiting for acknowledge");		 
	    ctrl = 0;
	    return -1;
	}
    }
    while ( (dummy & 0x1) );
    // cout << "Waittime was: " << timeout << endl;

    return 0;
    
}

int MotherboardI2C::sendData(unsigned char *data, int len)
{

    unsigned char dummy = 0;
    int timeout = 0;

    if ( !(ctrl & 0x08)) {
      DT_ERROR("mb::releaseSlot : no board selected !");
	return -1;
    }
    
    for (int i=0; i < len; i++)
    {
	dummy = 0;
	timeout=0;
	i2c->SingleByteWrite(DATAADDR, (unsigned char)(data[i]));

	// DS = 1;
	dummy |= 0x08;
	i2c->SingleByteWrite(CTRLADDR, dummy);
	
	do 
	{
	    i2c->SingleByteRead(ACKADDR, dummy);
	    i2c->SingleByteRead(ACKADDR, dummy);
//	    usleep(1000);
	    timeout++;
	    if (timeout > TIMEOUT) {
		DT_ERROR("<Motherboard I2C> getSlot: "
			 << "tired of waiting for acknowledge");
		ctrl = 0;
		return -1;
	    }
	}
	while ( !(dummy & 0x1) );
	// cout << "Waittime was: " << timeout << endl;        

	// DS = 0
	dummy = 0x0;
	i2c->SingleByteWrite(CTRLADDR, dummy);
	timeout = 0;
	do 
	{
	    i2c->SingleByteRead(ACKADDR, dummy);
	    i2c->SingleByteRead(ACKADDR, dummy);
//	    usleep(1000);
	    timeout++;
	    if (timeout > TIMEOUT) {
		DT_ERROR("<Motherboard I2C> getSlot: "
		     << "tired of waiting for acknowledge");
		ctrl = 0;
		return -1;
	    }
	}
	while ( (dummy & 0x1) );
	// cout << "Waittime was: " << timeout << endl;
    }
    return 0;
}
    

