#include <sys/wait.h>
#include <signal.h>
#include <unistd.h>
#include "ltparser.h"
#include "daqerrormsg.h"
#include "apv25i2cchannellt.h"
#include "benchposconnection.h"
#include "cmdlinedecode.h"
#include <vector>
#define MAXPOS 2 
LtParser* ltparser;

/** 
    \file scanlti2cchannellt.cpp

    \brief Preperation for the AuxChip test but this woks standalone too 
*/
bool HasToExit=false;
void  Ictrlc(int s)
{


  if (s == SIGCONT) return;
  printf("SIGINT:I: %d  got signal %d \n",getpid(),s);
  HasToExit=true;

}

 
void usage (){
  cout << "query alli2c addresses in the defined benchpositons unless -benchpos is given"<<endl;
  cout <<" stop with Ctrl-C , program will first finsh the loop before exiting"<<endl;
  cout <<endl<<" usage example : "<<endl;
  cout <<"\tTestLtI2cChannels  -benchpos 3  -ltconfig lttemp.xml " <<endl;
  cout <<"options:"<<endl;
  cout <<"\t-ltconfig :the configuration file (default is lt.xml)"<<endl;  
  cout <<"\t-benchpos : check apv25 for only this bench pos"<<endl;
  cout <<"\t-nrapv : set the number of apvs default = 4"<<endl;
  cout <<"\t-h : this message"<<endl;
  cout <<endl;
  std::exit(-1);
}

int   main (int argc ,  char **argv){   
  signal(SIGINT,Ictrlc); // Arm interrupt handler
  signal(SIGCONT,Ictrlc); // Arm interrupt handler
  int benchpos=-1;
  string cmdresult;
  string cmderr;
  int nrapv=4;
  if( "found"== CmdLineDecode(argc,argv,"h",cmdresult,false)){
    usage();
  } 
  if( "found"== CmdLineDecode(argc,argv,"nrapv",cmdresult,true)){
    nrapv= atoi(cmdresult.c_str());
  } 
  if( "found"== CmdLineDecode(argc,argv,"benchpos",cmdresult,true)){
    benchpos= atoi(cmdresult.c_str());
    DT_DEBUG_LTAPPL("Check only for "<< cmdresult.c_str());
  }
  ltparser = new LtParser();
  if( "found"== CmdLineDecode(argc,argv,"ltconfig",cmdresult,true)){
    // cout<< "use "<< result <<" as config " <<endl;
    ltparser->setParseFileName( cmdresult);
    DT_DEBUG_LTAPPL("Procedure: parse "<< cmdresult.c_str());
  }
  ltparser->parseFile();
  int nrPB=0;
  BenchPosConnection* BenchP[MAXPOS];
  if( benchpos<0 || benchpos > 10) {
    nrPB= ltparser->getNumberBenchPositions() ;
    for (int lc =0 ; lc < nrPB ; lc++) { 
      BenchP[lc]=new BenchPosConnection(*ltparser->getBenchPosConnections(lc));
      BenchP[lc]->info();
    }
  }
  else {
    nrPB=1;
    BenchP[0]=new BenchPosConnection(benchpos);
    if (!BenchP[0] ) {
      DAQEXIT("no info for bencpos "<<benchpos);
    }
  }

  Apv25I2cChannelInterface* Apv25I2c[20];  
  if  (nrPB > 20 ) exit(-1);
  for (int lc =0 ; lc < nrPB ; lc++) {      
    Apv25I2cChannelLt*  Apv25I2creal=  new Apv25I2cChannelLt(lc);
    if (!Apv25I2creal->isInitialized()) exit(0);
    //else 
    Apv25I2c[lc]   = Apv25I2creal;     
  }
  vector <unsigned int> erraddr;erraddr.clear();
  unsigned int dcuaddr=0;unsigned int apvaddr=32,muxaddr=67,plladdr=68;
  unsigned int apvprc=63;
  int waitres;bool errresp=false;
  unsigned int addr=0;
   for (int lc =0 ; lc < nrPB ; lc++) {
     for( addr=0 ; addr < 128;addr++) {
       if(  HasToExit) break;
       Apv25I2c[lc]->lock();
       Apv25I2c[lc]->setMode(0); 
       Apv25I2c[lc]->setAddress(addr);
       //Apv25I2c[lc]->update();
       Apv25I2c[lc]->unlock(); 
       Apv25I2c[lc]->justDoIt();
       waitres=Apv25I2c[lc]->wait();
       //cout << "addr "<< addr<<" waitres "<<waitres<<"nr errors" << erraddr.size()<<endl;
       if( (dcuaddr <= addr) && (addr < dcuaddr+8) ) {
	 if(  (errresp =(bool) waitres) ){
	   erraddr.push_back(addr);
	    }
	 continue;
       }
       if ( apvprc==addr || 
	    ((apvaddr <= addr)  && (addr < apvaddr+6)) 
       ){
	 if( (addr == apvaddr+2 || addr == apvaddr+3 ) 
	     &&  nrapv==4
	 ) {    
	   errresp= ! waitres  ;
	 }
	 else { errresp= (bool) waitres;}	    
	    if(errresp) { erraddr.push_back(addr);}
	 continue;
       }
       if( addr == muxaddr) {
	 if( (errresp= (bool) waitres) ){
	   erraddr.push_back(addr);
	 }
	 continue;
       }
       if(( plladdr <= addr) && (addr< plladdr+4)) {
	 if( (errresp= (bool) waitres) ){
	   erraddr.push_back(addr);
	 }
	 continue;
       }
       if( (errresp =! waitres) ){
	 erraddr.push_back(addr);
       }       
     }
   }
	

   if( erraddr.size() ){
     for ( addr=0; addr<erraddr.size();addr++) {
       cout <<"error on i2c address " << erraddr[addr] <<endl;
     }
   }
 // Release
 for (int lc =0 ; lc < nrPB ; lc++) delete  Apv25I2c[lc] ;
 
     return 0;
}
