#ifndef __COLDBOX
#define __COLDBOX
#include <vector.h>
#include <map.h>
#include <qthread.h>
#include "tempctrl.h" 
#include "tempctrlwien.h"
#include "tempctrldim.h"
/* this has already a call to most of the necessary include files" 
 */

/**  \class ColdBox
  

 \brief class  to control the temparture of a cold box .
        it can control more TempCtrl classes. 
	Each TempCtrl class controls one temperature sensor ( TemperatureChannel) and one 
	peltier (via the PowerChannel).
	For the moment the number of TempCtrl is fixed to two.
	If you don't need the second one set it to simulation in the configuration file 
	This class will also take care about threading if needed .
	For dim controlled cold box no thread will be started (ver 1.1) 

     \author W. Beaumont
     \date 20020327
     \version 0.1
  
     \author W. Beaumont
     \date 20021015
     \version 1.1  
     
  
*/ 

#ifdef QT_THREAD_SUPPORT 
#define COLDBOXTRHEADSUPPORT
#endif

class HumidityCtrlInterface;


class ColdBox :public IdExchange,  public  QThread
{  // definitions 
  friend class ColdBoxImpl;
  typedef   vector<TempCtrlInterface*>::const_iterator tcv;  
  enum CtrlStatus{UNKNOWN,NOTSTARTED,NORMAL,UNDERTEMPERATURE,OVERTEMPERATURE };
  private:
  bool fAutoCtrl;
  /** if thread of control should be started (so not for DIM )*/
  bool fThreadReq;
  virtual void  run()  ;
   bool fAutoCtrlActive;
   unsigned short int fCtrlLoopCnt;
  protected:
   bool fRunTrhread; 
   bool getAutoCtrlStatus(){return fAutoCtrlActive;} 
  vector <TempCtrlInterface*> fTempCtrl;
  bool fInitialized;
  /** special for switched polarity Daqs with common dio line*/
  int fPolStatus; 
  /** retrieved from the first temperature channel */
  BoxCharacteristics* fBoxChar;
  
  HumidityCtrlInterface* fHCtrl;
  /** last humidity set by the humidity ctrl
      if no humidity ctrl active it returns 0
  */
  float fActualHumidity;
  float fTargetTemp; //< the request temperature for the system
  float fActualTemp ; //< the last measured temperature of the system from tempctrl channnel 1
 
  float fActualPower;//< the actual power set on the power(Peltier) device tempctrl channnel 1
  CtrlStatus fCtrlStatus ;
  public :
    /** the TempCtrl initialization using the configuration file 
        @param deviceid the device Id as defined 
    */
    ColdBox(int deviceid );
  // 
    //  TempChannel  connected to the temperature probe
    // Power Channel the Dac channel connected to the steering power supply
    //  for the peltier element
    // information stored in configuration file

  


  virtual ~ColdBox();
 
  /**   \brief this call updates the steering value to achieve the required temperature
    should be called on regulair base. 
   returns 0 if o.k
   -1 if temperture read is not updated
   -5 unknow error 
  */
  int control();
  unsigned short int getCtrlLoopCounter(){ return fCtrlLoopCnt;}
  int getNrTempCtrls( ){ return fTempCtrl.size();}
  void setTargetTemperature (float target); 
  float getTargetTemperature () { return fTargetTemp ;}
  /** get the actual temperature from the first TempCtrl channel */ 
  float getActualTemperature () ;
  /** get the actual temperature from the i th TempCtrl channel */ 
  float getActualTemperature (int i) ;


  /** returns the actual power set for temp ctrl 0 */
  float ColdBox::getActualPower();
  /** returns the actual power set for temp ctrl i 
      @param i the tempctrl of which the power is requested
      @return the power if the channel exist else returns -1
  */   
  float getActualPower(int i) ;
  
  void cleanup();
  void printColdBox ();
  bool activeHumidityCtrl(){ 
    if(fHCtrl) return true; else return false;
  }
  
  /** gives the pointer to the humidity ctrl to use as humidity probe */
  HumidityCtrlInterface* getHumidityCtrl(){return fHCtrl;}
  /** get humidity from the humidity ctrl device , force and readout
      of the humidity probe
  */
  float getHumidity();
  //void* controlThread( ) ; where this comes from
  /** get the actual humidity ( updated during the last control
      doesn't force a real readout of the humidity ctrl probe
  */
  float getActualHumidity(){return fActualHumidity;}
  /** if maximum allowed humidity is reached it returns true
      this checks the actual humidity against the maximum humidity
  */
  bool maxHumidityReached();
  /** Checks if the cold box is defined 
      @return true is the  cold box was defined in the xml file 
  */
  bool isInitialized(){return fInitialized;}

  int startAutoCtrl()  ;
  bool getAutoCtrl(){return fAutoCtrlActive;}

  int stopAutoCtrl();

};




#endif
