#ifndef DIMSENSSERVER
#define DIMSENSSERVER

#include <dis.hxx>
#include <unistd.h>
#include <dimchannelnames.h>
#include <vector>
#include "temperaturechannelinterface.h"
#include "tempctrlinterface.h"
#include "humiditychannelinterface.h"
#include "hvchannelinterface.h"
#include "currentchannelinterface.h" 
/**  \class  DimSensServer
     \brief  Server interface class for slow remote slow control and cold box
     This class can be used as part of a program that readout temperatue probes, 
     humidity probes and take care of the control of a cold box. 
     ( and status information but this is not implemented yet).
     For usage look at the program testdimsensserver.
     In verson 2.0 temp channels etc are introduced 
     the set and get temperature ,humidity are removed 
     \version 1.0
     \date 2002 04 29
     \author W. Beaumont University Antwerpen 
     \version 2.0
     \date 2002 101 15
     \author W. Beaumont UA  , O. van der AA LLN 
 */


class DimSensServer : public  DimChannelNames, public DimServer
{ public:

  typedef vector<TemperatureChannelInterface*> tempchvec;
  typedef vector<TempCtrlInterface*> tempctrlvec;
  typedef vector<HumidityChannelInterface*> humchvec;
  typedef vector<HVChannelInterface*> hvchvec;
  typedef vector<CurrentChannelInterface*> curchvec;
  DimSensServer(int servernr, tempchvec tempsens , humchvec humiditysen, 
		tempctrlvec tempctrls, hvchvec hvchs, curchvec curchs );

  void commandHandler();
  DimCommand* fSensSrvCmd;
  DimService* fSensSrvAck;
  vector<DimCommand*> fDimTCmd; // T= temperature
  vector<DimService*> fDimTSrv;
  vector<DimCommand*> fDimHCmd; // H = humidity 
  vector<DimService*> fDimHSrv;
  vector<DimCommand*> fDimCurCmd; // Cur  = Current channel 
  vector<DimService*> fDimCurSrv;
  vector<DimCommand*> fDimCCmd; // C = Coldbox temp ctrl 
  vector<DimService*> fDimCSrv;
  vector<DimCommand*> fDimVCmd; // V = HV control (V and HV_I)
  vector<DimService*> fDimVSrv;
  bool getCtrlActive(int ch);
 private:
 tempchvec fTempSensors;
 humchvec fHumiditySensors;
 tempctrlvec fTempCtrls;
 hvchvec fHVUnits;
 curchvec fCurrentSensors;
 float fAnswer[255];
};

#endif
