#ifndef __HUMIDITYCHANNELLT
#define __HUMIDITYCHANNELLT
#include "CalibrationValues.h"
#include <adcchannelinterface.h>
#include "humiditychannelinterface.h"


/** \brief humidity  channel interface 
     
     
     The HumidityChannelLt  will be the interface class
     for the long term test station.
     It will take care of connecting this object to the 
     temperature probe with devic ID , what is defined in
     the lt.xml file.
     It also takes care of instantiation of necessary 
     I/O channels like ADC , serial ports .. 
     For an example how to use it look at
   
     \author W. Beaumont University Antwerpen 
     \version 1.0 
     \date 2001 05 23 
     \version 1.1 
     \date 20030201 
 
*/



class HumidityChannelLt : public HumidityChannelInterface
 
{
 private:
   HumidityChannelInterface * fT;
   int fType;
   AdcChannelInterface* fADCref; 
   // this is needed for the Humirel sensor 
  

 public:

  /// constructor, the deviceID is used as key to connect the correct probe to this object
   HumidityChannelLt( int deviceID  ) ;
   virtual ~HumidityChannelLt();
   /** this function returns the ref voltage adc value but for most implementaton it returns -1 (not used)
  */
   virtual float getRefVolt(){return fT->getRefVolt();}
  virtual float getHumidity() 
    { return fT->getHumidity();} 
  /** this function returns the input value for the implementation this can be used for 
      calibration purposes. For adc channel related this can be the adc value for 
      devices that provides already a humidity (like the dim devices) the input will be the same 
      as the output This is most cases this is not the actual value but the last readout value
      so to have the correct relation you need to do first the getHumidity and then getInputValue
  */
  virtual float getInputValue(){ return fT->getInputValue();}   

  virtual void info()
    { fT->info();
   }


};


#endif
