#ifndef __HUMIDITYCHANNELTRHX_SENS
#define __HUMIDITYCHANNELTRHX_SENS
#include "TShare.h"
#include "TDeviceControl.h"
#include "CalibrationValues.h"
#include "trhxdescription.h"
//#include "trhx.h"
#include "humiditychannelinterface.h"
/** \brief humidity channel class for the trhx sens humidity probe connection
  
     
     
     The class HumidityConversion takes care of the calibration issues
    
    
     \author W. Beaumont University Antwerpen 
     \version 1.0 
     \date 2001 10 01 
     
*/

class HumidityChannelTrhxSens :  public HumidityChannelInterface  

{ private:
   
  ChannelDescription* fCd;
  pid_t fPid;
  TShare *fMemoryClient;
  TrhxDescription *fTrhx;
  TrhxDescription::sensbuffer   *fSens;
  float fHumidityTrhx;
  bool fInitialized;
  int AttachTrhxHumidityChannel    (int serial_nr, int channelnr );
 public:
  /** \brief constructor 
      the Trhx humidity probe should be connected to a thrx server 
      It attach itself to the trhx server
      During instantiation a object CalExchange of the type HumidityCalValues should exist.
      The values in this class are copied into the CalExchange of the type HumidityCalValues 
      inherted class of this object. The member type of CalExchange should be HumidityConverDef::TRHXSENS
      @param TrhxBaseComPort  com port number to which the TRHX  base is connected.
      @param channel : the sensor channel number
  */
   HumidityChannelTrhxSens( int TrhxBaseComPort  ,CalExchange <HumidityConversionDef::HumidityCalValues>* c  ) ;
   
    /// returns the actual temperature 
    virtual float getHumidity() ;
    virtual float getInputValue(){ return fHumidityTrhx;}
    /// print some information about the temperature channel object only for debug reasons
    virtual void info()
    { double  humidity; 
      humidity = (*fSens)[TRHXHumidity][ getChannelNr()];
      humidity = Adc2HumidityConversion((float)humidity);
      printHCvalues((float) humidity );
    }

  /// returns the pointer to the ThrxDescription
    TrhxDescription* getTrhxDescripion() { return fTrhx;} 
    virtual void release();
    virtual ~HumidityChannelTrhxSens();
};

#endif 
