#ifndef HUMIDITYCTRLINTERFACE 
#define HUMIDITYCTRLINTERFACE

#include "humiditychannelinterface.h"

class ColdBox;

/**  \class HumidityCtrlInterface
  

 \brief interface class   to control thethe humidity 
  with a humidity probe and a gas flow interface 
    version 0.10 minimum implementation
     \author W. Beaumont
     \date 20021122
     \version 0.10
     even don't compile 
     
*/ 


class HumidityCtrlInterface: public HumidityChannelInterface 
{ 
  // definitions
  
  public :
    HumidityCtrlInterface
     (CalExchange <HumidityCtrlDef::HumidityCtrlValues>* c){ 
    fInitialized=false;
    fCtrlStatus=NOTSTARTED;
    fCV= new CalExchange <HumidityCtrlDef::HumidityCtrlValues>(* c);
    fHcv=fCV->getCalValues();
  }
   virtual ~HumidityCtrlInterface(){delete fCV;}
 
  /**   \brief this call updates the steering value 
	to achieve the required 	humidity 
	should be called on regulair base. 
	returns 0 if o.k
	-1 if humidity  read is not updated
	-5 unknow error 
  */
  virtual int control()=0;
  /** if maximum allowed humidity is reached it returns true
      this checks the actual humidity against the maximum humidity
  */
  bool maxHumidityReached(){ 
    return fActualHumidity > fHcv->fMaxHumidity;
  }
  /** sets  the gas flow to zero   
   */
  virtual void setZeroGasFlow()=0; 
  /** sets the gas flow to maximum */
  virtual void setMaxGasFlow()=0;
  virtual void setTargetHumidity(float target)=0; 
  virtual  float getTargetHumidity ()=0;
  virtual float getActualHumidity (){return  fActualHumidity;}

  // static is only switch on 
  enum CtrlStatus { UNKNOWN,NOTSTARTED,NORMAL,DRYING ,GASSTOP  ,STATIC,
		    UNDERHUMIDITY,OVERHUMIDITY };

 
  /** for direct cold box control */
   virtual void    setGasFlow(float flow )=0;
   /** return the value to which the gasflow is set  */
   virtual float getGasFlow()=0;
    virtual int getType()=0;
 public:
   // humiditychannel functions 
  virtual bool isInitialized(){return fInitialized;}
  virtual CalExchange <HumidityCtrlDef::HumidityCtrlValues>*
    getHumidityCtrlCalValues(){ return fCV;}
 protected:
  bool fInitialized;
  CtrlStatus fCtrlStatus;
  float fActualHumidity;
  CalExchange <HumidityCtrlDef::HumidityCtrlValues>* fCV;
  HumidityCtrlDef::HumidityCtrlValues* fHcv;  
};

#endif
