#ifndef __POWERCHANNEL
#define __POWERCHANNEL
#include "CalibrationValues.h"
#include <dacchannelinterface.h>
#include "powerchannelinterface.h"
/** \brief power channel class def file
  
     
     The PowerChannel for Voltage contrlolled current source to a  DAC output 
     The class PowerConversions takes care of the calibration issues
    
    
   
     \author W. Beaumont University Antwerpen 
     \version 0.1 
     \date 20011004 
     
 
*/
class TDeviceControl;
class TShare;


class PowerChannel :  public PowerChannelInterface  , public PowerConversion 


{ private:
   DacChannelInterface* fDacCh;
   unsigned short  fActualDacValue;  // in DAC counts  
   int fLastSpill;
   bool fInitialized;
  
 public:
  /** \brief constructor 
      the current source should be connected to a DAC  channel. 
      The implementation of the DAC  channel should provide the functions 
      define in the DacChannelInterface class.
      During instantiation a object CalExchange of the type TempCalValues should exist.
      The values in this class are copied into the CalExchange of the type TempCalValues 
      inherted class of this object
      @param DacCh pointer to a real DAC  channel 
      @param c pointer to a CalExchange <TempConverDef::TempCalValues> that will be used for intialisation
  */
   PowerChannel( DacChannelInterface  *DacCh ,CalExchange <PowerConversionDef::PowerCalValues>* c  ) ;
    /// returns the device ID of the power channel
    virtual int  getId()  { return CalExchange <PowerConversionDef::PowerCalValues>::getId();}
    /// set the ouptut device to deliver the actual power  returns the new DAC setting (in Volts)
    
    /// the destructor doesn't do special things (so also not deleting the dac object) 
    virtual ~PowerChannel() {;};
    virtual unsigned short  setPower(float power) ;
    
    /// read back the set power 
    /** depends from DAC if this is the last set value (software read back)
	or a read back from the (hardware) device setting
    */ 
    virtual float getPower( );
 
    /// print some information about the power channel object only for debug reasons
    virtual void info()
    {   
      PrintPCVvalues( fActualDacValue);
    }

    /**  \ brief returns the pointer to the DAC channel that this channel is using 
      This is not part of the PowerChannelInterface definition
      But is very usefull to delete the Dac object if only the PowerChannel object is known
    */
  DacChannelInterface* getDacChannelPointer(){ return fDacCh;}
};

#endif 
