#ifndef __POWERCHANNELPOLCOM
#define __POWERCHANNELPOLCOM
#include "diochannelinterface.h"
#include "powerchannel.h"
/** \brief power channel class def file
     
    Extention for the power channel class.
    By an dio channel the polarity can be exchanged 
    
   
     \author W. Beaumont University Antwerpen 
     \author  L. Demarial , G.   INFN Torino  
     \version 0.1 
     \version 0.2
     \date 20020405
     \date 20020902 
     
 
*/


class PowerChannelPol :  public PowerChannel
{ private:
   DioChannelInterface* fDioCh;
   bool fInitialized;
   int fNegativeValue;
   int fPositiveValue;
   
   static int pol_status;
   
   bool fPositive; //< the value of the pol output 
 public:
  /** \brief constructor 
      the current source should be connected to a DAC  channel. 
      The implementation of the DAC  channel should provide the functions 
      define in the DacChannelInterface class.
      During instantiation a object CalExchange of the type TempCalValues should exist.
      The values in this class are copied into the CalExchange of the type TempCalValues 
      inherted class of this object
      @param DacCh pointer to a real DAC  channel 
      @param DioCh pointer to a real Dio channel that exchange the polarity 
      @param c pointer to a CalExchange <TempConverDef::TempCalValues> that will be used for intialisation
  */
   PowerChannelPol( DacChannelInterface  *DacCh ,
		 DioChannelInterface *DioCh,
		 CalExchange <PowerConversionDef::PowerCalValues>* c  ) ;
    /// returns the device ID of the power channel
 
    /// the destructor doesn't do special things (so also not deleting the dac object) 
   virtual ~PowerChannelPol() ;

    /// extention of the powerchannel set power function 
    virtual unsigned short  setPower(float power) ;
    
    virtual bool heater(){return true;}

  
  /**  \ brief returns the pointer to the Dio  channel that this channel is using 
      This is not part of the PowerChannelInterface definition
      But is very usefull to delete the Dac object if only the PowerChannel object is known
    */
   DioChannelInterface* getDioChannelPointer(){ return fDioCh;}

};

#endif 
