#ifndef __TEMPCTRL 
#define __TEMPCTRL

#include "TDeviceControl.h"
#include "tempctrlinterface.h"
#include <adcchannelinterface.h>
#include "powerchannelinterface.h"
#include "temperaturechannelinterface.h"
#define MAX_HISTORY_ENTRIES 10

//class ColdBox;

/**  \class TempCtrl
  

 \brief class  to control the temparture 
     reads the temperature from 0ne channel   and controls a power channel output
     this power channel  output is a DAC assumed to be coneected to a device that controlles the 
     power stream ( Peltier) or heater
     The class itself doesn't provide an automatic activation of the control.
     the method control should be called regular to adjust the steering.
     If cotrol is not called the steering of the power will not change (in respect to the last 
     call of control) .
     The class can also be used as a temperaturechannel 
     \author W. Beaumont
     \date 20010112
     \version 0.10
     \date 20011004
     \version 2.0  no change in interface but  powerchannel is used instead of DAC and power conversion
     \date 20020410
     \version 2.1 
     Cold box takes over the automatic control temperaturechannelintrface added 
     \date 20020501
     \version 2.2
     TempCtrlInterface is added registrer functions are moved to tempctrlinterface 
*/ 


class TempCtrl: public TempCtrlInterface 
{  public :
    /** the TempCtrl initialization using the configuration file 
        @param deviceid the device Id as defined 
    */
    TempCtrl(int deviceid );
    virtual ~TempCtrl();
 
  /**   \brief this call updates the steering value 
	to achieve the required temperature
	should be called on regulair base. 
	returns 0 if o.k
	-1 if temperture read is not updated
	-5 unknow error 
  */
  virtual int control();
  
  
  /** set the power to inactive   
   */
  virtual void setZeroPower(); 
  virtual void setTargetTemperature (float target); 
  virtual  float getTargetTemperature () { return fTargetTemp ;}
  virtual float getActualTemperature () { return fActualTemp ;}
  
  // temperaturchannel functions
  virtual float getInputValue(){return fTemperature-> getInputValue();}
  virtual float getTemperature() { return fActualTemp ;}
  virtual void info(){ printTempCtrl();}
  // this has to be changed 
  virtual bool isInitialized(){return true; }

  // definitions 
  
  typedef struct {
    float temp;  // temperature
    int  time; // time stamp
  } temp_history;
 
//protected :
  /** for cold box with synchrone control set direct the power  */
   virtual void    setPower(float power ) ;
  /** for cold box with synchrone control set direct the power  */
   virtual float getActualPower(); 
   virtual int  getType()
      {return  fBoxChar->getType() ;}
  
  private:

  int fPowerChannelId; //< still needed ? 
  PowerChannelInterface* fPowerChannel;
  TemperatureChannelInterface* fTemperature;
  BoxCharacteristics* fBoxChar; 
  float  fMaxDelta ; //< maximal temperature difference without action
  temp_history  fTh[MAX_HISTORY_ENTRIES ]; //< temperature history 
  float fIntegratedError ;
  float fTargetTemp; //< the request temperature for the system
  float fActualTemp ; //< the last measured temperature of the system
  float fDeltaTemp;//< the difference between 
  float fActualPower;//< the actual power set on the power(Peltier) device
  bool fHeater; //< should be true if the power channel can also heat
  bool fRefferenceToChannel;
  // moved to the interface 
  // CtrlStatus fCtrlStatus ;
   
  float IntegratedTempError();
  float DifferentiatedError();
  float setTempHistory(float x);

  void getPowerConversion(int deviceId);
  
 protected: 
  float getIntegratedError() { return fIntegratedError;}
  void cleanup();
  void   printTempCtrl();
 

};

#endif
