#ifndef __TEMPCTRLDIM 
#define __TEMPCTRLDIM
#include <dic.hxx>
#include "dimchannelnames.h"
#include "tempctrlinterface.h"

/**  \class TempCtrlDim
  

 \brief class  to control the temparture of a remote temp control device
      connected by a dim socket 
     reads the temperature from 0ne channel   and controls a power channel output
     when control is asked the control is active
     if setzero power the control of the rempote box should be set to zero. 
     The class can also be used as a temperaturechannel 
     \author W. Beaumont
     \date 20020501
     \version 1.0
     
*/ 


class TempCtrlDim: public TempCtrlInterface ,
		   public DimChannelNames,
		   public  DimClient  
{  

public :
    /** the TempCtrlDim initialization using the configuration file 
        @param deviceid the device Id as defined 
    */
    TempCtrlDim(int deviceid, HumidityCtrlInterface* hc=0);
    virtual ~TempCtrlDim();
 
  /**   \brief this call updates the steering value 
	to achieve the required temperature
	should be called on regulair base. 
	returns 0 if o.k else communication errror
  */
  virtual int control();
  
 
  /** set the power to inactive   
   */
  virtual void setZeroPower(); 
  virtual void setTargetTemperature (float target); 
  virtual  float getTargetTemperature () { return fTargetTemp ;}
  virtual float getActualTemperature () { return fActualTemp ;}
 
 // temperaturchannel functions 
  virtual float getInputValue(){return fActualTemp;}
  virtual float getTemperature() { return fActualTemp ;}
  virtual void info(){ printTempCtrl();}
 
  virtual bool isInitialized(){return fIsInitialized; }
  // definitions 

//protected :
  /** for cold box with synchrone control set direct the power  */
   virtual void    setPower(float power ) ;
  /** for cold box with synchrone control set direct the power  */
   virtual float getActualPower(); 
   virtual int  getType()
      {return  fBoxChar->getType() ;}
  
   void infoHandler();



  private:
   // HumidityCtrlInterface* fHumidityCtrl; defined in interface; 
  BoxCharacteristics* fBoxChar; 
  float  fMaxDelta ; //< maximal temperature difference without action
  float fIntegratedError ;
  float fTargetTemp; //< the request temperature for the system
  float fActualTemp ; //< the last measured temperature of the system
  float fDeltaTemp;//< the difference between 
  float fActualPower;//< the actual power set on the power(Peltier) device
  bool fHeater; //< should be true if the power channel can also heat
  bool fIsInitialized;
  bool fRefferenceToChannel;
  CtrlStatus fCtrlStatus ;
  DimInfo* fTempCtrlinfo;
  char fCmd[80];
  float fSend[256];
  float  fRecBuf[256];
  int fNsent;
  int fNreceived;
  
 protected: 
  
  void   printTempCtrl();
  int  sendCmd();
  int wait();
};

#endif
