#ifndef __TEMPCTRLDUMMY
#define __TEMPCTRLDUMMY

#include "tempctrlinterface.h"
#include "temperaturechannelinterface.h"
#define MAX_HISTORY_ENTRIES 10

class ColdBox;

/**  \class TempCtrlDUMMY 
  

 \brief temperature dummy implementation , not supported by the lt.xml file 
    
     \version 1.0      
     \date 20021015
  
*/ 


class TempCtrlDummy: public TempCtrlInterface 
{  public :
    /** the TempCtrl initialization using the configuration file 
        @param deviceid the device Id as defined 
    */
    TempCtrlDummy(int deviceid );
    virtual ~TempCtrlDummy();
 
  /**   \brief this call updates the steering value 
	to achieve the required temperature
	should be called on regulair base. 
	returns 0 if o.k
	-1 if temperture read is not updated
	-5 unknow error 
  */
  virtual int control();
  
  
  /** set the power to inactive   
   */
  virtual void setZeroPower(); 
  virtual void setTargetTemperature (float target); 
  virtual  float getTargetTemperature () { return fTargetTemp ;}
  virtual float getActualTemperature () { return fActualTemp ;}
  
  // temperaturchannel functions 
  virtual float getTemperature() { return fActualTemp ;}
  virtual float getInputValue() { return fActualTemp ;}
  virtual void info(){ printTempCtrl();}
  // this has to be changed 
  virtual bool isInitialized(){return true; }

  // definitions 
  
  typedef struct {
    float temp;  // temperature
    int  time; // time stamp
  } temp_history;
 
//protected :
  /** for cold box with synchrone control set direct the power  */
   virtual void    setPower(float power ) ;
  /** for cold box with synchrone control set direct the power  */
   virtual float getActualPower(); 
   virtual int  getType()
      {return  fBoxChar->getType() ;}
  
  private:

  int fPowerChannelId; //< still needed ? 
 
  BoxCharacteristics* fBoxChar; 
  float  fMaxDelta ; //< maximal temperature difference without action
  temp_history  fTh[MAX_HISTORY_ENTRIES ]; //< temperature history 
  float fIntegratedError ;
  float fTargetTemp; //< the request temperature for the system
  float fActualTemp ; //< the last measured temperature of the system
  float fDeltaTemp;//< the difference between 
  float fActualPower;//< the actual power set on the power(Peltier) device
  bool fHeater; //< should be true if the power channel can also heat
  bool fRefferenceToChannel;
  CtrlStatus fCtrlStatus ;
   
  float IntegratedTempError();
  float DifferentiatedError();
  float setTempHistory(float x);

  void getPowerConversion(int deviceId);
  
 protected: 
  float getIntegratedError() { return fIntegratedError;}
  void cleanup();
  void   printTempCtrl();
 

};

#endif
