#ifndef __TEMPCTRLINTERFACE 
#define __TEMPCTRLINTERFACE

#include <string>
#include "CalibrationValues.h"
#include "temperaturechannelinterface.h"
#define MAX_HISTORY_ENTRIES 10

class ColdBox;
class TempCtrlReg;
class  HumidityChannelInterface;
/**  \class TempCtrlInterface
  

 \brief interface class   to control the temparture 
    
     \author W. Beaumont
     \date 20020501
     \version 1.00
      \author W. Beaumont
     \date 20030724
     \version 1.20
     
*/ 

class HumidityCtrlInterface;
class TempCtrlInterface: public TemperatureChannelInterface
{ 
 
 public:
  enum CtrlStatus { UNKNOWN,NOTSTARTED,NORMAL,HEATING , COOLING ,
		    UNDERTEMPERATURE,OVERTEMPERATURE, FASTHEATING,
		    FASTCOOLING,PASIVECOOLING,PASIVEHEATING
  };
 
  protected :
    TempCtrlReg* fRegTc;  
  friend class ColdBox;
  HumidityCtrlInterface* fHumidityCtrl;
 // new temperature channels
  TemperatureChannelInterface* fWaterTemp; 
  TemperatureChannelInterface* fHotSideTemp;
  float fWaterTemperature;
  float fHotSideTemperature;
  CtrlStatus fCtrlStatus ;
  public :
    /** general constructor. 
	One can add a humidity channel but if one use it one has to
	check if there is a valid address pointer ( != 0)
    */
    TempCtrlInterface(HumidityCtrlInterface* hc=0){  
      fHumidityCtrl=hc;
      fWaterTemp=0; fWaterTemperature=20;
      fHotSideTemp=0; fHotSideTemperature=0;
     fCtrlStatus =NOTSTARTED;
  } 
  // 
    //  TempChannel  connected to the temperature probe
    // Power Channel the Dac channel connected to the steering power supply
    //  for the peltier element
    // information stored in configuration file
  
    virtual ~TempCtrlInterface(){;}
    
  /**   \brief this call updates the steering value to achieve the required temperature
    should be called on regulair base. 
   returns 0 if o.k
   -1 if temperture read is not updated
   -5 unknow error 
  */
  virtual int control()=0;
  /** set the power to inactive   
   */
  virtual TempCtrlReg* getTempCtrlReg(){return fRegTc;}
  virtual void setZeroPower()=0; 
  virtual void setTargetTemperature(float target)=0; 
  virtual  float getTargetTemperature ()=0;
  virtual float getActualTemperature ()=0;

  virtual float getHotSideTemperature(){
    return fHotSideTemperature;
  }
  
  virtual float getWaterTemperature(){
    return fWaterTemperature;
  }
  
  /** returns true if the temperature ctrl has a water temperatur probe
   */
  virtual bool waterTemperatureProbe(){ 
    return fWaterTemp==0 ? false:true;
  }
  /** returns true if the temperature ctrl has a hotside  temperatur probe
   */
  virtual bool hotSideTemperatureProbe(){
    return fHotSideTemp==0 ? false:true;
  }
  CtrlStatus getCtrlStatus(){return fCtrlStatus ;}

  std::string ctrlStatusString(){
    return ctrlStatusDecoding( fCtrlStatus); 
  }

  std::string ctrlStatusDecoding( CtrlStatus c){
    std::string s;
    switch (c){
      case UNKNOWN: s="UNKNOWN"; break;
      case NOTSTARTED: s="NOTSTARTED"; break; 
      case NORMAL: s="NORMAL"; break;
      case HEATING: s=" HEATING"; break;
      case COOLING: s="COOLING"; break;
      case UNDERTEMPERATURE: s="UNDERTEMPERATURE"; break; 
      case OVERTEMPERATURE:s="OVERTEMPERATURE"; break;
      case FASTHEATING:s="FASTHEATING"; break;
      case FASTCOOLING:s="FASTCOOLING"; break;
      case PASIVECOOLING: s="PASIVECOOLING"; break;
      case PASIVEHEATING: s="PASIVEHEATING"; break;
      default:  s="Undefined state new ?"; break;
    }
    return s;
  }
  
 
  //protected:
  /* for cold box with synchrone control set direct the power  */
   virtual void    setPower(float power )=0;
  /* for cold box with synchrone control set direct the power  */
   virtual float getActualPower()=0;
    virtual int getType()=0;

 protected:
    

 public:
   // temperaturchannel functions 
 
/*      virtual float getTemperature()=0; */
/*      virtual void info()=0; */
/*      virtual bool isInitialized()=0; */
};

#endif
