#ifndef __TEMPCTRLQ 
#define __TEMPCTRLQ

#include <unistd.h>
#include <qthread.h>
#include <stdiostream.h>
#include "TDeviceControl.h"
// #include "Adcctrl.h"
// #include "Dacctrl.h"
#include "CalibrationValues.h"
#include <adcchannelinterface.h>
#include "powerchannelinterface.h"
#include "temperaturechannelinterface.h"

#define MAX_HISTORY_ENTRIES 10

/**  \class TempCtrlQ
  

 \brief class  to control the temparture 
     reads the temperature from 0ne channel ( or more but is not implemented)
     and controls a power channel output
     this power channel  output is a DAC assumed to be coneected to a device that controlles the 
     power stream ( Peltier) or heater
     the temperature controll is a permanent running system so the real control
     ( StartTempCtrl) should be started from a thread 
      The class itself doesn't provide an automatic activation of the control.
      the method control should be called regular to adjust the steering.
      If cotrol is not called the steering of the power will not change (in respect to the last 
      call of control) .
     \author W. Beaumont
     \date 20010112
     \version 0.10
     \date 20011004
     \version 2.0  // no change in interface but internal powerchannel is used instead of DAC and power conversion
     
*/ 


class TempCtrlQ : public  QThread
{ 
  // definitions 
 
  typedef struct {
    float temp;  // temperature
    int  time; // time stamp
  } temp_history;
  enum CtrlStatus { UNKNOWN,NOTSTARTED,NORMAL,UNDERTEMPERATURE,OVERTEMPERATURE };
  private:

  int fDeviceId ;
  bool fAutoCtrl;
  bool fAutoCtrlActive;
  int fPowerChannelId; //< still needed ? 
  PowerChannelInterface* fPowerChannel;
  TemperatureChannelInterface* fTemperature;
  BoxCharacteristics* fBoxChar; 
  float  fMaxDelta ; //< maximal temperature difference without action
  temp_history  fTh[MAX_HISTORY_ENTRIES ]; //< temperature history 
  float fIntegratedError ;
  unsigned short fActualDacValue; // last value set to the DAC 
  float fTargetTemp; //< the request temperature for the system
  float fActualTemp ; //< the last measured temperature of the system
  float fDeltaTemp;//< the difference between 
  float fActualPower;//< the actual power set on the power(Peltier) device
  CtrlStatus fCtrlStatus ;
   
  float IntegratedTempError();
  float DifferentiatedError();
  float setTempHistory(float x);

  void getPowerConversion(int deviceId);
  public :
    /** the TempCtrl initialization using the configuration file 
        @param deviceid the device Id as defined 
    */
    TempCtrlQ(int deviceid );
  // 
    //  TempChannel  connected to the temperature probe
    // Power Channel the Dac channel connected to the steering power supply
    //  for the peltier element
    // information stored in configuration file

  


  virtual ~TempCtrlQ();
 
  /**   \brief this call updates the steering value to achieve the required temperature
    should be called on regulair base. 
   returns 0 if o.k
   -1 if temperture read is not updated
   -5 unknow error 
  */
  int control();
  // inline float ReadTemperature()
  //{ // getAdcData(ActualAdcValue );
  // return  Adc2Tconversion (fActualAdcValue ); 
	//  }  
 
  void setTargetTemperature (float target); 
  float getTargetTemperature () { return fTargetTemp ;}
  float getActualTemperature () { return fActualTemp ;}
  // float  getActualAdcValue (){ return fActualAdcValue;}
  unsigned short   getActualDacValue (){ return fActualDacValue;}
  float getActualPower() { return fActualPower;}
  float getIntegratedError() { return fIntegratedError;}
  int getId(){return fDeviceId;}
  void cleanup();
  void printTempCtrl ();


  


  virtual void  run()  ;
 

  int stopAutoCtrl();

    
};


/*  ostream& operator<<(ostream& s, TempCtrl TC  ) 
  {  
    s << "T " << TC.getActualTemperature()<<"C Target:  "<< TC.getTargetTemperature() ; 
    s << " C  Power"  << TC.getActualPower() << "W =("<< <<" ADC counts ) \n" ; 
    return s ;
 }  */



#endif
