#ifndef __TEMPCTRLWIEN 
#define __TEMPCTRLWIEN

#include "tempctrlinterface.h"
#define MAX_HISTORY_ENTRIES 10

class ColdBox;
class PowerChannelInterface;

/**  \class TempCtrlWine
  

 \brief class  to control the temparture 
     Optimized temperature control for the Wien Cold box. 
     \author W. Beaumont L. Dimaria G. Favore 
     \date 20020901
     \version 1.00  based on TempCtrl
     \date 20011004
    
*/ 


class TempCtrlWien: public TempCtrlInterface 
{  public :
    /** the TempCtrl initialization using the configuration file 
        @param deviceid the device Id as defined 
    */
    TempCtrlWien(int deviceid, int* PolStatus,HumidityCtrlInterface* hc=0 );
    virtual ~TempCtrlWien();
 
  /**   \brief this call updates the steering value 
	to achieve the required temperature
	should be called on regulair base. 
	returns 0 if o.k
	-1 if temperture read is not updated
	-5 unknow error 
  */
  virtual int control();
  
  
  /** set the power to inactive   
   */
  virtual void setZeroPower(); 
  virtual void setTargetTemperature (float target); 
  virtual  float getTargetTemperature () { return fTargetTemp ;}
  virtual float getActualTemperature () { return fActualTemp ;}
  
  // temperaturchannel functions 
  virtual float getInputValue(){return fTemperature-> getInputValue();}
  virtual float getTemperature() { return fActualTemp ;}
  virtual void info(){ printTempCtrl();}
  // this has to be changed 
  virtual bool isInitialized(){return true; }

  // definitions 
  
  typedef struct {
    float temp;  // temperature
    int  time; // time stamp
  } temp_history;
 
//protected :
  /** for cold box with synchrone control set direct the power  */
   virtual void    setPower(float power ) ;
  /** for cold box with synchrone control set direct the power  */
   virtual float getActualPower(); 
   virtual int  getType()
      {return  fBoxChar->getType() ;}
  
  private:

  int fPowerChannelId; //< still needed ? 
  PowerChannelInterface* fPowerChannel;
  TemperatureChannelInterface* fTemperature;
//TemperatureChannelInterface* fWaterTemp; defined in interface
//TemperatureChannelInterface* fHotSideTemp; defined in interface 
  
//  HumidityCtrlInterface* fHumidityCtrl; defined in interface
  BoxCharacteristics* fBoxChar; 
  float  fMaxDelta ; //< maximal temperature difference without action
  temp_history  fTh[MAX_HISTORY_ENTRIES ]; //< temperature history 
  float fIntegratedError ;
  float fTargetTemp; //< the request temperature for the system
  float fActualTemp ; //< the last measured temperature of the system
  float fDeltaTemp;//< the difference between 
  float fActualPower;//< the actual power set on the power(Peltier) device
  bool fHeater; //< should be true if the power channel can also heat
  bool fRefferenceToChannel;
  // now an interface member  
  //CtrlStatus fCtrlStatus ;
   
  float IntegratedTempError();
  float DifferentiatedError();
  float setTempHistory(float x);

  void getPowerConversion(int deviceId);
  
 protected: 
  float getIntegratedError() { return fIntegratedError;}
  void cleanup();
  void   printTempCtrl();
 

};

#endif
