#ifndef __TEMPERATURECHANNEL
#define __TEMPERATURECHANNEL
#include  <unistd.h>
#include <iostream>
#include <fstream>
#include "CalibrationValues.h"
#include <adcchannelinterface.h>
#include "temperaturechannelinterface.h"
/** \brief temperature channel class def file
  
     
     The TemperatureChannel for AD590 probes connected to the ADC board
     The class TempConversion takes care of the calibration issues
    
     
     For an example how to use it look in ScMonitor
   
     \author W. Beaumont University Antwerpen 
     \version 1.0 
     \date 2001 03 24 
      \version 1.1
     \date 2001 05 11
     \version 1.2 
     \date 2001 05 22
 
*/
class TDeviceControl;
class TShare;


class TemperatureChannel :  public TemperatureChannelInterface 


{ private:
   AdcChannelInterface* fAdcCh;
   float fActualAdcValue;
   int fLastSpill;
   bool fInitialized;

 private:
  /** check if there is a other spill
      if yes return flase
      else true
  */
   bool checkspill();
   

 public:
  /** \brief constructor 
      the AD590 temperature probe should be connected to a adc channel. 
      The implementation of the ADC channel should provide the functions 
      define in the AdcChannelInterface class.
      During instantiation a object CalExchange of the type TempCalValues should exist.
      The values in this class are copied into the CalExchange of the type TempCalValues 
      inherted class of this object
      @param AdcCh pointer to a real adc channel 
      @param c pointer to a CalExchange <TempConverDef::TempCalValues> that will be used for intialisation
  */
   TemperatureChannel( AdcChannelInterface  *AdcCh ,CalExchange <TempConverDef::TempCalValues>* c  ) ;
   
     /// true if the temperature channel is sucessfull intialized 
    virtual bool isInitialized() { return fInitialized;}

    /// returns the actual temperature 
    virtual float getTemperature() ;
    /// return the last readed adc value what is used to calculate the current
    virtual float getInputValue(){return fActualAdcValue;}
    /// print some information about the temperature channel object only for debug reasons
    virtual void info()
    {   
      PrintTCVvalues( fActualAdcValue);
    }

  /// returns the pointer to the ADC channel that this probe is using is outside the interface
    AdcChannelInterface* getAdcChannelPointer(){ return fAdcCh;}
};

#endif 
