#ifndef __TEMPERATURECHANNELINTERFACE
#define __TEMPERATURECHANNELINTERFACE
#include  <unistd.h>
#include <iostream>
#include <fstream>
#include "CalibrationValues.h"
//#include <adcchannelinterface.h>

/** \brief temperature channel interface class
     
     The  TemperatureChannelInterface  is a minium interface definition
     implementations are assumed to take care about calibration issues and 
     attachment to real temperature probes
     For an example how to use it look in ScMonitor
     
   
     \author W. Beaumont University Antwerpen 
     \version 1.0 
     \date 2001 05 22 
      \version 1.1
     \date 2001 05 11
     \version 1.2 
     \date 2001 05 22
 
*/

class TemperatureChannelInterface : public TempConversion

{ 
 public:
  TemperatureChannelInterface(){ }
 TemperatureChannelInterface( CalExchange <TempConverDef::TempCalValues>* c  ):
    TempConversion(*c) {}
   
  /// get the actual temperature
   virtual float getTemperature() =0;
   /** get the actual temperature
       @param basevalue the value of the sensor ,e.g. actual resistance  for NTC
    */
   virtual float getTemperature(float& basevalue) { 
     float temperature= getTemperature();
     basevalue=getBaseValue();return temperature;
   }
/** return the raw input value like an adc input */
   virtual float getInputValue()=0;
   /** \brief  send inforation about the temperature channel to cout ,use only for debug reasons
       it is allowed to implement a empty function here */
   virtual void info()=0;
   virtual bool isInitialized()=0;   
   virtual ~TemperatureChannelInterface() {};
  
   
};

std::ostream& operator<<(std::ostream& s, TemperatureChannelInterface& TC  );
 
std::ofstream& operator<<(std::ofstream& s, TemperatureChannelInterface& TC  );


#endif 
