#ifndef __TEMPERATURECHANNELLT
#define __TEMPERATURECHANNELLT
#include  <unistd.h>
#include <iostream.h>
#include <fstream.h>
#include "temperaturechannelinterface.h"
//#include "ltparser.h"


/** \brief temperature channel for long term test
  
     
     The TemperatureChannelLt provides the interface to a temperature probe
     The probe connected to this object depends on the device ID given in the lt.xml file
     The class takes care of   correct temperature channel and instanciate
     of lower level channels like adc channel if needed.
     
     For an example how to use it look in ScMonitor (not yet)
   
     \author W. Beaumont University Antwerpen 
     \version 1.0 
     \version 1.01
     \version 1.11
     \date 2001 05 22 
     \date 2001 06 18
     \date 2001 09 23
 
*/

class TemperatureChannelLt :  public TemperatureChannelInterface  


{ private:
  
   int fDeviceId;
   TemperatureChannelInterface * fT;
   int fType;
   bool fInitialized;

 private:
   /** \brief the methode that does the real work of the instantiation.
         the deviceID is used as key to connect the correct probe to this object
        @param  deviceID is the device ID in the lt.xml file
    */   
    void TemperatureChannelLt::InitLtTemperatureChannel( int device);
 
 public:
   /** \brief constructor, the deviceID is used as key to connect the correct probe to this object
       
    @param  deviceID is the device ID in the lt.xml file
   */ 
   TemperatureChannelLt(  int deviceID  ) ;
   /** \brief constructor, the bench posisition together with the order numbe 
       is used to get the device ID of the temperature channel which is connected to the module
       the deviceID is used as key to connect the correct probe to this object
       
    @param  ordernr  is the order number of the temperature inside the module 
    @param  benchpos is the bench position of the module. 
   */ 
   TemperatureChannelLt(   int benchpos,int ordernr );
   
   virtual ~TemperatureChannelLt();
     /// returns the actual temperature in [C]
  virtual float getTemperature() 
    { return fT->getTemperature();} 
  
  virtual float getInputValue(){ return fT->getInputValue();}
 /** get the actual temperature
       @param basevalue the value of the sensor ,e.g. actual resistance  for NTC
    */
  virtual float getTemperature(float& basevalue){ return fT->getTemperature(basevalue ); }
     
  /// true if the temperature channel is sucessfull intialized 
    bool isInitialized() { return fInitialized;}
 
  /// send information to the cout use only for debug reasons.
  virtual void info()
    { fT->info();
   }


};


#endif 
