#ifndef __TEMPERATURECHANNELTRHX_SENS
#define __TEMPERATURECHANNELTRHX_SENS
#include  <unistd.h>
#include <iostream.h>
#include <fstream.h>
#include "TShare.h"
#include "TDeviceControl.h"
#include "CalibrationValues.h"
#include "trhxdescription.h"
//#include "trhx.h"
#include "temperaturechannelinterface.h"
/** \brief temperature channel class for the trhx sens temperature
  
     
     for accessing the temperature of the trhx sens temperature
     The class TempConversion takes care of the calibration issues
    
    
     \author W. Beaumont University Antwerpen 
     \version 0.1 
     \date 2001 08 31 
     
*/

class TemperatureChannelTrhxSens :  public TemperatureChannelInterface  


{ private:
   
  ChannelDescription* fCd;
  pid_t fPid;
  TShare *fMemoryClient;
  TrhxDescription *fTrhx;
  TrhxDescription::sensbuffer   *fSens;
  TrhxDescription::ntcbuffer  *fNtc;
  bool fInitialized;
  float fTrhxTemperature;
  int AttachTrhxTempChannel    (int serial_nr, int channelnr );
 public:
  /** \brief constructor 
      the Trhx  temperature probe should be connected to a thrx server 
      It attach itself to the trhx server
      During instantiation a object CalExchange of the type TempCalValues should exist.
      The values in this class are copied into the CalExchange of the type TempCalValues 
      inherted class of this object. The member type of CalExchange should be TempConverDef::TRHXSENS
      @param TrhxBaseComPort  com port number to which the TRHX  base is connected.
      @param channel : the sensor channel number
  */
   TemperatureChannelTrhxSens( int TrhxBaseComPort  ,CalExchange <TempConverDef::TempCalValues>* c  ) ;
    /// returns the device ID of the temperature channel
    virtual int  getId()  { return CalExchange <TempConverDef::TempCalValues>::getId();}
    /// returns the actual temperature 
    virtual float getTemperature() ;
    virtual float getInputValue(){ return fTrhxTemperature;}

    /// print some information about the temperature channel object only for debug reasons
    virtual void info()
    {    double  temperature ; 
    temperature= (*fSens)[TRHXTemperature][ getChannelNr()];
       	temperature= Adc2Tconversion((float)temperature);
      PrintTCVvalues((float) temperature );
    }
    /// returns true if intializing is sucessful
     bool isInitialized() { return fInitialized;}

     /** Lock share memory access */
     virtual void  lock() {fMemoryClient->Lock();}
     
     /** Unlock share memory access */
     virtual void  unlock() {fMemoryClient->Unlock();} 

  /// returns the pointer to the ThrxDescription
    TrhxDescription* getTrhxDescripion() { return fTrhx;} 
    virtual void release();
    virtual ~TemperatureChannelTrhxSens();
};

#endif 
