#include <cstdio>
#include "coldbox.h"
#include "ltparser.h"
#include "daqerrormsg.h"
#include "tempctrldummy.h"
#include "lttypewrappers.h"
#include "humidityctrlinterface.h" 
extern LtParser*  ltparser;



ColdBox::ColdBox( int device )                                     
{ fInitialized=false;  
 fRunTrhread=false; 
 fHCtrl=0;
 fActualHumidity=0;
 fCtrlLoopCnt=0;
 CalExchange <BoxCharacteristicsDef::BoxCharValues>*     
          c = ltparser-> getColdBox(device);
 fPolStatus =1; 
 fAutoCtrlActive=false;
 if(!c)     
   { DT_WARN("ColdBox deviceId " << device << " not found" ) ;
   fInitialized=false;  fBoxChar=0;
   }
 else
   { fBoxChar = new BoxCharacteristics( c);
     fBoxChar->setCalValues(fBoxChar->getCalValues() );// ?? 
     setCalibrationId(c->getCalibrationId());
     int hmcid=fBoxChar->getCalValues()->fHumidityCtrlId;
     if( hmcid >=0) {
       fHCtrl=lttest::getHumidityCtrl(hmcid);
     }
     else{  fHCtrl=0;} 
     vector<int>* m= &(fBoxChar->getCalValues()->fTempCtrlId);
     for (  vector<int>::const_iterator p=m->begin();p!=m->end();p++){ 
       CalExchange <BoxCharacteristicsDef::BoxCharValues>* c_tc=
	   ltparser->getTempCtrlInfo(*p); 
       TempCtrlInterface* tc=0; 
       switch (c_tc->getType() )
	 { case BoxCharacteristicsDef::DIMCTRL:
	   { tc= new TempCtrlDim(*p);
	   }
	   break;
	   case BoxCharacteristicsDef::TEMPCTRLDUMMY: {
	     //DT_WARN("ColdBox deviceId " << device << "as not existing" ) ;
	     //fInitialized=false;  fBoxChar=0;
	     tc =new TempCtrlDummy(*p);
	   }
	   break;
	   case  BoxCharacteristicsDef::WIENCTRL:	     
	   case  BoxCharacteristicsDef::STANDARD:
	   default:
	   {// copy the tempctrl specific things first 
	     c->getCalValues()->fTemperatureId= 
	       c_tc->getCalValues()->fTemperatureId;
	     c->getCalValues()->fPowerConversionId= 
	       c_tc->getCalValues()->fPowerConversionId;
	     c->getCalValues()->fWaterTemperatureId= 
	       c_tc->getCalValues()->fWaterTemperatureId;
	     c->getCalValues()->fHotSideTemperatureId= 
	       c_tc->getCalValues()->fHotSideTemperatureId;
	     DT_DEBUG_LTC("temp Id: "<<  
			  c_tc->getCalValues()->fTemperatureId<<" PowerID: "
			  <<c_tc->getCalValues()->fPowerConversionId);
	     // now copy the default settings (gain etc) from cold box to tempcrlt
	     c_tc->setCalValues(c->getCalValues()); 
	     if( c_tc->getType() ==  BoxCharacteristicsDef::WIENCTRL	){	       
	       tc= new TempCtrlWien(*p, &fPolStatus);
	     }
	     else{
	       tc= new TempCtrl(*p);
	     }  	
	     fRunTrhread=true; 
	   }
	   break;
	 }
       
       if (tc->isInitialized())
	 {      fTempCtrl.push_back( tc ); 
	 }
       }
     // no checks for correct intialisation yet   
     fTargetTemp=  fTempCtrl[0]->getActualTemperature() ;
     fCtrlStatus= NOTSTARTED;
     DT_DEBUG_LTC( "coldbox init done");
     fInitialized=true;
   }
 
}


ColdBox::~ColdBox()
{
   cleanup();
    }

void ColdBox::cleanup()
{ // is in principle the destructor but can be called in case of exit 
  // should disapear if I know how to handle exceptions 
  if (fAutoCtrlActive) // assume there is a thread if not zero
    { stopAutoCtrl();}
  for (  tcv  p=fTempCtrl.begin();p!=fTempCtrl.end();p++)
    { (*p)->setZeroPower();
    }
 
  for ( tcv p=fTempCtrl.begin();p!=fTempCtrl.end();p++)
    {  DT_DEBUG_LTC("Delete tempctrl "<< (*p)->getName());
       if(*p) delete *p ;
    }
  DT_DEBUG_LTC("Delete tempctrls , go to delete humidity ctrl");
  delete fHCtrl;
  if(fBoxChar) {delete fBoxChar;}
}

void ColdBox::setTargetTemperature (float target)  
 { fTargetTemp =target;
 if( !fInitialized) DT_WARN("Change temperature for not intialized box");
   for ( tcv p=fTempCtrl.begin();p!=fTempCtrl.end();p++)
     { (*p)->setTargetTemperature( target);}
 }

float ColdBox::getActualTemperature () 
{ 
  return  fInitialized?fTempCtrl[0]->getActualTemperature():-273;
}
float ColdBox::getActualTemperature (int i) 
{  if (fTempCtrl[i]){
  return fTempCtrl[i]->getActualTemperature();}
 else{ return -999;} // can never be  
}






float ColdBox::getActualPower() { return fInitialized?fTempCtrl[0]->getActualPower():0;}
float ColdBox::getActualPower(int i) 
{  if (fTempCtrl[i]) 
    {return fTempCtrl[i]->getActualPower();}
   else 
     { return -1;
     }
}


float ColdBox::getHumidity(){
  if(fHCtrl) { return fHCtrl->getHumidity();}
  else {return 0;}
}

bool  ColdBox::maxHumidityReached(){
  if(fHCtrl) { return fHCtrl->maxHumidityReached();}
  else { return false;}
}


int ColdBox::control()
{  int exit_value=0;
 float power =0;
 fCtrlLoopCnt++;
 if(fInitialized)
   { 
     if(fHCtrl) { 
       fHCtrl->control();
       fActualHumidity=fHCtrl->getActualHumidity();
       DT_DEBUG_THR("Actual Humidity= "<<fActualHumidity);
     }
     tcv p=fTempCtrl.begin();
     exit_value=(*p)->control();
     switch( (*p)->getType()){ 
       // type check is only done on the first temp ctrl
       case  BoxCharacteristicsDef::STANDARD:   {  
	 power= (*p)->getActualPower();
	 p++; //first is already done
	 while( p!=fTempCtrl.end())   { 
	   DT_DEBUG_THR("set power to: "<<power <<" for tmpctrl:"<< (*p)->getName());
	   (*p)->setPower(power);p++;
	 }
       }
       break;	 
       case  BoxCharacteristicsDef::WIENCTRL:
       default:	{
	 while( p!=fTempCtrl.end())
	   { DT_DEBUG_THR("Call control for "<< (*p)->getName() << " total: "<< fTempCtrl.size());
	     (*p)->control();p++;
	   }
       }
       break;      
     }
     fActualTemp=getActualTemperature();
   }
 return exit_value;
}

void  ColdBox::printColdBox ()
{ if(fInitialized)
    {      for (  tcv  p=fTempCtrl.begin();p!=fTempCtrl.end();p++)
	    { (*p)->info();
	    }     
    }
  else
    DT_DEBUG_LTC("no cold box initialized");
}

void ColdBox::run() 
{  int i=0;
    if(fInitialized) 
    {  while ( fAutoCtrl)
	{ DT_DEBUG_LTC("Thread target temp" << fTargetTemp 
		       <<" and act temp "<< fActualTemp<<" after try " << i++);
	control(); 
	sleep(2);
	}
    }
   fAutoCtrlActive=false;

}


 int ColdBox::startAutoCtrl()  
{ 
#ifdef COLDBOXTRHEADSUPPORT
 if(fInitialized && fRunTrhread )
    {fAutoCtrl=true ;

     if( !fAutoCtrlActive) //start only once 
       {fAutoCtrlActive=true;
       DT_DEBUG_LTC("start coldbox control loop");
       fCtrlLoopCnt=0;
       start();
       }
    
    return 0;
    }
  else 
    {  DT_DEBUG_LTC("failed to start control loop intialized = " 
		    << fInitialized);
      return -1;
    }
#else
 DT_DEBUG_LTC("failed to start controlloop no qt thread flag set");
    return -1;
#endif
    
  }

int  ColdBox::stopAutoCtrl()
{ 
   
     fAutoCtrl=false;

#ifdef COLDBOXTRHEADSUPPORT
 if(fInitialized)
 {  if( fAutoCtrlActive)
   { wait(); return 0;
   }
   else
     { return -1;  // can not stop not started thread
     }
 }
 else 
   { return -1;
   }
#else
   DT_ERROR( "can not stop not exiting thread ( fThread =0) ");
   return -3;
#endif 
}

