#include "dimsensserver.h"
#include "daqerrormsg.h"
// set env variable DAQPRINTINGLEVEL to 255 to see all DT_DEBUG messages

static bool GlobalExit=false;
static bool GlobalPause=false;
void DimSensServerCtrl(int s)
{  if (s == SIGUSR1)
    {  printf("Got %d %d  \n",getpid(),s);
      return;
    }
  if (s== SIGINT) 
    { GlobalExit = true;
      kill(getpid(),SIGCONT); // exit from pause in any case
    }
  if (s == SIGHUP) GlobalPause = true;
  if (s == SIGCONT) GlobalPause = false;
  printf ("Exit %d  Pause %d \n",GlobalExit,GlobalPause);
}



DimSensServer::DimSensServer(int servernr, tempchvec tempsens , humchvec humiditysen, 
			     tempctrlvec tempctrls, hvchvec hvchs,curchvec cursens ){ 
 setDimServerName ("TRACKERTEST/SENSSERV" );
 if (DimServerNameNotValid()) {
   DT_WARN("problems with dim server name");  return;
 } 
  signal(SIGUSR1,DimSensServerCtrl); 
 fSensSrvCmd = new DimCommand( (char*)DimSensDevCmdName(servernr),"I",this);
 fSensSrvAck = new DimService((char*)DimSensDevAckName(servernr),"I",&fAnswer,sizeof(int));  
 fTempSensors=tempsens;
 fHumiditySensors=humiditysen;
 fTempCtrls=tempctrls;
 fHVUnits=hvchs;
 fCurrentSensors=cursens;
 for (unsigned int lc=0;lc < fTempSensors.size(); lc++)
  { 
    int board=fTempSensors[lc]->getBoardNr();
    int chann=fTempSensors[lc]->getChannelNr();
    fDimTCmd.push_back(new DimCommand((char*)DimTempChannelCmdName(board,chann),"F:2",this));      
    fDimTSrv.push_back(new DimService((char*)DimTempChannelAckName(board,chann),"F:11",&fAnswer,11*sizeof( float)));
  }
 for (unsigned int lc=0;lc < fHumiditySensors.size(); lc++)
   { 
    int board=fHumiditySensors[lc]->getBoardNr();
    int chann=fHumiditySensors[lc]->getChannelNr();
    fDimHCmd.push_back(new DimCommand((char*)DimHumidityChannelCmdName(board,chann),"F:2",this));      
    fDimHSrv.push_back(new DimService((char*)DimHumidityChannelAckName(board,chann),"F:11",&fAnswer,11*sizeof(float)));
   }

 for (unsigned int lc=0;lc < fCurrentSensors.size(); lc++)
   { int board=fCurrentSensors[lc]->getBoardNr(); // fast test : 1 and 0 for chann
     int chann=fCurrentSensors[lc]->getChannelNr();
     fDimCurCmd.push_back(new DimCommand((char*)DimCurrentChannelCmdName(board,chann),"F:2",this));      
     fDimCurSrv.push_back(new DimService((char*)DimCurrentChannelAckName(board,chann),"F:11",&fAnswer,11*sizeof(float)));
    
   }
 for (unsigned int lc=0;lc < fTempCtrls.size(); lc++)
   { 
    int board=fTempCtrls[lc]->getBoardNr();
    int chann=fTempCtrls[lc]->getChannelNr();
    fDimCCmd.push_back(new DimCommand((char*)DimTCtrlChannelCmdName(board,chann),"F:2",this));      
    fDimCSrv.push_back(new DimService((char*)DimTCtrlChannelAckName(board,chann),"F:11",&fAnswer,11*sizeof( float)));
   }

 for (unsigned int lc=0;lc < fHVUnits.size(); lc++)
   {
    int board=fHVUnits[lc]->getBoardId();
    int chann=fHVUnits[lc]->getChannelId();
    fDimVCmd.push_back(new DimCommand((char*)DimHvChannelCmdName(board,chann),"F:4",this));
    fDimVSrv.push_back(new DimService((char*)DimHvChannelAckName(board,chann),"F:11",&fAnswer,11*sizeof( float)));
   }

  start((char*)BaseName());
  DT_DEBUG_LTC("DIM Server: "<<BaseName()<<" started");
}

void DimSensServer::commandHandler()
{ 
  //static short  chc;
 DimCommand* dcm=getCommand();
  DT_DEBUG_LTC("gen got commdand");
 if( dcm == fSensSrvCmd)
   { DT_WARN("Dev  got commdand "<<dcm->getInt());
     printf(" not expected  command \n");
   fAnswer[0]=333;
   fSensSrvAck->updateService();   
   }

  for (unsigned int  lc=0;lc < fDimTCmd.size(); lc++)
    { if ( dcm == fDimTCmd[lc])
      { float *cmd = (float*) dcm->getData();
      DT_DEBUG_LTC("TEMP  get command "<< (int) cmd[0] << " and value"<<cmd[1]);
      fAnswer[0]= 0;
      float temperature = float(fTempSensors[lc]->getTemperature());
      fAnswer[1]= temperature; 
      fDimTSrv[lc]->updateService(); 
      }
    }

  for (unsigned int lc=0;lc < fDimHCmd.size(); lc++)
    { if ( dcm == fDimHCmd[lc])
      { int *cmd = (int*) dcm->getData();
      DT_DEBUG_LTC("HUM get command "<< (int) cmd[0] << " and value"<<cmd[1]);
        fAnswer[0]=0;
	float humidity = float(fHumiditySensors[lc]->getHumidity());
	fAnswer[1]= humidity; 
	fDimHSrv[lc]->updateService(); 
      }
    }

  for (unsigned int lc=0;lc < fDimCurCmd.size(); lc++)
    { if ( dcm == fDimCurCmd[lc])
      { int *cmd = (int*) dcm->getData();
      DT_DEBUG_LTC("Cur get command "<< (int) cmd[0] << " and value"<<cmd[1]);
        fAnswer[0]=0;
	float current=  float( fCurrentSensors[lc]->getCurrent());
	fAnswer[1]= current; 
	fDimCurSrv[lc]->updateService(); 
      }
    }

  for (unsigned int lc=0;lc < fDimVCmd.size(); lc++)
    { if ( dcm == fDimVCmd[lc])
      { float *cmd = (float*) dcm->getData();
      DT_DEBUG_LTC("HV get command "<< (int) cmd[0] << " and value "<< cmd[1]);
      int task=(int)cmd[0];
      float tmpval=0;
      switch (task)
        { case GET_VOLT:
	          fAnswer[0]=(float)GET_VOLT;
		  tmpval = float(fHVUnits[lc]->getHvVolt());
		  fAnswer[ACTVOLT]=tmpval;
		  fDimVSrv[lc]->updateService();
	          break;
          case GET_HVCURR:
	          fAnswer[0]=(float)GET_HVCURR;
		  tmpval = float(fHVUnits[lc]->getHvCurrent());
		  fAnswer[ACTCUR]=tmpval;
		  fDimVSrv[lc]->updateService();
		  break;
          case SET_VOLT:
	          fAnswer[0]=(float)SET_VOLT;
		  fHVUnits[lc]->setHvVolt(cmd[SETHV]);
		  cout << "HEHEHEHEHEHEHEHEH" << endl;
		  cout << cmd[0] << " " << cmd[1] << " " << cmd[2] << " " << cmd[3] << endl;
		  cout << "Valeur de SETHV=" << SETHV << endl;
		  tmpval = float(fHVUnits[lc]->getHvVolt());
		  fAnswer[SETHV]=tmpval;
		  fDimVSrv[lc]->updateService();
		  break;
          default:
                  printf(" unexpected task");
                  fAnswer[0]=-1;
                  fAnswer[1]= -999;
                  fDimCSrv[lc]->updateService();
                  break;
        }
      }
    }
  for (unsigned int lc=0;lc < fDimCCmd.size(); lc++)
    { if ( dcm == fDimCCmd[lc]){ 
      float *cmd = (float*) dcm->getData();
      DT_DEBUG_LTC("TCTRL get command "<< (int) cmd[0] << " and value "<< cmd[1]);
      int task=(int)cmd[0];
      float crtltemp=0;
      switch (task)
	{ case GET_TEMP:   
	          fAnswer[0]=(float)GET_TEMP;
		  crtltemp = float(fTempCtrls[lc]->getActualTemperature());
		  fAnswer[1]= crtltemp; 
		  fDimCSrv[lc]->updateService(); 
	   break;
          case SET_TARGET_TEMP:  
	          fTempCtrls[lc]->setTargetTemperature(cmd[1]);
	          fAnswer[0]=GET_TEMP;
		  crtltemp = float(fTempCtrls[lc]->getActualTemperature());
		  fAnswer[1]= crtltemp; 
		  fDimCSrv[lc]->updateService(); 
	  break;
	  case START_CONTROL:  
	          fTempCtrls[lc]->setTargetTemperature(cmd[1]);
	          fAnswer[0]=GET_TEMP;
		  crtltemp = float(fTempCtrls[lc]->getActualTemperature());
		  fAnswer[1]= crtltemp; 
		  fDimCSrv[lc]->updateService(); 
	  break;
	  case STOP_CONTROL:  
	          fTempCtrls[lc]->setTargetTemperature(cmd[1]);
	          fAnswer[0]=GET_TEMP;
		  crtltemp = float(fTempCtrls[lc]->getActualTemperature());
		  fAnswer[1]= crtltemp; 
		  fDimCSrv[lc]->updateService(); 
	  break;
	  default:  
	          printf(" unexpected task");
	          fAnswer[0]=-1;
		  fAnswer[1]= -999; 
		  fDimCSrv[lc]->updateService(); 
	  break;
	}
     
      }
    }
}




bool DimSensServer::getCtrlActive(int ch) { return true; }

