#include "humiditychannel.h"
#include "daqerrormsg.h"

/** \brief humidity ADC  channel 
     
     
     look at HumidityChannel 
   
     \author W. Beaumont University Antwerpen 
     \version 1.0 
     \date 2001 05 23 

 
*/
HumidityChannel::HumidityChannel( AdcChannelInterface  *HAdcCh,AdcChannelInterface *RAdcCh,
		   CalExchange <HumidityConversionDef::HumidityCalValues>* c  )
  :HumidityChannelInterface(c)
 

    { 
      fHAdc=HAdcCh;  fRefAdc=RAdcCh;
      const int REF_TRIES= 2;
      float ref[REF_TRIES]; float avg=0; bool not_trustable=false;
      int totalcnt=0;int spill=-1;
      fRefAdc->lock();
      spill=  fRefAdc->getSpillNumber();
       fRefAdc->getAdcData(fRefAdcValue);
       fRefAdc->unlock();
      do
	{ avg=0; not_trustable=false;
        for ( int lc=0;lc < REF_TRIES;lc++)
	  { 
	    while (  fRefAdc->getSpillNumber() == spill) ;
	    
	    fRefAdc->lock();
	    spill=  fRefAdc->getSpillNumber();
	    fRefAdc->getAdcData(fRefAdcValue);
	    fRefAdc->unlock();
	    DT_DEBUG_LTC("ref"<<lc<<" " << fRefAdcValue) ;  
	    ref[lc] = fRefAdcValue;
            avg+= ref[lc];
	  }  
	
	avg= avg/REF_TRIES;
	
	for ( int lc=0;lc < REF_TRIES;lc++) //check for 10 % range 
	  not_trustable |= (abs((int)(ref[lc] -avg)) /avg  > 0.1 ); 
	totalcnt++;
	} while (not_trustable  && totalcnt < REF_TRIES);
     
      if (not_trustable){ DAQEXIT("Problems with getting correct reference adc value");}
    // second read because of zero read at first start 
    setVref(fRefAdcValue);
} 

float HumidityChannel::getHumidity()
    {
    float humidity ;  
    fHAdc->lock();
    while (! fHAdc->getAdcData(fActualAdcValue)) {}
    fHAdc->unlock();
    humidity=  Adc2HumidityConversion(fActualAdcValue);
  
    return humidity;
    }



ostream& operator<<(ostream& s, HumidityChannelInterface& HC  )
{ s << "H" <<HC.getId() << ": "<< HC.getHumidity()<<" [\%]"  ;
// printf("fAcutalAdcValue  : %p %f  \n",&HC, HC.getLastHumidityAdcValue());
   
    return s;
}



ofstream& operator<<(ofstream& s, HumidityChannelInterface& HC  )
{  s << "H" <<HC.getId() << ": "<< HC.getHumidity()<<":"  ;
 
 return s; 
}


