#include <signal.h>
/** \brief humidity channel Trhx code 
     
    
     \author W. Beaumont University Antwerpen 
     \version 1.0 
     \date 2001 10 01 
    
 
*/
#include "humiditychanneltrhxsense.h"

static TDeviceControl* fControl =0;

HumidityChannelTrhxSens::HumidityChannelTrhxSens
      ( int TrhxBaseComPort,
	 CalExchange <HumidityConversionDef::HumidityCalValues>* c )
    :HumidityChannelInterface(c) 
                         
{ fInitialized=false;
  // Attach to Main control share memory
  if (fControl== 0)
    fControl =new TDeviceControl("ControlePrincipal",TShare_OPEN);

  fPid = getpid();
  fCd = 0;
  if (AttachTrhxHumidityChannel(getBoardNr(),getChannelNr()) <0)
    {
      printf("TrhxChannel >> Cannot getTRHX  temperature channel %d from Trhx %d \n",
	     getChannelNr(), getBoardNr());
      if (fCd!=0)
        fControl->ReleaseChannel(fCd->fPidServer,fPid,fCd->fChannelNumber);
      return;
    }
  fHumidityTrhx=-1;
  fInitialized=true;
}


float HumidityChannelTrhxSens::getHumidity() {  
  double  humidity ; 
  humidity= (*fSens)[TRHXHumidity][ getChannelNr()];
  fHumidityTrhx=humidity;
  humidity= Adc2HumidityConversion((float)humidity);
  return (float)humidity;
}

HumidityChannelTrhxSens::~HumidityChannelTrhxSens()
{ 
  this->release();
}

void HumidityChannelTrhxSens::release()
{
  if (fInitialized && fCd!=0)
    fControl->ReleaseChannel(fCd->fPidServer,fPid,fCd->fChannelNumber);

}


int HumidityChannelTrhxSens::AttachTrhxHumidityChannel
        (int serial_nr, int channelnr )
{

  fCd = fControl->RequestChannel(fPid,TRHXSENS,channelnr,serial_nr);
  if (fCd <= 0) return -1;
  kill(fCd->fPidServer,SIGCONT);
  printf("je signal %d \n",fCd->fPidServer);
  printf("Shared  Memory %s %d \n",fCd->fShareName,fCd->fShareSize);
  fMemoryClient = new TShare(fCd->fShareName,TShare_OPEN,fCd->fShareSize);
  fTrhx = (TrhxDescription*) fMemoryClient->Attach();
  fSens =  fTrhx->getSensBuffer();
  
  return 0;
}


