// class  to control the temparture 
// reads the temperature from 0ne channel ( or more but is not implemented)
// and controls a DAC output 
// this DAC output is assumed to be coneected to a device that controlles the 
// power stream ( Peltier) or heater
// the temperature controll is a permanent running system so the real control
// ( StartTempCtrl) should be started from a thread 
// to start it is written for  a stand alone program that communicates via an sharred memory
// V0.1 20010112 W. Beaumont  University Antwerpen Belgium


#include <stdio.h>
#include <errno.h>
#include "tempctrl.h"
#include "ltparser.h"
#include "temperaturechannellt.h"
#include "tempctrlreg.h"
#include <dacchannelinterface.h>
#include <dacchanneldummy.h> 
#include "powerchannellt.h"
#include "daqerrormsg.h"

extern LtParser*  ltparser;

TempCtrl::TempCtrl( int device )
  // calls the temperatureinterface without parameters 
{ 
  fRegTc=new TempCtrlReg();
  if( fRegTc->registerTempCtrl(device,this)) 
   {
     CalExchange <BoxCharacteristicsDef::BoxCharValues>* c =0;
     int n =ltparser->getNumberTempCtrl();
     DT_DEBUG_LTC("TempCtrl's found  " << n);
     c = ltparser-> getTempCtrlInfo(device);
               
     if (c) 
       { 
	 DT_DEBUG_LTC( c); 
         fBoxChar = new BoxCharacteristics( c);
         fBoxChar-> setCalValues(fBoxChar->getCalValues() );
	 fPowerChannel = new PowerChannelLt
	                (fBoxChar->getCalValues()->fPowerConversionId);
	 fTemperature = new TemperatureChannelLt 
	                  (fBoxChar->getCalValues()->fTemperatureId);
      }
     else
       {DAQEXIT("Temperature Ctrl  deviceId " << device << " not found" ) ;
       }
     //copy the calibrationID of the tmpereature control 
     // to the calibrationID of the inherited temperaturechannelinterface
     TemperatureChannelInterface::setCalibrationId(c->getCalibrationId());
     // default settings:
     fHeater=fPowerChannel->heater(); 
     fTargetTemp=  fTemperature->getTemperature() ;
     
     fCtrlStatus= NOTSTARTED ;
     setTempHistory( 0);

     fIntegratedError=0;
     fPowerChannel->setPower(0);     
     fActualPower=0;
     DT_DEBUG_LTC( "tempctrl init done");
     fRefferenceToChannel=false;
   }
 else
   {   
     fRefferenceToChannel=true;
     // in principle this is an error 
   }
}


TempCtrl::~TempCtrl()
{   cleanup();
}

void TempCtrl::cleanup()
{  // is in principle the destructor but can be called in case of exit 
  // should disapear if I know how to handle exceptions 
      (void)fPowerChannel->setPower(0);
     
     delete fTemperature;
     // next two where created in the getPowerConversion function
    
     delete fPowerChannel;
     if(! fRefferenceToChannel){
       fRegTc->unRegisterTempCtrl();
     }
     delete fBoxChar;
}

void TempCtrl::setTargetTemperature (float target)  
 { fTargetTemp =target;
   setTempHistory( 0);
 }

float TempCtrl:: IntegratedTempError()
{ float sum=0; 
  
  for ( int lc= 0 ;lc < MAX_HISTORY_ENTRIES-1 ; lc++)
    {    sum +=  (MAX_HISTORY_ENTRIES- lc)*fTh[lc].temp ;
    }
   return sum / MAX_HISTORY_ENTRIES ;
}

float TempCtrl:: setTempHistory(float x)
{  for ( int lc= 0 ;lc < MAX_HISTORY_ENTRIES ; lc++)
    {             fTh[lc].temp=x ;
    }
 return 0; 
}

float TempCtrl::DifferentiatedError()
{ float diff=0;
 diff = fTh[0].temp - fTh[1].temp;

 return diff ; 
} 





int TempCtrl::control()
{   float dp; //DesiredPower
    int exit_value=-5;
    BoxCharacteristicsDef::BoxCharValues *fBCV =   
	     fBoxChar->getCalValues(); // local pointer to box charateristics
    fActualTemp=  fTemperature->getTemperature() ;
    fDeltaTemp = fActualTemp - fTargetTemp;
    fCtrlStatus= UNKNOWN;
    // shift history buffer  can be much more efficient but ...
    for ( int lc =  MAX_HISTORY_ENTRIES-1 ; lc > 0 ; lc --)
      { fTh[lc].temp = fTh[lc-1].temp; 
      }
    fTh[0].temp=fDeltaTemp;
    fIntegratedError =  IntegratedTempError(); // + 0.5 *fIntegratedError ;
    float max_delta = fBCV->fMaxDeltaTemperature; 	
    dp= fBCV->fLoopGain * fIntegratedError ;
    fCtrlStatus= (dp>0)? COOLING: HEATING;
    if(! fHeater)
      { if (fDeltaTemp < -max_delta) 
	{dp=0;	
	 fCtrlStatus= UNDERTEMPERATURE;
	}
      }
  
    fActualPower=dp;
    DT_DEBUG_LTC(TemperatureChannelInterface::getName()
		 <<"set power to "<< dp << "targetT"
		 << fTargetTemp<<" actT"<<fActualTemp
		 <<"heater="<<fHeater );
    (void)fPowerChannel->setPower(dp);
    
    exit_value=0;
    return exit_value;
}
float TempCtrl::getActualPower() {  return fActualPower;}

 void TempCtrl::setPower(float power)
 {(void)fPowerChannel->setPower(power);
 fActualPower=power;
 } 

 void TempCtrl::setZeroPower()
 {(void)fPowerChannel->setPower(0);
 } 


void  TempCtrl::printTempCtrl ()
{    cout <<" TempCtrl "<<  fBoxChar->getName()<<" Id:"<< 
       TemperatureChannelInterface::getId()<<endl;
          cout << "T " << getActualTemperature()<<" [C] Target:  ";
	    cout << getTargetTemperature() ;
	    cout  << " [C]  Power "  << getActualPower() << " [W] " ;
	    cout <<" Integrated Error: "<< getIntegratedError() << " [C/s] \n status " ; 
	    switch ( fCtrlStatus)
	      {
		case  UNKNOWN :  cout << "unknown" ;
		  break;
		case NOTSTARTED: cout << "not started" ;
		  break;
		  case NORMAL :  cout << "normal" ;
		    break;
		    case UNDERTEMPERATURE : cout << "temerature to low";
		      break;
		case  OVERTEMPERATURE : cout << "high temperature" ;
		  break;
		default:  cout << "invalid status" ;
				   }
	    cout << "\n";
}

 


