
#include <stdio.h>
#include <errno.h>
#include "tempctrldim.h"
#include "tempctrlreg.h"
#include "ltparser.h"
#include "daqerrormsg.h"
#include "humidityctrlinterface.h"



extern LtParser*  ltparser;

TempCtrlDim::TempCtrlDim( int device, HumidityCtrlInterface* hc ): TempCtrlInterface(){ 
  fIsInitialized=false; 
  fHumidityCtrl=hc;
  fRegTc=new TempCtrlReg();
  if( fRegTc->registerTempCtrl(device,this)) {
    fIsInitialized=false;
     CalExchange <BoxCharacteristicsDef::BoxCharValues>* c =0;
     int n =ltparser->getNumberTempCtrl();
     DT_DEBUG_LTC("TempCtrl's found  " << n);
     c = ltparser-> getTempCtrlInfo(device);
               
     if (c) 

       { DT_DEBUG_LTC( c); 
         fBoxChar = new BoxCharacteristics( c);
         fBoxChar-> setCalValues(fBoxChar->getCalValues() );
	 // make the Dim Connections
	 setDimServerName ( c->getShmName());
	 if (DimServerNameNotValid()) { DT_WARN("wrong dim server name");} 
	  fTempCtrlinfo= new DimInfo((char*)DimTCtrlChannelAckName(c->getBoardNr(),c->getChannelNr()),-1,this);
	  DT_DEBUG_LTC("DimInfo:"<<DimTCtrlChannelAckName(c->getBoardNr(),c->getChannelNr()));
	  memset(fCmd,0,80);
	  sprintf(fCmd,"%s",DimTCtrlChannelCmdName(c->getBoardNr(),c->getChannelNr()));
	  fNsent=0;
	  fNreceived=0;
      }
     else
       {DAQEXIT("Temperature Ctrl  deviceId " << device << " not found" ) ;
       }
    // default settings:
     TemperatureChannelInterface::setCalibrationId(c->getCalibrationId());
     fTargetTemp= -999;
     
     fCtrlStatus= UNKNOWN ;
     fActualPower=0;
     DT_DEBUG_LTC( "tempctrl init done");
     //delete ltparser;
     fIsInitialized=true;
     fRefferenceToChannel=false;
   }
 else
   {  fRefferenceToChannel=true;
       fIsInitialized=true;
     // in principle this is an error 
   }
}


TempCtrlDim::~TempCtrlDim()
{   // has to see what has to be done for dim channels
  // remove registration
  if(! fRefferenceToChannel){
    fRegTc->unRegisterTempCtrl();
  }
  delete fTempCtrlinfo;
  delete fBoxChar;
}


void TempCtrlDim::setTargetTemperature (float target)  
 { fTargetTemp =target;
    fSend[0] = (float) SET_TARGET_TEMP;
    fSend[1]= target;
    sendCmd();
 }

int TempCtrlDim::control()
  {   fSend[0] = (float) START_CONTROL;
  fSend[1]= fTargetTemp; // required 
    return sendCmd();
    
  }
 
float TempCtrlDim::getActualPower() {  return fActualPower;}

 void TempCtrlDim::setPower(float power)
 { fActualPower=power;
 // no action yet 
 } 

 void TempCtrlDim::setZeroPower()
   { fActualPower=0;
       fSend[0] = (float) STOP_CONTROL;
  fSend[1]= -999; // required but not target temperature
    sendCmd();
 } 


void  TempCtrlDim::printTempCtrl ()
{    cout <<" TempCtrl "<<  fBoxChar->getName()<<" Id:"<< 
       TemperatureChannelInterface::getId()<<endl;
          cout << "T " << getActualTemperature()<<" [C] Target:  ";
	    cout << getTargetTemperature() ;
	    cout  << " [C]  Power "  << getActualPower() << " [W] " ;
	    cout << " [C/s] \n status " ; 
	    switch ( fCtrlStatus)
	      {
		case  UNKNOWN :  cout << "unknown" ;
		  break;
		case NOTSTARTED: cout << "not started" ;
		  break;
		  case NORMAL :  cout << "normal" ;
		    break;
		    case UNDERTEMPERATURE : cout << "temerature to low";
		      break;
		case  OVERTEMPERATURE : cout << "high temperature" ;
		  break;
		default:  cout << "invalid status" ;
				   }
	    cout << "\n";
}

void TempCtrlDim::infoHandler()
{
  DimInfo* info=getInfo();
  // memcpy(fRecBuf,0, 256*sizeof(short));
  if (info == fTempCtrlinfo)
    {   fNreceived++;
       if(info->getSize() >  (int)(256* sizeof(float))) 
	 { fRecBuf[0]=-2; fRecBuf[1]=-990.99; }
       else
	 { memcpy(fRecBuf,info->getData(),info->getSize());    
	   DT_DEBUG_LTC("Temp info size"<<info->getSize()<<" cmd: "<< (int)fRecBuf[0]<<" Value: "<< fRecBuf[1] );
	 }
       switch ( (int)fRecBuf[0])
	 { case  GET_TEMP: { fActualTemp= fRecBuf[1];}
	   break;
	   default: // simple ignore 
	     break;
	 }
    }
  else
    {fRecBuf[0]=-1; fRecBuf[1]=-990.99; }
}



 int  TempCtrlDim::sendCmd()
{if(!   DimClient::sendCommand(fCmd,fSend,2*sizeof(int)))
  { DT_WARN("failed to send \n"); return -999; }
    fNsent++;
    if(  wait() < 0){ DT_WARN("no reaction from server\n");return -998;}
    DT_DEBUG_LTC("received" << fRecBuf[0] << "send "<< fNsent <<" fNreceived "<<  fNreceived);
    return 0;
}


int TempCtrlDim::wait()
{
  int nc=0;
  do
    {
      usleep(1000);
      nc++;
      if (nc>1000) return -1;
    } while (fNreceived<fNsent);
  return nc;
}



