// class  to control the temparture 
// reads the temperature from 0ne channel ( or more but is not implemented)
// and controls a DAC output 
// this DAC output is assumed to be coneected to a device that controlles the 
// power stream ( Peltier) or heater
// the temperature controll is a permanent running system so the real control
// ( StartTempCtrl) should be started from a thread 
// to start it is written for  a stand alone program that communicates via an sharred memory
// V0.1 20010112 W. Beaumont  University Antwerpen Belgium


#include <stdio.h>
#include <errno.h>
#include "tempctrlq.h"
#include "ltparser.h"
#include "temperaturechannellt.h" 
#include <dacchannelinterface.h>
#include <dacchanneldummy.h> 
#include "powerchannellt.h"
#include "daqerrormsg.h"

extern LtParser*  ltparser;

TempCtrlQ::TempCtrlQ( int device ): QThread()
                   
                  
{ 
 
     bool found=false;
     fDeviceId = device; 
     fAutoCtrl=false;
     fAutoCtrlActive=false;
    CalExchange <BoxCharacteristicsDef::BoxCharValues>* c =0;
    //LtParser* ltparser = new LtParser();
    //ltparser->parseFile(  LTFILENAME);
                            
    int n =ltparser->getNumberColdBoxes();
    DT_DEBUG_LTC("Boxes " << n);
    for (int lc =0 ; lc < n ; lc ++)
      {   c = ltparser-> getColdBox(lc);
           if ( c->getId() == fDeviceId) { found=true; break;}
      }
    if ( found) 
      {  fBoxChar = new BoxCharacteristics( c);
         fBoxChar-> setCalValues(fBoxChar->getCalValues() );
	 fPowerChannel = new PowerChannelLt( fBoxChar->getCalValues()->fPowerConversionId);
	 fTemperature = new TemperatureChannelLt (fBoxChar->getCalValues()->fTemperatureId);
      }
    else
      {      DAQEXIT("Temperature Ctrl  deviceId " << fDeviceId << " not found" ) ;
      }
    // default settings:
   
    fTargetTemp=  fTemperature->getTemperature() ;
    
    fCtrlStatus= NOTSTARTED ;
    setTempHistory( 0);

   fIntegratedError=0;
   fPowerChannel->setPower(0);
   fActualDacValue=1; // not zero to have a start value !!
   fActualPower=0;
   DT_DEBUG_LTC( "tempctrl init done");
   //delete ltparser;
}


TempCtrlQ::~TempCtrlQ()
{
   cleanup();
    }

void TempCtrlQ::cleanup()
{  // is in principle the destructor but can be called in case of exit 
  // should disapear if I know how to handle exceptions 
//  if (fThread) // assume there is a thread if not zero
    { stopAutoCtrl();}

     (void)fPowerChannel->setPower(0);
     
     delete fTemperature;
     // next two where created in the getPowerConversion function
    
     delete fPowerChannel;

     delete fBoxChar;
   

}

void TempCtrlQ::setTargetTemperature (float target)  
 { fTargetTemp =target;
   setTempHistory( 0);
 }

float TempCtrlQ:: IntegratedTempError()
{ float sum=0; 
  
  for ( int lc= 0 ;lc < MAX_HISTORY_ENTRIES-1 ; lc++)
    {    sum +=  (MAX_HISTORY_ENTRIES- lc)*fTh[lc].temp ;
    }
   return sum / MAX_HISTORY_ENTRIES ;
}

float TempCtrlQ:: setTempHistory(float x)
{  for ( int lc= 0 ;lc < MAX_HISTORY_ENTRIES ; lc++)
    {             fTh[lc].temp=x ;
    }
 return 0; 
}

float TempCtrlQ::DifferentiatedError()
{ float diff=0;
 diff = fTh[0].temp - fTh[1].temp;

 return diff ; 
} 

int TempCtrlQ::control()
{   float dp; //DesiredPower
    int exit_value=-5;
    BoxCharacteristicsDef::BoxCharValues *fBCV =   
	     fBoxChar->getCalValues(); // local pointer to box charateristics
    
    
        fActualTemp=  fTemperature->getTemperature() ;
        fDeltaTemp = fActualTemp - fTargetTemp;
	fCtrlStatus= UNKNOWN;
	
	{ // shift history buffer  can be much more efficient but ...
	   for ( int lc =  MAX_HISTORY_ENTRIES-1 ; lc > 0 ; lc --)
	     {  
	       fTh[lc].temp = fTh[lc-1].temp; 
	      
	     }
	
	   fTh[0].temp=fDeltaTemp;
	
	   fIntegratedError =  IntegratedTempError(); // + 0.5 *fIntegratedError ;
	

	   

	   float max_delta = fBCV->fMaxDeltaTemperature; 	
	   dp= fBCV->fLoopGain * fIntegratedError ;
	  
	   if (fDeltaTemp <  -max_delta  )
	     { // switch off power avoid further cooling 
	       dp =0;
	       fCtrlStatus= UNDERTEMPERATURE;
	     }
	   else
	     { fCtrlStatus= NORMAL;
	     }
	}

	fActualPower=dp;
	DT_DEBUG_LTC("set power to "<< dp << "target"<< fTargetTemp<<" act "<<fActualTemp );
	(void)fPowerChannel->setPower(dp);
	
	exit_value=0;
      
    
    return exit_value;
}



void TempCtrlQ::run() 
{  int i=0;
 if( !fAutoCtrlActive) //start only once 
  {fAutoCtrlActive=true;
   while ( fAutoCtrl)
     { printf(" the thread with target temp %f  and act temp %f  after try  %i \n", fTargetTemp,fActualTemp , i++);
     control(); 
  sleep(2);
  }
   fAutoCtrlActive=false;
  }
 
   
}






 
int  TempCtrlQ::stopAutoCtrl()
{ fAutoCtrl=false;
 
 wait();

 return 0; 
}


void  TempCtrlQ::printTempCtrl ()
    {  cout << "T " << getActualTemperature()<<" [C] Target:  ";
	    cout << getTargetTemperature() ;
	    cout  << " [C]  Power "  << getActualPower() << " [W] " ;
	    cout <<" Integrated Error: "<< getIntegratedError() << " [C/s] \n status " ; 
	    switch ( fCtrlStatus)
	      {
		case  UNKNOWN :  cout << "unknown" ;
		  break;
		case NOTSTARTED: cout << "not started" ;
		  break;
		  case NORMAL :  cout << "normal" ;
		    break;
		    case UNDERTEMPERATURE : cout << "temerature to low";
		      break;
		case  OVERTEMPERATURE : cout << "high temperature" ;
		  break;
		default:  cout << "invalid status" ;
				   }
	    cout << "\n";
    }




