// class  to control the temparture 
// reads the temperature from 0ne channel ( or more but is not implemented)
// and controls a DAC output 
// this DAC output is assumed to be coneected to a device that controlles the 
// power stream ( Peltier) or heater
// the temperature controll is a permanent running system so the real control
// ( StartTempCtrl) should be started from a thread 
// to start it is written for  a stand alone program that communicates via an sharred memory
// V0.1 20010112 W. Beaumont  University Antwerpen Belgium


#include <cstdio>
#include <cerrno>
#include "tempctrlwien.h"
#include "tempctrlreg.h"
#include "ltparser.h"
#include "temperaturechannellt.h" 
#include <dacchannelinterface.h>
#include <dacchanneldummy.h> 
#include "powerchannellt.h"
#include "daqerrormsg.h"
#include "humidityctrlinterface.h"

extern LtParser*  ltparser;

TempCtrlWien::TempCtrlWien( int device , int* PolStatus, HumidityCtrlInterface* hc):TempCtrlInterface(hc) { 
  fRegTc=new TempCtrlReg;
  fHumidityCtrl=hc;
  if(hc){
    CalExchange <HumidityCtrlDef::HumidityCtrlValues>* hcv=
      fHumidityCtrl->getHumidityCtrlCalValues();
    HumidityCtrlDef::HumidityCtrlValues hvalues(*hcv->getCalValues());
    float  maxhumidity= hvalues.fMaxHumidity;
  }
  if( fRegTc->registerTempCtrl(device,this)) {
    CalExchange <BoxCharacteristicsDef::BoxCharValues>* c =0;
    int n =ltparser->getNumberTempCtrl();
    DT_DEBUG_LTC("TempCtrl's found  " << n);
    c = ltparser-> getTempCtrlInfo(device);
    if (c) 

       {
         fBoxChar = new BoxCharacteristics( c);
         fBoxChar-> setCalValues(fBoxChar->getCalValues() );
	 fPowerChannel = new PowerChannelLt
	                (fBoxChar->getCalValues()->fPowerConversionId , PolStatus);
	 DT_DEBUG_LTC("Temperature probe id's "<< 
		      fBoxChar->getCalValues()->fTemperatureId<<" "<<
		      fBoxChar->getCalValues()->fWaterTemperatureId<<" "<<
		      fBoxChar->getCalValues()->fHotSideTemperatureId);
	 fTemperature = new TemperatureChannelLt 
	                  (fBoxChar->getCalValues()->fTemperatureId);
	 if ( fBoxChar->getCalValues()->fWaterTemperatureId >0) {
	   fWaterTemp=new TemperatureChannelLt
	     (fBoxChar->getCalValues()->fWaterTemperatureId);
	   // checks if the name of the water probe starts with CoolingWater 
	   if( strncmp(fWaterTemp->getName(),"Water",13) ){
	     DT_WARN("Water temperature probe with name "<< 
		     fWaterTemp->getName());
	   }
	 }
	 if ( fBoxChar->getCalValues()->fHotSideTemperatureId >0) {
	   fHotSideTemp=new TemperatureChannelLt
	     (fBoxChar->getCalValues()->fHotSideTemperatureId);
	   // check if the name of the hotside probe stats wit HotSide
	   if( strncmp(fWaterTemp->getName(),"HeatSink",7) ){
	     DT_WARN("Water temperature probe with name "<< 
		     fWaterTemp->getName());
	   }
	 }
      }
     else
       {DAQEXIT("Temperature Ctrl  deviceId " << device << " not found" ) ;
       }
    
     TemperatureChannelInterface::setCalibrationId(c->getCalibrationId());
     // default settings:
     fHeater=fPowerChannel->heater(); 
     fTargetTemp=  fTemperature->getTemperature() ;
     
     fCtrlStatus= NOTSTARTED ;
     setTempHistory( 0);

     fIntegratedError=0;
     fPowerChannel->setPower(0);     
     fActualPower=0;
     DT_DEBUG_LTC( "tempctrl init done");
     //delete ltparser;
     fRefferenceToChannel=false;
   }
 else
   {   
     fRefferenceToChannel=true;
     // in principle this is an error 
   }
}


TempCtrlWien::~TempCtrlWien()
{   cleanup();
}

void TempCtrlWien::cleanup()
{  // is in principle the destructor but can be called in case of exit 
  // should disapear if I know how to handle exceptions 
 if(! fRefferenceToChannel){
   DT_DEBUG_LTC("clean up ");
      (void)fPowerChannel->setPower(0);
     delete  fHotSideTemp;
     delete fWaterTemp;
     delete fTemperature;
     // next two where created in the getPowerConversion function
    
     delete fPowerChannel;
    
       fRegTc->unRegisterTempCtrl();

     delete fBoxChar;
     
     }
 else {
   DT_WARN("should come here normally");
   fRegTc->decreaseReference();  //?? make sense ?
 }
}

void TempCtrlWien::setTargetTemperature (float target)  
 { fTargetTemp =target;
   setTempHistory( 0);
 }

float TempCtrlWien:: IntegratedTempError()
{ float sum=0; 
  
  for ( int lc= 0 ;lc < MAX_HISTORY_ENTRIES-1 ; lc++)
    {    sum +=  (MAX_HISTORY_ENTRIES- lc)*fTh[lc].temp ;
    }
   return sum / MAX_HISTORY_ENTRIES ;
}

float TempCtrlWien:: setTempHistory(float x)
{  for ( int lc= 0 ;lc < MAX_HISTORY_ENTRIES ; lc++)
    {             fTh[lc].temp=x ;
    }
 return 0; 
}

float TempCtrlWien::DifferentiatedError()
{ float diff=0;
 diff = fTh[0].temp - fTh[1].temp;

 return diff ; 
} 




int TempCtrlWien::control()
{
/// Temperature Control  routine from Torino group for the Wien box: 

    float dp; //DesiredPower
    int exit_value=-5;
    bool debug = false;

//    ofstream myout("coolingdata.dat",ios::app);

    BoxCharacteristicsDef::BoxCharValues *fBCV =   
	     fBoxChar->getCalValues(); // local pointer to box charateristics
    DT_DEBUG_LTC(getId()<<" update temperatures ");
    fActualTemp=  fTemperature->getTemperature() ;
    if(fWaterTemp) 
      {  fWaterTemperature=fWaterTemp->getTemperature(); }
    else { fWaterTemperature=-TCZERO;}
    if(fHotSideTemp) 
      {    fHotSideTemperature=fHotSideTemp->getTemperature();  }
    else { fHotSideTemperature=-TCZERO;}
    fDeltaTemp = fActualTemp - fTargetTemp;
    fCtrlStatus= UNKNOWN;
    // shift history buffer  can be much more efficient but ...
    for ( int lc =  MAX_HISTORY_ENTRIES-1 ; lc > 0 ; lc --)
      { fTh[lc].temp = fTh[lc-1].temp; 
      }
    fTh[0].temp=fDeltaTemp;
    fIntegratedError =  IntegratedTempError(); // + 0.5 *fIntegratedError ;
    float max_delta = fBCV->fMaxDeltaTemperature; 	

// Some more informations are needed...
    float tamb =25.;       // ambient temperature 
    float tpelt= fBCV->fWaterTemperature;        // temperature of the water chilling the peltier
    float warm_pmax = -8;     
    int   chilling_status = 0;  // 0=around temperature, 1=rampdown, 2=rampup
    float pequilibrium = 0;

    if (fDeltaTemp>5)         {
// CASE 1):  system is in fast cooling, it does not need smart calculations, system to max power    
       chilling_status=1; dp=25;	       fCtrlStatus=FASTCOOLING;
    }
    else 
      if (fDeltaTemp < -5) {
// CASE 2): system is in fast warmup, it does not need smart calculations, system to max power
       chilling_status=-1; dp=-25;	 fCtrlStatus=FASTHEATING;    
      }    
      else
	{ fCtrlStatus=NORMAL; 
// CASE 3): feedback power to converge to target, here better calculation 
	  chilling_status=0;

// A) dp is first estimated from an equilibrium state

	  dp= fBCV->fLoopGain * fIntegratedError;

// ... with some small addition to take into mind uncertainty about the equilibrium state

	  if (fDeltaTemp>0) dp=dp+fBCV->fFullPowerMargin;
	  else              dp=dp-fBCV->fFullPowerMargin;

// B) dp has to then take into consideration an estimation of the equilibrium condition where
//   power is not zero 

// B.1) modules have thermal power to compensate for
	  float pmodules; pmodules = 0;    

// B.2) the cold box looses thermal power
	  float k;   k = 2;
	  float pbox = k/2*(tamb-fTargetTemp);
//
	  pequilibrium = pmodules+pbox;
	  dp=dp+pequilibrium;

// C) An efficiency of the Peltier has to take into mind, not all electrical power 
//    is converted to chilling power...

	  float dtpeltier_max = 62;
	  float effi; effi = (1-fabs(tpelt-fActualTemp)/dtpeltier_max)*300./350;
	  
	  dp = dp/effi;

// if box exceed target too much...
	  if ( (dp>0 &&(fDeltaTemp)<-max_delta)||(dp<0  && (fDeltaTemp)>max_delta)) 
	    {  dp=pequilibrium/3.;}
    
	  if (debug){ cout << " effi= " << effi << " dpe = " << dp << " dp0e= " << pequilibrium << endl;}

// D) Power has to be translated into Voltage for the Wien Box

       if (dp>=0)   { dp=1.375*sqrt(dp+1.634)-0.809; }                      
       else        {  dp=-(1.375*sqrt(-dp+1.634)-0.809); }

    } 
	    

//    myout <<  " TTarg=" << fTargetTemp << " T=" <<
//    fActualTemp << " DTintegr=" <<fIntegratedError << " dp(V)="
//    << dp << " dp0(W)=" << pequilibrium ;
    
// SAFETY IS IMPORTANT : box temperature has to stay inside limits
    if (fActualTemp>fBCV->fMaxTemperature){
      dp=-0.5;
      fCtrlStatus=OVERTEMPERATURE;
    }
    if (fActualTemp<fBCV->fMinTemperature){
      dp= 0.5;
      fCtrlStatus=UNDERTEMPERATURE;
    }

// Peltier PS protection: if temperature below tpelt then chilling ~only is allowed
//                        if temperature above tpelt then warming ~only is allowed
   
   
// warming...
   if (dp<-7) {   dp= warm_pmax; 
                  if (fActualTemp< tpelt)    { dp=0.5; if ((chilling_status=-1)) dp=-3;} 
		  if (fActualTemp<(tpelt-3)) { dp=0.5; if ((chilling_status=-1)) dp=-5;}		     
                  if (fActualTemp<(tpelt-8)) { dp=0.5; }
   }
   else if ( (dp>=-7)&&(dp<=0) ) {
                  if (fActualTemp< tpelt) { 
		      dp=0.5; }
   }   

// ...chilling

   else if ( (dp<=10)&&(dp>0) ){
                  if (fActualTemp>(tpelt+2)) dp= 2;
                  if (fActualTemp>(tpelt+6)) dp=-0.5;
   }
   else if (dp>10) {
                  if (fActualTemp>(tpelt+ 4)){dp= 2  ; if ((chilling_status=1)) dp=12 ;}
                  if (fActualTemp>(tpelt+ 7)){dp=-0.5; if ((chilling_status=1)) dp=7 ;}
                  if (fActualTemp>(tpelt+11)){dp=-0.5; if ((chilling_status=1)) dp=3 ;}
                  if (fActualTemp>(tpelt+18)) dp=-0.5;
   }   
      

//------------------------------------
//    { myout << " power==" << dp << " tpelt= " << tpelt << endl;}

    fActualPower=dp;
    (void)fPowerChannel->setPower(dp);
    DT_DEBUG_LTC(TemperatureChannelInterface::getName()
		 <<"set power to "<< dp << " targetT "
		 << fTargetTemp<<" actT "<<fActualTemp
		 <<" heater= "<<fHeater );

//     myout.close();
    
    exit_value=0;  // why not set to dp ?
    return exit_value;
}



float TempCtrlWien::getActualPower() {  return fActualPower;}

 void TempCtrlWien::setPower(float power)
 {(void)fPowerChannel->setPower(power);
 fActualPower=power;
 } 

 void TempCtrlWien::setZeroPower()
 {(void)fPowerChannel->setPower(0);
 } 


void  TempCtrlWien::printTempCtrl ()
{    cout <<" TempCtrl "<<  fBoxChar->getName()<<" Id:"<< 
       TemperatureChannelInterface::getId()<<endl;
          cout << "T " << getActualTemperature()<<" [C] Target:  ";
	    cout << getTargetTemperature() ;
	    cout  << " [C]  Power "  << getActualPower() << " [W] " ;
	    cout <<" Integrated Error: "<< getIntegratedError() << " [C/s] \n status " ; 
	    switch ( fCtrlStatus)
	      {
		case  UNKNOWN :  cout << "unknown" ;
		  break;
		case NOTSTARTED: cout << "not started" ;
		  break;
		  case NORMAL :  cout << "normal" ;
		    break;
		    case UNDERTEMPERATURE : cout << "temerature to low";
		      break;
		case  OVERTEMPERATURE : cout << "high temperature" ;
		  break;
		default:  cout << "invalid status" ;
				   }
	    cout << "\n";
}

 


