#include "temperaturechannelinterface.h"
#include "temperaturechannel.h"
#include "temperaturechanneltrhxsens.h"
#include "temperaturechannellt.h"
#include "temperaturechanneldim.h"
#include "temperaturechanneltempctrl.h"
#include "temperaturechanneldummy.h"
#include "adcchannel.h"
#include "adcchannelad4717.h"
#include "adcchanneldummy.h"
#include "ltparser.h"
#include "benchposconnection.h"
#include "daqerrormsg.h"
extern LtParser* ltparser; 
// for the moment as long I don't understand why I can not open the parser mechanism twice
TemperatureChannelLt::TemperatureChannelLt(   int slotpos, int ordernr  ) : TemperatureChannelInterface()
{ fInitialized=false;
   BenchPosConnection* BenchP;
   
   //   int nrPB= ltparser->getNumberBenchPositions() ;
//        if ( slotpos  > nrPB  )
//  	{  DAQEXIT("slot position "<< slotpos << " out of reach /n");
//  	}
    
  
     BenchP= new BenchPosConnection(*ltparser->getBenchPosConnections(slotpos));
     if (BenchP->getId() == -1) {     DAQEXIT("Benchpos" << slotpos <<" doesn't exist \n"<< endl) ;
                    } 
     InitLtTemperatureChannel(BenchP->getNTCDevice(ordernr));

    delete BenchP;
 
}



TemperatureChannelLt::TemperatureChannelLt(  int device) : TemperatureChannelInterface()
{  fInitialized=false;
 InitLtTemperatureChannel(device );
   DT_NOTE("TemperatureChannelLt"<<device<<"is constructed");
  }

void TemperatureChannelLt::InitLtTemperatureChannel( int device)
{  CalExchange <TempConverDef::TempCalValues>* c=0 ;
   fDeviceId = device; 
   if (fDeviceId==TempConverDef::DUMMYDEVICEID){
     fT= new TemperatureChannelDummy(  );
   }
   else {
   //LtParser* parser = new LtParser();
   //parser->parseFile( "lt1.xml");
                            
   int nTempProbes =ltparser->getNumberTemperatureProbes();
   DT_DEBUG_LTC("Probes " << nTempProbes ) ;
    c = ltparser-> getTemperatureCalibration(device);
    
    if (c)  // c will be 0 if device doesn't exist 
      {	
	
	DT_DEBUG_LTC(c->printIdValues(cout));
	fT=0;
	setCalibrationId(c->getCalibrationId());
	fType= c->getType();
	switch ( fType )  { 
	    case TempConverDef::TEMPCTRL:    {
	      fT=new TemperatureChannelTempCtrl(c);
	      if( !fT->isInitialized())	{   
		DAQEXIT("error in getting tempctrl  channel for device "
			<< fDeviceId );
	      }
	    }
	    break;
	    case TempConverDef::TEMPSPEC:
	              {  fT=new TemperatureChannelDim(c);
		      if( !fT->isInitialized())
			{   DAQEXIT("error in getting dim  channel for device "<< fDeviceId << endl) ;}
		      }
		  break;
	    case TempConverDef::AD590 :
	    case TempConverDef::NTC   :  
	      {  AdcChannelInterface* fADC= new AdcChannel(c->getBoardNr(),c->getChannelNr());
	         if ( ! fADC->isInitialized()) 
		 {     DAQEXIT("error in getting ADC channel for device "<< fDeviceId << endl);
		 }  					
		 fT= new TemperatureChannel( fADC , c );
		 //fT->info();
	      }
	    break;
	    case TempConverDef::NTCAD4717   :  
	      {  AdcChannelInterface* fADC= fADC= new AdcChannelAd4717(c->getBoardNr(),c->getChannelNr());
	         if ( ! fADC->isInitialized()) 
		 {     DAQEXIT("error in getting ADC channel for device "<< fDeviceId << endl);
		 }  					
		 fT= new TemperatureChannel( fADC , c );
		 //fT->info();
	      }
	    break;
	    case TempConverDef::TRHXSENS : 
	     case TempConverDef::TRHXNTC :    
	      {   fT = new TemperatureChannelTrhxSens(c->getBoardNr(), c );
	          if( !fT->isInitialized())
		    {         DAQEXIT("error in getting TRHX  channel for device "<< fDeviceId << endl) ;
		    }
	      }
	    break;
	    

	    case TempConverDef::AD590DUMMY :
	    case TempConverDef::DUMMY :	       {   
	      TempConverDef::TempCalValues* tc= c->getCalValues();
	      AdcChannelInterface* fADC= 
		new AdcChannelDummy(tc->CalAdcValue+20,10,c->getBoardNr(),c->getChannelNr());
	            fT= new TemperatureChannel( fADC , c );
		    //  fT->info();
	       }
	    break;
	     
	    default :         DAQEXIT("Temperature probe type" << fType << "not supported"<< endl) ;
					
	    break;
	  }
      
	DT_DEBUG_LTC( *fT); 
      }
    else
      {   DAQEXIT( "requested temperature  probe ID not found "<<fDeviceId << endl);
      }

    setCalibrationId(fT->getCalibrationId());
   }
    fInitialized=true;
    //delete parser;
    
    
  }

TemperatureChannelLt::~TemperatureChannelLt()
{
   switch ( fType ) 
     {case TempConverDef::AD590 :
      case TempConverDef::AD590DUMMY :      
      case TempConverDef::NTC   : 
       case TempConverDef::NTCAD4717 	:  
       { TemperatureChannel* fTC = (TemperatureChannel*) fT; 
         delete fTC->getAdcChannelPointer();
       }		
      break;
      default :                 
      break;
     }
   delete fT;
} 





