
#include "tempctrlinterface.h"
#include "temperaturechanneltempctrl.h"
#include "tempctrlreg.h"
#include "daqerrormsg.h"

  TemperatureChannelTempCtrl::TemperatureChannelTempCtrl( 
    CalExchange <TempConverDef::TempCalValues>* c )
                        :TemperatureChannelInterface(c) 
                         
{
  fDeviceId=getId();
  fInitialized=false;
   fTc= getTempCtrlMapInfo(c->getBoardNr());   
  if(!fTc){ DAQEXIT("error in getting tempctrl  channel for device "
		   << fDeviceId) ;
  }
  fTc->getTempCtrlReg()->increaseReference();
  fProbe=getChannelNr();
  if(fProbe == 2 && !fTc->waterTemperatureProbe()){
    DT_WARN("requested not existing water temperature probe from "
	    << fDeviceId) ;		
    return;
  }
  
  if(fProbe == 3 && !fTc->hotSideTemperatureProbe()){
    DT_WARN("requested not existing hotside  temperature probe from "
	    << fDeviceId) ;		
    return;
  }
  DT_DEBUG_LTC("Tempctrl device ="<<c->getBoardNr()<<" probe "<< fProbe);
  fInitialized=true;
  fLastReadTemperature=fTc->getTemperature();
    

}

float TemperatureChannelTempCtrl::getTemperature() 
{ float temperature=-TCZERO; 
  switch (fProbe){
    case 0:
    case 1: { 
      temperature=fTc->getTemperature();
    }
    break;
    case 2: {
      temperature=fTc->getWaterTemperature();
    }
    break;
    case 3: {
      temperature=fTc->getHotSideTemperature();
    }
    break;
    default: 
      DT_WARN("requested none extiting tempctrl temperatue channel "<<fProbe );
      temperature=-TCZERO;
  }
  fLastReadTemperature=temperature;  
  return temperature;
}

TemperatureChannelTempCtrl::~TemperatureChannelTempCtrl(){  
  fTc->getTempCtrlReg()->decreaseReference();
}

void TemperatureChannelTempCtrl::info(){
  fTc->info();
}
