#include <signal.h>
#include "daqerrormsg.h"
/** \brief temperature channel Trhx code 
     
    
     \author W. Beaumont University Antwerpen 
     \version 1.0 
     \date 2001 10 01 
    
 
*/
#include "temperaturechanneltrhxsens.h"

static TDeviceControl* fControl =0;

TemperatureChannelTrhxSens::TemperatureChannelTrhxSens
      ( int TrhxBaseComPort,
	 CalExchange <TempConverDef::TempCalValues>* c )
    :TemperatureChannelInterface(c)                          
{ fInitialized=false;
  // Attach to Main control share memory
  if (fControl== 0)
    fControl =new TDeviceControl("ControlePrincipal",TShare_OPEN);

  fPid = getpid();
  fCd = 0;
  if (AttachTrhxTempChannel(getBoardNr(),getChannelNr()) <0)
    {
      DT_ERROR("TrhxChannel Cannot getTRHX  temperature board "<<  getBoardNr() << " channel " << getChannelNr());
      if (fCd!=0)
        fControl->ReleaseChannel(fCd->fPidServer,fPid,fCd->fChannelNumber);
      return;
    }
  fTrhxTemperature=999;
  fInitialized=true;



    
   }


float TemperatureChannelTrhxSens::getTemperature() 
    {  double  temperature ; 
    if (getType() == TempConverDef::TRHXSENS)
      {	 
	temperature= (*fSens)[TRHXTemperature][ getChannelNr()];
	fTrhxTemperature=temperature;
    
      }
      else 
	{ temperature = (*fNtc)[getChannelNr()];
	  fTrhxTemperature=temperature;
	}
    temperature= Adc2Tconversion((float)temperature);
	
    return (float)temperature;
    }

TemperatureChannelTrhxSens::~TemperatureChannelTrhxSens()
{ 
  this->release();
}

void TemperatureChannelTrhxSens::release()
{
  if (fInitialized && fCd!=0)
    fControl->ReleaseChannel(fCd->fPidServer,fPid,fCd->fChannelNumber);

}


int TemperatureChannelTrhxSens::AttachTrhxTempChannel
        (int serial_nr, int channelnr )
{ 
  if (getType() == TempConverDef::TRHXSENS)
  {  fCd = fControl->RequestChannel(fPid,TRHXSENS,channelnr,serial_nr);
  }
  else 
  {  if (getType() == TempConverDef::TRHXNTC)
     {    fCd = fControl->RequestChannel(fPid,TRHXNTC,channelnr,serial_nr);
     }
     else
     {  fCd =0;
     }
  }
  if (fCd <= 0) return -1;
  kill(fCd->fPidServer,SIGCONT);
  printf("je signal %d \n",fCd->fPidServer);
  printf("Shared  Memory %s %d \n",fCd->fShareName,fCd->fShareSize);
  fMemoryClient = new TShare(fCd->fShareName,TShare_OPEN,fCd->fShareSize);
  fTrhx = (TrhxDescription*) fMemoryClient->Attach();
  fSens =  fTrhx->getSensBuffer();
  fNtc  =  fTrhx->getNtcBuffer();
  return 0;
}


