#include "viproxy.h"
#include "humiditychanneltcpip.h"
#include "temperaturechanneltcpip.h"
#include "hvchanneltcpip.h"
#include "currentchanneltcpip.h"
#include "tempctrltcpip.h"
#include "dimsensserver.h"
#include "ltparser.h"

LtParser* ltparser;

int main(){ 
  // the lt parser (configuration)
  ltparser = new LtParser();     
  if(ltparser->parseFile()) 
{}//    DAQEXIT("error in lt configuration file look at error output");
  // the proxy (tcpip connection)
  ViProxy*  theProxy = new ViProxy("admwks3.fynu.ucl.ac.be",7550);
  // prepare to use slow control info
  SlowCtrlConfig* thescc= ltparser->getSlowCtrl(1);
  int theNtemp=thescc->getNrTempProbes(); 
  int theNhum=thescc->getNrHumidityProbes(); 
  DT_DEBUG_LTAPPL(" Setup: Ntemp "<<theNtemp<<" Nhum "<<theNhum);
  vector<TemperatureChannelInterface*> mtempchvec;
  // the temperature channels 
   for(int itemp=0;itemp<theNtemp;itemp++) { 
     DT_DEBUG_LTAPPL("Temp " <<  itemp <<" Id "<<  thescc->getTempProbeId(itemp));
     CalExchange <TempConverDef::TempCalValues>* c=
        c = ltparser-> getTemperatureCalibration(thescc->getTempProbeId(itemp));
     if (c&&(c->getType()==TempConverDef::TEMPSPEC))
     {  
       mtempchvec.push_back(new TemperatureChannelTCPIP(theProxy,"T",c->getBoardNr(),c->getChannelNr())); 
     }
   }
   // the humidity channels 
   vector<HumidityChannelInterface*> mhumchvec;
   for(int ihum=0;ihum<theNhum;ihum++) {
     DT_DEBUG_LTAPPL("Hhm " <<  ihum <<" Id "<<  thescc->getHumidityProbeId(ihum)) ;
     CalExchange <HumidityConversionDef::HumidityCalValues>*
       c = ltparser-> getHumidityCalibration(thescc->getHumidityProbeId(ihum) );
     if (c&&(c->getType()==HumidityConversionDef::HUMSPECIAL))  
     { 
       mhumchvec.push_back(new HumidityChannelTCPIP(theProxy,"H",c->getBoardNr(),c->getChannelNr()));
     }
   }
   // the temperature controls for the cold box 
   vector<TempCtrlInterface*> mtempctrlvec;
   CalExchange <BoxCharacteristicsDef::BoxCharValues>*     
          c = ltparser-> getColdBox(thescc->getColdBoxId(0));
   if(c)
   {
     vector<int>* m= &(c->getCalValues()->fTempCtrlId);
     for (  vector<int>::const_iterator p=m->begin();p!=m->end();p++)
     {
       CalExchange <BoxCharacteristicsDef::BoxCharValues>* c_tc=
	 ltparser->getTempCtrlInfo(*p);  
       if (c_tc&&(c_tc->getType()==BoxCharacteristicsDef::DIMCTRL))
       {
         mtempctrlvec.push_back(new TempCtrlTCPIP(theProxy,"F",c_tc->getBoardNr(),c_tc->getChannelNr()));
       }
     }
   }
   // the hv
   vector<HVChannelInterface*> mhvunit;
   vector<CurrentChannelInterface*> curchvec;
   for (int slotpos =0 ; slotpos < 10 ; slotpos++) {  
     BenchPosConnection* BenchP= 
       new BenchPosConnection(*ltparser->getBenchPosConnections(slotpos));
     if ( BenchP->getId() == -1) continue ;
     DT_DEBUG_LTAPPL("hv found for slot pos "<<slotpos <<" Id " <<  BenchP->getId() );
     CalExchange <HvConversionDef::HvCalValues>* cv=
       ltparser-> getHvCalibration(BenchP->getBiasDevice());
     if(cv&&((cv->getType()==HvConversionDef::HvDIM)||((cv->getType()==HvConversionDef::HvSTRUCK_NHQ201)&&(cv->getConnectionType(),"Dim"))))
     {
       mhvunit.push_back(new HVChannelTCPIP(theProxy,"HV",cv->getBoardNr(),cv->getChannelNr()));
     }
     CalExchange <CurrentConvDef::CurrentCalValues>* c=
       c = ltparser-> getCurrentCalibration( BenchP->getHvCurrentDevice( ));
     if ( c&&(c->getType()==CurrentConvDef::CURRENTSPEC)) {  
       curchvec.push_back(new CurrentChannelTCPIP(theProxy,"HVI",c));
     }
   }
  // the dimsensserver... 
  DimSensServer dss(0, mtempchvec , mhumchvec , mtempctrlvec, mhvunit,curchvec);
  // run !
  printf("dim sens init done\n");
  if( dss.DimServerNameNotValid()){return -1;}
  while(1)sleep(5);
  return 0;
}

