#ifndef __HVCAENA33
#define __HVCAENA33
//#include <channelid.h>
#include "hvchannelinterface.h"

/*! \file hvcaena33.h 
  \brief  Class to use the HV Caen modules A33 dummy !!! 
*/

/*! \class  HvCaenA33Channel
    \brief  Class to use the HV Caen modules A33 dummy !!!
    Can be used as dummy Hv channel 
    This is a test class should be developed to a more realistic class
    The class should provide the implementation of attaching the object to
    the  Hv channel  with channel id cid via the shared memory and take care of all 
    conversion needed so the settings are
    in V or V/s 
    for this is should use the calibration values of this channel stored in the 
    calibration and/ or configuration files



    \author W. Beaumont Universtity Antwerpen
    \version 1.0
    \date 2001 03 20
*/



class HvChannelCaenA33 :  public HvChannelInterface 
{ private:
  float fVolt;
  float fCurrent;
  float fVoltMax;
  float fCurrentMax;
  float fRamp;
  int fDeviceId; 
 
 public: 
  HvChannelCaenA33(int Did  )
    { fVolt=0;
      fCurrent=0;
      fVoltMax=0;
      fCurrentMax=0;
      fRamp=0;
      fDeviceId=Did; 
    }
  virtual ~HvChannelCaenA33(){};
  
  
   //!<  gives the HV Volt in volts  since the last readout 
  virtual float  getHvVolt( )
    { return fVolt ;}
  
   virtual float  getHvCurrent( )
     { return fCurrent;}
     
   //! returns the device ID of the channel
   virtual int getId(){return fDeviceId;}

   //! get the current HV ramp setting of the device (since last readout) in [V/s]
   virtual float getHvRampVolt()
     { return fRamp;}

   //!  get the current HV current limit  setting of the device since last readout result is in uA    
   virtual float getHvCurrentLimit()
     {  return fCurrentMax ;}

   //!  get the status of the HV unit 
   /*!  unit depending like HV on, overflow ... 
     voltage set point reached , ramping ... 
     returns -1 if fails or function is not implemented for the chosen device
   */
   virtual int getHvStatus()
     { return 0 ;}
   
   //!  get the maximum voltage that is allowed 
   /*!  if return value is zero it could be this function is not available in the current device
    */
   virtual float getHvVoltLimit()  { return fVoltMax;}
   
   
   //! get the limit voltage that is allowed for this channel  this number is set by the setHvSwLimit
   virtual float getHvSwVoltLimit()
     { return fVoltMax;}   
   
   //!set the software hv limit  return the limit 
   virtual int  setHvSwVoltLimit(float newvalue)
     { fVoltMax= newvalue ; return 0;}
   
   //! set the HV value to  newvalue in volts return 0 if success
   /*!  returns -1 if fails or function is not implemented for the chosen device
     returns -2 if the requested value is higher than the setHvSwLimit ;
   */

   virtual int  setHvVolt(float newvalue)
     { fVolt= newvalue ; return 0;}
   
   //!  set the ramp voltage to newvalue [V/s]
   /*!  returns -1 if fails or function is not implemented for the chosen device
    */
   virtual int setHvRampVoltage(float newvalue )
     {  fRamp=newvalue ;return 0 ;}

   //! set the current limit to newvalue in uA
   /*!  returns -1 if fails or function is not implemented for the chosen device
    */  
   virtual int setHvCurrentLimit(float newvalue)
     { fCurrentMax = newvalue ;  return 0;}

   //! set the voltage limit to newvalue in V
   /*!   returns -1 if fails or function is not implemented for the chosen device
    */
   virtual int setHvVoltLimit(float newvalue)
     { fVoltMax = newvalue ; return 0;}
   
   virtual void  LockDataBuffer()
   { ;}
   virtual void  UnlockDataBuffer()
     { ;}
   
   //!< send info about the class to cout only for debugging
   /*!< displays the values of the 
     - current voltage
     - ramp voltage
     - current current
     - current limmit
   */
   virtual void Info()
     { cout << "dummychannel" << endl;}
   
   virtual  void* GetSharedMemPtr()
   {  return 0;} 


};

#endif 
