#ifndef HVCHANNELDIM_H
#define HVCHANNELDIM_H
#include "hvchannelinterface.h"
#include <dic.hxx>
#include "dimchannelnames.h"
#include "CalibrationValues.h"
#include "daqerrormsg.h"


/** \class HvChannelDim
    \author C. Delaere
    \date October 2002
    \version 1.0

    \brief Simple mostly dummy class to access via DIM a HV unit controled externally.
    Only set/get HvVolt and getHvCuttent are used.
*/
class HVChannelDim : public HVChannelInterface,
                     public DimChannelNames,
		     public DimClient
{
public:
  /** Constructor, implements HVChannelInterface 
      @param b Board Id
      @param c Channel number 
  */
  HVChannelDim(CalExchange <HvConversionDef::HvCalValues>* c); 
  virtual ~HVChannelDim(){}
  /** CallBacks to HV descriptions */
  virtual float getHvVolt(); 
  virtual float getHvCurrent();
  virtual int   getHvStatus() { return true; }
  virtual float getHvVoltLimit() { return fVoltLimit; }
  virtual float getHvSwVoltLimit() { return fSwVoltLimit; }
  virtual float getHvCurrentLimit() { return fCurrentLimit; }
  virtual float getHvRampUp() { return fRampUp; }
  virtual float getHvRampDown() { return fRampDown; }
  virtual int   getV0() { return (int)getHvVolt(); }
  virtual int   setHvVolt(float newvalue);
  virtual int   setHvSwVoltLimit(float newvalue) { fSwVoltLimit = newvalue; return 1;}
  virtual int   setHvVoltLimit(float newvalue) { fVoltLimit = newvalue; return 1;}
  virtual int   setHvCurrentLimit(float newvalue) { fCurrentLimit = newvalue; return 1;}
  virtual int   setHvRampUp(float newvalue) { fRampUp = newvalue; return 1;}
  virtual int   setHvRampDown(float newvalue) { fRampDown = newvalue; return 1;}
  /** abstract Wait loop */
  virtual int wait(); 
  void infoHandler();
private:
  float fSwVoltLimit, fVoltLimit, fCurrentLimit, fRampUp, fRampDown;  
  DimInfo* fHvinfo;
  char fCmd[80];
  float fSent[256];
  float  fRecBuf[256];
  int fNsent;
  int fNreceived;
  bool fIsInitialized;
};
#endif

