#ifndef __HVCHANNELSTRUCKNHQ201
#define __HVCHANNELSTRUCKNHQ201
#include <hvchannelinterface.h>

#include <TShare.h>
#include <TDeviceControl.h>

#include "nhq201description.h"

/** \file hvchannelstrucknhq201.h 
    \brief  interface class between  Shared memory and application program   to access the HV channel
*/ 


/**
    \brief  interface class between  Shared memory and application program   to access the HV channel
    

    <b> remark: The Hv Channel software limit is set to 0 during initialization (default C++)
    So set the  setHvSwVoltlimit() before oprating the device </b> otherwise no voltage will be accepted.
    At the moment the proposal for the HV interface 
    the setHvSwVoltLimit can be very usefull.
    Each Si detector has an maximum operating voltage (determined by the detector acceptance test)
    To implement this limitation in the setHvVolt method no other checks has to be done.
    Version 1.0 made use of a hvbasic class . This is implemented in the class iteself for 
    verion 2.0  Version 2.0 has not to be considered as a stable version 
    \author W. Beaumont University Antwerpen 
    \version 1.0 
    \date 2000 12 31 
    \version 2.0
    \date 20011111 
*/
class HvChannelStruckNhq201   :  public    HVChannelInterface

{  private:
  //bool UpdateDone();
  int AttachHvChannel( int serial_nr, int channelnr );
  void   InitHvChannel (long sernr,  long channel   )  ;
  
  Nhq201ChannelDescription*  fHv;
  Nhq201Description* fHvDevice ;
  TShare* fMemoryClient;
  
  TDeviceControl* fControl ;
  pid_t fPid;
  ChannelDescription* fCd;

  float fHvSwLimit;
  int   fBoardNr; //!<  no use for the moment 
  
  bool fInitialized;
  int fActiveChannels;
  int fChannel;
  
  
 public:
  HvChannelStruckNhq201(int  HVDevId, int  channel);
  
  ~HvChannelStruckNhq201() ;
  
   virtual float  getHvVolt( ); 
    virtual int getV0() ;
   virtual float  getHvCurrent( ); 
   virtual float getHvRampUp();
   virtual float getHvRampDown();
   virtual float getHvCurrentLimit();  
   virtual int getHvStatus();
 
   virtual float getHvVoltLimit();
    
   virtual float getHvSwVoltLimit();
   
   virtual int  setHvSwVoltLimit(float newvalue);
     
   virtual int  setHvVolt(float newvalue);
 
   virtual int setHvRampUp(float newvalue );
   virtual int setHvRampDown(float dummy);
   
 
   virtual int setHvCurrentLimit(float newvalue);
     
   virtual int setHvVoltLimit(float newvalue);
          
   virtual void Info();
   
     /** Access Lock to parameters */
  virtual void  lock() ;
  /** Access UnLock to parameters */
  virtual void  unlock() ;
  /** Release of channel to the DAQ */
  virtual void  release(); 
  /** Request DAQ to change parameter */
  virtual void  justDoIt();
  int wait();
} ;


#endif
