#include <stdio.h>
#include <errno.h>
#include "hvchanneldim.h"
#include "ltparser.h"
#include "daqerrormsg.h"


HVChannelDim::HVChannelDim(CalExchange<HvConversionDef::HvCalValues>* c):HVChannelInterface(c->getBoardNr(), c->getChannelNr())
{ 
  fIsInitialized=false; 
  setDimServerName ( c->getShmName());
  if (DimServerNameNotValid()) { DT_WARN("wrong dim server name");} 
  fHvinfo= new DimInfo((char*)DimHvChannelAckName(c->getBoardNr(),c->getChannelNr()),-1,this);
  DT_DEBUG_LTC("DimInfo:" << DimHvChannelAckName(c->getBoardNr(),c->getChannelNr()));
  memset(fCmd,0,80);
  sprintf(fCmd,"%s",DimHvChannelCmdName(c->getBoardNr(),c->getChannelNr()));
  fNsent=0;
  fNreceived=0;
  fIsInitialized=true;
}

float HVChannelDim::getHvVolt()
{
  fSent[0] = (float) GET_VOLT;
  fSent[1] = 0.3;
  if(!DimClient::sendCommand(fCmd,fSent,2*sizeof(int)))
	{ DT_WARN("failed to send \n"); return -999; }
  fNsent++;
  if(wait() < 0)
  	{ DT_WARN("no reaction from server\n");return -998;}
  DT_DEBUG_LTC("received" << fRecBuf[0] << "send "<< fNsent <<" fNreceived "<<  fNreceived);
  if( (int) fRecBuf[0] == (int) GET_VOLT){
    return fRecBuf[ ACTVOLT];
  }
  else {DT_WARN("wrong answer "<< fRecBuf[0]);
  return -997;
  }
}

float HVChannelDim::getHvCurrent()
{
  fSent[0] = (float) GET_HVCURR;
  fSent[1] = 0.3;
  if(!DimClient::sendCommand(fCmd,fSent,2*sizeof(int)))
	{ DT_WARN("failed to send "); return -999; }
  fNsent++;
  if(wait() < 0)
  	{ DT_WARN("no reaction from server\n");return -998;}
  DT_DEBUG_LTC("received" << fRecBuf[1] << "send "<< fNsent <<" fNreceived "<<  fNreceived);
  if( (int) fRecBuf[0] == (int) GET_HVCURR){
    return fRecBuf[ ACTCUR];
  }
  else {DT_WARN("wrong answer "<< fRecBuf[0]);
  return -997;
  }
}

int HVChannelDim::setHvVolt(float newvalue)
{ 
  fSent[0] = (float) SET_VOLT;
//CTK
  printf("WWWWWWWWWWWW: newvalue=%f SETHV=%d\n",newvalue,SETHV);
  fSent[SETHV] = newvalue;
//CTK
  fSent[1] = newvalue;
  fSent[2] = newvalue;
////
  if(!DimClient::sendCommand(fCmd,fSent,4*sizeof(int)))
	{ DT_WARN("failed to send \n"); return -999; }
  fNsent++;
  if(wait() < 0)
  	{ DT_WARN("no reaction from server");return -998;}
  DT_DEBUG_LTC("reqested "<<newvalue<<" received" << fRecBuf[SETHV] 
	       << "send "<< fNsent <<" fNreceived "<<  fNreceived
	       << "message was from" << fRecBuf[0]);
  ///CTK
  DT_DEBUG_LTC("SET_VOLT "<<SET_VOLT<<" fSent[0]" <<  fSent[0]
               <<" fSent[1]"<<fSent[1]<<" fSent[SETHV]"<<fSent[SETHV]
               <<" SETHV"<<SETHV);
  return 1;
}

void HVChannelDim::infoHandler()
{
  DimInfo* info=getInfo();
  // memcpy(fRecBuf,0, 256*sizeof(short));
  if (info == fHvinfo)
  {
    fNreceived++;
    if(info->getSize() >  (int)(256* sizeof(float))) 
    { 
      fRecBuf[0]=-2; fRecBuf[1]=-990.99; 
    }
    else
    { 
      memcpy(fRecBuf,info->getData(),info->getSize());    
      DT_DEBUG_LTC("Hv info size"<<info->getSize()<<" cmd: "<< (int)fRecBuf[0]<<" Value: "<< fRecBuf[1] );
    }
  }
  else
  {
    fRecBuf[0]=-1; fRecBuf[1]=-990.99; 
  }
}

int HVChannelDim::wait()
{
  int nc=0;
  do
    {
      usleep(1000);
      nc++;
      if (nc>1000) return -1;
    } 
  while (fNreceived<fNsent);
  return nc;
}

