
#include "hvchannellt.h"

#include "CalibrationValues.h"
#include "string.h"
#include "hvchannelstrucknhq201.h"
#include "hvchanneldummy.h" 
#include "ltparser.h"
#include "n470channel.h"
#include "sy127channel.h"
#include "dimsy127channel.h"
#include "hvchanneldim.h"
#include "dimn470channel.h"
#include "hvchannelsy2527.h"
#include "sy527channel.h"

/*! \file hvchannellt.cxx
    \brief implementation of the HvChannelLt class

    The getHVtype is used to extract the type of HV module
    This is used in the switch statement to intstanciate the correct HV channel object
    and connect this to the HvBiasInterface class.
    for version 1.1 more hv types are supported 
    
    \author W. Beaumont Universtity Antwerpen
    \version 1.0
    \date 2001 03 20
    \version 1.1
    \date 2003 03 
*/

extern LtParser* ltparser;

HvChannelLt::HvChannelLt( int DeviceId) : HVChannelInterface(-1,-1){  
  fDeviceId = DeviceId;  
  fIGain=1;
  map<int,RegHvCh*>::iterator p= fRegHv.find( fDeviceId) ;
  if( p == fRegHv.end()) { // no registered channel  
    if (fDeviceId == HvConversionDef::DUMMYDEVICEID){      
      HvBias = new HvChannelDummy(0,0);
    }
    else {
    CalExchange <HvConversionDef::HvCalValues>* c=0 ;
    int nChannels =ltparser->getNumberHvChannels();
    DT_DEBUG_LTC("HvChannels " << nChannels );
    c = ltparser-> getHvCalibration(fDeviceId);
    if ( c) {	
      DT_DEBUG_LTC(c->printIdValues( cout ));
      fType= c->getType();
      fIGain=c->getCalValues()->CurrentGain;
      DT_DEBUG_LTC("set current gain to "<<fIGain);
      switch ( fType )	  {
	case HvConversionDef:: HvSTRUCK_NHQ201  : {
	  if ( strcmp(c->getConnectionType(),"Dim"))  {
	    DT_DEBUG_LTC("init NHQ201 HV channel (device " << fDeviceId << ")");
	    HvBias = new HvChannelStruckNhq201(c->getBoardNr(), c->getChannelNr());
	    }
	  else
	  {
	    DT_DEBUG_LTC( "init generic dim channel (device " << fDeviceId << ")");
	    HvBias = new HVChannelDim(c);
	  }
	}
	break;
	case HvConversionDef:: HvDIM : {
	  DT_DEBUG_LTC( "init generic dim channel (device " << fDeviceId << ")");
	  HvBias = new HVChannelDim(c);
	 }
	break;
	case HvConversionDef::HvCAEN_N470 : {
	  if ( !strcmp(c->getConnectionType(),"Dim")) {  
  	    DT_DEBUG_LTC( "init N470 dim channel " ); 
  	    HvBias = new DimN470Channel(  0x320*10000+c->getBoardNr(), c->getChannelNr());	    
  	  }
  	  else 
	    {
	    HvBias = new N470Channel(c->getTDeviceControlName(),c->getBoardNr()*10000, c->getChannelNr()); 
	  }
	}
	break;
	case HvConversionDef::HvCAEN_A434 : {
	  if ( !strcmp(c->getConnectionType(),"Dim"))  {  
	    cout << "init sy127 dim channel " << endl;    
	    HvBias =new DimSY127Channel(0x320*10000+c->getBoardNr(), c->getChannelNr()); 
	    //HvBias = new HvChannelDummy(c->getBoardNr(), c->getChannelNr());
	  }
	  else { 
	    cout << "init sy127 channel " << endl;  
	    HvBias = new SY127Channel(c->getTDeviceControlName(),c->getBoardNr(), c->getChannelNr()); 
	  }
	    HvBias -> lock();
	    HvBias -> setHvCurrentLimit(50);
	    HvBias -> justDoIt();
	    HvBias -> unlock();
	    HvBias -> wait();
	    HvBias -> lock();HvBias -> setHvVolt(0); 
	    HvBias -> justDoIt();HvBias -> unlock(); HvBias -> wait();                    
	    HvBias -> lock(); HvBias -> setHvRampUp(10); HvBias -> setHvRampDown(10);
	    HvBias -> justDoIt(); HvBias -> unlock();   HvBias -> wait();	   
	    HvBias -> lock();HvBias -> turnOn();HvBias -> justDoIt();HvBias -> unlock();
	    HvBias -> wait();  
	  
	}
	break;		
	case HvConversionDef::HvCAEN_527  :{
	  int bd_num = int(c->getChannelNr()/20);
          int ch_num = (c->getChannelNr()%20);

          SY527Channel* temp_chan = 
	    new SY527Channel(c->getTDeviceControlName(), 
			     c->getBoardNr()*10000, 
			     c->getChannelNr());
          // Unpack the Board number and Channel number and Set them
          temp_chan->setSY527BoardId(bd_num);
          temp_chan->setSY527ChannelId(ch_num);
          HvBias= temp_chan; 
          if (!temp_chan->isInitialized()) {
	    cout << " Channel not initialised " << endl;
            delete HvBias;
	    HvBias = 0;
            break;
          }
          cout  << " sy527 HV unit with Board # " << bd_num 
                << " and Channel # " << ch_num << " is initialized " << endl;
	  HvBias -> lock();
          HvBias -> setReadOnly(false);
	  HvBias -> setHvVolt(0);
	  HvBias -> setHvCurrentLimit(100);
          HvBias -> setHvRampUp(11);
          HvBias -> setHvRampDown(20);
          HvBias -> turnOn();
          HvBias -> justDoIt();
          HvBias -> unlock();
          HvBias -> wait(); 
	}
	break; 
	case HvConversionDef::HvCAEN_2527 :{
	  // check connection type but only direct connection is implemented
	  // shared memory name is the name of the url name of the crate
	  HvBias = new HvChannelSy2527(c->getTDeviceControlName(),
	    c->getShmName(),c->getBoardNr(), 
				       c->getChannelNr());
	}
	break;
	case HvConversionDef::DUMMY :	{
	  HvBias = new HvChannelDummy(c->getBoardNr(), c->getChannelNr());
	}
	break ;	        	 
	default : { 
         DAQEXIT( "requested type  not defined: " <<fType) ;
	}         
	break;
      }
    
      RegHvCh *hr=new RegHvCh;
      hr->hv=this; hr->nrreg=0;
      fRegHv[fDeviceId]=hr;
      fRefferenceToChannel=false;	
    }
    else  {
      {DAQEXIT( "requested probe ID not found: " <<fDeviceId) ;
      }
    }
    }
  }
  else  { //existing device ID  
    DT_DEBUG_LTC("exising hv " );
    HvBias = fRegHv[fDeviceId]->hv;
    fRegHv[fDeviceId]->nrreg++;
    fType = 0;
    fRefferenceToChannel=true;    
  }
}

 HvChannelLt::~HvChannelLt()
{  
  map<int,RegHvCh*>::iterator p= fRegHv.find( fDeviceId) ;
  if( p != fRegHv.end()){
    if ( ! fRefferenceToChannel) // real instantiation 
      { DT_DEBUG_LTC("delete hv source ");
	delete HvBias ;
	if(p->second->nrreg > 0){ 
	  DT_WARN("Deleted hv channel with references unstable system left");	      
	}
	fRegHv.erase(p);   
      }
    else{
      p->second->nrreg--;
    }
  }
  else {
    DT_WARN("Deleted unregistered hvchannel ");
  }
}

void HvChannelLt::Info(){    
   std::cout<<"current gain" << fIGain<<std::endl;
   HvBias->Info();
}
  


map <int,HvChannelLt::RegHvCh*> HvChannelLt::fRegHv;
