#include "daqerrormsg.h"
#include <stdio.h>
#include <signal.h>
#include "hvchannelstrucknhq201.h"
#include <iostream>
/*! \file hvchannelstrucknhq201.cxx
    \brief implementations of the Hv Channel class for the struck HV power supply
    \author W. Beaumont Universtity Antwerpen
    \version 1.0
    \date 2001 03 20
*/


HvChannelStruckNhq201::HvChannelStruckNhq201(int  HvDevId, int  channel) :HVChannelInterface(HvDevId,channel)
{printf("init hv StruckNhq201 \n");
  fActiveChannels=2;
  InitHvChannel( HvDevId, channel);
}

HvChannelStruckNhq201::~HvChannelStruckNhq201() 
{   setHvVolt(0);     
  delete fMemoryClient; 
 
  fControl->ReleaseChannel(fCd->fPidServer,fPid,fChannel);
  delete fControl;
}

void    HvChannelStruckNhq201::InitHvChannel (long ser_nr,  long channel_arg  )  
{   
   fChannel=channel_arg;
   fBoardNr=ser_nr;
 
   fHvSwLimit=0;
   fActiveChannels=0; // has to be overwritten by the physical  HV channel
  
   
   { fControl = new TDeviceControl("ControlePrincipal",TShare_OPEN);
        // in case new fails the program will exit
        fPid  = getpid();
     }
  
 
       
   if ( AttachHvChannel(fBoardNr,fChannel))
     {  fprintf(stderr," channel %d , board %d could not be attached \n",
	fChannel,fBoardNr);
     fInitialized=false;
        exit(-1); // throw ??
     }
   //else  now the channel is found
 { fInitialized=true; }
   fActiveChannels=fHvDevice->getChannels();
   

  
  
 return ;       
}

int HvChannelStruckNhq201::AttachHvChannel( int serial_nr, int channelnr )
{ 
//printf("try to get getHv  channel %d of board %d \n", channelnr, serial_nr);
  fCd = fControl->RequestChannel(fPid,NHQ104HV,channelnr,serial_nr);
  // name should be filled in by server because board_id is given
  if (fCd == NULL)
    { fprintf(stderr,"channel request failed \n");
      
	  delete fControl;
	  return -1;
	
    }
  // else adc is found 
 
  fMemoryClient = new TShare(fCd->fShareName,TShare_OPEN,fCd->fShareSize);
  
  fHvDevice  = (Nhq201Description*) fMemoryClient->Attach();
  fHv = fHvDevice->getNhq201ChannelDescription(channelnr);
  if (kill(fCd->fPidServer,SIGCONT) )
    { return -1; // seems the server died 
    }
   return 0;
}	  


//  bool HvChannelStruckNhq201::UpdateDone()
//  {  bool NotDone;
//            NotDone =true;
//            while (NotDone)
//              {   usleep(1000); // always start with waiting 1 s  
//  	        lock();
//  		NotDone = (fHv->fStatus != DONE_NHQ); 
//                  unlock();		
//              }
//  	  return !NotDone ;
//  }        

float    HvChannelStruckNhq201::getHvVolt( )
{  float result;
// justDoIt();
// wait();
//lock();
   result = (float) fHv->getVmon() ;
   //unlock();
   return  result ;
	   
}
 int  HvChannelStruckNhq201::getV0() {
 float result;
 //  justDoIt();
 //  wait();
 //  lock();
   result =  fHv->getV0() ;
   //  unlock();
   return (int) result ;
 }


 void  HvChannelStruckNhq201::Info()
{ // this is not written with effeciency in mind !!!
  
  printf(" Hv Software limit %f \n", fHvSwLimit );
  printf(" actual value %d \n",  fHv->getVmon());
  printf(" Hv limit %f \n", getHvVoltLimit());
  
}

 float  HvChannelStruckNhq201::getHvCurrent( )
{  float result; // justDoIt();
//  lock();
     result =  (float) fHv->getImon() ;
     //   unlock();    
     return result ;
}

float  HvChannelStruckNhq201::getHvRampUp()
{  float result; 
//   lock();
   result =  (float)fHv->getRampUp()  ; //assumes set value is real value
   //  unlock();

   return result ;
}

float  HvChannelStruckNhq201::getHvRampDown()
{ cout <<"Warning NHQ201 rampdown function not supported by the Nhq201 " ;
   return -1;}


float  HvChannelStruckNhq201::getHvCurrentLimit()
{float result;
//  lock();
     result =  fHv->getI0() ; //assumes set value is real value
//    unlock();
    
    return result ;
}

 float  HvChannelStruckNhq201::getHvVoltLimit()
{   float result;
//  lock();
    result =  fHv->getVmax()  ; //assumes set value is real value
    //  unlock();
    return result ;
}

 int  HvChannelStruckNhq201::getHvStatus()
{  int  result;
//   lock();
     result =  fHv->getStatus() ; //assumes set value is real value
     //  unlock();
    return result ;

}


float HvChannelStruckNhq201::getHvSwVoltLimit()
{ return  fHvSwLimit;
}



int  HvChannelStruckNhq201::setHvSwVoltLimit(float newvalue)
{  float hwlimit;
  if ( newvalue > (hwlimit=getHvVoltLimit()))
    {  fHvSwLimit=hwlimit;
     return -2;
  }
  //else ok
  fHvSwLimit = newvalue;
  return 0;       
}


int  HvChannelStruckNhq201::setHvVolt(float newvalue)
{ bool value_valid = false ; 
  // check valid negative values 
  if ( newvalue < 0 && fHvSwLimit <  0   && fHvSwLimit <  newvalue )
    { value_valid = true;  }
  // check valid positive values
   if ( newvalue > 0 && fHvSwLimit >  0   && fHvSwLimit > newvalue )    
     { value_valid = true;  }
   if ( newvalue == 0)  { value_valid = true;  }
   if( value_valid)
     {
       //   lock();
       fHv->setV0((int) newvalue) ;
      
       //  unlock();
       //   justDoIt();
     }
 
   return value_valid?0:-2;

}


 int  HvChannelStruckNhq201::setHvRampUp(float newvalue )
{ //lock();
      fHv->setRampUp((int) newvalue) ;
      
      //unlock();
   
      // justDoIt();
   return 0;
}

 int  HvChannelStruckNhq201::setHvRampDown(float dummy)
     { cout<< "Warning NHQ201 rampdown function not inmplemented" ;
                                  return -1;
     }

int  HvChannelStruckNhq201:: setHvCurrentLimit(float newvalue )
{ //lock();
      fHv->setI0((int)newvalue) ;
       
      //  unlock();
   
//   justDoIt();
   return 0;
}

int  HvChannelStruckNhq201::setHvVoltLimit(float newvalue)
{ //lock();
      fHv->setV0( (int) newvalue) ;
    
      // unlock();
   
 
      // justDoIt();
   return 0;
}



void HvChannelStruckNhq201::justDoIt()
{
  if (fInitialized && fHv!=0 && fCd!=0)
  { 
   fHv->requestAction(getpid());
   //  cout << "pid server =" << fCd->fPidServer;
    kill(fCd->fPidServer,SIGCONT);
  }
  else
    { cout << "just do it failed ";
    if ( !fInitialized) cout << " not initialized " ;
    if ( !fHv) cout<< " not attached to HV shared memory ";
    if ( !fCd)cout << " not connected to Tcontrol ";
    cout << endl;
    }	
	
}

void HvChannelStruckNhq201::lock(){  fMemoryClient->Lock();}

void HvChannelStruckNhq201::unlock(){  fMemoryClient->Unlock();}

void HvChannelStruckNhq201::release()
{
  if (fInitialized && fCd!=0)
    {
    fControl->ReleaseChannel(fCd->fPidServer,fPid,fCd->fChannelNumber);
    fMemoryClient->Detach(fHv);
    }
}

int HvChannelStruckNhq201::wait()
{    int r=0;
  do {
    r++;if( r > 20) { r=0; DT_DEBUG_LTC("long wait");}
    usleep(300000);
  } while (!fHv->isDone());
  return r;
}
