#ifndef __AdcChanneldummy
#define  __AdcChanneldummy
#include "adcchannelinterface.h"
#include "adcdescription.h"
#include "TShare.h"
#include "TDeviceControl.h"

/** 
    \class AdcChannelDummy
    \author W. Beaumont
    \date May  2001
    \version 1.0


    \brief Adc dummy  implementation of AdcChannelInterface

    <h2> Usage </h2>
    It implements AdcChannelInterface class without making use of the e Daq  
    the user:
    <ul>
    <li> it simple generates fixed data as output is requested. All other funcitons are dummies and will have no real action
    </ul>
    <pre>
    // Request 64 channel 

    for(int i =0;i < 64 ; i++)
    {
    
    adcc[i] = new AdcChannelDummy(64 ,i);
    if (!adcc[i]->isInitialized()) exit(-1);
    }
    // Loop
    int lastspill =0 ; short mybuf[10000];
    while (event_to_get)
    {
     adcc[0]->lock();
     // Check new spill
     if (adcc[0]->getSpillNumber() == lastspill) 
        adcc[0]->unlock();  
     // Spill number
     lastspill = adcc[0]->getSpillNumber();
     // Get events
     for (int j=0;j<8;j++)
       {
         adcc[i]->getAdcData(mybuf,1024);
         ....
         // do what you want with mybuf
         ....
       }
       
     adcc[0]->unlock();
    }

    // Release
    for (int i=0;i<64;i++)
     adcc[i]->release();
    </pre>



*/

class AdcChannelDummy  : public AdcChannelInterface
{ 
 private:
  static int fSpill ;
  int fRandomRange ;
  int  variation( );
  void InitAdcChannelDummy(float level, int fluctuation);
  static char AdcChannelDummy::h[10]; 
  static int  AdcChannelDummy::r;

  float fLevel;

public:
  /** Constructor
      @param number the adc device number determins which adc will be used.
      @param channel  wich channel of the adc will be used
       */   
  AdcChannelDummy(long number, long channel);

/** Constructor
    @param level  the avarage output level of the dummy adc
    @param fluctuation the fluctuation 
    @param number the adc device number determins which adc will be used.
    @param channel  wich channel of the adc will be used
       */ 
AdcChannelDummy(float level, int fluctuation,
		long adc_ser_nr, long channel_arg);


  virtual ~AdcChannelDummy();
  /** give the avarage ADC value for the channel 
      returns the number of samples used to calculate the avarage 
      @param adcvalue the avarage value of the ADC input 
  */ 

  virtual int getAdcData(float& adcvalue );
  

  /** Copy Adc Data to user buffer 
      @param out_buf Output buffer
      @param size_of_buf Size of output buffer
   */   
   virtual int getAdcData(unsigned short *out_buf, int size_of_buf );
 /** Return Sample i */
  virtual unsigned short getAdcSample(int i);
  /** Return Sample i from the whole buffer */
  virtual unsigned short getSampleFromBuffer(int i);
  /** Get Spill number (buffer counts) */
  virtual int getSpillNumber(){return fSpill++;} 
  /** Lock share memory access */
  virtual void  lock() {;}
  /** Unlock share memory access */
  virtual void  unlock() {;} 
  /** detach share memory access and release daq channel */
  virtual void  release();
  /** only for debug reasons */ 
  virtual void   info(std::ostream& sout);
};

#endif 
	

