#ifndef __BENCHPOSCONNECTION
#define __BENCHPOSCONNECTION

#include <cstdio>
#include <string>

/**
   \class BenchPosConnection
   \author W. Beaumont  
   \date May 2001
   \version 1.00

   \brief class that gives the device numbers connected to the station slot
   The class is created to use as an interface between an application (long term ) and 
   the parser that reads the configuration for the corresponding setup
   It is tested with the LtParser class. 
   By using this class the parser and application can be developed Independent.
 
   It gives information about the device ID's to which the modules are connected.
   The application can get informataion about these devices with the LtParser class. 
   
 */
class BenchPosConnection{ 
 public:
 enum FedChannelType {DUMMY,DIMDEV,SHAREDMEM};
 enum I2cDevType{DUMMYI2C,
		 APV25,APV25DIM,MUX25,MUX25DIM,
		 DCU1,DCU2,DCU1DIM,DCU2DIM,
		 OPTO,OPTODIM,PLLDIM, PLL,
		  CCU6,CCU25};
 typedef struct{
    /// the mux channel the vutri channel is connected to
    int fMuxChannel;
    /// the mux device the vutr channel is connected to
    int fMuxDevice;
    /// the mux card (inside the fMuxDevice) the vutri channel is connected to
    int fMuxCard;
    /// to the switch group the channel belongs to 
    int fMuxGroup;
  } MuxChId;
 private:
  typedef struct 
{ int fFedSerialNr;
  int fFedChannel;
  bool  fFedSimulation; 
  MuxChId fMuxId;
  FedChannelType fFedType;
} UtriConnection;

  private :
    int fDeviceId; // slot number;
  int fDetectorId;  
  int fNTC[2];
  int fPowerSwitch;
  bool fFecSimulation;
  int fBias;
  int fHvCurrent;
  int fLvCurrent[2];
  UtriConnection  fUtri[3];
  int fFec;
  int fCcuAddress;
  int fCcuChannel;
  I2cDevType fCcuType;
  I2cDevType fDcuType;
  I2cDevType fOptoType;
  I2cDevType fPllType;
  unsigned int fLatencyOffset;
  I2cDevType fApvType;
  I2cDevType fMuxType;
  std::string fDimHost ;
  public :
   BenchPosConnection( int Id)  { 
    fDeviceId = Id;
    fDetectorId=-1;
    fHvCurrent=-1;
    fNTC[0]=-1; fNTC[1]=-1;
    fBias=-1;
    fLvCurrent[0]=-1;fLvCurrent[1]=-1;
    
     for (int lc=0; lc <3;lc++){
       fUtri[lc].fFedSerialNr=-1;
       fUtri[lc].fFedChannel=-1;
       fUtri[lc].fMuxId.fMuxChannel=-1;
       fUtri[lc].fMuxId.fMuxDevice=-1;
       fUtri[lc].fMuxId.fMuxCard=-1;
     }

     fFec=-1;
     fCcuAddress=-1;
     fCcuChannel=-1;
     fPowerSwitch=-1;
    }
  // setters
  // take care no check for array boundaries
  void setDimHostName(std::string contype){fDimHost=contype;}
  void setNTCDevice( int id, int nr){ fNTC[nr]=id;}
  void setHvCurrentDevice( int id) { fHvCurrent=id;}
  void setFedChannel( int id, int nr){ fUtri[nr].fFedChannel=id;}
  void setFedType(FedChannelType i,int nr){fUtri[nr].fFedType=i;} 
  void setBiasDevice( int id) { fBias=id;} 
  void setLvCurrentDevice( int id, int nr){ fLvCurrent[nr]=id;}
  void setPowerSwitch (int id){fPowerSwitch=id;}
  void setFedNr(int id, int nr ) { fUtri[nr].fFedSerialNr=id;}
  void setFecNr(int id) { fFec=id;}
  void setCcuAddress(int id) { fCcuAddress=id;}
  void setCcuChannel(int id) { fCcuChannel=id;}
  void setFedSimulation (bool sim, int nr)
                     { fUtri[nr].fFedSimulation=sim;}  
  void setFecSimulation (bool sim) { fFecSimulation=sim;}  
  void setOptoType(I2cDevType type) {fOptoType=type;}
  void setPllType(I2cDevType type) {fPllType=type;}
  void setLatencyOffset(unsigned short in) {fLatencyOffset=in;}
  void setDcuType(I2cDevType type) {fDcuType=type;}
  void setMuxType(I2cDevType type) {fMuxType=type;}
  void setApvType(I2cDevType type) {fApvType=type;}
  void setMuxId ( MuxChId muxid,int nr) {fUtri[nr].fMuxId=muxid;}
  // getters
  int getId(){ return fDeviceId;}
  int getNTCDevice(  int nr){  return fNTC[nr];}
  int getHvCurrentDevice( ) {  return fHvCurrent;}
  int getUtriChannel( int nr){ return  fUtri[nr].fFedChannel;}
  int getBiasDevice( ) { return  fBias;} 
  int getLvCurrentDevice(  int nr){  return fLvCurrent[nr];}
  int getPowerSwitch(){ return fPowerSwitch;}
  int getFedNr(int nr) {  return  fUtri[nr].fFedSerialNr;}
  FedChannelType getFedType(int nr){return fUtri[nr].fFedType;} 
  int getFecNr() {  return  fFec;}
  int getMuxChannel(int nr ){ return  fUtri[nr].fMuxId.fMuxChannel;}
  int getCcuAddress() { return fCcuAddress;}
  int getCcuChannel() { return fCcuChannel;}
  int getFedSimulation(int nr) { return fUtri[nr].fFedSimulation;}
  int getFecSimulation() { return fFecSimulation;}
  std::string  getDimHostName(){ return fDimHost;}
  I2cDevType getOptoType(){ return fOptoType;}
  I2cDevType getPllType(){ return fPllType;}
  unsigned int getLatencyOffset(){return fLatencyOffset;}
  I2cDevType getDcuType(){ return fDcuType;}
  I2cDevType getApvType(){ return fApvType;}
  I2cDevType getMuxType(){ return fMuxType;}
  MuxChId getMuxChId(int nr ) { return fUtri[nr].fMuxId;}
  /// use this function only for debug reasons
  void info()
    {  printf("module nr: \t %d \n",fDeviceId);
    for ( int lc =0 ; lc < 2 ; lc++)
      { printf("NTC device%d: \t %d \n",lc,fNTC[lc]);
      }
    printf("Bias: \t %d \n",fBias);
    printf("HvCurrent \t %d \n",fHvCurrent);
    printf("Fednr \t %d \n", fUtri[0].fFedSerialNr);
    printf("CCu address\t %d\t ",fCcuAddress);
    printf("CCu channes\t %d\n",fCcuChannel );
    for (int i=0 ; i < 3; i++)
      { printf("Utri %d -->fedch %d, muxch %d  ",
	       i,fUtri[i].fFedChannel, fUtri[i].fMuxId.fMuxChannel);
	 
      }
    printf("\n");
    }
    
};

#endif
