#ifndef __DIOCHANNELINTERFACE 
#define  __DIOCHANNELINTERFACE 

#include "channelid.h"
#include <iostream>
/** 
    \class DioChannelInterface
    \author W.Beaumont, L.Mirabito
    \date March 2002
    \version 1.0

    \brief Abstract all functionnalities of a  DioChannelInterface
    \version 1.0
    
*/

class DioChannelInterface: public ChannelId 
{ public:
  enum diomode {DIO_IN, DIO_OUT, DIO_INOUT, DIO_INVALID};



  //variable 
protected: 
  /** Connect to the Daq and check if channel is available,
      it turns isInitialized() to true if Ok
  */
  bool fInitialized;
 diomode fMode;  
public:
  /** Constructor
      @param number the adc device number determins which adc will be used.
      @param channel  wich channel of the adc will be used
       */   
  DioChannelInterface
  (long number, long channel);

  virtual ~DioChannelInterface();

  /** get the value  of the bit , 1 or 0 
   */   
  virtual int getBit()=0;
 /** get the value  of the bit , 1 or 0 
     @param status  will be 0 if o.k. else -1 
   */   
  
  virtual diomode getMode(){ return fMode;}
  // setMode can only be done during initialization 

  virtual int getBit(int& status) { status=0; return getBit();}
  /** set the bit 
    @param i  if 0 bit is set to 0 else to 1 other values than 1 can have special meaning
    @return  0 if ok. else returns -1 if error 
  */
  virtual int  setBit( int i) =0;
  /** should send a signal to the server if implemented*/
  virtual void justDoIt(){;} 

  /** Lock access */
  virtual void  lock(){;}
  /** UnLock access */
  virtual void  unlock(){;}
  /** Release access */
  virtual void  release() {;}
  /** True if channel correctly initialized */
  inline bool isInitialized() { return fInitialized;}
 
  /** wait until ready  */
  virtual int wait()=0;

  /** only for debug reasons */ 
  virtual void   info(std::ostream& sout){;}
};

#endif 
	

