#ifndef __DIOCHANNELTRHX
#define  __DIOCHANNELTRHX
#include "diochannelinterface.h"
#include "trhxdescription.h"
#include "TShare.h"
#include "TDeviceControl.h"

/** 
    \class DioChannelTrhx
    \author W .Beaumont
    \date March 2002
    \version 1.0


    \brief Cio  implementation of DioChannelInterface

   

    <h2> Usage </h2>
    It implementsDioChannelInterface class and make the Daq transparent to 
    the user:
    <ul>
    <li> it connects to the Main TDeviceControl and request channel
    <li> it maps the Ddc Share memory corresponding part to a AdcDescription
    <li> it controls access to this AdcDescrition thru methods lockDataBuffer,
    unlockDataBuffer,getApvEventData and release.
    </ul>
    <pre>
    




*/

class DioChannelTrhx : public DioChannelInterface
{ private:
  TrhxDescription* fDio; 
   ChannelDescription* fCd;
  pid_t fPid;
  TShare *fMemoryClient;
  int fLastStatus ;
 
  //variable 
protected: 
  /** Connect to the Daq and check if channel is available,
      it turns isInitialized() to true if Ok
  */
  virtual int  AttachDIOChannel(  int serialnr, int channelnr );
public:
  /** Constructor
      @param TDeviceControlName in case the name has to be given
      @param dioserialnr the board serial nr or device nr 
      @param channel  wich channel of the adc will be used
      @parm mode can be input or output  
       */   
  DioChannelTrhx(char* TDeviceControlName,long dioserialnr, long channel,diomode  mode);

  virtual ~DioChannelTrhx();

  /** set the bit 
    @param i  if 0 bit is set to 0 else to 1 , 2 pulse 
    !!! locks also the buffer during requese 
  */
  virtual int  setBit(int i  );
 
  /** get the status of the bit , 1 or 0 
   */   
  virtual int getBit();
 /** get the value  of the bit , 1 or 0 
     @param status  will be 0 if o.k. else -1 
   */   
  virtual int getBit(int& status) ;

  
  /** Lock share memory access */
  virtual void  lock() {fMemoryClient->Lock();}
  /** Unlock share memory access */
  virtual void  unlock() {fMemoryClient->Unlock();} 
  /** detach share memory access and release daq channel */
  virtual void  release();
  /** give the average ADC value for the channel 
      returns the number of samples used to calculate the avarage 
      @param adcvalue the average value of the ADC input 
  */
  
  virtual int wait();

  /** only for debug reasons */ 
  virtual void   info(std::ostream& sout);
   



};

#endif 
	

