#ifndef IDEXCHANGE__
#define  IDEXCHANGE__ 
/** /brief  class to get and set the general information from the system data base or file 
   this class contains the code to get and set the values needed for the differeent physic channels
   like  temperature channel, hv channel etc.
   This class provides the storage and access of the standard data like id, name etc. 

   \author W. Beaumont University Antwerpen 
     \version 1.0 
     \date 2002 06 02 
*/

#include <iostream>
#include <cstdio>
#include <cstring>
class IdExchange
{  public:
  //! the structure that define the fields that each object that use the calibration interface should have
 

  typedef struct
  { int fId ;
    int fBoardNr;
    int fType;
    int fChannel;// channel or device ID reffered to if refference
    char  fChannelMode[30];  // the mode in which the channel has to be set or is expected 
    char fShmName[50]; //< the shared memory name not in use or dim server name or the tcpip for direct access. 
    int fDevice ; 
    int fModule; //< not in use
    char  fConnectionType[50]; //< connection type normal , dim , reference
    char fName[50];
    char fTdname[50]; //<  Name of the TDeviceControl, controlling the Daq or the dim host name 
    bool fSimulation; // in case of simulation required  this can be set true
  } CalibrationId;
 protected:  
 CalibrationId fCId;
 
 public:
  IdExchange(){}
  virtual ~IdExchange(){}
   void setType (int type ) {fCId.fType =type;}
   void setBoardNr ( int BN) { fCId.fBoardNr =BN;}
   void setChannelNr ( int CN ) { fCId.fChannel=CN;}
   void setModuleNr ( int CN ) { fCId.fModule=CN;}
   void setCalibrationId( const CalibrationId ci){ 
      memcpy( &fCId , &ci , sizeof (fCId));
     }
   void setCalibrationId(const  CalibrationId *ci)
     { memcpy( &fCId , ci , sizeof (fCId));
     }
   void setName( char *c) { strcpy( fCId.fName, c);}
   void setTDeviceControlName( char * c){ strcpy( fCId.fTdname, c);}
   void setId(int i){ fCId.fId=i;}
   void setShmName( char * c)
     { strcpy( fCId.fShmName, c);}
   void setSimulationState(bool simulate  ){ fCId.fSimulation=simulate;}
   void setChannelMode( char * c){ strcpy( fCId.fChannelMode, c);}
   void setConnectionType( char * c){ strcpy( fCId.fConnectionType, c);}
   int getBoardNr() {return fCId.fBoardNr ;}
   int getId () {return fCId.fId;}
   int getChannelNr() {return fCId.fChannel;}
   int getModuleNr() {return fCId.fModule;}
   virtual char*  getName() { return fCId.fName ;}
   char*  getShmName() { return fCId.fShmName ;}
   /// return the name for the TDeviceControl sharred memory
   char* getTDeviceControlName( ) {return fCId.fTdname ;}
    int getDevice(){ return fCId.fDevice;}
    int getType () {return fCId.fType;};
    char* getChannelMode(){ return  fCId.fChannelMode; }
   char* getConnectionType(){ return  fCId.fConnectionType; }
   bool getSimulationState( ) {return fCId.fSimulation;}
   /// returns the calibration ID structure (not the pointer !!)
   CalibrationId getCalibrationId( ){ return fCId; }

  std::ostream&      printIdValues( std::ostream&    s   )
       { 
       s  << " ID \t  " << fCId.fId << std::endl;
       printf(" Board nr\t %d\n", fCId.fBoardNr);  
       printf(" Channel \t %d\n", fCId.fChannel);  
       printf(" Type nr \t %d\n", fCId.fType);
       if (strlen(fCId.fChannelMode))
	 {   printf(" Mode  \t %s\n",fCId.fChannelMode);}    
       if (strlen(fCId.fName))
       {   printf(" Name \t %s\n",fCId.fName);      }
      if (strlen(fCId.fShmName))
       {   printf(" ShmName \t %s\n",fCId.fShmName);     
       }  
       if (strlen(fCId. fTdname))
       {   printf(" TdcName \t %s\n",fCId. fTdname);     
       }  
       return s;   
     }

};
#endif 
