#ifndef LT_PARSER
#define LT_PARSER
#include "commonparser.h"
#include "CalibrationValues.h"
//#include "benchposconnection.h"
#include "muxexchangedef.h"
#include "pmap.h"
#include <string>
class BenchPosConnection;
class TscChannelInterface;
class LedCardInterface;
class FBCardInterface;
class FedProcessCtrlInterface;
class FecCtrlDevInterface;
class DioChannelInterface;
class SlowCtrlConfig;

/**
   \class LtParser
   \author W. Beaumont 
   \date May 2001
   \version 1.00
   \version 2.00
   \version 2.2
   \version 2.3 
   \date December 2002 
   \date March 2003 
   \brief  XML parser for the connections for the long term test used daqparser as example

   It Parse a given file, create and fill corresponding description.
   Take care for the getDevice (i) the is changed from 
   the order in the available devices to the device number version 1.1   !!!!'
  
   2.3 extention with ledcard 
   2.4 extention with forward bias card
   
 */
class DOM_Node;

#define MAXBENCHPOS 20

class LtParser: public CommonParser
{
  friend class BenchPositionParser;
  private :
    std::string fParseFileName;

private:

  /// indicates if first used. If true than the file will not parsed
 static  bool fileparsed;


  /// number of temperature probes in the system
  
   pmap<int,CalExchange <TempConverDef::TempCalValues>*> temperatures;
  
   //pmap<int,  CalExchange<HumidityConversionDef::HumidityCalValues>*> humiditys;
   /// number of current channels ( LV and HV channels)
    int ncurrents;
    CalExchange<CurrentConvDef::CurrentCalValues>*  current[MAXBENCHPOS*2+10];
   /// number of hv channels in the system
    int nhvs;
    CalExchange<HvConversionDef::HvCalValues>*  hv[MAXBENCHPOS+10];
   /// number of bench positions 
    //int nbenchpos;
    // BenchPosConnection* benchpos[MAXBENCHPOS];
    pmap<int,  CalExchange <int>*> tsc;
    pmap<int,  CalExchange <int>*> fedctrl;
    pmap<int,  CalExchange <int>*> fecctrl;
    pmap<int,  CalExchange <int>*> ledcard;
    pmap<int,  CalExchange <int>*> fbcard;
    pmap<int, CalExchange <MuxExchangeDef::MuxValues>*> muxdev;
    pmap<int,
      CalExchange <BoxCharacteristicsDef::BoxCharValues>*>  coldbox;
    pmap<int,
      CalExchange <BoxCharacteristicsDef::BoxCharValues>*>  tempctrls;


    /// power channels that controls the peltier in the system
    // pmap <int,CalExchange<PowerConversionDef::PowerCalValues>*> 
    //  power;
    /// gas device controls for control of flow control devices 
    pmap <int,CalExchange<GasFlowDef::GasFlowValues>*> gasflows;
  /// humidty ctrl for control of the humidyt in a cold box 
    //pmap <int,CalExchange<HumidityCtrlDef::HumidityCtrlValues>*> humidityctrls;
    //HumidityCtrlParser* fHumidityCtrlParser;
    /// set all devices to emulation 
    // now in common parser  bool fSimulation; 

    //pmap< int, CalExchange<int>* >  Dioline;
    pmap <int , CalExchange<int[3]>*> Interlock;
    pmap<int,SlowCtrlConfig*> slowctrls;

protected:
  /// Parse temperature  tag
  int TemperatureParser(DOM_Node &ei);
  int MuxCardParser(DOM_Node &ei,
		    CalExchange<MuxExchangeDef::MuxValues>* CE);
  int MultiPlexerParser(DOM_Node &ei);
  int MuxCtrlParser(DOM_Node &ei);
  //int HumidityParser(DOM_Node &ei);
  int CurrentParser(DOM_Node &ei);
  //int BenchPosConnectionParser(DOM_Node &ei);
  int HvParser(DOM_Node &ei);
  int ColdBoxParser(DOM_Node &ei,BoxCharacteristicsDef::BoxType kind );
  //int CoolingPowerParser(DOM_Node &ei);
  int SimulationParser (DOM_Node &ei);
  /// detail parser for the TSC card
  int TscCardParser(DOM_Node &ei );
  int FedProcessCtrlParser(DOM_Node &ei );
  void  FedCtrlParser(DOM_Node &ei);
  int FecCtrlParser(DOM_Node &ei) ;
  int  InterLockParser(DOM_Node &ei);
  //int  DioLineParser(DOM_Node &ei);
  int  SlowCtrlConfigParser(DOM_Node &ei);
  int DaqProcessCtrlParser(DOM_Node &ei) ;
  int LedCardIoParser(DOM_Node &ei);
  int LedCardKarlsParser(DOM_Node &ei);
  int LedCtrlParser(DOM_Node &ei);
  int FBCardIoParser(DOM_Node &ei);
/*    /// parser for DAC !!!  and ADC channels   */
/*    template <class T >  */
/*      void AdcChannelParser(DOM_Node &ei, CalExchange<T>* ); */

/*    template < class T>  */
/*      void I2cChannelParser(DOM_Node &ei,CalExchange<T>*  CE ); */

/*    /// parser for dummy channel  */
/*    template < class T>  */
/*      void DummyChannelParser(DOM_Node &ei,CalExchange<T>*  ); */

/*    /// parser for HV channels  connected via the CaenNet */
/*    template < class T> void CaenNetChannelParser(DOM_Node &ei,CalExchange<T>*  CE); */

/*    /// parser for probes connected via the serial channel to get the serial channel details  */
/*    template < class T> void SerialChannelParser(DOM_Node &ei,CalExchange<T>* ); */
  /// Ad590 probe details , connected via ADC channel
  void TemperatureAd590Parser(DOM_Node &ei,
			 CalExchange <TempConverDef::TempCalValues>*  CalValues);
  /// NTC  probe details , connected via ADC channel
  void TemperatureNTCParser(DOM_Node &ei,
			    CalExchange <TempConverDef::TempCalValues>*  CV);
  /// Temperature probe connected via serial channel not implemented yet
  void TemperatureSerialParser(DOM_Node &ei,
			    CalExchange <TempConverDef::TempCalValues>*  CV);
  void TemperatureTempCtrlParser(DOM_Node &ei,
			 CalExchange <TempConverDef::TempCalValues>*  CV);
  /// Humidity channel for Dim 
  // void HumiditySpecialparser(DOM_Node &ei,
//			     CalExchange<HumidityConversionDef::HumidityCalValues>*  CV);

  /// Humidity probe Mirel  connected via (two) ADC channels
//  void HumidityMirel1050parser(DOM_Node &ei,
//		    CalExchange<HumidityConversionDef::HumidityCalValues>*  CV);
  /// Humidity probe Mirel connected to the Trhx box
//  void HumidityTrhxparser(DOM_Node &ei,
  //		  CalExchange<HumidityConversionDef::HumidityCalValues>*  CV);

  /// Current channel connected to a ADC
 void CurrentAdcParser(DOM_Node &ei,
		    CalExchange<CurrentConvDef::CurrentCalValues>*  CV);
 /// current channel read from a voltage module (integrated current measurement
 void CurrentHvModuleParser(DOM_Node &ei,
		    CalExchange<CurrentConvDef::CurrentCalValues>*  CV);
 /// details of the calibration values for the current channel
 void CurrentChannelCalValuesParser(DOM_Node &ei,
				    CurrentConvDef::CurrentCalValues*  HCV) ;

 /// parser for the Hv struck device  details
 void HvStruckParser(DOM_Node &ei,CalExchange<HvConversionDef::HvCalValues>*  CV);
 /// parser for a power channel connected via an DAC channel
 ///void PeltierDacParser(DOM_Node &ei, 
//		       CalExchange < PowerConversionDef:: PowerCalValues>*  CV);
 /// parser for the details of the connections to the I2C (FEC, CCU)
/*   int I2cConnectionParser(DOM_Node &ei,BenchPosConnection* a ); */
/*   /// parser for details how the UTRI card is conneced to the FED's */
/*   int FedConnectionParser(DOM_Node &ei,BenchPosConnection* a ); */
/*   /// detail parser for each UTRI channel */
/*   int UtriChannelParser(DOM_Node &ei,BenchPosConnection* a ); */
 /// humidity ctrl parser 
 //int HumidityCtrlParser(DOM_Node &ei);
 /// gas flow parser 
 int GasFlowParser(DOM_Node &ei);
 /// parser for the dac type used by the gasflow device 
 void GasDacParser(DOM_Node &ei, 
		   CalExchange < GasFlowDef::GasFlowValues>*  CV);

public:
 LtParser();
  /** Parse the file
      @param name File Name
   */
 virtual ~LtParser(){}
// ~LtParser() { clear();}
 /// the call that parse the document. returns a non zero value if there was an error.  
  int  parseFile(const char* name);
  /** parse the file , use the default config file({DAQHOME}/Config/lt.xml
      or the name give by setParseFileName
  */
  int parseFile() ;

  /** returns the name of the configuration file */
  std::string  getParseFileName();
  
  /// replace the contents of s to the standard config file ({DAQHOME}/filename
  char*  getStandardConfigFile(char* s, const char * filename );
  /// returns the default configuration directory 
  std::string getStandardConfigPath( );
  /** set the file name for the configuration 
      if filename starts with / the absolute path is assumed
      if filename start without / the $DAQHOME/Config directory is assumed
      @return the file name (full path) 
   */
  std::string  setParseFileName( std::string filename);   


  /// Clear all Description
  void clear();
  /// returns true if the overall simulation flag is set
  bool getSimulation() { return fSimulation;}
  /// Return number of current probes
  int getNumberTemperatureProbes() const { return (int)temperatures.size();}
  /// Return the number of current channels in the system
  int getNumberCurrentProbes() const { return ncurrents;}
  /// Returns the number of modules  / bench positions in the system 
  int getNumberBenchPositions() const; 
  /// number of HV channels in the system 
  int getNumberHvChannels() const { return nhvs;}
  /// number of power channels to control a peltier in the system
//int getNumberPowerChannels() const { return power.size();}
  /// number of cold boxes in the system
  int getNumberColdBoxes() const{return coldbox.size();}
  int getNumberTempCtrl() const { return tempctrls.size();}
  /// returns the number of humidity probes in the system
  ///int getNumberHumidityProbes() const { return humiditys.size();}
  int getNumberTscCards() { return tsc.size();}
  int getNumberFedProcessCtrl() { return fedctrl.size();}
  int getNumberFecCtrlDev() { return fecctrl.size();}
 /*  { */
/*                                     cout << ncoldboxes<<endl;   */
/*  				   return ncoldboxes;} */
  
  
  /** Return the temperature CalExchange class settings
      @param  device  the device number , return 0 if the device is not found
  */ 
  CalExchange <TempConverDef::TempCalValues>* 
    getTemperatureCalibration( int device);
  
  
  /** returns the calibration values for the current channel 
      @param the current device number 
  */ 
  CalExchange<CurrentConvDef::CurrentCalValues>*
    getCurrentCalibration(int i) const;

  /** returns the logical bench position number of the first module registered
      this can be different from the bench position.
      the follow order in the xml file gives the number i,
      the device number in the lt.xml file is the real position number
  */
  
  BenchPosConnection* getRegisteredBenchPos(int i);
  
  /** returns the bench position connection information for the
      benchposition i ( the device id )
      If the bench position is not defined an empty bench position
      connection information is returned.
  */

  BenchPosConnection* getBenchPosConnections(int i) ;//const;
 
  
  /** retruns  the cal value class for humidity
        @param the current device number
  */
//  CalExchange <HumidityConversionDef::HumidityCalValues>*
//    getHumidityCalibration( int device ) ; 

  
 /** retruns  the cal value class for hv channel
        @param the current device number
  */
   CalExchange <HvConversionDef::HvCalValues>*
    getHvCalibration( int device )const ;

    /**   returns the calibration values for the power  channel to control a peltier
     @param the current device number
  */
CalExchange < PowerConversionDef:: PowerCalValues>* 
    getPowerChannelCalibration(int device  );
   /** returns detail settings for the cold box.
       @param the current device number
  */

   CalExchange <BoxCharacteristicsDef::BoxCharValues>*
     getColdBox (int device ){ return coldbox.getP(device);}


 
 /// returns detail settings for the tsc   i should be 0 (only one card ).
   CalExchange <int>* getTscCardInfo(int i)  { if (i!=0 ) {return 0;}
   else {return tsc.getP(i);} }
   TscChannelInterface* getTscCard(int i) ;
 
   /** returns the fedprocess control information 
       if ordernr =false it return the information for feddevnr i
       otherwise it return the pointer to the i'st declaration of 
       the fedprocess informatoim in the configuration file 
   */
   CalExchange <int>* getFedProcessCtrlInfo(int i,bool ordernr=false)  { 


#ifdef SUBSTRIMPL
	if( ordernr){ fedctrl.getPnr(i);}       
     else   { return fedctrl.getP(i);}
#else  
	if (i!=0 ) {return 0;}
     else {return fecctrl.getP(i);} 
#endif
   }

   FedProcessCtrlInterface* getFedProcessCtrl(int i,bool ordernr=false);

   CalExchange <int>* getFecCtrlDevInfo(int i)  { 
     if (i!=0 ) {return 0;}
     else {return fecctrl.getP(i);} 
   }
   FecCtrlDevInterface* getFecCtrlDev(int i);

CalExchange <BoxCharacteristicsDef::BoxCharValues>*
 getTempCtrlInfo (int device)  
  {  CalExchange <BoxCharacteristicsDef::BoxCharValues>*
    a = 
    tempctrls.getP(device) ;
  // cout << a-> printIdValues(cout) << endl;
  return a;
}
 int getNrLedCards() {return ledcard.size();} 
 CalExchange <int>* getLedCardInfo(int device) {
   return ledcard.getP(device);
 }
 LedCardInterface* getLedCard(int i);

 int getNrFBCards() {return fbcard.size();} 
 CalExchange <int>* getFBCardInfo(int device) {
   return fbcard.getP(device);
 }
 FBCardInterface* getFBCard(int i);

 int getNrMuxDevices() {return muxdev.size();} 
 CalExchange <MuxExchangeDef::MuxValues>* getMuxDevInfo(int device) {
   return muxdev.getP(device);
 }

   CalExchange<int[3]>* getInterlockInfo( int device) 
     { return Interlock.getP(device); }

   /**returns detailed information of the dio channel 
       in respect to the other devices a dio channel has no device number
       the dio channel gets the id during parsing by the 
       order they are defined in xml file. 
       dio channels are supposed to be a sub element of an device.
      @param order the sequence number in which the dio  channel if found in the xml file
      @return the info of the dio channel
    */
   CalExchange<int>* getDioLineInfo( int order) ;
//  {return   Dioline.getP(order);}
 /** returns the pointer to the dio line that is defined by the
     information of the calexchange object c 
     @param c the calexchange class c that contains the info for the dioline
     @return the pointer to the instantiated dio channel
 */
 DioChannelInterface  * getDioChannel(CalExchange<int>*  c);
 /** returns the pointer to the dio line that was the defined 
     in the as the i'th dio line in the config file
     @param order  order number of the dio line in the config file
     @return the pointer to the instantiated dio channel
 */
 DioChannelInterface  * getDioChannel(int order);
//{
   // return  
//getDioChannel(Dioline.getP(order));
// }
 /** returns the pointer to the diochannel that switch on or off the backplane pulse as
     defined in the tsc card info
 */
 DioChannelInterface* getBackPlanePulseCtrl( int i);  

 SlowCtrlConfig* getSlowCtrl(int id){ return slowctrls.getP(id); }


};

 
#endif


