#ifndef SET_PARSER
#define SET_PARSER
#include "ModuleDescriptor.h"
#include "SetupDescriptor.h"
#include "ProcedureDescriptor.h"
#include "FlagsDescriptor.h"
#include "daqctrldescriptor.h"
#include "daqerrormsg.h"
#include <map>
#include <string>
         
/**
   \class SetParser
   \author W. Beaumont V.Zhukov
   \date 2003/09
   \version 2.2 V.Z.

   \brief settings.dtd XML parser based on Xerces library 

   Parse a given file, create and fill corresponding descriptors.
   The descriptions are for (default ) setting for the devices
   
 */


class DOM_Node;


class SetParser
{
 
private:
  string fFileName;
  /// number of I2C descriptors found in the file 
  int nI2cDescriptors;
  std::map<std::string,I2cDescriptor> fI2cDescriptorMap;
  //typedef  map<std::string,I2cDescriptor>::const_iterator I2cDecriptorIterator ;
  int nPedRunDescriptors;
  std::map<std::string,PedRunDescriptor> fPedRunDescriptorMap;


   int nCalRunDescriptors;
   std::map<std::string,CalRunDescriptor> fCalRunDescriptorMap;

  int nLightRunDescriptors;
   std::map<std::string,LightRunDescriptor> fLightRunDescriptorMap;
 
 int nFbRunDescriptors;
   std::map<std::string,FbRunDescriptor> fFbRunDescriptorMap;

  int nCalProfRunDescriptors;
   std::map<std::string,CalProfRunDescriptor> fCalProfRunDescriptorMap;

  int nLatRunDescriptors;
   std::map<std::string,LatRunDescriptor> fLatRunDescriptorMap;

 int nLedRunDescriptors;
   std::map<std::string,LedRunDescriptor> fLedRunDescriptorMap;

 int nBPlaneRunDescriptors;
   std::map<std::string,BPlaneRunDescriptor> fBPlaneRunDescriptorMap;
 
 int nConnectivityRunDescriptors;
   std::map<std::string,ConnectivityRunDescriptor> fConnectivityRunDescriptorMap;

 int nAuxChipRunDescriptors;
   std::map<std::string,AuxChipRunDescriptor> fAuxChipRunDescriptorMap;



    int nSensorDescriptors;
   std::map<std::string,SensorDescriptor> fSensorDescriptorMap;  

   int nDaqCtrlDescriptors;
   std::map<std::string,DaqCtrlDescriptor> fDaqCtrlDescriptorMap;

   int nSetupDescriptors;
   std::map<std::string,SetupDescriptor> fSetupDescriptorMap;  
   
  int nProcedureDescriptors;
   std::map<std::string,ProcedureDescriptor> fProcedureDescriptorMap;  

  int nFlagsDescriptors;
   std::map<std::string,FlagsDescriptor> fFlagsDescriptorMap;

protected:
  /// Parse temperature  tag
  int I2cDescriptorParser(DOM_Node &ei);
  int PedRunDescriptorParser(DOM_Node &ei);
  int CalRunDescriptorParser(DOM_Node &ei);
  int LightRunDescriptorParser(DOM_Node &ei);
  int FbRunDescriptorParser(DOM_Node &ei);
  int LatRunDescriptorParser(DOM_Node &ei);
  int CalProfRunDescriptorParser(DOM_Node &ei);
  int BPlaneRunDescriptorParser(DOM_Node &ei);
  int LedRunDescriptorParser(DOM_Node &ei);
  int ConnectivityRunDescriptorParser(DOM_Node &ei);
  int AuxChipRunDescriptorParser(DOM_Node &ei);
    int SensorDescriptorParser(DOM_Node &ei);
    int SetupDescriptorParser(DOM_Node &ei);
    int ProcedureDescriptorParser(DOM_Node &ei);
    int FlagsDescriptorParser(DOM_Node &ei);
    int DaqCtrlDescriptorParser(DOM_Node &ei);
  
public:
 SetParser()
 {  nI2cDescriptors=0; nPedRunDescriptors=0;
 nCalRunDescriptors=0 ;nSensorDescriptors=0;
 nSetupDescriptors=0; nProcedureDescriptors=0;
 nLatRunDescriptors=0;nDaqCtrlDescriptors=0;
 nLightRunDescriptors=0;nFbRunDescriptors=0;
 nFlagsDescriptors=0;nConnectivityRunDescriptors=0;
 nAuxChipRunDescriptors=0;
 fFileName="none";
   clear();
   DT_NOTE(" init setting parser clear  done ");
 }
  /** Parse the file
      @param name File Name
   */
 ~SetParser()
   { clear();
   }
  /// the call that parse the document. returns a non zero value if there was an error.  
  int  parseFile(const char* name);
  string getFileName(){ return fFileName;}
  /// Clear all Description
  void clear();
  int getNumberI2cDescriptors() const { return nI2cDescriptors;}

   int getNumberPedRunDescriptors() const { return  nPedRunDescriptors;}
   int getNumberLatRunDescriptors() const { return  nLatRunDescriptors;}
   int getNumberCalRunDescriptors() const { return  nCalRunDescriptors;}
   int getNumberLightRunDescriptors() const { return  nLightRunDescriptors;}
   int getNumberFbRunDescriptors() const { return  nFbRunDescriptors;}
   int getNumberSensorDescriptors() const { return  nSensorDescriptors;}
   int getNumberSetupDescriptors() const { return  nSetupDescriptors;}
   int getNumberProcedureDescriptors() const { return  nProcedureDescriptors;}
   int getNumberFlagsDescriptors() const { return  nProcedureDescriptors;}
   int getNumberCalProfDescriptors() const { return  nCalProfRunDescriptors;}
   int getNumberBPlaneRunDescriptors() const { return  nBPlaneRunDescriptors;} 
   int getNumberDaqCtrlDescriptors() const {return nDaqCtrlDescriptors;}
   int getLedRunDescriptors() const { return  nLedRunDescriptors;}  
   int getConnectivityRunDescriptors() const { return fConnectivityRunDescriptorMap.size();}  
    int getAuxChipRunDescriptors() const { return fAuxChipRunDescriptorMap.size();}   
   /** returns the pointer to the I2cDescription (settings) 
      @param selname  the id- name for the I2c settings  that should be found in the xml file.
    */
  I2cDescriptor*   getI2cDescriptor( std::string selname )  
    { if ( fI2cDescriptorMap.end() == fI2cDescriptorMap.find(selname))
      { DT_WARN( "I2cDescriptor " << selname <<" not found" ); 
	 return 0;
      }
      else {  return  &fI2cDescriptorMap[ selname];
      }  
    ;}
   
  PedRunDescriptor*   getPedRunDescriptor( std::string selname )  
    { if ( fPedRunDescriptorMap.end() == fPedRunDescriptorMap.find(selname))
      { DT_WARN( " PedRunDescriptor " << selname <<" not found" ); 
	 return 0;
      }
      else {  return  &fPedRunDescriptorMap[ selname];
      }  
    ;}
  LatRunDescriptor*   getLatRunDescriptor( std::string selname )  
    { if ( fLatRunDescriptorMap.end() == fLatRunDescriptorMap.find(selname))
      { DT_WARN( " LatRunDescriptor " << selname <<" not found" ); 
	 return 0;
      }
      else {  return  &fLatRunDescriptorMap[ selname];
      }  
    ;}

   CalRunDescriptor*   getCalRunDescriptor( std::string selname )  
    { if ( fCalRunDescriptorMap.end() == fCalRunDescriptorMap.find(selname))
      { DT_WARN( " CalRunDescriptor " << selname <<" not found" ); 
	 return 0;
      }
      else {  return  &fCalRunDescriptorMap[ selname];
      }  
    ;}

  LightRunDescriptor*   getLightRunDescriptor( std::string selname )  
    { if ( fLightRunDescriptorMap.end() == fLightRunDescriptorMap.find(selname))
      { DT_WARN( " LightRunDescriptor " << selname <<" not found" ); 
	 return 0;
      }
      else {  return  &fLightRunDescriptorMap[ selname];
      }  
    ;}
 FbRunDescriptor*   getFbRunDescriptor( std::string selname )  
    { if ( fFbRunDescriptorMap.end() == fFbRunDescriptorMap.find(selname))
      { DT_WARN( " FbRunDescriptor " << selname <<" not found" ); 
	 return 0;
      }
      else {  return  &fFbRunDescriptorMap[ selname];
      }  
    ;}


  CalProfRunDescriptor*   getCalProfRunDescriptor( std::string selname )  
    { if ( fCalProfRunDescriptorMap.end() == fCalProfRunDescriptorMap.find(selname))
      { DT_WARN( " CalProfRunDescriptor " << selname <<" not found" ); 
	 return 0;
      }
      else {  return  &fCalProfRunDescriptorMap[ selname];
      }  
    ;}

  BPlaneRunDescriptor*   getBPlaneRunDescriptor( std::string selname )  
    { if ( fBPlaneRunDescriptorMap.end() == fBPlaneRunDescriptorMap.find(selname))
      { DT_WARN( " BPlaneRunDescriptor " << selname <<" not found" ); 
	 return 0;
      }
      else {  return  &fBPlaneRunDescriptorMap[ selname];
      }  
    ;}

  LedRunDescriptor*   getLedRunDescriptor( std::string selname )  
    { if ( fLedRunDescriptorMap.end() == fLedRunDescriptorMap.find(selname))
      { DT_WARN( " LedRunDescriptor " << selname <<" not found" ); 
	 return 0;
      }
      else {  return  &fLedRunDescriptorMap[ selname];
      }  
    ;}
  ConnectivityRunDescriptor*   getConnectivityRunDescriptor( std::string selname )  
    { if ( fConnectivityRunDescriptorMap.end() == fConnectivityRunDescriptorMap.find(selname))
      { DT_WARN( " ConnectivityRunDescriptor " << selname <<" not found" ); 
	 return 0;
      }
      else {  return  &fConnectivityRunDescriptorMap[ selname];
      }  
    ;}

  AuxChipRunDescriptor*   getAuxChipRunDescriptor( std::string selname )  
    { if ( fAuxChipRunDescriptorMap.end() == fAuxChipRunDescriptorMap.find(selname))
      { DT_WARN( " AuxChipRunDescriptor " << selname <<" not found" ); 
	 return 0;
      }
      else {  return  &fAuxChipRunDescriptorMap[ selname];
      }  
    ;}

   SensorDescriptor*   getSensorDescriptor( std::string selname )  
    { if ( fSensorDescriptorMap.end() == fSensorDescriptorMap.find(selname))
      { DT_WARN( " SensorRunDescriptor " << selname <<" not found" ); 
	 return 0;
      }
      else {  return  &fSensorDescriptorMap[ selname];
      }  
    ;} 
   
   DaqCtrlDescriptor* getDaqCtrlDescriptor( std::string selname) {
     if ( fDaqCtrlDescriptorMap.end() == fDaqCtrlDescriptorMap.find(selname)){
       DT_WARN( " DaqCtrlDescriptor" << selname <<" not found" ); 
       return 0;
     }
     else {  return  &fDaqCtrlDescriptorMap[ selname];
     } 
   }
 
   SetupDescriptor*   getSetupDescriptor( std::string selname )  { 
     if ( fSetupDescriptorMap.end() == fSetupDescriptorMap.find(selname))
       { DT_WARN( " SetupRunDescriptor" << selname <<" not found" ); 
       return 0;
       }
     else {  return  &fSetupDescriptorMap[ selname];
     }  
   } 
  ProcedureDescriptor*   getProcedureDescriptor( std::string selname )  
    { if ( fProcedureDescriptorMap.end() == fProcedureDescriptorMap.find(selname))
      { DT_WARN( " ProcedureRunDescriptor" << selname <<" not found" ); 
	 return 0;
      }
      else {  return  &fProcedureDescriptorMap[ selname];
      }  
    ;} 
 FlagsDescriptor*   getFlagsDescriptor( std::string selname )  
    { if ( fFlagsDescriptorMap.end() == fFlagsDescriptorMap.find(selname))
      { DT_WARN( " FlagsDescriptor" << selname <<" not found" ); 
	 return 0;
      }
      else {  return  &fFlagsDescriptorMap[ selname];
      }  
    ;} 
 ModuleDescriptor* getModuleDescriptor( std::string name);
 
};

#endif


