#include "CalibrationValues.h"

// tmeperature conversion
float TempConverDef::NtcResistor2temperature
 ( float R , float R0 , float T0, float beta){  
  // only first order  so Rac = R0 exp { Beta ( 1/T -1/To)
  float temperature ; // resistor NTC at calibration temperature 
  temperature     =-TCZERO +  1/ 
    ( (1/(T0  +TCZERO) ) + 
      (1/beta) * log(R / R0) );
  return temperature ;
}     

float TempConverDef::Temperature2NtcResitor
 ( float T , float R0 , float T0, float beta){  
  // only first order  so Rac = R0 exp { Beta ( 1/T -1/To)
  return R0* exp( beta/((T+TCZERO)- 1/(T0+TCZERO))) ;
}  



float    TempConversion:: Adc2Tconversion (float  AdcValue){ 
  float R0;     float Ractual; float temperature ;
  switch (fCId.fType) { 
    case  TempConverDef::NTCAD4717: 
    case  TempConverDef::NTC : {
      if( fCV.OpenAdcValue != 0){ 
	// assum fixed voltage plus resistor 
	// fCV.Gain should be the fixed serie resistor
	// fCV.OpenAdcValue should be the value
	// of the ADC if there is no termistor , even if
	// this is higher then the ADC range 
	R0= fCV.Gain /
	  (-1 +(float)fCV.OpenAdcValue /(float)fCV.CalAdcValue);
	Ractual=fCV.Gain/
	  (-1+ ((float)fCV.OpenAdcValue/AdcValue));   
      }
      else { // current source NTC   for the NTC
	// recalculate  resistor value
	R0= fCV.Gain * (float) fCV.CalAdcValue;
	Ractual = fCV.Gain * AdcValue;
      }
      fBaseValue=Ractual;
      // only first order  so Rac = R0 exp { Beta ( 1/T -1/To)
      temperature =  TempConverDef::NtcResistor2temperature
	( Ractual, R0,fCV.CalTemp,fCV.Beta);	   
    }
    break;
    case  TempConverDef::TEMPSPEC : {
      temperature=AdcValue;
      fBaseValue=AdcValue;
    }
    break;
    case TempConverDef::TRHXSENS : {
      fBaseValue=AdcValue;
      temperature=AdcValue+fCV.CalTemp;
    }
    break;
    case  TempConverDef::TRHXNTC  : { 
      if( fCV.OpenAdcValue != 0){ fCV.OpenAdcValue=5;}
      // assum fixed voltage plus resistor 
      // fCV.Gain should be the fixed serie resistor
      // fCV.OpenAdcValue should be the value
      // of the ADC if there is no termistor so 5 V for the trhx
      R0= fCV.Gain * (float) fCV.CalAdcValue/
	  ((float)fCV.OpenAdcValue -(float)fCV.CalAdcValue);
      Ractual=fCV.Gain * AdcValue/
	  ((float)fCV.OpenAdcValue -AdcValue); 
      fBaseValue=Ractual;
      temperature = TempConverDef::NtcResistor2temperature
	( Ractual, R0,fCV.CalTemp,fCV.Beta);  ;       
    }	          
    break;
    default : { // the AD590 case 
      fBaseValue=AdcValue;
      temperature =  (AdcValue-(float)fCV.CalAdcValue) 
	* fCV.Gain + fCV.CalTemp;
    }
    break;		
  }      
  return temperature ;
}



// power conversions 
 
unsigned short   PowerConversion::Power2Dacconversion(float PValue) {  
  int  signeddacvalue  = 
    (int)( (1/fCV.Gain) * ( PValue - fCV.CalPower)) 
    + (int)fCV.CalDacValue;
  unsigned short dacvalue = 
    (signeddacvalue <0) ? 0: (unsigned short) signeddacvalue ;
  return (dacvalue >fCV.MaxDac ) ? fCV.MaxDac : dacvalue ;
}

float PowerConversion::Dac2Powerconversion ( unsigned short DValue) { 
  return (DValue-fCV.CalDacValue) * fCV.Gain + fCV.CalPower ;
}
 
void PowerConversion::PrintPCVvalues(float  TestPowerValue) { 
  std::cout << printIdValues(std::cout);
  printf(" CalPower \t %f\n", fCV.CalPower);  
  printf(" CalDacValue \t %d\n", fCV.CalDacValue);  
  printf(" Gain \t %f\n", fCV.Gain);  
  printf(" TestPowerValue \t %f\n", TestPowerValue);  
  printf(" Result \t %d \n", Power2Dacconversion(TestPowerValue));
}




float HumidityConversion::Adc2HumidityConversion(float HAdcValue) { 
  float humidity;
  switch (fCId.fType)    { 
    case  HumidityConversionDef::TRHXSENS : {
      humidity= HAdcValue ;
    }
    break;
    default: { // for the moment the Humirel to ADC 
      // old value for humirel 
      //humidity=  ( ((HAdcValue / fCV.VRefAdcValue ) -0.2354) / 0.00474);
      // general function
      humidity =((fCV.CalVoltageRef *HAdcValue / fCV.VRefAdcValue) 
		 -fCV.Offset) /fCV.Gain;
      // so for humirel offset = 5*.2354 = 1.177 V  , gain = .0237
    }
    break;
  }
  return humidity;
}
