#include "TDeviceControl.h"
#include "TShare.h"
#include "adcchannelad4717.h"
#include <stdio.h>
#include <signal.h>
#include "daqerrormsg.h"
using namespace std;
static TDeviceControl* fControl =0;

AdcChannelAd4717::AdcChannelAd4717(int boardnr, int  channel):  AdcChannelInterface((long)boardnr,(long)channel)
{
  fInitialized=false;
  // Attach to Main control share memory
  if (fControl== 0)
    fControl =new TDeviceControl("ControlePrincipal",TShare_OPEN);

  fPid = getpid();
  fCd = 0;
   //int boardnr=fDev->calculateBoardNr(adcaddress,devchannel,devicenr,devtype);
 
  if (AttachADCChannel(boardnr,channel)<0)
    { DT_WARN("Cannot get channel"<<channel<<" from Adc "<<boardnr);
      if (fCd!=0)
        fControl->ReleaseChannel(fCd->fPidServer,fPid,fCd->fChannelNumber);
      return;
    }
  
  fAdc=0;
  
// scan all Ad4717 descriptors for the correct board nr. 
  for(int i=0;i < fDev->getNumberOfChannels();i++)
  {  for( int lc=0;lc < 8;lc++)
    { Ad4717Description* ad=fDev->getAd4717Description(i,lc); 
      if (ad->getBoardNr()  == boardnr)
	{ fAdc=ad; 
	lock();
	DT_DEBUG_LTC("set channel active for ch:"<<getChannelId()<<" addr:"<<lc
		     <<" board:"<<boardnr<<" devch:"<<i);
	  fAdc->setAdcChannelActive(true ,getChannelId());
	if( fAdc->getAddress() == -1)
	  {  fAdc->setAddress(lc);
	  }
	else
	  { if( fAdc->getAddress() != lc )
	    { fInitialized=false;break;
	    }
	  }
	unlock();
	  fInitialized=true;break;
	}        
    }
    if(fInitialized) 
      {  DT_DEBUG_LTC("correct board "<< boardnr<<"  found for device" <<i<< "channel"
		      << getChannelId()<<" act"<<fAdc->getAdcChannelActive(getChannelId()) );
	break;
	}
  }
  if( !fInitialized) {DT_WARN("Board " << boardnr<<" not found");}
}

AdcChannelAd4717::~AdcChannelAd4717()
{
  //  printf("Destructeur  init %d  desc %d \n",fInitialized, fCd);
  this->release();
}
void AdcChannelAd4717::release()
{ lock();
	  fAdc->setAdcChannelActive(false,getChannelId());
 unlock(); 
 if (fInitialized && fCd!=0){
   fControl->Lock();
   DeviceControlDescription* d = (DeviceControlDescription*) fControl->Attach();
   int ncx = d->NumberOfChannels;
   for (int i=0;i<ncx;i++)
     { if (d->Channel[i].fType != fCd->fType) continue; 
       if (d->Channel[i].fPidServer != fCd->fPidServer) continue;
       if (d->Channel[i].fPidClient != fCd->fPidClient) continue;
       if (d->Channel[i].fChannelNumber != fCd->fChannelNumber) continue;
       if (d->Channel[i].fBoardId != fCd->fBoardId) continue;
       d->Channel[i].fPidClient =0;
       fControl->Detach(d);
       fControl->Unlock();
       return;      
     }
   DT_WARN(" registration not found not released ");
   fControl->Detach(d);
   fControl->Unlock();  
   fInitialized=false;
   //fControl->ReleaseChannel(fCd->fPidServer,fPid,fCd->fChannelNumber);
 }
}

int AdcChannelAd4717::AttachADCChannel(int serial_nr, int channelnr )
{ DT_DEBUG_LTC("look for channel"<<channelnr<<" from Adc "<<serial_nr);
  fCd = fControl->RequestChannel(fPid,ADCI2C,channelnr,serial_nr);
  DT_DEBUG_LTC("found channel"<<channelnr<<" from Adc "<<serial_nr);
  if (fCd <= 0) return -1;
  kill(fCd->fPidServer,SIGCONT);
  DT_DEBUG_LTC("je signal" <<fCd->fPidServer);
  DT_DEBUG_LTC("Share Memory" <<fCd->fShareName<<" size " <<fCd->fShareSize);
  fMemoryClient = new TShare(fCd->fShareName,TShare_OPEN,fCd->fShareSize);
  fDev= (Ad4717DevDescription*) fMemoryClient->Attach();
  return 0;
}


int AdcChannelAd4717::getAdcData(unsigned short *out_buf, int size_of_buf )
{ // Check Size
  if (size_of_buf< (int)sizeof(unsigned short)) 
    {DT_WARN("invalid size! Needed "<< sizeof(unsigned short)<< "given" <<size_of_buf);	 
    return -2;
    }
  *out_buf =  fAdc->getAdcChannel(getChannelId());
  //device starts to count from 1 
  return 1;
}

unsigned short AdcChannelAd4717::getAdcSample(int isample)
{  
  return  fAdc->getAdcChannel(getChannelId());
}

unsigned short AdcChannelAd4717::getSampleFromBuffer(int i)
{  
  return  fAdc->getAdcChannel(getChannelId());
 
}

int AdcChannelAd4717::getAdcData(float& mean )
{  
  mean= (float) fAdc->getAdcChannel(getChannelId());
 
  return 1;

}


  void   AdcChannelAd4717::info(ostream& sout)
{ // this is not written with effeciency in mind !!!
  // take care no lock inside here 
  sout << "Adc Channel info" ;
  sout << " Channel: "<< getChannelId(); 
  sout << " Actual spill :" << getSpillNumber() ;
  float mean;
  (void)getAdcData( mean);

  sout << " Actual value :" << mean; 
 
    sout << "sample 10 :" << getAdcSample(10);

}


