#include "TDeviceControl.h"
#include "TShare.h"
#include "diochannelad4717.h"
#include <cstdio>
#include <signal.h>
#include "daqerrormsg.h"
static TDeviceControl* fControl =0;

DioChannelAd4717::DioChannelAd4717(char* TDeviceControlName,long boardnr, long  channel,diomode mode)
  :  DioChannelInterface((long)boardnr,(long)channel)
{fMode=DIO_INVALID ;
  fInitialized=false;
  // Attach to Main control share memory
 if ( mode != DIO_OUT )
   {  fMode=DIO_INVALID ;
      DT_ERROR("Cannot get channel "<< channel <<" for " << mode );
     return;
   }

  if (fControl== 0)
    fControl =new TDeviceControl(TDeviceControlName,TShare_OPEN);

  fPid = getpid();
  fCd = 0;
   //int boardnr=fDev->calculateBoardNr(dioaddress,devchannel,devicenr,devtype);
 
  if (AttachDIOChannel((int)boardnr,(int)channel)<0)
    { DT_WARN("Cannot get channel"<<channel<<" from Dio "<<boardnr);
      if (fCd!=0)
        fControl->ReleaseChannel(fCd->fPidServer,fPid,fCd->fChannelNumber);
      return;
    }
  
  fDio=0;
  
// scan all Ad4717 descriptors for the correct board nr. 
  for(int i=0;i < fDev->getNumberOfChannels();i++)
  {  for( int lc=0;lc < 8;lc++)
    { Ad4717Description* ad=fDev->getAd4717Description(i,lc); 
      if (ad->getBoardNr()  == (int)boardnr)
	{ fDio=ad; 
	lock();
	DT_DEBUG_LTC("set channel active for ch:"<<getChannelId()<<" addr:"<<lc
		     <<" board:"<<boardnr<<" devch:"<<i);
	fDio->setPolarityActive(false  ); // becomes only active on request
	if( fDio->getAddress() == -1)
	  {  fDio->setAddress(lc);
	  }
	else
	  { if( fDio->getAddress() != lc )
	    { fInitialized=false;break;
	    }
	  }
	unlock();
	  fInitialized=true;break;
	}        
    }
    if(fInitialized) 
      {  DT_DEBUG_LTC("correct board "<< boardnr<<"  found for device" <<i<< "channel"
		      << getChannelId()<<" act"<<fDio->getPolarityActive() );
       fMode=mode; 
	break;
	}
  }
  if( !fInitialized) {DT_WARN("Board " << boardnr<<" not found");}
}

DioChannelAd4717::~DioChannelAd4717()
{
  //  printf("Destructeur  init %d  desc %d \n",fInitialized, fCd);
  this->release();
}
void DioChannelAd4717::release()
{ lock();
	  fDio->setPolarityActive(false);
 unlock(); 
 if (fInitialized && fCd!=0){
   DT_DEBUG_LTC("release server "<<fCd->fPidServer<< 
		   " pid "<< fPid << "channelnr "<< fCd->fChannelNumber <<
                   " client " <<fCd->fPidClient <<
                   " board "  <<fCd->fBoardId);
   //fControl->ReleaseChannel(fCd->fPidServer,fPid,fCd->fChannelNumber);
   fControl->Lock();
   DeviceControlDescription* d = (DeviceControlDescription*) fControl->Attach();
   int ncx = d->NumberOfChannels;
   for (int i=0;i<ncx;i++)
     { if (d->Channel[i].fType != fCd->fType) continue; 
       if (d->Channel[i].fPidServer != fCd->fPidServer) continue;
       if (d->Channel[i].fPidClient != fCd->fPidClient) continue;
       if (d->Channel[i].fChannelNumber != fCd->fChannelNumber) continue;
       if (d->Channel[i].fBoardId != fCd->fBoardId) continue;
       d->Channel[i].fPidClient =0;
       fControl->Detach(d);
       fControl->Unlock();
       return;      
     }
   DT_WARN(" registration not found not released ");
   fControl->Detach(d);
   fControl->Unlock();  
   fInitialized=false;
 }

}

int DioChannelAd4717::AttachDIOChannel(int serial_nr, int channelnr )
{ DT_DEBUG_LTC("look for channel"<<channelnr<<" from Dio "<<serial_nr);
  fCd = fControl->RequestChannel(fPid,DIOI2C,channelnr,serial_nr);
  DT_DEBUG_LTC("found channel"<<channelnr<<" from Dio "<<serial_nr);
  if (fCd <= 0) return -1;
  kill(fCd->fPidServer,SIGCONT);
  //DT_DEBUG_LTC("je signal" <<fCd->fPidServer);
  //DT_DEBUG_LTC("Share Memory" <<fCd->fShareName<<" size " <<fCd->fShareSize);
  fMemoryClient = new TShare(fCd->fShareName,TShare_OPEN,fCd->fShareSize);
  fDev= (Ad4717DevDescription*) fMemoryClient->Attach();
  return 0;
}


int  DioChannelAd4717::getBit()
{ 
int result=0;
  //  lock();no  read
//        result= fDio->setPolarity(1);
//    unlock();
 
  return result;
}
int  DioChannelAd4717::getBit(int& status )
{ 
int result=0;
 //   lock();
//        result= fDio->setPolarity(1);
//    unlock();
 status =-1;
  return result;
}


int   DioChannelAd4717::setBit(int i ){  
  lock();
  DT_DEBUG_LTC("Set to : "<<i);
    fDio->setPolarity(i);
    fDio->setPolarityActive(true);
    unlock();
  
    return wait();
   
}




int DioChannelAd4717::wait()
{int const loops=1000;
 int r=0;
  do {
    r++;
    usleep(30000);
    //DT_DEBUG_LTC(" pol acitve "<<fDio->getPolarityActive());
  } while ( fDio->getPolarityActive()  && ( r < loops));
  r = (r >= loops) ?r=-1 : r=0;
  return r;
}

  void   DioChannelAd4717::info(std::ostream& sout)
{ // this is not written with effeciency in mind !!!
  // take care no lock inside here 
  sout << "Dio Channel Ad4717  info" ;
  sout << " Channel: "<< getChannelId(); 
   sout << " Channel: "<< getBoardId(); 
   sout << "can not get bit status"<<std::endl ;

}


