#include "TDeviceControl.h"
#include "TShare.h"

#include "daqerrormsg.h"
#include "diochannelcio.h"
#include <stdio.h>
#include <signal.h>
static TDeviceControl* fControl =0;

DioChannelCio::DioChannelCio(char* TDeviceControlName, long adc_ser_nr, long channel_arg, diomode mode  ) 
                               : DioChannelInterface(adc_ser_nr,channel_arg) 
{
  fInitialized=false;
  //check channel possiblities
  if (! (   ( mode == DIO_IN  &&  channel_arg < 8  )  ||
	    ( mode== DIO_OUT  &&  channel_arg >7)
        )
    )
    {  fMode=DIO_INVALID ;
      DT_ERROR("Cannot get channel "<< channel_arg <<" for " << mode );
     return;
   }
  fMode=mode;
  // Attach to Main control share memory
  if (fControl== 0)
    fControl =new TDeviceControl(TDeviceControlName,TShare_OPEN);

  fPid = getpid();
  fCd = 0;
  if (AttachDIOChannel(adc_ser_nr,channel_arg)<0)
    {  DT_ERROR(" Cannot get channel "<< channel_arg <<" from DIO "<< adc_ser_nr);
      if (fCd!=0)
        fControl->ReleaseChannel(fCd->fPidServer,fPid,fCd->fChannelNumber);
      return;
    }
  fInitialized=true;
}

DioChannelCio::~DioChannelCio()
{
  //  printf("Destructeur  init %d  desc %d \n",fInitialized, fCd);
  this->release();
}
void DioChannelCio::release()
{
  if (fInitialized && fCd!=0)
 {    fControl->ReleaseChannel(fCd->fPidServer,fPid,fCd->fChannelNumber);
 fInitialized= false;
 }

}

int DioChannelCio::AttachDIOChannel(int serial_nr, int channelnr )
{

  fCd = fControl->RequestChannel(fPid,CB64DIO,channelnr,serial_nr);
  if (fCd <= 0)
    { DT_DEBUG_LTC("no channel "<<channelnr<<"for board "<< serial_nr);
                 return -1;
    }
  kill(fCd->fPidServer,SIGCONT);
  DT_DEBUG_LTC("je signal n"<<fCd->fPidServer);
  DT_DEBUG_LTC("Share Memory"<<fCd->fShareName <<" size "<<fCd->fShareSize);
  fMemoryClient = new TShare(fCd->fShareName,TShare_OPEN,fCd->fShareSize);
  fDio= (DioChannel*) fMemoryClient->Attach();
  
  return 0;
}

int DioChannelCio::getBit()
{ int result;
  lock();
 
  
  fDio->fStatus= READ_DIO;
  fDio->fChannel=getChannelId();
  result= fDio->fApplyValue;
  unlock();
 
  return result;
}


int DioChannelCio::getBit(int& status)
{ int result=getBit();
   status=  wait();
 
   return result;
}

int  DioChannelCio::setBit(int i )
{  lock();
   fDio->fStatus = WRITE_DIO;
   fDio->fChannel=getChannelId();
    fDio->fRequestValue =i;
    unlock();
  
    return wait();
   
}




int DioChannelCio::wait()
{  lock();
    while  (fDio->fStatus !=DONE_DIO   && fDio->fStatus !=DONE_DIO_ERROR  ) 
      { unlock();
         usleep(2000000); 
	 lock(); 
       }
    unlock();
    if ( fDio->fStatus ==DONE_DIO )
      { fLastStatus= 0;}     else        { fLastStatus= -1; } 
  return fLastStatus;
}
void    DioChannelCio::info(std::ostream& sout)
{ // this is not written with effeciency in mind !!!
  sout << "Dio Channel info" ;
  sout << " Channel: "<< getChannelId(); 
  sout << " value  "  << getBit();
  sout << " last status (0 = ok) " <<   fLastStatus << std::endl;


}


