#include "TDeviceControl.h"
#include "TShare.h"

#include "daqerrormsg.h"
#include "diochannelpcf8574.h"
#include <stdio.h>
#include <signal.h>
#include <cstdlib>
#include "i2cid.h"
#include "ad4717devdescription.h"
static TDeviceControl* fControl =0;
using namespace I2cId;

DioChannelPcf8574::DioChannelPcf8574(char* TDeviceControlName, long adc_ser_nr, long channel_arg, diomode mode  ) 
                               : DioChannelInterface(adc_ser_nr,channel_arg) 
{
  fInitialized=false;
//  #ifndef AD4717DEVNEW
//    DAQEXIT("This Class can not be used in this version ");
//    return ;
//  #else 
  //check channel possiblities
  // all modes are acceptable 
  fMode=mode;
  // Attach to Main control share memory
  if (fControl== 0)
    fControl =new TDeviceControl(TDeviceControlName,TShare_OPEN);

  fPid = getpid();
  fDev = 0;fDio=0;fCd=0;
  if (AttachDIOChannel(adc_ser_nr,channel_arg)<0)
    {  DT_ERROR(" Cannot get channel "<< channel_arg <<" from  "<< adc_ser_nr);
      if (fCd!=0)
        fControl->ReleaseChannel(fCd->fPidServer,fPid,fCd->fChannelNumber);
      return;
    }
  fInitialized=true;
//#endif 
}

DioChannelPcf8574::~DioChannelPcf8574()
{
  //  printf("Destructeur  init %d  desc %d \n",fInitialized, fCd);
  this->release();
}
void DioChannelPcf8574::release()
{
  if (fInitialized && fCd!=0)
    { DT_DEBUG_LTC("release server "<<fCd->fPidServer<< 
		   " pid "<< fPid << "channelnr "<< fCd->fChannelNumber <<
                   " client " <<fCd->fPidClient <<
                   " board "  <<fCd->fBoardId);
    // special release channel 
    // simple force 
  fControl->Lock();
  DeviceControlDescription* d = (DeviceControlDescription*) fControl->Attach();
  int ncx = d->NumberOfChannels;
  for (int i=0;i<ncx;i++)
    { if (d->Channel[i].fType != fCd->fType) continue; 
      if (d->Channel[i].fPidServer != fCd->fPidServer) continue;
      if (d->Channel[i].fPidClient != fCd->fPidClient) continue;
      if (d->Channel[i].fChannelNumber != fCd->fChannelNumber) continue;
      if (d->Channel[i].fBoardId != fCd->fBoardId) continue;
      d->Channel[i].fPidClient =0;
      fControl->Detach(d);
      fControl->Unlock();
      return;

    }
  fControl->Detach(d);
  fControl->Unlock();  

  fInitialized= false;
 }

}

int DioChannelPcf8574::AttachDIOChannel(int serial_nr, int channelnr )
{
  int adcnr; int devI2cchannel;int subdev;int  devicenr; i2cdevicetypes type ;
   decodeI2cBoardNr(serial_nr, adcnr, devI2cchannel,subdev,devicenr, type  );
  int fulladcnr= adcnr& 0xFF;
  int boardnr= calculateI2cBoardNr(fulladcnr,devI2cchannel,subdev,devicenr, type  ); 
  DT_DEBUG_LTC("look for channel"<<channelnr<<" from Dio "<<boardnr);
  fCd = fControl->RequestChannel(fPid,DIOI2C,channelnr,boardnr);
  DT_DEBUG_LTC("found channel"<<channelnr<<" from Dio "<<boardnr);
  if (fCd <= 0) return -1;
  kill(fCd->fPidServer,SIGCONT);
  DT_DEBUG_LTC("je signal" <<fCd->fPidServer);
  DT_DEBUG_LTC("Share Memory" <<fCd->fShareName<<" size " <<fCd->fShareSize);
  fMemoryClient = new TShare(fCd->fShareName,TShare_OPEN,fCd->fShareSize);
  fDev= (Ad4717DevDescription*) fMemoryClient->Attach();
//#ifndef  AD4717DEVNEW
//  fDio=0;
//#else
  fDio=fDev->getDioChannelDescription(0,(adcnr&0xF));
//#endif 
  return 0;
}
int DioChannelPcf8574::getBit(int& status)
{ int result;
  lock();
   fDio->fStatus= READ_DIO;
  fDio->fChannel=getChannelId();
  unlock();
   status= wait(); 
  result= fDio->fApplyValue;
  return result;
}


int DioChannelPcf8574::getBit()
{  int status ;
 return  getBit(status);

}

int  DioChannelPcf8574::setBit(int i )
{  lock();
   fDio->fStatus = WRITE_DIO;
   fDio->fChannel=getChannelId();
   fDio->fRequestValue =i;
   unlock();
   return wait(); 
}




int DioChannelPcf8574::wait()
{ int maxwait=20; 
  lock();
    while  (fDio->fStatus !=DONE_DIO   && fDio->fStatus !=DONE_DIO_ERROR  && maxwait--) 
      { 
	unlock();
	usleep(400000); 
	lock(); 
      }
    unlock();
    if ( fDio->fStatus ==DONE_DIO )
      { fLastStatus= 0;}     else        { fLastStatus= -1; } 
    if(maxwait ) {  fLastStatus= -2; }
  return fLastStatus;
}
void    DioChannelPcf8574::info(std::ostream& sout)
{ // this is not written with effeciency in mind !!!
  sout << "Dio Channel info" ;
  sout << " Channel: "<< getChannelId(); 
  sout << " value  "  << getBit();
  sout << " last status (0 = ok) " <<   fLastStatus << std::endl;


}


