#include "humidityctrlparser.h"
#include "humidityparser.h"
#include <dom/DOM_Node.hpp>
#include <dom/DOM_NamedNodeMap.hpp>
#include "daqerrormsg.h"
#include "XercesExt.hpp"

using namespace XercesExt;
void HumidityCtrlParser::clear(){
  humidityctrls.clearpointermap();
}

pmap <int,CalExchange<HumidityCtrlDef::HumidityCtrlValues>*> 
  HumidityCtrlParser::humidityctrls;

int HumidityCtrlParser::parseNode(DOM_Node &ei){
  simulation_=false;
  DOM_NamedNodeMap attributes = ei.getAttributes();
  int device=-1;
  DOM_Node e = attributes.getNamedItem("device");
  device=atoi(  e.getNodeValue().transcode());
  CalExchange <  HumidityCtrlDef::HumidityCtrlValues >* a =
    new CalExchange< HumidityCtrlDef::HumidityCtrlValues >(device);
  HumidityCtrlDef::HumidityCtrlValues* cv =a->getCalValues();
  // in case no specific ctrl is defined 
  a->setType( HumidityCtrlDef::DUMMY);
  getItem( a->getName(),"name",attributes);
  e = attributes.getNamedItem("minimumhumidity");
  cv->fMinHumidity=atof(  e.getNodeValue().transcode());
  e = attributes.getNamedItem("maximumhumidity");
  cv->fMaxHumidity=atof(  e.getNodeValue().transcode());
  e = attributes.getNamedItem("cooldownhumidity");
  cv->fCoolDownHumidity=atof(  e.getNodeValue().transcode());
  getItem(simulation_,"simulation",attributes);
  DT_DEBUG_LP("items parsed ");
  DOM_Node n =ei.getFirstChild();
  HumidityParser hp;
  while (!n.isNull()) { 
    if (strcmp(n.getNodeName().transcode(),"HumidityCtrlStd")==0){
      parseHumidityCtrlStd(n,a );
    }
    if (strcmp(n.getNodeName().transcode(),"HumidityProbe")==0){
       cv->fHumidityId=hp.parseNode(n);
       a->setType( HumidityCtrlDef::PROBEONLY);
    }
    n = n.getNextSibling(); 
  }
  if(simulation_ || fSimulation){
    a->setType( HumidityCtrlDef::DUMMY);
    hp.setDummy(cv->fHumidityId);
  }
  humidityctrls[device]=a;
  DT_DEBUG_LP("humidityctrl parser done");
  return device;
}

void HumidityCtrlParser::parseHumidityCtrlStd(DOM_Node &ei,
					  CalExchange <  HumidityCtrlDef::HumidityCtrlValues >* a ){
  bool simulation=false;
  DOM_NamedNodeMap attributes = ei.getAttributes();
  HumidityCtrlDef::HumidityCtrlValues* cv =a->getCalValues();
  cv->fGasFlowId=-1;
  getItem(  simulation,"simulation",attributes);
  HumidityParser hp;
  DT_DEBUG_LP("humidity probe parser created ");
  DOM_Node n =ei.getFirstChild();
  while (!n.isNull()) {
    //if (strcmp(n.getNodeName().transcode(),"GasFlowCtrl")==0){
    //  cv->fGasFlowId=GasFlowParser(n);
    //}
    if (strcmp(n.getNodeName().transcode(),"HumidityProbe")==0){
      cv->fHumidityId=hp.parseNode(n);
      DT_DEBUG_LP("humidity parsed node");
    }
    n = n.getNextSibling(); 
  }

  if(simulation_ || fSimulation || simulation){
    a->setType( HumidityCtrlDef::DUMMY);
    // set the lower devices to dummy opperation
  //    CalExchange <  GasFlowDef::GasFlowValues >* 
//        gf=gasflows[cv->fGasFlowId];
//      if(gf > 0){ gf->setType(GasFlowDef::DUMMY);}
//      CalExchange <HumidityConversionDef::HumidityCalValues>*
//        hum=humiditys[cv->fHumidityId];
//      if(hum > 0) {hum->setType(HumidityConversionDef::DUMMY);}
    DT_DEBUG_LP("set dummy");
	hp.setDummy(cv->fHumidityId);
  }
  else{
    DT_DEBUG_LP("set type ");
    a->setType(HumidityCtrlDef::STANDARD);
  }  
 DT_DEBUG_LP("humidityctrl std done ");
}

CalExchange<HumidityCtrlDef::HumidityCtrlValues>* 
HumidityCtrlParser::getInfo(int device ){
  return humidityctrls.getP(device);
}
