#include <util/PlatformUtils.hpp>
#include <util/XMLString.hpp>
#include <util/XMLUniDefs.hpp>
#include <framework/XMLFormatter.hpp>
#include <util/TranscodingException.hpp>
#include <dom/DOM_DOMException.hpp>
#include <parsers/DOMParser.hpp>
#include <dom/DOM.hpp>
#include "DOMTreeErrorReporter.hpp"
#include <cstring>
#include <cstdlib>

#include "ltparser.h"

#include "daqerrormsg.h"
#include "tscchanneldef.h"  
#include "tscchannelinterface.h"
#include "ledcarddef.h"
#include "fedprocessctrlinterface.h"
#include "fecctrldevinterface.h"
#include "muxdevinterface.h"
#include "interlock.h"
#include "diochannelinterface.h"
#include "slowctrlconfig.h"
#include "humidityctrlparser.h"
#include "basicchparser.h"
#include "diolineparser.h"
#include "humidityparser.h"
#include "benchpositionparser.h"
#include "powerparser.h"
#include "structconparser.h"

using  namespace std;
using  namespace BasicChParser;
//static char*                    gXmlFile               = 0;
static bool                     gDoNamespaces          = false;
static bool                     gDoCreate              = false;
static XMLCh*                   gEncodingName          = 0;
//static XMLFormatter::UnRepFlags gUnRepFlags            = XMLFormatter::UnRep_CharRef;
static DOMParser::ValSchemes    gValScheme             = DOMParser::Val_Auto;
static XMLFormatter*            gFormatter             = 0;

//  ostream& operator<< (ostream& target, const DOMString& s)
//  {
//    char *p = s.transcode();
//    target << p;
//    delete [] p;
//    return target;
//  }

LtParser:: LtParser(){
  ncurrents=0; //nbenchpos=0;
  nhvs=0;
 fSimulation=false;
 fParseFileName="lt.xml";
   clear();
}





 char*   LtParser::getStandardConfigFile(char* s, const char * filename )
    {   strcpy(s,  getStandardConfigPath().c_str() );
        strcat(s,filename );
	return s; 
    } 

string LtParser::getStandardConfigPath( ) {
  string s(getenv("DAQHOME") );
  s+="/Config/";  
  return s;
}

int  LtParser::parseFile() {
  // or fParseFilename is already calle and is absolute path 
  // of fParseFilename is "lt.xml" and is now changed to absolute path 
  string filename(setParseFileName(fParseFileName));
  return parseFile(filename.c_str());
}



string  LtParser::setParseFileName( string filename ){
  if( filename.at(0) == '/') {
    // absolute path assumed 
    fParseFileName=filename;
  }
  else {
    // default path assumed 
    fParseFileName=getStandardConfigPath();
    fParseFileName+=filename;
  }
  return fParseFileName;
}

string  LtParser::getParseFileName( )
{       return fParseFileName;
}

void LtParser::clear()
{
  DT_DEBUG_LP( " clearmaps ") ;
  //Dioline.clearpointermap();
  DioLineParser dp; dp.clear();
  Interlock.clearpointermap(); 
  coldbox.clearpointermap( ); 
  tempctrls.clearpointermap();
  PowerParser pp; pp.clear();
//  power.clearpointermap();
  tsc.clearpointermap();
  slowctrls.clearpointermap();
  fedctrl.clearpointermap();
  fecctrl.clearpointermap();
  ledcard.clearpointermap();
  fbcard.clearpointermap();
  //gasflows.clearpointermap();
  HumidityCtrlParser hctrlp; hctrlp.clear();
  HumidityParser hp; hp.clear();
  //humiditys.clearpointermap();
  muxdev.clearpointermap();
  temperatures.clearpointermap();
     
 for (int i=0;i<ncurrents;i++)
    if (current[i]!= 0) delete current[i];
    ncurrents=0;
  memset(current,0,(MAXBENCHPOS*2+10)*
	 sizeof( CalExchange<CurrentConvDef::CurrentCalValues>*));

//   for (int i=0;i<nbenchpos;i++)
//      if (benchpos[i]!= 0) delete benchpos[i];
//      nbenchpos=0;
//    memset(benchpos,0,(MAXBENCHPOS)*
//  	 sizeof( BenchPosConnection*));
 for (int i=0;i<nhvs;i++)
    if (hv[i]!= 0) delete hv[i];
    nhvs=0;
  memset(hv,0,(MAXBENCHPOS+10)*
	 sizeof( CalExchange<HvConversionDef::HvCalValues>*  ));
  fileparsed=false;

}
int  LtParser::parseFile(const char* name)
{    
  DT_DEBUG_LP("Parse " << name );
  if (fileparsed) { DT_DEBUG_LP( "file already parsed ");
  return -1;}

  else fileparsed=true;
  try
    {
      XMLPlatformUtils::Initialize();
    }
  
  catch(const XMLException& toCatch)
    {
      cerr << "Error during Xerces-c Initialization.\n"
           << "  Exception message:"
           << DOMString(toCatch.getMessage()).transcode() << endl;
      return -2 ;
    }
 
  //
  //  Create our parser, then attach an error handler to the parser.
  //  The parser will call back to methods of the ErrorHandler if it
  //  discovers errors during the course of parsing the XML document.
  //
  DOMParser *parser = new DOMParser;
  parser->setValidationScheme(gValScheme);
  parser->setDoNamespaces(gDoNamespaces);
  parser->setCreateEntityReferenceNodes(gDoCreate);
  parser->setToCreateXMLDeclTypeNode(true);
  DOMTreeErrorReporter *errReporter = new DOMTreeErrorReporter();
  parser->setErrorHandler(errReporter);
  //
  //  Parse the XML file, catching any XML exceptions that might propogate
  //  out of it.
  //
  bool errorsOccured = false;
  try
    {
      parser->parse(name);

    }

  catch (const XMLException& e)
    {
      cerr << "An error occured during parsing\n   Message: "
           << DOMString(e.getMessage()).transcode() << endl;
      errorsOccured = true;
    }

 
  catch (const DOM_DOMException& e)
    {
      cerr << "An error occured during parsing\n   Message: "
           << DOMString(e.msg).transcode() << endl;
      errorsOccured = true;
    }

  catch (...)
    {
      cerr << "An error occured during parsing\n " << endl;
      errorsOccured = true;
    }
  errorsOccured|=errReporter->getSawErrors();
  // If the parse was successful, output the document data from the DOM tree
  if (!errorsOccured)
    {
      DOM_Node doc = parser->getDocument();

        

      DOM_Node n1 = doc.getFirstChild();
      while (!n1.isNull())
        {
          DT_DEBUG_LP( n1.getNodeName().transcode());
          DT_DEBUG_LP( n1.getNodeType() );
          if (n1.getNodeType() == DOM_Node::ELEMENT_NODE) break;
          n1 = n1.getNextSibling();
        }
      DOM_Node n = n1.getFirstChild();
      if (n!=0 && n.isNull())
        DT_NOTE( " Bad element "<< n.getNodeName().transcode());
      if (n == 0)DT_NOTE( "pb");
      while( !n.isNull() ) 
        {
	    DT_DEBUG_LP( n.getNodeName().transcode());
	  if (strcmp("Simulation",n.getNodeName().transcode())==0)  
            SimulationParser(n);
          // the node was really an element.
	  if (strcmp("BenchPositionConnection",n.getNodeName().transcode())==0)  
            BenchPositionParser p(n);
        if (strcmp("SlowCtrlConfig",n.getNodeName().transcode())==0)  
            SlowCtrlConfigParser(n);
          if (strcmp("TemperatureProbe",n.getNodeName().transcode())==0)  
            TemperatureParser(n);
	  if (strcmp("HumidityProbe",n.getNodeName().transcode())==0) {
            HumidityParser hp;
	    hp.parseNode(n);
	  }
             if (strcmp("InterLock",n.getNodeName().transcode())==0)  
            InterLockParser(n);
          if (strcmp("ColdBox",n.getNodeName().transcode())==0)  
            ColdBoxParser(n,BoxCharacteristicsDef::COLDBOX );
          if ( strcmp("CurrentChannel",n.getNodeName().transcode())==0) 
	    { CurrentParser(n);}
          if ( strcmp("TscCard",n.getNodeName().transcode())==0) 
	    { TscCardParser(n);}
	   if ( strcmp("DaqProcessCtrl",n.getNodeName().transcode())==0) 
	    { DaqProcessCtrlParser(n);}
	   if ( strcmp("StructureConnections",n.getNodeName().transcode())==0) 
	    { StructConParser sp; sp.parseNode(n);}
          n = n.getNextSibling();
          if (n!=0 && n.isNull())
            DT_DEBUG_LP( " Bad element "<< n.getNodeName().transcode());
        }
      DT_NOTE( "lt parse file processed clean up parser " );

    }

    
  delete gFormatter;


  delete gEncodingName;
  //
  //  Clean up the error handler. The parser does not adopt handlers
  //  since they could be many objects or one object installed for multiple
  //  handlers.
  //


  //
  //  Delete the parser itself.  Must be done prior to calling Terminate, below.
  //
  delete errReporter;
  DT_DEBUG_LP( "delete parser");
  delete parser;

  // And call the termination method
 
// done by DomInit()
//   XMLPlatformUtils::Terminate();
  // DomMemDebug().print();
    
  //
  //  The DOM document and its contents are reference counted, and need
  //  no explicit deletion.
  //
 
  return (int) errorsOccured;
}

// general  parser functions

int  LtParser::SimulationParser (DOM_Node &ei)
{  fSimulation=false ;
  DOM_NamedNodeMap attributes = ei.getAttributes();
  //int attrCount = attributes.getLength();  // why ?

  DOM_Node  e = attributes.getNamedItem("simulation");
 if(! e.isNull() )   { if ('y' == (e.getNodeValue().transcode())[0]) { fSimulation=true ;}  }
 
 
 
  return 0 ;
}

int LtParser::DaqProcessCtrlParser(DOM_Node &ei) 
{ //bool simulation=false ;
  DOM_NamedNodeMap attributes = ei.getAttributes();
  DOM_Node n =ei.getFirstChild();
  while (!n.isNull()) {  
    if (strcmp(n.getNodeName().transcode(),"TscCard")==0)
      { TscCardParser(n);}
    if (strcmp(n.getNodeName().transcode(),"FedProcessCtrl")==0)
      { FedProcessCtrlParser(n);}
     if (strcmp(n.getNodeName().transcode(),"FedCtrl")==0)
      { FedCtrlParser(n);}
    if (strcmp(n.getNodeName().transcode(),"FecCtrl")==0)
      { FecCtrlParser(n);}
    if (strcmp(n.getNodeName().transcode(),"MuxCtrl")==0)
      { MuxCtrlParser(n);}
    if (strcmp(n.getNodeName().transcode(),"LedCtrl")==0)
      { LedCtrlParser(n);}
     if (strcmp(n.getNodeName().transcode(),"ForwardBiasCtrl")==0)
      { LedCtrlParser(n);}
    n = n.getNextSibling();    
  }
  return 0;
}

int LtParser::LedCtrlParser(DOM_Node &ei){
  DOM_NamedNodeMap attributes = ei.getAttributes();
  DOM_Node n =ei.getFirstChild();
  while (!n.isNull()) {
   if (strcmp(n.getNodeName().transcode(),"LedCardKalrs")==0){ 
     LedCardKarlsParser(n);
   }
   if (strcmp(n.getNodeName().transcode(),"LedCardIo")==0){ 
     LedCardIoParser(n);
   }   
   if (strcmp(n.getNodeName().transcode(),"ForwardBiasCardIo")==0){ 
     FBCardIoParser(n);
   }
   n = n.getNextSibling();    
  }  
  
 return 0;
}

int LtParser::LedCardIoParser(DOM_Node &ei) { 
  bool simulation=false ;
  int device; 
  DOM_NamedNodeMap attributes = ei.getAttributes();
  DOM_Node e = attributes.getNamedItem("device");
  device=atoi(e.getNodeValue().transcode()); 
  sscanf(e.getNodeValue().transcode(),"%d",&device  );
  CalExchange<int>*
    a= new CalExchange <int>(device);
  e = attributes.getNamedItem("simulation");
  if(! e.isNull() )   { 
    if ('y' == (e.getNodeValue().transcode())[0]) { simulation=true ;}  }
  e = attributes.getNamedItem("name");
  if(! e.isNull() )   { sscanf(e.getNodeValue().transcode(),"%s",a->getName());  }
  int *ionr = a->getCalValues();
 
  DOM_Node n =ei.getFirstChild();
  while (!n.isNull()) {  

    if (strcmp(n.getNodeName().transcode(),"DioLine")==0)
      { DioLineParser dl;
	*ionr= dl.parseNode(n);
	dl.getInfo(  *ionr)->setName("LedPulseCtrl") ;
	a->setType(LedCardDef::IOCARD);
      }    
    n = n.getNextSibling();    
  }
  if(simulation || fSimulation ) {
    a->setSimulationState(true);
    a->setType(LedCardDef::DUMMY);
  }
  ledcard[device]=a;
  return device;
}

int LtParser::FBCardIoParser(DOM_Node &ei) { 
  bool simulation=false ;
  int device; 
  DOM_NamedNodeMap attributes = ei.getAttributes();
  DOM_Node e = attributes.getNamedItem("device");
  device=atoi(e.getNodeValue().transcode()); 
  sscanf(e.getNodeValue().transcode(),"%d",&device  );
  CalExchange<int>*
    a= new CalExchange <int>(device);
  e = attributes.getNamedItem("simulation");
  if(! e.isNull() )   { 
    if ('y' == (e.getNodeValue().transcode())[0]) { simulation=true ;}  }
  e = attributes.getNamedItem("name");
  if(! e.isNull() )   { sscanf(e.getNodeValue().transcode(),"%s",a->getName());  }
  int *ionr = a->getCalValues();
 
  DOM_Node n =ei.getFirstChild();
  while (!n.isNull()) {  

    if (strcmp(n.getNodeName().transcode(),"DioLine")==0){
      DioLineParser dl;
      *ionr= dl.parseNode(n);
      dl.getInfo(  *ionr)->setName("FBPulseCtrl") ;
//  *ionr= DioLineParser(n);
//        Dioline[  *ionr]->setName("FBPulseCtrl") ;
      a->setType(LedCardDef::IOCARD);
      }    
    n = n.getNextSibling();    
  }
  if(simulation || fSimulation ) {
    a->setSimulationState(true);
    a->setType(LedCardDef::DUMMY);
  }
  fbcard[device]=a;
  return device;
}

int LtParser::LedCardKarlsParser(DOM_Node &ei) { 
  bool simulation=false ;
  int device; 
  DOM_NamedNodeMap attributes = ei.getAttributes();
  DOM_Node e = attributes.getNamedItem("device");
  device=atoi(e.getNodeValue().transcode()); 
  sscanf(e.getNodeValue().transcode(),"%d",&device  );
  CalExchange<int>*
    a= new CalExchange <int>(device);
  e = attributes.getNamedItem("simulation");
  if(! e.isNull() )   { 
    if ('y' == (e.getNodeValue().transcode())[0]) { simulation=true ;}  }
  e = attributes.getNamedItem("name");
 if(! e.isNull() )   { sscanf(e.getNodeValue().transcode(),"%s",a->getName());  }
 e = attributes.getNamedItem("nrchannels");
 int *channels = a->getCalValues();
 if(! e.isNull() )   { sscanf(e.getNodeValue().transcode(),"%d",channels);  }


  DOM_Node n =ei.getFirstChild();
  while (!n.isNull()) {  
    if (strcmp(n.getNodeName().transcode(),"I2cChannel")==0){
      I2cChannelParser(n, a);
      a->setType(LedCardDef::KARLS);
     } 
    n = n.getNextSibling();    
  }
  if(simulation || fSimulation ) {
    a->setSimulationState(true);
    a->setType(LedCardDef::DUMMY);
  }
  ledcard[device]=a;
  return device;
}



int LtParser::MuxCtrlParser(DOM_Node &ei){
  DOM_NamedNodeMap attributes = ei.getAttributes();
  DOM_Node n =ei.getFirstChild();
  while (!n.isNull()) {
   if (strcmp(n.getNodeName().transcode(),"Multiplexer")==0){ 
     MultiPlexerParser(n);
   }
   n = n.getNextSibling();    
  }  
  
 return 0;
}


int LtParser::MuxCardParser(DOM_Node &ei,
			    CalExchange<MuxExchangeDef::MuxValues>*  CE ){
  int cardnr;
  DOM_NamedNodeMap attributes = ei.getAttributes();
  DOM_Node e = attributes.getNamedItem("cardnr");
  cardnr=atoi(e.getNodeValue().transcode()); 
  MuxExchangeDef::MuxValues* MCV= CE->getCalValues();
  MuxExchangeDef::FedMuxInfo fmi;
  e = attributes.getNamedItem("fedserialnr");
  fmi.fedsrnr=atoi(e.getNodeValue().transcode()); 
  e = attributes.getNamedItem("fedchannel");
  fmi.fedchnr=atoi(e.getNodeValue().transcode()); 
  // for the moment hard coded 
  for (int lc =0; lc < 10;lc++){
    fmi.grp_ch[lc]=lc;
  }
  (*MCV)[cardnr]=fmi;
  return cardnr;
}


int LtParser::MultiPlexerParser(DOM_Node &ei) { 
  bool simulation=false ;
  int device; 
  DOM_NamedNodeMap attributes = ei.getAttributes();
  DOM_Node e = attributes.getNamedItem("device");
  device=atoi(e.getNodeValue().transcode()); 
  sscanf(e.getNodeValue().transcode(),"%d",&device  );
  CalExchange<MuxExchangeDef::MuxValues>*
    a= new CalExchange <MuxExchangeDef::MuxValues>(device);
  //e = attributes.getNamedItem("dxxxxr");
  e = attributes.getNamedItem("simulation");
  if(! e.isNull() )   { 
    if ('y' == (e.getNodeValue().transcode())[0]) { simulation=true ;}  }
  e = attributes.getNamedItem("name");
 if(! e.isNull() )   { sscanf(e.getNodeValue().transcode(),"%s",a->getName());  }

  DOM_Node n =ei.getFirstChild();
  while (!n.isNull()) {  
    if (strcmp(n.getNodeName().transcode(),"MuxCard")==0)
      { MuxCardParser(n,a);}
    if (strcmp(n.getNodeName().transcode(),"I2cChannel")==0){
      I2cChannelParser(n, a);
      a->setType(MuxExchangeDef::KARLS);
     } 
    n = n.getNextSibling();    
  }
  if(simulation || fSimulation ) {
    a->setSimulationState(true);
    a->setType(MuxExchangeDef::DUMMY);
  }
  muxdev[device]=a;
  return device;
}

int  LtParser::TscCardParser(DOM_Node &ei)
{ bool simulation=false ;
  DOM_NamedNodeMap attributes = ei.getAttributes();
  //int attrCount = attributes.getLength();  // why ?

  int device=-1;
  DOM_Node e = attributes.getNamedItem("device");
  sscanf(e.getNodeValue().transcode(),"%d",&device  );

CalExchange <int>*
    a= new  CalExchange <int>(device);

 e = attributes.getNamedItem("name");
 if(! e.isNull() )   { sscanf(e.getNodeValue().transcode(),"%s",a->getName());  }

 e = attributes.getNamedItem("simulation");
 if(! e.isNull() )   { if ('y' == (e.getNodeValue().transcode())[0]) { simulation=true ;}  }
  e = attributes.getNamedItem("boardnr");
 int boardnr=1; //default
 if(! e.isNull() ) { sscanf(e.getNodeValue().transcode(),"%d",&boardnr); }
  e = attributes.getNamedItem("channel");
  int channel=1; //default 
 if(! e.isNull() ){ sscanf(e.getNodeValue().transcode(),"%d",&channel);}

 e = attributes.getNamedItem("shmname");
 if(! e.isNull() ) {sscanf(e.getNodeValue().transcode(),"%s",a->getTDeviceControlName());} 
  

 a->setType( TscChannelDef::SHAREDMEM );
 a->setBoardNr(boardnr);
 a->setChannelNr(channel);
 
 int* cv=  a->getCalValues();
 *cv=-1;
  DOM_Node n =ei.getFirstChild();
  DioLineParser dl;
  while (!n.isNull())
    {  if (strcmp(n.getNodeName().transcode(),"BackPlanePulseCtrl")==0)
       {
	*cv= dl.parseNode(n);
	dl.getInfo(  *cv)->setName("BackPlanePulseCtrl") ;
	// *cv= DioLineParser(n);
        //Dioline[  *cv]->setName("BackPlanePulseCtrl") ;
       }    
    if (strcmp(n.getNodeName().transcode(),"DimChannel")==0){
      DummyChannelParser(n, a);
      a->setType(TscChannelDef::DIMTSCDEV);
    }   
    if (strcmp(n.getNodeName().transcode(),"DeviceChannel")==0){
      DummyChannelParser(n, a);
      a->setType( TscChannelDef::SHAREDMEM);
    }  
      n = n.getNextSibling();        
    }
  if(simulation || fSimulation ) {  
    a->setType( TscChannelDef::DUMMY);
    if ( *cv >=0 )
      { //Dioline[*cv]->setType(DioChannelDef::DUMMY);}
	dl.setDummy(*cv);
      } 
  }
  tsc[device] = a;
  return device;
}

void  LtParser::FedCtrlParser(DOM_Node &ei){
 
 DOM_Node n = ei.getFirstChild();
  while (!n.isNull()) {  
    if (strcmp(n.getNodeName().transcode(),"FedProcessCtrl")==0)
      { FedProcessCtrlParser(n);}
    n = n.getNextSibling();    
  }
 
}


int  LtParser::FedProcessCtrlParser(DOM_Node &ei){ 
  bool simulation=false ;
  DOM_NamedNodeMap attributes = ei.getAttributes();
  //int attrCount = attributes.getLength();  // why ?
  int device=-1;
  DOM_Node e = attributes.getNamedItem("device");
  sscanf(e.getNodeValue().transcode(),"%d",&device  );
  
  CalExchange <int>*
    a= new  CalExchange <int>(device);
  e = attributes.getNamedItem("name");
  if(! e.isNull() )   { 
    sscanf(e.getNodeValue().transcode(),"%s",a->getName());  
  }  
  e = attributes.getNamedItem("simulation");
  if(! e.isNull() )   { 
    if ('y' == (e.getNodeValue().transcode())[0]) { simulation=true ;}  
  }
  a->setType( FedProcessCtrlInterface::SHAREDMEM);  
  DOM_Node n =ei.getFirstChild();
  while (!n.isNull()) {  
    if (strcmp(n.getNodeName().transcode(),"DimChannel")==0){
      DummyChannelParser(n, a);
      a->setType( FedProcessCtrlInterface::DIMFEDDEV);
    }   
    if (strcmp(n.getNodeName().transcode(),"DeviceChannel")==0){
      DummyChannelParser(n, a);
      a->setType( FedProcessCtrlInterface::SHAREDMEM);
    }  
    
    n = n.getNextSibling();        
  }
    
  if(simulation || fSimulation ) {  
    a->setType( FedProcessCtrlInterface::DUMMY);
    a->setSimulationState(true);
  } 
  else {
    a->setSimulationState(false);
  }
  fedctrl[device] = a;
  return device;
}//end fedctrlparser

int  LtParser::FecCtrlParser(DOM_Node &ei){ 
  bool simulation=false ;
  DOM_NamedNodeMap attributes = ei.getAttributes();
  //int attrCount = attributes.getLength();  // why ?
  int device=-1;
  DOM_Node e = attributes.getNamedItem("device");
  sscanf(e.getNodeValue().transcode(),"%d",&device  );
  
  CalExchange <int>*
    a= new  CalExchange <int>(device);
  e = attributes.getNamedItem("name");
  if(! e.isNull() )   { 
    sscanf(e.getNodeValue().transcode(),"%s",a->getName());  
  }  
  e = attributes.getNamedItem("simulation");
  if(! e.isNull() )   { 
    if ('y' == (e.getNodeValue().transcode())[0]) { simulation=true ;}  
  }
  a->setType( FecCtrlDevInterface::DUMMY);  
  DOM_Node n =ei.getFirstChild();
  while (!n.isNull()) {  
    if (strcmp(n.getNodeName().transcode(),"DimChannel")==0){
      DummyChannelParser(n, a);
      DT_DEBUG_LP("Dim channel set");
      a->setType( FecCtrlDevInterface::DIMFECCTRL);
    }   
    if (strcmp(n.getNodeName().transcode(),"DeviceChannel")==0){
      DummyChannelParser(n, a);
      DT_DEBUG_LP("Shared mem channel set");
      a->setType( FecCtrlDevInterface::SHAREDMEM);
    }    
    n = n.getNextSibling();        
  }
    
  if(simulation || fSimulation ) {  
    a->setType( FedProcessCtrlInterface::DUMMY);
    a->setSimulationState(true);
  } 
  else {
    a->setSimulationState(false);
  }
  fecctrl[device] = a;
  return device;
}//end fecctrlparser

//temperature related parsers

void LtParser::TemperatureAd590Parser(DOM_Node &ei,CalExchange <TempConverDef::TempCalValues>*  CV)
{  
  TempConverDef::TempCalValues* TCV= CV->getCalValues();
 DOM_NamedNodeMap attributes = ei.getAttributes();
// int attrCount = attributes.getLength();

 DOM_Node e = attributes.getNamedItem("gain");
 sscanf(e.getNodeValue().transcode(),"%f",&TCV->Gain);
 e = attributes.getNamedItem("caltemp");
 sscanf(e.getNodeValue().transcode(),"%f",&TCV->CalTemp);
 e = attributes.getNamedItem("caladcvalue");
 sscanf(e.getNodeValue().transcode(),"%d",&TCV->CalAdcValue);

 
 DOM_Node n =ei.getFirstChild();
 while (!n.isNull())
   {
     if (strcmp(n.getNodeName().transcode(),"AdcChannel")==0)
     {  AdcChannelParser(n, CV);
     CV->setType(  TempConverDef::AD590);
     }
     if (strcmp(n.getNodeName().transcode(),"DummyChannel")==0)
     {  DummyChannelParser(n, CV);
     CV->setType(  TempConverDef::AD590DUMMY);
     }
      if (strcmp(n.getNodeName().transcode(),"DimChannel")==0)
     {  DummyChannelParser(n, CV);
     CV->setType(  TempConverDef::TEMPSPEC);
     }
     n = n.getNextSibling();        
   }
}

void LtParser::TemperatureNTCParser(DOM_Node &ei,CalExchange <TempConverDef::TempCalValues>*  CV)
{  
  TempConverDef::TempCalValues* TCV= CV->getCalValues();
 DOM_NamedNodeMap attributes = ei.getAttributes();
// int attrCount = attributes.getLength();

 DOM_Node e = attributes.getNamedItem("gain");
 sscanf(e.getNodeValue().transcode(),"%f",&TCV->Gain);
 e = attributes.getNamedItem("caltemp");
 sscanf(e.getNodeValue().transcode(),"%f",&TCV->CalTemp);
 e = attributes.getNamedItem("caladcvalue");
 sscanf(e.getNodeValue().transcode(),"%d",&TCV->CalAdcValue);
  e = attributes.getNamedItem("openadcvalue");
 sscanf(e.getNodeValue().transcode(),"%d",&TCV->OpenAdcValue);
 e = attributes.getNamedItem("beta");
 
 sscanf(e.getNodeValue().transcode(),"%f",&TCV->Beta);

 
 DOM_Node n =ei.getFirstChild();
 while (!n.isNull())
   { if (strcmp(n.getNodeName().transcode(),"SerialChannel")==0)
      { SerialChannelParser(n, CV);        
      CV->setType(  TempConverDef::TRHXNTC);  // correct type ?
      }
     if (strcmp(n.getNodeName().transcode(),"AdcChannel")==0)
     { AdcChannelParser(n, CV);
        CV->setType(  TempConverDef::NTC);
     }
     if (strcmp(n.getNodeName().transcode(),"AdcAd4717")==0)
       {       I2cChannelParser(n, CV);
            CV->setType(  TempConverDef::NTCAD4717);
       }
     n = n.getNextSibling();        
   }
}



void LtParser::TemperatureSerialParser(DOM_Node &ei,CalExchange <TempConverDef::TempCalValues>*  CV)
{ 
  TempConverDef::TempCalValues* TCV= CV->getCalValues();
  DOM_NamedNodeMap attributes = ei.getAttributes();
  //int attrCount = attributes.getLength();  // why ?
 
   DOM_Node e = attributes.getNamedItem("gain");
 sscanf(e.getNodeValue().transcode(),"%f",&TCV->Gain);
 e = attributes.getNamedItem("caltemp");
 sscanf(e.getNodeValue().transcode(),"%f",&TCV->CalTemp);
 e = attributes.getNamedItem("caladcvalue");
 sscanf(e.getNodeValue().transcode(),"%d",&TCV->CalAdcValue);
  DOM_Node n =ei.getFirstChild();
 while (!n.isNull())
   { if (strcmp(n.getNodeName().transcode(),"SerialChannel")==0)
     {SerialChannelParser(n,CV);}
     n = n.getNextSibling();        
   }
}

void LtParser::TemperatureTempCtrlParser
           (DOM_Node &ei,CalExchange <TempConverDef::TempCalValues>*  CV)
{ char name[80]="reference";
 DOM_NamedNodeMap attributes = ei.getAttributes();
 DOM_Node e = attributes.getNamedItem("tempctrldevice");
 int tempctrldevice;
 sscanf(e.getNodeValue().transcode(),"%d",&tempctrldevice);
 CV->setBoardNr(tempctrldevice);
 e = attributes.getNamedItem("temperaturechannel");
  sscanf(e.getNodeValue().transcode(),"%d",&tempctrldevice);
 CV->setChannelNr(tempctrldevice);
// if(! e.isNull() ) {sscanf(e.getNodeValue().transcode(),"%s",name);}  //reuse of the name variable
// CV->setConnectionType(name);
}


int  LtParser::TemperatureParser(DOM_Node &ei)
{ bool simulation=false ;
  DOM_NamedNodeMap attributes = ei.getAttributes();
  int device=-1;
  DOM_Node e = attributes.getNamedItem("device");
  sscanf(e.getNodeValue().transcode(),"%d",&device  );
  CalExchange <TempConverDef::TempCalValues>*
    a= new  CalExchange <TempConverDef::TempCalValues>(device);
  e = attributes.getNamedItem("name");
  if(! e.isNull() )   { sscanf(e.getNodeValue().transcode(),"%s",a->getName());  }
  e = attributes.getNamedItem("simulation");
  if(! e.isNull() )   { if ('y' == (e.getNodeValue().transcode())[0]) { simulation=true ;}  }
 
  DOM_Node n =ei.getFirstChild();
  while (!n.isNull())
    { if (strcmp(n.getNodeName().transcode(),"TemperatureAD590")==0)
      {    TemperatureAd590Parser(n, a);
      } 
    if (strcmp(n.getNodeName().transcode(),"TemperatureDummy")==0)
      {    TemperatureAd590Parser(n, a);
           a->setType(  TempConverDef::AD590DUMMY);
      }
    if (strcmp(n.getNodeName().transcode(),"TemperatureNTC")==0)
      {    TemperatureNTCParser(n, a);
      //type def done in TemperatureNTCParser
         
      }
      if (strcmp(n.getNodeName().transcode(),"TemperatureSerial")==0)
      {  TemperatureSerialParser(n,a);
         a->setType(  TempConverDef::LINEAR);
      } 
      if (strcmp(n.getNodeName().transcode(),"TemperatureTrhxSens")==0)
      {  TemperatureSerialParser(n,a);
         a->setType(  TempConverDef::TRHXSENS);	 
      } 
      if (strcmp(n.getNodeName().transcode(),"TemperatureTrhxNtc")==0)
      {  TemperatureSerialParser(n,a);
         a->setType(  TempConverDef::TRHXNTC);
      } 
      if (strcmp(n.getNodeName().transcode(),"TemperatureTempCtrl")==0)
	{  TemperatureTempCtrlParser(n,a);
	 a->setType(  TempConverDef::TEMPCTRL);
	} 
      if (strcmp(n.getNodeName().transcode(),"TemperatureSpecial")==0)
	{  TemperatureAd590Parser(n,a);
	   a->setType(  TempConverDef::TEMPSPEC);
	} 
      n = n.getNextSibling();        
    }
  if(simulation || fSimulation ) {  a->setType( TempConverDef::DUMMY);} 
  temperatures[device] = a;
  return device;
}

// humidity related parsers

//  void LtParser::HumidityMirel1050parser(DOM_Node &ei,CalExchange<HumidityConversionDef::HumidityCalValues>*  CV)
//  { HumidityConversionDef::HumidityCalValues* HCV= CV->getCalValues();
//    DOM_NamedNodeMap attributes = ei.getAttributes();

//    DOM_Node e = attributes.getNamedItem("gain");
//    sscanf(e.getNodeValue().transcode(),"%f",&HCV->Gain);
//    e = attributes.getNamedItem("calhumidity");
//    sscanf(e.getNodeValue().transcode(),"%f",&HCV->CalHumidity);
//    e = attributes.getNamedItem("caladcvalue");
//    sscanf(e.getNodeValue().transcode(),"%f",&HCV->CalAdcValue);
//    e = attributes.getNamedItem("vrefgain");
//    sscanf(e.getNodeValue().transcode(),"%f",&HCV->VRefGain);
//    e = attributes.getNamedItem("offset");
//    sscanf(e.getNodeValue().transcode(),"%f",&HCV->Offset);
//   e = attributes.getNamedItem("calvoltage");
//    sscanf(e.getNodeValue().transcode(),"%f",&HCV->CalVoltageRef);
  
//    DOM_Node n =ei.getFirstChild();
//    while (!n.isNull())
//     { if (strcmp(n.getNodeName().transcode(),"AdcChannel")==0)
//         {AdcChannelParser(n, CV);}
//        n = n.getNextSibling();        
//     }
//  }

//  void LtParser::HumidityTrhxparser(DOM_Node &ei,CalExchange<HumidityConversionDef::HumidityCalValues>*  CV){  
//     HumidityConversionDef::HumidityCalValues* HCV= CV->getCalValues();
//     HCV->Offset=0;HCV->CalVoltageRef=5;
//     DOM_NamedNodeMap attributes = ei.getAttributes();
//     DOM_Node e = attributes.getNamedItem("gain");
//     sscanf(e.getNodeValue().transcode(),"%f",&HCV->Gain);
//     e = attributes.getNamedItem("calhumidity");
//     sscanf(e.getNodeValue().transcode(),"%f",&HCV->CalHumidity);
//     e = attributes.getNamedItem("caladcvalue");
//     sscanf(e.getNodeValue().transcode(),"%f",&HCV->CalAdcValue);
//     e = attributes.getNamedItem("vrefgain");
//     sscanf(e.getNodeValue().transcode(),"%f",&HCV->VRefGain);
 
//     DOM_Node n =ei.getFirstChild();
//     while (!n.isNull())
//       {
//         if (strcmp(n.getNodeName().transcode(),"SerialChannel")==0)
//  	 { SerialChannelParser(n,CV);}
//         n = n.getNextSibling();    
//       }
//  }

//  void LtParser::HumiditySpecialparser(DOM_Node &ei,CalExchange<HumidityConversionDef::HumidityCalValues>*  CV){ 
//    HumidityConversionDef::HumidityCalValues* HCV= CV->getCalValues();
//    HCV->Offset=0;HCV->CalVoltageRef=5;
//    DOM_NamedNodeMap attributes = ei.getAttributes();
//    DOM_Node e = attributes.getNamedItem("gain");
//    sscanf(e.getNodeValue().transcode(),"%f",&HCV->Gain);
//    e = attributes.getNamedItem("calhumidity");
//    sscanf(e.getNodeValue().transcode(),"%f",&HCV->CalHumidity);
//    e = attributes.getNamedItem("caladcvalue");
//    sscanf(e.getNodeValue().transcode(),"%f",&HCV->CalAdcValue);
//    e = attributes.getNamedItem("vrefgain");
//    sscanf(e.getNodeValue().transcode(),"%f",&HCV->VRefGain);
 
//    DOM_Node n =ei.getFirstChild();
//    while (!n.isNull())
//      { if (strcmp(n.getNodeName().transcode(),"DimChannel")==0)
//        { DummyChannelParser(n,CV);}
//        n = n.getNextSibling();    
//      }
//  }

//  int LtParser::HumidityParser(DOM_Node &ei)
//  { DOM_NamedNodeMap attributes = ei.getAttributes();
//     bool simulation=false ;
//    int device=-1;
//    DOM_Node e = attributes.getNamedItem("device");
//    sscanf(e.getNodeValue().transcode(),"%d",&device  );
//    CalExchange <HumidityConversionDef::HumidityCalValues>*
//      a= new  CalExchange <HumidityConversionDef::HumidityCalValues>(device);
//    e = attributes.getNamedItem("name");
//    if(! e.isNull() )   { sscanf(e.getNodeValue().transcode(),"%s",a->getName());  }
//    e = attributes.getNamedItem("simulation");
//    if(! e.isNull() )   { if ('y' == (e.getNodeValue().transcode())[0]) { simulation=true ;}  }
 
//    DOM_Node n =ei.getFirstChild();
//    while (!n.isNull())
//      { if (strcmp(n.getNodeName().transcode(),"HumidityMirel1050")==0)
//        {    HumidityMirel1050parser(n, a);
//             a->setType( HumidityConversionDef::HUMIRELADC);
//        }
//       if (strcmp(n.getNodeName().transcode(),"HumidityTrhx")==0)
//        {    HumidityTrhxparser(n, a);
//             a->setType( HumidityConversionDef::TRHXSENS);
//        }
//       if (strcmp(n.getNodeName().transcode(),"HumiditySpecial")==0)
//        {    HumiditySpecialparser(n, a);
//             a->setType( HumidityConversionDef::HUMSPECIAL);
//        }
//        n = n.getNextSibling();        
//      }
//    if(simulation || fSimulation) {  a->setType( HumidityConversionDef::DUMMY);}
//    humiditys[device] = a;
//    return device;
//  }

// current related parsers

void LtParser::CurrentAdcParser(DOM_Node &ei,
				  CalExchange<CurrentConvDef::CurrentCalValues>*  CV)
{ CurrentConvDef::CurrentCalValues* HCV= CV->getCalValues();
  DOM_NamedNodeMap attributes = ei.getAttributes();
  DOM_Node n =ei.getFirstChild();
   CV->setType(CurrentConvDef::DUMMY);
  while (!n.isNull())
    { if (strcmp(n.getNodeName().transcode(),"CalValues")==0)
      {  CurrentChannelCalValuesParser(n, HCV);}
     if (strcmp(n.getNodeName().transcode(),"AdcChannel")==0)
       {  AdcChannelParser(n, CV);
          CV->setType(CurrentConvDef::ADC);
       }
     if (strcmp(n.getNodeName().transcode(),"AdcAd4717")==0)
       {  I2cChannelParser(n, CV);
          CV->setType(CurrentConvDef::AD4717);
       }
     if (strcmp(n.getNodeName().transcode(),"DimChannel")==0)
       {  DummyChannelParser(n, CV);
          CV->setType(  CurrentConvDef::CURRENTSPEC);
       }
     n = n.getNextSibling();        
   }
}
             
void LtParser:: CurrentHvModuleParser(DOM_Node &ei,
				  CalExchange<CurrentConvDef::CurrentCalValues>*  CV)
{ DOM_NamedNodeMap attributes = ei.getAttributes();
  DOM_Node e = attributes.getNamedItem("hvdevice");
  int hvdevice;
  sscanf(e.getNodeValue().transcode(),"%d",&hvdevice);
  CV->setChannelNr(hvdevice);
  CV->setConnectionType("reference");
  // only for simulation mode :
  CV->getCalValues()->CalAdcValue=39522;
  CV->getCalValues()->Gain=0.042250966;
  CV->getCalValues()->CalCurrent=1630;
} 

void LtParser::CurrentChannelCalValuesParser(DOM_Node &ei,
				  CurrentConvDef::CurrentCalValues*  HCV)
{ DOM_NamedNodeMap attributes = ei.getAttributes();
  DOM_Node e = attributes.getNamedItem("gain");
  sscanf(e.getNodeValue().transcode(),"%f",&HCV->Gain);
  e = attributes.getNamedItem("calcurrent");
  sscanf(e.getNodeValue().transcode(),"%f",&HCV->CalCurrent);
  e = attributes.getNamedItem("caladcvalue");
  sscanf(e.getNodeValue().transcode(),"%f",&HCV->CalAdcValue);
  
  DOM_Node n =ei.getFirstChild();
  while (!n.isNull())
   { n = n.getNextSibling();        
   }
}

int LtParser::CurrentParser(DOM_Node &ei)
{ bool simulation=false ;
  DOM_NamedNodeMap attributes = ei.getAttributes();
  int device=-1;
  DOM_Node e = attributes.getNamedItem("device");
  sscanf(e.getNodeValue().transcode(),"%d",&device  );
  CalExchange  <CurrentConvDef::CurrentCalValues>*
    a= new  CalExchange<CurrentConvDef::CurrentCalValues>(device);
  e = attributes.getNamedItem("name");
  if(! e.isNull() )   { sscanf(e.getNodeValue().transcode(),"%s",a->getName());  }
  e = attributes.getNamedItem("simulation");
  if(! e.isNull() )   { if ('y' == (e.getNodeValue().transcode())[0]) { simulation=true ;}  }  
 
  DOM_Node n =ei.getFirstChild();
  while (!n.isNull())                        
    { if (strcmp(n.getNodeName().transcode(),"BiasCurrentAdcChannel")==0)
      {    CurrentAdcParser(n, a);
           a->setType(CurrentConvDef::ADCELECTROMETER);
      }
    if (strcmp(n.getNodeName().transcode(),"CurrentAdc")==0)
      {    CurrentAdcParser(n, a);
          // a->setType(CurrentConvDef::ADC); 
          //set by the CurrentAdcParser, depends on the ADC
      }
    if (strcmp(n.getNodeName().transcode(),"BiasCurrentHvModule")==0)
      {    CurrentHvModuleParser(n,a);
           a->setType(CurrentConvDef::HVMODULE);
      }
    if (strcmp(n.getNodeName().transcode(),"BiasCurrentDimChannel")==0){  
        CurrentAdcParser(n, a);
        a->setType(CurrentConvDef::CURRENTSPEC);
    } 
    if (strcmp(n.getNodeName().transcode(),"CurrentSpecial")==0) {  
      CurrentAdcParser(n, a);
    } 
      n = n.getNextSibling();        
    }
  if(simulation || fSimulation ) { 
    if( a->getType() != CurrentConvDef::HVMODULE ) {
      a->setType( CurrentConvDef::DUMMY);
    }
  }
  current[ncurrents++] = a;
  return device;
}

// HV / bias  related parses

void LtParser::HvStruckParser(DOM_Node &ei,CalExchange<HvConversionDef::HvCalValues>*  CV)
{ HvConversionDef::HvCalValues* HCV= CV->getCalValues();
  DOM_NamedNodeMap attributes = ei.getAttributes();
  DOM_Node e = attributes.getNamedItem("gain");
  sscanf(e.getNodeValue().transcode(),"%f",&HCV->Gain);
  e = attributes.getNamedItem("calvoltage");
  sscanf(e.getNodeValue().transcode(),"%f",&HCV->CalVoltage);
  e = attributes.getNamedItem("calvoltagesetting");
  sscanf(e.getNodeValue().transcode(),"%f",&HCV->CalVoltageSetting);
  e = attributes.getNamedItem("zerovoltage");
  sscanf(e.getNodeValue().transcode(),"%f",&HCV->ZeroVoltage);
  e = attributes.getNamedItem("currentgain");
  if(! e.isNull() ) {
    sscanf(e.getNodeValue().transcode(),"%f",&HCV->CurrentGain);
  }
  DOM_Node n =ei.getFirstChild();
  while (!n.isNull()) { 
    if (strcmp(n.getNodeName().transcode(),"SerialChannel")==0)
      {       SerialChannelParser(n, CV); }
    if (strcmp(n.getNodeName().transcode(),"DeviceChannel")==0){
      DummyChannelParser(n, CV);
    }
    if (strcmp(n.getNodeName().transcode(),"CaenNetChannel")==0)
	{       CaenNetChannelParser(n, CV);       }
    if (strcmp(n.getNodeName().transcode(),"DimChannel")==0) 
       {       DummyChannelParser(n, CV);    } 
    n = n.getNextSibling();        
  }  
}

int LtParser::HvParser(DOM_Node &ei)
{ DOM_NamedNodeMap attributes = ei.getAttributes();
   int device=-1; bool simulation=false ;
   DOM_Node e = attributes.getNamedItem("device");
   sscanf(e.getNodeValue().transcode(),"%d",&device  );
   CalExchange  <HvConversionDef::HvCalValues>*
     a= new  CalExchange<HvConversionDef::HvCalValues>(device);
   e = attributes.getNamedItem("name");
   if(! e.isNull() )   { sscanf(e.getNodeValue().transcode(),"%s",a->getName());  }
   e = attributes.getNamedItem("simulation");
   if(! e.isNull() )   { if ('y' == (e.getNodeValue().transcode())[0]) { simulation=true ;}  }
   // set default values 
   HvConversionDef::HvCalValues* HCV= a->getCalValues();
   HCV->CurrentGain=1;
   HCV->Gain=1;

  DOM_Node n =ei.getFirstChild();
  while (!n.isNull())                        
    { if (strcmp(n.getNodeName().transcode(),"HvStruck")==0)
      {   HvStruckParser(n, a);
           a->setType(HvConversionDef::HvSTRUCK_NHQ201);
      }
     if (strcmp(n.getNodeName().transcode(),"HvN470")==0)
      {    HvStruckParser(n, a);
           a->setType(HvConversionDef::HvCAEN_N470);
      } 
     if (strcmp(n.getNodeName().transcode(),"HvCaen2527")==0) {   
       DT_DEBUG_LP("HvCaen2527 found");
       HvStruckParser(n, a);
       a->setType(HvConversionDef::HvCAEN_2527);
     }
      if (strcmp(n.getNodeName().transcode(),"HvCaenA434")==0)
      {   DT_DEBUG_LP("HvCaenA43 found");
	  HvStruckParser(n, a);
           a->setType(HvConversionDef::HvCAEN_A434);     }
      if (strcmp(n.getNodeName().transcode(),"HvCaen127")==0)
      {   DT_DEBUG_LP("HvCaenA43 found");
	  HvStruckParser(n, a);
           a->setType(HvConversionDef::HvCAEN_A434);     }
      if (strcmp(n.getNodeName().transcode(),"HvCaen527")==0)
      {   DT_DEBUG_LP("HvCaen527 found");
	  HvStruckParser(n, a);
           a->setType(HvConversionDef::HvCAEN_527);
      }
      if (strcmp(n.getNodeName().transcode(),"HvDim")==0){   
	DT_DEBUG_LP("HvDim (generic) found \n");
	HvStruckParser(n, a);  // HvDimParser(n, a);
	a->setType(HvConversionDef::HvDIM);
      }
      n = n.getNextSibling();      
    }
  if(simulation  || fSimulation) {  a->setType( HvConversionDef::DUMMY);}
  hv[nhvs++] = a;
  return device;
}

int  LtParser::SlowCtrlConfigParser(DOM_Node &ei)
{ DOM_NamedNodeMap attributes = ei.getAttributes();
   int device=-1;
  DOM_Node e = attributes.getNamedItem("device");
  sscanf(e.getNodeValue().transcode(),"%d",&device  );
  SlowCtrlConfig* a= new  SlowCtrlConfig(device);

  DOM_Node n =ei.getFirstChild();
  while (!n.isNull())
     { if (strcmp(n.getNodeName().transcode(),"TemperatureProbe")==0)
       { a->addTempProbe(  TemperatureParser(n));   
         DT_DEBUG_LP( "TempId" << a->getNrTempProbes() <<" = "<<  a->getTempProbeId(a->getNrTempProbes()-1));
       }
       if (strcmp(n.getNodeName().transcode(),"HumidityProbe")==0)
	 { HumidityParser hp;
	   a->addHumidityProbe(hp.parseNode(n));
	 DT_DEBUG_LP("Hum Id"<< a->getNrHumidityProbes()<<" = "<<a->getHumidityProbeId(a->getNrHumidityProbes()-1));
	 }
        if (strcmp(n.getNodeName().transcode(),"ColdBox")==0)
	 { a->addColdBox(ColdBoxParser(n,BoxCharacteristicsDef::COLDBOX ));
	 }
      if (strcmp(n.getNodeName().transcode(),"InterLock")==0)
	 { a->addInterLock(InterLockParser(n));
	 }
        n = n.getNextSibling();    
     }
  slowctrls[device]=a;
  return device;
}

/// Bench related parsers

//  int  LtParser::BenchPosConnectionParser(DOM_Node &ei)
//  { DOM_NamedNodeMap attributes = ei.getAttributes();
//    int device=-1;
//    DOM_Node e = attributes.getNamedItem("device");
//    sscanf(e.getNodeValue().transcode(),"%d",&device  );
//    BenchPosConnection* a= new  BenchPosConnection(device);
//    DOM_Node n =ei.getFirstChild();
//    while (!n.isNull())
//      { if (strcmp(n.getNodeName().transcode(),"TemperatureProbeNtc0")==0)
//        {   a->setNTCDevice(TemperatureParser(n),0);
//         }
//       if (strcmp(n.getNodeName().transcode(),"TemperatureProbeNtc1")==0)
//        {   a->setNTCDevice(TemperatureParser(n),1);
//         }
//       if (strcmp(n.getNodeName().transcode(),"Current125")==0)
//        {   a->setLvCurrentDevice(CurrentParser(n),0);
//         }
//        if (strcmp(n.getNodeName().transcode(),"Current250")==0)
//        {   a->setLvCurrentDevice(CurrentParser(n),1);
//         }
//        if(strcmp(n.getNodeName().transcode(),"BiasCurrentConnection")==0)
//  	{ a->setHvCurrentDevice(CurrentParser(n));
//  	}
//        if(strcmp(n.getNodeName().transcode(),"BiasHvConnection")==0)
//  	{ a->setBiasDevice(HvParser(n));
//  	}
//         if(strcmp(n.getNodeName().transcode(),"I2cConnection")==0)
//  	 { I2cConnectionParser(n,a);
//  	}
//         if(strcmp(n.getNodeName().transcode(),"FedConnection")==0)
//  	 { FedConnectionParser(n,a);
//  	 }    
//        if(strcmp(n.getNodeName().transcode(),"PowerSwitch")==0)
//  	 { DioLineParser dl;
//  	 a->setPowerSwitch(dl.parseNode(n));
	 
//  	 //a->setPowerSwitch(DioLineParser(n));
//  	 }   
//         n = n.getNextSibling();        
//      }
//    benchpos[nbenchpos++] = a;
//    return device;
//  }


//  int  LtParser::I2cConnectionParser(DOM_Node &ei,BenchPosConnection* a ){  
//    int value; 
//    bool sharedmem=false; 
//    DOM_NamedNodeMap attributes = ei.getAttributes();
//    DOM_Node e=attributes.getNamedItem("dimhost");
//    string dimhost(e.getNodeValue().transcode());
//    if( dimhost == "sharedmemory") { sharedmem=true;}
//    if( dimhost == "local"){
//      int len=80;
//      char hname[80];
//      gethostname(hname,len);
//      string host(hname);
//      dimhost=host;
//    } 
//    a->setDimHostName(dimhost);
//    e= attributes.getNamedItem("fecboard");
//    sscanf(e.getNodeValue().transcode(),"%d",&value  );
//    a->setFecNr(value);
//    e = attributes.getNamedItem("ccuaddress");
//     sscanf(e.getNodeValue().transcode(),"%d",&value  );
//     a->setCcuAddress(value);
//     e = attributes.getNamedItem("ccuchannel");
//     sscanf(e.getNodeValue().transcode(),"%d",&value  ); 
//     a->setCcuChannel(value) ;
//     e = attributes.getNamedItem("latencyoffset");
//     sscanf(e.getNodeValue().transcode(),"%d",&value  );    
//     a->setLatencyOffset((unsigned int)value);
//     //set the defaults 
//     if(sharedmem){
//       a->setOptoType(BenchPosConnection::DUMMYI2C);
//       a->setPllType(BenchPosConnection::PLL);
//       a->setDcuType(BenchPosConnection::DCU2);
//       a->setApvType(BenchPosConnection::APV25);
//       a->setMuxType(BenchPosConnection::MUX25);
//     }
//     else {
//       a->setOptoType(BenchPosConnection::DUMMYI2C);
//       a->setPllType(BenchPosConnection::PLLDIM);
//       a->setDcuType(BenchPosConnection::DCU2DIM);
//       a->setApvType(BenchPosConnection::APV25DIM);
//       a->setMuxType(BenchPosConnection::MUX25DIM);
//     }
//     { e = attributes.getNamedItem("optotype");
//       if(!strcmp("opto",e.getNodeValue().transcode())){
//         if( sharedmem){
//  	 a->setOptoType(BenchPosConnection::OPTO);
//         }
//         else{
//  	 a->setOptoType(BenchPosConnection::OPTODIM);
//         }
//       }
//     }
//     { e = attributes.getNamedItem("plltype");
//       if(! e.isNull() ) {
//         if(!strcmp("dummy",e.getNodeValue().transcode())){
//  	 a->setPllType(BenchPosConnection::DUMMYI2C);
//         }
//         if(!strcmp("plldim",e.getNodeValue().transcode())){
//  	 a->setPllType(BenchPosConnection::PLLDIM);
//         }
//       }  // else keep previous defined 
//     }
//     { e = attributes.getNamedItem("dcutype");
//       if(! e.isNull() ) {
//        if(!strcmp("dummy",e.getNodeValue().transcode())){
//  	a->setDcuType(BenchPosConnection::DUMMYI2C);
//        }
//        if(!strcmp("dcudim",e.getNodeValue().transcode())){
//  	a->setDcuType(BenchPosConnection::DCU2DIM);
//        }
//       }// else keep pervious defined 
//     }
//     a->setFecSimulation(false);
//     e = attributes.getNamedItem("simulation");
//      if(! e.isNull() )
//       { if ('y' == e.getNodeValue().transcode()[0] || fSimulation  ) {  a->setFecSimulation(true);}
//       }
//     DOM_Node n =ei.getFirstChild();
//     while (!n.isNull())
//       {   n = n.getNextSibling();  }
//     return 0;
//  }

//  int  LtParser::FedConnectionParser(DOM_Node &ei,BenchPosConnection* a )
//  {  DOM_Node n =ei.getFirstChild();
//     while (!n.isNull())
//       {  if(strcmp(n.getNodeName().transcode(),"UtriChannel")==0)
//  	{ UtriChannelParser(n,a);
//  	}
//         n = n.getNextSibling();        
//       }
//     return 1;
//  }

//  int  LtParser::UtriChannelParser(DOM_Node &ei,BenchPosConnection* a )
//  {  int value;int utrichannel;
//     DOM_NamedNodeMap attributes = ei.getAttributes();
//     DOM_Node e = attributes.getNamedItem("utrichannel");
//     sscanf(e.getNodeValue().transcode(),"%d",&utrichannel  );
//     e = attributes.getNamedItem("fedserialnr");
//     sscanf(e.getNodeValue().transcode(),"%d",&value );
//     a->setFedNr(value, utrichannel);
//     a->setFedSimulation(false,utrichannel);
//     e = attributes.getNamedItem("simulation");
//     if(! e.isNull() )
//       { if ('y' == e.getNodeValue().transcode()[0] || fSimulation ) {  a->setFedSimulation(true,utrichannel);}
//       }
//     e = attributes.getNamedItem("type");
//     if( strcmp(e.getNodeValue().transcode(),"sharedmem")==0){
//       a->setFedType(BenchPosConnection::SHAREDMEM,utrichannel);
//     }
//     else { a->setFedType(BenchPosConnection::DIMDEV,utrichannel);}
//     e = attributes.getNamedItem("muxcard");
//     sscanf(e.getNodeValue().transcode(),"%d",&value );
//     if (value != -1)  { 
//       BenchPosConnection::MuxChId muxid;
//       muxid.fMuxCard=value;
//       e = attributes.getNamedItem("muxdevice");
//       sscanf(e.getNodeValue().transcode(),"%d",&value );
//       muxid.fMuxDevice=value;
//       e = attributes.getNamedItem("muxchannel");
//       sscanf(e.getNodeValue().transcode(),"%d",&value );
//       muxid.fMuxChannel=value;
//       a->setMuxId (muxid,utrichannel);
//     }
//     else     {   
//       e = attributes.getNamedItem("fedchannel");
//       sscanf(e.getNodeValue().transcode(),"%d",&value  );     
//       a->setFedChannel(value, utrichannel);
//     }
//     return 0;
//  }

// interlock related parsers 

//  int  LtParser::DioLineParser(DOM_Node &ei)
//  { DOM_NamedNodeMap attributes = ei.getAttributes();
//    int device=Dioline.size();// dioline has no device nr ,device is there to find the dio line back
//     bool simulation=false ;
//     CalExchange <int>*
//       a= new  CalExchange <int>(device);
//     int* activelevel= a->getCalValues();
//     *activelevel=1;
//     DOM_Node e = attributes.getNamedItem("name");
//     if(! e.isNull() )   { sscanf(e.getNodeValue().transcode(),"%s",a->getName());  }
//     e = attributes.getNamedItem("simulation");
//     if(! e.isNull() )   { 
//       if ('y' == (e.getNodeValue().transcode())[0]) { simulation=true ;}  
//     }
//     e = attributes.getNamedItem("activelevel");
//     if(! e.isNull() )   { 
//       // activelevel is set to 0 ?
//       if( ! atoi(e.getNodeValue().transcode())) *activelevel=0;
//     }
    
//     DOM_Node n =ei.getFirstChild();
//     while (!n.isNull())
//     {
//       if (strcmp(n.getNodeName().transcode(),"DioChannel")==0)
//       {  AdcChannelParser(n, a);
//       a->setType(  DioChannelDef::CIODIO );
//       }
//       if (strcmp(n.getNodeName().transcode(),"DummyChannel")==0)
//       {  DummyChannelParser(n, a);
//       a->setType(   DioChannelDef::DUMMY);
//       }
//        if (strcmp(n.getNodeName().transcode(),"SerialChannel")==0)
//       {  SerialChannelParser(n, a);
//          a->setType( DioChannelDef::SERIAL);
//       }
//        if (strcmp(n.getNodeName().transcode(),"DioAd4717")==0)
//       {  I2cChannelParser(n, a);
//          a->setType( DioChannelDef::AD4717DIO);
//       }
//        if (strcmp(n.getNodeName().transcode(),"DioPcf8574")==0)
//       {  I2cChannelParser(n, a);
//          a->setType( DioChannelDef::PCF8574DIO);
//       }
//      if(simulation || fSimulation ) 
//        { a->setType( DioChannelDef::DUMMY);}
//       n = n.getNextSibling();        
//     }
//     DT_DEBUG_LP("dioline "<< device<<" found board "<<a->getBoardNr()<<" channel "<<a->getChannelNr()<<" type "
//  	       << a->getType());
//   Dioline[device]=a;
//   return device;
//  }

int  LtParser::InterLockParser(DOM_Node &ei)
{ DOM_NamedNodeMap attributes = ei.getAttributes();
  int device=-1;
   bool simulation=false ;
  DOM_Node e = attributes.getNamedItem("device");
  sscanf(e.getNodeValue().transcode(),"%d",&device  );
  //CalExchange<int>* iolineref=0;
  CalExchange <int[3]>*
    a= new  CalExchange <int[3] >(device);
  DT_DEBUG_LP("interlock "<< device<<" found");
 
  e = attributes.getNamedItem("name");
  if(! e.isNull() )   { sscanf(e.getNodeValue().transcode(),"%s",a->getName());  }
  e = attributes.getNamedItem("simulation");
  if(! e.isNull() )   { if ('y' == (e.getNodeValue().transcode())[0]) { simulation=true ;}  }
  int* cv = (int*) a->getCalValues();

  DOM_Node n =ei.getFirstChild();
  DioLineParser dl;
  while (!n.isNull())                        
    { if (strcmp(n.getNodeName().transcode(),"WatchDog")==0){ 
      *(cv+ InterLock::WATCHDOG)= dl.parseNode(n);// DioLineParser(n); 
      dl.getInfo(*(cv+ InterLock::WATCHDOG))->setName("WatchDog") ;
         //Dioline[  *(cv+ InterLock::WATCHDOG)]->setName("WatchDog") ;
      }
      if (strcmp(n.getNodeName().transcode(),"StatusLine")==0)
	{ *(cv+ InterLock::STATUS) =dl.parseNode(n); // DioLineParser(n); 
	dl.getInfo(*(cv+ InterLock::STATUS))->setName("StatusLine");
	// Dioline[  *(cv+ InterLock::STATUS)]->setName("StatusLine");
	}
      if (strcmp(n.getNodeName().transcode(),"EmergencyStop")==0)
	{ *(cv+ InterLock::EMERGENCY) =dl.parseNode(n); //  DioLineParser(n);   
	dl.getInfo(*(cv+ InterLock::EMERGENCY))->setName("EmergencyLine");
//         Dioline[  *(cv+ InterLock::EMERGENCY)]->setName("EmergencyLine");
	}
      n = n.getNextSibling();        
    }
  if(simulation || fSimulation ) {
    a->setType(InterLock::DUMMY);
    dl.setDummy(*(cv+ InterLock::EMERGENCY));
    dl.setDummy(*(cv+ InterLock::STATUS));
    dl.setDummy(*(cv+ InterLock::WATCHDOG));
     //   if( (iolineref = Dioline[  *(cv+ InterLock::EMERGENCY)])) ;
//        { iolineref->setType( DioChannelDef::DUMMY);}
//        iolineref = Dioline[  *(cv+ InterLock::STATUS)] ;
//        if(iolineref) {    iolineref->setType( DioChannelDef::DUMMY);}   
//        iolineref = Dioline[  *(cv+ InterLock::WATCHDOG)] ;
//        if(iolineref) {    iolineref->setType( DioChannelDef::DUMMY);}
  }
  else {  a->setType(InterLock::ACTIVE );}
  
  Interlock[device] = a;
  return device;
}


// device info  return functions 

CalExchange <TempConverDef::TempCalValues>* 
    LtParser::getTemperatureCalibration( int device){ 
  return temperatures.getP(device);
}

  CalExchange<CurrentConvDef::CurrentCalValues>*
    LtParser::getCurrentCalibration(int device ) const 
{ CalExchange<CurrentConvDef::CurrentCalValues>* found=0 ;
  for (int i=0;i < ncurrents; i++)
       {      if ( device  == current[i]->getId())
	   {  found= current[i]; break;
	   }
	 }
    return found;
}

//  CalExchange <HumidityConversionDef::HumidityCalValues>*
//      LtParser::getHumidityCalibration( int device )  { 
//    if( humiditys.end() == humiditys.find(device))   { 
//      DT_WARN("Humidity probe" << device <<" not found");
//      return 0;}
//    else {return humiditys[device];}
//  }


 CalExchange <HvConversionDef::HvCalValues>*
    LtParser::getHvCalibration( int device ) const
{  CalExchange <HvConversionDef::HvCalValues>* found=0;
    
 for (int i=0;i < nhvs  ;i++)
       {      if ( device  == hv[i]->getId())
	   {  found= hv[i]; break;
	   }
	 }
    return found;
}



CalExchange < PowerConversionDef:: PowerCalValues>* 
    LtParser::getPowerChannelCalibration(int i  )  {  
  PowerParser pp; 
  return pp.getInfo(i);
}




// cold box related parsers 

int  LtParser::ColdBoxParser(DOM_Node &ei,
			     BoxCharacteristicsDef::BoxType kind)
{ DT_DEBUG_LP(" go to parse box of kind "<< kind);
  DOM_NamedNodeMap attributes = ei.getAttributes();
  int device=-1;
  bool simulation=false ;
  DOM_Node e = attributes.getNamedItem("device");
  sscanf(e.getNodeValue().transcode(),"%d",&device  );
  CalExchange <BoxCharacteristicsDef::BoxCharValues>*
    a= new  CalExchange <BoxCharacteristicsDef::BoxCharValues>(device);
  a->setBoardNr(device);
  e = attributes.getNamedItem("name");
  if(! e.isNull() )   { sscanf(e.getNodeValue().transcode(),"%s",a->getName());  }
  BoxCharacteristicsDef::BoxCharValues* cv = a->getCalValues();
  cv->fHumidityCtrlId=-1;
  cv->fWaterTemperatureId=-1;
  cv->fHotSideTemperatureId=-1;
  cv->fTemperatureId=-1;
 e = attributes.getNamedItem("weight");
 if(! e.isNull() )  {sscanf(e.getNodeValue().transcode(),"%f",&cv->fWeigth);}
  e = attributes.getNamedItem("heatcapacity");
  if(! e.isNull() ) { sscanf(e.getNodeValue().transcode(),"%f",&cv->fHeatCapacity);}
  e = attributes.getNamedItem("leakage");
  if(! e.isNull() ) { sscanf(e.getNodeValue().transcode(),"%f",&cv->fLeakage);}
  e = attributes.getNamedItem("loopgain");
  if(! e.isNull() ) {sscanf(e.getNodeValue().transcode(),"%f",&cv->fLoopGain);}
  e = attributes.getNamedItem("maxdelta");
  if(! e.isNull() ) {sscanf(e.getNodeValue().transcode(),"%f",&cv->fMaxDeltaTemperature);}
  e = attributes.getNamedItem("maxtemperature");
  cv->fMaxTemperature=atof(e.getNodeValue().transcode());
  e = attributes.getNamedItem("mintemperature");
  sscanf(e.getNodeValue().transcode(),"%f",&cv->fMinTemperature);
  e = attributes.getNamedItem("watertemperature");
  if(! e.isNull() ) {  cv->fWaterTemperature= atof(e.getNodeValue().transcode());}
    e = attributes.getNamedItem("fullpowermargin");
  if(! e.isNull() )   
    {  sscanf(e.getNodeValue().transcode(),"%f",&cv->fFullPowerMargin);}
  e = attributes.getNamedItem("simulation");
  a->setSimulationState(false);//defualt value 
  if(! e.isNull() )   
    { if ('y' == (e.getNodeValue().transcode())[0]) { a->setSimulationState(true); simulation=true ;}  
    }
  
  DOM_Node n =ei.getFirstChild();
  while (!n.isNull())                        
    { switch (kind)
     { case BoxCharacteristicsDef::COLDBOX:
            { if (strcmp(n.getNodeName().transcode(),"TempCtrl")==0)
	      { cv->fTempCtrlId.push_back( 
		   ColdBoxParser(n, BoxCharacteristicsDef::STANDARD));      
	      }
	      if (strcmp(n.getNodeName().transcode(),"TempCtrlDim")==0)
	      { cv->fTempCtrlId.push_back( 
		   ColdBoxParser(n, BoxCharacteristicsDef::DIMCTRL));      
	      }
	      if (strcmp(n.getNodeName().transcode(),"TempCtrlWien")==0)
	      { cv->fTempCtrlId.push_back( 
		   ColdBoxParser(n, BoxCharacteristicsDef::WIENCTRL));      
	      }
	      if (strcmp(n.getNodeName().transcode(),"HumidityCtrl")==0){
		HumidityCtrlParser hp;
		cv->fHumidityCtrlId=hp.parseNode(n);
		DT_DEBUG_LP("Humidityctrl  parser done ");
	      }
	    }
       break;
       case BoxCharacteristicsDef::DIMCTRL:
       { if (strcmp(n.getNodeName().transcode(),"DimChannel")==0)
	      { cv->fPowerConversionId= -1;
	        cv->fTemperatureId=-1;
	        DummyChannelParser(n,a);
	      }
       }
       break;
       case  BoxCharacteristicsDef::DUMMY:  // only because the compiler checks it
       { DT_WARN("Can not define dummy cold box ");
       }
       break;
       case BoxCharacteristicsDef::TEMPCTRLDUMMY: {
	 DT_WARN("Dummy temp ctrl defined");
       }
       break;
       case BoxCharacteristicsDef::STANDARD:
       case BoxCharacteristicsDef::WIENCTRL:
       default:{ 
	 if (strcmp(n.getNodeName().transcode(),"CoolingPower")==0){ 
	   PowerParser pp;
	   cv->fPowerConversionId=  pp. CoolingPowerParser(n);
	 }
	 if (strcmp(n.getNodeName().transcode(),"TemperatureProbe")==0){ 
	   int deviceid=TemperatureParser(n);
	   if( cv->fTemperatureId < 0) {  cv->fTemperatureId=deviceid;}
	   else { 
	     if( cv->fWaterTemperatureId<0){
	       cv->fWaterTemperatureId=deviceid;
	     }
	     else cv->fHotSideTemperatureId=deviceid;;
	   }	 
	 }
       }
       break;    
     }
    // all others ignore should be handled by the dtd file def
      n = n.getNextSibling();        
    }
  if(simulation || fSimulation && kind != BoxCharacteristicsDef::COLDBOX ) {
    // check if it is a tempctrl
    a->setType( BoxCharacteristicsDef::TEMPCTRLDUMMY);
    // now change the types for the devices 
    /* followin code is absolete after introduction of dummy tempctrl class    
    CalExchange < PowerConversionDef:: PowerCalValues>* 
      cp= getPowerChannelCalibration(cv->fPowerConversionId );
      if(cp>0)  { 
	if (  cp->getType() >=  PowerConversionDef::POL) {  
	  cp->setType(PowerConversionDef::POL+PowerConversionDef::DUMMY);
	}
	else{ cp->setType( PowerConversionDef::DUMMY);}
      }
      // has to set humidity ctrl to simulated 
      for (int i=0;i < ntemperatureprobes; i++)	{ 
	CalExchange <TempConverDef::TempCalValues>* ct= temperature[i];
	if ( cv->fTemperatureId == ct->getId())
	  {  ct->setType(TempConverDef::DUMMY);}
      }
    */
    }
  
  
  else {  a->setType( kind );}
  
  if (a->getType()==BoxCharacteristicsDef::COLDBOX )
    {   coldbox[device] = a; }
  else
    {  tempctrls[device]=a;  }
  return device;
}


//  int LtParser::HumidityCtrlParser(DOM_Node &ei){
//   bool simulation=false ;
//    DOM_NamedNodeMap attributes = ei.getAttributes();
//    int device=-1;
//    DOM_Node e = attributes.getNamedItem("device");
//    device=atoi(  e.getNodeValue().transcode());
//    CalExchange <  HumidityCtrlDef::HumidityCtrlValues >* a =
//      new CalExchange< HumidityCtrlDef::HumidityCtrlValues >(device);
//    // only one type defined for the moment
//    a->setType(HumidityCtrlDef::STANDARD);
//    HumidityCtrlDef::HumidityCtrlValues* cv =a->getCalValues();
//    e = attributes.getNamedItem("name");
//    if(! e.isNull() )   
//      { sscanf(e.getNodeValue().transcode(),"%s",a->getName()); }
//    e = attributes.getNamedItem("minimumhumidity");
//    cv->fMinHumidity=atof(  e.getNodeValue().transcode());
//    e = attributes.getNamedItem("maximumhumidity");
//    cv->fMaxHumidity=atof(  e.getNodeValue().transcode());
//    e = attributes.getNamedItem("cooldownhumidity");
//    cv->fCoolDownHumidity=atof(  e.getNodeValue().transcode());
//    e = attributes.getNamedItem("simulation");
//    if(! e.isNull() )   
//      { if ('y' == (e.getNodeValue().transcode())[0]) { simulation=true ;}  }
//    DOM_Node n =ei.getFirstChild();
//    while (!n.isNull()) {
//      if (strcmp(n.getNodeName().transcode(),"GasFlowCtrl")==0){
//        cv->fGasFlowId=GasFlowParser(n);
//      }
//      if (strcmp(n.getNodeName().transcode(),"HumidityProbe")==0){
//        cv->fHumidityId=HumidityParser(n);
//      }
//      n = n.getNextSibling(); 
//    }
//    if(simulation || fSimulation){
//      a->setType( HumidityCtrlDef::DUMMY);
//      // set the lower devices to dummy opperation
//      CalExchangHumidityCtrlParsere <  GasFlowDef::GasFlowValues >* 
//        gf=gasflows[cv->fGasFlowId];
//      if(gf > 0){ gf->setType(GasFlowDef::DUMMY);}
//      CalExchange <HumidityConversionDef::HumidityCalValues>*
//        hum=humiditys[cv->fHumidityId];
//      if(hum > 0) {hum->setType(HumidityConversionDef::DUMMY);}
//    }
//    //humidityctrls[device]=a;
//    return device;
//  }
//  int LtParser::GasFlowParser(DOM_Node &ei){
//    bool simulation=false ;
//    DOM_NamedNodeMap attributes = ei.getAttributes();
//    int device=-1;
//    DOM_Node e = attributes.getNamedItem("device");
//    device=atoi(  e.getNodeValue().transcode());
//    CalExchange <  GasFlowDef::GasFlowValues >* a = 
//      new CalExchange <  GasFlowDef::GasFlowValues >(device)  ;
//      e = attributes.getNamedItem("name");
//    if(! e.isNull() )   
//      { sscanf(e.getNodeValue().transcode(),"%s",a->getName()); }
//    //GasFlowDef::GasFlowValues* cv=a->getCalValues(); 
//    e = attributes.getNamedItem("simulation");
//    if(! e.isNull() )   
//      { if ('y' == (e.getNodeValue().transcode())[0]) { simulation=true ;}  }
//    DOM_Node n =ei.getFirstChild();
//    while (!n.isNull()) { 
//       if (strcmp(n.getNodeName().transcode(),"GasFlowDac")==0)
//         {  GasDacParser(n, a); //type is set depending on the Dac
//         }
//       n = n.getNextSibling(); 
//     }  
//     if(simulation || fSimulation ) {
//       a->setType( GasFlowDef::DUMMY);
//     }
//     gasflows[device]=a;
//     return device;
//  }

//  void LtParser::GasDacParser(DOM_Node &ei, 
//  				 CalExchange < GasFlowDef::GasFlowValues>*  CV){  
//    GasFlowDef::GasFlowValues* cv= CV->getCalValues();
//    DOM_NamedNodeMap attributes = ei.getAttributes();
//    DOM_Node  e = attributes.getNamedItem("maxdac");
//    sscanf(e.getNodeValue().transcode(),"%hu",&cv->fMaxDac);
//    e = attributes.getNamedItem("gain");
//    cv->fGain=atoi( e.getNodeValue().transcode());
//    DOM_Node n =ei.getFirstChild();
//    while (!n.isNull())  {
//      if (strcmp(n.getNodeName().transcode(),"DacChannel")==0) {  
//        AdcChannelParser(n, CV);// for the moment there 
//        //is no difference between DAC and ADC parser 
//        CV->setType(GasFlowDef::DAC);
//      }
//      if (strcmp(n.getNodeName().transcode(),"SerialChannel")==0) {
//        SerialChannelParser(n, CV); 
//        CV->setType(GasFlowDef::SERIAL);
//      }
//      if (strcmp(n.getNodeName().transcode(),"DummyChannel")==0){
//        DummyChannelParser(n, CV); 
//        CV->setType(GasFlowDef::DUMMY);
//      }
//      n = n.getNextSibling(); 
//   }

//  }



//  int  LtParser::CoolingPowerParser(DOM_Node &ei)
//  {  bool simulation=false ;
//     DOM_NamedNodeMap attributes = ei.getAttributes();
//     int device=-1;
//     DOM_Node e = attributes.getNamedItem("device");
//     sscanf(e.getNodeValue().transcode(),"%d",&device  );
//     CalExchange < PowerConversionDef:: PowerCalValues>* 
//       a  =   new CalExchange < PowerConversionDef:: PowerCalValues>(device);
//     e = attributes.getNamedItem("name");
//     if(! e.isNull() )   
//       { sscanf(e.getNodeValue().transcode(),"%s",a->getName()); }
//     e = attributes.getNamedItem("simulation");
//     if(! e.isNull() )   
//       { if ('y' == (e.getNodeValue().transcode())[0]) { simulation=true ;}  }
   
//     DOM_Node n =ei.getFirstChild();
//     while (!n.isNull())
//       { if (strcmp(n.getNodeName().transcode(),"PeltierDac")==0)
//         {  PeltierDacParser(n, a); //type is set depending on the Dac  type 
//         }
//         if (strcmp(n.getNodeName().transcode(),"PeltierDacPol")==0)
//         {  PeltierDacParser(n, a); //type is set depending on the Dac  type    
//         }  
//         n = n.getNextSibling(); 
//       }  
//     if(simulation || fSimulation ) {
//       if ( a->getType() >=  PowerConversionDef::POL){  
//         if ( a->getType() >=  PowerConversionDef::POLCOM){
//  	 a->setType(PowerConversionDef::POL+PowerConversionDef::DUMMY);
//         }
//         else {	   
//  	 a->setType(PowerConversionDef::POL+PowerConversionDef::DUMMY);
//         }
//       }
//       else{ a->setType( PowerConversionDef::DUMMY);
//       } 
//     }
//     power[device]=a;
//     return device;
//  }

//  void LtParser::PeltierDacParser(DOM_Node &ei, 
//  				 CalExchange < PowerConversionDef:: PowerCalValues>*  CV)
//  {  
//    PowerConversionDef:: PowerCalValues* HCV= CV->getCalValues();
//   DOM_NamedNodeMap attributes = ei.getAttributes();
//  // int attrCount = attributes.getLength();

//   DOM_Node e = attributes.getNamedItem("gain");
//   sscanf(e.getNodeValue().transcode(),"%f",&HCV->Gain);
//   e = attributes.getNamedItem("calpower");
//   sscanf(e.getNodeValue().transcode(),"%f",&HCV->CalPower);
//   e = attributes.getNamedItem("caldacvalue");
//   sscanf(e.getNodeValue().transcode(),"%hu",&HCV->CalDacValue);

//   e = attributes.getNamedItem("maxdac");
//   sscanf(e.getNodeValue().transcode(),"%hu",&HCV->MaxDac);
//   e = attributes.getNamedItem("positivevalue");
//   if(! e.isNull() )   // can be for normal DACPELTIER
//     {HCV->PosOutput=  atoi(  e.getNodeValue().transcode());}
//   e = attributes.getNamedItem("negativevalue");
//   if(! e.isNull() )   
//     {HCV->NegOutput=  atoi(  e.getNodeValue().transcode());}
  
//   DOM_Node n =ei.getFirstChild();
//   while (!n.isNull())
//     {
//       if (strcmp(n.getNodeName().transcode(),"DacChannel")==0)
//       {  AdcChannelParser(n, CV);// for the moment there 
//                  //is no difference between DAC and ADC parser 
//          CV->setType(PowerConversionDef::DAC);
//       }
//        if (strcmp(n.getNodeName().transcode(),"SerialChannel")==0)
//       {  SerialChannelParser(n, CV); 
//          CV->setType(PowerConversionDef::SERIAL);
//       }
//        if (strcmp(n.getNodeName().transcode(),"DummyChannel")==0)
//       {  DummyChannelParser(n, CV); 
//          CV->setType(PowerConversionDef::DUMMY);
//       }
//        if (strcmp(n.getNodeName().transcode(),"DioLine")==0)  
//  	// only allowed for PeltierDacPol 
//         {HCV->DioChannelId= DioLineParser(n);
//           CV->setType(CV->getType()+PowerConversionDef::POL);
//         }
//        if (strcmp(n.getNodeName().transcode(),"CommonDioLine")==0)  
//  	// only allowed for PeltierDacPol 
//         {HCV->DioChannelId= DioLineParser(n);
//           CV->setType(CV->getType()+PowerConversionDef::POLCOM);
//         }
//       n = n.getNextSibling();        
//     }
//  }



 BenchPosConnection* LtParser::getRegisteredBenchPos(int i)  {
   BenchPositionParser bp;
   return  bp.getRegisteredBenchPos(i);

//    BenchPosConnection*   benchdummy=0;
//    if ( 0 <= i &&  i < nbenchpos){
//      benchdummy= benchpos[i];
//    }
//    else {  
//      benchdummy= new BenchPosConnection (-1); //everything set  to -1 
//    }  
//    return benchdummy;
  }


 BenchPosConnection*  LtParser::getBenchPosConnections(int i)
   // const 
{
   BenchPositionParser bp;
   return  bp.getBenchPosConnections( i);
   //   bool found=false; int search =-1;BenchPosConnection*   benchdummy;
//      while ( !found && (search <  nbenchpos-1) )
//        { search++; 
//        if ( benchpos[search]->getId() == i){ found = true ; }
//        }

//      if ( !found )
//        {  benchdummy= new BenchPosConnection (-1); //everything set  to -1 
	
//  	   }
//      else 
//        {
//  	 benchdummy= benchpos[search];
//        }
//      return benchdummy; 
  }
int LtParser::getNumberBenchPositions() const{
  BenchPositionParser bp;
  return bp.getNumberBenchPositions();
}


// section for wrappers  
#include "tsctypewrapper.h" 
TscChannelInterface* LtParser::getTscCard(int i) 
{ TscTypeWrapper* t = new TscTypeWrapper(i,this);
 TscChannelInterface* r = t->getTscDevice();
 delete t;
 return r;
}

#include "fedprocessctrlwrapper.h"
FedProcessCtrlInterface* LtParser::getFedProcessCtrl(int i,bool ordernr)
{ FedProcessCtrlWrapper* t=new FedProcessCtrlWrapper(i,this,ordernr);
  FedProcessCtrlInterface*  r = t->getFedProcessCtrl();
 delete t;
 return r;
} 

#include "fecctrldevwrapper.h"
FecCtrlDevInterface* LtParser::getFecCtrlDev(int i)
{ FecCtrlDevWrapper* t=new FecCtrlDevWrapper(i,this);
  FecCtrlDevInterface*  r = t->getFecCtrlDev();
 delete t;
 return r;
} 



#include "ledcardwrapper.h"
LedCardInterface* LtParser::getLedCard(int i){
  LedCardWrapper* t=new LedCardWrapper(i,this);
  LedCardInterface*  r = t->getLedCard();
 delete t;
 return r;
} 

#include "fbcardwrapper.h"
FBCardInterface* LtParser::getFBCard(int i){
  FBCardWrapper* t=new FBCardWrapper(i,this);
  FBCardInterface*  r = t->getFBCard();
 delete t;
 return r;
} 

DioChannelInterface*  LtParser::getBackPlanePulseCtrl( int i)
{  DioChannelInterface* ret=0;
    CalExchange <int>* c = getTscCardInfo(i);
    if(c)
      { 
	DioLineParser dl; 
	CalExchange <int>* a=dl.getInfo(*(c->getCalValues()));
	  //getDioLineInfo( *(c->getCalValues()));
             if(a) { ret= getDioChannel(a);}
    }
    return ret;
}

#include <diotypewrapper.h>

DioChannelInterface* LtParser::getDioChannel(CalExchange<int>*  c ) 
 { DioTypeWrapper* t = new DioTypeWrapper(c);
  DioChannelInterface* r = t->getDioChannel();
   delete t;
   return r;
 }


bool LtParser::fileparsed=false;

DioChannelInterface* LtParser::getDioChannel(int order){
  DioLineParser dl;
  return getDioChannel( dl.getInfo(order));
}


CalExchange<int>* LtParser::getDioLineInfo( int order){
 DioLineParser dl;
 return dl.getInfo(order);
}
