#include "setparser.h"
#include <util/PlatformUtils.hpp>
#include <util/XMLString.hpp>
#include <util/XMLUniDefs.hpp>
#include <framework/XMLFormatter.hpp>
#include <util/TranscodingException.hpp>


#include <dom/DOM_DOMException.hpp>

#include <parsers/DOMParser.hpp>
#include <dom/DOM.hpp>
#include "DOMTreeErrorReporter.hpp"
#include <cstdio>

using namespace std;

// ---------------------------------------------------------------------------
//  ostream << DOMString
//
//  Stream out a DOM string. Doing this requires that we first transcode
//  to char * form in the default code page for the system
// ---------------------------------------------------------------------------
//  ostream& operator<< (ostream& target, const DOMString& s)
//  {
//    char *p = s.transcode();
//    target << p;
//    delete [] p;
//    return target;
//  }

//static char*                    gXmlFile               = 0;
static bool                     gDoNamespaces          = false;
static bool                     gDoCreate              = false;
static XMLCh*                   gEncodingName          = 0;
//static XMLFormatter::UnRepFlags gUnRepFlags            = XMLFormatter::UnRep_CharRef;
static DOMParser::ValSchemes    gValScheme             = DOMParser::Val_Auto;
static XMLFormatter*            gFormatter             = 0;


void SetParser::clear()
{ 
  DT_DEBUG_SP( " Clear I2cDescriptor "<<nI2cDescriptors); 
   fI2cDescriptorMap.clear();
 nI2cDescriptors=0; 

    DT_DEBUG_SP(" Clear PedDescriptor "<<nPedRunDescriptors); 
     fPedRunDescriptorMap.clear();
 nPedRunDescriptors=0;
 
    DT_DEBUG_SP(" Clear CalDescriptor "<<nCalRunDescriptors); 
     fCalRunDescriptorMap.clear();
 nCalRunDescriptors=0;

   DT_DEBUG_SP(" Clear LightDescriptor "<<nLightRunDescriptors); 
     fLightRunDescriptorMap.clear();
 nLightRunDescriptors=0;

  DT_DEBUG_SP(" Clear FbDescriptor "<<nFbRunDescriptors); 
     fFbRunDescriptorMap.clear();
 nFbRunDescriptors=0;

   DT_DEBUG_SP(" Clear CalProfDescriptor "<<nCalProfRunDescriptors); 
     fCalProfRunDescriptorMap.clear();
 nCalProfRunDescriptors=0;
 
   DT_DEBUG_SP(" Clear BPlaneDescriptor "<<nBPlaneRunDescriptors); 
     fBPlaneRunDescriptorMap.clear();
 nBPlaneRunDescriptors=0;

   DT_DEBUG_SP(" Clear LedDescriptor "<<nLedRunDescriptors); 
     fLedRunDescriptorMap.clear();
 nLedRunDescriptors=0;

  DT_DEBUG_SP(" Clear AuxRunDescriptor "<<nAuxChipRunDescriptors); 
     fAuxChipRunDescriptorMap.clear();
 nAuxChipRunDescriptors=0;

 DT_DEBUG_SP(" ClearConnectivityRunDescriptor "<<fConnectivityRunDescriptorMap.size()); 
 fConnectivityRunDescriptorMap.clear();
 
 
   DT_DEBUG_SP(" Clear LatDescriptor "<<nLatRunDescriptors); 
     fLatRunDescriptorMap.clear();
 nLatRunDescriptors=0;
  
    DT_DEBUG_SP(" Clear SensorDescriptor "<<nSensorDescriptors); 
     fSensorDescriptorMap.clear();
 nSensorDescriptors=0;
    DT_DEBUG_SP(" Clear SetupDescriptor "<<nSetupDescriptors); 
    fSetupDescriptorMap.clear();
 nSetupDescriptors=0;

     DT_DEBUG_SP(" Clear ProcedureDescriptor "<<nProcedureDescriptors); 
      fProcedureDescriptorMap.clear();
 nProcedureDescriptors=0; 

     DT_DEBUG_SP(" Clear FlagsDescriptor "<<nFlagsDescriptors); 
      fFlagsDescriptorMap.clear();
 nFlagsDescriptors=0; 

 DT_NOTE(" clear done");
}
int  SetParser::parseFile(const char* name)
{    
  fFileName=name;

  DT_NOTE(" parseFile ");
  //  cout<<"parseFile "<<name<<endl;
//     try
//      {
//        XMLPlatformUtils::Initialize();
//      }
  
//    catch(const XMLException& toCatch)
//      {
//        cerr << "Error during Xerces-c Initialization.\n"
//             << "  Exception message:"
//             << DOMString(toCatch.getMessage()).transcode() << endl;
//        return ;
//      }
 
  //
  //  Create our parser, then attach an error handler to the parser.
  //  The parser will call back to methods of the ErrorHandler if it
  //  discovers errors during the course of parsing the XML document.
  //
   DOMParser *parser;
  parser = new DOMParser;
    
  parser->setValidationScheme(gValScheme);
  parser->setDoNamespaces(gDoNamespaces);
  parser->setCreateEntityReferenceNodes(gDoCreate);
  parser->setToCreateXMLDeclTypeNode(true);
  ErrorHandler *errReporter = new DOMTreeErrorReporter();
 
  parser->setErrorHandler(errReporter);
  //
  //  Parse the XML file, catching any XML exceptions that might propogate
  //  out of it.
  //
  bool errorsOccured = false;
  try
    {
      parser->parse(name);

    }

  catch (const XMLException& e)
    {
      cerr << "An error occured during parsing 1\n   Message: "
           << DOMString(e.getMessage()).transcode() << endl;
      errorsOccured = true;
    }

  
  catch (const DOM_DOMException& e)
    {
      cerr << "An error occured during parsing 2 \n   Message: "
           << DOMString(e.msg).transcode() << endl;
      errorsOccured = true;
    }

  catch (...)
    {
      cerr << "An error occured during parsing 4 \n " << endl;
      errorsOccured = true;
    }
  // If the parse was successful, output the document data from the DOM tree
 
  if (!errorsOccured)
    {
       DT_DEBUG_SP( " get doc===== ");
      DOM_Node doc = parser->getDocument();

     

      DOM_Node n1 = doc.getFirstChild();
 
      while (!n1.isNull())
        {
	  if (DOM_Node::TEXT_NODE == n1.getNodeType()) { DT_DEBUG_SP( "skip 2" );}
	  else
	    { DT_DEBUG_SP( n1.getNodeName().transcode() << " type "<< n1.getNodeType());
	    }
 
          if (n1.getNodeType() == DOM_Node::ELEMENT_NODE) break;
          n1 = n1.getNextSibling();
        }

      DOM_Node n = n1.getFirstChild();
  
      if (n!=0 && n.isNull())
        cout <<" Bad element "<< n.getNodeName().transcode()<< endl;
      while( !n.isNull() )  { 
	// the node was really an element.
	DT_DEBUG_SP(" look for tags for node type  "<<n.getNodeType());
	if (DOM_Node::TEXT_NODE == n.getNodeType()) { 
	  DT_DEBUG_SP("skip 1");
	}
	else {
	  if (strcmp("I2CDescriptor",n.getNodeName().transcode())==0)  {  
	    I2cDescriptorParser(n);
	    DT_DEBUG_SP("i2cdescriptor "<< n.getNodeName().transcode());
	  }
	  if (strcmp("PedRunDescriptor",n.getNodeName().transcode())==0)  {   
	    PedRunDescriptorParser(n);
	    DT_DEBUG_SP(" pedrundescriptor"<< n.getNodeName().transcode());
	  }
	  if (strcmp("CalRunDescriptor",n.getNodeName().transcode())==0) { 
	    CalRunDescriptorParser(n);
	    DT_DEBUG_SP("calrundescriptor"<< n.getNodeName().transcode());
	  }
         if (strcmp("LightRunDescriptor",n.getNodeName().transcode())==0) { 
	    LightRunDescriptorParser(n);
	    DT_DEBUG_SP("lightrundescriptor"<< n.getNodeName().transcode());
	  }
        if (strcmp("FbRunDescriptor",n.getNodeName().transcode())==0) { 
	    FbRunDescriptorParser(n);
	    DT_DEBUG_SP("fbrundescriptor"<< n.getNodeName().transcode());
	  }
      if (strcmp("AuxChipRunDescriptor",n.getNodeName().transcode())==0) { 
	    AuxChipRunDescriptorParser(n);
	    DT_DEBUG_SP("auxchiprundescriptor"<< n.getNodeName().transcode());
	  }
	 if (strcmp("ConnectivityRunDescriptor",n.getNodeName().transcode())==0) { 
	    ConnectivityRunDescriptorParser(n);
	    DT_DEBUG_SP("lightrundescriptor"<< n.getNodeName().transcode());
	  }
	  if (strcmp("CalProfRunDescriptor",n.getNodeName().transcode())==0) {  
	    CalProfRunDescriptorParser(n);
	    DT_DEBUG_SP("calprofrundescriptor"<< n.getNodeName().transcode());
	  }
          if (strcmp("BPlaneRunDescriptor",n.getNodeName().transcode())==0) {  
	    BPlaneRunDescriptorParser(n);
	    DT_DEBUG_SP("bprundescriptor"<< n.getNodeName().transcode());
	  }
	  if (strcmp("LedRunDescriptor",n.getNodeName().transcode())==0){  
	    LedRunDescriptorParser(n);
	    DT_DEBUG_SP("ledrundescriptor"<< n.getNodeName().transcode());
	  }
	  if (strcmp("LatRunDescriptor",n.getNodeName().transcode())==0) {  
	    LatRunDescriptorParser(n);
	    DT_DEBUG_SP("latrundescriptor"<< n.getNodeName().transcode());
	  }
	  if (strcmp("SensorDescriptor",n.getNodeName().transcode())==0) {  
	    SensorDescriptorParser(n);
	    DT_DEBUG_SP("sensordescripotor"<< n.getNodeName().transcode());
	  } 
	  if (strcmp("SetupDescriptor",n.getNodeName().transcode())==0) {  
	    SetupDescriptorParser(n);
	    DT_DEBUG_SP("Setupdescriptor"<< n.getNodeName().transcode() );
	  }
	  if (strcmp("ProcedureDescriptor",n.getNodeName().transcode())==0){
	    ProcedureDescriptorParser(n);
	    DT_DEBUG_SP("proceduredescriptor"<< n.getNodeName().transcode());
	  }
          if (strcmp("FlagsDescriptor",n.getNodeName().transcode())==0){
	    FlagsDescriptorParser(n);
	    DT_DEBUG_SP("flagsdescriptor"<< n.getNodeName().transcode());
	  }
	  if (strcmp("DaqCtrlDescriptor",n.getNodeName().transcode())==0){
	    DaqCtrlDescriptorParser(n);
	    DT_DEBUG_SP("daqctrldescriptor"<< n.getNodeName().transcode());
	  }
	}


          n = n.getNextSibling();
          if (n!=0 && n.isNull())
            cout << " Bad element "<< n.getNodeName().transcode() << endl;
        }
      DT_DEBUG_SP("file processed clean up parser" );
  DT_NOTE(" File parsed ");
    }

    
  delete gFormatter;


  delete gEncodingName;
  //
  //  Clean up the error handler. The parser does not adopt handlers
  //  since they could be many objects or one object installed for multiple
  //  handlers.
  //


  //
  //  Delete the parser itself.  Must be done prior to calling Terminate, below.
  //

  delete parser;

  // And call the termination method
// done by ~DomInit()
//    XMLPlatformUtils::Terminate();
   // DomMemDebug().print();
    
  //
  //  The DOM document and its contents are reference counted, and need
  //  no explicit deletion.
  //
 
  return (int) errorsOccured ;
}

// general  parser functions

int  SetParser::PedRunDescriptorParser(DOM_Node &ei)
{
 DT_NOTE(" PedRun ");
  DOM_NamedNodeMap attributes = ei.getAttributes();
  //int attrCount = attributes.getLength();  // why ?

  string nameid;
  char temp[30];
  DOM_Node e = attributes.getNamedItem("name");
  
  sscanf(e.getNodeValue().transcode(),"%s",temp  );
  nameid=temp;
  PedRunDescriptor* a = new PedRunDescriptor();
 
 e = attributes.getNamedItem("nped");
 if(! e.isNull() )   { a->nped = atoi(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("npedraw");
 if(! e.isNull() ) { a->npedraw=atoi(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("modped");
 if(! e.isNull() ) { a->modped=atoi(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("tskip");
 if(! e.isNull() ) { a->tskip=atof(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("tbad");
 if(! e.isNull() ) { a->tbad=atof(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("rmsmax");
 if(! e.isNull() ) { a->rmsmax=atof(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("rmsmin");
 if(! e.isNull() ) { a->rmsmin=atof(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("pedmax");
 if(! e.isNull() ) { a->pedmax=atoi(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("pedmin");
 if(! e.isNull() ) { a->pedmin=atoi(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("pskip");
 if(! e.isNull() ) { a->pskip=atof(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("modcmn");
 if(! e.isNull() ) { a->modcmn=atoi(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("nstrcmn");
 if(! e.isNull() ) { a->nstrcmn=atoi(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("ncmniter");
 if(! e.isNull() ) { a->ncmniter=atoi(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("lat");
 if(! e.isNull() ) { a->lat=atoi(e.getNodeValue().transcode());  }

 e = attributes.getNamedItem("rmsin1");
 if(! e.isNull() ) { a->rmsmin1=atof(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("rmsmin2");
 if(! e.isNull() ) { a->rmsmin2=atof(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("rmsmin3");
 if(! e.isNull() ) { a->rmsmin3=atof(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("rmsmax4");
 if(! e.isNull() ) { a->rmsmax4=atof(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("rmsmax2");
 if(! e.isNull() ) { a->rmsmax2=atof(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("rmsmax3");
 if(! e.isNull() ) { a->rmsmax3=atof(e.getNodeValue().transcode());  }

 fPedRunDescriptorMap[temp]= *a;
 delete a;
   nPedRunDescriptors++ ; 
  return 0;
}

int  SetParser::CalProfRunDescriptorParser(DOM_Node &ei)
{
 DT_NOTE(" CalProfRun ");
  DOM_NamedNodeMap attributes = ei.getAttributes();
  //int attrCount = attributes.getLength();  // why ?

  string nameid;
  char temp[30];
  DOM_Node e = attributes.getNamedItem("name");
  
  sscanf(e.getNodeValue().transcode(),"%s",temp  );
  nameid=temp;
  CalProfRunDescriptor* a = new CalProfRunDescriptor();

 e = attributes.getNamedItem("nevpoint");
 if(! e.isNull() )   { a->nevpoint = atoi(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("latstart");
 if(! e.isNull() )   { a->latstart= atoi(e.getNodeValue().transcode());  } 
 e = attributes.getNamedItem("latend");
 if(! e.isNull() )   { a->latend= atoi(e.getNodeValue().transcode());  } 
 e = attributes.getNamedItem("calamp");
 if(! e.isNull() )   { a->calamp = atoi(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("tbad");
 if(! e.isNull() )   { a->tbad = atof(e.getNodeValue().transcode()); }
  e = attributes.getNamedItem("finedelayskip");
 if(! e.isNull() )   {  a->timeskip = atoi(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("groupstodrive");
 if(! e.isNull() )   {  a->drvskip = atoi(e.getNodeValue().transcode());  }
 a->pedestalskip=false;
 e = attributes.getNamedItem("pedestalskip");
 if(! e.isNull() ) { 
   if (e.getNodeValue().transcode()[0]=='y'){
    a->pedestalskip=true;  
   }
 }

 e = attributes.getNamedItem("calTPmin1");
 if(! e.isNull() )   { a->calTPmin1 = atof(e.getNodeValue().transcode()); }
 e = attributes.getNamedItem("calTPmin2");
 if(! e.isNull() )   { a->calTPmin2 = atof(e.getNodeValue().transcode()); }
 e = attributes.getNamedItem("calTPmin3");
 if(! e.isNull() )   { a->calTPmin3 = atof(e.getNodeValue().transcode()); }
 e = attributes.getNamedItem("calTPmax4");
 if(! e.isNull() )   { a->calTPmax4 = atof(e.getNodeValue().transcode()); }
 e = attributes.getNamedItem("calTPmax2");
 if(! e.isNull() )   { a->calTPmax2 = atof(e.getNodeValue().transcode()); }
 e = attributes.getNamedItem("calTPmax3");
 if(! e.isNull() )   { a->calTPmax3 = atof(e.getNodeValue().transcode()); }

 e = attributes.getNamedItem("calPAmin1");
 if(! e.isNull() )   { a->calPAmin1 = atof(e.getNodeValue().transcode()); }
 e = attributes.getNamedItem("calPAmax1");
 if(! e.isNull() )   { a->calPAmax1 = atof(e.getNodeValue().transcode()); }

 fCalProfRunDescriptorMap[temp]= *a;
 delete a;
   nCalProfRunDescriptors++ ; 
  return 0;
}

int  SetParser::CalRunDescriptorParser(DOM_Node &ei)
{
 DT_NOTE(" CalRun ");
  DOM_NamedNodeMap attributes = ei.getAttributes();
  //int attrCount = attributes.getLength();  // why ?

  string nameid;
  char temp[30];
  DOM_Node e = attributes.getNamedItem("name");
  
  sscanf(e.getNodeValue().transcode(),"%s",temp  );
  nameid=temp;
  CalRunDescriptor* a = new CalRunDescriptor();

 e = attributes.getNamedItem("nevpoint");
 if(! e.isNull() )   { a->nevpoint = atoi(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("lat");
 if(! e.isNull() )   { a->lat= atoi(e.getNodeValue().transcode());  } 
 e = attributes.getNamedItem("calamp");
 if(! e.isNull() )   { a->calamp = atoi(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("tbad");
 if(! e.isNull() )   { a->tbad = atof(e.getNodeValue().transcode());  }

 e = attributes.getNamedItem("calAmin1");
 if(! e.isNull() )   { a->calAmin1 = atof(e.getNodeValue().transcode()); }
 e = attributes.getNamedItem("calAmax1");
 if(! e.isNull() )   { a->calAmax1 = atof(e.getNodeValue().transcode()); }


 fCalRunDescriptorMap[temp]= *a;
 delete a;
   nCalRunDescriptors++ ; 
  return 0;
}

int  SetParser::LightRunDescriptorParser(DOM_Node &ei)
{
 DT_NOTE(" LightRun ");
  DOM_NamedNodeMap attributes = ei.getAttributes();
  //int attrCount = attributes.getLength();  // why ?

  string nameid;
  char temp[30];
  DOM_Node e = attributes.getNamedItem("name");
  
  sscanf(e.getNodeValue().transcode(),"%s",temp  );
  nameid=temp;
  LightRunDescriptor* a = new LightRunDescriptor();

 e = attributes.getNamedItem("nevpoint");
 if(! e.isNull() )   { a->nevpoint = atoi(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("lat");
 if(! e.isNull() )   { a->lat= atoi(e.getNodeValue().transcode());  } 
 e = attributes.getNamedItem("calamp");
 if(! e.isNull() )   { a->calamp = atoi(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("tbad");
 if(! e.isNull() )   { a->tbad = atof(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("alight");
 if(! e.isNull() )   { a->alight = atof(e.getNodeValue().transcode());  }

 e = attributes.getNamedItem("calLightmin1");
 if(! e.isNull() )   { a->calLightmin1 = atof(e.getNodeValue().transcode()); }
 e = attributes.getNamedItem("calLightmax1");
 if(! e.isNull() )   { a->calLightmax1 = atof(e.getNodeValue().transcode()); }

 fLightRunDescriptorMap[temp]= *a;
 delete a;
   nLightRunDescriptors++ ; 
  return 0;
}
int  SetParser::FbRunDescriptorParser(DOM_Node &ei)
{
 DT_NOTE(" FbRun ");
  DOM_NamedNodeMap attributes = ei.getAttributes();
  //int attrCount = attributes.getLength();  // why ?

  string nameid;
  char temp[30];
  DOM_Node e = attributes.getNamedItem("name");
  
  sscanf(e.getNodeValue().transcode(),"%s",temp  );
  nameid=temp;
  FbRunDescriptor* a = new FbRunDescriptor();

 e = attributes.getNamedItem("nevpoint");
 if(! e.isNull() )   { a->nevpoint = atoi(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("lat");
 if(! e.isNull() )   { a->lat= atoi(e.getNodeValue().transcode());  } 
 e = attributes.getNamedItem("calamp");
 if(! e.isNull() )   { a->calamp = atoi(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("tbad");
 if(! e.isNull() )   { a->tbad = atof(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("vfb");
 if(! e.isNull() )   { a->vfb = atof(e.getNodeValue().transcode());  }

 e = attributes.getNamedItem("calFbmin1");
 if(! e.isNull() )   { a->calFbmin1 = atof(e.getNodeValue().transcode()); }
 e = attributes.getNamedItem("calFbmax1");
 if(! e.isNull() )   { a->calFbmax1 = atof(e.getNodeValue().transcode()); }


 fFbRunDescriptorMap[temp]= *a;
 delete a;
   nFbRunDescriptors++ ; 
  return 0;
}


int  SetParser::BPlaneRunDescriptorParser(DOM_Node &ei)
{
  DOM_NamedNodeMap attributes = ei.getAttributes();
  //int attrCount = attributes.getLength();  // why ?

  string nameid;
  char temp[30];
  DOM_Node e = attributes.getNamedItem("name");
  
  sscanf(e.getNodeValue().transcode(),"%s",temp  );
  nameid=temp;
  BPlaneRunDescriptor* a = new BPlaneRunDescriptor();

 e = attributes.getNamedItem("nevpoint");
 if(! e.isNull() )   { a->nevpoint = atoi(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("lat");
 if(! e.isNull() )   { a->lat= atoi(e.getNodeValue().transcode());  } 
 e = attributes.getNamedItem("tsclat");
 if(! e.isNull() )   { a->tsclat= atoi(e.getNodeValue().transcode());  } 
 e = attributes.getNamedItem("calamp");
 if(! e.isNull() )   { a->calamp = atoi(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("tbad");
 if(! e.isNull() )   { a->tbad = atof(e.getNodeValue().transcode());  }


 e = attributes.getNamedItem("calBpmin1");
 if(! e.isNull() )   { a->calBpmin1 = atof(e.getNodeValue().transcode()); }
 e = attributes.getNamedItem("calBpmax1");
 if(! e.isNull() )   { a->calBpmax1 = atof(e.getNodeValue().transcode()); }


 fBPlaneRunDescriptorMap[temp]= *a;
 delete a;
   nBPlaneRunDescriptors++ ; 
  return 0;
}

int  SetParser::LatRunDescriptorParser(DOM_Node &ei)
{
   DT_NOTE(" LatRun ");
  DOM_NamedNodeMap attributes = ei.getAttributes();
  //int attrCount = attributes.getLength();  // why ?

  string nameid;
  char temp[30];
  DOM_Node e = attributes.getNamedItem("name");
  
  sscanf(e.getNodeValue().transcode(),"%s",temp  );
  nameid=temp;
  LatRunDescriptor* a = new LatRunDescriptor();

 e = attributes.getNamedItem("nevpoint");
 if(! e.isNull() )   { a->nevpoint = atoi(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("latstart");
 if(! e.isNull() )   { a->latstart= atoi(e.getNodeValue().transcode());  } 
e = attributes.getNamedItem("latend");
 if(! e.isNull() )   { a->latend= atoi(e.getNodeValue().transcode());  } 
e = attributes.getNamedItem("tbad");
 if(! e.isNull() )   { a->tbad= atof(e.getNodeValue().transcode());  } 

 fLatRunDescriptorMap[temp]= *a;
 delete a;
   nLatRunDescriptors++ ; 
  return 0;
}

int  SetParser::LedRunDescriptorParser(DOM_Node &ei)
{
  DOM_NamedNodeMap attributes = ei.getAttributes();
  //int attrCount = attributes.getLength();  // why ?

  string nameid;
  char temp[30];
  DOM_Node e = attributes.getNamedItem("name");
  
  sscanf(e.getNodeValue().transcode(),"%s",temp  );
  nameid=temp;
  LedRunDescriptor* a = new LedRunDescriptor();

 e = attributes.getNamedItem("nevpoint");
 if(! e.isNull() )   { a->nevpoint = atoi(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("lat");
 if(! e.isNull() )   { a->lat= atoi(e.getNodeValue().transcode());  } 
 e = attributes.getNamedItem("calamp");
 if(! e.isNull() )   { a->calamp = atoi(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("group");
 if(! e.isNull() )   { a->group = atoi(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("tbad");
 if(! e.isNull() )   { a->tbad = atof(e.getNodeValue().transcode());  }

 fLedRunDescriptorMap[temp]= *a;
 delete a;
   nLedRunDescriptors++ ; 
  return 0;
}

int  SetParser::ConnectivityRunDescriptorParser(DOM_Node &ei)
{
 DT_NOTE(" ConnectivityRun ");
  DOM_NamedNodeMap attributes = ei.getAttributes();
  //int attrCount = attributes.getLength();  // why ?

  string nameid;
  char temp[30];
  DOM_Node e = attributes.getNamedItem("name");
  
  sscanf(e.getNodeValue().transcode(),"%s",temp  );
  nameid=temp;
  ConnectivityRunDescriptor* a = new ConnectivityRunDescriptor();

  e = attributes.getNamedItem("nevpoint");
  if(! e.isNull() )   { a->nevpoints = atoi(e.getNodeValue().transcode());  }
  e = attributes.getNamedItem("noisedifference");
  if(! e.isNull() )   { a->noisedifference= atof(e.getNodeValue().transcode());  } 
  e = attributes.getNamedItem("lowtestvoltage");
  if(! e.isNull() )   { a->lowtestvoltage = atof(e.getNodeValue().transcode());  }
  e = attributes.getNamedItem("hightestvoltage");
  if(! e.isNull() )   { a->hightestvoltage = atof(e.getNodeValue().transcode());  }

  a->apvaddr[0]=32;a->apvaddr[1]=33;a->apvaddr[2]=34;a->apvaddr[3]=35;a->apvaddr[4]=36;
  a->apvaddr[5]=37;a->apvaddr[6]=63;
  e = attributes.getNamedItem("apvaddr");
  if(! e.isNull() )   { 
    sscanf(e.getNodeValue().transcode(),
	   "%hd %hd %hd %hd %hd %hd %hd",
	   &(a->apvaddr[0]), &(a->apvaddr[1]), &(a->apvaddr[2]),  &(a->apvaddr[3]),
	   &(a->apvaddr[4]), &(a->apvaddr[5]) ,&(a->apvaddr[6]));
  }
 
  e = attributes.getNamedItem("Mode");
  if(! e.isNull() )   { a->fMode=atoi(e.getNodeValue().transcode());  }
  e = attributes.getNamedItem("Vfp");
  if(! e.isNull() )   { a->fVfp=atoi(e.getNodeValue().transcode());  }
  e = attributes.getNamedItem("displacement");
  if(! e.isNull() )   { a->displacement=atoi(e.getNodeValue().transcode());  }
  e = attributes.getNamedItem("currentdifference");
  if(! e.isNull() )   { a->currentdifference=atoi(e.getNodeValue().transcode());  }
  fConnectivityRunDescriptorMap[temp]= *a;
  delete a;
  return 0;
}

int  SetParser::AuxChipRunDescriptorParser(DOM_Node &ei)
{
   DT_NOTE(" AuxChipRun ");
  DOM_NamedNodeMap attributes = ei.getAttributes();
  //int attrCount = attributes.getLength();  // why ?

  string nameid;
  char temp[30];
  DOM_Node e = attributes.getNamedItem("name");
  
  sscanf(e.getNodeValue().transcode(),"%s",temp  );
  nameid=temp;
  AuxChipRunDescriptor* a = new AuxChipRunDescriptor();

  e = attributes.getNamedItem("nevpoint");
  if(! e.isNull() )   { a->nevpoints = atoi(e.getNodeValue().transcode());  }
 
  fAuxChipRunDescriptorMap[temp]= *a;
  delete a;
  return 0;
}
//
int  SetParser::I2cDescriptorParser(DOM_Node &ei)
{
  DT_DEBUG_SP(" i2cDescriptorparser");
  DOM_NamedNodeMap attributes = ei.getAttributes();
  //int attrCount = attributes.getLength();  // why ?

  string nameid;
  char temp[30];
  DOM_Node e = attributes.getNamedItem("name");
  //nameid=*e.getNodeValue().transcode(); doesn't work 
  sscanf(e.getNodeValue().transcode(),"%s",temp  );
  nameid=temp;
  I2cDescriptor* a = new I2cDescriptor();
 
  e = attributes.getNamedItem("Status");
  if(! e.isNull() )   { a->fStatus=atoi(e.getNodeValue().transcode()); }
  e = attributes.getNamedItem("Address");
  if(! e.isNull() )   { a->fAddress=atoi(e.getNodeValue().transcode());  }
  e = attributes.getNamedItem("Mode");
  if(! e.isNull() )   { a->fMode=atoi(e.getNodeValue().transcode());  }
  e = attributes.getNamedItem("Latency");
  if(! e.isNull() )   { a->fLatency=atoi(e.getNodeValue().transcode());  }
  e = attributes.getNamedItem("Muxgain");
  if(! e.isNull() )   { a->fMuxgain=atoi(e.getNodeValue().transcode());  }
  e = attributes.getNamedItem("Ipre");
  if(! e.isNull() )   { a->fIpre=atoi(e.getNodeValue().transcode());  }
  e = attributes.getNamedItem("Ipcasc");
  if(! e.isNull() )   { a->fIpcasc=atoi(e.getNodeValue().transcode());  }
  e = attributes.getNamedItem("Ipsf");
  if(! e.isNull() )   { a->fIpsf=atoi(e.getNodeValue().transcode());  }
  e = attributes.getNamedItem("Isha");
  if(! e.isNull() )   { a->fIsha=atoi(e.getNodeValue().transcode());  }
  e = attributes.getNamedItem("Issf");
  if(! e.isNull() )   { a->fIssf=atoi(e.getNodeValue().transcode());  }
  e = attributes.getNamedItem("Ipsp");
  if(! e.isNull() )   { a->fIpsp=atoi(e.getNodeValue().transcode());  }
  e = attributes.getNamedItem("Imuxin");
  if(! e.isNull() )   { a->fImuxin=atoi(e.getNodeValue().transcode());  }
  e = attributes.getNamedItem("Ical");
  if(! e.isNull() )   { a->fIcal=atoi(e.getNodeValue().transcode());  }
  e = attributes.getNamedItem("Vfp");
  if(! e.isNull() )   { a->fVfp=atoi(e.getNodeValue().transcode());  }
  e = attributes.getNamedItem("Vfs");
  if(! e.isNull() )   { a->fVfs=atoi(e.getNodeValue().transcode());  }
  e = attributes.getNamedItem("Vpsp");
  if(! e.isNull() )   { a->fVpsp=atoi(e.getNodeValue().transcode());  }
  e = attributes.getNamedItem("Cdrv");
  if(! e.isNull() )   { a->fCdrv=atoi(e.getNodeValue().transcode());  }
  e = attributes.getNamedItem("Csel");
  if(! e.isNull() )   { a->fCsel=atoi(e.getNodeValue().transcode());  }
  e = attributes.getNamedItem("Addressmux");
  if(! e.isNull() )   { a->fAddressmux=atoi(e.getNodeValue().transcode());  }
  e = attributes.getNamedItem("Resistormux");
  if(! e.isNull() )   { a->fResistormux=atoi(e.getNodeValue().transcode());  }
  int initpll=0 ; // no init 
  e = attributes.getNamedItem("Initpll");
  if(! e.isNull() )   { if ( e.getNodeValue().transcode()[0]== 't' ) { initpll |= 2;}  }
  e= attributes.getNamedItem("HardReset");
  if(! e.isNull() )   {if ( e.getNodeValue().transcode()[0]== 't' ) { initpll |= 4;}   }
  e= attributes.getNamedItem("InitTtcrx");
  if(! e.isNull() )   {if ( e.getNodeValue().transcode()[0]== 't' ) { initpll |= 1;}   }
  a->fInitpll=initpll;
  e = attributes.getNamedItem("Addresspll");
  if(! e.isNull() )   { a->fAddresspll=atoi(e.getNodeValue().transcode());  }
   e = attributes.getNamedItem("Latencypll");
  if(! e.isNull() )   { a->fLatencypll=atoi(e.getNodeValue().transcode());  }
 

  fI2cDescriptorMap[temp]= *a;
 delete a;
   nI2cDescriptors++ ; 
  return 0;
}

int  SetParser::SensorDescriptorParser(DOM_Node &ei)
{
  DOM_NamedNodeMap attributes = ei.getAttributes();
  //int attrCount = attributes.getLength();  // why ?

  string nameid;
  char temp[30];
  DOM_Node e = attributes.getNamedItem("name");
  
  sscanf(e.getNodeValue().transcode(),"%s",temp  );
  nameid=temp;
  SensorDescriptor* a = new SensorDescriptor();

 e = attributes.getNamedItem("vmax");
 if(! e.isNull() )   { a->vmax = atof(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("hvrumpup");
 if(! e.isNull() )   { a->hvrumpup= atof(e.getNodeValue().transcode());  } 
 e = attributes.getNamedItem("hvrumpdwn");
 if(! e.isNull() )   { a->hvrumpdwn= atof(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("imax");
 if(! e.isNull() )   { a->imax= atof(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("imaxhw");
 if(! e.isNull() )   { a->imaxhw= atof(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("vstartiv");
 if(! e.isNull() )   { a->vstartiv= atof(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("vendiv");
 if(! e.isNull() )   { a->vendiv= atof(e.getNodeValue().transcode());  }
e = attributes.getNamedItem("npointsiv");
 if(! e.isNull() )   { a->npointsiv= atoi(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("dtiv");
 if(! e.isNull() )   { a->dtiv= atoi(e.getNodeValue().transcode());  } 
 e = attributes.getNamedItem("dimax");
 if(! e.isNull() )   { a->dimax= atoi(e.getNodeValue().transcode());  } 
 e = attributes.getNamedItem("npointsi");
 if(! e.isNull() )   { a->npointsi= atoi(e.getNodeValue().transcode());  } 
 e = attributes.getNamedItem("dv");
 if(! e.isNull() )   { a->dv= atof(e.getNodeValue().transcode());  }
  e = attributes.getNamedItem("vtest1");
 if(! e.isNull() )   { a->vtest1= atof(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("vtest2");
 if(! e.isNull() )   { a->vtest2= atof(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("vtest3");
 if(! e.isNull() )   { a->vtest3= atof(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("imax1");
 if(! e.isNull() )   { a->imax1= atof(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("imax2");
 if(! e.isNull() )   { a->imax2= atof(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("imax3");
 if(! e.isNull() )   { a->imax3= atof(e.getNodeValue().transcode());  }

 e = attributes.getNamedItem("dtvset");
 if(! e.isNull() )   { a->dtvset= atoi(e.getNodeValue().transcode());  } 
 else { a->dtvset=1;}
 fSensorDescriptorMap[temp]= *a;
 delete a;
   nSensorDescriptors++ ; 
  return 0;
}

int  SetParser::DaqCtrlDescriptorParser(DOM_Node &ei){
  DOM_NamedNodeMap attributes = ei.getAttributes();
  string nameid;
  char temp[30];
  DOM_Node e = attributes.getNamedItem("name");  
  sscanf(e.getNodeValue().transcode(),"%s",temp  );
  nameid=temp;
  DaqCtrlDescriptor* a = new DaqCtrlDescriptor();
  e = attributes.getNamedItem("tsctriggerdelay");
  if(! e.isNull() ){ a->tsctriggerdelay = atoi(e.getNodeValue().transcode());  }
  e = attributes.getNamedItem("tsccaldelay");
  if(! e.isNull() ){ a->tsccaldelay= atoi(e.getNodeValue().transcode());  }
  e = attributes.getNamedItem("tscresetdelay");
  if(! e.isNull() ){ a->tscresetdelay= atoi(e.getNodeValue().transcode());  }
  e = attributes.getNamedItem("tscfeddelay");
  if(! e.isNull() )   { a->tscfeddelay= atoi(e.getNodeValue().transcode());  }
  e = attributes.getNamedItem("spillsize");
  if(! e.isNull() )   { a->spillsize= atoi(e.getNodeValue().transcode());  }
  else { a->spillsize=100;}
 e = attributes.getNamedItem("tsctriggerfrequency");
 if(! e.isNull() ){ a->tsctriggerfrequency= atoi(e.getNodeValue().transcode());}
 else{ a->tsctriggerfrequency=17;}
  fDaqCtrlDescriptorMap[temp]= *a;
  delete a;
  nDaqCtrlDescriptors++ ; 
  return 0;
}

int  SetParser::SetupDescriptorParser(DOM_Node &ei)
{
  DOM_NamedNodeMap attributes = ei.getAttributes();
  //int attrCount = attributes.getLength();  // why ?

  string nameid;
  char temp[30];
  DOM_Node e = attributes.getNamedItem("name");
  
  sscanf(e.getNodeValue().transcode(),"%s",temp  );
  nameid=temp;
  SetupDescriptor* a = new SetupDescriptor();

 e = attributes.getNamedItem("maxtemp");
 if(! e.isNull() )   { a->maxtemp = atoi(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("dt");
  if(! e.isNull() )   { a->dt= atoi(e.getNodeValue().transcode());  }

 fSetupDescriptorMap[temp]= *a;
 delete a;
   nSetupDescriptors++ ; 
  return 0;
}

int  SetParser::ProcedureDescriptorParser(DOM_Node &ei){
  DOM_NamedNodeMap attributes = ei.getAttributes();
  //int attrCount = attributes.getLength();  // 

  string nameid;
  char temp[30];
  DOM_Node e = attributes.getNamedItem("name");
  
  sscanf(e.getNodeValue().transcode(),"%s",temp  );
  nameid=temp;
  ProcedureDescriptor* a = new ProcedureDescriptor();

 e = attributes.getNamedItem("scenariofile");
 if(! e.isNull() )   { a->setScenarioFile( e.getNodeValue().transcode()); }
 e = attributes.getNamedItem("dtslowctrl");
 if(! e.isNull() )   { a->dtslowctrl= atoi(e.getNodeValue().transcode()); }
 e = attributes.getNamedItem("updatewait");
 if(! e.isNull() )   { a->updatewait= atoi(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("updatetries");
 if(! e.isNull() )   { a->updatetries= atoi(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("wait");
 if(! e.isNull() )   { a->wait= atoi(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("resultsfile");
 if(! e.isNull() )   { a->setResultsFile( e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("xmlfile");
 if(! e.isNull() )   { a->setXmlFile( e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("maxhumidityhvset");
 if(! e.isNull() )   { a->maxhumidityhvset= atoi(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("maxhumiditycooldown");
 if(! e.isNull() )   { a->maxhumiditycooldown= atoi(e.getNodeValue().transcode());  }
e = attributes.getNamedItem("powerswitchondelay");
 if(! e.isNull() )   { a->powerswitchondelay= atoi(e.getNodeValue().transcode());  }
e = attributes.getNamedItem("temperatureaccuracy");
 if(! e.isNull() )   { a->temperatureaccuracy= atoi(e.getNodeValue().transcode());  }
 fProcedureDescriptorMap[temp]= *a;
 delete a;
   nProcedureDescriptors++ ; 
  return 0;
}

int  SetParser::FlagsDescriptorParser(DOM_Node &ei)
{
  DOM_NamedNodeMap attributes = ei.getAttributes();
  //int attrCount = attributes.getLength();  // 

  string nameid;
  char temp[30];
  DOM_Node e = attributes.getNamedItem("name");
  
  sscanf(e.getNodeValue().transcode(),"%s",temp  );
  nameid=temp;
  FlagsDescriptor* a = new FlagsDescriptor();
 e = attributes.getNamedItem("chfnoisy");
 if(! e.isNull() )   { a->chfnoisy= atoi(e.getNodeValue().transcode());  } 
 e = attributes.getNamedItem("chfopen1");
 if(! e.isNull() )   { a->chfopen1= atoi(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("chfopen2");
 if(! e.isNull() )   { a->chfopen2= atoi(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("chfopen3");
 if(! e.isNull() )   { a->chfopen3= atoi(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("chfshort");
 if(! e.isNull() )   { a->chfshort= atoi(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("chfpin1");
 if(! e.isNull() )   { a->chfpin1= atoi(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("chfpin2");
 if(! e.isNull() )   { a->chfpin2= atoi(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("chfunknown");
 if(! e.isNull() )   { a->chfunknown= atoi(e.getNodeValue().transcode());  }

 e = attributes.getNamedItem("fitestB");
 if(! e.isNull() )   { a->fitestB= atof(e.getNodeValue().transcode());  } 
 e = attributes.getNamedItem("fdcurange");
 if(! e.isNull() )   { a->fdcurange= atof(e.getNodeValue().transcode());  } 
 e = attributes.getNamedItem("fpbadchA");
 if(! e.isNull() )   { a->fpbadchA= atof(e.getNodeValue().transcode());  } 
 e = attributes.getNamedItem("fpbadchB");
 if(! e.isNull() )   { a->fpbadchB = atof(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("exitflag2 ");
 if(! e.isNull() )   { a->exitflag1 = atoi(e.getNodeValue().transcode());  }
 e = attributes.getNamedItem("exitflag2");
 if(! e.isNull() )   { a->exitflag2= atoi(e.getNodeValue().transcode());  }
 
 fFlagsDescriptorMap[temp]= *a;
 delete a;
   nFlagsDescriptors++ ; 
  return 0;
}


ModuleDescriptor*   SetParser::getModuleDescriptor(string name) {
  if(getSensorDescriptor(name)!=0) return  getSensorDescriptor(name);
  if(getI2cDescriptor(name)!=0) return  getI2cDescriptor(name);
  if( getPedRunDescriptor(name)!=0) return  getPedRunDescriptor(name);
  if(getLatRunDescriptor(name)!=0) return  getLatRunDescriptor(name);
  if(getCalRunDescriptor(name)!=0) return  getCalRunDescriptor(name);
  if(getLightRunDescriptor(name)!=0) return  getLightRunDescriptor(name);
  if(getCalProfRunDescriptor(name)!=0) return  getCalProfRunDescriptor(name);
  if(getBPlaneRunDescriptor(name)!=0) return  getBPlaneRunDescriptor(name);
  if(getLedRunDescriptor(name)!=0) return  getLedRunDescriptor(name);
  return 0;
}
