#include "adcchanneldummy.h"
#include "temperaturechannel.h"
#include "currentchannel.h"
#include "humiditychannel.h"
#include "hvchanneldummy.h"
#include "tempctrldummy.h" 
#include "dimsensserver.h"
#include "ltparser.h"

LtParser* ltparser;  

int main(){ 
  // very usefull multipurpose device 
  AdcChannelInterface* fADC=
		       new AdcChannelDummy(0 ,2);
  ltparser = new LtParser();     
  if(ltparser->parseFile()) {
    DAQEXIT("error in lt configuration file look at error output");}   
  SlowCtrlConfig* thescc= ltparser->getSlowCtrl(1);
  int theNtemp=thescc->getNrTempProbes(); 
  int theNhum=thescc->getNrHumidityProbes(); 
  DT_DEBUG_LTAPPL(" Setup: Ntemp "<<theNtemp<<" Nhum "<<theNhum);
  vector<TemperatureChannelInterface*> mtempchvec;
  // the temperature channels 
   for(int itemp=0;itemp<theNtemp;itemp++) { 
     DT_DEBUG_LTAPPL("Temp " <<  itemp <<" Id "<<  thescc->getTempProbeId(itemp));
     CalExchange <TempConverDef::TempCalValues>* c=
        c = ltparser-> getTemperatureCalibration(thescc->getTempProbeId(itemp));
     if (c)     {  
       mtempchvec.push_back( new TemperatureChannel( fADC , c )); 
     }
   }
   // the humidity channels 
   vector<HumidityChannelInterface*> mhumchvec;
   for(int ihum=0;ihum<theNhum;ihum++) {
     DT_DEBUG_LTAPPL("Hhm " <<  ihum <<" Id "<<  thescc->getHumidityProbeId(ihum)) ;
     CalExchange <HumidityConversionDef::HumidityCalValues>*
       c = ltparser-> getHumidityCalibration(thescc->getHumidityProbeId(ihum) );
     if (c)  { mhumchvec.push_back(new HumidityChannel( fADC , fADC, c )); }
   }
   // the temperature controls for the cold box 
   vector<TempCtrlInterface*> mtempctrlvec;
   CalExchange <BoxCharacteristicsDef::BoxCharValues>*     
          c = ltparser-> getColdBox(thescc->getColdBoxId(0));
   if(c){ 
     cout <<c;
     cout <<endl;
     vector<int>* m= &(c->getCalValues()->fTempCtrlId);
     for (  vector<int>::const_iterator p=m->begin();p!=m->end();p++){
       CalExchange <BoxCharacteristicsDef::BoxCharValues>* c_tc=
	 ltparser->getTempCtrlInfo(*p);  
       cout << c_tc; 
       cout << endl;
          mtempctrlvec.push_back( new TempCtrlDummy(*p));
     }
   }
   // for the moment only the current channels for the remote hv 
   // also do the hv connections because of the similarity 
   // check for all (max 10 bench positions ) 
   vector<HVChannelInterface*> mhvunit;
   vector<CurrentChannelInterface*> curchvec;
   for (int slotpos =0 ; slotpos < 10 ; slotpos++) {  
     BenchPosConnection* BenchP= 
       new BenchPosConnection(*ltparser->getBenchPosConnections(slotpos));
     if ( BenchP->getId() == -1) continue ;
     cout << "info found for slot pos "<<slotpos <<" Id " <<  BenchP->getId()<< endl;
     //else create the current bias voltage channel and bias current 
     CalExchange <HvConversionDef::HvCalValues>* cv=
       ltparser-> getHvCalibration(BenchP->getBiasDevice());
     if(cv){
       cout << cv;
       cout <<"hv"<<endl;
       mhvunit.push_back(new HvChannelDummy(cv->getBoardNr(), cv->getChannelNr()));
     }
     CalExchange <CurrentConvDef::CurrentCalValues>* c=
       c = ltparser-> getCurrentCalibration( BenchP->getHvCurrentDevice( ));
     if ( c) {  
       cout << c;
       cout <<"c"<<endl;
       curchvec.push_back( new CurrentChannel( fADC , c ));
     }
   }
   
  DimSensServer dss(0, mtempchvec , mhumchvec , mtempctrlvec, mhvunit,curchvec);
  
printf("dim sens init done\n");
if( dss.DimServerNameNotValid()){return -1;}
while(1)sleep(5);
return 0;
}
