#ifndef RecModHeader_h_
#define RecModHeader_h_

#include <map>
#include <vector>

#include <TVector.h>
#include <TGraph.h>
#include <TProfile.h>
#include <TH1.h>
#include <TFile.h>
#include <TDirectory.h>


/** 
\brief 
<p>     RecordModuleHeadrer for the Record
        Dinamicaly store data for the tested module
        
               
     \author V.Zhukov  
     \version 1.2
     \date 2003/09
*/


class ModId;
class QString;

class RecModHeader {
 private:


Int_t napvs;
Int_t nbench;

Int_t statApv;
Int_t statMux;
Int_t statPll;
Int_t statDcu;
Int_t statMod;
Int_t statSensor; 
Int_t nbadch;
Int_t napverrors;


TH1F* hchflag;

std::vector<int> vbadchlist;
std::map<int, int, std::less<int> >  mbadch;

std::map<int, int, std::less<int> >  mbadchpinvon;
std::map<int, int, std::less<int> >  mbadchpinvoff;
std::map<int, int, std::less<int> >  mbadchdinvon;
std::map<int, int, std::less<int> >  mbadchdinvoff;
std::vector<float>  vnbadcht;



 Int_t val;  // validation flag for the test

char modid[80];
char modtype[80];
char dcuid[80]; 

void bookhist();
  
 public:
RecModHeader(int nbench, ModId* id, int napvs);
RecModHeader(int nbench, int napvs);
virtual ~RecModHeader();

void Save(TDirectory* dir);
bool Read(TDirectory* dir);
void Reset();
void ResetXt();
Int_t AddMeast(Float_t t);
void ResetFlags();

void setmodid(char* i) { strcpy(modid,i); }
void setmodtype(char* i) { strcpy(modtype,i); }
void setnbench(int i) {nbench=i;}
void setdcuid(unsigned int i) { sprintf(dcuid,"%d",i); }
void setTHchflag(TH1F* h)     {hchflag=h;}

void setstatAPV(Int_t n)   { statApv=n;}
void setstatPLL(Int_t n)   {statPll=n;}
void setstatMUX(Int_t n)   {statMux=n;}
void setstatDCU(Int_t n)   {statDcu=n;}
void setstatSensor(Int_t n)  {statSensor=n;}
void setnbadch(int i) {nbadch=i;}
void setval(int i) {val=i;}
void setnapverrors(int i) {napverrors=i;}
void setbadchpinvon(const std::map<int, int, std::less<int> >& m) {mbadchpinvon=m;}
void setbadchpinvoff(const std::map<int, int, std::less<int> >& m) {mbadchpinvoff=m;}
void setbadchdinvon(const std::map<int, int, std::less<int> >& m) {mbadchdinvon=m;}
void setbadchdinvoff(const std::map<int, int, std::less<int> >& m) {mbadchdinvoff=m;}
void setbadchm(const std::map<int, int, std::less<int> >& m) {mbadch=m;}
void setbadchmap(const QString& s,const  std::map<int, int, std::less<int> >& m );

 const char*  getmodid() const { return modid; }
 const char* getmodtype() const { return modtype; }
 const char* getdcuid() { return dcuid; }
 Int_t getnapvs()  const {return  napvs;}
 Int_t getnbench()  const {return  nbench;}
 Int_t getnbadch() const {return nbadch;}
 Int_t getnapverrors() const {return napverrors;}
 std::map<int, int, std::less<int> > getbadchm()  const {return mbadch;}
 std::map<int, int, std::less<int> > getbadchpinvon() const {return mbadchpinvon;}
 std::map<int, int, std::less<int> > getbadchpinvoff() const {return mbadchpinvoff;} 
 std::map<int, int, std::less<int> > getbadchdinvon() const {return mbadchdinvon;}
 std::map<int, int, std::less<int> > getbadchdinvoff() const {return mbadchdinvoff;}
 std::map<int, int, std::less<int> > getbadchmap(const QString& s);
 std::vector<float>* nbadcht() {return  &vnbadcht;}

 Int_t getstatAPV()  const {return statApv;}
 Int_t getstatPLL()  const {return statPll;}
 Int_t getstatMUX()  const {return statMux;}
 Int_t getstatDCU()  const {return statDcu;}
 Int_t getstatSensor() const {return statSensor;}
 Int_t getval() const {return val;}
 TH1F* THchflag() {return hchflag;} 


};
#endif
