#ifndef RecModTest_h_
#define RecModTest_h_

#include <vector>
#include <map>
#include <TGraph.h>
#include <TProfile.h>
#include <TH1.h>
#include <TH2.h>
#include <TFile.h>
#include <TDirectory.h>


/** 
\brief 
<p>     RecordModuleTest for the Record
        Dynamically stores data for the tested module
        book histos on demand.
               
     \author V.Zhukov  
     \version 1.2
     \date 2003/09
*/

class Stat;

class RecModTest {
 private:

 Int_t apvmped;
 Int_t apvmcal;
 Int_t apvmcalprof;
 Int_t apvmlight;
 Int_t apvmlat;
 Int_t apvmbp;
 Int_t apvmfb;
 Int_t apvmode;

  Float_t avgpedestal;
  Float_t avgnoise;
  Float_t avgrawnoise;
  Float_t avgcalA;
  Float_t avgcalAlight;
  Float_t avgcalPApeak;
  Float_t avgcalPTrise;
  Float_t avgcalBP;
  Float_t avgcalAled;
  Float_t avgcalAfb;
  Float_t sgpedestal;
  Float_t sgnoise;
  Float_t sgrawnoise;
  Float_t sgcalA;
  Float_t sgcalAlight;
  Float_t sgcalPApeak;
 Float_t  sgcalPTrise;
  Float_t sgcalBP;
  Float_t sgcalAled;
  Float_t sgcalAfb;

TH1F* hpedestal;
TH1F* hnoise;
TH1F* hrawnoise;
TH1F* hcalA;
TH1F* hcalPApeak;
TH1F* hcalSbackpl;
TH1F* hcalSled;
TH1F* hcalAlight;
TH1F* hcalAfb;

TH1F* hcalPTrise;
TH1F* hcalPTfall;


TProfile* pbp;
TProfile* pled;

TH2F* h2led;

 Float_t itest1;  // nA
 Float_t itest2;   //nA
 Float_t itest3;   //nA

 Float_t vtest1;  
 Float_t vtest2;   
 Float_t vtest3;   



std::vector<TH1F*> vCmn;
std::vector<TH1F*> vCalProf;
std::vector<TProfile*> vLat;
std::vector<TH1F*> vPipe;

std::vector<float>  vavrmst;
std::vector<float>  vavcalat;
std::vector<float>  vvtimet;

std::vector<float> vivv;
std::vector<float> vivi;

std::vector<int> vcons;

std::map<int, int, std::less<int> > mbadchped;
std::map<int, int, std::less<int> >  mbadchcalprof;
std::map<int, int, std::less<int> >  mbadchcal;
std::map<int, int, std::less<int> >  mbadchfb;
std::map<int, int, std::less<int> >  mbadchlight;
std::map<int, int, std::less<int> >  mbadchlat;
std::map<int, int, std::less<int> >  mbadchbp;


Stat* cstat;

int napvs;
int nbench;

  
void bookhistped();
void bookhistlat();
void bookhistcalprof();
void bookhistled();
void bookhistbp();
void bookhistcal();
void bookhistlight();
void bookhistfb();

 public:
RecModTest(int nbench, int napvs);
virtual ~RecModTest();

void Save(TDirectory* dir);
bool Read(TDirectory* dir);
void Reset();



 void ResetPed();
 void ResetCalProf();
 void ResetLat();
 void ResetLed();
 void ResetCal();
 void ResetLight();
 void ResetFb();
 void ResetBp();
 void ResetIV();
 void ResetXt();

 void setavgpedestal(Float_t f)   {avgpedestal=f; } 
 void setavgnoise(Float_t f)      {avgnoise=f; } 
 void setavgrawnoise(Float_t f)   {avgrawnoise=f;}
 void setavgcalA(Float_t f)        {avgcalA=f;}
 void setavgcalAlight(Float_t f)  {avgcalAlight=f;}
 void setavgcalPApeak(Float_t f)  {avgcalPApeak=f;}
 void setavgcalPTrise(Float_t f)  {avgcalPTrise=f;}
 void setavgcalBP(Float_t f)      {avgcalBP=f;}
 void setavgcalAled(Float_t f)    {avgcalAled=f;}
 void setavgcalAfb(Float_t f)    {avgcalAfb=f;}
 void setsgpedestal(Float_t f)    {sgpedestal=f;}
 void setsgnoise(Float_t f)       {sgnoise=f;}
 void setsgrawnoise(Float_t f)    {sgrawnoise=f;}
 void setsgcalAlight(Float_t f)   {sgcalAlight=f;}
 void setsgcalA(Float_t f)         {sgcalA=f;}
 void setsgcalPApeak(Float_t f)   {sgcalPApeak=f;}
 void setsgcalPTrise(Float_t f)   {sgcalPTrise=f;}
 void setsgcalBP(Float_t f)       {sgcalBP=f;}
 void setsgcalAled(Float_t f)     {sgcalAled=f;}
 void setsgcalAfb(Float_t f)     {sgcalAfb=f;}
 void setconnectivity(std::vector<int> cons) { vcons=cons; } 

void setTHpedestal(TH1F* h)    {hpedestal=h;}
void setTHnoise(TH1F* h)       {hnoise=h;}
void setTHrawnoise(TH1F* h)   {hrawnoise=h;}


void setTHcalA(TH1F* h)       {hcalA=h;}
void setTHcalPApeak(TH1F* h)  {hcalPApeak=h;}
void setTHcalSbackpl(TH1F* h) { hcalSbackpl=h;}
void setTHcalSled(TH1F* h)    { hcalSled=h;}
void setTHcalAlight(TH1F* h)    {hcalAlight=h;}
void setTHcalAfb(TH1F* h)    {hcalAfb=h;}

TH1F* hcmn(int i);
TH1F* hcalprof(int i);
TH1F* hpipe(int i);
TProfile* hplatency(int i);


TH1F* THpedestal() {return hpedestal;} 
TH1F* THnoise() {return hnoise;} 
TH1F* THrawnoise() {return hrawnoise;}
TH1F* THcalA();
TH1F* THcalAlight();
TH1F* THcalAfb();
TH1F* THcalPApeak(); 
TH1F* THcalSbackpl();
TH1F*  THcalSled();
TProfile* TPbp();
TProfile* TPled();
TH2F* TH2calSled();

TH1F* THcalPTrise();
TH1F* THcalPTfall(); 

TGraph* TGiv();



std::vector<float>*  avrmst() {return &vavrmst;}
std::vector<float>*  avcalat() {return &vavcalat;}
std::vector<float>*  vtimet()    {return &vvtimet;}
std::vector<float>*  TVivi() {return &vivi;}
std::vector<float>*  TVivv() {return &vivv;}
std::vector<int>* connectivity() {return &vcons;}
Int_t AddMeasIV(Float_t i, Float_t v);
Int_t AddMeast(Float_t t);


  Float_t getavgpedestal()   const     {return avgpedestal; } 
  Float_t getavgnoise()      const        {return avgnoise; } 
  Float_t getavgrawnoise()   const      {return avgrawnoise;}
  Float_t getavgcalAlight()  const    {return avgcalAlight; }
  Float_t getavgcalA()       const    {return avgcalA; }
  Float_t getavgcalPApeak()  const     {return avgcalPApeak;}
  Float_t getavgcalBP()      const         {return avgcalBP;}
  Float_t getavgcalAled()    const       {return avgcalAled;}
  Float_t getavgcalAfb()    const       {return avgcalAfb;}
  Float_t getsgpedestal()    const       {return sgpedestal;}
  Float_t getsgnoise()       const          {return sgnoise;}
  Float_t getsgrawnoise()    const       {return sgrawnoise;}
  Float_t getsgcalAlight()   const      {return sgcalAlight;}
  Float_t getsgcalA()       const      {return sgcalA;}
  Float_t getsgcalPApeak()   const      {return sgcalPApeak;}
  Float_t getsgcalBP()       const          {return sgcalBP;}
  Float_t getsgcalAled()     const        {return sgcalAled;}
  Float_t getsgcalAfb()     const        {return sgcalAfb;}

Stat* bstat() {return cstat;}

void setapvmped(Int_t m) {apvmped=m;}
 void setapvmcal(Int_t m) {apvmcal=m;}
 void setapvmlat(Int_t m) {apvmlat=m;}
 void setapvmcalprof(Int_t m) {apvmcalprof=m;}
 void setapvmlight(Int_t m) {apvmlight=m;}
 void setapvmbp(Int_t m) {apvmbp=m;}
 void setapvmfb(Int_t m) {apvmfb=m;}
 void setapvmode(Int_t m) {apvmode=m;}

void setbadchped(const std::map<int, int, std::less<int> >& m) {mbadchped=m;}
void setbadchcalprof(const std::map<int, int, std::less<int> >& m) {mbadchcalprof=m;}
void setbadchcal(const std::map<int, int, std::less<int> >& m) {mbadchcal=m;}
void setbadchdlat(const std::map<int, int, std::less<int> >& m) {mbadchlat=m;}
void setbadchfb(const std::map<int, int, std::less<int> >& m) {mbadchfb=m;}
void setbadchlight(const std::map<int, int, std::less<int> >& m) {mbadchlight=m;}
void setbadchbp(const std::map<int, int, std::less<int> >& m) {mbadchbp=m;}

 Int_t getapvmod()  const {return apvmode;}
 Int_t getapvmped()  const {return apvmped;}
 Int_t getapvmcal()  const {return apvmcal;}
 Int_t getapvmlat()  const {return apvmlat;}
 Int_t getapvmcalprof()  const {return apvmcalprof;}
 Int_t getapvmlight()  const {return apvmlight;}
 Int_t getapvmbp()  const {return apvmbp;}
 Int_t getapvmfb()  const {return apvmfb;}

 std::map<int, int, std::less<int> > getbadchped() const {return mbadchped;}
 std::map<int, int, std::less<int> > getbadchcalprof() const {return mbadchcalprof;} 
 std::map<int, int, std::less<int> > getbadchcal() const {return mbadchcal;}
 std::map<int, int, std::less<int> > getbadchfb() const {return mbadchfb;}
 std::map<int, int, std::less<int> > getbadchlight()const {return mbadchlight;}
 std::map<int, int, std::less<int> > getbadchlat()const {return mbadchlat;}
 std::map<int, int, std::less<int> > getbadchbp() const {return mbadchbp;}


void setitest1(Float_t f) {itest1=f;}
void setitest2(Float_t f) {itest2=f;}
void setitest3(Float_t f) {itest3=f;}
void setvtest1(Float_t f) {vtest1=f;}
void setvtest2(Float_t f) {vtest2=f;}
void setvtest3(Float_t f) {vtest3=f;}
Float_t getitest1() const {return itest1;}
Float_t getitest2() const  {return itest2;}
Float_t getitest3() const  {return itest3;}
Float_t getvtest1() const {return vtest1;}
Float_t getvtest2() const  {return vtest2;}
Float_t getvtest3() const  {return vtest3;}

};


class Stat {
 public:
Stat() {;}
void reset();
bool bcalprof;
bool blat;
bool bpipe;
bool bled;
bool bbp;
bool bcal;
bool blight; 
bool bped; 
bool bfb;
bool biv;

};

#endif
