#ifndef RecSetup_h_
#define RecSetup_h_

#include <TObject.h>
#include <TDirectory.h>
#include <iostream>
#include <TGraph.h>

#include <vector>
#include <map>

/** \brief Record Setup for the Record
           Dinamicaly store data related to the test setup


     \author V.Zhukov,KA   
     \version 1.2 
     \date 2003/09
 
*/



class RecSetup  {

 private:
 Float_t temptag;
 Float_t tempcold;
 Float_t tempbox;
 Float_t tempwarm; 
 Float_t tempext;
 Float_t humext;
 Float_t humbox;
 Int_t ntemp;
 Int_t nhum;
 std::vector<Float_t> temps;
 std::vector<Float_t> hums;

 std::vector<Float_t> vtemptagt;
 std::vector<Float_t> vtempcoldt;
 std::vector<Float_t> vtempwarmt;
 std::vector<Float_t> vtempboxt;
 std::vector<Float_t> vtempextt;
 std::vector<Float_t> vhumboxt;
 std::vector<Float_t> vhumextt;
 std::map<Int_t, std::vector<Float_t> > tempst ;
 std::map<Int_t, std::vector<Float_t> > humst ; 
 std::vector<Float_t> vtime;

 Int_t Nmeas;
 
 public:
RecSetup(int ntemp, int nhum);
virtual ~RecSetup();
void Reset();
void Save(TDirectory* dir);
//
 void settemptag(Float_t f) {temptag=f;}
 void settempcold(Float_t f) {tempcold=f;}
 void settempbox(Float_t f) {tempbox=f;}
 void settempext(Float_t f) {tempext=f;}
 void settempwarm(Float_t f) {tempwarm=f;}
 void sethumbox(Float_t f) {humbox=f;}
 void sethumext(Float_t f) {humext=f;}
 void sethums(Int_t i, Float_t f) {hums[i]=f;}
 void settemps(Int_t i, Float_t f) {temps[i]=f;}

 Float_t gettemptag()  const {return temptag;}
 Float_t gettempcold() const {return tempcold;}
 Float_t gettempbox() const {return tempbox;}
 Float_t gettempext() const {return tempext;}
 Float_t gettempwarm() const {return tempwarm;}
 Float_t gethumbox() const {return humbox;}
 Float_t gethumext() const {return humext;}
std::vector<Float_t>* gettemps()  {return &temps;}
std::vector<Float_t>* gethums() {return &hums;}

std::vector<Float_t>* gettemptagt()  {return &vtemptagt;}
std::vector<Float_t>* gettempcoldt()  {return &vtempcoldt;}
std::vector<Float_t>* gettempwarmt()  {return &vtempwarmt;}
std::vector<Float_t>* gettempboxt()  {return &vtempboxt;}
std::vector<Float_t>* gettempextt()  {return &vtempextt;}

std::vector<Float_t>* gethumboxt()  {return &vhumboxt;}
std::vector<Float_t>* gethumextt()  {return &vhumextt;}
std::vector<Float_t>* gettime()  {return &vtime;}

std::vector<Float_t>* gettempst(Int_t i) 
{ if(i < (int)tempst.size() && i>=0 ) return &tempst[i];
 else { std::cout<<" tempst out of range "<<std::endl; 
return NULL;}}

std::vector<Float_t>* gethumst(Int_t i)  
{ if(i<(int)humst.size() && i>=0 ) return &humst[i];
 else { std::cout<<" humst out of range "<<std::endl;
return NULL;} }

Int_t AddMeast(Float_t time);
Int_t nmeas() const {return Nmeas;}

 
TGraph* TGtemptagt();
TGraph* TGtempcoldt();
TGraph* TGtempwarmt();
TGraph* TGtempboxt();
TGraph* TGhumboxt();
TGraph* TGhumextt();
TGraph* TGtempextt();


};

#endif
