#ifndef _Record_H_
#define _Record_H_


#include <vector>
#include <string> 
#include <TObject.h>
#include <TH1.h>
#include <TGraph.h>
#include <TMath.h>
#include <TFile.h>
#include <TVector.h>
#include <TNamed.h>
#include <TDirectory.h>



/** 
\brief 
<p>     Record: data storage for the Module test
        consists of 3 sections: header, setup and
        vector<RecModule*> record for each module.
        Provides method Save(name) information
        related to each module(+setup+header similar for each module)
        into separate file modid+name 
               
     \author V.Zhukov  
     \version 1.2 
     \date 2003/09
 
*/


class ModId;
class FlagsDescriptor;
class QWidget;
class RecModule;
class RecHeader;
class RecSetup;



class Record   {
private:

RecHeader* header;
RecSetup*  recsetup;
std::vector<RecModule*> recmodules;
 /** this is needed to return a value for the getFileName in case of 
    no module name */
std::string  theNoString;
Int_t Nrec;
Int_t Nmod;
/** this is needed to return a value for the getFileName in case of 
    no module name */


 public:
Record();
virtual ~Record();
void ConstructModule(int bench, ModId* id, int napvs);
void ConstructModule(int bench, int napvs);
void ConstructSetup(int ntemp, int nhum);
void Reset();
bool Save(const char* filename,QWidget* mon);
bool Save(const char* filename);

void Analyze(FlagsDescriptor* f);
void DestructModule(int bench);
void DestructAllModules();

RecHeader* getheader() {return header;}
RecSetup* getsetup() {return recsetup;}
std::vector<RecModule*> getmodules() {return recmodules;}
RecModule* getmodule(int bench);
RecModule* getfirstmodule();
const char*  getFileName(int module);
int getNrec()  const {return Nrec;}

};
#endif
