#include "RecAnalyzer.h"
#include "RecModHeader.h"
#include "RecModTest.h"
#include "RecModSctrl.h"
#include "RecUtil.h"
#include "RecModule.h"
#include  "ModuleDescriptor.h"
#include  "FlagsDescriptor.h"
#include <vector>
#include <map>
#include <TVector.h>
#include <TGraph.h>
#include <TProfile.h>
#include <TH1.h>
#include <TFile.h>
#include <qstring.h>

#ifndef NAPVCH
#define NAPVCH  128
#endif


using namespace RecUtil;
using namespace std;

RecAnalyzer::RecAnalyzer(RecModule* r)  {
    therecmodule=r;
    nchan=NAPVCH*(r->getmodheader()->getnapvs());
 }

RecAnalyzer::~RecAnalyzer() {
}

/* fill flags in the badchmap for the test */
void RecAnalyzer::AnalyzeLight(const QString& s,ModuleDescriptor* ss) {
  map<int, int, less<int> > badchm=therecmodule->getmodheader()->getbadchmap(s); 
  map<int, int, less<int> > badchlight=therecmodule->getmodtest()->getbadchlight();
 
  LightRunDescriptor* sets=(LightRunDescriptor*) ss;
  Float_t av=0.;
  Float_t sg=0.;
  TH1F* h= therecmodule->getmodtest()->THcalAlight(); 
  average(h, badchm,av,sg);
  therecmodule->getmodtest()->setavgcalAlight(av);
  therecmodule->getmodtest()->setsgcalAlight(sg);
  Float_t thr=sets->tbad;
  Float_t amin1=sets->calLightmin1*av;
  Float_t amax1=sets->calLightmax1*av;  

  for(int i=1;i<=h->GetNbinsX();i++) {
    float x=h->GetBinContent(i);
    if(x<=amin1) maskbadmap(badchlight,i,2);
    if(x>amax1) maskbadmap(badchlight,i,4);
// relative cut
    if((av-x)>thr*sg) 
       maskbadmap(badchlight,i,256);
    if((x-av)>thr*sg) 
       maskbadmap(badchlight,i,512);  
  }
   therecmodule->getmodtest()->setbadchlight(badchlight);
  } 
void RecAnalyzer::AnalyzeFb(const QString& s,ModuleDescriptor* ss) {
  map<int, int, less<int> > badchm=therecmodule->getmodheader()->getbadchmap(s); 
  map<int, int, less<int> > badchfb;

  FbRunDescriptor* sets=(FbRunDescriptor*) ss;
  Float_t av=0.;
  Float_t sg=0.;
  TH1F* h= therecmodule->getmodtest()->THcalAfb(); 
  average(h, badchm,av,sg);
  therecmodule->getmodtest()->setavgcalAfb(av);
  therecmodule->getmodtest()->setsgcalAfb(sg);
  Float_t thr=sets->tbad;
  Float_t amin1=sets->calFbmin1*av;
  Float_t amax1=sets->calFbmax1*av;  
 
  for(int i=1;i<=h->GetNbinsX();i++) {
    float x=h->GetBinContent(i);
    if(x<=amin1)  maskbadmap(badchfb,i,2);
    if(x>amax1)  maskbadmap(badchfb,i,4);
// relative
    if((av-x)>thr*sg) 
      maskbadmap(badchfb,i,256);
    if((x-av)>thr*sg ) 
      maskbadmap(badchfb,i,512);
  }

  therecmodule->getmodtest()->setbadchfb(badchfb);
  } 
//
void RecAnalyzer::AnalyzeLat(const QString& s,ModuleDescriptor* ss) {

}
//
void RecAnalyzer::AnalyzeIV(const QString& s,ModuleDescriptor* ss) {
 SensorDescriptor* sets=(SensorDescriptor*) ss;
// analyse itest
 float temp1= therecmodule->getmodsctrl()->gettempsenntsc1(); 
 float  temp2= therecmodule->getmodsctrl()->gettempsenntsc2();
  int statSensor=therecmodule->getmodheader()->getstatSensor();
  float imax1=sets->imax1; // in nA
  float imax2=sets->imax2;
  float imax3=sets->imax3;
  float itest1=therecmodule->getmodtest()->getitest1();
  float itest2=therecmodule->getmodtest()->getitest2();
  float itest3=therecmodule->getmodtest()->getitest3();
  float dV=2.; // maximum deviation for measured voltages
  bool bt1=fabs(sets->vtest1-therecmodule->getmodtest()->getvtest1())<dV&&correctileak(temp1,itest1)<imax1;  
  bool bt2=fabs(sets->vtest2-therecmodule->getmodtest()->getvtest2())<dV&&correctileak(temp1,itest2)<imax2;
  bool bt3=fabs(sets->vtest3-therecmodule->getmodtest()->getvtest3())<dV&&correctileak(temp1,itest3)<imax3;
 
  if(bt1) statSensor|=2;
  if(bt2) statSensor|=4;
  if(bt3) statSensor|=8;
  if(fabs(sets->vtest3-therecmodule->getmodtest()->getvtest3())>dV) statSensor|=16;
  therecmodule->getmodheader()->setstatSensor(statSensor);
  therecmodule->getmodtest()->bstat()->biv=true;
}


//
void RecAnalyzer::AnalyzePed(const QString& s, ModuleDescriptor* ss) {
  map<int, int, less<int> > badchm=therecmodule->getmodheader()->getbadchmap(s); 
  map<int, int, less<int> > badchped=therecmodule->getmodtest()->getbadchped();

   PedRunDescriptor* sets=(PedRunDescriptor*) ss; 
  Float_t av=0.;
  Float_t sg=0.;
  TH1F* hped= therecmodule->getmodtest()->THpedestal(); 
  average(hped, badchm,av,sg);
  therecmodule->getmodtest()->setavgpedestal(av);
  therecmodule->getmodtest()->setsgpedestal(sg);

//rnoise
      av=0.;
      sg=0.;
 TH1F* hrn=therecmodule->getmodtest()->THrawnoise(); 
  average(hrn, badchm,av,sg);
 therecmodule->getmodtest()->setavgrawnoise(av);
 therecmodule->getmodtest()->setsgrawnoise(sg);
//noise 
      av=0.;
      sg=0.;
 TH1F* hn=therecmodule->getmodtest()->THnoise(); 
  average(hn, badchm, av,sg);
  therecmodule->getmodtest()->setavgnoise(av);
  therecmodule->getmodtest()->setsgnoise(sg);
   Float_t thr=sets->tbad;
   Float_t amin1=sets->rmsmin1;;
   Float_t amin2=sets->rmsmin2;;
   Float_t amax2=sets->rmsmax2;; 
   Float_t amin3=sets->rmsmin3;;
   Float_t amax3=sets->rmsmax3;; 
   Float_t amax4=sets->rmsmax4;;
 
  for(int i=1;i<=hn->GetNbinsX();i++)  {
    float x=hn->GetBinContent(i);
    if(x<=amin1)  maskbadmap(badchped,i,2);
   if(x>amin2&&x<=amax2)  maskbadmap(badchped,i,4);
   if(x>amin3&&x<=amax3)  maskbadmap(badchped,i,8);
   if(x>amax4) maskbadmap(badchped,i,16);
// relative cut 
  if((av-x)>thr*sg ) 
       maskbadmap(badchped,i,256);
  if((x-av)>thr*sg ) 
       maskbadmap(badchped,i,512);
  }
//pedrange 
  for(int i=1;i<=hped->GetNbinsX();i++)     {
    float x=hped->GetBinContent(i);
    if(x>sets->pedmax)    maskbadmap(badchped,i,32);        
    if(x<=sets->pedmax)  maskbadmap(badchped,i,64);            
     }  
  therecmodule->getmodtest()->setbadchped(badchped);
}
//
void RecAnalyzer::AnalyzeBP(const QString& s,ModuleDescriptor* ss) {
  map<int, int, less<int> > badchm=therecmodule->getmodheader()->getbadchmap(s); 
  map<int, int, less<int> > badchbp=therecmodule->getmodtest()->getbadchbp();
  BPlaneRunDescriptor* sets=(BPlaneRunDescriptor*) ss;
  Float_t av=0.;
  Float_t sg=0.;
  TH1F* h=therecmodule->getmodtest()->THcalSbackpl(); 
  average(h,badchm,av,sg);
 
  therecmodule->getmodtest()->setavgcalBP(av);
  therecmodule->getmodtest()->setsgcalBP(sg);
  Float_t thr=sets->tbad;
  Float_t amin1=sets->calBpmin1*av;
  Float_t amax1=sets->calBpmax1*av;  
 
  for(int i=1;i<=h->GetNbinsX();i++) {
    float x=h->GetBinContent(i);
    if(x<=amin1) maskbadmap(badchbp,i,2);
    if(x>amax1) maskbadmap(badchbp,i,4);

//relative
    if((av-x)>thr*sg) 
            maskbadmap(badchbp,i,256);
   if((x-av)>thr*sg) 
            maskbadmap(badchbp,i,512);
  }
   therecmodule->getmodtest()->setbadchbp(badchbp);
} 
//
void RecAnalyzer::AnalyzeCal(const QString& s,ModuleDescriptor* ss) {
  map<int, int, less<int> > badchm=therecmodule->getmodheader()->getbadchmap(s); 
  map<int, int, less<int> > badchcal=therecmodule->getmodtest()->getbadchcal();

  CalRunDescriptor* sets=(CalRunDescriptor*) ss;
  Float_t av=0.;
  Float_t sg=0.;
  TH1F* h= therecmodule->getmodtest()->THcalA(); 
  average(h, badchm,av,sg);
  therecmodule->getmodtest()->setavgcalA(av);
  therecmodule->getmodtest()->setsgcalA(sg);
  Float_t thr=sets->tbad;
  Float_t amin1=sets->calAmin1*av;
  Float_t amax1=sets->calAmax1*av;  
 
  
  for(int i=1;i<=h->GetNbinsX();i++) {
    float x=h->GetBinContent(i);
    if(x<=amin1) maskbadmap(badchcal,i,2);
    if(x>amax1)  maskbadmap(badchcal,i,4);
// relative
    if((x-av)>thr*sg || (av-x)>thr*sg)
        maskbadmap(badchcal,i,256);
    if((x-av)>thr*sg)
        maskbadmap(badchcal,i,512);
  }     
  therecmodule->getmodtest()->setbadchcal(badchcal);
  } 
//
//
void RecAnalyzer::AnalyzeLed(const QString& s,ModuleDescriptor* ss) {
  map<int, int, less<int> > badchm=therecmodule->getmodheader()->getbadchmap(s); 
map<int, int, less<int> > badchled;
  LedRunDescriptor* sets=(LedRunDescriptor*) ss;
  Float_t av=0.;
  Float_t sg=0.;
  TH1F* h= therecmodule->getmodtest()->THcalSled(); 
  average(h, badchm,av,sg);
  therecmodule->getmodtest()->setavgcalAled(av);
  therecmodule->getmodtest()->setsgcalAled(sg);
  Float_t thr=sets->tbad;

  for(int i=1;i<=h->GetNbinsX();i++) {
    float x=h->GetBinContent(i);
    if((av-x)>=thr*sg) 
      maskbadmap(badchled,i,256);
    if((x-av)>thr*sg ) 
      maskbadmap(badchled,i,512);
  }
// not used 
  } 
//
void RecAnalyzer::AnalyzeCalProf(const QString& s, ModuleDescriptor* ss) {
  int Napvs=therecmodule->getmodheader()->getnapvs();
  int nch=Napvs*NAPVCH;
  TH1F* hcala=therecmodule->getmodtest()->THcalPApeak();
  TH1F* hcaltr=therecmodule->getmodtest()->THcalPTrise();
  TH1F* hcaltfall=therecmodule->getmodtest()->THcalPTfall();
  for(int ch=0;ch<Napvs;ch++) {
    for(int is=0;is<NAPVCH;is++) {
      TH1F* hp=therecmodule->getmodtest()->hcalprof(ch*NAPVCH+is);
      
//fill hist (can do fit here)
      float xmax=hp->GetMaximum();
      float xmin=hp->GetMinimum();
      float xtmx=hp->GetMaximumBin();
      float xtmn=hp->GetMinimumBin();
      float dtrise=xtmx;
      hcala->SetBinContent(ch*NAPVCH+is+1,xmax);
      hcaltr->SetBinContent(ch*NAPVCH+is+1,dtrise);
    }
  }
// flag bad ch
 map<int, int, less<int> > badchm=therecmodule->getmodheader()->getbadchmap(s); 
 map<int, int, less<int> > badchcalprof=therecmodule->getmodtest()->getbadchcalprof();

  CalProfRunDescriptor* sets=(CalProfRunDescriptor*) ss;
//analyse amplitude
  Float_t av=0.;
  Float_t sg=0.;
  TH1F* h= therecmodule->getmodtest()->THcalPApeak();
  average(h, badchm,av,sg);
  therecmodule->getmodtest()->setavgcalPApeak(av);
  therecmodule->getmodtest()->setsgcalPApeak(sg);
  Float_t thr=sets->tbad;

  Float_t amin1=sets->calPAmin1*av;
  Float_t amax1=sets->calPAmax1*av;  
  
  for(int i=1;i<=h->GetNbinsX();i++) {
    float x=h->GetBinContent(i);

    if(x<=amin1)  maskbadmap(badchcalprof,i,2);
    if(x>amax1)  maskbadmap(badchcalprof,i,4);
// relative
    if((av-x)>thr*sg) 
       maskbadmap(badchcalprof,i,256);
    if((x-av)>thr*sg) 
       maskbadmap(badchcalprof,i,512);
  }     
// analyze peak time 
  TH1F* ht= therecmodule->getmodtest()->THcalPTrise();
  average(ht, badchm,av,sg);
  therecmodule->getmodtest()->setavgcalPTrise(av);
  therecmodule->getmodtest()->setsgcalPTrise(sg);
  Float_t avtr=av;
  Float_t tmin1=sets->calTPmin1;
  Float_t tmin2=sets->calTPmin2;
  Float_t tmax2=sets->calTPmax2;  
  Float_t tmin3=sets->calTPmin3;
  Float_t tmax3=sets->calTPmax3;
  Float_t tmax4=sets->calTPmax4;  
 
 for(int i=1;i<=ht->GetNbinsX();i++) {
    float x=ht->GetBinContent(i);
    if((x-avtr)<=tmin1)  maskbadmap(badchcalprof,i,8);
    if((x-avtr)>tmin2&&(x-avtr)<=tmax2) maskbadmap(badchcalprof,i,16);
    if((x-avtr)>tmin3&&(x-avtr)<=tmax3) maskbadmap(badchcalprof,i,32);
    if((x-avtr)>tmax4)   maskbadmap(badchcalprof,i,64);
 }
 therecmodule->getmodtest()->setbadchcalprof(badchcalprof);
  }  

// general flags
void RecAnalyzer::Analyze(FlagsDescriptor* f) {

std::map<int, int, less<int> > mpinvon=therecmodule->getmodheader()->getbadchpinvon();
std::map<int, int, less<int> > mpinvoff=therecmodule->getmodheader()->getbadchpinvoff();
std::map<int, int, less<int> > mdinvon=therecmodule->getmodheader()->getbadchdinvon();
std::map<int, int, less<int> > mdinvoff=therecmodule->getmodheader()->getbadchdinvoff();

std::map<int, int, less<int> > mbad=therecmodule->getmodheader()->getbadchm();

std::map<int, int, std::less<int> > mbadchped=therecmodule->getmodtest()->getbadchped();
std::map<int, int, std::less<int> >  mbadchcalprof=therecmodule->getmodtest()->getbadchcalprof();
std::map<int, int, std::less<int> >  mbadchcal=therecmodule->getmodtest()->getbadchcal();
std::map<int, int, std::less<int> >  mbadchfb=therecmodule->getmodtest()->getbadchfb();
std::map<int, int, std::less<int> >  mbadchlight=therecmodule->getmodtest()->getbadchlight();
std::map<int, int, std::less<int> >  mbadchlat=therecmodule->getmodtest()->getbadchlat();
std::map<int, int, std::less<int> >  mbadchbp=therecmodule->getmodtest()->getbadchbp();
// identify bad channel defects
 for(int i=1;i<=nchan;i++) {
//1. noisy
   bool bn=findmapch(mbadchped,i)&16;
   int fn=f->chfnoisy;
   if(bn) maskbadmap(mbad,i,fn); 
//2.short
   bool bsa=(findmapch(mbadchcalprof,i)&2)||(findmapch(mbadchcal,i)&2);
   bool bsaa=(findmapch(mbadchcalprof,i-1)&2)||(findmapch(mbadchcalprof,i+1)&2)||
             (findmapch(mbadchcal,i-1)&2)||(findmapch(mbadchcal,i+1)&2);
   bool bsn=findmapch(mbadchped,i)>0||findmapch(mbadchped,i+1)>0||findmapch(mbadchped,i-1)>0;
   fn=f->chfshort;
   if(bsa&&bsaa&&bsn) maskbadmap(mbad,i,fn);
//3. open1: hybrid-sensor
   bool bo1p=!((findmapch(mbadchcalprof,i)&2)||(findmapch(mbadchcal,i)&2));
   bool bo1n=findmapch(mbadchped,i)&4;
   bool bo1t=findmapch(mbadchcalprof,i)&16;
   fn=f->chfopen1;
   if(bo1p&&bo1n&&bo1t)  maskbadmap(mbad,i,fn);
//4. open2: sensor-sensor (in case of 2 sensors)
  bool bo2p=!((findmapch(mbadchcalprof,i)&2)||(findmapch(mbadchcal,i)&2));
   bool bo2n=findmapch(mbadchped,i)&8;
   bool bo2t=findmapch(mbadchcalprof,i)&32;
    fn=f->chfopen2;
   if(bo1p&&bo1n&&bo1t)  maskbadmap(mbad,i,fn);
//5. open 3: unknown open
   bool b3n=findmapch(mbadchped,i)>0;
   bool b3c=findmapch(mbadchcalprof,i)&4||findmapch(mbadchcal,i)&4;
   fn=f->chfopen3;
   if(b3n&&b3c) maskbadmap(mbad,i,fn);

//6. pinhole1: low leakage pinhole
    bool bp1n=findmapch(mbadchped,i)&2;
    bool bp1c=(findmapch(mbadchcalprof,i)&2)||(findmapch(mbadchcal,i)&2);
    if(bp1n&&bp1c) maskbadmap(mbad,i,f->chfpin1);
//7. high leakage pinhole
     bool bp2p=(findmapch(mbadchfb,i)&2)||(findmapch(mbadchlight,i)&2);
     bool bp2n=findmapch(mbadchped,i)>0;
     fn=f->chfpin2;
     if(bp2p)   maskbadmap(mbad,i,fn); 
//8. others 
     fn=f->chfunknown;
     if((findmapch(mbadchped,1)>0||findmapch(mbadchfb,i)>0)&&
	(findmapch(mbadchcalprof,1)>0
         ||findmapch(mbadchcal,1)>0
         ||findmapch(mbadchlight,1)>0
         ||findmapch(mbadchbp,1)>0        
	 ||findmapch(mbadchlat,1)>0))
       maskbadmap(mbad,i,fn);
 }
// fill histo
  TH1F* h=therecmodule->getmodheader()->THchflag();
  if(h!=0) {
    h->Reset();
for(int i=1;i<=h->GetNbinsX();i++) {
   if(mbad.find(i)!=mbad.end())
     h->SetBinContent(i,mbad[i]);
               }      
  }

 int nbad=mbad.size(); // number of bad channels
 for( map<int, int, less<int> >:: const_iterator im=mbad.begin();im!=mbad.end();im++) {
   if(im->second==0) mbad.erase(im->first);
 }
   therecmodule->getmodheader()->setnbadch(nbad);
   therecmodule->getmodheader()->setbadchm(mbad);


// identify  component defect
// APV from connectivity run
  int statAPV=therecmodule->getmodheader()->getstatAPV();
  vector<int> c= *(therecmodule->getmodtest()->connectivity());
  if(c.size()>5) {
  if(c[0]!=0) statAPV|=0x01;
  if(c[1]!=0) statAPV|=0x02;  
  if(c[2]!=0) statAPV|=0x04; 
  if(c[3]!=0) statAPV|=0x10; 
  if(c[4]!=0) statAPV|=0x20; 
  if(c[5]!=0) statAPV|=0x40;
  if(statAPV>0) statAPV=-statAPV;
  } 
  therecmodule->getmodheader()->setstatAPV(statAPV);
// sensor
  int statSensor=therecmodule->getmodheader()->getstatSensor();
 if(c.size()>6) if(c[6]!=0) if(statSensor>0) statSensor=-statSensor;
  therecmodule->getmodheader()->setstatSensor(statSensor);
// statMUX, not used yet
   int statMux=therecmodule->getmodheader()->getstatMUX();
   int st=0;
   therecmodule->getmodheader()->setstatMUX(st);
//statPLL,  not used yet
   int statPLL=therecmodule->getmodheader()->getstatPLL();
   therecmodule->getmodheader()->setstatPLL(st);
//statDCU
  int statDCU=therecmodule->getmodheader()->getstatDCU();
  float maxadc=f->fdcurange;
  if(therecmodule->getmodsctrl()->getvhyb25dcu()>maxadc) statDCU|=0x01;
  if(therecmodule->getmodsctrl()->getvhyb125dcu()>maxadc)  statDCU|=0x02; 
  if(therecmodule->getmodsctrl()->getileakdcu()>maxadc)  statDCU|=0x04;
  if(statDCU>0) statDCU=-statDCU;
  therecmodule->getmodheader()->setstatDCU(statDCU);


//  module  qualification
int val=0;
int nerr=  therecmodule->getmodheader()->getnapverrors();
bool bstatreadout = (therecmodule->getmodheader()->getstatAPV()==0 &&
           therecmodule->getmodheader()->getstatMUX()==0 &&
	   therecmodule->getmodheader()->getstatPLL()==0 &&
           therecmodule->getmodheader()->getstatDCU()==0 &&
           nerr<5);
bool bstatsensor = (therecmodule->getmodheader()->getstatSensor()==0);
 float pbadthA=f->fpbadchA;
 float pbadthB=f->fpbadchB;
bool bgradeA=(therecmodule->getmodheader()->getnbadch()/nchan<=pbadthA);
bool bgradeB=(therecmodule->getmodheader()->getnbadch()/nchan<=pbadthB);
bool bgradeBcur=(therecmodule->getmodtest()->getitest3() > f->fitestB);  // current uA
 if(bstatreadout&&bstatsensor) {
   if(bgradeA) val=0;
   if(bgradeBcur) val=1;
   if(bgradeB) val=2;
   if(bgradeBcur&&bgradeB) val=3;
  } else if(!bstatreadout) {
    val=-1;

  } else if(!bstatsensor) {
    val=-2;
  } else if(!bstatsensor&&!bstatreadout) {
    val=-3;
  }
 therecmodule->getmodheader()->setval(val);
}

void RecAnalyzer::AnalyzeConnectivity(const QString& s, ModuleDescriptor* d){}

void RecAnalyzer::AnalyzeAuxChip(const QString& s, ModuleDescriptor* d){}
