#include "RecModHeader.h"
#include "RecUtil.h"
#include "ModId.h"
#include <qstring.h>


#ifndef NAPVCH
#define NAPVCH  128
#endif


using namespace RecUtil;
using namespace std;

RecModHeader::RecModHeader(int bench, ModId* id, int napv) {
  napvs=napv;
  strcpy(modid,id->getModId().c_str());
  strcpy(modtype,id->getModuleType().c_str());
  sprintf(dcuid,"%d",id->getDcuId());
  nbench=bench;
  bookhist();
  Reset();
   statApv=0;statMux=0;statPll=0;statDcu=0;statSensor=0; 
   val=0;

}

RecModHeader::RecModHeader(int bench, int napv) {
  napvs=napv;
  nbench=bench;
  bookhist();
  Reset();
   statApv=0;statMux=0;statPll=0;statDcu=0;statSensor=0;
   val=0;

}


RecModHeader::~RecModHeader() {
  Reset();
  if(hchflag) delete  hchflag;  hchflag=0;
}


void RecModHeader::bookhist() {
char name[80],title[80];
int  ib=napvs*NAPVCH;
float max=float(ib);
 const float bs =0.1; // binshift 

sprintf(name,"badChflag_%.1d",nbench);
  hchflag = new TH1F(name,"badChflag",ib, bs,max+bs);

}

void RecModHeader::Reset() {
napverrors=0;
ResetFlags();
ResetXt();
}

void RecModHeader::ResetFlags() {
  nbadch=0;
  mbadch.clear();
  mbadchpinvon.clear();
  mbadchpinvoff.clear();
  mbadchdinvon.clear();
  mbadchdinvoff.clear();
  hchflag->Reset();
}

void RecModHeader::Save(TDirectory* dir) {

  if(dir->cd("Summary") ) {
  totn(modid).Write("TNmodid");
  totn(modtype).Write("TNmodtype");
  totn(dcuid).Write("TNdcuid");
  totv(napvs).Write("TInapvs");
  totv(nbench).Write("TInbench");
  totv(napverrors).Write("TVnapverrors");
  totv(val).Write("TLval");
  totv(statApv).Write("TIstatApv");
  totv(statMux).Write("TIstatMux");
  totv(statPll).Write("TIstatPll");
  totv(statDcu).Write("TIstatDcu");
  totv(statSensor).Write("TIstatSensor");
  totv(nbadch).Write("TInbadch");
  totv(mbadch).Write("TIbadch");
  vbadchlist.clear();
  for(map<int , int, less<int> >::const_iterator i=mbadch.begin();i!=mbadch.end();i++)
    vbadchlist.push_back(i->first);
  totv(vbadchlist).Write("TIbadchlist");
  totv(mbadchpinvon).Write("TIbadchpinvon");
  totv(mbadchpinvoff).Write("TIbadchpinvoff");
  totv(mbadchdinvon).Write("TIbadchdinvon");
  totv(mbadchdinvoff).Write("TIbadchdinvoff");
  hchflag->Write("THchflag");

  }

}

Int_t  RecModHeader::AddMeast(Float_t t) {
 vnbadcht.push_back(nbadch);
 return vnbadcht.size();
}

void RecModHeader::ResetXt() {
      vnbadcht.clear();
}

map<int, int, less<int> > RecModHeader::getbadchmap(const QString& s) {
 if(s=="pinvon")   return getbadchpinvon();
 else  if(s=="pinvoff") return getbadchpinvoff();
 else  if(s=="dinvoff") return getbadchdinvoff();
 else  if(s=="dinvon")  return getbadchdinvon();
 else  return getbadchpinvoff();
}
void RecModHeader::setbadchmap(const QString& s,const  map<int, int, less<int> >& m ) {
 if(s=="pinvon")    setbadchpinvon(m);
 else  if(s=="pinvoff") setbadchpinvoff(m);
 else  if(s=="dinvoff") setbadchdinvoff(m);
 else  if(s=="dinvon")  setbadchdinvon(m);
 else setbadchpinvoff(m);
}

