#include "RecSetup.h"
#include "RecUtil.h"
#include <TVector.h>
#include <TFile.h>
#include <cstring>

using namespace std;
using namespace RecUtil;

RecSetup::RecSetup(int nt, int nh) {
  Reset();
  ntemp=nt; temps.clear(); 
  if (ntemp < 0) {
    ntemp=0;
  }
  temps.resize(ntemp);
  nhum=nh; 
  if(nhum <0){
    nhum=0;
  }
  hums.clear();;  hums.resize(nhum); 
}
/**/
RecSetup::~RecSetup() {
  Reset();
}
/**/
void RecSetup::Reset() {
  Nmeas=0;
  vtemptagt.clear();
  vtempcoldt.clear();
  vtempwarmt.clear();
  vtempboxt.clear();
  vtempextt.clear();
  vhumboxt.clear();
  vhumextt.clear();
  vtime.clear();
  temps.clear();
  hums.clear();
  humst.clear();
  tempst.clear();
  temptag=100;tempcold=10;tempbox=100;tempwarm=100;tempext=0;humext=0;humbox=0;

}
/**/
Int_t  RecSetup::AddMeast(Float_t t) {
 Nmeas++;
 vtime.push_back(t);
 vtemptagt.push_back(temptag);
 vtempcoldt.push_back(tempcold);
 vtempwarmt.push_back(tempwarm);
 vtempboxt.push_back(tempbox);
 vtempextt.push_back(tempext);
 vhumboxt.push_back(humbox);
 vhumextt.push_back(humext);

 for(int i=0;i<ntemp;i++){ 
   tempst[i].push_back(temps[i]);
 }
 for(int i=0;i<nhum;i++) {
  humst[i].push_back(hums[i]);
 }
 Int_t n=vtime.size();
 return n;
}

void RecSetup::Save(TDirectory* dir) {

  TDirectory* dmod=dir->mkdir("Environment");
  dmod->cd();
  
  totv(temptag).Write("TVtemptag");
  totv(tempcold).Write("TVtempcold");
  totv(tempbox).Write("TVtempbox");
  totv(tempwarm).Write("TVtempwarm"); 
  totv(tempext).Write("TVtempext");
  totv(humext).Write("TVhumext");
  totv(humbox).Write("TVhumbox");
  totv(temps).Write("TVtemps");
  totv(hums).Write("TVhums");
  totv(vtime).Write("TVtime");
  
  totg(vtime,vtemptagt,"TGtemptagt")->Write("TGtemptagt");
  totg(vtime,vtempcoldt,"TGtempcoldt")->Write("TGtempcoldt");
  totg(vtime,vtempwarmt,"TGtempwarmt")->Write("TGtempwarmt");
  totg(vtime,vtempboxt,"TGtempboxt")->Write("TGtempboxt");
  totg(vtime,vtempextt,"TGtempextt")->Write("TGtempextt");
  totg(vtime,vhumboxt,"TGhumboxt")->Write("TGhumboxt");
  totg(vtime,vhumextt,"TGhumextt")->Write("TGhumextt");

  char vname[12];
  for(int i=0;i<ntemp;i++){ 
    sprintf(vname,"TVtemp%03d",i);
    int n=tempst[i].size();
    totv(tempst[i]).Write(vname);
  }
  for(int i=0;i<nhum;i++){ 
    sprintf(vname,"TVhumidity%03d",i);
    totv(humst[i]).Write(vname);
  }



// ARC compatible
  totv(tempext).Write("TempExt_F");
  totv(humext).Write("HumExt_F");
  totv(humbox).Write("HumSetup_F");
  totv(tempbox).Write("TempSetup_F");
}

TGraph* RecSetup::TGtemptagt() { 
Int_t ns=vtime.size();
  Double_t  x[ns], y[ns];
  for(Int_t i=0; i<ns;i++) {
    x[i]=vtime[i];
    y[i]=vtemptagt[i];
  }
  return new TGraph(ns,x,y);
}


TGraph* RecSetup::TGtempcoldt() {
 Int_t ns=vtime.size();
  Double_t  x[ns], y[ns];
  for(Int_t i=0; i<ns;i++) {
    x[i]=vtime[i];
    y[i]=vtempcoldt[i];
  }
  return new TGraph(ns,x,y);
}
TGraph* RecSetup::TGtempwarmt() {
 Int_t ns=vtime.size();
  Double_t  x[ns], y[ns];
  for(Int_t i=0; i<ns;i++) {
    x[i]=vtime[i];
    y[i]=vtempwarmt[i];
  }
  return new TGraph(ns,x,y);

}
TGraph* RecSetup::TGtempboxt() {
 Int_t ns=vtime.size();
  Double_t  x[ns], y[ns];
  for(Int_t i=0; i<ns;i++) {
    x[i]=vtime[i];
    y[i]=vtempboxt[i];
  }
  return new TGraph(ns,x,y);

}
TGraph* RecSetup::TGhumboxt() {
 Int_t ns=vtime.size();
  Double_t  x[ns], y[ns];
  for(Int_t i=0; i<ns;i++) {
    x[i]=vtime[i];
    y[i]=vhumboxt[i];
  }
  return new TGraph(ns,x,y);

}
TGraph* RecSetup::TGhumextt() {
 Int_t ns=vtime.size();
  Double_t  x[ns], y[ns];
  for(Int_t i=0; i<ns;i++) {
    x[i]=vtime[i];
    y[i]=vhumextt[i];
  }
  return new TGraph(ns,x,y);

}

TGraph* RecSetup::TGtempextt() {
 Int_t ns=vtime.size();
  Double_t  x[ns], y[ns];
  for(Int_t i=0; i<ns;i++) {
    x[i]=vtime[i];
    y[i]=vtempextt[i];
  }
  return new TGraph(ns,x,y);

}



