#include "RecUtil.h"
#include "daqerrormsg.h"
using namespace std;

/**/
TVector RecUtil::totv(Int_t n) {
  TVector v(1);
  v(0)=n;
  return v;
}
/**/
TVector RecUtil::totv(Float_t f) {
  TVector v(1);
  v(0)=f;
  return v;
}
/**/
TVector RecUtil::totv(const vector<float>& f) {
  TVector v(1);
  if(f.size() == 0)  { v.Zero(); return v;}
  else v.ResizeTo(f.size());
  for (int i=0;i<f.size();i++) v(i)=f[i];
  return v;
}
/**/
TVector RecUtil::totv(const vector<int>& n) {
  TVector v(1);
  if(n.size() == 0)  { v.Zero(); return v;}
  else v.ResizeTo(n.size());
  for (int i=0;i<n.size();i++) v(i)=n[i];
  return v;
}
//
TVector RecUtil::totv(vector<float>* f) {
  TVector v(1);
  if((*f).size() == 0)  { v.Zero(); return v;}
  else v.ResizeTo((*f).size());
  for (int i=0;i<(*f).size();i++) v(i)=(*f)[i];
  return v;
}
/**/
TVector RecUtil::totv(vector<int> *n) {
  TVector v(1);
  if((*n).size() == 0)  { v.Zero(); return v;}
  else v.ResizeTo((*n).size());
  for (int i=0;i<(*n).size();i++) v(i)=(*n)[i];
  return v;
}
/**/
TVector RecUtil::totv(const map<int, int, less<int> >&  n) {
  TVector v(1);
  if(n.size() == 0)  { v.Zero(); return v;}
  else v.ResizeTo(2*n.size());
  int k=0;
  for (map<int, int, less<int> >::const_iterator i=n.begin();
       i !=n.end(); i++) {
    v(k)=i->first;
    v(k+1)=i->second;
    k=k+2;
  }
  return v;
}

TGraph* RecUtil::totg(vector<float>* x,vector<float>* y,const char* title)
{
 
TVector x1=totv(x);
TVector y1=totv(y);
 TGraph* G=new TGraph(x1,y1);
 G->SetTitle(title);
 return G;
}
//
TGraph* RecUtil::totg(const vector<float>& x, const vector<float>& y,const char* title)
{
 TVector x1=totv(x);
 TVector y1=totv(y);
 TGraph* G=new TGraph(x1,y1);
 G->SetTitle(title);
 return G;
}
//
TNamed RecUtil::totn(const char* c) 
{ 
  TNamed s;
  s.SetName(c); 
  return s; 
}
//
TNamed RecUtil::totn(Int_t n) 
{ 
  TNamed s;
  char c[10];
  sprintf(c,"version_%d",n);
  s.SetName(c); 
  return s; 
}

//
void RecUtil::average(TH1F* h,const map<int, int, less<int> >&  mask, float& av, float& s)
{
  av=0.;
  s=0.;
  float sum=0.;
  float sum2=0.;
  int n=0;
  for(int is=1;is<=h->GetNbinsX();is++) {
    if(mask.find(is)!=mask.end()) { continue;}
  float x=h->GetBinContent(is);
  sum+=x;
  sum2+=x*x;
  n++;
  }
if(n>0) {
  av=sum/n;
  s=sqrt(fabs(sum2/n-av*av));
  }
}

Float_t RecUtil::tvtofloat(const TVector* v) {
   DT_DEBUG_DB(" nooflemf "<<v->GetNoElements());
  if(v->GetNoElements()>0) {
  const  Float_t* x=v->GetElements();
  DT_DEBUG_DB(" tvtofloat "<<(*x));
  return (*x);} else return 0.;
}


Int_t RecUtil::tvtoint(const TVector* v) {
  DT_DEBUG_DB(" nooflemi "<<v->GetNoElements());
  if(v->GetNoElements()>0) {
  const  Float_t* x=v->GetElements();
   DT_DEBUG_DB(" tvtoint "<<(*x));
  return Int_t(*x);} else return 0;
}

const char* RecUtil::tntochar(const TNamed* c) {
  return c->GetName();
}


vector<float> RecUtil::tvtovector(const TVector* v) {
  int ns=v->GetNoElements();
 vector<float> vx;
 if(ns>0) {
   const  Float_t* x=v->GetElements();
  for(int i=0;i<ns;i++)
    vx.push_back(*x);
 DT_DEBUG_DB(" tvtovector "<<(*x));
  x++;
 }
  return vx; 
}


map<int, int, less<int> >  RecUtil::tvtomap(const TVector* v) {
  int ns=v->GetNoElements();
 map<int, int, less<int> >  mx;
 if(ns>0) {
   const  Float_t* x=v->GetElements();
   int i=0;
   while(i<ns) {
     int k=int((*x)); x++;
     int kv=int((*x));
     mx[k]=kv;     
     i=i+2;
   }
 }   
  return mx; 
}

void  RecUtil::maskbadmap(map<int, int, std::less<int> >&  m,int i,int flag) {
     if(m.find(i)!=m.end()) {
	if(m[i]<0) m[i]=flag;
	else m[i]|=flag; 
      } else { m[i]=flag;  }

}

const char*  RecUtil::dirname(Int_t m) {

 if(m&0x08) {
  if(m&0x20) 
  return "PeakInvOn";
  else
  return "PeakInvOff";
 } else {
 if(m&0x20) 
 return "DecInvOn";
  else
 return "DecInvOff";
 }
}

TH1F*  RecUtil::toth(map<int, int, std::less<int> >&  m, int c1,int c2,int nch) {
  TH1F* h=new TH1F("badch","badch",nch,0,nch);
  h->Reset();
 for (map<int, int, less<int> >::const_iterator i=m.begin();
       i !=m.end(); i++) {
   int bin=i->first;
   int cont=i->second;
   if(cont>=c1&&cont<=c2)
   h->SetBinContent(bin,cont);
  }
 return h;
}

int  RecUtil::findmapch(map<int, int, std::less<int> >&  m,int i) {
  if(m.find(i)!=m.end()) return m[i];
  else return 0;
}

float  RecUtil::correctileak(float t, float i) {
//t in C, i in nA
  float ic=i;
  return ic;
}
