#include "Record.h"
#include "RecModule.h"
#include "RecHeader.h"
#include "RecSetup.h"
#include "RecModHeader.h"
#include "RecModTest.h"
#include "RecModSctrl.h"
#include "RecAnalyzer.h"
#include "ModId.h"
#include <qwidget.h>
#include <qfile.h>
#include <qmessagebox.h>
#include <qstring.h>
#include "daqerrormsg.h"

using namespace std;
Record::Record() {
 header=new RecHeader();
 recmodules.clear();
 Nrec=0;
 Nmod=0;
 theNoString="no";
}
/**/
void Record::ConstructModule(int bench, ModId* id, int napvs) {
recmodules.push_back(new RecModule(bench,id,napvs));
Nmod++;
}
/**/
void Record::ConstructModule(int bench,  int napvs) {
recmodules.push_back(new RecModule(bench,napvs));
Nmod++;
}

void Record::DestructAllModules() {
  int i=0;
  for(vector<RecModule*>::iterator irec=recmodules.begin();
      irec != recmodules.end(); irec++) { 
  }
  recmodules.clear();
  Nmod=0;
}

void Record::DestructModule(int bench) {
  vector<RecModule*>::iterator irecmod;
  for(irecmod=recmodules.begin();irecmod != recmodules.end(); irecmod++){
    if(bench == (*irecmod)->getmodheader()->getnbench() ){
      delete *irecmod;
      recmodules.erase(irecmod);
      Nmod--;
      break;
    }
  }
}

/**/
void Record::ConstructSetup(int ntemp, int nhum) { 
 recsetup=new RecSetup(ntemp,nhum);
}


/**/
Record::~Record() {
  delete header;
  delete recsetup;
  DestructAllModules();
}
/*
 */
void Record::Reset() {
  DT_NOTE(" reset");
  if(header)  header->Reset();
  if(recsetup) recsetup->Reset();
   for(vector<RecModule*>::iterator recmod=recmodules.begin();
      recmod != recmodules.end(); recmod++)
      (*recmod)->Reset();     
   // has the filenames to be cleared ?
}
//
const char*  Record::getFileName(int imod){ 
  if (imod<Nmod && imod>=0){ 
    return (recmodules[imod]->getModuleFilename()).c_str(); //gfilename[imod].c_str(); 
  } 
  else  return theNoString.c_str(); 
}

/**/
bool  Record::Save(const char* filename) {
  int i=0;
  TDirectory* drec;
for(vector<RecModule*>::iterator irec=recmodules.begin();
      irec != recmodules.end(); irec++) 
{
   const char* id=(*irec)->getmodheader()->getmodid();
   string f(filename);
   char str[100];
   sprintf(str,"_M_%s.root",id);
   string ext(str);
   string gfile=f+ext;
   if((*irec)->getModuleFilename()=="no") {
     (*irec)->setModuleFilename(gfile); 
   }
   if(!strstr(filename, ".root")){
     (*irec)->setModuleFilename(gfile); 
   }
   char cdir[80];
   sprintf(cdir,"Record%d",Nrec);
   TFile* file=new TFile(((*irec)->getModuleFilename()).c_str(),
			     "update","CMS Si Module Test UIA");
   TDirectory* dr=(TDirectory*)file->Get(cdir);
   while (dr) 
     {
       strcat(cdir,"B");
       cout << "Overwriting record  " << cdir << endl;
       dr=(TDirectory*)file->Get(cdir);
     }
   drec = file->mkdir(cdir);
   file->cd();
   header->Save(drec);
   file->cd();
   recsetup->Save(drec);
   file->cd();

   (*irec)->Save(drec);
   file->Close();
   i++;
 }
 Nrec++;
 return true;
}
//
bool  Record::Save(const char* filename,QWidget* mon) {
  int i=0;
  TDirectory* drec;
for(vector<RecModule*>::iterator irec=recmodules.begin();
      irec != recmodules.end(); irec++) 
{
   const char* id=(*irec)->getmodheader()->getmodid();
   string f(filename);
   char str[100];
   sprintf(str,"_M_%s.root",id);
   string ext(str);
   string gfile=f+ext;
   if((*irec)->getModuleFilename()=="no") {
     (*irec)->setModuleFilename(gfile); 
   }
   if(!strstr(filename, ".root")){
     (*irec)->setModuleFilename(gfile); 
   }
   QFile fl((*irec)->getModuleFilename().c_str());
   if (fl.exists())
     {
       int k= QMessageBox::warning(mon,"Monitor",
				   "file already exists. Overwrite?",
				   "Yes",
				   "Append", 0, 0, 1 );
       if (k==0) fl.remove();
     }
   TFile* file=new TFile(((*irec)->getModuleFilename()).c_str(),
			 "UPDATE","CMS Si Module Test UIA");
   char cdir[80];
   sprintf(cdir,"Record%d",Nrec);
   drec = file->mkdir(cdir);
   file->cd();
   header->Save(drec);
   file->cd();
   recsetup->Save(drec);
   file->cd();
   (*irec)->Save(drec);
   file->Close();
   i++;
 }
 Nrec++;
 return true;
}



/**/
RecModule* Record::getmodule(int bench) {
  vector<RecModule*>::const_iterator irecmod;
  for(irecmod=recmodules.begin();irecmod != recmodules.end(); irecmod++)
    if(bench == (*irecmod)->getmodheader()->getnbench() ) return (*irecmod);
    return 0;
}

RecModule*  Record::getfirstmodule(){
  RecModule* rm;
  if(recmodules.size() != 0) {  rm= recmodules[0];}
  else {rm=0;}
  return rm;

}
/**/
void  Record::Analyze(FlagsDescriptor* f) {
  vector<RecModule*>::const_iterator irecmod;
  for(irecmod=recmodules.begin();irecmod != recmodules.end(); irecmod++)
    (*irecmod)->getrecanalyzer()->Analyze(f);
}
