#include "DbParser.h"
#include "daqerrormsg.h"
#include <fstream>
#include <iostream>
using namespace std;

DbParser::DbParser(const char* filename) {  
  DT_NOTE("DbParser: construct; filename "<<filename);
  templatename=filename;
  ncomp=-1;
  QString sfile(filename);
  if(getTree(sfile)) DT_NOTE("  parsing into Db file ");
}
/**/
DbParser::~DbParser() {}
/**/
bool DbParser::getTree(const QString filename) {
  DT_DEBUG_LTAPPL("DbParser:getTree");
  QFile infile(filename);
  if( !infile.open( IO_ReadOnly )) {
    DT_WARN("*******  cannot open "<<filename) ;
    return false; 
  }
  DT_DEBUG_LTAPPL(" opened "<<filename);
  if(!domTree.setContent(&infile)) {
    DT_WARN("****** cannot parse as an XML "<<filename);
    infile.close(); return false;
  }
  infile.close();

  DT_DEBUG_LTAPPL("DbParser: getTree  setContent ");
  return true;
}
/**/
void DbParser::getCompositAction( const  QDomElement &composite) {  
 DT_DEBUG_LTAPPL(" getCompositeAction ");     
  QDomNode node = composite.firstChild();
  while( !node.isNull() ) {
    if(node.isElement() ) {
      if(node.nodeName() == "action_description" ) {
	DT_DEBUG_LTAPPL(" node = composite action_description ");      
	QDomElement act=node.toElement();    
	if( !act.isNull() ) {
	  DbActionDescriptor*  ad=new DbActionDescriptor(
	    act.attribute("name"),
	    act.attribute("version"),
	    act.attribute("object_name"),
	    act.attribute("object_type"),
	    act.attribute("input_id"));
	  compositeactiondescriptor=ad;
	}  
      } 

      if(node.nodeName() == "action" ) {
	DT_DEBUG_LTAPPL(" node =  basic action");
        QDomElement action=node.toElement();

       if(!therecreader->setRecord(ncomp)) {DT_WARN("found  no Record"<<ncomp);return;}

        if(!action.isNull() )  getBasicAction(action);
        else DT_WARN(" empty basic action ");
      }

      if(node.nodeName() == "composite" ) {
	if(!therecreader->setRecord(ncomp)) DT_WARN("found  no Record"<<ncomp);
	ncomp++;
	DT_DEBUG_LTAPPL(" node = composite action"<<ncomp); 
        QDomElement comp2action=node.toElement();
        if(!comp2action.isNull() )  getCompositAction(comp2action);
        else DT_WARN(" empty composite action ");
      }
    }
    node=node.nextSibling();
  }
}
//
void DbParser::getBasicAction(const QDomElement &action)  {
 QDomNode node = action.firstChild();
 while( !node.isNull() ) {
   if(node.isElement() ) {
     if(node.nodeName() ==   "action_description" ) {
       DT_DEBUG_LTAPPL(" node= basic action description"); 
       QDomElement act=node.toElement();    
       if( !act.isNull() ) {
	 DbActionDescriptor*  ad=new DbActionDescriptor(
	   act.attribute("name"),
	   act.attribute("version"),
	   act.attribute("object_name"),
	   act.attribute("object_type"),
	   act.attribute("input_id"));
	 basicactiondescriptor.push_back(ad);
       }  
     }
     if(node.nodeName() == " input " ) { 
       DT_DEBUG_LTAPPL(" node= input "); }

      if(node.nodeName() == "result" ) { 
       DT_DEBUG_LTAPPL("node =result ");
       QDomElement result=node.toElement();
       setAttribute(result);
     }
   }
   node=node.nextSibling();
 }
}
//
void DbParser::setAttribute(QDomElement &result)  {
  DT_DEBUG_LTAPPL(" setattribute ");
  QString  attrib;
  if( !result.isNull() )  { 
    QString sname=result.attribute("name");
    const  QString data=therecreader->Find(sname);
    DT_DEBUG_LTAPPL(" attribute "<<sname<<" data "<<data);
   if(data!="-0")  result.setAttribute("value",data);
  }
  DT_DEBUG_LTAPPL(" attribute set ")
}
/*
 * define recordreader and  records for composite actions
 */
void DbParser::populate(RecordReader* rec) {
 DT_DEBUG_LTAPPL(" populate rec ");
therecreader=rec;
 fillresults(); 
}

/* parse  domTree
 */
void DbParser::fillresults() {
  DT_DEBUG_LTAPPL(" fillresults ");
  QDomElement root=domTree.documentElement();
  QDomNode node0=root.firstChild();
  QDomNodeList nlist=node0.childNodes();
  QDomNode node=nlist.item(0);
  while( !node.isNull() ) {  

       if(node.isElement() && node.nodeName() == "object" ) {
	   QDomElement obj=node.toElement();
	   DT_DEBUG_LTAPPL(" populate modid "<<therecreader->modId());
	   obj.setAttribute("id",therecreader->modId());
	 }       
    if(node.isElement() && node.nodeName() == "composite" )  {
      if(!therecreader->setRecord(ncomp)) {DT_WARN("found  no Record"<<ncomp);return;}
      DT_DEBUG_LTAPPL("  composite action  "<<ncomp);
      QDomElement composite=node.toElement();
      if(!composite.isNull())  getCompositAction(composite);
      else DT_WARN(" empty composite action ");
      break;
    }
    node=node.nextSibling();
  }
}
/*
 * write to Xml file
*/
void DbParser::write(const QString filename)  {
  ofstream outfile(filename);
 cout<<"write ______________"<<endl;
 cout<<domTree.toString()<<endl;
 cout<<"_____________done"<<endl;
 outfile<<domTree.toString()<<endl; 
} 
/**/
void DbParser::write()  {
  QString  id=therecreader->modId();
  std::ofstream outfile;
  DT_DEBUG_LTAPPL(" write  "<<templatename<<" "<<id);
  // returns the template file name 
  QString  dir=templatename.section('/',-1);
  if(dir!=NULL){
    dir='_'+dir;
    dir=id+dir;
  } 
  else { dir="NN_";
    dir = dir+templatename;
  }
  outfile.open(dir);
  cout<<"write______________ "<<dir<<endl;
  cout<<domTree.toString()<<endl;
  cout<<"_____________done"<<endl;
  outfile<<domTree.toString()<<endl; 
} 
