#include "ad4717ctrlimpl.h"

#include <qlcdnumber.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qspinbox.h>
#include <normi2c.h>
#include "ad4717calc.h"

#include <iostream> 
using namespace std;

Ad4717CtrlImpl::Ad4717CtrlImpl(NormI2c * ni2c,
  QWidget* parent , const char* name , WFlags fl):
  Ad4717CtrlUi(parent){
  fNI2c=ni2c;
  fAd=new Ad4717Calc(fNI2c,(unsigned char) SBAddress->value());
  ResetValues();
}

void Ad4717CtrlImpl::ResetValues(){
  ResetCh(PBch0,LCh0,LMinCh0,LMaxCh0);
  ResetCh(PBch1,LCh1,LMinCh1,LMaxCh1);
  ResetCh(PBch2,LCh2,LMinCh2,LMaxCh2);
  ResetCh(PBch3,LCh3,LMinCh3,LMaxCh3);
  ResetCh(PBchT,LChT,LMinChT,LMaxChT);
  QPalette pal=PBPower->palette();
  QColorGroup cg=pal.active(); 
  cg.setColor(QColorGroup::Background,QColor(85,255,255));
  pal.setActive(cg);
  PBPower->setPalette(pal); 
}


void Ad4717CtrlImpl::ResetCh(QPushButton *PBch,
			     QLCDNumber* LCh, 
			     QLCDNumber* LMinCh,QLCDNumber*  LMaxCh ){
  QPalette pal;
  QColorGroup cg; 
  pal=PBch->palette();
  cg=pal.active();
  cg.setColor(QColorGroup::Button,QColor(85,255,255));
  pal.setActive(cg);
  PBch->setPalette(pal);
  LCh->display(0);
  LMinCh->display(0);
  LMaxCh->display(0);
}

Ad4717CtrlImpl::~Ad4717CtrlImpl(){
  delete fAd;
}


void Ad4717CtrlImpl::setAddress( int i){ fAd->setAddress(i);
 ResetValues(); 
}
void Ad4717CtrlImpl::ReadCh0(){
  ReadCh(0,PBch0,SbAvgNrCh0,LCh0, LMinCh0,LMaxCh0);
}

void Ad4717CtrlImpl::ReadCh( unsigned char  chan,QPushButton *PBch,
			     QSpinBox* SbAvgNrCh,  QLCDNumber* LCh, 
			     QLCDNumber* LMinCh,QLCDNumber*  LMaxCh ) 
{
  QPalette pal;
  QColorGroup cg;
  float value=0; float sig=0;
  unsigned short  max=0;
  unsigned short  min=0;
  pal=PBch->palette();
  cg=pal.active();
  int err = fAd->getChannelValue(chan+1,(int)SbAvgNrCh->value(),value,min,max,sig);
  cout << "rb "<< value<<endl;
  if( err < 0 ){
    cg.setColor(QColorGroup::Button,red);
  }
  else {
    LCh->display( (int) value);
    LMinCh->display((int) min);
    LMaxCh->display((int)max);
    cg.setColor(QColorGroup::Button,green);
  }
  pal.setActive(cg);
  PBch->setPalette(pal); 
}

void Ad4717CtrlImpl::ReadCh1(){
 ReadCh(1,PBch1,SbAvgNrCh1,LCh1, LMinCh1,LMaxCh1);
}

void Ad4717CtrlImpl::ReadCh2(){
 ReadCh(2,PBch2,SbAvgNrCh2,LCh2, LMinCh2,LMaxCh2);
}
void Ad4717CtrlImpl::ReadCh3(){
 ReadCh(3,PBch3,SbAvgNrCh3,LCh3, LMinCh3,LMaxCh3);
}
void Ad4717CtrlImpl::ReadChT(){
QPalette pal;
  QColorGroup cg;
  float value=0; float sig=0;
  float  max=0;
  float  min=0;
  pal=PBchT->palette();
  cg=pal.active();
  int err = fAd->getTemperature(0,(int)SbAvgNrChT->value(),value,min,max,sig);
  cout << "rb "<< value<<endl;
  if( err < 0 ){
    cg.setColor(QColorGroup::Button,red);
  }
  else {
    LChT->display( (int) value);
    LMinChT->display((int) min);
    LMaxChT->display((int)max);
    cg.setColor(QColorGroup::Button,green);
  }
  pal.setActive(cg);
  PBchT->setPalette(pal); 
}








void Ad4717CtrlImpl:: SetPower( bool on ){
  
  QPalette pal=PBPower->palette();
  QColorGroup cg=pal.active();
  int err = fAd->PowerOn(on);
  if( err < 0 ){
    cg.setColor(QColorGroup::Button,red);
  }
  else {
    if( on){ 
      cg.setColor(QColorGroup::Button,green);
    }
    else {
      cg.setColor(QColorGroup::Button,QColor(160,160,160));
    }
  }
  pal.setActive(cg);
  PBPower->setPalette(pal); 
}

