#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/file.h>

#include <signal.h>
#include <wait.h>
#include "adc.h"


static bool GlobalExit=false;
static bool GlobalPause=false;
void AdcCtrlc(int s)
{
  printf("Got %d %d  \n",getpid(),s);
  if (s == SIGUSR1 || s== SIGINT) 
    {
      GlobalExit = true;
      kill(getpid(),SIGCONT); // exit from pause in any case
    }
  if (s == SIGHUP) GlobalPause = true;
  if (s == SIGCONT) GlobalPause = false;
  printf ("Exit %d  Pause %d \n",GlobalExit,GlobalPause);
}

const int MAXADCCHANNEL=64;
const int MAXADCSAMPLE=2048;

int main( int argc, char** argv) {
  int fd;
 unsigned short destbuf[MAXADCCHANNEL*MAXADCSAMPLE];
 /* Arm signal handler */
  signal(SIGHUP,AdcCtrlc);
  signal(SIGCONT,AdcCtrlc);
  signal(SIGINT,AdcCtrlc);
  signal(SIGUSR1,AdcCtrlc);
 

  //fixed settings 
  int device=0;
  int adcgain=UP_1_25V ;
  bool  adcburst=true;
  bool setSingle=true;
  int nrChannels=64;
  int adcFrequency=1000;
  int nrSamples=2048;
  int mode= ADC_PACER_CLOCK;

  if( nrSamples > MAXADCSAMPLE) { nrSamples=MAXADCSAMPLE;}

  char cmd[80];
  sprintf(cmd,"/dev/adc%.2d",device);
  printf("COmmande %s \n",cmd);

  if ((fd = open(cmd, mode )) < 0) 
    {
      perror("Cannot open \n");
      printf("error opening device %s\n", cmd);
      exit(2);
    }
  // Set gain
  
  int ier = ioctl(fd, ADC_SET_GAINS, adcgain);
  // Adjust Mode
  
  if( adcburst)  ier=ioctl(fd, ADC_SET_BURSTE, 0);
  if(setSingle)   ier=ioctl(fd, ADC_SET_SINGLE, 0);

  
  // Number of channels
  ier =ioctl(fd, ADC_SETMUX_LIMITS,nrChannels-1);
  
  // Frequency
  int  freq = adcFrequency;;
  if (!adcburst) freq= freq*nrChannels;

  ier=ioctl(fd,ADC_SET_PACER_FREQ,freq);
 

  /* Start of Main loop */
  int k =0;
  int nc =nrSamples*nrChannels;
  int count=0;

  while (!GlobalExit) { // Exit on SIGUSR1 or SIGINT
    while (GlobalPause) {pause();} // Turn on on SIGHUP off on SIGCONT

    if (GlobalExit) {
      printf("I have to quit \n");break;}

    // Start the run
    count = read(fd,destbuf,nc);
    printf("I copied ADC Data %d %d \n",count,nc/2048);
    bool patchcheck=false;
    for (int j=0;j<64;j++)
    {	printf("%5d",destbuf[j]>>4);
 // unkonw problem not detected by driver or hardware ? 
      // patch works only for slow changing values 
      if (destbuf[count-128 +j] - 100 > destbuf[128 +j]) 
	{  if( (destbuf[count-128 +j] >> 4) ==  4095 ) // check only for saturation
	  {	patchcheck=true; break;}
	}
    }
    if (count!=nc)
      {
        printf("Run():Wrong number of conversions %d %d \n",count,nc);
        continue;
      }

    /* update the buffer */
    	k++;
	if (k%10 == 0) printf("Copy %d  \n",k);


  }
  
  close(fd);
   return 0;
}




