#include "dutsparser.h"
#include <util/PlatformUtils.hpp>
#include <util/XMLString.hpp>
#include <util/XMLUniDefs.hpp>
#include <framework/XMLFormatter.hpp>
#include <util/TranscodingException.hpp>
#include <dom/DOM_DOMException.hpp>
#include <parsers/DOMParser.hpp>
#include <dom/DOM.hpp>
#include "DOMTreeErrorReporter.hpp"
#include <cstring>
#include <cstdlib>
#include <XercesExt.hpp>
//#include "optomodinfoparser.h"
#include "structureparser.h"
#include "daqerrormsg.h"
#include "centerinfo.h"
// these should not needed in final:
//#include "optomodinfo.h"

using  namespace std;
using namespace XercesExt;
//static char*                    gXmlFile               = 0;
static bool                     gDoNamespaces          = false;
static bool                     gDoCreate              = false;
static XMLCh*                   gEncodingName          = 0;
//static XMLFormatter::UnRepFlags gUnRepFlags            = XMLFormatter::UnRep_CharRef;
static DOMParser::ValSchemes    gValScheme             = DOMParser::Val_Auto;
static XMLFormatter*            gFormatter             = 0;

//  ostream& operator<< (ostream& target, const DOMString& s)
//  {
//    char *p = s.transcode();
//    target << p;
//    delete [] p;
//    return target;
//  }

DutsParser::DutsParser(){
  fParseFileName="petal.xml";
  clear();
}




 char*   DutsParser::getStandardConfigFile(char* s, const char * filename )
    {   strcpy(s,  getStandardConfigPath().c_str() );
        strcat(s,filename );
	return s; 
    } 

string DutsParser::getStandardConfigPath( ) {
  string s(getenv("DAQHOME") );
  s+="/Config/";  
  return s;
}

int  DutsParser::parseFile() {
  // or fParseFilename is already calle and is absolute path 
  // of fParseFilename is "lt.xml" and is now changed to absolute path 
  string filename(setParseFileName(fParseFileName));
  return parseFile(filename.c_str());
}



string  DutsParser::setParseFileName( string filename ){
  if( filename.at(0) == '/') {
    // absolute path assumed 
    fParseFileName=filename;
  }
  else {
    // default path assumed 
    fParseFileName=getStandardConfigPath();
    fParseFileName+=filename;
  }
  return fParseFileName;
}

string  DutsParser::getParseFileName( )
{       return fParseFileName;
}

void DutsParser::clear()
{
  DT_DEBUG_LP( " clearmaps ") ;
  delete fCenterInfo;

}
int  DutsParser::parseFile(const char* name)
{ 
  DT_DEBUG_LP("parse file "<<name);   
  if (fileparsed) { DT_DEBUG_LP( "file already parsed ");
  return -1;}

  else fileparsed=true;
  try
    {
      XMLPlatformUtils::Initialize();
    }
  
  catch(const XMLException& toCatch)
    {
      cerr << "Error during Xerces-c Initialization.\n"
           << "  Exception message:"
           << DOMString(toCatch.getMessage()).transcode() << endl;
      return -2 ;
    }
 
  //
  //  Create our parser, then attach an error handler to the parser.
  //  The parser will call back to methods of the ErrorHandler if it
  //  discovers errors during the course of parsing the XML document.
  //
  parser = new DOMParser;
  parser->setValidationScheme(gValScheme);
  parser->setDoNamespaces(gDoNamespaces);
  parser->setCreateEntityReferenceNodes(gDoCreate);
  parser->setToCreateXMLDeclTypeNode(true);
  DOMTreeErrorReporter *errReporter = new DOMTreeErrorReporter();
  parser->setErrorHandler(errReporter);
  //
  //  Parse the XML file, catching any XML exceptions that might propogate
  //  out of it.
  //
  bool errorsOccured = false;
  try
    {
      parser->parse(name);

    }

  catch (const XMLException& e)
    {
      cerr << "An error occured during parsing\n   Message: "
           << DOMString(e.getMessage()).transcode() << endl;
      errorsOccured = true;
    }

 
  catch (const DOM_DOMException& e)
    {
      cerr << "An error occured during parsing\n   Message: "
           << DOMString(e.msg).transcode() << endl;
      errorsOccured = true;
    }

  catch (...)
    {
      cerr << "An error occured during parsing\n " << endl;
      errorsOccured = true;
    }
  errorsOccured|=errReporter->getSawErrors();
  // If the parse was successful, output the document data from the DOM tree
  if (!errorsOccured)
    {
      fDocument=(parser->getDocument()); 
            DOM_Node n1 = (fDocument).getFirstChild();
      while (!n1.isNull())
        {
          DT_DEBUG_LP( n1.getNodeName().transcode());
          DT_DEBUG_LP( n1.getNodeType() );
          if (n1.getNodeType() == DOM_Node::ELEMENT_NODE) break;
          n1 = n1.getNextSibling();
        }
      DOM_Node n = n1.getFirstChild();
      if (n!=0 && n.isNull())
        DT_NOTE( " Bad element "<< n.getNodeName().transcode());
      if (n == 0)DT_NOTE( "pb");
      while( !n.isNull() )     {
	DT_DEBUG_LP( n.getNodeName().transcode());
	if (strcmp("Structure",n.getNodeName().transcode())==0) {
	  DT_DEBUG_LP(" go to parse the structure");
	  StructureParser sp(n);
	}
	if (strcmp("CenterInfo",n.getNodeName().transcode())==0) {
	  DT_DEBUG_LP(" go to parse the center info");
	  centerParser(n);
	}
	n = n.getNextSibling();
	if (n!=0 && n.isNull())
	  DT_DEBUG_LP( " Bad element "<< n.getNodeName().transcode());
      }
      DT_NOTE( "parse file done ");
      
    }

 
  delete gFormatter;


  delete gEncodingName;
  //
  //  Clean up the error handler. The parser does not adopt handlers
  //  since they could be many objects or one object installed for multiple
  //  handlers.
  //


  //
  //  Delete the parser itself.  Must be done prior to calling Terminate, below.
  //
  delete errReporter;
  DT_DEBUG_LP( "delete parser");
  delete parser; parser=0;

  // And call the termination method
 
// done by DomInit()
//   XMLPlatformUtils::Terminate();
  // DomMemDebug().print();
    
  //
  //  The DOM document and its contents are reference counted, and need
  //  no explicit deletion.
  //

  return (int) errorsOccured;
}

// general  parser functions




int  DutsParser::centerParser (DOM_Node &ei) { 
 
   fCenterInfo = new CenterInfo();
  DOM_NamedNodeMap a = ei.getAttributes(); 
  getItem(fCenterInfo->fCenter,"center",a);
  getItem(fCenterInfo->fOperator,"operator",a);
  getItem(fCenterInfo->fToolId,"toolid",a);
  return 0;
}

//statics :

bool DutsParser::fileparsed=false;

CenterInfo*  DutsParser::fCenterInfo=0;
CenterInfo* DutsParser::getCenterInfo(){return fCenterInfo;}
