#include "optohybinfoparser.h"
#include "dom/DOMString.hpp"
#include "dom/DOM_Node.hpp"
#include <dom/DOM_NamedNodeMap.hpp>
#include <dom/DOM_Document.hpp>
#include <iostream>

#include "DOMTreeWriter.hpp"
#include "XercesExt.hpp"
#include "daqerrormsg.h"
using namespace std;
using namespace XercesExt;


int OptoHybInfoParser::parseNode(OptoHybInfo* mi,DOM_Node &ei){
  //mi->fDNMod=ei;
  DOM_NamedNodeMap a = ei.getAttributes();
  if(getItem(mi->fNrLasers,"nrlasers",a,-1)){
    DT_DEBUG_LP("fNrLasers "<< mi->fNrLasers);
  }
  getItem(mi->fOptoHybId,"optohybid",a,string("none"));
  getItem(mi->fOptoHybType,"type",a,string("none"));
  int nrlasers=0;
  DOM_Node n =ei.getFirstChild();
  while (!n.isNull() && nrlasers < 3) {
    if (strcmp(n.getNodeName().transcode(),"LaserInfo")==0){     
      OptoHybInfo::laserinfo li;
      laserInfoParser(li,n);
      if( li.fLaserNr >=0 && li.fLaserNr <3 ){
	mi->fLi[li.fLaserNr]=li;
	nrlasers++;
      }
      else {
	DT_WARN("invalid laser nr "<<li.fLaserNr);
      }
    }
    n = n.getNextSibling();    
  }
  if( mi->fNrLasers > 0){
    if ( mi->fNrLasers != nrlasers){
      DT_WARN("number of laser doesn't match nr of laser definitons");
  
    }
  }
  // always :
  mi->fNrLasers = nrlasers   ;
  // follownr not filled in yet !!
  return mi->fFollowNr;
}


int OptoHybInfoParser::laserInfoParser(
  OptoHybInfo::laserinfo& li,DOM_Node &ei){
  DOM_NamedNodeMap a = ei.getAttributes();
  getItem(li.fConnectorNr,"connectornr",a,-1);
  getItem(li.fFiberId,"fiberid",a,string("none"));
  getItem(li.fCircuitNr,"circuitnr",a,-1);
  getItem(li.fBias,"calvalue",a,-1);
  getItem(li.fLaserNr,"lasernr",a,(unsigned int) 0);
  return li.fLaserNr;
}



void  OptoHybInfoParser::setNode(OptoHybInfo* mi,
				DOM_Node &ei){
  DT_DEBUG_LP("set opto hyb info");
  setItem(mi->fOptoHybId,"optohybid",ei);
  setItem(mi->fOptoHybType,"type",ei);
  setItem(mi->fNrLasers,"nrlasers",ei); 
  DOM_Node n =ei.getFirstChild();
  while (!n.isNull()) {
    if( strcmp(n.getNodeName().transcode(),"LaserInfo")==0){
      ei.removeChild(n);
       n =ei.getFirstChild();
    }
    else {
      n = n.getNextSibling();
    }
  }
  for ( unsigned int lc=0;lc< mi->fLi.size();lc++){
    if( mi->fLi.find(lc) != mi->fLi.end()){
      createLaserNode(mi->fLi[lc],ei);
    
    }
  }

}
   
void OptoHybInfoParser::createLaserNode(OptoHybInfo::laserinfo & li,
					DOM_Node &ei){
  DOM_Node n= ei.getOwnerDocument().createElement("LaserInfo");
  DT_DEBUG_LP("set laser info for laser "<<li.fLaserNr);
  setItem(li.fFiberId,"fiberid",n);
  setItem(li.fConnectorNr,"connectornr",n);
  setItem(li.fCircuitNr,"ciruitnr",n);
  setItem(li.fBias,"calvalue",n);
  setItem(li.fLaserNr,"lasernr",n);
  ei.appendChild(n);
  DT_DEBUG_LP("laser node added for laser " << li.fLaserNr);
}
