#include "optohybwidget.h"
#include <qpainter.h>
#include <qcolor.h>
#include <qvariant.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include "optohybmenuimpl.h"
#include "qpushbutton.h"
#include "qaction.h"
#include "qpopupmenu.h"
#include "optomodinfo.h"
/* 
 *  Constructs a OptoHybWidget which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f'.
 */
OptoHybWidget::OptoHybWidget(OptoModInfo* mi, QWidget* parent,  const char* name, WFlags fl )
    : QWidget( parent, name, fl )
{
  fMi=mi;
  fHybNr=mi->getFollowNr();
  fParent=parent;
  //  QPixmap image0( ( const char** ) image0_data );
    if ( !name )
	setName( "OptoHybWidget" );
    resize(50, 30 ); 
    setBackgroundOrigin( QWidget::ParentOrigin );
    setCaption( trUtf8( "OptoHybWidget" ) );
    fHybColor=QColor(Qt::darkGreen);   
    fOptoHybState=NOTINITIALIZED;
    fSelected =false;
    fHM=0;//new OptoHybMenuImpl(this, "module 2");fHM->raise();
    fMenuFocus=false;

  //    PushButton1 = new QPushButton( this, "PushButton1" );
//      PushButton1->setGeometry( QRect( 10, 20, 40, 30 ) ); 
//      PushButton1->setText( trUtf8( "pb" ) );

    propAction=new QAction( this,"propAction");
    actAction=new QAction( this,"actAction");
    leftBMenu=new QPopupMenu(this);
    actAction->addTo(leftBMenu);
    actAction->setText( trUtf8("actions"));
    actAction->setMenuText( trUtf8( "actions" ) );
    propAction->addTo(leftBMenu);
    propAction->setText( trUtf8( "Properties" ) ); 
    propAction->setMenuText( trUtf8("Properties" ) ); 
    connect(propAction,SIGNAL( activated() ), this, SLOT(createOptoHybMenu()));
    connect(actAction,SIGNAL( activated() ), this, SLOT(createActionDialog()));
    
}

/*  
 *  Destroys the object and frees any allocated resources
 */
OptoHybWidget::~OptoHybWidget()
{
  delete fHM;
}


void OptoHybWidget::createOptoHybMenu(){
  if(! fHM){
    qWarning("create optohyb menu");
    fHM= new OptoHybMenuImpl(fMi,this, "optohyb 2",Qt::WType_Dialog );
    fHM->show();
    if(fHM){
      connect(fHM,SIGNAL(focusStatus(int ) ),
	      this,SLOT(menuFocus(int )));
    }
    else { qWarning("unexpectd fHM");}
    fMenuFocus=true;
  }
  update(); 
}

void OptoHybWidget::createActionDialog(){
 
    qWarning("action dialog not yet implemented");
    
}

void OptoHybWidget::removeOptoHybMenu(){
  qWarning("delete  optohyb menu");
  delete fHM;fHM=0; fMenuFocus=false;
  update();
}


void OptoHybWidget::paintRectFrame(  QPainter* qp){

   QPainter painter(this);
    QBrush lightBrush(fHybColor);
  QBrush darkBrush(Qt::black);
  int w=width();
  int h=height();
  QColor black=Qt::black;
  QColor white=Qt::white; 
  int fw=2;
  int r= 0;
  QPointArray fillarea(4);
  fillarea.putPoints(0,4, fw+1,fw+1  ,  r+fw,h-fw-1 , 
		     w-r-fw,h-fw-1 , w-fw-1,fw+1);
  painter.setBrush( 
    (fOptoHybState==NOTINITIALIZED)? lightBrush : darkBrush);
     {
      if ( fSelected){
	//qWarning("is selected");
	painter.setPen(QPen(white,fw));
	painter.drawLine(0, 0, r, h-fw);
	painter.drawLine(fw, 0, w-fw, 0);
	painter.setPen(QPen(black,fw));
	painter.drawLine(r+fw, h-fw, w-r-fw, h-fw);
	painter.drawLine(w-fw, fw, w-r, h-fw);
      }
      else {
	//qWarning("is not selected");
	painter.setPen(QPen(backgroundColor(),fw));
	painter.drawLine(0, 0, r, h-fw);
	painter.drawLine(1, 0, w-fw, 0);
	painter.drawLine(r+fw, h-fw, w-r-fw, h-fw);
	painter.drawLine(w-fw, fw, w-r, h-fw);
      }
      painter.setPen(Qt::NoPen); 
      painter.drawPolygon(fillarea);
      //qWarning("frame painted");      
     } 
     if( fHM) {
       painter.setPen(Qt::NoPen); 
       painter.setBrush(Qt::blue);
       qDebug("paint blue ");
       int rad=5;
       painter.drawEllipse (fw+1+rad,fw+1+rad,rad,rad); 
     }
}

void OptoHybWidget::simplePaint(){
  QPainter paint( this );
  paint.setPen( Qt::blue );
  paint.drawText( rect(), AlignCenter, "The Text" );
  qWarning("simple paint finished");
}

void OptoHybWidget::paintEvent(QPaintEvent *){
  
  paintRectFrame();
  if( fMenuFocus){
    drawFocusRect();
  }


}

void OptoHybWidget::menuFocus(int menustatus){
  OptoHybMenuImpl::OptoHybMenuStatus mms= 
    (OptoHybMenuImpl::OptoHybMenuStatus)menustatus;
  switch( mms){
    case OptoHybMenuImpl::OUTFOCUS : {
      fMenuFocus=false;
    }
    break;
    case OptoHybMenuImpl::CLOSSED : {
      fMenuFocus=false;
      removeOptoHybMenu();
    }
    break;
    case OptoHybMenuImpl::INFOCUS : {
      fMenuFocus=true;
    }
    break;
    default: fMenuFocus=false;
  }
  update();
//  fParent->update();
  
}



void OptoHybWidget::drawFocusRect(){
  
  QPainter painter(this);
  int w=width();
  int h=height();
  painter.setPen(Qt::red);
  painter.drawLine(0,0, 0,h-1);
  painter.drawLine(1,h-1,w-2,h-1);
  painter.drawLine(w-1,h-1,w-1,0);
  painter.drawLine(w-2,0,1,0);
  

}

void OptoHybWidget::mouseDoubleClickEvent ( QMouseEvent * e ) {
  
  switch(e->button()){
    case Qt::NoButton :{
    }
    break;
    case Qt::LeftButton:{
    }
    break; 
    case Qt::RightButton:{
      removeOptoHybMenu();
    }
    break;  
    case Qt::MidButton :{
    }
    break;
    case Qt::ShiftButton :{
    }
    break; 
    case Qt::ControlButton:{
    }
    break; 
    case Qt::AltButton  :{
    }
    break;
    case Qt::MetaButton  :{
    }
    break;
    case Qt::Keypad :{
    }
    break;
    case Qt::KeyButtonMask  :{
    }
    break;
    case Qt::MouseButtonMask  :{
    }
    break;
  }
 
}
void OptoHybWidget::mousePressEvent ( QMouseEvent * e ){
  switch(e->button()){ 
    case Qt::NoButton :{
    }
    break;
    case Qt::LeftButton:{
      fSelected=!fSelected;
      update();
      //optohybSelected();
    }
    break; 
    case Qt::RightButton:{
      leftBMenu->move(this->mapToGlobal(e->pos()));
      leftBMenu->show();
    }
    break;  

    case Qt::MidButton :{
    }
    break;
    case Qt::ShiftButton :{
    }
    break; 
    case Qt::ControlButton:{
    }
    break; 
    case Qt::AltButton  :{
    }
    break;
    case Qt::MetaButton  :{
    }
    break;
    case Qt::Keypad :{
    }
    break;
    case Qt::KeyButtonMask  :{
    }
    break;
    case Qt::MouseButtonMask  :{
    }
    break;
  }

}

/*
switch(){
  case Qt::NoButton :{
  }
  break;
  case Qt::LeftButton:{
  }
  break; 
  case Qt::RightButton:{
  }
  break;  
  case Qt::MidButton :{
  }
  break;
  case Qt::ShiftButton :{
  }
  break; 
  case Qt::ControlButton:{
  }
  break; 
  case Qt::AltButton  :{
  }
  break;
  case Qt::MetaButton  :{
  }
  break;
  case Qt::Keypad :{
  }
  break;
  case Qt::KeyButtonMask  :{
  }
  break;
  case Qt::MouseButtonMask  :{
  }
  break;
}
}
*/
