#include "paacbtesterimpl.h"
#include "ad4717ctrlimpl.h"
#include "measurechannel.h"
#include "qtextbrowser.h"
#include "qpushbutton.h"
#include "qlcdnumber.h"
#include "qspinbox.h"
#include "qdatetime.h"
#include "qfile.h"
#include "qtextstream.h"
#include "calccalvalues.h"
#include "CalibrationValues.h"
#include "normi2cdummy.h"
#include "normi2cpmc.h"
#include "normi2cvme.h"
#include "i2cpmcdevice.h"
#include "pcf8574ctrlimpl.h"
#include "vi2c.h"
#include <iostream>
using namespace std;
PaacbTesterImpl::PaacbTesterImpl(QWidget* parent,
				   const char* name, WFlags f ):
  PaacbTesterUi(parent,name)
{
  fI2cdevice=0;fNI2c=0;
  fI2cdevice=0; //new I2cPmcDevice(0);
  fI2cVme= new VI2C(0x060000,2);
  //fI2cTsc=0
  fNI2c=0;
  if( fI2cdevice) { fNI2c=new NormI2cPmc(fI2cdevice);}
  if(fI2cVme) {  fNI2c=new NormI2cVme(fI2cVme); fI2cVme->VI2C_init();}
  if(!fNI2c) { fNI2c= new NormI2cDummy(0); }
  fAd=new Ad4717CtrlImpl(fNI2c,tab_2,"tester",0);
  fIo=new  Pcf8574CtrlImpl(fNI2c,tab_5,"testio",0);
  fTmp2=0;
  fIth=0; 
  fC25=0;
  fC125=0;
  clearResults();
}

PaacbTesterImpl::~PaacbTesterImpl(){
  delete fTmp2;
  delete fIth;
  delete fC25;
  delete fC125;
  delete fAd;
  delete fNI2c;
  delete fI2cdevice;
}

void PaacbTesterImpl::clearResults()
{ delete fTmp2;
  fTmp2=new MeasureChannel(Ttmp2,
			   CBtmp2,LEphytmp2,LEunittmp2,LEtmp2,PBtmp2,LCtmp2,LErefadctmp2, TBtmp2);
  fTmp2->addMeasureLine("Pos 1",60.5,"K",1.67,43840);
  fTmp2->addMeasureLine("Pos 2",26.9,"K",.914,23936);
  fTmp2->addMeasureLine("Pos 3",10.10,"K",.084,10048);
  fTmp2->addMeasureLine("Pos 4",4.67,"K",.0395,4800);
  TBtmp2->clear();
  delete fIth;
  fIth=new MeasureChannel(Tith,CBith,LEphyith,LEunitith,LEith,PBith,LCith,LErefadcith, TBith);
  fIth->addMeasureLine("Pos 1",57.63,"K",0.781,40700);
  fIth->addMeasureLine("Pos 2",13.54,"K",.207,10752);
  fIth->addMeasureLine("Pos 3",5.62,"K",.167,424);
  fIth->addMeasureLine("Pos 4",2.88,"K",.077,2000);
  TBith->clear();
  delete fC25;
  fC25=new MeasureChannel(Tc25,CBc25,LEphyc25,LEunitc25,LEc25,PBc25,LC25,LErefadcc25,TBc25);
  fC25->addMeasureLine("Norm",1.58,"A",0.0162,44144);		  
  fC25->addMeasureLine("Offset",0,"A",.001,500);
  TBc25->clear();
  delete fC125;
  fC125=new MeasureChannel(Tc125,CBc125,LEphyc125,LEunitc125,LEc125,PBc125,LC125,LErefadcc125,TBc125);
  fC125->addMeasureLine("Norm",0.82,"A",0.0151,41000);		  
  fC125->addMeasureLine("Offset",0,"A",0.001,500);
  TBc125->clear();
  TBgeneral->clear();
  TBReport->clear();
}

void PaacbTesterImpl::TMP2Update(){
  
  fTmp2->updateLine();
}

void PaacbTesterImpl::ITHUpdate(){
  
  fIth->updateLine();
}

void PaacbTesterImpl::C25Update(){
  
  fC25->updateLine();
}

void PaacbTesterImpl::C125Update(){
  
  fC125->updateLine();
}

void  PaacbTesterImpl::ITHMeas(){
 fIth->Measure(  fAd->PBch1,fAd->LCh1);
}

void  PaacbTesterImpl::TMP2Meas(){;
 fTmp2->Measure(  fAd->PBch2,fAd->LCh2);
}


void  PaacbTesterImpl::C25Meas(){
 fC25->Measure(  fAd->PBch3,fAd->LCh3);
}

void  PaacbTesterImpl::C125Meas(){
 fC125->Measure(  fAd->PBch0,fAd->LCh0);
}

void PaacbTesterImpl::EnableEditResults(bool ed){
  TBc125->setReadOnly(!ed);
  TBc25->setReadOnly(!ed);
  TBith->setReadOnly(!ed);
  TBtmp2->setReadOnly(!ed);
  TBgeneral->setReadOnly(!ed);
}

 void PaacbTesterImpl::store(){
  fIth->storeMeasurement();
  fC125->storeMeasurement();
  fTmp2->storeMeasurement();
  fC25->storeMeasurement();
  fAd->PBchT->animateClick();
  float temperature= fAd->LChT->value();
  TBgeneral->clear();
  TBgeneral->setText("Temperature Indication: " +QString::number((double)temperature));

}

void PaacbTesterImpl::MakeReport(){
  vector<MeasureChannel:: mresults> mr;
  TBReport->clear();
  TBReport->setText("Calibration report for PAACB "+ QString::number((int)SBserialnr->value()));
  QDateTime today(QDateTime::currentDateTime());
  TBReport->append("\nDate  "+ today.toString()+"\n");
  TBReport->append("TMP2 data , the termistor connection \n");
  TBReport->append("Analogue value only as indication");
  TBReport->append(TBtmp2->text());
  TBReport->append("Calibration values for ltmod program TemperatureProbeNtc1");
  TBReport->append("gain=1");
  TBReport->append("openadcvalue=131070");
  mr=fTmp2->getResultValues();
  unsigned int  refpos=2;
  if(mr.size() <refpos ) {
    TBReport->append("not enough points to calucluate calibration values");
  }
  else {    
    float temperature=TempConverDef::NtcResistor2temperature(
      mr[refpos-1].fPhysRef,10,25,3380);
    TBReport->append("caltemp="+QString::number(temperature));
    TBReport->append("caladcvalue="+QString::number((int)mr[refpos-1].fResult));
  }
  mr.clear();

  TBReport->append("\nITH data, the termistor connected to the DCU");
  TBReport->append("Analogue value only for indication\n");
  TBReport->append(TBith->text());
  TBReport->append("Calibration values for ltmod program TemperatureProbeNtc0");
  TBReport->append("gain=1");
  TBReport->append("openadcvalue=0");
  mr=fIth->getResultValues();
  if(mr.size() < refpos-1) {
    TBReport->append("not enough points to calucluate calibration values");
  }
  else {
    float temperature=TempConverDef::NtcResistor2temperature(
      mr[refpos-1].fPhysRef,5,25,3380);
    TBReport->append("caltemp="+QString::number(temperature));
    TBReport->append("caladcvalue="+QString::number((int)mr[refpos-1].fResult));
  }
  mr.clear();
  TBReport->append("\nCurrent 2.5V line ");
  TBReport->append("Analogue value only as indication");
  TBReport->append(TBc25->text());
  TBReport->append("Calibration values for ltmod program ");
  // assumption that there are two values in the meausrement
   mr=fC25->getResultValues();
  if(mr.size() < 2) {
    TBReport->append("not enough points to calucluate calibration values");
  }
  else {
    float gain = CalcCalValues::CalcGain( 
      1000*mr[1].fPhysRef,mr[1].fResult,
      1000*mr[0].fPhysRef,mr[0].fResult);
    TBReport->append("gain="+QString::number(gain));
    TBReport->append("caladcvalue="+QString::number((int)mr[0].fResult));
    TBReport->append("calcurrent ="+QString::number(1000*mr[0].fPhysRef));  
  }
  mr.clear();

  TBReport->append("\nCurrent 1.25V line ");
  TBReport->append("Analogue value only as indication");
  TBReport->append(TBc125->text());
 TBReport->append("Calibration values for ltmod program ");
  // assumption that there are two values in the meausrement
   mr=fC125->getResultValues();
  if(mr.size() < 2) {
    TBReport->append("not enough points to calucluate calibration values");
  }
  else {
    float gain =CalcCalValues::CalcGain(
      1000*mr[1].fPhysRef,mr[1].fResult,
      1000*mr[0].fPhysRef,mr[0].fResult);
    cout<< "gain " << gain <<endl;
    TBReport->append("gain="+QString::number(gain));
    TBReport->append("caladcvalue="+QString::number((int)mr[0].fResult));
    TBReport->append("calcurrent ="+QString::number(1000*mr[0].fPhysRef));  
  }
  TBReport->append("\ngeneral results ");
  TBReport->append(TBgeneral->text());
  TBReport->append("\n all values are only valid for the calibration setup");
  TBReport->append("Using VUTRI adapters will introduce discrepancies with the current calibration");
  TBReport->append("NTC on the module have tolerances");
  TBReport->append("As the measurements  are not critical the above calibration values can be used");  
		   
}


void PaacbTesterImpl::fileSave(){

  QString filename("PAACB_"+ QString::number((int)SBserialnr->value())+"_CalData.txt");
  QFile calf(filename);
  calf.open(IO_WriteOnly);
  QTextStream qstream(&calf);
  qstream<<TBReport->text();
  calf.close();
}

void PaacbTesterImpl::fileExit(){
  close();
}
