#include "structureimpl.h"
#include "petalwidgetimpl.h"
#include "rodwidgetimpl.h"
#include "optomodinfoparser.h"
#include "optomodinfo.h"
#include <qcombobox.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qstring.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qtabwidget.h>

#include "dutsparser.h"
#include "structureparser.h"
#include "daqerrormsg.h"
#include <string> 
#include "structureinfo.h"
#include "modinfotobenchpos.h"
#include "benchpositionparser.h"
#include "benchposconnection.h"

using namespace std;
using namespace lttest;
StructureImpl::StructureImpl
(QWidget* parent , const char* name , WFlags fl ):
  StructureUi(parent,name,fl)
{
  fInitialized=false; 
	
}

StructureImpl::StructureImpl
(substructtypes st,QWidget* parent , const char* name , WFlags fl ):
  StructureUi(parent,name,fl)
{ 
  fInitialized=false; 
  structSelection(st);
}

void  StructureImpl::selectStructFile(){
  // create the filename from the structure id 
  // now with a file dialog
  QString  sf(QFileDialog::getOpenFileName( 
	  QString::null, " (*.xml)", this,
	  "openstructinformatoin","Open Structure File" ));
  if ( sf.isEmpty() ) { 
	  QMessageBox::information( this, "Monitor",
		   "Can't find requested file try again \n");
	  return;
  }
  DutsParser dutparser;
  if(dutparser.parseFile(sf.latin1()  )) { 
    DT_WARN ( "parse error see above  restart");
    // as long the clear for the parser is not implemented restart is needed 
    return ;
  }
  switch (StructureParser::getNrStructInfos()){
    case 0 : {
      // open error message box now only DT_WARN
      DT_WARN("no valid structure found in file restart program");
      return ;
    }
    break;
    case 1: {
      // so only one substructure is defined 
      fSi=StructureParser::getFirstStructInfo();
      if( !fSi) { 
	DT_WARN("error in program no structure info found ");
	return;
      }
      DT_DEBUG_GUI("only one structrure info found of type "<<
		   fSi->getStructureType()   );
      if(!structSelection(strtosubstruct(fSi->getStructureType()))){
	  DT_WARN(" structure of type "<< fSi->getStructureType() 
		  <<" not initialized");
	  return;
      }
      DT_DEBUG_GUI("start to add modules");
      addModules();
    }
    break;
    default : {
      // connectivity test so structure templates expected 
      // select the structure type
      DT_DEBUG_GUI("found "<<StructureParser::getNrStructInfos() <<
		   " structure infos");
      CBStructSelection->setEnabled(true);
      
    }
  }
}

void StructureImpl::structSelection( const QString& s){
 
  if( !(fSi=StructureParser::getStructureInfo(s.latin1() ) ) ) {
    DT_WARN(" structure of type "<< s.latin1()<<" is not found  ");
    return ;
  }
  DT_DEBUG_GUI("found structure with Id "<< fSi->getStructureId());
  // remove all structures from the ltduts document beside this 
  if(! structSelection(strtosubstruct(s.latin1())) ){
    DT_WARN(" structure of type "<< s.latin1()<<" not initialized");
    return;
  }
  addModules();
}

    

StructureImpl::~StructureImpl(){}

void StructureImpl::addModules(){
  if(! fSi) {return; }
  
  int nrmodules=29;
 BenchPositionParser bps;

 fModVector.clear();
  for (int i=0;i<nrmodules;i++){
    OptoModInfo* mi=fSi->getOptoModInfo(i);
    if( mi){
      BenchPosConnection* bp=
	new BenchPosConnection((modInfoToBenchPos(mi)));
      bp->info();
      bps.addBenchPosConnection( bp);
      // update the mod information 
      fModVector.push_back(mi);
      OptoModInfoParser::setNode(mi);
      DT_DEBUG_GUI("module "<<i<<"added");
    }    
  }  
}

ModId*  StructureImpl::getModIdInfoOrderNr(int lc){
  if((unsigned int) lc < fModVector.size()){
    return fModVector[lc];
  }
  else { 
    DT_DEBUG_GUI("no info for mod with ordnr "<<lc<< "/"<<fModVector.size());
    return 0;}
}

OptoModInfo*  StructureImpl::getModIdInfo(int lc){
 
  return fSi->getOptoModInfo(lc);
 
}

int  StructureImpl::initModuleInfo(){
  int nr=0;
 BenchPositionParser bps;
 nr=bps.getNumberBenchPositions();
 cout << "nr bench pos for substr"<<nr<<endl; 
 return nr;
}


std::vector<ModId*>* StructureImpl::getModIdInfo(){
  // fModVector exist as long structure impl exist 
  // have to consider to move this function to structureinfo. 
  // any how tye to change to pmap<ModId*>
  DT_DEBUG_GUI("number of modinfo "<<fModVector.size()); 
  return &fModVector;
}


bool StructureImpl::structSelection(substructtypes st){
  //make place for the tabs 
  CBStructSelection->hide();
  PBSelectStructXml->hide();
  TLStructId->hide();
  LEStructId->hide();
  { // tab creation  this is copied from the generate qt.
    // structureui.ui cvs 1.3  
    TWStructure = new QTabWidget( this, "TWStructure" );
    QRect df(this->frameGeometry());
    TWStructure->setGeometry(df );   
    FrontView = new QWidget( TWStructure, "FrontView" );
    TWStructure->insertTab( FrontView, trUtf8( "FronView" ) );

    BackView = new QWidget( TWStructure, "BackView" );
    TWStructure->insertTab( BackView, trUtf8( "BackView" ) );

    FrontStereo = new QWidget( TWStructure, "FrontStereo" );
    TWStructure->insertTab( FrontStereo, trUtf8( "FrontStereo" ) );

    BackStereo = new QWidget( TWStructure, "BackStereo" );
    TWStructure->insertTab( BackStereo, trUtf8( "BackStereo" ) );
    
    TWStructure->show();

  }

  switch (st) {
    case PETAL : {
      CBStructSelection->setEnabled(false);   
      createPetalView();
      fInitialized=true;
    }
    break;
    case ROD : {
      CBStructSelection->setEnabled(false);
      createRodView();
      DT_DEBUG_GUI("Create Rod Done");
      fInitialized=true;
    
    }
    break;
    default :fInitialized=false ;
      DT_WARN(" Undefined struct "<< st);
      break;
  }
  if( fInitialized) {
    fQw[PositionInfo::FRONTSIDE]->show();
    fQw[PositionInfo::BACKSIDE]->show();
    fQw[PositionInfo::FRONTSTEREO]->show();
    fQw[PositionInfo::BACKSTEREO]->show();
  }
  DT_DEBUG_GUI("Struct selection done");
  return fInitialized;
}


void StructureImpl::createPetalView( ){
  if(fInitialized) {return; }
  fPw[PositionInfo::FRONTSIDE]= 
    new PetalWidgetImpl(FrontView, "FrontView");
  fPw[PositionInfo::BACKSIDE]= 
    new PetalWidgetImpl(BackView, "BackView");
  fPw[PositionInfo::FRONTSTEREO]= 
    new PetalWidgetImpl(FrontStereo, "FrontViewStereo");
  fPw[PositionInfo::BACKSTEREO]= 
    new PetalWidgetImpl(BackStereo, "BackViewStereo");
  int maxmod=28;//should come from struct parser 
  DT_DEBUG_GUI("start  to create modules"<< fSi->getNrOptoMods());
  OptoModInfo* mi=0;
  for (int lc=0;lc<maxmod;lc++){ 
    if ( (mi=fSi->getOptoModInfo(lc))) {
      DT_DEBUG_GUI("create module "<< lc);
      fPw[mi->getSidePos()]->addOptoMod(mi);
    }
    else { DT_WARN("Module not found");}
  }
  fInitialized=true;
  fQw[PositionInfo::FRONTSIDE]=fPw[PositionInfo::FRONTSIDE];
  fQw[PositionInfo::BACKSIDE]=fPw[PositionInfo::BACKSIDE];
  fQw[PositionInfo::FRONTSTEREO]=fPw[PositionInfo::FRONTSTEREO];
  fQw[PositionInfo::BACKSTEREO]=fPw[PositionInfo::BACKSTEREO];
}

void StructureImpl::createRodView( ){
  if(fInitialized) {return; }
  fRw[PositionInfo::FRONTSIDE]= 
    new RodWidgetImpl(FrontView, "FrontView");
  fRw[PositionInfo::BACKSIDE]= 
    new RodWidgetImpl(BackView, "BackView");
  fRw[PositionInfo::FRONTSTEREO]= 
    new RodWidgetImpl(FrontStereo, "FrontViewStereo");
  fRw[PositionInfo::BACKSTEREO]= 
    new RodWidgetImpl(BackStereo, "BackViewStereo");
  int maxmod=12;
  qDebug("nr of modules defined %d ",fSi->getNrOptoMods()); 
  OptoModInfo* mi=0;
  for (int lc=0;lc<maxmod;lc++){ 
    if ( (mi=fSi->getOptoModInfo(lc))) {
      fRw[mi->getSidePos()]->addOptoMod(mi);
    }
    else { DT_WARN("Module not found");}
  }
  fInitialized=true;
  fQw[PositionInfo::FRONTSIDE]=fRw[PositionInfo::FRONTSIDE];
  fQw[PositionInfo::BACKSIDE]=fRw[PositionInfo::BACKSIDE];
  fQw[PositionInfo::FRONTSTEREO]=fRw[PositionInfo::FRONTSTEREO];
  fQw[PositionInfo::BACKSTEREO]=fRw[PositionInfo::BACKSTEREO];
}

vector<ModId*> StructureImpl::fModVector;
