#ifndef _Monitorimpl_H
#define _Monitorimpl_H
#include "Monitor.hxx"
#include <vector>
#include <qapplication.h>
#include  <zoomplot.h>
#include <qspinbox.h>
#include <qlabel.h>
#include <qprogressbar.h>
#include <qmessagebox.h>
#include <qcombobox.h>
#include <qtimer.h>
#include <qradiobutton.h> 
#include <qtabwidget.h>
#include <qcolor.h>
#include <qfiledialog.h>
#include <qlcdnumber.h>
#include <qwt_plot.h>
#include <qtextview.h>
#include <qpainter.h>
#include <qpen.h>
#include <qfont.h> 
#include <qpalette.h>
#include <qpushbutton.h>
#include <qlistbox.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <iostream>
#include <fstream>
#include <cstdio>
#include <cstdlib>
#include <sys/time.h>
#include <string>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>


/** 
\brief  main GUI

<p>   GUI interface for the Module test     
      Based on Qt gui tools, inherits from Monitor created by Qt designer
      public slots methodes are connected to the Gui widgets       


  \class Monitorimpl
   \authors: V.Zhukov,UIA
   \date 06/2002
   \version 2.1
   \authors: W. Beaumont UA
   \date 06/2003
   \version 2.2

*/

class  ModId;
class  ModuleTestSetup;
class  MonitorStatus;
class  OnlineMonitor;
class  SetParser;
class  I2cDescriptor;
class  Record;
class ModuleInfoGen;
//class ModuleInfoDisplayImpl;
class ModStatusImpl;
class HybridSettingsImpl;
class RunSettingsImpl;
class DaqSettingsImpl;
class MuxCtrlImpl;
class ColdBoxImpl;
class DioChannelInterface;

class Monitorimpl: public Monitor  {

   Q_OBJECT

public:

Monitorimpl(const char* file,QApplication* a,QWidget* parent = 0, const char* name = 0, WFlags fl = 0);

~Monitorimpl();

 bool initDone();

public slots:
virtual void CalProfRun();
virtual  void CalRun();
virtual  void LightRun();
virtual  void ChangeHV();
virtual void changePrintingLevel(const QString&);
virtual  void ChangeTemp();
virtual  void CheckSlowctrl();
virtual  void ContinueScenario();
virtual  void IVRun();
virtual  void BPlaneRun();
virtual  void LedRun();
virtual void FBiasRun();
virtual  void ConnectivityRun();
virtual  void InitSetup();
virtual  void LoadScenario();
virtual void AuxChipRun();
virtual  void PauseScenario();
virtual  void PedRun();
virtual  void Quit();
virtual  void StartScenario();
virtual  void StopScenario();
virtual  void LatencyRun();
virtual void SaveResults();
virtual  void ReadSettings();
virtual  void SaveDb();
virtual  void TabSelected(QWidget*);
virtual  void updateTime();
virtual  void updateHist();
virtual  void drawpbar(); 
virtual  void displaystatus();
virtual  void Abort();
virtual  void ClearHist();
virtual  void ResetRecord();
virtual  void StartOnline();
virtual void InitBaseSetup(); 
virtual void StopTest();
 virtual void setErrorButtonColor();
 virtual void plotCalProfStrip();
 virtual void debugAction();
 virtual void updateLatency();
 virtual void setHwConfigFile();

 private  slots :

 /** handles the high status action when the daqctrl status has changed */
     void updateDaqSettings(QString str);
 /** handles the signals from the test procedure */
 void handleProcedureFeedBack( QString run );
 
 /** is called when the settings tab is changed updates the active tab contents 
  */
 void displaysets();

 
private:
 
 void updatePedestal(Record* therec,bool drawall, int iselm);
 void updateNoise(Record* therec,bool drawall, int iselm);
 void updateCal(Record* therec,bool drawall, int iselm);
 void updateCalProf(Record* therec,bool drawall, int iselm);
 void updateIt(Record* therec);
 void updateVtIt(Record* therec,bool drawall, int iselm);
 void updateLatency(Record* therec,bool drawall, int iselm);
 void updatePipe(Record* therec,bool drawall, int iselm);
 void updateCext(Record* therec,bool drawall, int iselm);
 void updateIV(Record* therec,bool drawall, int iselm);
 void setplots(); 
 /// set the default lt file name field 
 void updateInitDisplay();
 void moduleNrSelect(int& modsel,bool& drawall);
  ModuleTestSetup* thetestsetup;
  MonitorStatus* themstatus;
  OnlineMonitor* theonline;
  int Nmod;
  int Napv;
  const char* setupfile;
  bool foverwrite;
  QTimer* curtimer;
  QTimer* pbartimer;
  QTimer* statustimer;
  QApplication* qapp;

  ModuleInfoGen* fMGen;
  //ModuleInfoDisplayImpl* fMdi;
  // the "plug ins " 
  ModStatusImpl* fMs;
  HybridSettingsImpl* fHs;
  RunSettingsImpl* fRs;
  DaqSettingsImpl* fDs;
  ColdBoxImpl* fCb;
  MuxCtrlImpl* fMc;
  // the related tabs 
  QWidget* fDaqSettingsPage;
  QWidget* fMuxCtrlPage;
  QWidget* fColdBoxPage;
  QPen* bluepen;
 QPen* greenpen;
 QPen* redpen;
 QColor* b[10];
 QColor* r[10];
 QColor* g[10]; 
 QPen* bpen[10];
 QPen* rpen[10];
 QPen* gpen[10];



};
#endif
