#include "DaqSettingsImpl.h"
#include "ModuleDescriptor.h"
#include "ModuleTestSetup.h"
#include "TestProcedure.h"
#include "TestStatus.h"
#include "DaqCtrl.h"
#include "daqerrormsg.h"
#include "qapplication.h" 
#include <qstring.h>
#include <qpushbutton.h>
#include <qspinbox.h>
#include <qlcdnumber.h>
#include <qradiobutton.h>
#include <iostream>
#include "zoomplot.h"
#include <qwt_plot.h>
#include "fedprocessctrl.h"

using namespace std;
DaqSettingsImpl::DaqSettingsImpl(    QWidget* parent ,
					      const char* name , 
					      WFlags fl  ):
 DaqSettingsUi( parent,  name )
{
 fDaqCtrl=0;
 fMonRunning=false;
 fLastReadEvent=0;
 fScopeMode=false;
 ZPRaw->setAxisScale(QwtPlot::yLeft,0,512);
 ZPRaw->setTitle("Raw Data");
 ZPRaw->enableGridX(false);
 ZPRaw->setCanvasBackground(white);
 ZPRaw->setAxisScale(QwtPlot::xBottom,0,140);
 
}

DaqSettingsImpl::~DaqSettingsImpl(){

}

void DaqSettingsImpl::setDaqCtrl( DaqCtrl* daqctrl ) {
  fDaqCtrl=daqctrl; 
  writeDaqSettingsDisplay();
}

void DaqSettingsImpl::writeDaqSettingsDisplay(){
  if( fDaqCtrl){
    SBSpillSize->setValue(fDaqCtrl->getSpillSize());
    SBTscLat->setValue(fDaqCtrl->getTscLatency());
  }
}

 
/*
 * slot from settings menu
 * init the pll and set the delays 
*/
void DaqSettingsImpl::FedThresholdFinding(){
  int low=0; int high =0; 
  if( fDaqCtrl ) {     
    fDaqCtrl->AdjustFedThresholds( low,high) ;
    LCDFedLowTh->display(low); 
    LCDFedHighTh->display(high); 
  }
  return;
}

void DaqSettingsImpl::SetFedClkDelay(){
  if(fDaqCtrl ) {
    fDaqCtrl->stopDaqTest();
    int delay=SBFedClkDelay->text().toInt();
    fDaqCtrl->setFedClkDelay(delay);
    SBFedClkDelay->setValue(delay);    
  }
  return;
}

void DaqSettingsImpl::setScopeMode(bool value){
  if (value != fScopeMode ){
    fScopeMode=value;
    if(fDaqCtrl ) {
      fDaqCtrl->stopDaqTest();
      if(value ) { 
	fDaqCtrl->setScopeMode(1);
      }
      else {
	fDaqCtrl->setScopeMode(0);
      }
      usleep(100000);
      fDaqCtrl->startDaqTest();
    }
  }
}

void DaqSettingsImpl::newSpill(){
  if(fDaqCtrl) {
    fDaqCtrl->NewSpill(true);
    fLastReadEvent=0;
  }
}

void DaqSettingsImpl::FedMon(bool switchon){
  if(fDaqCtrl) {
    if( switchon) {
      if(!fMonRunning){
	fMonRunning=true;
	start();
      }
      else {
	DT_WARN("fed mon already running");
      }
    }
    else{
      if(fMonRunning){
	qApp->lock();
	fMonRunning=false;
	qApp->unlock();
	wait();      
      }
      else{
	DT_WARN("fed mon not running");
      }
    }
  }
}

void DaqSettingsImpl::run(){
  QPen qp[16];
  for (int i=0;i<16;i++){
    qp[i].setColor(QColor(255-i*10,i*10, 128+i*5));
  }
  double  xaxis[140];
  for (int i=0;i<140;i++){ xaxis[i]=i;}
  double apvdata[16][140];
  FedProcessCtrlInterface* fc= fDaqCtrl->getFedProcessCtrl();
  int  nrevents=0;
  while(fMonRunning) {
    if( (nrevents=fDaqCtrl->getNrEvents()) > fLastReadEvent ){
      for(int ch=0;ch<8;ch++) {
	for (int apv=0;apv<2;apv++){
	  DT_DEBUG_GUI("display ch "<< ch  
		       << "event"<< fLastReadEvent<<" of "<<nrevents);
	  for (int strip=0;strip<140;strip++){	    
	    apvdata[ch+apv][strip]= (double)
	      fc->getApvEventData(fLastReadEvent,ch,apv,strip);
	  }
	}
      }
      qApp->lock();
      ZPRaw->clear();
      long curve[16];
      for (int apvnr=0;apvnr<16;apvnr++){
	curve[apvnr]=ZPRaw->insertCurve("APV 1");
	ZPRaw->setCurveData(curve[apvnr], xaxis, apvdata[apvnr], 140);
	ZPRaw->setCurvePen(curve[apvnr],qp[apvnr]);
	ZPRaw->setCurveStyle(curve[apvnr],QwtCurve::Steps,0);
	ZPRaw->replot();
      }
      qApp->unlock();
      fLastReadEvent++;
      usleep(100); // go back to main control ?
    }
    else { msleep(200);}
  }
}

/*
 * slot from settings menu
 * set i2c settings from settings menu to the objects 
*/
void DaqSettingsImpl::ApplyTscSettings() {
  if(fDaqCtrl) {
    fDaqCtrl->setTscLatency(SBTscLat->text().toInt());
    fDaqCtrl->setSpillSize(SBSpillSize->text().toInt());
    if(RBenableReset->isChecked()) {
      DT_DEBUG_GUI("enable reset ");
      fDaqCtrl->enableReset();
      emit DaqSettingsChanged("EnableTscReset");
      // effect for Monitor impl: 
      //thetestsetup->getTestProcedure()->status()->SetApvctrl();      
    } 
    else  {
      DT_DEBUG_GUI("disable reset ");
      fDaqCtrl->disableReset();     
      emit DaqSettingsChanged("DisableTscReset");   
      // effect for Monitor impl:
      //thetestsetup->getTestProcedure()->status()->ResetApvctrl();
    }         
  }
}


