#include "ModuleTestSetup.h"
#include "Monitorimpl.h"
#include "ModuleInfoGen.h"
#include "ModuleInfoDisplayImpl.h"
#include "HybridSettingsImpl.h"
#include "DaqSettingsImpl.h"
#include "modstatusimpl.h"
#include "muxctrlimpl.h"
#include "coldboximpl.h"
#include "RunSettingsImpl.h"
#include "TestProcedure.h"
#include "Module.h"
#include "ProcedureDescriptor.h"
#include "SetupDescriptor.h"
#include "ModuleDescriptor.h"
#include "OnlineMonitor.h"
#include "ltparser.h"
#include "Record.h"
#include "RecModule.h"
#include "RecModTest.h"
#include "RecModHeader.h"
#include "RecModSctrl.h"
#include "RecSetup.h"
#include "ModId.h"
#include "MonitorStatus.h"
#include "LtDefs.h"
#include "benchposconnection.h"
#include "setparser.h"
#include "daqerrormsg.h"
#include "TestStatus.h"
#include "cmdlinedecode.h"
#include "qframe.h"
#include "kled.h"
extern LtParser *ltparser; 

using namespace std;

#define TDTIME  2000
#define TDSTATUS  10000
#define TDPB  10000

 
/*
 *    main GUI class
*/


 
Monitorimpl::Monitorimpl(const char* file,QApplication* a,QWidget* parent , const char* name , WFlags fl)
  : Monitor( parent,name,fl) {
  // set tab widgets  
  // init tab tab name is modules 
  //fMdi=new ModuleInfoDisplayImpl(ltparser,modules,"init");
  fMGen=new  ModuleInfoGen(ltparser,modules,"init");
  fHs= new HybridSettingsImpl(this,HybridSettingsPage,"HybridSettings");
  fRs= new RunSettingsImpl(this,RunSettingsPage,"RunSettings");
  fMs= new ModStatusImpl(ModuleStatus,"ModuleStatus");
  fDs=0;
  fMc=0;
  fDaqSettingsPage=0;
  fMuxCtrlPage=0;
   thetestsetup=0;
  Napv=6;
  setupfile=file;
  themstatus=new MonitorStatus();
  qapp=a;
 connect(TabSteering, SIGNAL( currentChanged(QWidget*) ), this, SLOT(TabSelected(QWidget*)));
 connect(TWPlot, SIGNAL( currentChanged(QWidget*) ),this,SLOT(updateHist()) );
 connect(TabSettings, SIGNAL( currentChanged(QWidget*)),this,SLOT(displaysets()) );
 curtimer=new QTimer(this);
 connect(curtimer,SIGNAL(timeout()),this,SLOT(updateTime()) );
 statustimer=new QTimer(this);
 connect(statustimer,SIGNAL(timeout()),this,SLOT(displaystatus()) );
 pbartimer=new QTimer(this);
 connect(pbartimer,SIGNAL(timeout()),this,SLOT(drawpbar()) );
  PBLoad->setEnabled(false);
  updateInitDisplay();
DT_NOTE("Monitor is constructed ");
  PBScenario->setReadOnly(true);
  PBScenario->setText("no scenario selected");
#ifdef  LTDEBUG1 
  DaqToolsErrorMessage dte;
  int pl=dte.getPrintingLevel();
  QString s=QString("0x%1").arg(pl,0,16);
  LEPrintingLevel->setText(s);
#else 
  PBAuxChipRun->hide();
  LEPrintingLevel->hide();
  TLPrintingLevel->hide();
  PBDebugAction->hide();
#endif
}





Monitorimpl::~Monitorimpl() {
  delete curtimer;
  delete statustimer;
  delete pbartimer;
  if(thetestsetup){ delete thetestsetup; thetestsetup=0;}
  if(themstatus) {delete  themstatus; }
  delete fMGen;
}

void Monitorimpl::changePrintingLevel(const QString& s){
  int pl= (int)strtol( s.latin1(),0,0);
  DaqToolsErrorMessage dte(pl);
}

bool Monitorimpl::initDone() { return themstatus->initDone();}



/*
 * public slots
 ********************
 * scenario steering from the control menu
*/
void  Monitorimpl::StopTest(){
  curtimer->stop();
  statustimer->stop();
//signal connection is disconnected when TestProcedure is destroyed 
  if( thetestsetup->StopTest() < 0) { 
    QMessageBox::critical( this, "Monitor","Not able to remove existing modules , restart program","OK",0);
    PBStopTest->setEnabled(false);  
    return ;
  }
  thetestsetup->powerOffNotRegisteredMod( );
  themstatus->resetInit();
  fMGen->resetModuleInfo();
  PBLoad->setEnabled(true);
  PBStopTest->setEnabled(false);
}

void Monitorimpl::setHwConfigFile(){
  QString startdir(ltparser->getStandardConfigPath().c_str());
  QString ltf(QFileDialog::getOpenFileName( startdir, "Lt (*.xml)", this ) );
  
  if ( ltf){
    ltparser->setParseFileName(ltf.latin1());
  }
  updateInitDisplay();
}

void Monitorimpl::InitBaseSetup(){
  if(ltparser->parseFile()) {    
    QString  s1("syntax error in lt configuration file: ");
    s1 += (ltparser->getParseFileName()).c_str();
    s1 += "\n";
    s1 += " look at the  error output in mainMonitor xterm and edit the file of choose an other one";   
    QMessageBox::critical( this, "Config error",s1,"OK",0);
    ltparser->clear();
    PBLoadSetup->setOn(false);
    return;
  } 
  fMGen->setActiveBenchEntries();
 thetestsetup= new ModuleTestSetup();	
 LESettingFile->setText(thetestsetup->getSetParser()->getFileName().c_str());
 fHs->setTestSetup(thetestsetup);
 fRs->setTestSetup(thetestsetup);
// set the expert mode settings 
 // check if expert mode is required 
 string expertmode;
 if("found" == CmdLineDecode(qApp->argc(),qApp->argv(),"expmode",
			       expertmode,false)
 ) {
   fDaqSettingsPage = new QWidget( TabSettings, "DaqSettingsPage" );
   TabSettings->insertTab( fDaqSettingsPage, trUtf8( "DAQ" ) );
   fDs=new DaqSettingsImpl(fDaqSettingsPage,"DaqSettings");
   QRect df(fDs->frameGeometry());
   df.moveBy(0,0);
   fDs->setGeometry(df);
   fDs->setDaqCtrl(thetestsetup->getDaqCtrl() );
   connect(fDs, SIGNAL( DaqSettingsChanged(QString)),
	   this, SLOT(updateDaqSettings(QString))) ;
   DT_DEBUG_GUI("daqsettings window created");
   if( ltparser->getNrMuxDevices()) {
     fMuxCtrlPage = new QWidget( TabSettings, "MuxCtrlPage" );
     TabSettings->insertTab( fMuxCtrlPage, trUtf8( "Mux" ) );
     fMc=new MuxCtrlImpl(fMuxCtrlPage,"MuxCtrl");
     fMc->setMuxCtrl(thetestsetup->getDaqCtrl() );
     fMc->show();
     DT_DEBUG_GUI("muxctrl  window created");
   }


   if( thetestsetup->coldBoxValid()){
     fColdBoxPage = new QWidget( TabSettings, "ColdBoxPage" );
     TabSettings->insertTab(fColdBoxPage , trUtf8( "ColdBox" ) );
     fCb=new ColdBoxImpl(fColdBoxPage,"cold box ctrl");
     fCb->setColdBox(thetestsetup->getColdBox());
     fCb->show();
   }
   
   
 }

 PBLoadSetup->setEnabled(false);
 PBHwConfigFile->setEnabled(false);
 PBLoad->setEnabled(true);
 theonline=new OnlineMonitor(thetestsetup->getRecord());
 if(RBOnline->isChecked()) theonline->StartThread();
 DT_DEBUG_GUI(" started monitor timers ");
}

void Monitorimpl::InitSetup()  {
  if(!themstatus->setInit()) { 
    QMessageBox::information( this, "Monitor",
			      "Setup is already initialized\n"
			      "" ); 
    DT_WARN("Monitor:setup already initialised ");return;
  }
 
  QString s;
  QString txtstr;
  s.sprintf("  -----start initialization, please wait------");
  StatusBox->insertItem(s);
  StatusBox->ensureCurrentVisible ();
  int nrvalidmodules=fMGen->initModuleInfo();// read the info delete old info
  if(!nrvalidmodules){  
    s="No modules define setup can not be initialized";
    QMessageBox::critical( this, "Monitor",s,"OK",0);
    themstatus->resetInit();
    return;
  }
  DT_DEBUG_GUI("nr of modules " << nrvalidmodules);
  // create the setup info for the message box
  s.setLength(0);
  QString s2;
  txtstr= "Init setup can take time\n";s+=txtstr;
  txtstr= "Run test with these modules :\n";s+=txtstr;
  for (int lc=0;lc< nrvalidmodules ;lc++){
    txtstr= "Bench Pos: "; s+=txtstr;
    ModId*  modid= fMGen->getModIdInfo(lc);
    if(! modid ){ 
      DT_WARN("register module "<<lc<< "has no info");
      continue;
    }
    DT_DEBUG_GUI("modid found "<< lc);
    s2.setNum(1+modid->getBenchPosition());s+=s2;
    txtstr=" Module Id: ";s+=txtstr;
    txtstr=modid->getModId().c_str();s+=txtstr;
    txtstr="\n";s+=txtstr;
  }
  DT_DEBUG_GUI("setup string p2 done :" << s);
  switch (  QMessageBox::information( this, "Monitor",
				  s,
				  "Accept",
				  "Cancel", 0, 1 ) ) {
    case 0:{  
      if(CBPowerAll->isChecked()) {          	 
	thetestsetup->powerOnNotRegisteredMod(fMGen->getNotValidMod());
      }
      if (thetestsetup->AddModules(fMGen->getModIdInfo())==0){
	themstatus->resetInit();
	s="No valid modules defined,  setup can not be initialized";
	QMessageBox::critical( this, "Monitor",s,"OK",0);
	thetestsetup->powerOffNotRegisteredMod();
	return;
      }
      //else 
      thetestsetup->CreateTestProcedure(this);
      connect (thetestsetup->getTestProcedure(),SIGNAL(feedBackSignal(  QString )),
	       this,SLOT(handleProcedureFeedBack( QString )));

    }
    break;
    default:{  
      //thetestsetup=new ModuleTestSetup(setupfile);
      // case cancel or ESC was choosen 
      // do nothing default , no init 
      themstatus->resetInit();
      return;// go back to the init window 
    }        
  }
  
  DT_NOTE(" Procedure : initsetup");
//  setOperator(Operator);
  Nmod=thetestsetup->getNmod();
  SBModule->setMaxValue(Nmod-1);
  Napv=thetestsetup->getNapv();
  if(Nmod==0) return;
  DT_DEBUG_GUI(" Monitor constructed  Nmod "<<Nmod);
  fMs->setSetup(thetestsetup);
  curtimer->start(TDTIME,FALSE);
  statustimer->start(TDSTATUS,FALSE);
//
  
  displaysets();
  setplots();
// init the i2c settings to the default values in the default files 
  
  //SetParser* sets=thetestsetup->getSetParser();
   fHs->writeI2cSettingsDisplay(thetestsetup->getSetParser()->getI2cDescriptor("default"));
   fRs->writeRunSettingsDisplay( thetestsetup->getSetParser());
   if(fDs){ 
     fDs->writeDaqSettingsDisplay();
   }
  s.sprintf(" Time:%d -----Construct Setup------",thetestsetup->getCTime());
  StatusBox->insertItem(s);
  StatusBox->ensureCurrentVisible ();
  DT_NOTE("Monitor: constructed ModuleTestSetup with "<<Nmod<<" nmodules ");
  fMGen->setEditEnable(false);
  PBLoad->setEnabled(false);
  PBStopTest->setEnabled(true);
  TabSteering->setCurrentPage(1);
}
/*
 * public slot
 * control menu
 */
void Monitorimpl::StartScenario()
{
 if(themstatus->setRunScenario() ) {
 QString s;
 s.sprintf(" Time:%d =========Start Scenario",thetestsetup->getCTime());
 StatusBox->insertItem(s);
 StatusBox->ensureCurrentVisible ();
	     if(themstatus->setLoad() )  
	       {thetestsetup->getTestProcedure()->ScenarioSteering("StartScenario");}
             else  {thetestsetup->getTestProcedure()->LoadScenario();  
   thetestsetup->getTestProcedure()->ScenarioSteering("StartScenario");} 

 pbartimer->start(TDPB,FALSE);
 PBScenario->setText("starting scenario");
// PBScenario->setTotalSteps(int(thetestsetup->getTestProcedure()->getSTimeEnd()));
//PBScenario->setProgress(int(thetestsetup->getTestProcedure()->getSTime()));
TLProgressBar->setText(tr("Scenario is running "));
    } else DT_WARN(" cannot start scenario ");
}
/*
 * public slot
 * control menu
 */
void Monitorimpl::StopScenario() {
  if(themstatus->setStopScenario() ) {
            thetestsetup->getTestProcedure()->ScenarioSteering("Stop");
            pbartimer->stop();   
 QString s;
 s.sprintf(" Time:%d #########Stop Scenario ",thetestsetup->getCTime());
 StatusBox->insertItem(s);
 StatusBox->ensureCurrentVisible ();
 PBScenario->setText(PBScenario->displayText() + " stopped");
TLProgressBar->setText(tr("abort Scenario "));
  } else DT_WARN("scenario is not active ");
}
/*
 * public slot
 * control menu
 */
void Monitorimpl::LoadScenario() {
  if(themstatus->setLoad()) {
    DT_NOTE(" Monitor: LoadScenario");
    QString s;
    switch (  QMessageBox::warning( this, "Monitor",
	       "use Default Scenario  file or Select own \n",
				    "Default",
				    "Select", 0, 0, 1 ) ) {
      case 1: {
	QString  sf(QFileDialog::getOpenFileName( 
	  QString::null, "Lt (*.Lt *.lt)", this,
	  "openSenario","Open Scenario File" ) 
	);
	if ( sf.isEmpty() ) { 
	  QMessageBox::information( this, "Monitor",
		   "Can't find test setup file. Try again \n");
	  return;
	}
	else {
	  thetestsetup->getTestProcedure()->LoadScenario(sf);
	  s= sf + "scenario file ";
	  PBScenario->setText(s);
	}
      }
      break;
      case 0: 
      default:  {  
	thetestsetup->getTestProcedure()->LoadScenario();  
	PBScenario->setText("default file  selected");
      }
      break;
    }
    s.sprintf(" Time:%d Load Scenario",thetestsetup->getCTime());
    StatusBox->insertItem(s);
    StatusBox->ensureCurrentVisible ();
  }
}
/*
 * public slot
 * control menu
 */
void Monitorimpl::ContinueScenario() {
DT_NOTE("Monitor: Continue");
if(themstatus->setContinue() ) {
 QString s;
 s.sprintf(" Time:%d -----Continue Scenario ",thetestsetup->getCTime());
 StatusBox->insertItem(s);
 StatusBox->ensureCurrentVisible ();
 pbartimer->start(TDPB,FALSE);
TLProgressBar->setText(tr("Scenario is running "));
 thetestsetup->getTestProcedure()->ScenarioSteering("Continue");
  }
}
/*
 * public slot
 * control menu
 */
void Monitorimpl::PauseScenario() {
  DT_NOTE("Monitor: pause");
   if(themstatus->setPause() ) {

 QString s;
 s.sprintf(" Time:%d -----Pause Scenario ",thetestsetup->getCTime());
 StatusBox->insertItem(s);
 StatusBox->ensureCurrentVisible ();
 pbartimer->stop(); 
TLProgressBar->setText(tr("Pause Scenario "));
 thetestsetup->getTestProcedure()->ScenarioSteering("Pause");
  }
}
/*
 * public slot
 * quit button
 */
void Monitorimpl::Quit() {

QMessageBox d("MainMonitor ","Quit Application\n"
			  "Are You really sure sure?",
			  QMessageBox::Warning,
	                  QMessageBox::Yes,
	                  QMessageBox::No | QMessageBox::Default,
	                  QMessageBox::Cancel | QMessageBox::Escape);
      d.setButtonText(QMessageBox::Yes,"Yes");
      d.setButtonText(QMessageBox::No,"No");
      switch (d.exec())
	{
 case QMessageBox::Yes:{
	    close();
	    // accept();
	    DT_NOTE(" Quit ");}
 break;
 case QMessageBox::No:
	    break;
	}
}
/*
 * public slot
 * steering menu
 */
void Monitorimpl::PedRun() {
  if(themstatus->initDone())  {
    QString s;
    s.sprintf(" Time:%d demanded PedRun ",thetestsetup->getCTime());
    StatusBox->insertItem(s);
    StatusBox->ensureCurrentVisible ();  
    if(thetestsetup->getTestProcedure()->status()->scenarioOn()) {
      QMessageBox::information( this, "Monitor"," Scenario is active\n"
                                                         "press Pause");
    }
    thetestsetup->getTestProcedure()->StartTest("PedRun");
    displaystatus();
  }   
}
/*
 * public slot
 * steering menu
 */
void Monitorimpl::CalProfRun() {
  if(themstatus->initDone())  { 
 QString s;
 s.sprintf(" Time:%d demanded CalProfRun",thetestsetup->getCTime());
 StatusBox->insertItem(s);
 StatusBox->ensureCurrentVisible ();
  if(thetestsetup->getTestProcedure()->status()->scenarioOn()) 
 QMessageBox::information( this, "Monitor"," Scenario is active\n"
                                                         "press Pause");   
      thetestsetup->getTestProcedure()->StartTest("CalProfRun");
  }
  displaystatus();
}
/*
 * public slot
 * steering menu
 */
void Monitorimpl::CalRun()  {  
 if(themstatus->initDone())  { 
 QString s;
 s.sprintf(" Time:%d demanded CalRun",thetestsetup->getCTime());
 StatusBox->insertItem(s);
 StatusBox->ensureCurrentVisible ();
  if(thetestsetup->getTestProcedure()->status()->scenarioOn()) 
 QMessageBox::information( this, "Monitor"," Scenario is active\n"
                                                         "press Pause");
       thetestsetup->getTestProcedure()->StartTest("CalRun");
  }
 displaystatus();
}
/*
 * public slot
 * backplane run
 */
void Monitorimpl::BPlaneRun() {
 if(themstatus->initDone())  { 
 QString s;
 s.sprintf(" Time:%d demanded BPRun" ,thetestsetup->getCTime());
 StatusBox->insertItem(s);
 StatusBox->ensureCurrentVisible ();
  if(thetestsetup->getTestProcedure()->status()->scenarioOn()) 
 QMessageBox::information( this, "Monitor"," Scenario is active\n"
                                                         "press Pause");

       thetestsetup->getTestProcedure()->StartTest("BPlaneRun");
  }
 displaystatus();
}
/*
 * public slot
 * led  run
 */
void Monitorimpl::LedRun() {
  if(themstatus->initDone())  { 
 QString s;
 s.sprintf(" Time:%d demanded LedRun" ,thetestsetup->getCTime() );
  StatusBox->insertItem(s);
  StatusBox->ensureCurrentVisible ();
  if(thetestsetup->getTestProcedure()->status()->scenarioOn()) 
 QMessageBox::information( this, "Monitor"," Scenario is active\n"
                                                         "press Pause");      
       thetestsetup->getTestProcedure()->StartTest("LedRun");
      
  }
  displaystatus();
}
/*
 * public slot
 * light run
 */
void Monitorimpl::LightRun() {
  if(themstatus->initDone())  { 
 QString s;
 s.sprintf(" Time:%d demanded LightRun" ,thetestsetup->getCTime() );
  StatusBox->insertItem(s);
  StatusBox->ensureCurrentVisible ();
  if(thetestsetup->getTestProcedure()->status()->scenarioOn()) 
 QMessageBox::information( this, "Monitor"," Scenario is active\n"
                                                         "press Pause");      
       thetestsetup->getTestProcedure()->StartTest("LightRun");
      
  }
  displaystatus();
}


/*
 * public slot
* fb run
*/
void Monitorimpl::FBiasRun() {
  if(themstatus->initDone())  { 
 QString s;
 s.sprintf(" Time:%d demanded FbRun" ,thetestsetup->getCTime() );
  StatusBox->insertItem(s);
  StatusBox->ensureCurrentVisible ();
  if(thetestsetup->getTestProcedure()->status()->scenarioOn()) 
 QMessageBox::information( this, "Monitor"," Scenario is active\n"
                                                         "press Pause");      
       thetestsetup->getTestProcedure()->StartTest("FbRun");
      
  }
  displaystatus();
}

/*
 * public slot
 * steering menu 
 */
void Monitorimpl::LatencyRun() {
  if(themstatus->initDone())  { 
 QString s;
 s.sprintf(" Time:%d demanded Latency scan",thetestsetup->getCTime());
 StatusBox->insertItem(s);
 StatusBox->ensureCurrentVisible ();
  if(thetestsetup->getTestProcedure()->status()->scenarioOn()) 
 QMessageBox::information( this, "Monitor"," Scenario is active\n"
                                                         "press Pause");

 thetestsetup->getTestProcedure()->StartTest("LatRun");
  }
}
/*
 * public slot
 * steering menu 
 */
void Monitorimpl::IVRun() {
 if(themstatus->initDone())  { 
 QString s;
 s.sprintf(" Time:%d demanded IVRun ",thetestsetup->getCTime());
 StatusBox->insertItem(s);
 StatusBox->ensureCurrentVisible ();
  if(thetestsetup->getTestProcedure()->status()->scenarioOn()) 
 QMessageBox::information( this, "Monitor"," Scenario is active\n"
                                                         " press Pause");

    thetestsetup->getTestProcedure()->StartTest("IVRun");
  }
 displaystatus();
}


void Monitorimpl::AuxChipRun() 
{
  QString s;
  
  if(themstatus->initDone())   {
    PBAuxChipRun->setPaletteBackgroundColor(QColor(85,255,255));
      s.sprintf(" Time:%d AuxChip",thetestsetup->getCTime());
      StatusBox->insertItem(s);
      StatusBox->ensureCurrentVisible ();
      if(thetestsetup->getTestProcedure()->status()->scenarioOn()) 
	QMessageBox::information( this, "Monitor"," Scenario is active\n",
				  "press Pause");
      thetestsetup->getTestProcedure()->StartTest("AuxChipRun");
      displaystatus();
    
    }
}

void Monitorimpl::ConnectivityRun() 
{
  QString s;
  
  if(themstatus->initDone())   {
    PBConnectivity0->setPaletteBackgroundColor(QColor(85,255,255));
      s.sprintf(" Time:%d Connect",thetestsetup->getCTime());
      StatusBox->insertItem(s);
      StatusBox->ensureCurrentVisible ();
      if(thetestsetup->getTestProcedure()->status()->scenarioOn()) 
	QMessageBox::information( this, "Monitor"," Scenario is active\n",
				  "press Pause");
      thetestsetup->getTestProcedure()->StartTest("ConnectivityRun");
      displaystatus();
    
    }
}

/*
 * public slot
 * steering menu
 */
void Monitorimpl::ChangeHV() {
   if(themstatus->initDone())  { 
  QString s;
 s.sprintf(" Time:%d demanded ChangeHV ",thetestsetup->getCTime() );
 StatusBox->insertItem(s);
 StatusBox->ensureCurrentVisible ();
   thetestsetup->getTestProcedure()->ChangeHV(SBHV->text().toFloat());
  }
   displaystatus();
}
/*
 * public slot
 * steering menu
 */
void Monitorimpl::ChangeTemp() {
   if(themstatus->initDone())  { 
QString s;
s.sprintf(" Time:%d demanded changeTemp  ",thetestsetup->getCTime() );
 StatusBox->insertItem(s);
 StatusBox->ensureCurrentVisible ();
  if(thetestsetup->getTestProcedure()->status()->scenarioOn()) 
 QMessageBox::information( this, "Monitor"," Scenario is active\n"
                                                         " press Pause");

   thetestsetup->getTestProcedure()->ChangeTemp(SBChangeTemp->text().toFloat());
  }
}
/*
 * public slot
 * steering menu
 */
void Monitorimpl::CheckSlowctrl() {
  if(themstatus->initDone())  { 
QString s;
s.sprintf(" Time:%d demanded CheckSlowctr ",thetestsetup->getCTime() );
 StatusBox->insertItem(s);
 StatusBox->ensureCurrentVisible ();
 if(thetestsetup->getTestProcedure()->status()->scenarioOn()) 
 QMessageBox::information( this, "Monitor"," Scenario is active\n"
                                                         " press Pause");
    thetestsetup->getTestProcedure()->CheckEnv();
  }
  displaystatus();
}

/*
 * public slot
 * read settings
 */
void Monitorimpl::ReadSettings() {
// can always be done if the setup exist
  if( thetestsetup) {
    StatusBox->insertItem(" ReadSettings");
    StatusBox->ensureCurrentVisible ();
    switch (  QMessageBox::warning( this, "Monitor",
                   "use Default settings.xml   or Select own \n",
				    "Default",
				    "Select","Cancel", 0, 2 ) ) {
      case 0:  {   
	char* envir=getenv("DAQHOME");
	string s(envir);
	string setfile=s+"/Config/settings.xml";
	QString set(setfile.c_str());
	thetestsetup->ParseSettings(set);
	LESettingFile->setText(set);
      }
      break;
      case 1: {
	QString  sf(QFileDialog::getOpenFileName( 
	  QString::null, "(*.xml)", this ) );
	if(sf.isEmpty()) {
	  QMessageBox::information( this, "Monitor",
				    "Cann't find  file. Try again \n"); 
	  return;
	}
	else {
	  thetestsetup->ParseSettings(sf);
	  LESettingFile->setText(sf);
	}
      }
      break;
      case 2:
      default:
	return;
	break;        
    }
  }
}

/*
 * public slot
 * save results button in the steering menu
 */
void Monitorimpl::SaveResults() {
  if(themstatus->initDone() ) {
    StatusBox->insertItem(" Save results");
    StatusBox->ensureCurrentVisible ();
    switch (  QMessageBox::warning( this, "Monitor",
                  "use Default Results file  or Select another \n",
                   "Default", "Select", "Cancel", 0, 2 ) ) {
      case 0: {   
	thetestsetup ->getTestProcedure()->SaveResults("default",this);
      }
      break;
      case 1: {
	QString  sf(QFileDialog::getSaveFileName( 
	  QString::null, "Root (*.root)", this ) );
	if ( sf.isEmpty() ) { 
	  QMessageBox::information( this, "Monitor",
				    "Cann't open file file. Try again \n"); 
	  return;
	}
	else {
	  thetestsetup ->getTestProcedure()->SaveResults(sf,this);   
	}
      }
      break;
      case 2:
      default: {
	// noting to do
	//  thetestsetup ->getTestProcedure()->SaveResults("default",this);
       }        
      break;
    }
  }
}
/*
 * public slot
 * save to the Db xml file
 */
void Monitorimpl::SaveDb()
{
 if(themstatus->initDone() ) {
StatusBox->insertItem(" Save Db action");
StatusBox->ensureCurrentVisible ();
  QString s(QFileDialog::getOpenFileName( QString::null, " *.xml", this ) );
  if ( s.isEmpty() ) return;
   thetestsetup->getTestProcedure()->SaveDb(s); 
 }
}

/*
 * public slot
 * abort test
 */
void Monitorimpl::Abort() {
  qApp->lock();
  if(themstatus->initDone() ) {
    DT_NOTE(" Monitor: abort ");
    StatusBox->insertItem(" Abort test");
StatusBox->ensureCurrentVisible ();
    thetestsetup->getTestProcedure()->Abort();
  }
  qApp->unlock();
}
/*
 * main menu  tabs selection slot
 */
void Monitorimpl::TabSelected(QWidget*)
{
  int idx=TabSteering->currentPageIndex();
  if(idx==0) updateInitDisplay();
  if(idx==1) displaystatus();
  if(idx==2) ;
  if(idx==4) updateHist();
  if(idx==3) displaysets();
  if(idx==5) { fMs->updateModuleStatus();}
}

void  Monitorimpl::updateInitDisplay(){
  QString s(ltparser->getParseFileName().c_str());
  s= s.section('/',-1);
  LEHwConfigFile->setText(s); 
}


/*
 *  scenario bar  timer slot
 * display progress bar in the control menu
 */
void Monitorimpl::drawpbar() {
  if(themstatus->initDone() ) {
    //float ct= thetestsetup->getTestProcedure()->getSTime();
    // float et=thetestsetup->getTestProcedure()->getSTimeEnd();
    PBScenario->setText(
      thetestsetup->getTestProcedure()->getCurrentScenarioLineInfo()
    );
   
    // PBScenario->setTotalSteps((int)et);
    //PBScenario->setProgress(int(ct+0.5)); 
    
    if(!thetestsetup->getTestProcedure()->status()->scenarioOn() && themstatus->scenarioOn()) {
      themstatus->setStopScenario(); 
      pbartimer->stop();
      QString s;
      s.sprintf(" Time:%d #########End Scenario ",thetestsetup->getCTime());
      StatusBox->insertItem(s);
      StatusBox->ensureCurrentVisible ();
      //PBScenario->setProgress(int(et+0.5));
      TLProgressBar->setText(tr("Scenario is done "));  
    }
  }
}
/*
 *  private
 * display status in the control menu
 */
void Monitorimpl::displaystatus() {
  if(themstatus->initDone() ) {
 int nbad=0;
 int  nerror=0;
 vector<RecModule*> recmod=thetestsetup->getRecord()->getmodules();
 for(vector<RecModule*>::iterator i=recmod.begin();i!=recmod.end();i++) {
   nbad+=(*i)->getmodheader()->getnbadch();
   nerror+=(*i)->getmodheader()->getnapverrors();
 }
 LCDNbad->display(nbad);
 LCDNerror->display(nerror);
 LCDHybRun->display(thetestsetup->getTestProcedure()->getNacq());
  LCDTemp->display(thetestsetup->getRecord()->getsetup()->gettempbox());
  LCDHum->display(thetestsetup->getRecord()->getsetup()->gethumbox());
 //
 SBHV->setValue((int)thetestsetup->getRecord()->getfirstmodule()->getmodsctrl()->gethvbias());
 SBChangeTemp->setValue((int)thetestsetup->getRecord()->getsetup()->gettemptag());
//
 LETestStatus->setReadOnly(true);
 QString s=thetestsetup->getTestProcedure()->status()->currentTest()+"  is Running";
 LETestStatus->setText(s);

 }
  
  
}

void Monitorimpl::setErrorButtonColor(){
  int conectstatus =
    thetestsetup->getTestProcedure()->status()
    ->getErrorStatus("ConnectivityRun");
  if(conectstatus |= 0 ){
    if (conectstatus > 0){ 
      PBConnectivity0->setPaletteBackgroundColor(green);      
    }
    else {
      PBConnectivity0->setPaletteBackgroundColor(red);	 
    }    
  }
  

}


/*
 * elapsed time slot
 * update time in the control menu
 */
void  Monitorimpl::updateTime()
{
  LCDTime->display(int(thetestsetup->getCTime()));
}
/*
 * set plots seetings for monitor menu
 *  
 */
#ifdef QWT3
#define setCanvasBackground setPlotBackground
#endif 

void Monitorimpl::setplots()
{
 DT_DEBUG_GUI("Monitor: setplots ");
 QFont f("Times",9);
  int nfr=Napv*140;
ZPFrame->setAxisScale(QwtPlot::yLeft,0,512);
ZPFrame->setTitleFont(f);
ZPFrame->setTitle("Frame");
ZPFrame->enableGridX(false);
ZPFrame->setCanvasBackground(white);
ZPFrame->setAxisFont(QwtPlot::xBottom,f);
ZPFrame->setAxisFont(QwtPlot::yLeft,f);
ZPFrame->setAxisScale(QwtPlot::xBottom,0,nfr+1);
ZPFrame->setAxisTitle(QwtPlot::xBottom, "ch.");
ZPFrame->setAxisTitle(QwtPlot::yLeft, "adc");

  int nch=Napv*NAPVCH;
ZPPed->setAxisScale(QwtPlot::yLeft,0,512);
ZPPed->setTitleFont(f);
ZPPed->setTitle("Pedestal");
ZPPed->enableGridX(false);
ZPPed->setCanvasBackground(white);
ZPPed->setAxisFont(QwtPlot::xBottom,f);
ZPPed->setAxisFont(QwtPlot::yLeft,f);
ZPPed->setAxisScale(QwtPlot::xBottom,0,nch+1);
ZPPed->setAxisTitle(QwtPlot::xBottom, "ch.");
ZPPed->setAxisTitle(QwtPlot::yLeft,"adc");

//ZPNoise->setAxisScale(QwtPlot::yLeft,0,20);
ZPNoise->setTitleFont(f);
ZPNoise->setTitle("Noise RMS and rawRMS");
ZPNoise->enableGridX(false);
ZPNoise->setCanvasBackground(white);
ZPNoise->setAxisFont(QwtPlot::xBottom,f);
ZPNoise->setAxisFont(QwtPlot::yLeft,f);
ZPNoise->setAxisScale(QwtPlot::xBottom,0,nch+1);
ZPNoise->setAxisTitle(QwtPlot::xBottom, "ch.");
ZPNoise->setAxisTitle(QwtPlot::yLeft, "adc");


ZPCalA->setAxisScale(QwtPlot::yLeft,-100,200);
ZPCalA->setTitleFont(f);
ZPCalA->setTitle("CalA");
ZPCalA->enableGridX(false);
ZPCalA->setCanvasBackground(white);
ZPCalA->setAxisFont(QwtPlot::xBottom,f);
ZPCalA->setAxisFont(QwtPlot::yLeft,f);
ZPCalA->setAxisScale(QwtPlot::xBottom,0,nch+1);
ZPCalA->setAxisTitle(QwtPlot::xBottom, "ch.");
ZPCalA->setAxisTitle(QwtPlot::yLeft, "Acal,adc");

ZPCalAprof->setAxisScale(QwtPlot::yLeft,-100,200);
ZPCalAprof->setTitleFont(f);
ZPCalAprof->setTitle("CalAprof");
ZPCalAprof->enableGridX(false);
ZPCalAprof->setCanvasBackground(white);
ZPCalAprof->setAxisFont(QwtPlot::xBottom,f);
ZPCalAprof->setAxisFont(QwtPlot::yLeft,f);
ZPCalAprof->setAxisScale(QwtPlot::xBottom,0,nch+1);
ZPCalAprof->setAxisTitle(QwtPlot::xBottom, "ch.");
ZPCalAprof->setAxisTitle(QwtPlot::yLeft, "Acalprof,adc");

ZPCalProf->setAxisScale(QwtPlot::yLeft,-100,100);
ZPCalAprof->setTitleFont(f);
ZPCalProf->setTitle("Channel CalProfile");
ZPCalProf->enableGridX(true);
ZPCalProf->setCanvasBackground(white);
ZPCalProf->setAxisFont(QwtPlot::xBottom,f);
ZPCalProf->setAxisFont(QwtPlot::yLeft,f);
ZPCalProf->setAxisScale(QwtPlot::xBottom,0,350);
ZPCalProf->setAxisTitle(QwtPlot::xBottom, "t,ns");
ZPCalProf->setAxisTitle(QwtPlot::yLeft, "A,adc");



//ZPIV->setAxisScale(QwtPlot::yLeft,0,10);
ZPIV->setTitle("IV ");
ZPIV->enableGridX(true);
ZPIV->setCanvasBackground(white);
//ZPIV->setAxisScale(QwtPlot::xBottom,0,1000);
ZPIV->setAxisTitle(QwtPlot::xBottom, "Volt,V");
ZPIV->setAxisTitle(QwtPlot::yLeft, "Ileak,nA");



//ZPTt->setAxisScale(QwtPlot::yLeft,-40,40);
//ZPTt->setTitle("T-time");
ZPTt->enableGridX(true);
ZPTt->setCanvasBackground(white);
ZPTt->setAxisFont(QwtPlot::xBottom,f);
ZPTt->setAxisFont(QwtPlot::yLeft,f);
ZPTt->setAxisScale(QwtPlot::xBottom,0,1000);
ZPTt->setAxisTitle(QwtPlot::xBottom, "time,s");
ZPTt->setAxisTitle(QwtPlot::yLeft, "T,C");
 ZPTt->setAutoLegend(true);
 ZPTt->setLegendFont(f);
 ZPTt->setLegendPos(Qwt::Bottom);
//ZPTt->enableLegend(true);


//ZPHt->setAxisScale(QwtPlot::yRight,0,70);
//ZPHt->setTitle("RH-time");
ZPHt->enableGridX(true);
ZPHt->setCanvasBackground(white);
ZPHt->setAxisFont(QwtPlot::xBottom,f);
ZPHt->setAxisFont(QwtPlot::yLeft,f);
ZPHt->setAxisScale(QwtPlot::xBottom,0,1000);
ZPHt->setAxisTitle(QwtPlot::xBottom, "time,s");
ZPHt->setAxisTitle(QwtPlot::yLeft, "RelHum,%");
ZPHt->setAutoLegend(true);
ZPHt->setLegendFont(f);
 ZPHt->setLegendPos(Qwt::Bottom);
//ZPHt->enableLegend(TRUE);

//ZPVt->setAxisScale(QwtPlot::yLeft,0,400);
ZPVt->setTitleFont(f);
ZPVt->setTitle("V-time");
ZPVt->enableGridX(true);
ZPVt->setCanvasBackground(white);
ZPVt->setAxisFont(QwtPlot::xBottom,f);
ZPVt->setAxisFont(QwtPlot::yLeft,f);
ZPVt->setAxisScale(QwtPlot::xBottom,0,1000);
ZPVt->setAxisTitle(QwtPlot::xBottom, "time,s");
ZPVt->setAxisTitle(QwtPlot::yLeft, "HV,V");

//ZPIt->setAxisScale(QwtPlot::yLeft,0,10);
ZPIt->setTitleFont(f);
ZPIt->setTitle("I-t");
ZPIt->enableGridX(true);
ZPIt->setCanvasBackground(white);
ZPIt->setAxisFont(QwtPlot::xBottom,f);
ZPIt->setAxisFont(QwtPlot::yLeft,f);
ZPIt->setAxisScale(QwtPlot::xBottom,0,1000);
ZPIt->setAxisTitle(QwtPlot::xBottom, "time,s");
ZPIt->setAxisTitle(QwtPlot::yLeft, "Ileak,nA");

//ZPPipe->setAxisScale(QwtPlot::yLeft,0,5000);
ZPPipe->setTitleFont(f);
ZPPipe->setTitle("Pipeaddress");
ZPPipe->enableGridX(true);
ZPPipe->setCanvasBackground(white);
ZPPipe->setAxisFont(QwtPlot::xBottom,f);
ZPPipe->setAxisFont(QwtPlot::yLeft,f);
ZPPipe->setAxisScale(QwtPlot::xBottom,0,500);
ZPPipe->setAxisTitle(QwtPlot::xBottom, "pipeaddress");
ZPPipe->setAxisTitle(QwtPlot::yLeft, "N");

//ZPLat->setAxisScale(QwtPlot::yLeft,-50,50);
ZPLat->setTitleFont(f);
ZPLat->setTitle("Latscan");
ZPLat->enableGridX(true);
ZPLat->setCanvasBackground(white);
ZPLat->setAxisFont(QwtPlot::xBottom,f);
ZPLat->setAxisFont(QwtPlot::yLeft,f);
ZPLat->setAxisScale(QwtPlot::xBottom,0,200);
ZPLat->setAxisTitle(QwtPlot::xBottom, "latency");
ZPLat->setAxisTitle(QwtPlot::yLeft, "Asig,adc");

//ZPFlagCh->setAxisScale(QwtPlot::yLeft,0,500);
ZPFlagCh->setTitleFont(f);
ZPFlagCh->setTitle("BadChannels");
ZPFlagCh->enableGridX(true);
ZPFlagCh->setCanvasBackground(white);
ZPFlagCh->setAxisFont(QwtPlot::xBottom,f);
ZPFlagCh->setAxisFont(QwtPlot::yLeft,f);
ZPFlagCh->setAxisScale(QwtPlot::xBottom,0,nch);
ZPFlagCh->setAxisTitle(QwtPlot::xBottom, "ch");
ZPFlagCh->setAxisTitle(QwtPlot::yLeft, "ChFlag");

//ZPBPlane->setAxisScale(QwtPlot::yLeft,-100,500);
ZPBPlane->setTitleFont(f);
ZPBPlane->setTitle("BackPlane");
ZPBPlane->enableGridX(true);
ZPBPlane->setCanvasBackground(white);
ZPBPlane->setAxisFont(QwtPlot::xBottom,f);
ZPBPlane->setAxisFont(QwtPlot::yLeft,f);
ZPBPlane->setAxisScale(QwtPlot::xBottom,0,nch);
ZPBPlane->setAxisTitle(QwtPlot::xBottom, "ch");
ZPBPlane->setAxisTitle(QwtPlot::yLeft, "Abpl,adc");

//ZPLed->setAxisScale(QwtPlot::yLeft,-100,500);
ZPLed->setTitleFont(f);
ZPLed->setTitle("LedProfile");
ZPLed->enableGridX(true);
ZPLed->setCanvasBackground(white);
ZPLed->setAxisFont(QwtPlot::xBottom,f);
ZPLed->setAxisFont(QwtPlot::yLeft,f);
ZPLed->setAxisScale(QwtPlot::xBottom,0,nch);
ZPLed->setAxisTitle(QwtPlot::xBottom, "ch");
ZPLed->setAxisTitle(QwtPlot::yLeft, "Aled,adc");

// set colors
bluepen=new QPen(blue,1);
greenpen=new QPen(green,1);
redpen=new QPen(red,1);
for(int ic=0;ic<10;ic++) {
    bpen[ic]=new QPen( QColor(ic*10,0,255-ic*10),1);
    rpen[ic]=new QPen( QColor(255-ic*10,0,ic*10),1);
    gpen[ic]=new QPen( QColor(ic*10,255-ic*10,0),1);
 }

}
/*
 * monitor menu slot
 * extract data and plot them in monitor menu
 * also store histos  in  hist root file if asked
 *  (ugly and will be changed to Root)
 */

static double xaxis[100000];
static double yaxis[100000];
void Monitorimpl::moduleNrSelect(int& modsel , bool& drawall){
  int all;
  int iselm= SBModule->text().toInt();
  //   if( drawall == false) { 
  //      if( iselm < 0 ) iselm=0;
  //       SBModule->setValue(Nmod);
  //    }
  all=(iselm==-1)? all=true:all=false;
  drawall=all;
  modsel=iselm;
}

void Monitorimpl::updateHist()
{
if(themstatus->initDone() ) {
   DT_DEBUG_GUI(" Monitor: updateHist");

   if(Nmod==0) return;
   
Record* therec=thetestsetup->getRecord();

//
 
  //select
 int idtab=TWPlot->currentPageIndex();
 int iselm=-1;
 bool drawall=true;
 moduleNrSelect(iselm,drawall);
 if( drawall) { LEModId->setText("all");}
 else { 
   LEModId->setText(therec->getmodules()[iselm]->getmodheader()->getmodid());
 }
 DT_DEBUG_GUI("Monitor updatehist; select "<<iselm<<" idtab "<<idtab);
 if(iselm>=Nmod) { drawall=true; iselm=-1;}
 // the following could also a switch(idtab) but that is sensitive to the order
 if( "Xt" == TWPlot->label(idtab)) { 
   DT_DEBUG_GUI("update Xt  graph");
   updateIt(therec) ; 
   return;
 }
 if( "It" == TWPlot->label(idtab)) { 
   DT_DEBUG_GUI("update It graph");
   updateVtIt(therec,drawall,iselm) ; return;
 }
 if( "IV" == TWPlot->label(idtab)) { 
   DT_DEBUG_GUI("update IV graph");
   updateIV(therec,drawall,iselm) ; 
   return;
 }
 if("CalProf"== TWPlot->label(idtab)) {
   DT_DEBUG_GUI("CalProfUpdate");
   updateCalProf(therec,drawall,iselm);
   return;
 }
 if("Lat"== TWPlot->label(idtab)) {
   DT_DEBUG_GUI("Latency Update");
   updateLatency(therec,drawall,iselm);
   updatePipe(therec,drawall,iselm);
   return;
 }
 if("Noise"== TWPlot->label(idtab)) {
   DT_DEBUG_GUI("Noise Update");
   updateNoise(therec,drawall,iselm);
   return;
 }
 if("Ped"== TWPlot->label(idtab)) {
   DT_DEBUG_GUI("Pedestal Update");
   updatePedestal(therec,drawall,iselm);
   return;
 }
 
 if("Cext"== TWPlot->label(idtab)) {
   DT_DEBUG_GUI("Extern exitation Update");
   updateCext(therec,drawall,iselm);
   return;
 }

 if("Cal"== TWPlot->label(idtab)) {
   DT_DEBUG_GUI("Cal Update");
   updateCal(therec,drawall,iselm);
   return;
 }   
} // end if initDone 
}

void Monitorimpl::updatePedestal(Record* therec,bool drawall, int iselm){
  ZPPed->clear(); ZPPed->replot();
  ZPFrame->clear(); ZPFrame->replot(); 
  int startmod=0; int endmod=Nmod;
  if (!drawall){ startmod=iselm; endmod=iselm+1;}
  for(int ich=startmod;ich<endmod;ich++) { 
    { 
      // raw frames one after another
      int nfr=Napv*140;
      int ib=0;
      vector<Module*> themodules=thetestsetup->getModules();
      for(vector<Module*>::iterator imod=themodules.begin();imod!=themodules.end();imod++) {
	vector<Spill_Analysis*> theanal=(*imod)->apvanalysis();
	for(vector<Spill_Analysis*>::const_iterator ianal=theanal.begin();
	    ianal!=theanal.end();ianal++)
	  {
	    short* sraw=(*ianal)->getRaw();
	    if(sraw==0) continue;
	    for(int i=0;i<140;i++)
	      {xaxis[ib]=ib ;yaxis[ib]=sraw[i]; ib++;}
	  }
      }
      long curve= ZPFrame->insertCurve("Frames");
      curve=ZPFrame->insertCurve("Frames");
      ZPFrame->setCurveData(curve,xaxis,yaxis,nfr);
      ZPFrame->setCurvePen(curve,*rpen[ich]);
      ZPFrame->setCurveStyle(curve, QwtCurve::Steps,0);
    }
    
    {
      TH1F* h= therec->getmodules()[ich]->getmodtest()->THpedestal(); 
      long curve= ZPPed->insertCurve("Pedestal");
      for(int i=0;i<h->GetNbinsX();i++)
	{xaxis[i]=h->GetBinCenter(i+1);yaxis[i]=h->GetBinContent(i+1);}
      xaxis[h->GetNbinsX()]= 1+h->GetBinCenter(h->GetNbinsX());
      yaxis[h->GetNbinsX()]=h->GetBinContent(h->GetNbinsX());
      curve=ZPPed->insertCurve("Pedestal");
      ZPPed->setCurveData(curve,xaxis,yaxis,h->GetNbinsX()+1);
      ZPPed->setCurvePen(curve,*rpen[ich]);
      ZPPed->setCurveStyle(curve, QwtCurve::Steps,0); 
    }    
  }
  ZPFrame->replot();
  ZPPed->replot();
}



void Monitorimpl::updateNoise(Record* therec,bool drawall, int iselm){  
  ZPFlagCh->clear();ZPFlagCh->replot();
  ZPNoise->clear();ZPNoise->replot();
  int startmod=0; int endmod=Nmod;
  if (!drawall){ startmod=iselm; endmod=iselm+1;}
  for(int ich=startmod;ich<endmod;ich++) { 
    {     
      TH1F* h= therec->getmodules()[ich]->getmodtest()->THnoise(); 
      long curve= ZPNoise->insertCurve("Noise");
      for(int i=0;i<h->GetNbinsX();i++)
	{xaxis[i]=h->GetBinCenter(i+1);yaxis[i]=h->GetBinContent(i+1);}
      xaxis[h->GetNbinsX()]= 1+h->GetBinCenter(h->GetNbinsX());
      yaxis[h->GetNbinsX()]=h->GetBinContent(h->GetNbinsX()+1);
      ZPNoise->setCurveData(curve,xaxis,yaxis,h->GetNbinsX());
      ZPNoise->setCurvePen(curve,*rpen[ich]);
      ZPNoise->setCurveStyle(curve, QwtCurve::Steps,0);
    }
    
    {
      TH1F* h= therec->getmodules()[ich]->getmodtest()->THrawnoise(); 
      long curve= ZPNoise->insertCurve("RawNoise");
      for(int i=0;i<h->GetNbinsX();i++)
	{xaxis[i]=h->GetBinCenter(i+1);yaxis[i]=h->GetBinContent(i+1);}
      xaxis[h->GetNbinsX()]= 1+h->GetBinCenter(h->GetNbinsX());
      yaxis[h->GetNbinsX()]=h->GetBinContent(h->GetNbinsX()+1);
      ZPNoise->setCurveData(curve,xaxis,yaxis,h->GetNbinsX());
      ZPNoise->setCurvePen(curve,*bpen[ich]);
      ZPNoise->setCurveStyle(curve, QwtCurve::Steps,0);
    }
    
    {
      TH1F* h= therec->getmodules()[ich]->getmodheader()->THchflag(); 
      long curve= ZPFlagCh->insertCurve("calfCh");
      for(int i=0;i<h->GetNbinsX();i++)
	{xaxis[i]=h->GetBinCenter(i+1);yaxis[i]=h->GetBinContent(i+1);}
      ZPFlagCh->setCurveData(curve,xaxis,yaxis,h->GetNbinsX());
      ZPFlagCh->setCurvePen(curve,*rpen[ich]);
      ZPFlagCh->setCurveStyle(curve, QwtCurve::Steps,0); 
    }    
  }
 ZPFlagCh->replot();
 ZPNoise->replot();
}


void  Monitorimpl::updateCal(Record* therec,bool drawall, int iselm){
  ZPCalA->clear(); ZPCalA->replot();
  ZPCalAprof->clear(); ZPCalAprof->replot();
  int startmod=0; int endmod=Nmod;
  if (!drawall){ startmod=iselm; endmod=iselm+1;}
  for(int ich=startmod;ich<endmod;ich++) { 
    
    if(therec->getmodules()[ich]->getmodtest()->bstat()->bcal) {
      TH1F* h= therec->getmodules()[ich]->getmodtest()->THcalA(); 
      long curve= ZPCalA->insertCurve("calA");
      for(int i=0;i<h->GetNbinsX();i++)
	{xaxis[i]=h->GetBinCenter(i+1);yaxis[i]=h->GetBinContent(i+1);}
      ZPCalA->setCurveData(curve,xaxis,yaxis,h->GetNbinsX());
      ZPCalA->setCurvePen(curve,*rpen[ich]);
      ZPCalA->setCurveStyle(curve, QwtCurve::Steps,0); 
    }
    
    if(therec->getmodules()[ich]->getmodtest()->bstat()->bcalprof) {
      TH1F* h= therec->getmodules()[ich]->getmodtest()->THcalPApeak();
      long curve= ZPCalAprof->insertCurve("calAProf");
      for(int i=0;i<h->GetNbinsX();i++)
	{xaxis[i]=h->GetBinCenter(i);yaxis[i]=h->GetBinContent(i+1);}
      ZPCalAprof->setCurveData(curve,xaxis,yaxis,h->GetNbinsX());
      ZPCalAprof->setCurvePen(curve,*rpen[ich]);
      ZPCalAprof->setCurveStyle(curve, QwtCurve::Steps,0);
    }
    
    if(therec->getmodules()[ich]->getmodtest()->bstat()->blight) {
      TH1F* h= therec->getmodules()[ich]->getmodtest()-> THcalAlight();
      long curve= ZPCalA->insertCurve("calAlight");
      for(int i=0;i<h->GetNbinsX();i++)
	{xaxis[i]=h->GetBinCenter(i+1);yaxis[i]=h->GetBinContent(i+1);}
      ZPCalA->setCurveData(curve,xaxis,yaxis,h->GetNbinsX());
      ZPCalA->setCurvePen(curve,*bpen[ich]);
      ZPCalA->setCurveStyle(curve, QwtCurve::Steps,0); 
    }
  }
  ZPCalA->replot();
  ZPCalAprof->replot();
}

void Monitorimpl::plotCalProfStrip(){
  if(themstatus->initDone() && Nmod > 0  ){ 
    Record* therec=thetestsetup->getRecord();
    int iselm=-1;
    bool drawall=false;
    moduleNrSelect(iselm,drawall); 
    updateCalProf(therec,drawall,iselm);
  }
}

void Monitorimpl::updateCalProf(Record* therec,bool drawall, int iselm){
  //calprof, only one module
  int module = iselm <0 ? 0 : iselm;
  ZPCalProf->clear();ZPCalProf->replot();
  int str=SBStrip->text().toInt();
  //cout << "cal prof for strip "<<str<< "module "<<module<<"/"<<Nmod<<endl;
  if(therec->getmodules()[module]->getmodtest()->bstat()->bcalprof) { 
    if(str<0) { str=0; SBStrip->setValue(str);} 
    int nrmodstrips= 
      therec->getmodules()[module]->getmodheader()->getnapvs()*NAPVCH-1;
    if(str>nrmodstrips) {str=nrmodstrips; SBStrip->setValue(str);}
    TH1F* h=therec->getmodules()[module]->getmodtest()->hcalprof(str);
    long  curve=ZPCalProf->insertCurve("CalProf");
    for(int i=0;i<h->GetNbinsX();i++)
      {xaxis[i]=h->GetBinCenter(i);yaxis[i]=h->GetBinContent(i);}
    ZPCalProf->setCurveData(curve,xaxis,yaxis,h->GetNbinsX());
    ZPCalProf->setCurvePen(curve,*bluepen);
    ZPCalProf->setCurveStyle(curve, QwtCurve::Lines,0);  
    ZPCalProf->replot();
  }
}


void Monitorimpl::updateIV(Record* therec,bool drawall, int iselm){  
  ZPIV->clear(); ZPIV->replot();
  //float maxXscale=0;
  //float maxYscale=0;
  vector <long> curve;
  unsigned int size;
  int startmod=0; int endmod=Nmod;
  if (!drawall){ startmod=iselm; endmod=iselm+1;}
  for(int ich=startmod;ich<endmod;ich++)  {
    if(!drawall) ich=iselm;
    vector<float>* ivi= therec->getmodules()[ich]->getmodtest()->TVivi() ;
    if( ivi->size() == 0) { continue; }
    vector<float>* ivv=therec->getmodules()[ich]->getmodtest()->TVivv() ;
    if( (size=ivv->size()) == 0) { continue; }
    if( ivi->size() < size) { size= ivi->size() ;}
    curve.push_back(ZPIV->insertCurve("ivgraph"));
    int ic1=curve.size()-1; 
    for(unsigned  int lc=0; lc<size  ;lc++){
      xaxis[lc]=(*ivv)[lc];
      yaxis[lc]=(*ivi)[lc];
    }
    ZPIV->setCurveData(curve[ic1],xaxis,yaxis,size);   
    ZPIV->setCurvePen(curve[ic1],*rpen[ic1]);
  }
  if( curve.size() >0){
    ZPIV->replot();
  }
  curve.clear();
}


void Monitorimpl::updateVtIt(Record* therec,bool drawall, int iselm){
 ZPIt->clear();ZPIt->replot();
 ZPVt->clear(); ZPVt->replot();
  int xm=int(thetestsetup->getCTime());
  ZPIt->setAxisScale(QwtPlot::xBottom,0,xm);
  ZPVt->setAxisScale(QwtPlot::xBottom,0,xm);

  QwtSymbol sym;
  sym.setStyle(QwtSymbol::Cross);
  sym.setPen(black);
  sym.setSize(5);
  int startmod=0; int endmod=Nmod;
  if (!drawall){ startmod=iselm; endmod=iselm+1;}
  for(int ich=startmod;ich<endmod;ich++)   {
    if(!drawall) ich=iselm;
    { 
      long curve=ZPIt->insertCurve("itgraph");
      vector<float> t=*(therec->getmodules()[ich]->getmodsctrl()->vtimet());
      vector<float> x=*(therec->getmodules()[ich]->getmodsctrl()->it());
//#define SPECIALHVBUGDEBUG
#ifndef SPECIALHVBUGDEBUG
      for(unsigned int i=0;i<t.size();i++)
	{xaxis[i]=t[i];yaxis[i]=x[i];
	cout << " t "<<t[i] << " y "<<x[i]<<" " ;
	if(!( i% 8)) cout<<endl;
	}
      cout<<" i" << endl;
#endif
      ZPIt->setCurveSymbol(curve,sym); 
      ZPIt->setCurveData(curve,xaxis,yaxis,t.size());
      ZPIt->setCurvePen(curve,*rpen[ich]);
      ZPIt->setCurveStyle(curve, QwtCurve::Lines,0);
    }
   
    { 
      long curve=ZPVt->insertCurve("vtgraph");
      vector<float> t=*(therec->getmodules()[ich]->getmodsctrl()->vtimet());
      vector<float> x=*(therec->getmodules()[ich]->getmodsctrl()->vt());
      for(unsigned int i=0;i<t.size();i++)
	{xaxis[i]=t[i];yaxis[i]=x[i];
	cout << " t "<<t[i] << " y "<<x[i]<<" " ;
	if(!( i% 8)) cout<<endl;
	}
      cout<<" v" << endl;
      ZPVt->setCurveSymbol(curve,sym); 
      ZPVt->setCurveData(curve,xaxis,yaxis,t.size());
      ZPVt->setCurvePen(curve,*rpen[ich]);
      ZPVt->setCurveStyle(curve, QwtCurve::Lines,0);
    } 
  }
  ZPIt->replot();
  ZPVt->replot();
}

void Monitorimpl::updateIt(Record* therec){
  ZPTt->clear();ZPTt->replot();
  ZPHt->clear(); ZPHt->replot();
  int xm=int(thetestsetup->getCTime());
  ZPTt->setAxisScale(QwtPlot::xBottom,0,xm);
  ZPHt->setAxisScale(QwtPlot::xBottom,0,xm);
  QwtSymbol sym;
  sym.setStyle(QwtSymbol::Cross);
  sym.setPen(black);
  sym.setSize(5);

 vector<float> t=*(therec->getsetup()->gettime());
 {
   long curve=ZPTt->insertCurve("target");
   vector<float> x=*(therec->getsetup()->gettemptagt());
   for(unsigned int i=0;i<t.size();i++)
     {xaxis[i]=t[i];yaxis[i]=x[i];}
   ZPTt->setCurveSymbol(curve,sym); 
   ZPTt->setCurveData(curve,xaxis,yaxis,t.size());
   ZPTt->setCurvePen(curve,*gpen[1]);
   ZPTt->setCurveStyle(curve, QwtCurve::Lines,0);
 }

 sym.setStyle(QwtSymbol::QwtSymbol::Rect);
 {
   long curve=ZPTt->insertCurve("Tempctl");
   vector<float> x=*(therec->getsetup()->gettempcoldt());
   for(unsigned int i=0;i<t.size();i++)
     {xaxis[i]=t[i];yaxis[i]=x[i];}
   ZPTt->setCurveData(curve,xaxis,yaxis,t.size());
   ZPTt->setCurvePen(curve,*bpen[1]);
   ZPTt->setCurveStyle(curve, QwtCurve::Lines,0);
   ZPTt->setCurveSymbol(curve,sym); 
 }

 sym.setStyle(QwtSymbol::QwtSymbol::Triangle);
 {
   long curve=ZPTt->insertCurve("HeatSink");
   vector<float> x=*(therec->getsetup()->gettempwarmt());
   for(unsigned int i=0;i<t.size();i++)
     {xaxis[i]=t[i];yaxis[i]=x[i];}
   ZPTt->setCurveData(curve,xaxis,yaxis,t.size());
   ZPTt->setCurvePen(curve,*rpen[1]);
   ZPTt->setCurveStyle(curve, QwtCurve::Lines,0);
   ZPTt->setCurveSymbol(curve,sym); 
 }
 sym.setStyle(QwtSymbol::QwtSymbol::DTriangle);
 {
   long curve=ZPTt->insertCurve("Box");
   vector<float> x=*(therec->getsetup()->gettempboxt());
   for(unsigned int i=0;i<t.size();i++)
     {xaxis[i]=t[i];yaxis[i]=x[i];}
   ZPTt->setCurveData(curve,xaxis,yaxis,t.size());
   ZPTt->setCurvePen(curve,*bpen[5]);
   ZPTt->setCurveStyle(curve, QwtCurve::Lines,0);
   ZPTt->setCurveSymbol(curve,sym); 
 }

 sym.setStyle(QwtSymbol::QwtSymbol::XCross);
 {
   long curve=ZPTt->insertCurve("Ambient");
   vector<float> x=*(therec->getsetup()->gettempextt());
   for(unsigned int i=0;i<t.size();i++)
     {xaxis[i]=t[i];yaxis[i]=x[i];}   
   ZPTt->setCurveData(curve,xaxis,yaxis,t.size());
   ZPTt->setCurvePen(curve,*bpen[9]);
   ZPTt->setCurveStyle(curve, QwtCurve::Lines,0);
   ZPTt->setCurveSymbol(curve,sym); 
 }
 ZPTt->replot();

 sym.setStyle(QwtSymbol::QwtSymbol::DTriangle);
 {
   long curve=ZPHt->insertCurve("Box");
   vector<float> x=*(therec->getsetup()->gethumboxt());
   for(unsigned int i=0;i<t.size();i++)
     {xaxis[i]=t[i];yaxis[i]=x[i];}
   ZPHt->setCurveData(curve,xaxis,yaxis,t.size());
   ZPHt->setCurvePen(curve,*bpen[1]);
   ZPHt->setCurveStyle(curve, QwtCurve::Lines,0);
   ZPHt->setCurveSymbol(curve,sym); 
 }

 sym.setStyle(QwtSymbol::QwtSymbol::XCross);
 {
   long curve=ZPHt->insertCurve("Ext");
   vector<float> x=*(therec->getsetup()->gethumextt());
   for(unsigned int i=0;i<t.size();i++)
     {xaxis[i]=t[i];yaxis[i]=x[i];}
   ZPHt->setCurveData(curve,xaxis,yaxis,t.size());
   ZPHt->setCurvePen(curve,*bpen[9]);
   ZPHt->setCurveSymbol(curve,sym);
   ZPHt->setCurveStyle(curve, QwtCurve::Lines,0);
 }
 ZPHt->replot();
 sym.setStyle(QwtSymbol::QwtSymbol::Cross);
}

void Monitorimpl::updateLatency(){
  if(themstatus->initDone() ) {
    if(Nmod==0) return;
    Record* therec=thetestsetup->getRecord();
    int iselm=-1;
    bool drawall=true;
    moduleNrSelect(iselm,drawall);
    updateLatency( therec,drawall,iselm);
  }
}

void Monitorimpl::updateLatency(Record* therec,bool drawall, int selm){
  int iselm=selm<0 ?0:selm;
 ZPLat->clear(); ZPLat->replot();
 // latency scan
 int  ich=iselm;
 if (ich<0||ich>Nmod) ich=0;
 int str=SBCh->text().toInt();
 if(str<0) { str=0; SBStrip->setValue(str);} 
 if(therec->getmodules()[ich]->getmodtest()->bstat()->blat) {
   int nrmodstrips= 
     therec->getmodules()[ich]->getmodheader()->getnapvs()*NAPVCH-1;
   if(str>nrmodstrips) {str=nrmodstrips; SBStrip->setValue(str);}
   TProfile* pl=therec->getmodules()[ich]->getmodtest()->hplatency(str); 
   long curve=ZPLat->insertCurve("LatScan");
   for(int i=0;i<pl->GetNbinsX();i++)
     {xaxis[i]=pl->GetBinCenter(i);yaxis[i]=pl->GetBinContent(i);}
   ZPLat->setCurveData(curve,xaxis,yaxis,pl->GetNbinsX());
   ZPLat->setCurvePen(curve,*bluepen);
   ZPLat->setCurveStyle(curve, QwtCurve::Lines,0); 
 }
 ZPLat->replot();
}

void Monitorimpl::updatePipe(Record* therec,bool drawall, int iselm){
  ZPPipe->clear();ZPPipe->replot(); 
  int  ich=iselm<0 ? 0:iselm ;
  // pipe
  int napvs=therec->getmodules()[ich]->getmodheader()->getnapvs();
  for(int k=0;k<napvs;k++) {
    if(therec->getmodules()[ich]->getmodtest()->bstat()->bpipe) {
      TH1F* h=therec->getmodules()[ich]->getmodtest()->hpipe(k);
      long curve=ZPPipe->insertCurve("Pipeaddress");
      for(int i=0;i<h->GetNbinsX();i++)
	{xaxis[i]=h->GetBinCenter(i);yaxis[i]=h->GetBinContent(i);}
      ZPPipe->setCurveData(curve,xaxis,yaxis,h->GetNbinsX());
      ZPPipe->setCurvePen(curve,*rpen[k]);
      ZPPipe->setCurveStyle(curve, QwtCurve::Steps,0);
    }
  }
  ZPPipe->replot();   
}


void  Monitorimpl::updateCext(Record* therec,bool drawall, int iselm){
  ZPBPlane->clear();ZPBPlane->replot();
  ZPLed->clear();ZPLed->replot();
  int startmod=0; int endmod=Nmod;
  if (!drawall){ startmod=iselm; endmod=iselm+1;}
  for(int ich=startmod;ich<endmod;ich++) { 
    
    if(therec->getmodules()[ich]->getmodtest()->bstat()->bled) { 
      TH1F* h= therec->getmodules()[ich]->getmodtest()->THcalSled(); 
      long curve= ZPLed->insertCurve("LedS");
      for(int i=0;i<h->GetNbinsX();i++)
	{xaxis[i]=h->GetBinCenter(i);yaxis[i]=h->GetBinContent(i+1);} 
      ZPLed->setCurveData(curve,xaxis,yaxis,h->GetNbinsX());
      ZPLed->setCurvePen(curve,*rpen[ich]);
      ZPLed->setCurveStyle(curve, QwtCurve::Steps,0);
    }
    
    if(therec->getmodules()[ich]->getmodtest()->bstat()->bbp) {
      TH1F* h= therec->getmodules()[ich]->getmodtest()->THcalSbackpl(); 
      long curve= ZPBPlane->insertCurve("BackPlaneS");
      for(int i=0;i<h->GetNbinsX();i++) 
	{xaxis[i]=h->GetBinCenter(i);yaxis[i]=h->GetBinContent(i+1);}
      ZPBPlane->setCurveData(curve,xaxis,yaxis,h->GetNbinsX());
      ZPBPlane->setCurvePen(curve,*rpen[ich]);
      ZPBPlane->setCurveStyle(curve, QwtCurve::Steps,0);
    }
  }
    ZPLed->replot();
    ZPBPlane->replot();
}

/*
 * private  
 * display settings in settings menu
 * when clicked the tab
 */
void Monitorimpl::displaysets( ){
  if(themstatus->initDone() ) {
    DT_DEBUG_GUI("Monitor: displaysets ");
    int idtab=TabSettings->currentPageIndex();
    if( "Hybrid" == TabSettings->label(idtab)) {
      I2cDescriptor* curi2c= thetestsetup->getTestProcedure()->getCurrentI2c();
      fHs->writeI2cSettingsDisplay(curi2c);  
    }
    if( "Run" == TabSettings->label(idtab)) {
      fRs->writeRunSettingsDisplay(thetestsetup->getSetParser());
    }
    if( "DAQ" == TabSettings->label(idtab)) {
      if(fDs){
	fDs->writeDaqSettingsDisplay();
      }
    }
    else {
      // make sure modes are correct 
      if(fDs) {
	fDs->RBScopeMode->setChecked(false);
	fDs->RBFedMon->setChecked(false);
      }
    }
  }
  DT_DEBUG_GUI("Monitor: displaysets done " );
}


/*
 */
void Monitorimpl::ClearHist()  {
  DT_DEBUG_GUI(" clear plots");
 ZPIt->clear();ZPIt->replot();
 ZPVt->clear(); ZPVt->replot();
 ZPTt->clear(); ZPTt->replot();
 ZPHt->clear(); ZPHt->replot();
 ZPCalA->clear(); ZPCalA->replot();
 ZPCalAprof->clear(); ZPCalAprof->replot();
 ZPPipe->clear();ZPPipe->replot();
 ZPLat->clear(); ZPLat->replot();
 ZPCalProf->clear();ZPCalProf->replot();
 ZPFlagCh->clear();ZPFlagCh->replot();
 ZPNoise->clear();ZPNoise->replot();
 ZPPed->clear(); ZPPed->replot();
 ZPFrame->clear(); ZPFrame->replot(); 
 ZPBPlane->clear();ZPBPlane->replot();
 ZPLed->clear();ZPLed->replot();
}
/*
 */
void Monitorimpl::StartOnline()
{
  DT_NOTE(" toggle Online");
 if(themstatus->initDone() ) {
  if(RBOnline->isChecked()) { DT_NOTE("Monitor: startonline"); 
 QString s;
 s.sprintf(" Time:%d start onlinemonitor ",thetestsetup->getCTime());
 StatusBox->insertItem(s);
 StatusBox->ensureCurrentVisible ();
theonline->StartThread();
 }  else 
 { DT_NOTE("Monitor: stop online "); 
 QString s;
 s.sprintf(" Time:%d stop onlinemonitor ",thetestsetup->getCTime());
 StatusBox->insertItem(s);
 StatusBox->ensureCurrentVisible ();
  theonline->StopThread();
}
 }
}
/*
 */
void Monitorimpl::ResetRecord()  {
if(themstatus->initDone() ) {
  DT_NOTE(" resetRecord");

QMessageBox dd("MainMonitor ","Reset All Data\n"
			  "Are You really  sure?",
			  QMessageBox::Warning,
			  QMessageBox::Yes,
	                  QMessageBox::No  | QMessageBox::Default, 
                          QMessageBox::Cancel | QMessageBox::Escape);
      dd.setButtonText(QMessageBox::Yes,"Yes");
      dd.setButtonText(QMessageBox::No,"No");
      switch (dd.exec())
	{
 case QMessageBox::Yes:{ 
 thetestsetup->getRecord()->Reset();
 QString s;
 s.sprintf(" Time:%d Reset Data ",thetestsetup->getCTime());
 StatusBox->insertItem(s);
 StatusBox->ensureCurrentVisible ();    }
 case QMessageBox::No:
	    break;
	
	}
 }
}

/* private slots */

void Monitorimpl::updateDaqSettings(QString str){
  if( str == "DisableTscReset") {
    thetestsetup->getTestProcedure()->status()->ResetApvctrl();
    QString s("TSC reset sequence is disabled");
    StatusBox->insertItem(s);
    StatusBox->ensureCurrentVisible ();
   

  }
  if( str == "EnableTscReset") {
    thetestsetup->getTestProcedure()->status()->SetApvctrl();
    QString s("TSC reset sequence is disabled");
    StatusBox->insertItem(s);
    StatusBox->ensureCurrentVisible ();
  }
}



void  Monitorimpl::handleProcedureFeedBack( QString message ){

  //if( run == "ConnectivityRun")
  {
    if (message == "finishedrun"){
      setErrorButtonColor();
    }
  }
  if(message == "interlockactive"){
    KLInterLock->setColor(red);
  }
  if (message =="maxhumidityreached"){
    FRHumidity->setPaletteBackgroundColor(red);
  }
  if (message =="humidity_ok"){
    FRHumidity->setPaletteBackgroundColor(green);
  }

}


void  Monitorimpl::debugAction(){
  if(thetestsetup && Nmod > 0 ) {
    int iselm=-1;bool drawall=true;
    moduleNrSelect(iselm,drawall);
    if( drawall ){ iselm=0;}
    if( iselm >= Nmod ) { iselm = Nmod -1;}
    Record* therec=thetestsetup->getRecord();
    vector<float> x=*(therec->getmodules()[iselm]->getmodsctrl()->vt());
    for(unsigned int i=0;i<x.size();i++){
      cout << " t "<<i << " y "<<x[i]<<" " ;
      if(!( i% 8)) cout<<endl;
    }
  }
}
