#include "coldboximpl.h"
#include "qspinbox.h"
#include "qpushbutton.h"
#include "qlcdnumber.h"
#include "qcheckbox.h"
#include "qtabwidget.h"
#include "coldbox.h"
#include "daqerrormsg.h"
#include "tempctrlimpl.h"
#include "humctrlimpl.h"
#include "kled.h"

ColdBoxImpl::ColdBoxImpl(QWidget* parent , const char* name , WFlags fl ):
  ColdBoxUi(parent){
  fCb=0;
  CbTempSet->setEnabled(false);
  CbUpdate->setEnabled(false);
  CbControl->setEnabled(false);
  CbUpdate->setEnabled(false);
  CbCtrlCh->setMaxValue(0);
  CbPowCh->setMaxValue(0);
  fHCtrl=0;
  fContRunning=false;
}


void ColdBoxImpl::setColdBox( ColdBox* cbp) {
  fCb=cbp;
  fCb->setTargetTemperature(20);
  CbCtrlCh->setMaxValue(fCb->getNrTempCtrls());
  CbPowCh->setMaxValue(fCb->getNrTempCtrls());
  for(int lc=0 ; lc <fCb->getNrTempCtrls();lc++) {
    if( fCb->fTempCtrl[lc]){ 
      QString s; s=s.setNum(lc);
      s="tempctrl "+s;
      QWidget* newtab= new QWidget (TabWidget,s);
      TabWidget->insertTab(newtab,s);
      TempCtrlImpl* tc=new TempCtrlImpl(newtab);
      tabinfotmpctrls ti; ti.fQw=newtab; ti.fTci=tc;
      tc->setTempCtrl(fCb->fTempCtrl[lc]);
      fTabs[s]=ti;    
    } 
  }
  if( fTabs.size() > 0){
    CbCtrlCh->setMaxValue(fTabs.size()-1);
    CbPowCh->setMaxValue(fTabs.size()-1);
    CbTempSet->setEnabled(true);
    CbUpdate->setEnabled(true);
    CbControl->setEnabled(true);
    CbUpdate->setEnabled(true);
  }
  if( fCb->activeHumidityCtrl()){
    QString s="HumidityCtrl";
    QWidget* newtab= new QWidget (TabWidget,s);
    TabWidget->insertTab(newtab,s);
    fHCtrl= new HumCtrlImpl(newtab);
    fHCtrl->setHumCtrl(fCb->fHCtrl);
  }
  connect(TabWidget, SIGNAL( currentChanged(QWidget*) ), this, SLOT(TabSelected(QWidget*)));
  CbUpdatePressed();
}

void ColdBoxImpl::TabSelected( QWidget* selectedwidget){
  QString s= selectedwidget->name();
  if( s.contains("tempctrl")) {
    if( fTabs.find(s) != fTabs.end()){
      fTabs[s].fTci->CbUpdatePressed();
    }
  }
  if( s.contains("HumidityCtrl")){
    if( fHCtrl){
      fHCtrl->CbUpdatePressed();
    }
  }
  if(s.contains("ColdBox"))  {
    CbUpdatePressed();
  }
}


void ColdBoxImpl::CbCtrlPressed(){
  if( CbContCtrl->isChecked() ){
    if(! fCb->getAutoCtrl()){
      fContRunning=true;
      fCb->startAutoCtrl() ;
      DT_DEBUG_GUI("started autoctr");
      qDebug("started autoctr");
    }
  }
  else{
    fCb->control();
    CbControl->toggle();
  }

}

void ColdBoxImpl::CbCtrlReleased(){
 if(fCb->getAutoCtrl()){
   if(! CbControl->isOn()){
     fCb->stopAutoCtrl() ;
     qDebug("stop loop");
     DT_DEBUG_GUI("stop loop");
   }
 }

}

void ColdBoxImpl::CbUpdatePressed(){
  CbTemp->display(fCb->getActualTemperature(CbCtrlCh->value()));
  CbPower->display(fCb->getActualPower(CbPowCh->value()));
  CbHumidity->display(fCb->getActualHumidity());
  CbTempSet->setValue((int)fCb->getTargetTemperature());
  LCDCtrlLoopCnt->display((int)fCb->getCtrlLoopCounter());
  if( fCb->getAutoCtrl() ){
    KLAutoCtrlActive->setState(KLed::On);    
  }
  else {
    KLAutoCtrlActive->setState(KLed::Off);    
  }
}

void ColdBoxImpl::CbSetTemp(){
  fCb->setTargetTemperature(CbTempSet->value());
}

