#include "modstatusimpl.h"
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qspinbox.h>
#include <qlcdnumber.h>
#include <qstring.h>
#include "kled.h"
#include "ModuleTestSetup.h"
#include "Module.h"
#include "HybridI2C.h"
#include "RecModule.h"
#include "RecModTest.h"
#include "RecModHeader.h"
#include "RecModSctrl.h"
#include "daqerrormsg.h"
#ifdef  LTDEBUG1 
#include "dcuconversion.h"
#endif
using namespace std; 

ModStatusImpl::ModStatusImpl( QWidget* parent ,  const char* name , WFlags fl  ):
  ModStatusUi( parent,  name ){
  thesetup=0;
}

ModStatusImpl::~ModStatusImpl(){}

void ModStatusImpl::setSetup(  ModuleTestSetup* setup ){
  thesetup=setup;
  
}

void ModStatusImpl::updateModuleStatus(){
  if (thesetup) {
    // use setup as fixed value because this is only instatiates ones
    // record can be "re created" as well the modules so this has to build up
    // everytime again. 

    vector<Module*> theModules=thesetup->getModules() ;
    SBModule->setMaxValue(theModules.size()-1);
    unsigned int iselm= (unsigned int)SBModule->text().toInt();
    if( iselm < theModules.size() && iselm >=0) {
      Module* module=theModules[iselm];
      LEModId->setText(module->getModId().c_str());	
      LEBenchPos->setText(QString("%1").arg((int)module->getBenchPos()));
      LEDcuId->setText(QString("%1").arg(module->getDcuId()));
      LEModType->setText(module->getModuleType().c_str());
      if(module->getModuleErrStatus()) {
	KLModuleStatus->setColor(red);
      }
      else {
	KLModuleStatus->setColor(green);
      }
      if(module->ConntectToDAQ() ) {
	KLConnectToDaq->on();
      }
      else {
	KLConnectToDaq->off();
      }
      HybridI2C* hyb=module->hybridi2c();
      LCDDCUerr->display(hyb->getNrI2cDcuErr());
      LCDAPVerr->display(hyb->getNrI2cApvErr());
      LCDMUXerr->display(hyb->getNrI2cMuxErr());
      LCDPLLerr->display(hyb->getNrI2cPllErr());
      LCDDAQerr->display(module->getModuleAcqErrors());
      RecModule* rec=module->getrecmod();
      RecModSctrl* sctrl=rec->getmodsctrl();
      
      LEDcuTemp->setText(QString("%1").arg(sctrl->gettemphybdcu1() ));     
      LEDcuNtc->setText(QString("%1").arg(sctrl->gettemphybdcu3() ));      
      LEDcuTHyb->setText(QString("%1").arg(sctrl->gettemphybdcu2() ));      
      LEDcuBias->setText(QString("%1").arg(sctrl->getileakdcu() ));     
      LEDcu25->setText(QString("%1").arg(sctrl->getvhyb25dcu() ));     
      LEDcu125->setText(QString("%1").arg(sctrl->getvhyb125dcu() )); 

#ifdef  LTDEBUG1 
      DcuConversion dc;
      LEDcuTempValue->setText(QString("%1").arg(
	dc.DcuTsTemperature((unsigned int)sctrl->gettemphybdcu1() )));
      LEDcuNTCValue->setText(QString("%1").arg(
	dc.DcuExternTemperature((unsigned int)sctrl->gettemphybdcu3() )));
      LEDcuTHybValue->setText(QString("%1").arg(
	dc.DcuHybridTemperature ((unsigned int)sctrl->gettemphybdcu2() )));
      LEDcuBiasValue->setText(QString("%1").arg(
	dc.DcuLeakageCurrent((unsigned int)sctrl->getileakdcu() )));
      LEDcu25Value->setText(QString("%1").arg(
	dc.Dcu250Voltage((unsigned int)sctrl->getvhyb25dcu() )));
      LEDcu125Value->setText(QString("%1").arg(
	dc.Dcu125Voltage((unsigned int)sctrl->getvhyb125dcu() )));
#endif

      LEI125V->setText(QString("%1 A").arg(sctrl->getihyb25(),4 ));
      LEI25V->setText(QString("%1 A").arg(sctrl->getihyb125() ,4));
      LETSi->setText(QString("%1 C").arg(sctrl->gettempsenntsc1(),4 ));
      LETSi2->setText(QString("%1 C").arg(sctrl->gettempsenntsc2(),4 ));
      LEBiasVoltage->setText(QString("%1 V").arg(sctrl->gethvbias(),4 ));
      LEBiasCurrent->setText(QString("%1 uA").arg(sctrl->getitest(),4 ));
    }
  }
  

}
