#ifndef CMDLINEDECODE_H
#define CMDLINEDECODE_H
#include <string>

/** \function  CmdLineDecode 
    
    This function decodes the command line options
    It looks for the optin with the name name preceded with -
    if it is found it checks if this option has an argument.
    If so it will be filled in result. If not and an argument is 
    given an error string is returned if an argument is given.
    It doesn't check for invalid options
    examples 
    res=CmdLineDecode(argc,argv,tryopt,result,true);
    -tryop | res: "argument expected for tryop  result "error"
    -tryop  -otherop | res: "argument expected for tryop  result "error"
    -tryop exa  | res : found    result exa  
    -tryop=exa  | res : found    result exa
    tryop exa  | res: tryop not started with option character result :error
    -thistryop exa | res:error not started with tryopt result: error
    
    res=CmdLineDecode(argc,argv,tryopt,result,false);
     -tryop exa  | res: no argument expected
     -tryop=exa  | res: no argument expected
     -tryop    |  res: found result  ""  (empty)

     @param argC number of arrays in argV
     @param argV pointer to array of argC char arrays
     @param name name of the option to be find
     @param result contains the argument of the option if requested
     @param argument should be true if a argument is required
     @retrun string will be found if option was correctly found \ 
             else contains the information of the error 
*/

std::string  CmdLineDecode(int argC , char** argV,
		       const std::string& name, std::string& result,
		       bool argument=false);

#endif
