#ifndef __DAQERRORMSG
#define __DAQERRORMSG 
#include <cstdlib>
#include <unistd.h>
#include <iostream>
#include <cstring>

// globals
enum DaqToolsMessageLevel  { DT_ERROR_LEVEL=0  ,
			     DT_WARNING_LEVEL=1,
			     DT_NOTE_LEVEL=2, 
			     DT_DEBUG_LEVEL=0x4,
			     DT_DEBUG_LAST=0x4,
			     DT_DEBUG_SP=0x8 , //set parser 
			     DT_DEBUG_LP=0x10,  // lt parser
			     DT_DEBUG_LTC=0x20, // lt tools
			     DT_DEBUG_LTAPPL=0x40, 
			     DT_DEBUG_SRV=0x80,
			     DT_DEBUG_PROC=0x100,
			     DT_DEBUG_ND3=0x200,
			     DT_DEBUG_MUX=0x400,
			     DT_DEBUG_DB=0x800,
			     DT_DEBUG_DBW=0x1000,
			     DT_DEBUG_GUI=0x2000,
			     DT_DEBUG_THR=0x4000,
			     DT_DEBUG_ALL=0xFFFF
			     	     
			     
                      };
  


// define to expand debug statements from code
#ifdef LTDEBUG1 
#define DT_DEBUG_ENABLE
#endif 
/***********************************************************************
 * printing
 ***********************************************************************/
class DaqToolsErrorMessage{ 
  private : static int printing_level;
  public :
    DaqToolsErrorMessage(){
    char envvar[40];
    if(printing_level == -1){ // first call   
      if ( getenv( "DAQPRINTINGLEVEL")){
	strcpy(envvar,   getenv("DAQPRINTINGLEVEL") );
        printing_level= (int)strtol( envvar,0,0);
      }
      else { printing_level = 1; // default value}
      }
      printing_level|= 0; // set always on values
    }
    // if printing level != -1 it will be already initialized.
  }

  DaqToolsErrorMessage(int i ){ printing_level=i;}
  void setPrintingLevel(int i){ printing_level=i;}
  int getPrintingLevel(){ return printing_level;}
};


#ifdef  DT_DEBUG_ENABLE
#define DT_DEBUG(args)						\
do { DaqToolsErrorMessage dte;		                        \
   if (dte.getPrintingLevel() >= DT_DEBUG_LEVEL  )				\
   {	std::cout <<	 __PRETTY_FUNCTION__  <<" module "<< 	__FILE__ <<" at line "<< __LINE__;\
       std::cout <<" "<<  args;						        \
       std::cout << std::endl;						        \
	     }								        \
} while (0);

#define DT_DEBUG_LTAPPL(args)                       \
{  DaqToolsErrorMessage dte;		        \
   if (dte.getPrintingLevel()  & DT_DEBUG_LTAPPL )  \
   {  std::cout << __PRETTY_FUNCTION__<<": "<< __LINE__<<": ";       \
       std::cout   << args;				\
	std::cout <<std::endl;		  	        \
   }	}							       

#define DT_DEBUG_PROC(args)                       \
{  DaqToolsErrorMessage dte;		        \
   if (dte.getPrintingLevel()  & DT_DEBUG_PROC )  \
   {  std::cout << __PRETTY_FUNCTION__<<": "<< __LINE__<<": ";       \
       std::cout   << args;				\
	std::cout <<std::endl;		  	        \
   }	}	

#define DT_DEBUG_SP(args)                       \
{  DaqToolsErrorMessage dte;		        \
   if (dte.getPrintingLevel()  & DT_DEBUG_SP )  \
   {  std::cout << __PRETTY_FUNCTION__<<":"<< __LINE__<<": ";       \
       std::cout   << args;				\
	std::cout <<std::endl;		  	        \
   }	}							       

#define DT_DEBUG_LP(args)                       \
{  DaqToolsErrorMessage dte;		        \
   if (dte.getPrintingLevel()  & DT_DEBUG_LP )  \
   {  std::cout << __PRETTY_FUNCTION__<<":"<< __LINE__<<": "; \
       std::cout   << args;				\
	std::cout <<std::endl;		  	        \
   }	}							       

#define DT_DEBUG_LTC(args)                       \
{  DaqToolsErrorMessage dte;		        \
   if (dte.getPrintingLevel()  & DT_DEBUG_LTC )  \
   {  std::cout << __PRETTY_FUNCTION__<<":"<< __LINE__<<": ";  \
       std::cout   << args;				\
	std::cout <<std::endl;		  	        \
   }	}				

#define DT_DEBUG_SRV(args)                       \
{  DaqToolsErrorMessage dte;		        \
   if (dte.getPrintingLevel()  & DT_DEBUG_SRV )  \
   {  std::cout << __PRETTY_FUNCTION__<<" :  "<< __LINE__<<": "; \
       std::cout   << args;				\
	std::cout <<std::endl;		  	        \
   } }

#define DT_DEBUG_ND3(args)                       \
{  DaqToolsErrorMessage dte;		        \
   if (dte.getPrintingLevel()  & DT_DEBUG_ND3 )  \
   {  std::cout << __PRETTY_FUNCTION__<<" :  "<< __LINE__<<": ";       \
       std::cout   << args;				\
	std::cout <<std::endl;		  	        \
   }}
#define DT_DEBUG_MUX(args)                       \
{  DaqToolsErrorMessage dte;		        \
   if (dte.getPrintingLevel()  & DT_DEBUG_MUX )  \
   {  std::cout << __PRETTY_FUNCTION__<<" :  "<< __LINE__<<": ";   \
       std::cout   << args;				\
	std::cout <<std::endl;		  	        \
   }}
#define DT_DEBUG_DB(args)                       \
{  DaqToolsErrorMessage dte;		        \
   if (dte.getPrintingLevel()  & DT_DEBUG_DB )  \
   {  std::cout << __PRETTY_FUNCTION__<<" :  "<< __LINE__<<": ";   \
       std::cout   << args;				\
	std::cout <<std::endl;		  	        \
   }}
#define DT_DEBUG_DBW(args)                       \
{  DaqToolsErrorMessage dte;		        \
   if (dte.getPrintingLevel()  & DT_DEBUG_DBW )  \
   {  std::cout << __PRETTY_FUNCTION__<<" :  "<< __LINE__<<": ";   \
       std::cout   << args;				\
	std::cout <<std::endl;		  	        \
   }}
#define DT_DEBUG_GUI(args)                       \
{  DaqToolsErrorMessage dte;		        \
   if (dte.getPrintingLevel()  & DT_DEBUG_GUI )  \
   {  std::cout << __PRETTY_FUNCTION__<<" :  "<< __LINE__<<": ";   \
       std::cout   << args;				\
	std::cout <<std::endl;		  	        \
   }}

#define DT_DEBUG_THR(args)                       \
{  DaqToolsErrorMessage dte;		        \
   if (dte.getPrintingLevel()  & DT_DEBUG_GUI )  \
   {  std::cout << __PRETTY_FUNCTION__<<" :  "<< __LINE__<<": ";   \
       std::cout   << args;				\
	std::cout <<std::endl;		  	        \
   }}

#else
#define DT_DEBUG(args)	
#define DT_DEBUG_SP(args)
#define DT_DEBUG_LP(args)
#define DT_DEBUG_LTC(args) 
#define DT_DEBUG_LTAPPL(args) 
#define DT_DEBUG_SRV(args) 
#define DT_DEBUG_PROC(args) 
#define DT_DEBUG_ND3(args) 
#define DT_DEBUG_MUX(args)
#define DT_DEBUG_DB(args)
#define DT_DEBUG_GUI(args)
#define DT_DEBUG_THR(args)
#endif
#define DAQEXIT(args) \
                  std::cout <<std::endl;std::cout << args ;\
                                  std::cout << std::endl <<  "****exit at "  << __FILE__ << " in function " ;\
      	          std::cout <<  __PRETTY_FUNCTION__ << " at line " << __LINE__    << std::endl;\
		 std::exit(1);
 


#define DT_PANIC(args)						\
do {	std::cout << args ;\
                                  std::cout << std::endl <<  "****Panic exit at "  << __FILE__ << " in function " ;\
      	          std::cout <<  __PRETTY_FUNCTION__ << " at line " << __LINE__    << std::endl;\
		  std::exit(1);								\
} while (0);

#define DT_NOTE(args)						\
   {DaqToolsErrorMessage dte;		        \
   if (dte.getPrintingLevel()>= DT_NOTE_LEVEL)			        \
    {	std::cout << __PRETTY_FUNCTION__ ; \
        std::cout << args;						        \
	std::cout <<std::endl;						\
    }	}								


#define DT_WARN(args)					\
do {	DaqToolsErrorMessage dte;		        \
   if (dte.getPrintingLevel() >=DT_WARNING_LEVEL)			\
    {  std::cout << "*** Warning in " << __FUNCTION__ << " module " \
            << __FILE__<< " at "<< __LINE__ <<"\n\t";\
       std::cout << args;				     \
       std::cout << std::endl;				\
    }							\
} while (0);

#define DT_ERROR(args)						\
do {	DaqToolsErrorMessage dte;		        \
   if (dte.getPrintingLevel()	>= DT_ERROR_LEVEL)		        \
	{								\
           std::cout <<" *** Error in"  << __FUNCTION__ <<" module " << __FILE__ << " at line " << __LINE__;\
     		std::cout << args;						\
		std::cout << std::endl ;   		\
			}				         \
} while (0);

#endif

