#ifndef __APVREADOUTCTRL_H
#define __APVREADOUTCTRL_H

#include "FecCtrl.h"
#include <string>


/** \class  ApvReadoutCtrl 
    \author W.Beaumont,
    \date 02/2003 
    \version 2.1

    \brief help function for apv readout process controll
    is an class that combines the control of the devices that influence the 
    readout of an apv 
    - tsc : enabeling triggers ,sending resets , enabling cal sequence
    - fed :  check for new spill (blocking the buffer)
    - enabling / disabling  external calibration 
    This class is written special for the module test application
    This class is not " stable" in this sense that the interfaces can be changed if this
    is required for the module test application. 
    The FEC control is not implemented so 
    In principle the class has to be split in a part that is readable by all
    so also the fed channesl
    and a class that can only manipulate the values. 
*/


class TscChannelInterface;
class FedCtrl;
class DioChannelInterface;
class MoveInterface;
class SetParser;
class LedRunDescriptor;
class DaqCtrlDescriptor;
class FedProcessCtrlInterface;

class ApvReadoutCtrl : public FecCtrl  { 
 private:
 static TscChannelInterface* thetsc;
 /// the power channel is only a temperaly solution 
 static FedCtrl* thefedctrl ;
 static DioChannelInterface* theBpEnable;
 static MoveInterface* themove;
 static bool fInitialized ;
 static int fApvsconnected ;
 int fApvnr ;
 bool fFreshSpillRequest;
DaqCtrlDescriptor*  thedaqctrldescriptor ;
 // take care starts from 1 for the first Apv , the contsturctor for the tsc and others
 // has fApvnr 0
 int fSpillSize;
 int fTscLatency; 
 public:

 ApvReadoutCtrl(SetParser* thesettings, int tscdevice, int bpulsectrl   );
 /** this one is used for inheritance by the fed channels */
 ApvReadoutCtrl( ); 

 TscChannelInterface* getTsc() {return thetsc;}
 FedCtrl* getFedCtrl(){return thefedctrl ;}
 /** returns the pointer to the FedProcess Control of Fed nr fednr */
 FedProcessCtrlInterface* getFedProcessCtrl(int fednr=0);
 ApvReadoutCtrl::~ApvReadoutCtrl();

 void MoveSpeed( int speed);
 void GoToStartPos(LedRunDescriptor* ledset  );
 void StartMoving ();

/** sets the starting condition for a DAQ test run (ped run etc)
    
    @return always true for the moment 
 */
 bool startDaqTest(); 
 
/** stops the acquistion loop 
 */
 bool stopDaqTest();
 
/** sets the spill size , the number of events that are provided
    to be processed by default the spillsize is set to 100 
*/

 void setSpillSize ( int spill){fSpillSize=spill;}
 int  getSpillSize (){return fSpillSize;}




 /** enables the reset 101 on the tsc and waits until the server 
     confirms the setting if update is true
     only the request is done and during the next update 
     request the reset is enabled 
 */
 void enableReset(bool update=true);


 
 /** disables the reset 101 on the tsc and waits until the server 
     confirms the setting if update is true
     only the request is done and during the next update 
     request the reset is enabled 
 */
 void disableReset(bool update=true);
 /**  enables or disalbes the cal 11 sequence  on the tsc and 
     waits until the server  confirms the setting if update is true
     only the request is done and during the next update 
     request the reset is enabled 
     @param enable is true the cal pulse is set active else no cal puls
     @param update if true direct update of the sequence
 */
 void enableCalSeq(bool enable=true,bool update=true);


 void setTscLatency(int lat);
 void setTscLatency();
 int  getTscLatency();






/// Fed ctrl parts

 bool  AdjustFedThresholds(int & low, int& high);
 bool  AdjustFedThresholds();
 bool  setFedClkDelay(int & delay);
 /** set the fed in scope mode if mode > 0  */
 void setScopeMode(int mode);
 /** if fresh is true a fresh spill will be initialized 
     in this case it always returns false but the acquisition
     for the new spill is started
     In case of fresh is false it returns true if the new spill
     is filled otherwise it returns false 
  */ 
 bool NewSpill(bool fresh=false) ;
 void NextSpill(); 
 int getNrEvents();

 /** initialize the tsc and fedctrl (spill size) from the settings file
     @param thesetting pointer to the setparser class that contails the information .
  */ 
int ApvReadoutCtrlSettings(SetParser* thesettings);
/// Tsc sends a reeset (101 ) sequece to the apv
void   sendApvReset();
/** Tsc sends a reeset (101 ) sequece to the apv if it is the first or last one
    @param c  c can be "last" or "first"  in all other cases no sequence will be send
    @return true is 101 is send otherwise false 
*/
 bool  sendApvReset(std::string c );
 void disableBackPlanePulse();
 void enableBackPlanePulse();

 bool isInitialized(){ return fInitialized;}

 bool isFirstApv() ;
 bool isLastApv ();
};

#endif 
