#ifndef Apv_Analysis_H_
#define Apv_Analysis_H_

#include <vector>
#include <valarray>
#include <map>


/**
  \brief APV analysis class implementation: pedestal, noise, badch
         Different methodes are implemented: updating and accumulating
         for pedestal and noise,
         mean and median for the common mode noise.
         tdr, fed for zero suppression
   \class 
   \authors: V.Zhukov,UIA
   \date 02/2003
   \version 2.1

*/


class Apv_Analysis {

public:

typedef std::valarray<double>  Fvec;
typedef std::valarray<int>    Ivec;
typedef std::valarray<bool>    Bvec;
typedef std::map<float, int, std::less<float> > FloatMapType;
typedef std::map<int, int, std::less<int> > DigitalMapType;
typedef std::map<int,float,std::less<int> >  SignalMapType;
typedef std::map<int, bool, std::less<int> > BoolMapType;


Apv_Analysis(int i);
virtual ~Apv_Analysis();

  //methodes
void  UpdatePedestal();    //update pedestal
void  UpdateNoise();       //update noise
void  SubtractCM();        //subtract pedestal and cmn
void  SubtractPed();        //subtract pedestal 
int   FindCluster();       //find cluster
void  UpdateDead();          // update dead channels 
void AnalyzePed(int ev);// local algorithm exampl
void AnalyzeCalProf(int ev);// local algorithm exampl
void Clear();  // Clear all
  //getters 
short* getRaw() {return &fRaw[0];}
int  getId() {return theId;}
std::valarray<double>*   getData()  {return &theData;} 
std::valarray<double>*   getPedestal()  {return &thePedestal;} 
std::valarray<double>*   getRms()  {return &theRms;} 
std::valarray<double>*   getRawRms2()  { return &theRawRms2;} 
std::valarray<double>*    getCmn()  {return &theCmc;}
std::valarray<double>*    getSignal() {return &theSignal;}
std::vector<int>*  getChStatus()  {return &theChStatus;}
int      getNBadChan()  {return theNBad;}
float   getMeanRms()    {return theMeanRms;}  
float   getMeanRawRms(); 
std::vector<float>*  getClusQ()   {return &theClusQ;}
std::vector<float>*  getClusNoise()   {return &theClusNoise;}
std::vector<int>*   getClusSize()   {return &theClusSize;}
std::vector<float>*  getClusPos()   {return &theClusPos;}
  //setters
virtual void setPedestal(const Fvec& ped) {thePedestal=ped;}
virtual void setRms(const Fvec& rms) {theRms=rms;}
  // 
inline   void setNoiseEstimator(int nev, int nevraw,int mode, float tskip, float tbad, float rmsmin, float rmsmax,float pedmin, float pedmax,float pskip){
theNrawevent=nevraw;
theNpedevent=nev;
theModped=mode;
theTskip=tskip;
theTbad=tbad;
theRmsmin=rmsmin;
theRmsmax=rmsmax;
thePedmin=pedmin;
thePedmax=pedmax;
thePskip=pskip; 
  }
inline   void setCmnEstimator(int mode, int nstrips,int niter){
theModcmn=mode;
theNStrips=nstrips;
theNcmniter=niter;
  }
inline  void setClusterFinder(int mode, float t1,float t2,float t3){
theModclus=mode;
theTstrip=t1; 
theTseed=t2;
theTclus=t3; 
}
 void  setTframe(float thr) {theTframe=thr;}
 void  setModped(int mod) {theModped=mod;}
 void  setModcmn(int mod) {theModcmn=mod;}
 void setupdate(int n) {theUpdate=n;}
// access from Spill_Analysis

 protected:
short fRaw[1024];
Fvec theData;       // populated data 

private:
  //private methodes
std::valarray<double>   cmnMean(const Fvec& rawdata, const std::vector<int>& skip); //returns cmn-mean algorithm
std::valarray<double>   cmnMedian(const Fvec& rawdata,const std::vector<int>& skip);// returns cmn-median algorithm
float mean(const Fvec& data, float& Rms);// returns  stat mean, provides stat Rms
float median(const Fvec& data, float& Rms);// returns median, provides 'gaussian' Rms  
std::vector<int> updateskipCh(const Fvec& sig,const Fvec& rms); // return skip channels for the event
void  updatebadCh(); // update bad channels                  
float  findMedian(const FloatMapType& map,const int nused, float& Rms); //find median and rms
void  updateMeanPed(); // update pedestal mean algo
void  updateMeanNoise(); //update noise mean algo
void  updateMedianPed(); //update pedestal median algo
void  updateMedianNoise(); //update noise median algo
int   defineDead();       //define dead 
std::map<int,float,std::less<int> >  findHits(); // find hist snr>Tstrip

private:

std::vector<int>  nusedped;
std::vector<int>  nusednois;
std::vector<FloatMapType> pedDistribution;
std::vector<FloatMapType> noisDistribution;
DigitalMapType  theNoisyCh;  // accumulated Noisy channels
 
private:
  //data
  int theId;
  Fvec thePedestal;   // mean pedestal
  Fvec theCmc;        // common mode coeff
  Fvec  theRms;       // rms noise after cmn
  Fvec theRms2;       // rms2
  Fvec theRawRms2;    // rms noise before  cmn
  float theMeanRms;   // mean rms
  Fvec  theZero;      
  Fvec theSignal;      //  signal = data - pedestal - cm
  int  theNBad;        // n bad ch with chstatus>0
  int  theNPed;       // Nevent in update pedestal
  int  theNRms;       // Nevent in update noise
  std::vector<int> theChStatus; // ch status bit; 0-ok >0 bad
  
  
  //cluster
   std::vector<float>  theClusQ;           // Cluster Charge
   std::vector<float> theClusNoise;       // Cluster Noise
   std::vector<float>  theClusPos;       // Cluster Position
   std::vector<float> theSeedSNR;       // seed signal
   std::vector<float> theClusSeedStrip;// Highest strip of cluster
   std::vector<int> theClusSize;       //cluster size
  //settings
  int theModped;  //1-mean;2-median
  int theModcmn;  //1-mean;2-median
  int theNcmniter; //Niteration for cmn
  int theModclus;  //1-TDR;2-FED2;3-FED1
  int  thePolarity; //signal polarity +1/-1
  int theNStrips;  //strips in cmn group
  float theTframe; //Threshold frame
  float theTskip;  // Threshold signal/noisy strips for noise update
  float theTbad;   // Threshold bad channels rms>Tbad* meanrms
  float theRmsmin; //min rms for bad channels
  float theRmsmax;  //max rms
  float thePedmin;
  float thePedmax;
  float thePskip;   //mask bad if in pskip events is >Tskip
  int theNpedevent; //number of events for pedestal 
  int theNrawevent;  //number of events for raw pedestal 
  float theTseed;  // Threshold seed strip for clustering
  float theTclus;  // Threshold cluster 
  float theTstrip;  // Threshold strip
  int  theUpdate;  // update for accumulating algorithms
  int theNevents;   //current event
};
#endif
