#ifndef _CalProfRun_H
#define _CalProfRun_H

#include <vector>
#include <valarray>
#include <iostream.h>
#include <stdio.h>

#include <TObjArray.h>
#include <TH1.h>
#include "TestBase.h"

/** \brief Calibration Profile Run for  TestProcedure

           Calculate cal(i)-ped(i)  for each channel vs time (3.1ns)
           between latstart latend defined in the settings file or from settings menu.
           Does a pedestal run when latency is changed.
           Fills calibration profile for each channel, calculates maximum amplitude
          and fill Apeak(channel) histo, flag bad channels.          
          Uses already created in Module  objects; feds, analysis,record, i2c
          InitTest-define settings, 
          Update-acquires data and stops at the end
          FillRecord-fill results into Record 


     \author V.Zhukov,UIA   
     \version 2.1 
     \date 2003/02
 
*/



class Module;
class HybridI2C;
class RecModule;
class CalProfRunDescriptor;


class CalProfRun : public TestBase  { 

public:

CalProfRun(Module* m);
~CalProfRun();

void InitTest(SetParser* s);
bool Update(int n);
void FillRecord();
void Abort() {done=true;}
bool isDone()  {return done;}

bool LatencyChanged() const {return latchanged;}
bool UpdatePedestal(int n);


private:

Analysis theapvanals;
Feds thefeds;
HybridI2C* thei2c;
RecModule* therec;
CalProfRunDescriptor* calsets; 

bool SkipPedestal; 
int Ncalev;
int Nev;
int Ntot;
int Napvs;
short prevmode;
short curmode;
int latencystart;
int latencyend;
short lat;
short sel;
short drv;
bool done;
bool latchanged;
bool pedrun;
bool binv;
bool convmode;
float Tbad;
int fDriveStep,fTimeStep;
unsigned char drivepattern;
double fCorr[NAPV*NAPVCH][1000];
};

#endif 




