#ifndef _HybridI2C_H
#define _HybridI2C_H


#include <qstring.h>
#include "DaqCtrl.h" 


/** \brief  Hybrid I2C channels: all i2c functionality for the apv hybrid


     \author V.Zhukov,UIA   
     \version 2.1 
     \date 02/2003
 
*/

class Apv25I2cChannelLt;
class MuxI2cChannelLt;
class PllI2cChannelLt;
class DcuChannelLt;
class OptoI2cChannelLt;
class I2cDescriptor;


class HybridI2C  { 
 public:
HybridI2C(int bench, DaqCtrl* apv);
~HybridI2C();
/** set all values defined in the I2c descriptor so 
    for pll, mux and apv
    @aparm i2cset the new values t
    @return 0 if o.k. if not o.k. a negative number 
*/
int  SetI2C(I2cDescriptor* i2cset);
/** set only the pll values. If initpll is set to active
    the pll will be reset and 
    the member  thei2cset will be changed  to thei2cset  
    @param i2cset pointer to the values that should be set
    @return 0 if o.k. if not o.k. a negative number 
*/
int setPll(I2cDescriptor* i2cset);

/** set only the mux values. 
    the mux values of member  thei2cset will be changed  
    to thei2cset  
    @param i2cset pointer to the values that should be set
    @return 0 if o.k. if not o.k. a negative number 
*/
int  setMux(I2cDescriptor* i2cset);

/** set only the apv values. 
    the apv values of member  thei2cset will be changed  
    to thei2cset  
    @param i2cset pointer to the values that should be set
    @return 0 if o.k. if not o.k. a negative number 
*/
 int setApv(I2cDescriptor* i2cset);

 /** sets the values for the linear laser driver (LLD)
     @param gain  the gain that has to be set 
     @param bias the bias for the LLD 
     @param channel  the channel if -1 all 3 channels will be set 
     @return 0 if o.k. if not o.k. a negative number 
 */ 
 int  setLld(int gain, int bias, int channel=-1);



/**
   set the apv latence to lat 
   return the result of the read back of the APV
 */
int  SetLatency(int lat);
void SetSkew(short s);
void SetDrv(short d);
short SetMode(short m);
void SetCalMode(short m);
void SetCalAmp(int a);
void UnsetCalMode(short m);
void EnableReset()  {thedaqctrl->enableReset();}
void DisableReset()  {thedaqctrl->disableReset();}
short GetMode();
QString  CheckMode(short m);
void ChangeCal(int lat,short sel, short drv);
bool CheckInvMode(short m);
bool CheckCalMode(short m);
bool CheckDecMode(short m);
//
Apv25I2cChannelLt*  apvi2c() {return theapvi2c;}
MuxI2cChannelLt*   muxi2c() {return themuxi2c;}
PllI2cChannelLt*  plli2c() {return theplli2c;}
DcuChannelLt*    dcui2c() {return  thedcui2c;}
 OptoI2cChannelLt* optoi2c(){ return theopto;}
DaqCtrl* daqctrl() {return  thedaqctrl;}
I2cDescriptor* lasti2c() {return thei2cset;}

/** returns the number of detected I2C communication errors for the APV */
 int getNrI2cApvErr(){return fNrapverr;}
/** returns the number of detected I2C communication errors for the DCU */
 int getNrI2cDcuErr(){return fNrdcuerr;}
/** returns the number of detected I2C communication errors for the MUX */
 int getNrI2cMuxErr(){return fNrmuxerr;}
/** returns the number of detected I2C communication errors for the PLL */
 int getNrI2cPllErr(){return fNrpllerr;}
/** returns the number of detected I2C communication errors for the LLD */
 int getNrI2cLldErr(){return fNrllderr;}


private:
/// individual latency offset for ccu delay compensation. 
unsigned short  fLatencyOffset;
Apv25I2cChannelLt* theapvi2c;
MuxI2cChannelLt* themuxi2c;
PllI2cChannelLt* theplli2c;
DcuChannelLt* thedcui2c;
OptoI2cChannelLt* theopto;
DaqCtrl* thedaqctrl;
I2cDescriptor* thei2cset;
 int fBench;
 int fNrapverr;
int fNrmuxerr;
int fNrdcuerr;
int fNrpllerr;
 int fNrllderr;

};
#endif 
