#ifndef _LatRun_H
#define _LatRun_H

#include <vector>
#include <valarray>
#include <iostream.h>
#include <stdio.h>

#include <TH1.h>
#include <TProfile.h>

#include "TestBase.h"
#include "HybridI2C.h"


/** 
 \brief Latency Run for the TestProcedure

<p>
           Calculate sig(i)-ped(i)  for each channel vs latency range
           defined in settings file (default 1-192)
           if calibration is On is analogous to calprofrun but
           without an overall scan for each cal group and fine delays.
           Also fills pipeline addresses histo and counts Nerrorbitlow.
           Default with reset mode, can be disabled from settings menu.
           Uses already created in Module  objects; feds, analysis,record, i2c
          InitTest-define settings, 
          Update-acquires data and stops at the end
          FillRecord-fill results into Record 



     \author V.Zhukov,UIA   
     \version 2.2
     \date 09/2003
 
*/

class Module;
class RecModule;
class LatRunDescriptor;

class LatRun : public TestBase { 

public:
LatRun(Module* m);
~LatRun();

void InitTest(SetParser* s);
bool Update(int nev);
void FillRecord();
void Abort() {done=true;}
bool isDone() {return done;}

bool LatencyChanged() const {return latchanged;}
int nerror() const {return Nerror;}

private:

HybridI2C* thei2c;
Analysis theapvanals;
Feds thefeds;
RecModule* therec;
LatRunDescriptor* latsets;


int Nev;
int Ncalev;
int Ntot;
int Napvs;
 int Nerror;
short prevmode;
int latencystart;
int latencyend;
int lat;
bool done;
bool latchanged;
};

#endif 




