#ifndef LIGHTCTRL__
#define LIGHTCTRL__
#include "pmap.h"

#include "i2cid.h"

/** \class  LightCtrl 
    \author W.Beaumont,
    \date Feb   2002
    \version 1.0

    \brief light (Led ctrl) for ModuleTest


    This class provides the interface of the setting of the Light 
    control. To set the light intensity.
    The LightCtrl instatiates a number of lighctrlcards depending 
    on the xml file (if the light control is not defined in 
    the xml file a dummy card is created
    The version 1.0 only supports the Led Ctrl  card from karslruhe
    and only the ligh part (not the pulser part this is set to zero)
    Each card is indentified by an ID obtaind by its 
    "coodinates"  (device it is connected to, sub device etc. 
    Indiviual channel on the card can be selected and identified by a 
    channel .  So only the base address is used for identification.
    The channel has to be requested explecite if needed. 
    
*/

class  LedCardInterface;


class LightCtrl {

 private:

  /// the list of the information about the devices
  //vector < MuxExchange*> thedev ;
  /// the list of connected mux devices
  pmap <int , LedCardInterface*> fLedCard;
  
 public:

  LightCtrl();
  ~LightCtrl();
  /** set the light level to level for all led cars.
      returns the level returned by the LAST card (for the moment)
  */
  float setLightLevel(float level);
  /** set the light level to level for the card with ledcardid
   */
  float setLightLevel(float level,int ledcardid);
   /** set the light level to level for the card with ledcardid
       for the given channel
   */
  float setLightLevel(float level,int ledcardid, int channel);

};
#endif 
