#ifndef _Module_H
#define _Module_H


#include <vector>
#include <qdatetime.h> 

#include "DaqCtrl.h"
#include "ModId.h"
#include <TH1.h>
#include <TProfile.h>
#include <TGraph.h>


#include "HybridI2C.h"
#include "TestBase.h"
#include "PedestalRun.h"
#include "CalProfRun.h"
#include "CalRun.h"
#include "LatRun.h"
#include "LightRun.h"
#include "FbRun.h"
#include "LedRun.h"
#include "BPlaneRun.h"
#include "IVRun.h"
#include "AuxChipRun.h"
#include "ConnectivityRun.h"
#include "Spill_Analysis.h"

/** \brief Module object with napvs at benchpos with  test methodes
           provides:
           Pedestal,Rms,CalibrationProfile,Calibration etc
           I2C sets  through apv,pll,mux i2c channels
              
     \author V.Zhukov,UIA  
     \version 2.1
     \date 02/2003
 
*/

class ModuleHv;
class Record;
class RecModule;
class SetParser;
class BenchPosConnection;
class CurrentChannelInterface;
class DioChannelInterface;
class TemperatureChannelLt;
class FedChannelLt;

class Module :public ModId {  
public:

Module(ModId* modid, Record* r,DaqCtrl* ctrl);
 friend class TestProcedure;
~Module();

bool Init(const QString& s);
bool Results(const QString& s);
TestBase* Test(const QString& s);
void UpdateXt();

//setters getters
void Settings(SetParser* sp);
PedestalRun* pedestalrun() { return thepedestalrun;}
CalProfRun*  calprofrun() { return thecalprofrun;}
CalRun*      calrun() { return thecalrun;}
LightRun*    lightrun() { return thelightrun;}
FbRun*       fbrun() { return thefbrun;}
LatRun*      latrun() { return thelatrun;}
BPlaneRun*   bplanerun() { return thebplanerun;}
LedRun*      ledrun()   { return theledrun;}
HybridI2C*   hybridi2c() {return thei2c;}
IVRun*       ivrun() {return theivrun;}
ModuleHv*    modulehv() {return themodulehv;}
ConnectivityRun* connectrun() {return theconnectivityrun;}
AuxChipRun* auxchiprun() {return theauxchiprun;}

std::vector<Spill_Analysis*>  apvanalysis() {return theapvanals;}
std::vector<FedChannelLt*>    feds() {return thefeds;}
RecModule* getrecmod() {return therecmod;}
DaqCtrl* getdaqctrl() {return thectrl;}
 
void setTime(QDateTime& time) {stime=time;}

int getBenchPos() const {return theBench;}
/** this function returns the number of acquistioin errors for this module
*/
 int getModuleAcqErrors(){ return fModuleErrStatus;}


/** this returns the module error status 
    If this is set to true the module should be considered as 
    defect and should be used anymore in a test
    This is a result of I2c errors and acquisition errors 
*/
 bool  getModuleErrStatus();
/** switch on or off the power for the module
    @param onstatus  if true switch on else switch off
*/ 
void switchPowerOn(bool onstatus);
int getGroupNr() const {return themuxgroupnr;}
/** return true if the mux channel is set to this module */
bool ConntectToDAQ() {
   return thectrl->getActiveGroup() == themuxgroupnr;
}

/** returns true if one of the APV's of this module
    is connected to fed channel 0 
*/
bool isConnectedToFedCh0(){ return fFedChannel0;}

/** returns the daqctrl pointer */
 DaqCtrl* getDaqCtrl(){return  thectrl;}


private:
std::vector<FedChannelLt*> thefeds;
std::vector<Spill_Analysis*> theapvanals;

Record* therec;
RecModule* therecmod;
HybridI2C* thei2c; 
ModuleHv* themodulehv; 
BenchPosConnection* thebpos;
CurrentChannelInterface*  thecurrent125 ;
CurrentChannelInterface*  thecurrent250 ;
DioChannelInterface* thepowerswitch;
TemperatureChannelLt* thetempsen1;
TemperatureChannelLt* thetempsen2;
SetParser* thesettings; 

PedestalRun* thepedestalrun;
CalProfRun* thecalprofrun;
LatRun* thelatrun;
CalRun* thecalrun;
BPlaneRun* thebplanerun;
LedRun* theledrun;
IVRun*  theivrun;
LightRun*  thelightrun;
FbRun*  thefbrun;
AuxChipRun* theauxchiprun;
DaqCtrl* thectrl;
ConnectivityRun* theconnectivityrun;
 bool theOn;
QDateTime stime;
int theBench;
int themuxgroupnr;
// will be true if one of the APV's is connected to FED channel 0; 
 bool fFedChannel0;
 protected:
/** this variable is increased by the TestProcedure
    in case an error is detected 
*/
 int fModuleErrStatus;
};

#endif //_Module_H




