#ifndef _ModuleTestSetup_H
#define _ModuleTestSetup_H

#include <qdatetime.h> 
#include <qstring.h>
#include  <vector>


/**  
\brief ModuleTestSetup includes : setup , steering initialization


<p>   Creates  objects: setup, apvreadoutctrl, record
      Parse configuration file settings.xml
      Modules setups can be added 
      Creates TestProcedure 


   \class ModuleTestSetup
   \authors: V.Zhukov,UIA W.Beaumont
   \date 06/2002
   \version 1.0
*/  

class Setup;
class DaqCtrl;
class TestProcedure;
class Module;
class ModId;
class Record;
class SetParser;
class QObject;
class DioChannelInterface;
class ColdBox;

class ModuleTestSetup {
 public:
ModuleTestSetup();
  ~ModuleTestSetup();

public:
void ResetRecord();                      // reset all data 
void ParseSettings(const QString& s);    // read settings.xml file
void ParseLt();                          // read lt.xml  configuration file


//  getters base test
int   getCTime() const;
QDateTime getStartTime() const {return stime;}
Setup*          getSetup()  {return thesetup;}
DaqCtrl* getDaqCtrl()  {return thedaqctrl;}
Record*         getRecord() {return therecord;}
SetParser*     getSetParser() {return thesettings;}
TestProcedure* getTestProcedure() {return thetestproc;}
/** returns the pointer to the cold box in SetUp */
 ColdBox* getColdBox();
/** is true if a cold box is defined */
 bool coldBoxValid();
//  getters module info
/// the the number of  regtistered modules in the setup
int   getNmod() const {return theNmod;} 
// get themax number of apvs per module 
int   getNapv() const {return theNapvmax;} 
/// 
std::vector<Module*> getModules()  {return themodules;} 
/** add modules to the test setup 
    @param modid the vector with the information of the modules
    @retruns the number of modules 
*/
int AddModules( std::vector<ModId*>* modid);
 /// powers on the defined bench position where no valid module is registered      
 void powerOnNotRegisteredMod( std::vector<ModId*>* vmi);
 /// powers off the not registered benchpos and release the diochannels 
 void powerOffNotRegisteredMod( );

/// create the test procedure 
void CreateTestProcedure(QObject* qo=0);

/** Remove the modules and delete the test procedure
    @return 0 if o.k. -1 if errors (descruction not possible)
 */
 int StopTest();

private:
 

Setup* thesetup;
DaqCtrl* thedaqctrl;
SetParser* thesettings;
Record* therecord;
TestProcedure* thetestproc;
std::vector<Module*> themodules;
 QDateTime  stime;

 // base test setup
 int theNtemp; //ntemp probes in the setup
 int theNhum;  // nhumidity probes in the  setup
 int thecstep; // current steps in scenario 
 int theRunNr; // the run number 

 // module information
 int theNmod;  //nmodules in the setup
 int theNapvmax;  //max number of apvs
 std::vector<DioChannelInterface*>  fNotRegPowerSwitches;

};
#endif
