#ifndef MUXCTRL__
#define MUXCTRL__
#include "pmap.h"
#include <vector>
#include <map>
/** \class  MuxCtrl 
    \author W.Beaumont,
    \date Oct  2002
    \version 1.0

    \brief multiplexer control for the module test program


    This class provides the interface of the setting of the MUX 
    (active mux group  and the active status of the mux channel. 
    The active group is the group that is at that moment connected
    to the FED. Most natural the group is the same number as the 
    mux channel but this is not necessary.
    The implementation of the class is complex because the option
    to have more than one mux device is left open. ( although not fully
    coded in version 1.0). For the Karlsruhe multiplexer there is one
    device ( I2c channel) and more cards that are controlled by that card. 
    Most natural was to control only one mux channel ( so the device
    (card) that is connected to one fed channel. But as they are connected
    to the same channel the cards are set per device. 
    This results in a relative complex initialization of the hardware.
    The hardware initialization is done by the muxdev class .
    The basic device for the modules is the fedchannel. This is connected to 
    a mux card and have in principle not to know to wich mux device it is
    connected. For this reason the muxcards itself have  id that is made up t
    form the mux card nr , the mux device number and the device type 
*/
class MuxExchange;
class  MuxDevInterface;
class MuxCtrl {
/** the attachment to the mux devices  is based on the the lt.xml file 
    no parameters are needed
*/
 private:
  /// indicates if a real device is defined 
  bool fRealMuxCtrl;
  /// the list of the information about the devices
  std::vector < MuxExchange*> themuxdev ;
  /// the list of connected mux devices
  pmap <int , MuxDevInterface*> fMuxDev;
  /** the lis of group nr to which modules are connected 
      the key is the groupnr the second number is the nr of 
      modules connected to this group 
   */
  std::map<int,int> theRegisterGroups;
  std::map<int,int>::const_iterator fActPointer;
 public:

  MuxCtrl();
  ~MuxCtrl();
  /** returns the acctive group nr this doesn't access the hardware so is
      fast (and it is inline) 
      It rely's on the assumption that all muxes are controlled by 
      this class.
  */
  int getActiveGroup() {return fActPointer->first;}
  /** @return the number of groups register */
  int getNrGroupsRegistered(){ return theRegisterGroups.size();}
  /** set the active group (hw and sw ) for all defined
      multiplexer devices 
      if the groupnr isn't registerd it returns false
      it sets the actual pointer to this group ! 
      @param groupnr the active group nr
      @returns true if no error is found 
  */
  bool setActiveGroup( int groupnr);
  /** set the active group (hw and sw ) for all defined
      multiplexer devices 
      This doesn't check if the group nr is defined and
      doesn't update the actual pointer
      @param groupnr the active group nr
      @returns true if no error is found 
  */
  bool setActiveDevGroup( int groupnr);
  /** register the groupnr as a group to which a module is
      connected. If the group already exist 
      the connected nr will be increased
  */
  void registerGroup(int groupnr) ;
  void unregisterGroup(int groupnr);
  int getFirstGroup();
  /** sets the mux devices to the next group and returns
      next group
      @return the new active mux group 
   */
  int getNextActiveGroup();
  /** returns true if as long the next group is not reached.
      to be used in a for loop.
      for( group=muxctrl.getFirstGroup;
      muxctrl.lastGroupNotReaced;
      group=muxctrl.nextActiveGroup())
      setActiveGroup(group);
  */
  bool lastGroupNotReached();
  /** returns the numbe of Mux devices controlled by this object */
  int getNrMuxDev(){ return themuxdev.size();}
  /** return true if a mux device was defined in the ltparser*/
  bool getNrMuxCtrlDev(){return fRealMuxCtrl;}
};
#endif 
