#ifndef _OnlineMonitor_H
#define _OnlineMonitor_H

#include <qapplication.h>
#include <qthread.h>
#include <vector>

#include <TNamed.h>
#include <TGraph.h>
#include <TVector.h>
#include <TH1.h>
#include <TProfile.h>
#include <TMessage.h>
#include <TSocket.h>
#include <TServerSocket.h>
#include <TDirectory.h>



/**  
 \brief Online monitor based on Root sockets


   \class OnlineMonitor
   \authors: V. Hovhannisyan, V.Zhukov  UIA
   \date 02/2003
   \version 2.0

<p>      Test data TCP/IP server is starting as  a thread
         The data is stored in "header", "setup",
         "modules"  groups.
         The server works together with the TCP/IP client
         and DB Browser omon.C or omon.
         
 <h2> Usage </h2>
<pre>
       
</pre>                      

*/
class Record;

class OnlineMonitor : public QThread  {

public:

  virtual void run();

  OnlineMonitor(Record *rec);  
  void  StartThread();  
  int StopThread();
  void setUpdate(int i) {dtsleep=i;}  // change update rate sleep(i)

private:

  Record* therec;
  int dtsleep;
  TSocket* ss;
  TMessage msg;
  bool Sending;
  bool SendingActive;

  void send();
  void Save(TDirectory* dir) {;}

  void Send(TH1F* histo) 
{ msg.Reset();msg.WriteObject(histo);ss->Send(msg); }

  void Send(TProfile* prof) 
{ msg.Reset();msg.WriteObject(prof);ss->Send(msg); }

  void Send(TVector inpvector) 
{ msg.Reset();msg.WriteObject(&inpvector);ss->Send(msg); }

  void Send(TNamed named) 
{ msg.Reset();msg.WriteObject(&named);ss->Send(msg); }

void TGSend(std::vector<float>* x,std::vector<float>* y,const char* title);

};
#endif


