#ifndef _Setup_H
#define _Setup_H

#include  <vector>
#include <qdatetime.h>

/**  
  \brief        Experimental Setup 

<p> Setup includes temperature and humidity channels and cooling steering   

   \class Setup
   \authors: V.Zhukov,UIA
   \date 02/2003
   \version 2.0

*/
class ColdBox;
class Record;
class SetParser;
class SlowCtrlConfig;
class TemperatureChannelLt;
class TemperatureChannelInterface;
class HumidityChannelLt;
class HumidityChannelInterface;



class Setup  {
public:
Setup(Record* r);
~Setup();

  void ChangeTemp(float T); 
  void UpdateXt(); 
  void UpdateCool();

public:
  //getters
std::vector<TemperatureChannelLt*>  getTempCh()  {return thetempch;}
std::vector<HumidityChannelLt*>    getHumCh()   {return thehumch;}
ColdBox* getTempCtrl()   {return thetempctrl;}
/** is true if a cold box is defined */
 bool  coldBoxValid();
/** return the requested temperature */ 
 float  getCurTemp(){return theCurTemp;}
/** returns the actual box temperature */
 float getActualTemp();
float getBoxHumidity();
 bool maxHumidityReached();
  //setters
void setTime(QDateTime& time) {stime=time;}
void Settings(SetParser* set);

private:
SetParser* thesettings;
SlowCtrlConfig* thescc; 
std::vector<TemperatureChannelLt*> thetempch; 
std::vector<HumidityChannelLt*> thehumch;
ColdBox*  thetempctrl; 
Record* therec;
  //
  int theNtemp;
  int theNhum;
  float theCurTemp;
  HumidityChannelInterface* theboxhumidity;
  HumidityChannelInterface* theambienthumidity;
  TemperatureChannelInterface* theboxtemp;
  TemperatureChannelInterface* theambienttemp;
  TemperatureChannelInterface* thetempctrltemp;
  TemperatureChannelInterface*  theheatsinktemp;
  //
QDateTime stime;
};
#endif
