#ifndef _TestProcedure_H
#define _TestProcedure_H

#include <qobject.h>
#include <qthread.h>
#include <qstring.h>
#include <qdatetime.h> 
#include <string>
#include  <vector>


/**  
\brief Tests Steering
<p>   
      Provides test actions and 
      test scenario steering based on Qt timers.
      It uses already constructed objects: setup, modules, apvctrl,record

   \class TestProcedure
   \authors: V.Zhukov,UIA
   \date 02/2002
   \version 2.1
   \date 02/2003
      


*/  
 
class SetParser;
class QObject;
class Record;
class DaqCtrl;
class I2cDescriptor;
class Module;
class TestStatus;
class Setup;
class QTimer;
class TestProcedure : public QObject,public  QThread  {

  Q_OBJECT

 public:
TestProcedure(QObject  *callingparent=0,
const char *name=0);

void Construct(Setup* s,std::vector<Module*> m,DaqCtrl* c, SetParser* p,  Record* r);
~TestProcedure();

public slots:  // running with timers

void ScenarioAction();  //select test according to the scenario by timer
void CheckEnv(); // check Slowctrl parameters by timer

 public:

bool LoadScenario(const QString&  s);    // load  scenario s
bool LoadScenario();                     // load predefined in settings.xml scenarion
bool ScenarioSteering(const QString& s); // steer(start, stop, pause ) scenario
bool StartTest(const QString& s);        // start test s
/** call the set pll for all modules and set the pll delay to
    the value defined in the i2c descriptor also inits the pll
    if this is indicated in the i2cset
*/
void setPll(I2cDescriptor* i2cset);
 void setLld(int gain, int bias, int channel=-1);
void SetI2c(I2cDescriptor* i2scset);     // set i2cset for all modules
/**change hv for all modules 
   @param hv the requested hv 
   @param block if true , wait until voltage is reaced else return without waiting
*/
void ChangeHV(float hv,bool block=true);                
/** check if all  the modules reached the requested voltage.
    the requested voltage is set to the modules by the ChangeHv method
    The module will indicate if the voltage is reached. 
    @param wait if wait is true it waits before it checks the first module 
    @return true if the hv is same as the last requested voltage 
*/
bool HVReached(bool wait=true);


void ChangeTemp(float temp);             // change cooling 
void Abort();                            // abort running test
 void InterLockAction();                // called when interlock becomes active

void SaveResults(const QString&  s,QWidget* vof); // save results into root file
 void SaveResults(const QString&  s);
void SaveDb(const QString&  s);          // parse to the db  
// setters getters
void  setDt(int dt);             //change dt for the slowctrl update
float getSTimeEnd() const {return theScenEndTime+dtpause;} //estimated endscenariotime
float getSTime() const {return (fTime->elapsed()/1000-theScenStartTime);} //scenario time
QString  getCurrentScenarioLineInfo(); 
void setTime(QDateTime& time) {stime=time;}
int   getNacq()  const {return theNacq;}   // number of fed spills
TestStatus*    status() {return thestatus;}
I2cDescriptor* getCurrentI2c() {return  currenti2c;}
bool  UpdateTestAction();


void Tologfile(const  char* s, float f);
void Tologfile(const  char*  s,const  QString& c);
void Tologfile(const char* s);

struct Scenario {
  int dt;
  std::string rtype;
  std::string  arg1; 
};


signals :
    /* is send when apply tsc settings is called */
    void feedBackSignal( QString message);

private:

 virtual void  run();  // thread executable
 
bool updateTestAction();
bool readscenario(const QString& s);
void setapvctrl(const QString& s);

/** set to true if a test needs the daq (FED) readout
 */
 bool fDaqUse;
 int fActiveGroup;
std::vector<Module*> themodules;

Setup* thesetup;
DaqCtrl* thedaqctrl;
SetParser* thesettings;
I2cDescriptor* currenti2c;

Scenario*  myburn;
TestStatus* thestatus;
Record* therecord;
std::string  theScType;//<indicates longterm or default test
QTimer* actionTimer;
QTimer* slctrlTimer; 
QString curtest;
QTime* fTime;
QDateTime stime;
float dtpause;

float theScenStartTime; //time start scenario
float theScenEndTime;  // time end scenario
int theNsteps; // nsteps in scenario
int thecstep; // current steps in scenario
 int thelastchtempstep; // the line in the scneario where tempchange is requested 
 float therequestedtemp; //last requested box temperature
int theNacq;       // number of spill acquisitions 
QString rfname;   // results filename
int theNrecord; // current record in save results 
int dwait;    // waiting time to the next test
bool checkenvpending; /**< is set if a check env is not done because 
			  a run is bussy 
		      */                      
std::vector<int> selectedrec;
 int theUpdateTries;
// map<QString, int> theselectedrec;
};
#endif
