
#include "ApvReadoutCtrl.h" 

#include <tscchannelinterface.h>
#include "moveinterface.h"
#include "daqerrormsg.h"
#include "setparser.h"
#include "daqctrldescriptor.h"
#include "diochannelinterface.h"
#include "FedCtrl.h"
#include "ltparser.h"

//#include "DMC1000.h"
using namespace std;
extern LtParser* ltparser;  



ApvReadoutCtrl::ApvReadoutCtrl(SetParser* thesettings, int tscdevice, int bpulsectrl ){
  fInitialized =true;
  fSpillSize=100;
  thetsc=0;thefedctrl=0;themove=0;
  fFreshSpillRequest=false;
  thedaqctrldescriptor = thesettings->getDaqCtrlDescriptor("default");
  //tscdevice should always be 0 ;
  thetsc=  ltparser->getTscCard(0);    
  if(! thetsc ){ DT_WARN("TSC not initialized");}
  thefedctrl = new FedCtrl(ltparser) ; 
   //return 0 if not correct initialized 
  if (! thefedctrl->isInitialized() ){fInitialized =false ; return;}
  theBpEnable= ltparser->getBackPlanePulseCtrl(tscdevice);
  disableBackPlanePulse();		    

#ifdef LASERMOVER 
  themove =MoveInterface() ; //  DMC1000( "/dev/dmc1040" );
  themove->SetSpeedXY( 10000, 10000 ); 
  themove->GotoAbsoluteOrigin( WAIT );
  themove->ResetCoordinates();
  themove->DoHardReset();
#else
  themove=new MoveInterface();;
#endif  			    
  ApvReadoutCtrlSettings(thesettings);
  fApvsconnected=0;fApvnr=0;
  fInitialized =true;  
}

FedProcessCtrlInterface* ApvReadoutCtrl::getFedProcessCtrl(int fednr=0){
  return thefedctrl->getFedProcessCtrl(fednr);
}


int ApvReadoutCtrl::ApvReadoutCtrlSettings(SetParser* thesettings){
  thedaqctrldescriptor = thesettings->getDaqCtrlDescriptor("default");
  fSpillSize= thedaqctrldescriptor->spillsize;
  thefedctrl->setSpillSize(fSpillSize);
  thetsc->lock();  
  thetsc->setClockMin(3);
  thetsc->setTriggerFrequency(thedaqctrldescriptor->tsctriggerfrequency);
  thetsc->setInternalTrigger();
  thetsc->disableInterrupt();
  thetsc->setTriggerCount(fSpillSize+5);
  thetsc->setResetLatency(thedaqctrldescriptor->tscresetdelay);
  thetsc->setFedLatency(thedaqctrldescriptor->tscfeddelay);
  fTscLatency=thedaqctrldescriptor->tsctriggerdelay;
  thetsc->setApvLatency(fTscLatency);
  thetsc->setCalibrationLatency(thedaqctrldescriptor->tsccaldelay);
    
  thetsc->enableReset();
  thetsc->enableCalibration();
  thetsc->disableInhibitTrigger();
  //thetsc->setReenableTrigger(true);
  thetsc->justDoIt();
  thetsc->unlock();
  thetsc->wait();
  
  return 0;
}

ApvReadoutCtrl::ApvReadoutCtrl( )
   {  fFreshSpillRequest=false;
  fApvsconnected++;
 fApvnr=  fApvsconnected;
 if (! fInitialized )    {    
   //  thetsc=0; theBpEnable=0; shoudn't be don here !!!
   DT_WARN( " apv readout not initilazed when creating apv " <<  fApvnr );
 }

}
bool  ApvReadoutCtrl::startDaqTest(){
  thefedctrl->setSpillSize(fSpillSize);
  thefedctrl->clearFed();
  thetsc->lock();
  thetsc->setTriggerCount(fSpillSize+5);
  thetsc->setResetFed(true);
  thetsc->setResetApv(true);
  thetsc->justDoIt();
  thetsc->unlock();
  thetsc->wait();   
  return true;
}
 
bool ApvReadoutCtrl::stopDaqTest(){
  thetsc->setReenableTrigger(false);
  thefedctrl->stopFedDaqLoop();
  return true;
};

bool ApvReadoutCtrl::NewSpill(bool fresh )  { 
  bool  newspill=false;
  int flushtriggers=6;
  DT_DEBUG_LTAPPL("request  with  "<< fresh);
  if (fresh) {fFreshSpillRequest=true;}
  if(fFreshSpillRequest)  { 
    thefedctrl->setSpillSize(fSpillSize);
    // clear the buffers after the FED  
    thefedctrl->NextSpill();
    thetsc->lock();
    // jus trigger a number to flush existing buffers
    thetsc->setTriggerCount(flushtriggers);
    thetsc->setResetFed(true);
    thetsc->setResetApv(true);
    thetsc->setReenableTrigger(true);
    thetsc->justDoIt();
    thetsc->unlock();
    thetsc->wait();
    int waits=0;
    while ( thefedctrl->getNumberOfEvents()<flushtriggers-2) {
     usleep(3000) ;
     if((waits++)>1000) {
       DT_WARN("Flush function failed restart DAQ");
       stopDaqTest();
       usleep(100000);
       startDaqTest();
       break;
     }
    }
    //thefedctrl->clearFed(); to much delay 
    thefedctrl->NextSpill();
    thetsc->lock();
    thetsc->setTriggerCount(fSpillSize+5);
    thetsc->setResetFed(true);
    thetsc->setResetApv(true);
    thetsc->setReenableTrigger(true);
    thetsc->justDoIt();
    thetsc->unlock();
    thetsc->wait();
    fFreshSpillRequest=false;     
    newspill=false;
  }
  else { 
    newspill=thefedctrl->NewSpill();
   //   if(newspill){ 
//        thetsc->lock();
//        thetsc->setReenableTrigger(false);
//        thetsc->justDoIt();
//        thetsc->unlock();
//        thetsc->wait();  
//      }
  }
  return  newspill;     
}

void ApvReadoutCtrl::NextSpill() {
  thefedctrl->NextSpill();
  thetsc->lock();
  thetsc->setTriggerCount(fSpillSize+5);
  thetsc->setReenableTrigger(true);
  thetsc->justDoIt();
  thetsc->unlock();
  thetsc->wait();
  
}




bool ApvReadoutCtrl::isFirstApv() 
{ if (fApvnr == 1)
  return true;
  else
    return false;
}
 bool ApvReadoutCtrl::isLastApv ()
{  if ( fApvnr ==fApvsconnected)
   return true;
  else
    return false;
}


TscChannelInterface* ApvReadoutCtrl::thetsc ;

DioChannelInterface*  ApvReadoutCtrl::theBpEnable;
MoveInterface*  ApvReadoutCtrl::themove;
FedCtrl* ApvReadoutCtrl::thefedctrl ;

bool ApvReadoutCtrl::fInitialized ;
int ApvReadoutCtrl::fApvsconnected ;

ApvReadoutCtrl::~ApvReadoutCtrl()

{   if ( theBpEnable ) { delete theBpEnable ;}
  if(thetsc){  thetsc->release(); delete thetsc;}
  if(thefedctrl) {delete thefedctrl;}
}


void ApvReadoutCtrl::enableReset(bool update){ 
  if(thetsc)   {
    DT_DEBUG_LTAPPL("update = "<< update);
    thetsc->lock();  
    thetsc->enableReset();
    if( update)    thetsc->justDoIt();
    thetsc->unlock();
    if( update)thetsc->wait();
  }
}


//
void ApvReadoutCtrl::disableReset(bool update){ 
  if(thetsc)    {
    DT_DEBUG_LTAPPL("disableReset ");
    thetsc->lock();
    thetsc->disableReset();
    if( update)thetsc->justDoIt();
    thetsc->unlock();
    if( update)thetsc->wait();
  }
}



void ApvReadoutCtrl::enableCalSeq(bool enable,bool update){ 
  if(thetsc)    {
    DT_DEBUG_LTAPPL("set to  "<<enable << "update now : "<< update );
    thetsc->lock();
    if(enable)thetsc->enableCalibration();
    else thetsc->disableCalibration();
    if( update)thetsc->justDoIt();
    thetsc->unlock();
    if( update)thetsc->wait();
  }
}


//
void ApvReadoutCtrl::setTscLatency(int lat){ 
  if(thetsc)  {  
    thetsc->lock();
    fTscLatency=lat;
    thetsc->setApvLatency(lat);
    thetsc->justDoIt();
    thetsc->unlock();
    thetsc->wait();
  }
}
//
void ApvReadoutCtrl::setTscLatency(){ 
  if(thetsc)  {  
    thetsc->lock();
    fTscLatency=thedaqctrldescriptor->tsctriggerdelay;
    thetsc->setApvLatency(fTscLatency);
    thetsc->justDoIt();
    thetsc->unlock();
    thetsc->wait();
  }
}
int  ApvReadoutCtrl::getTscLatency() {
  return fTscLatency;
} 


void ApvReadoutCtrl::sendApvReset()
{ 
  if(thetsc)
  {  thetsc->lock();
     thetsc->setResetApv(true);
     thetsc->justDoIt();
     thetsc->unlock();
     thetsc->wait();
  }
  // if the apv reset is used after changing setting so 
  // take care for sending fresh spill 
  fFreshSpillRequest=true;
}


bool  ApvReadoutCtrl::sendApvReset(string c )
{ bool issend =false;
  if ( c== "first"  ) 
    {  if (fApvnr == 1) 
      { sendApvReset(); issend=true;
       fFreshSpillRequest=true; }
    }
 if ( c== "last" ) 
    {  if (fApvnr == fApvsconnected) 
      { sendApvReset(); issend=true;
       fFreshSpillRequest=true; }
    }
    
 return issend;
}

void ApvReadoutCtrl::disableBackPlanePulse()
{ if(theBpEnable)
  {   theBpEnable->setBit(1);  }
}


void ApvReadoutCtrl::enableBackPlanePulse()
{ if(theBpEnable)
  { theBpEnable->setBit(0);  }
}





// motor control



void ApvReadoutCtrl::MoveSpeed( int speed)
{ if(themove)
  { themove->SetSpeedXY(  speed ,  speed ); }
}
void ApvReadoutCtrl::GoToStartPos(LedRunDescriptor* ledset  )
{ // ledsets->startX, ledsets->startY 
}
void ApvReadoutCtrl::StartMoving ()
{ // set ledsets->speedX 
  //void ApvReadoutCtrl::MoveTo( ledsets->stopX, ledsets->stopY );

}
// methodes from inline

int ApvReadoutCtrl::getNrEvents()
 {return thefedctrl->getNumberOfEvents();}

bool  ApvReadoutCtrl::AdjustFedThresholds(int& low , int& high){ 
 bool retval =thefedctrl->AdjustThresholds(low , high);
//  cout <<"low " << low<<" high " << high <<endl;
  return  retval;
}

bool  ApvReadoutCtrl::AdjustFedThresholds(){ 
   return thefedctrl->AdjustThresholds();
}

bool  ApvReadoutCtrl::setFedClkDelay(int & delay){ 
   return thefedctrl->TuneFedSampleTime(delay);
}

void  ApvReadoutCtrl::setScopeMode(int mode){
  thefedctrl->setScopeMode(mode);
}
