#include <algorithm>
#include "Apv_Analysis.h"
#include "LtDefs.h"
#include "daqerrormsg.h"
using namespace std;

Apv_Analysis::Apv_Analysis(int i)
{                                            
// default
  thePolarity=1;  
  theNStrips=32;
  theNpedevent=1000;
  theRmsmin=0.2;
  theRmsmax=50; 
  thePedmin=0.;
  thePedmax=500.;
  thePskip=0.3; 
  theUpdate=60;                                                 
  theId=i;

 Clear();
 DT_DEBUG_LTAPPL("constructed apvanalysis "<<i);
}

//
Apv_Analysis::~Apv_Analysis(){
 pedDistribution.clear();
 noisDistribution.clear();
 theChStatus.clear();
 theNoisyCh.clear();
}
//
void Apv_Analysis::Clear(){

 pedDistribution.clear(); pedDistribution.resize(NAPVCH);
 noisDistribution.clear(); noisDistribution.resize(NAPVCH);
 theData.resize(NAPVCH); theData=0.; 
 thePedestal.resize(NAPVCH); thePedestal=0.;
 theRms2.resize(NAPVCH); theRms2=0.;
 theRms.resize(NAPVCH); theRms=0.;
 theRawRms2.resize(NAPVCH); theRawRms2=0.;
 theZero.resize(NAPVCH); theZero=0.;
 int ns=NAPVCH/theNStrips;
 theCmc.resize(ns); theCmc=-1000.;
 theSignal.resize(NAPVCH);theSignal=0.;
 theChStatus=vector<int>(NAPVCH,0);
 theNPed=0;
 theNBad=0;
 theNRms=0;
 theMeanRms=0.;
 theNevents=0;
 theNoisyCh.clear();   
}
//
void Apv_Analysis::updateMeanPed()
{ 
 theSignal=theData-thePedestal;
 thePedestal+=theSignal/(theNPed+1.);
 if(theNPed!=0) 
 theRawRms2=(theRawRms2*(theNPed-1.)+theSignal*theSignal)/theNPed;
 theNPed++;
}
//
void Apv_Analysis::updateMeanNoise()
{
  vector<int> skip=updateskipCh(theSignal,theRms);
   Fvec sig(0.,NAPVCH);
 for(int i=0;i<NAPVCH;i++) {
   if(find(skip.begin(),skip.end(),i)!=skip.end() ) 
     sig[i]=0.;
    else  sig[i]=theSignal[i];}

 Fvec d(sig.size());
  d=sig-theZero;
  theZero+=d/(theNRms+1.);
  if(theNRms!=0)
  theRms2=(theRms2*(theNRms-1.)+d*d)/theNRms;
  theNRms++;
}
//
void Apv_Analysis::updateMedianPed()
 {
       theNPed++;
   for(int is=0;is<NAPVCH;is++){   
      float adc=theData[is];
   if(pedDistribution[is].find(adc) != pedDistribution[is].end())
    pedDistribution[is][adc]++;
    else pedDistribution[is][adc]=1; 
          }
  if((theNPed>theUpdate)&&!(theNPed%theUpdate))   {
  for(int is=0;is<NAPVCH;is++){  
      float rms=0.;
     FloatMapType map=pedDistribution[is];
     thePedestal[is]=findMedian(map,theNPed,rms);
      theRawRms2[is]=rms*rms;}
              }
}
//
void Apv_Analysis::updateMedianNoise()
  {
    theNRms++;
     for(int is=0;is<NAPVCH;is++){   
   float adc=theSignal[is];
   if(noisDistribution[is].find(adc) != noisDistribution[is].end())
       noisDistribution[is][adc]++;
    else noisDistribution[is][adc]=1; }
     //update each 20 event
 if((theNRms>theUpdate)&&!(theNRms%theUpdate)) { 
  for(int is=0;is<NAPVCH;is++){   
  float rms=0.;
  FloatMapType map=noisDistribution[is];
  findMedian(map,theNRms,rms);
    theRms[is]=rms;
    theRms2[is]=rms*rms;
     }
   }
}
//
void Apv_Analysis::SubtractPed()
{
 theSignal=theData-thePedestal;
}
//
void Apv_Analysis::SubtractCM()
{
 int ngr=int(NAPVCH/theNStrips);
 theCmc=0.;
 Fvec cmc(0.,ngr);
 Fvec cmcall(0.,NAPVCH);
 theSignal.resize(theData.size());
 theSignal=theData-thePedestal;
 
 Fvec rms(theRawRms2.size());
    rms=sqrt(theRawRms2);
   for(int iter=0;iter<theNcmniter;iter++){ 
      vector<int>  skip=updateskipCh(theSignal,rms);
     if(theModcmn==1) cmc=cmnMean(theSignal,skip);
     if(theModcmn==2) cmc=cmnMedian(theSignal,skip);
       rms=theRms;
       for(int i=0;i<ngr;i++){
         int ig=i*theNStrips;
         slice s(ig,theNStrips,1);
         cmcall[s]=cmc[i];}
     
     theSignal=theSignal-cmcall;
     theCmc+=cmc; }
}
//
float Apv_Analysis::findMedian(const FloatMapType& map,const int nused, float& Rms)
{
  float med=0.;
  float rmsdwn=0.;
  float rmsup=0.;
  Rms=0.;   
     int half=nused/2;
     int n_32=int(nused*0.32/2.); // 0.32% of Ntot ~rms
     int nbelow=0;
     int nmax=0;
     bool bfoundmedian=false;
     bool bfoundrmsdwn=false;
     bool bfoundrmsup=false;
  // find median (sum of left entries=sum of right entries)
     FloatMapType::const_iterator j, jp,jn; 
     for (j = map.begin();j != map.end(); j++) {
       float ich=j->first;
       jp=map.find(ich-1);
       jn=map.find(ich+1);


       nbelow+=j->second;
       if(nbelow>=half&&!bfoundmedian) 
       { med=j->first; nmax=j->second;bfoundmedian=true;}
       if(nbelow>=n_32&&!bfoundrmsdwn) 
      { rmsdwn=(j->first)-(nbelow-n_32)/((j->second)+0.5);bfoundrmsdwn=true;
      // cout<<" fmed "<<r<<" "<<rmsdwn<<"nb  "<<nbelow<<" "<<n_32<<" "<<nl<<" g "<<g<<endl;
          }
       if(nbelow>=(nused-n_32)&&!bfoundrmsup) 
      { rmsup=(j->first)-(nbelow-nused+n_32)/((j->second)+0.5);bfoundrmsup=true;}      
      }  
      Rms=fabs(rmsup-rmsdwn)/2.;
  return med;
}
//
valarray<double>  Apv_Analysis::cmnMedian(const Fvec& sig,const vector<int>& skip)
{
    int ngr=int(NAPVCH/theNStrips);
    Fvec cmc(0.,ngr);
    vector<int>  nused(ngr,0);
    vector<FloatMapType> adcDistribution (ngr);
 
  for(int i=0;i<NAPVCH;i++)
    {
      if(find(skip.begin(),skip.end(),i)!=skip.end() ) continue;

     if(theChStatus[i]>0) continue;
      float adc   =sig[i];     
      int igr = int(i/theNStrips);
       nused[igr]++; 
      if (adcDistribution[igr].find(adc) != adcDistribution[igr].end() ) 
        adcDistribution[igr][adc]++;
      else adcDistribution[igr][adc] = 1;
  }
  for (int k = 0; k<ngr ; k++) {
    float rms=0.;
    if(nused[k]>1) {
     FloatMapType map=adcDistribution[k];
     cmc[k]=findMedian(map,nused[k],rms);
    }
  }
    return cmc;
}
//
valarray<double>  Apv_Analysis::cmnMean(const Fvec& sig, const vector<int>& skip)
{
 int ngr=int(NAPVCH/theNStrips);
 Fvec cmc(0.,ngr); 
  float cmn;
  int ich;
     for(int ig=0;ig<ngr;ig++){
        int n=0;
        cmn=0.;
    for(int i=0;i<theNStrips;i++){
    ich=ig*theNStrips+i;
    if(find(skip.begin(),skip.end(),i)!=skip.end() )  continue;
   if(theChStatus[i]>0) continue;
         cmn+=sig[ich]; n++;
      }
if(n>0) cmc[ig]=cmn/n;
   }
  return cmc;
}
//
vector<int>  Apv_Analysis::updateskipCh(const Fvec& sig,const Fvec& rms)
{
  vector<int>  skip;
  Fvec asig(sig.size());
    asig =abs(sig);
  float thr=asig.sum()*theTskip/NAPVCH;
 
     for(int i=0;i<NAPVCH;i++) {
         if(fabs(sig[i])>theTskip*rms[i]) {
          skip.push_back(i);
           if(theNoisyCh.find(i)!=theNoisyCh.end())  theNoisyCh[i]++;
           else theNoisyCh[i]=1; }

        if(asig[i]>thr) {
	  if(!(find(skip.begin(),skip.end(),i)!=skip.end()))   skip.push_back(i);  
           if(theNoisyCh.find(i)!=theNoisyCh.end())  theNoisyCh[i]++;
           else theNoisyCh[i]=1; }      
       }
     return skip;
}
//
void  Apv_Analysis::updatebadCh()
{
  float rmsofrms;
  float tnoisy=thePskip; // if more than in  30% events maksed -> noisy channel
  //  theMeanRms=mean(theRms,rmsofrms);  //also possible
   theMeanRms=median(theRms,rmsofrms);
 float thr=theTbad*rmsofrms;
 for(int i=0;i<NAPVCH;i++) {
   int flag=0;
   if(thePedestal[i]>thePedmax || thePedestal[i]<thePedmin )  flag|=1;
   if(theRms[i]>theRmsmax || theRms[i]<theRmsmin) flag|=2;
   if((theRms[i]-theMeanRms)>thr ) flag|=4; // noisy
   if((theMeanRms-theRms[i])>thr) flag|=8;  // quiet
   theChStatus[i]=flag;
  }

    for (DigitalMapType::const_iterator j = theNoisyCh.begin();
           j != theNoisyCh.end(); j++) {
      int is=j->first;
      int nen=j->second;
      if(nen>theNRms*tnoisy)  theChStatus[is]|=4; // noisy
    }
}
//
float  Apv_Analysis::mean(const Fvec& data, float& Rms)
{
 Rms=0.;
int n=data.size();
float  summ=data.sum()/n;
 Fvec data2(n); 
 data2=data*data;
 float summ2=data2.sum();
 if(n>0) Rms=sqrt(fabs(summ2/n-summ*summ));
 float mean=summ/n;
 return mean;
}
//
float  Apv_Analysis::median(const Fvec& data, float& Rms) 
 {
  FloatMapType dist;
  Rms=0.;
  int nused=0;
  float median=0.;
  int size=data.size();
  for(int i=0;i<size;i++){
      float  x=data[i];
      nused++;
     if (dist.find(x) != dist.end()) 
            dist[x]++;
       else dist[x] = 1;      
    }
     if (nused > 0) {
     float rms=0.;
     median=findMedian(dist,nused,rms);
     Rms=rms;
     }
     return median;
  }
//
int  Apv_Analysis::defineDead()
{
 int n=0;
  for(int ich=0;ich<NAPVCH;ich++)
   if(theChStatus[ich]>0)
     { theSignal[ich]=0; n++;}
  return n;
}
//
map<int,float,less<int> >   Apv_Analysis::findHits(){
  SignalMapType hits;
  Fvec sn(theSignal.size());
  sn=double(thePolarity)*theSignal/theRms;
    for(int ich=0;ich<NAPVCH;ich++)
      if(sn[ich]>theTstrip) hits[ich]=sn[ich];
  return hits;
}
//
int Apv_Analysis::FindCluster()
{
  SignalMapType hits=findHits();
  if(hits.size()==0) return 0;

  theClusNoise.clear();
  theClusQ.clear();
  theClusPos.clear(); 
  theClusSeedStrip.clear();
  theSeedSNR.clear();
  theClusSize.clear(); 

  int ncl=0;
  int sizeclu=0;
  int seedstr=0;
  float seedsnr=0.;
  float qclu=0.;
  float noiseclu2=0.;
  float xclu=0;
  SignalMapType::const_iterator icur, inext;

  for(icur=hits.begin();icur!=hits.end();icur++)
    {
      int is=icur->first;
     inext=hits.find(is+1);
      if(icur->second>seedsnr) {seedsnr=icur->second; seedstr=is;}
      qclu+=theSignal[is];
      noiseclu2+=theRms2[is];
      xclu+=is*theSignal[is];
      sizeclu++;
      if(inext==hits.end()) {
                bool accept=false;
                switch (theModclus) {
                case 1: //TDR
                  accept=(seedsnr>theTseed&&qclu/(sqrt(noiseclu2/sizeclu))>theTclus);break;
                case 2: //FED2
                  accept=(seedsnr>theTseed||sizeclu>1);break;
                case 3: //FED1
                  accept=true;break;}
                if(accept){
theClusPos.push_back(xclu/qclu);
theClusQ.push_back(qclu);
theClusNoise.push_back(sqrt(noiseclu2/sizeclu));
theClusSeedStrip.push_back(seedstr);
theSeedSNR.push_back(seedsnr);
theClusSize.push_back(sizeclu);
 ncl++;}
sizeclu=0;
seedstr=0;
seedsnr=0.;
qclu=0.;
noiseclu2=0.;
xclu=0.; 
    }
 }
  return ncl;
}
//
void Apv_Analysis::AnalyzePed(int ev)
{
  // example algorithm
  int updbad=theUpdate;
if(ev<0) {
  DT_DEBUG_LTAPPL("the settings");
theNpedevent=500;
theNrawevent=100;
theTskip=3.;
theNStrips=32;
theNcmniter=2;
theTbad=3.;
theTstrip=3.;
theTclus=5.;
theTseed=5.;
theModclus=2;
theNevents=0;
}else if(ev==0){
theNevents=0;
}else{ 
     
// cout<<" Analyze "<<theNevents<<" "<<theModped<<" "<<theTbad<<" "<<theTskip<<endl;

    theNevents++;
     if(theNevents<theNpedevent)        
       {
     UpdatePedestal(); 
//     if(theNevents==theNrawevent) theRawRms=sqrt(theRawRms2);
    if(theNevents>theNrawevent) {
    UpdateNoise();
    if(!((theNevents+updbad)%updbad)) { updatebadCh();theNBad=defineDead();} 
        }
       }
      if(theNevents==theNpedevent-1)  {theNBad=defineDead();  }
  }
}
//
void Apv_Analysis::AnalyzeCalProf(int ev)
{
if(ev<=0) {
  DT_DEBUG_LTAPPL("Apv_Analysis: settings ");
theNpedevent=500;
theNrawevent=100;
theTskip=3.;
theNStrips=32;
theNcmniter=2;
theTbad=3.;
theModped=2;
theModcmn=2;
}else{        
    theNevents++;
    if(theNevents<theNpedevent)               
     UpdatePedestal();
  }
}

//
void Apv_Analysis::UpdatePedestal()
{
    if(theModped==1) updateMeanPed();
    if(theModped>1) updateMedianPed();
    theRms=sqrt(theRawRms2);
}
//
void Apv_Analysis::UpdateNoise()
{ 
     if(theModped==1) {SubtractCM();updateMeanNoise();}
     if(theModped>1) {SubtractCM();updateMedianNoise();}
   theRms=sqrt(theRms2);
}
//
void Apv_Analysis::UpdateDead()
{
 updatebadCh();
}
//

float Apv_Analysis::getMeanRawRms()
{ Fvec rawrms(theRawRms2.size());
  rawrms=sqrt(theRawRms2);
  float rmsofrms; 
  return median(rawrms,rmsofrms);
}
