#include "CalProfRun.h"
#include "Module.h"
#include "ModuleDescriptor.h"
#include "HybridI2C.h"
#include "RecModule.h"
#include "RecAnalyzer.h"
#include "RecModHeader.h"
#include "RecModTest.h"
#include "Spill_Analysis.h"
#include "setparser.h"
#include "daqerrormsg.h"

using namespace std;
const unsigned char DriveStepPattern[9]=
  // drivestep=  0    1    2  3     4     5     6    7   8
               {0x0,0x1,0x11,0x00,0x55,0x00,0x00,0x00,0xff};

CalProfRun::CalProfRun(Module* m) {
  thefeds=m->feds();
  theapvanals=m->apvanalysis();
  thei2c=m->hybridi2c();
  therec=m->getrecmod();
  latchanged=false;
  Napvs=thefeds.size();
  DT_DEBUG_LTAPPL(" created CalProfRun ");
}
/**/
CalProfRun::~CalProfRun() {
}
/**/
void CalProfRun::InitTest(SetParser* s) {
  prevmode= thei2c->GetMode();
  binv= thei2c->CheckInvMode(prevmode);
  calsets=0;
  if (thei2c->CheckDecMode(prevmode))
    {   calsets=s->getCalProfRunDescriptor("deconv");}
  else
    {   calsets=s->getCalProfRunDescriptor("peak");} 
  Ncalev=calsets->nevpoint;
  Tbad=calsets->tbad;
  cout<<"==== Calprofruninit: "<<Ncalev<<" "<<Tbad<<endl;
  latencystart=calsets->latstart;
  latencyend=calsets->latend;
  SkipPedestal=  calsets->pedestalskip;

  thei2c->SetCalMode(prevmode);
 int cm=int( thei2c->GetMode());
   therec->getmodtest()->setapvmcalprof(cm);

  pedrun=false;
  sel=7;
  drv=0; 
  switch(calsets->timeskip){
    case 2     :fTimeStep = 2 ; break;  
    case 3:
    case 4   : fTimeStep =4 ; break;
    default : fTimeStep=1; break;
  }
  fDriveStep=1; 
  DT_DEBUG_LTAPPL(" Init latstart "<<latencystart<<" latend "<<latencyend 
		  << "Timeskip: "<< fTimeStep);
  // development not finished. to pass drivepatern HybridI2c has to be changed
  // because now it only can handle 1 group to drive 
  drivepattern=DriveStepPattern[fDriveStep];
  lat=latencystart;
  thei2c->ChangeCal(lat,sel,drv); 
  //thei2c->ChangeCal(lat,sel,drivepattern); 
  // latchanged has to set to true to force a pedestal calculation 
  latchanged=true;
  Ntot=0;
  Nev=0;
  done=false;
  therec->getmodtest()->ResetCalProf();
  memset(fCorr,0,Napvs*NAPVCH*1000*sizeof(double));
}
/*
 */

bool CalProfRun::Update(int nevinspill) {
  DT_DEBUG_PROC("Calprofrun: update nevinspill "<<nevinspill);
  if(done) {thei2c->SetMode(prevmode); return true;}
  if(latchanged) {UpdatePedestal(nevinspill); return true;}

  loadSpill(thefeds,theapvanals, nevinspill);
  int nbadev=0;
  for(int iev =0; iev < nevinspill; iev++)  {
    Nev++;   Ntot++;
    DT_DEBUG_PROC(" CalProf: Nev "<<Nev<<" Ntot "<<Ntot);
    if(Nev>=Ncalev || Nev>999) { DT_DEBUG_PROC(" too many events"); continue;}
    for (int ich=0;ich<Napvs;ich++) {        
      theapvanals[ich]->loadEvent(iev);          
      if (theapvanals[ich]->FindFrame()) {                 
	theapvanals[ich]->SubtractCM();
	valarray<double> cmn=*(theapvanals[ich]->getCmn());         
	if ( fabs(cmn.sum())<30.*cmn.size()) 	  {                    
	  valarray<double> sig=*(theapvanals[ich]->getSignal());     
	  for (int is=drv;is<NAPVCH;)  {
	    if(!binv)   fCorr[ich*NAPVCH+is][Nev] = sig[is];
	    else    fCorr[ich*NAPVCH+is][Nev] = -sig[is];
	    is+=8/fDriveStep;;
	  }
      }  else DT_WARN("CalProfRun: badcmn ");
    }  else {DT_WARN("CalProfRun:Update Cannot find frame ");nbadev++;}
    }
  }     
  if(nbadev>=nevinspill) {DT_WARN("too many bad events ");return false;}
 
  if(Nev >= Ncalev) {
    DT_DEBUG_LTAPPL(" booking "<<Nev);
          // Update histo
    int ti = (latencystart-lat)*8+(7-sel);
     double sorted[1000];
      int ifi=TMath::Nint(Nev*0.2);
      int ila=TMath::Nint(Nev*0.8);
      for (int i=drv;i<Napvs*NAPVCH;) 
	{
        memcpy(sorted,&fCorr[i][0],1000*sizeof(double));
        sort(&sorted[0],&sorted[Nev-1]);             
        float mean =0;
        mean= accumulate(&sorted[ifi],&sorted[ila],0.)/(ila-ifi+1); //truncated  
        TH1F* hp = therec->getmodtest()->hcalprof(i);
       
        if (hp!=0){
	  // fill in the skipped times 
	  for( int lc=ti; lc <ti+fTimeStep;lc++)
	  { hp->SetBinContent(lc,mean);
	  }
	}
        i+=8/fDriveStep;;
      }
    DT_DEBUG_LTAPPL(" booking done ");
   memset(fCorr,0,Napvs*NAPVCH*1000*sizeof(double));
    Nev=0;
    drv+=fDriveStep;
    //drivepattern=drivepattern<<1;
    if (drv == 8 ) {
      drv =0;
      //drivepattern= DriveStepPattern[fDriveStep];
      sel-=fTimeStep;  
      if(sel <= -1)  {
	sel=7;
	lat--; 
	latchanged=!SkipPedestal;
	// if latchanged if false the ped run is not activated
      }
    }
    //thei2c->ChangeCal(lat,sel,(int) drivepattern);
    thei2c->ChangeCal(lat,sel,drv); 
    DT_DEBUG_LTAPPL("CalProfRun: Update  change lat"<<lat<<" drv "<<drv<<" sel "<<sel);
    if(lat<=latencyend){
      DT_DEBUG_LTAPPL("  CalProf is done");
      thei2c->SetMode(prevmode); done=true;return true;
    }  
  }
  return true;
}
/*
 */
void CalProfRun::FillRecord() {
  DT_DEBUG_LTAPPL(" CalProfRun: fillrecord ");
  thei2c->SetMode(prevmode);
// all this is in RecAnalyzer now
  int nch=Napvs*NAPVCH;
  TH1F* hcala=therec->getmodtest()->THcalPApeak();
  TH1F* hcaltr=therec->getmodtest()->THcalPTrise();
  TH1F* hcaltfall=therec->getmodtest()->THcalPTfall();
  for(int ch=0;ch<Napvs;ch++) {
    for(int is=0;is<NAPVCH;is++) {
      TH1F* hp=therec->getmodtest()->hcalprof(ch*NAPVCH+is);
      
// can do fit here
      float xmax=hp->GetMaximum();
      float xmin=hp->GetMinimum();
      float xtmx=hp->GetMaximumBin();
      float xtmn=hp->GetMinimumBin();
      float dtrise=xtmx;
      hcala->SetBinContent(ch*NAPVCH+is+1,xmax);
      hcaltr->SetBinContent(ch*NAPVCH+is+1,dtrise);
    }
  }
//

// therec->AnalyzeCalProf(thei2c->CheckMode(prevmode),calsets); 
therec->getrecanalyzer()->AnalyzeCalProf(thei2c->CheckMode(prevmode),calsets);
}
/**/
bool CalProfRun::UpdatePedestal(int nevinspill) {
  DT_DEBUG_LTAPPL(" CalprofRun: updatepedestal ");
  if(!pedrun) {
    curmode=thei2c->GetMode();
    thei2c->UnsetCalMode(curmode);// i2c change
    pedrun=true; 
    return true; //ask for fresh spill, i2c is changed 
  }
  loadSpill(thefeds, theapvanals, nevinspill);
  for(int iev =0; iev < nevinspill; iev++) {
    Nev++; Ntot++;
    DT_DEBUG_PROC("CalProfRun:Ped Nev "<<Nev<<"of  req "<<Ncalev<< " tot "<<Ntot);
    if(Nev>Ncalev) {
      Nev=0;pedrun=false; latchanged=false;
      curmode=thei2c->GetMode(); 
      thei2c->SetCalMode(curmode); 
      DT_DEBUG_LTAPPL("ped update done with "<<
		      Nev <<" event of "<< Ncalev <<"requested");
      return false;
    }
    for (int ich=0;ich<Napvs;ich++) { 
      theapvanals[ich]->loadEvent(iev);      
      if (theapvanals[ich]->FindFrame()) {
	theapvanals[ich]->AnalyzePed(1); 
      } else DT_WARN("PedestalRun::Update  Cannot find frame ");
    }   
  }
  DT_DEBUG_PROC(" CalProfRun: done updatepedetsal ");
  return true;
}
