#include "CalRun.h"
#include "Module.h"
#include "daqerrormsg.h"
#include "TestBase.h"
#include  "HybridI2C.h"
#include  "ModuleDescriptor.h"
#include "RecModule.h"
#include "RecModHeader.h"
#include "RecAnalyzer.h"
#include "RecModTest.h"
#include "Spill_Analysis.h"
#include "setparser.h"

using namespace std;

CalRun::CalRun(Module* m)  {
 thefeds=m->feds();
 theapvanals=m->apvanalysis();
 thei2c=m->hybridi2c();
 therec=m->getrecmod();
 Napvs=thefeds.size();
 DT_DEBUG_LTAPPL(" created CalRun ");
}
/**/
CalRun::~CalRun() {
}
/**/
void CalRun::InitTest(SetParser* s) {

   calsets= 0 ;
  prevmode= thei2c->GetMode();
  if (thei2c->CheckDecMode(prevmode))
    {   calsets=s->getCalRunDescriptor("deconv");}
  else
    {   calsets=s->getCalRunDescriptor("peak");}
  Ncalev=calsets->nevpoint;
  Tbad=calsets->tbad;
  thei2c->SetCalAmp(calsets->calamp);
  thei2c->SetCalMode(prevmode);
  int cm=int( thei2c->GetMode());
   therec->getmodtest()->setapvmcal(cm);
  thei2c->SetLatency(calsets->lat);
  drv=0; 
  thei2c->SetDrv(drv);
  Ntot=0;
  Nev=0;
  done=false;
  therec->getmodtest()->ResetCal();
  memset(fCorr,0,Napvs*NAPVCH*1000*sizeof(double));
  DT_DEBUG_LTAPPL("init calrun in mode"<<thei2c->GetMode());
}
/*
 */

bool CalRun::Update(int nevinspill) {
  if(done) {thei2c->SetMode(prevmode); return true;}
  loadSpill(thefeds,theapvanals, nevinspill);
  int nbadev=0;
  for(int iev =0; iev < nevinspill; iev++)  {
    Nev++;   Ntot++;
    DT_DEBUG_LTAPPL(" Nev "<<Nev<<" Ntot "<<Ntot);
    if(Nev>=Ncalev || Nev>999) { DT_DEBUG_LTAPPL(" too many events"); continue;}
    for (int ich=0;ich<Napvs;ich++) {        
      theapvanals[ich]->loadEvent(iev);          
      if (theapvanals[ich]->FindFrame()) {                 
	theapvanals[ich]->SubtractCM();
	valarray<double> cmn=*(theapvanals[ich]->getCmn());         
	if ( fabs(cmn.sum())<30.*cmn.size()) {                    
	  valarray<double> sig=*(theapvanals[ich]->getSignal());     
	  for (int is=drv;is<NAPVCH;)  {
	    fCorr[ich*NAPVCH+is][Nev] = sig[is];
	    is+=8;}
	}  else  DT_WARN(" badcmn ");
      }  else { DT_WARN("Cannot find frame ");nbadev++;}
    }
  }
  if(nbadev>=nevinspill-1) return false;
     // 
  if(Nev>= Ncalev) { 
        DT_DEBUG_LTAPPL(" booking  at Nev "<<Nev);
          // Update histo
          double sorted[1000];
          int ifi=TMath::Nint(Nev*0.2);
          int ila=TMath::Nint(Nev*0.8);
          for (int i=drv;i<Napvs*NAPVCH;) {
              memcpy(sorted,&fCorr[i][0],1000*sizeof(double));
              sort(&sorted[0],&sorted[Nev-1]);             
              float mean =0;
              mean= accumulate(&sorted[ifi],&sorted[ila],0.)/(ila-ifi+1); //truncated           
 
              TH1F* hp =therec->getmodtest()->THcalA();
              if (hp!=0)
                hp->SetBinContent(i+1,mean);
                i+=8;
            }
	  memset(fCorr,0,Napvs*NAPVCH*1000*sizeof(double));
          Nev=0;
          drv++;
           DT_DEBUG_LTAPPL(" change drv "<<drv); 
	   if (drv == 8 ) {
	     DT_DEBUG_LTAPPL(" CalRun is done");
	    thei2c->SetMode(prevmode); done=true; return true;
	   }
	   thei2c->SetDrv(drv); 
	   DT_DEBUG_LTAPPL(" CalRun: Update drv changed to  "<<drv); 
  }    
  return true;
}
/**/
void CalRun::FillRecord() {
  thei2c->SetMode(prevmode);
//  therec->AnalyzeCal(thei2c->CheckMode(prevmode),calsets); 
therec->getrecanalyzer()->AnalyzeCal(thei2c->CheckMode(prevmode),calsets); 
  DT_DEBUG_LTAPPL(" Filled Record "<<drv); 
}


