#include "ConnectivityRun.h"
#include "Module.h"
#include "setparser.h"
#include "ModuleDescriptor.h"
#include "RecModule.h"
#include "RecAnalyzer.h"
#include "RecModTest.h"
#include "RecModHeader.h"
#include "Spill_Analysis.h"
#include "daqerrormsg.h"
using namespace std;

const short APV_MODE=0;/// mode for lowest base line 



ConnectivityRun::ConnectivityRun(Module* m)
{
  int i;
  di2c=m->hybridi2c();
  hv=m->modulehv();
  thefeds=m->feds();
  theapvanals=m->apvanalysis();
  recmod=m->getrecmod();
  apvctrl=m->getdaqctrl();
  napv=thefeds.size();
  apvaddr=new short[6];
  apvaddr[0]=32;apvaddr[1]=33;
  if( napv==4) { apvaddr[2]=37;apvaddr[3]=63;}
  else {
    apvaddr[2]=34;apvaddr[3]=35;apvaddr[4]=36;apvaddr[5]=37;
  }
  apv_com_addr=63;
  apvtestmode=15;
  nped==1000;
  consets=0;
  DT_DEBUG_LTAPPL("connectivity constructed");
}

ConnectivityRun::~ConnectivityRun()
{
  delete[] apvaddr;
  result0.clear();
}
 void ConnectivityRun::InitTest(SetParser* s) {   
   setp=s;done=false;

   consets=s->getConnectivityRunDescriptor("default");

   if (consets) {
     if (napv==4) {        
       for (int i=0;i<2;i++) {apvaddr[i]=consets->apvaddr[i];} 
       for (int i=2;i<napv;i++) {apvaddr[i]=consets->apvaddr[i+2];} 
     }
     else {  //asume 6 apv's 
       for (int i=0;i<napv;i++) apvaddr[i]=consets->apvaddr[i];  
     }
     apv_com_addr= consets->apvaddr[6];
     nped=consets->nevpoints;
     apvtestmode=consets->fMode;

   }
   nped=consets->nevpoints;
   PedRunDescriptor* ps = s->getPedRunDescriptor("default");
   float pedmin=50;
   float pedmax=500;
   for(vector<Spill_Analysis*>::iterator iapv=theapvanals.begin();
       iapv!=theapvanals.end();iapv++) {
     (*iapv)->setNoiseEstimator(consets->nevpoints,ps->npedraw,
				ps->modped,ps->tskip,ps->tbad,
				ps->rmsmin,ps->rmsmax,pedmin,pedmax,ps->pskip);
     (*iapv)->setCmnEstimator(ps->modcmn,ps->nstrcmn,ps->ncmniter);
   }  
 }


vector<double> ConnectivityRun::pedestalStatus(int nevinspill) 
{
  int iev,ich,nev;
  vector<double> ped(10);
  double avped,avrms;
//
  DT_DEBUG_LTAPPL( "PedRun/start");
  prevmode=di2c->GetMode();
  di2c->UnsetCalMode(prevmode);

  for(ich=0;ich<napv;ich++) theapvanals[ich]->Clear();
  //loadSpill(thefeds,theapvanals,nevinspill);
  apvctrl->NextSpill();
  nev=0;
  int daqcount=0;
  while (nev<=nped)
    {
      if (apvctrl->NewSpill())	{
	  daqcount=0;
	  loadSpill(thefeds,theapvanals,nevinspill);
	  DT_DEBUG_LTAPPL("nev: "<<nev<<" nped "<< nped<<" nevinspill"<<nevinspill);
	  for(iev=0;iev<nevinspill;iev++) 
	    {
	      nev++;
	      if (nev>nped) break;
	      for (ich=0;ich<napv;ich++) 	{ 
		  theapvanals[ich]->loadEvent(iev);      
		  if (theapvanals[ich]->FindFrame()) {
		    theapvanals[ich]->AnalyzePed(1);
		  }
		  else {
		    DT_DEBUG_LTAPPL("frame not found");
		  }
		}
	    }
	  apvctrl->NextSpill();
	}
      daqcount++;
      if(daqcount>10) { apvctrl->NewSpill(true);}	
      usleep(100000);
    }
//
  avped=0;avrms=0;
  for (ich=0;ich<napv;ich++)     {
      valarray<double> p=*(theapvanals[ich]->getPedestal());
      ped[ich]=p.sum()/p.size();
      avped+=ped[ich];
      valarray<double> r=*(theapvanals[ich]->getRms());
      avrms+=r.sum()/r.size();
    }
  ped[6]=avped/napv;ped[7]=avrms/napv;
//
  DT_DEBUG_LTAPPL( "pedrun/APV:pedestal : ");
  for (int i=0;i<napv;i++)  DT_DEBUG_LTAPPL(  i << ":" << ped[i] );
  DT_DEBUG_LTAPPL( "PedRun/pedestal  " << ped[6] << " noise  " << ped[7]); 
//
  return ped;
}

void ConnectivityRun::setModeAddress(short mode,short addr, short vpsp )
{
  I2cDescriptor* ci2c;
  ci2c=di2c->lasti2c();
  ci2c->fMode=mode;
  ci2c->fAddress=addr;
  if( vpsp >=0) {  ci2c->fVpsp=vpsp;}
  di2c->SetI2C(ci2c);
  //cout << "Mocde  " << ci2c->fMode << " Address " << ci2c->fAddress << endl;
}
 
bool ConnectivityRun::hvConnectionTest(int nevinspill)
{
  int j;
  double d, curreduction,curr[2],mean[2];
  bool result;
  float HV_L[2]={0,15};
  float noise_rel = 0.4, cur_rel=10;
  if(consets){
    HV_L[0]=consets->lowtestvoltage;
    HV_L[1]=consets->hightestvoltage;
    noise_rel =consets->noisedifference;
    cur_rel=consets->currentdifference;
  }
  if (consets) { 
    HV_L[0]=consets->lowtestvoltage;  HV_L[1]=consets->hightestvoltage; 
    noise_rel = consets->noisedifference;
    cur_rel=  consets->currentdifference;
  }
  DT_DEBUG_LTAPPL( "STARTING HV CONNECTIONS TEST" ); 
  for (j=0;j<=1;j++)    
    {
      hv->ChangeHV(HV_L[j]);
      sleep(1);
      curr[j]=hv->getItest(HV_L[j]);
       DT_DEBUG_LTAPPL("Setting HV level to " << HV_L[j]  << 
	" V . Current " << curr[j] << " mA");
      sleep(1);
      vector<double> v=pedestalStatus(nevinspill);
      mean[j]=v[7];
    }
   DT_DEBUG_LTAPPL("HV CONNECTIONS TEST RESULTS " );
  d=(mean[0]-mean[1])/mean[0];
  if( curr[0] <= 0) curr[0]=1e-10;
  
  curreduction= curr[1]/ curr[0];
  if (d>=noise_rel)
    { 
      result=true;
      DT_DEBUG_LTAPPL("HV connectivity o.k. reduction ="<< d*100 <<"%" ); 
    }
  else
    { 
      result=false; 
      DT_WARN("HV connectivity failed reduction ="<< d*100 <<"% " 
	      <<  noise_rel*100<< "% \n\t noise@ "<<HV_L[0]<<"V: "<< mean[0]
	      <<" noise@" << HV_L[1]<<"V: " << mean[1] <<"V");
    }
  if (curreduction >= cur_rel)
    { 
      DT_DEBUG_LTAPPL( "HV current  O.K.  current@" <<HV_L[0] <<"V= "
		       <<curr[0]<<"uA  current@"
		       <<HV_L[1]<<"V= "<<curr[1]<<"uA");
    }
  else
    { 
      result=false; 
      DT_WARN( "HV current  FAILED  (current@" <<HV_L[1] <<"V= "<<curr[1]
	       <<"uA) /( current@"
	       <<HV_L[0]<<"V= "<<curr[0]<<"uA)= "
	       <<curreduction <<" requested " << cur_rel  );
    }


   DT_DEBUG_LTAPPL("HV CONNECTION TEST IS OVER");
  return result;
} 

vector<bool> ConnectivityRun::apvConnectionsTest(int nevinspill)
{
  int j;
  double s,ped0;
  vector<bool> result;
  DT_DEBUG_LTAPPL("STARTING APV CONNECTIONS TEST");
  result.resize(7);
  short vfp=22;float ped_rel=40;
  if(consets) { 
    vfp=consets->fVfp;ped_rel=consets->displacement;    
  }
  for(int lc=0;lc<7;lc++){ result[lc]=true;}
  setModeAddress(apvtestmode, apv_com_addr,vfp);
  vector<double> v=pedestalStatus(nevinspill);
  ped0=v[6];
  for (j=0;j<napv;j++)
    {
      setModeAddress(APV_MODE,apvaddr[j],-1);
      v=pedestalStatus(nevinspill);
      s=abs(ped0-v[j]);
      if (s>=ped_rel)
	{ 
	  result[j]=true;
	  DT_DEBUG_LTAPPL("O.K  for APV "<<j <<" Displacement" << s <<
			  " required "<<ped_rel );
       	}
      else
	{ 
	  result[j]=false;
	  DT_WARN("APV conectivity test failded for APV "<<j <<" Displacement "
		  << s<<" required "<<ped_rel );     
	}
      setModeAddress(apvtestmode,apvaddr[j],-1);
    }
  DT_DEBUG_LTAPPL( "APV CONNECTIONS TEST IS OVER" );
  return result;
}

bool ConnectivityRun::Update(int nevinspill)
{
  vector<double> v=pedestalStatus(nevinspill);
  short savm,sava,savevpsp;
  I2cDescriptor* ci2c=di2c->lasti2c();
  savm=ci2c->fMode;
  sava=ci2c->fAddress;
  savevpsp=ci2c->fVpsp;
  setModeAddress(apvtestmode,apv_com_addr,-1);
  bool res=hvConnectionTest(nevinspill);
  result0=apvConnectionsTest(nevinspill);
  result0[6]=res;
//result0 contains results for 4 or 6 apvs in its first 0-5 elements
//and HV test result in its 6 element
  res=true;
  for(int lc=0;lc < napv;lc++){
    res &=result0[lc];
  }
  res&=result0[6];
  if(!res) {theTestResult=-1;}
  else {theTestResult=1;}
  done=true;
  setModeAddress(savm,sava,savevpsp);
  return true;
}


void ConnectivityRun::FillRecord()
{
  vector<int> v;
  v.resize(result0.size());
  for (int i=0;i<result0.size();i++) 
    if (result0[i]) v[i]=1; else v[i]=0;
recmod->getmodtest()->setconnectivity(v);

recmod->getrecanalyzer()->AnalyzeConnectivity("",consets);
}

//ConnectivityRun

