#include "FedCtrl.h" 
#include "fedprocessctrlinterface.h"
#include "daqerrormsg.h"
#include "ltparser.h"
#include <unistd.h>
using namespace std;

FedCtrl::FedCtrl( LtParser* ltparser){
  const unsigned int maxretry=5;
  const unsigned int loopdelay=100000;
  ffeds.clearpointermap( );
  fInitialized=false;
  // if go to more FED use only FED for which the channels are used 
  int nrfeds= ltparser->getNumberFedProcessCtrl();
  //cout<<" found "<< nrfeds  <<"feds"<<endl;
  for( int nr=0;nr<nrfeds;nr++){
    // thefedctrl is instantiated by the getFedProcessCtrl call
    // so the class is responsible tpo delete it 
   FedProcessCtrlInterface*  thefedctrl =ltparser->getFedProcessCtrl(nr);
    // returns should be replaced by throws !! 
   if(!thefedctrl) { DT_WARN("empty fed ctrl for nr "<<nr); return;}
   // cout<<"start to init fed "<< thefedctrl-> getFedId()<<endl;
   FedCtrlInfo* fci=new FedCtrlInfo(thefedctrl);
    ffeds.replaceP(thefedctrl-> getFedId(),fci);
    unsigned int lc=0;
    while ( ! thefedctrl->clearFed()  && lc <  maxretry) { 
      lc++;    usleep( loopdelay);
    }
    if( lc >= maxretry)  { DT_WARN("fail clear fed");  return;}
    lc=0;
    while (  thefedctrl->stopFedDaqLoop()  && lc <maxretry) {
      lc++;    usleep(loopdelay);
    }
    if( lc >= maxretry)  { DT_WARN("fail stopDAQ "); }
  }
  fInitialized=true;
}

FedCtrl::~FedCtrl(){
  ffeds.clearpointermap( );
}

FedCtrl::FedCtrlInfo::~FedCtrlInfo(){
  delete fFedpc;
}


void FedCtrl::setSpillSize(int spillsize){
  for( fi=ffeds.begin(); fi!=ffeds.end();  fi++){
    fi->second->fFedpc->setSpillSize(spillsize);
  }
}

bool FedCtrl::clearFed(int fed ){
  bool res=true;
  if( fed <0) {
    for( fi=ffeds.begin(); fi!=ffeds.end();  fi++){
    res&=fi->second->fFedpc->clearFed();
    }
  } 
  else{
   FedProcessCtrlInterface* thefedctrl=ffeds.getP(fed)->fFedpc;res=false;
   if( thefedctrl) {res= thefedctrl->clearFed(); }
  }
  return res; 
}

bool FedCtrl::stopFedDaqLoop(int fed){
 bool res=true;
 if( fed <0) {
    for( fi=ffeds.begin(); fi!=ffeds.end();  fi++){
    res&=fi->second->fFedpc->stopFedDaqLoop();
    }
  } 
  else{
   FedProcessCtrlInterface* thefedctrl=ffeds.getP(fed)->fFedpc;res=false;
   if( thefedctrl) {res= thefedctrl->stopFedDaqLoop(); }
  }
  return res; 
}

void FedCtrl::NextSpill(int fed){
  if( fed <0) {
    for( fi=ffeds.begin(); fi!=ffeds.end();  fi++){
      fi->second->fFedpc->NextSpill();
    }
  } 
  else{
   FedProcessCtrlInterface* thefedctrl=ffeds.getP(fed)->fFedpc;
   if( thefedctrl) { thefedctrl->NextSpill(); }
  }
}

int FedCtrl::getNumberOfEvents(int fed){
  int  res=-1;
  if( fed <0) {
    for( fi=ffeds.begin(); fi!=ffeds.end();  fi++){
      int nrevents = fi->second->fFedpc->getNumberOfEvents();
      if( fi==ffeds.begin()){res=nrevents;}
      else { if(nrevents < res) { res=nrevents;} }
    }
  } 
  else{
    FedProcessCtrlInterface* thefedctrl=ffeds.getP(fed)->fFedpc;res=false;
    if( thefedctrl) {res= thefedctrl->getNumberOfEvents(); }
  }
  return res; 
}
  

bool FedCtrl::NewSpill(int fed,int skip){
  bool res=true;
  if( fed <0) {
    for( fi=ffeds.begin(); fi!=ffeds.end();  fi++){
      res&=fi->second->fFedpc->NewSpill();
    }
  } 
  else{
    FedProcessCtrlInterface* thefedctrl=ffeds.getP(fed)->fFedpc;res=false;
    if( thefedctrl) {res= thefedctrl->NewSpill(); }
  }
  return res; 
}



bool  FedCtrl::AdjustThresholds(int& low , int& high,int fed){ 
 bool res=true;
  if( fed <0) {
    int lowall=low;int highall=high; 
    for( fi=ffeds.begin(); fi!=ffeds.end();  fi++){      
      lowall=low; highall=high;
      res&=fi->second->fFedpc->AdjustThresholds(lowall ,highall);
      fi->second->thresholdlow=lowall;fi->second->thresholdhigh=highall;
    }
    low=lowall;high=highall;
  } 
  else{
    FedProcessCtrlInterface* thefedctrl=ffeds.getP(fed)->fFedpc;res=false;
    if( thefedctrl) {res= thefedctrl->AdjustThresholds(low , high);
    ffeds.getP(fed)->thresholdlow=low;ffeds.getP(fed)->thresholdhigh=high;
    }
  }
  return res; 
}

bool  FedCtrl::AdjustThresholds(int fed){ 
  bool res=true;
  if( fed <0) {
    for( fi=ffeds.begin(); fi!=ffeds.end();  fi++){
      res&=fi->second->fFedpc->AdjustThresholds();
    }
  } 
  else{
    FedProcessCtrlInterface* thefedctrl=ffeds.getP(fed)->fFedpc;res=false;
    if( thefedctrl) {res= thefedctrl->AdjustThresholds(); }
  }
  return res; 
}

bool  FedCtrl::TuneFedSampleTime(int & delay,int fed){ 
  bool res=true;
  int locdelay=delay;
  if( fed <0) {
    for( fi=ffeds.begin(); fi!=ffeds.end();  fi++){
      res&=fi->second->fFedpc->TuneFedSampleTime(locdelay);
      fi->second->feddelay=locdelay;
    }
    delay=locdelay;
  } 
  else{
    FedProcessCtrlInterface* thefedctrl=ffeds.getP(fed)->fFedpc;res=false;
    if( thefedctrl) {
      res= thefedctrl->TuneFedSampleTime(delay); 
      ffeds.getP(fed)->feddelay=delay;
    }
  }
  return res; 
}

void  FedCtrl::setScopeMode(int mode,int fed){
  if( fed <0) {
    for( fi=ffeds.begin(); fi!=ffeds.end();  fi++){
      fi->second->fFedpc->setScopeMode(mode);
    }
  } 
  else{
   FedProcessCtrlInterface* thefedctrl=ffeds.getP(fed)->fFedpc;
   if( thefedctrl) {thefedctrl->setScopeMode(mode); }
  }  
}

FedProcessCtrlInterface* FedCtrl::getFedProcessCtrl(int fed){
  if( ffeds.getP(fed)) {
    return ffeds.getP(fed)->fFedpc;
  }
  else { return 0; }
}

void FedCtrl::getThreshold(int & low, int& high,int fednr){
  if(  (FedCtrlInfo* fci=ffeds.getP(fed))) {
    low=fci->thresholdlow;high=fci->thresholdhigh;
  }
  else { return 0; }
}

