#include "LatRun.h"
#include "Module.h"
#include "ModuleDescriptor.h"
#include "RecModule.h"
#include "RecModHeader.h"
#include "RecModTest.h"
#include "RecAnalyzer.h"
#include "Spill_Analysis.h"
#include "setparser.h"
#include "daqerrormsg.h"
using namespace std;

LatRun::LatRun(Module* m)  {
 thefeds=m->feds();
 theapvanals=m->apvanalysis();
 thei2c=m->hybridi2c();
 therec=m->getrecmod();
 latchanged=false;
 Napvs=thefeds.size();

 DT_DEBUG_LTAPPL(" created LatRun ");
}
/**/
LatRun::~LatRun() {
}
/**/
void LatRun::InitTest(SetParser* s) {
  prevmode= thei2c->GetMode();
  latsets=0;
   if (thei2c->CheckDecMode(prevmode))
    {   latsets=s->getLatRunDescriptor("deconv");}
  else
    {   latsets=s->getLatRunDescriptor("peak");}
  
  Ncalev=latsets->nevpoint;
  latencystart=latsets->latstart;
  latencyend=latsets->latend;

  
  int cm=int(prevmode);
   therec->getmodtest()->setapvmlat(cm);
  thei2c->SetLatency(latencystart);
  lat=latencystart;
  latchanged=true;
  Ntot=0;
  Nev=0;
  Nerror=0;
  done=false;
  therec->getmodtest()->ResetLat();
}
/*
 */

bool LatRun::Update(int nevinspill) {
  if(done) {thei2c->SetMode(prevmode); return true;}
  loadSpill(thefeds,theapvanals, nevinspill);
  int nbadev=0;
  for(int iev =0; iev < nevinspill; iev++)  {
    Nev++;   Ntot++;
    if(Nev>=Ncalev || Nev>999) { DT_DEBUG_LTAPPL(" too many events"); continue;}
    for (int ich=0;ich<Napvs;ich++) {        
      theapvanals[ich]->loadEvent(iev);          
      if (theapvanals[ich]->FindFrame()) { 
	
	int addr = theapvanals[ich]->FindCellAddress();
	if(addr>200) Nerror++;
	TH1F* hp =therec->getmodtest()->hpipe(ich);
	hp->Fill(addr,1.);              
	theapvanals[ich]->SubtractCM();
	valarray<double> cmn=*(theapvanals[ich]->getCmn());         
	if ( fabs(cmn.sum())<30.*cmn.size()) {                    
	  valarray<double> sig=*(theapvanals[ich]->getSignal());     
	  for (int is=0;is<NAPVCH;is++)  { 
	    TProfile* hp =therec->getmodtest()->hplatency(ich*NAPVCH+is);
	    hp->Fill(lat*1.+1,sig[is]); }
	}  else DT_WARN(" badcmn "); 

      }  else { DT_WARN("LatRun:Update Cannot find frame ");nbadev++;}
    }
  }
  if(nbadev>=nevinspill) return false;
     
  if(Nev>= Ncalev) { 
    DT_DEBUG_LTAPPL(" LatRun: booking "<<Nev);   
    latchanged=false;
    Nev=0;
    lat++;
    DT_DEBUG_LTAPPL(" LatRun: change latency  "<<lat);       
    if (lat>latencyend ) {
      DT_DEBUG_LTAPPL("LatRun is done");
      thei2c->SetMode(prevmode); done=true; return true;
    }
    latchanged=true;
    thei2c->SetLatency(lat); 
  }    
  return true;
}
/**/
void LatRun::FillRecord() {
  therec->getmodheader()->setnapverrors(Nerror/Ntot);
 thei2c->SetMode(prevmode);
 therec->getrecanalyzer()->AnalyzeLat(thei2c->CheckMode(prevmode),latsets);
}


