#include "Module.h"
#include <cmath>
#include "ModId.h"
#include "Record.h"
#include "RecModule.h"
#include "RecHeader.h"
#include "RecModSctrl.h"
#include "RecModTest.h"
#include "RecModHeader.h"
#include "ModuleHv.h"
#include "ModuleDescriptor.h"
#include "setparser.h"
#include "daqerrormsg.h"
#include "benchposconnection.h"
#include "temperaturechannellt.h"
#include "currentchannelinterface.h"
#include "currentchannellt.h" 
#include "diochannelinterface.h"
#include "dcuchannellt.h"
#include "ltparser.h"
#include "fedchannellt.h"   

using namespace std;
extern LtParser* ltparser;

#ifndef NAPV
#define NAPV  6
#endif 

#ifndef NAPVCH
#define NAPVCH  128
#endif 

Module::Module( ModId* id,  Record* r, DaqCtrl* ctrl):ModId(*id) {
  fModuleErrStatus=0;
  theBench=getBenchPosition();
  therec=r;  
  thectrl=ctrl;
  thepowerswitch = ltparser->getDioChannel(
    ltparser->getBenchPosConnections(theBench)->getPowerSwitch()
  );
  theOn=false;
 switchPowerOn(true);
 fFedChannel0=false;
 //usleep(20000);
// record
  int napvs= id->getNrApvs();
  therec->ConstructModule(theBench,id,napvs);
  DT_DEBUG_LTAPPL("the record for the module "<<id->getModId()
		  <<" bench "<< theBench);
  char str0[100];
  strcpy(str0,id->getModuleType().c_str());
  therec->getmodule(theBench)->getmodheader()->setmodtype(str0);
  therec->getmodule(theBench)->getmodheader()->setdcuid(getDcuId());
  therec->getheader()->setoperator(getOperator());
  therec->getheader()->setcenter(getTestCenter());
  therecmod=therec->getmodule(theBench);
//i2c
  thei2c=new HybridI2C(theBench,ctrl);
  int muxgroupnr=-3;
  bool isinitialized=true;
//fed
 for(int i=0;i<napvs;i++) { 
   int realapvnr;
     // wrap code for 4 and 6 APV hybird 
   if ( napvs == 4 &&  i > 1)  
     { realapvnr = i+2;  //for a 4 APV hybrids  apv 3 and 4  
     // are sitting on position 5 and 6 on the hybird 
     // so connected to VUTRI channel 3 
     }
   else { realapvnr = i;}
   FedChannelLt* fc = new FedChannelLt(realapvnr,theBench);
   if(muxgroupnr == -3) {muxgroupnr= fc->getGroupNr();}
   else {  // check all group nr are the same 
     if (fc->getGroupNr() != muxgroupnr) {muxgroupnr=-1 ;}//invalidgroupnr
   }
   isinitialized &= fc->isInitialized();  
   thefeds.push_back(fc);
   fFedChannel0 =fFedChannel0 |  (fc->getFedPtr()->getChannelId() == 0);
 }
 themuxgroupnr=muxgroupnr;
 if( themuxgroupnr < -1 || !isinitialized) {
   DT_WARN("problems with attaching FED channel for benchpos "<<theBench);
 }
 // special case if fed channel 0 is used 
 DT_DEBUG_LTAPPL(" Feds are constructed ");
//spillanalysis
 for(int i=0;i<napvs;i++)
      theapvanals.push_back(new Spill_Analysis(i));

 themodulehv=new ModuleHv(theBench, therec->getmodule(theBench));
 DT_DEBUG_LTAPPL(" ModuleHv  is constructed ");
//  lvoltage currents 
thecurrent125 =new 
 CurrentChannelLt(ltparser->getBenchPosConnections(theBench)->getLvCurrentDevice(0));
thecurrent250 =new 
 CurrentChannelLt(ltparser->getBenchPosConnections(theBench)->getLvCurrentDevice(1));
// sensor temperature
thetempsen1=new  TemperatureChannelLt(ltparser->getBenchPosConnections(theBench)->getNTCDevice(0));
thetempsen2=new  TemperatureChannelLt(ltparser->getBenchPosConnections(theBench)->getNTCDevice(1));

  thepedestalrun=0;
  thecalprofrun=0;
  thelatrun=0;
  thecalrun=0;
  thebplanerun=0;
  theledrun=0;
  thelightrun=0;
  theivrun=0;

 DT_NOTE(" hybrid is constructed at bench= "<<theBench);
}
/*
 */
Module::~Module() {
  switchPowerOn(false);
  delete thei2c; thei2c=0;
  for (int i=0;i<thefeds.size();i++) {
      if(thefeds[i] !=0) { thefeds[i]->release(); delete thefeds[i]; thefeds[i]=0;}
      DT_DEBUG_LTAPPL("Channel FED Released ");
    }
  if(thepowerswitch) {
    delete thepowerswitch;thepowerswitch=0;}
  delete themodulehv; themodulehv=0;//sets the hv to 0
  sleep(1);
  delete thecurrent125; thecurrent125=0;
  delete thecurrent250;  thecurrent250=0;
  delete thetempsen1; thetempsen1=0;
  delete thetempsen2; thetempsen2=0;
  therec->DestructModule(theBench);
 DT_NOTE(" Hybrid"<<theBench<<" is destructed");
}
/*
 */
bool Module::Init(const QString& s) {
  DT_DEBUG_LTAPPL("Module :Init "<<s<<"  "<<theBench);
  if(s=="PedRun")
    {if(!thepedestalrun) thepedestalrun=new PedestalRun(this); thepedestalrun->InitTest(thesettings);    return true;}
   if(s=="CalProfRun")
     {if(!thecalprofrun) thecalprofrun=new CalProfRun(this);
     thecalprofrun->InitTest(thesettings);      return true;}
   if(s=="CalRun")
     {if(!thecalrun) thecalrun=new CalRun(this);
     thecalrun->InitTest(thesettings);   return true;}
   if(s=="BPlaneRun")
     {if(!thebplanerun) thebplanerun=new BPlaneRun(this);
     thebplanerun->InitTest(thesettings);  return true;}
   if(s=="LatRun")
     {if(!thelatrun) thelatrun=new LatRun(this); 
     thelatrun->InitTest(thesettings);  return true;}
   if(s=="LedRun")
     {if(!theledrun)  theledrun=new LedRun(this);
     theledrun->InitTest(thesettings);  return true;}
  if(s=="LightRun")
     {if(!thelightrun)  thelightrun=new LightRun(this);
     thelightrun->InitTest(thesettings);  return true;}
 if(s=="FbRun")
     {if(!thefbrun)  thefbrun=new FbRun(this);
     thefbrun->InitTest(thesettings);  return true;}
 if(s=="AuxChipRun")
     {if(!theauxchiprun)  theauxchiprun=new AuxChipRun(this);
     theauxchiprun->InitTest(thesettings);  return true;}
   if(s=="IVRun")
     {if(!theivrun)  theivrun=new IVRun(this); theivrun->InitTest(thesettings);return true;}
     if(s=="ConnectivityRun")
       { 
//       if(!theconnectivityrun)
 theconnectivityrun=new ConnectivityRun(this);
     theconnectivityrun->InitTest(thesettings);
     return true;}
   return false;
}

void Module::switchPowerOn(bool onstatus){
  if( onstatus) {
    thepowerswitch->setBit(0);
  }    
  else{
    thepowerswitch->setBit(1);
  }
  theOn=onstatus;
}


/*
 */
TestBase*  Module::Test(const QString& s) {
  DT_DEBUG_LTAPPL(" Module:Test "<<s<<" bench "<<theBench);
   if(s=="PedRun") return  thepedestalrun;
   if(s=="CalProfRun") return  thecalprofrun;
   if(s=="CalRun") return  thecalrun;
   if(s=="BPlaneRun") return thebplanerun;
   if(s=="LatRun") return  thelatrun;
   if(s=="LedRun") return  theledrun;
   if(s=="LightRun") return  thelightrun;
   if(s=="FbRun") return  thefbrun;
   if(s=="AuxChipRun") return  theauxchiprun;
   if(s=="IVRun") return  theivrun;
   if(s=="ConnectivityRun") return  theconnectivityrun;
   return theivrun;
}

/*
 */
bool  Module::Results(const QString& s) {
  DT_DEBUG_LTAPPL(" Module: Results "<<s);
  if(s=="PedRun")
    {thepedestalrun->FillRecord();delete thepedestalrun; thepedestalrun=0;return true; }   
  if(s=="CalProfRun")
    {thecalprofrun->FillRecord();delete thecalprofrun; thecalprofrun=0;return true;}
  if(s=="CalRun")
    {thecalrun->FillRecord(); delete thecalrun; thecalrun=0; return true;}
  if(s=="BPlaneRun")
    {thebplanerun->FillRecord();delete thebplanerun; thebplanerun=0;return true;}
  if(s=="LatRun")
    {thelatrun->FillRecord();delete thelatrun; thelatrun=0; return true;}
  if(s=="LedRun")
    { theledrun->FillRecord();delete theledrun; theledrun=0;return true;}
  if(s=="LightRun")
  { thelightrun->FillRecord();delete thelightrun; thelightrun=0;return true;}
 if(s=="FbRun")
  { thefbrun->FillRecord();delete thefbrun; thefbrun=0;return true;}
 if(s=="AuxChipRun")
  { theauxchiprun->FillRecord();delete theauxchiprun; theauxchiprun=0;return true;}
  if(s=="IVRun")
    { theivrun->FillRecord();delete theivrun; theivrun=0;return true;}
  if(s=="ConnectivityRun")
    { theconnectivityrun->FillRecord();delete theconnectivityrun; theconnectivityrun=0;return true;}
  return false;
}
/*
 */
void Module::UpdateXt() {
  QDateTime dc = QDateTime::currentDateTime();
  float ftime=float(stime.secsTo(dc));
  DT_DEBUG_LTAPPL("time "<<ftime); 
  thei2c->dcui2c()->update();
  DT_DEBUG_LTAPPL(" DCU  update done"); 
  if( thei2c->getNrI2cDcuErr() < 10) {
    fDcuId= thei2c->dcui2c()->getDcuHardId();
    therecmod->getmodsctrl()->settemphybdcu1(thei2c->dcui2c()->getDcuTempInt());
    therecmod->getmodsctrl()->settemphybdcu2(thei2c->dcui2c()->getDcuTempHyb());
    therecmod->getmodsctrl()->settemphybdcu3(thei2c->dcui2c()->getDcuTempSi());
    therecmod->getmodsctrl()->setvhyb125dcu(thei2c->dcui2c()->getDcuV125());
    therecmod->getmodsctrl()->setvhyb25dcu(thei2c->dcui2c()->getDcuV250());
    therecmod->getmodsctrl()->setileakdcu(thei2c->dcui2c()->getDcuBiasCurrent());
    therecmod->getmodsctrl()->setihybi125(thecurrent125->getCurrent());
 }
  therecmod->getmodsctrl()->settempsenntsc1(thetempsen1->getTemperature()); 
  therecmod->getmodsctrl()->settempsenntsc2(thetempsen2->getTemperature());
  therecmod->getmodsctrl()->setihybi25(thecurrent250->getCurrent());

  float c=themodulehv->getCurrent();
  DT_DEBUG_LTAPPL(" current "<<c);
  therecmod->getmodsctrl()->setitest(c);
  float v=themodulehv->getVbias();
  DT_DEBUG_LTAPPL(" voltage "<<v);
  therecmod->getmodsctrl()->sethvbias(v);
  float dv=1.; //volts
  if(fabs(v- thesettings->getSensorDescriptor("default")->vtest1)<dv){
    therecmod->getmodtest()->setitest1(themodulehv->getCurrent());
  }
  if(fabs(v- thesettings->getSensorDescriptor("default")->vtest2)<dv){
    therecmod->getmodtest()->setitest2(themodulehv->getCurrent());
  }
  DT_DEBUG_LTAPPL(" Module: add meas "<<ftime);
  therecmod->AddMeast(ftime); 
}
/*
 * define run settings and set default apvanalysis algos
 */
void Module::Settings(SetParser* s)
{
  DT_DEBUG_LTAPPL(" Module: Settings ");
  thesettings=s;
 themodulehv->Settings(thesettings);
//default
  PedRunDescriptor*  ps=thesettings->getPedRunDescriptor("default");  
//  float pedmin=50.;
  // float pedmax=500.;
   vector<Spill_Analysis*>::iterator iapv;
 for(iapv=theapvanals.begin();iapv!=theapvanals.end();iapv++) {
   (*iapv)->setNoiseEstimator(ps->nped,ps->npedraw,
                             ps->modped,ps->tskip,ps->tbad,
                              ps->rmsmin,ps->rmsmax,ps->pedmin,ps->pedmax,ps->pskip);
   (*iapv)->setCmnEstimator(ps->modcmn,ps->nstrcmn,ps->ncmniter);
   }
}


bool  Module::getModuleErrStatus(){
  bool errstatus=false;
  if ( fModuleErrStatus > 10)  errstatus=true;
  if( thei2c->getNrI2cApvErr() > 10) errstatus=true;
  if( thei2c->getNrI2cMuxErr() > 10) errstatus=true;
  if( thei2c->getNrI2cDcuErr() > 10) errstatus=true;
  if( thei2c->getNrI2cPllErr() > 10) errstatus=true;
  return errstatus;
}
