#include "ModuleHv.h"

#include "RecModule.h"
#include "RecModSctrl.h"
#include "hvchannellt.h"
#include "currentchannellt.h"
#include "temperaturechannellt.h"
#include "benchposconnection.h"
#include "ModuleDescriptor.h"
#include "setparser.h"
#include "daqerrormsg.h"
#include "ltparser.h"
#include <qstring.h>   

extern LtParser* ltparser;  

ModuleHv::ModuleHv(int bench, RecModule* r) {
  theBench=bench;
  therec=r;
  theWait=1000; // default is 1 ms wait between two hv checks
  DT_DEBUG_LTAPPL(" create hv channel "<<theBench);
  thehvch=new  HvChannelLt(ltparser->getBenchPosConnections(theBench)->getBiasDevice());
  DT_DEBUG_LTAPPL("HV  done ");

 thecurch=new  CurrentChannelLt(ltparser->getBenchPosConnections(theBench)->getHvCurrentDevice( ));
 DT_DEBUG_LTAPPL("IBIAS  done ");
// set default
 thehvch->lock() ;
 thehvch->setReadOnly(false);
 thehvch->setHvCurrentLimit(1);
 thehvch->setHvVolt(0); 
 thehvch->justDoIt();
 thehvch->unlock();
 thehvch->wait();
 thehvch->lock() ;
 thehvch->setHvRampUp(10);
 thehvch->setHvRampDown(10);
 thehvch->justDoIt();
 thehvch->unlock();
 thehvch->wait();
 thehvch->lock() ;
 thehvch->turnOn(); 
 thehvch->justDoIt();
 thehvch->unlock();
 thehvch->wait();
 theHv=thehvch->getHvVolt();
 DT_DEBUG_LTAPPL("ModuleHv: is constructed "<<theBench);  
}
/*
 */
ModuleHv::~ModuleHv() { 
 thehvch->lock() ;
 thehvch->setHvVolt(0); 
 thehvch->justDoIt();
 thehvch->unlock();
 thehvch->wait();
 thehvch->lock() ;
 thehvch->turnOff(); 
 thehvch->justDoIt(); 
 thehvch->unlock(); 
 thehvch->wait();
 if (thecurch!=0) delete thecurch;thecurch=0;
 if (thehvch!=0) delete thehvch;thehvch=0;

}
/*
 * define settings and set hv channel
 */
void ModuleHv::Settings(SetParser* sp) {
   thesettings=sp;
   SensorDescriptor* s=sp->getSensorDescriptor("default");
   theWait= s->dtvset*1000;
   thehvch->lock();
   DT_DEBUG_LTAPPL ("Hv setters: SwLimit: " << s->vmax << " Rampup: "<<s->hvrumpup <<" rmpdwn: " << s->hvrumpdwn);
   thehvch->setHvSwVoltLimit(s->vmax);
   thehvch->setHvRampUp(s->hvrumpup);
   thehvch->setHvRampDown(s->hvrumpdwn);
   thehvch->setHvCurrentLimit(s->imaxhw);
   thehvch->justDoIt();
   thehvch->unlock(); 
   thehvch->wait();
}
/*
 * change HV 
 *(can take a while in case of rs232)
 */
bool ModuleHv::ChangeHV(float volt , bool block ) {   

  float vmax=thesettings->getSensorDescriptor("default")->vmax;
  
  if(volt>vmax) {
    volt=vmax-1; // max can result in a rejection of the value
  }
  theSetHv=volt;
  thehvch->lock(); 
  thehvch->setHvVolt(volt);
  thehvch->justDoIt();
  thehvch->unlock();
  thehvch->wait();
  if (!block) {return false;}
  for(int itry=0;itry<10;itry++) {
    if( HvReached(true)) {
      return true;
    }
    else { DT_WARN(" present hv "<<theHv<<" requested : "<<  volt << "try " <<itry <<" max v= "<< vmax);
    }
    
  }  
  return false;
}

  
bool  ModuleHv::HvReached(bool waitforcheck){
  if( waitforcheck){
    usleep(theWait);
  }
  float dv=thesettings->getSensorDescriptor("default")->dv;//resolution for the HV
  thehvch->justDoIt(); 
  thehvch->wait(); 
  thehvch->lock(); 
  theHv=thehvch->getHvVolt();
  thehvch->unlock();
  therec->getmodsctrl()->sethvbias(theHv);
  DT_DEBUG_LTAPPL("current hv " << theHv <<" requested " <<theSetHv);
  return fabs(theHv-theSetHv) < dv ;
}


/*
 * get current, n meas averaged
 */ 
float ModuleHv::getCurrent(bool update) {
  if( !update ) { return theCurrent;}
  float cur=0.;
  int n=thesettings->getSensorDescriptor("default")->npointsi;
  for(int i=0;i<n;i++) {
    cur+=thecurch->getCurrent();
    usleep(100);
  }
  theCurrent=cur/n; 
  therec->getmodsctrl()->setitest(theCurrent);
  DT_DEBUG_LTAPPL(" ModuleHv: Current"<< theCurrent);
  return theCurrent;
}
/*
 * get Itest
 */ 
float ModuleHv::getItest(float v) {
  if(!ChangeHV(v)) DT_WARN("ModuleHv: can't set voltages");
  return getCurrent();
}
/*
 * get HV
 */ 
float ModuleHv::getVbias(bool update ) {
  if( !update ) { return theHv;}
  float v=0.;
  int n=thesettings->getSensorDescriptor("default")->npointsi;
  for(int i=0;i<n;i++) {
    thehvch->lock();
    v+=thehvch->getHvVolt();
    thehvch->unlock(); 
  } 
  float volt= v/n;
  therec->getmodsctrl()->sethvbias(volt);
  theHv=volt;
  return volt;
}
