#include <qapplication.h>
#include "ModuleTestSetup.h"
#include <fstream>
#include "Record.h"
#include "RecHeader.h"
#include "ModId.h"
#include "setparser.h"
#include "benchposconnection.h"
#include "daqerrormsg.h"
#include "ltparser.h"
#include "Setup.h"
#include "DaqCtrl.h"
#include "TestProcedure.h"
#include "Module.h"
#include "diochannelinterface.h"
#include "cmdlinedecode.h"
using namespace std;

extern   LtParser *ltparser;

ModuleTestSetup::ModuleTestSetup(){
  thesetup=0; thetestproc=0;
  thedaqctrl=0;therecord=0;
  thesettings=0;
  themodules.clear();//??
 DT_NOTE(" start  basesetupsetup ");
 therecord=new Record();
  std::ifstream inr("runN");
 int rN=0;
 while(inr>>rN);
 rN++;
 DT_DEBUG_LTAPPL(" runN "<<rN);
 std::ofstream ofr("runN");
 ofr<<rN<<endl;;
 theRunNr=rN;
 therecord->getheader()->setrun(rN);

 DT_NOTE(" base setup is constructed  ");


 thesetup=new Setup(therecord);
//parse settings.xml 
  char* envir=getenv("DAQHOME");
  string s(envir);
  string setfile;
  if("found" == CmdLineDecode(qApp->argc(),qApp->argv(),"settings",
			       setfile,true) ) {
    if (setfile[0] != '/') {
      // relative path 
      setfile=s+"/Config/"+setfile;
    }
  }
  else {
    setfile=s+"/Config/settings.xml";
  }
  QString set(setfile.c_str());

  stime=QDateTime::currentDateTime();
  char st[30];
  sprintf(st,"%s",QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss").latin1());
  therecord->getheader()->setctime(st);

  ParseSettings(set);
  thedaqctrl= new DaqCtrl(thesettings );
//create testprocedure
}

ModuleTestSetup::~ModuleTestSetup(){
  StopTest(); // deleters the modules ,remove modules from record and deletes TestProcedure
  delete thesettings ;
  if(thesetup) {delete thesetup;}
  delete therecord;
  delete thedaqctrl;
  powerOffNotRegisteredMod( );
}
int ModuleTestSetup::getCTime() const{
  return stime.secsTo(QDateTime::currentDateTime());
}


/*
 *  parse xml settings file for modules , setup and procedures
 *  use ClientTools/setparser
 */
void ModuleTestSetup::ParseSettings(const QString& setfile) {
  DT_NOTE("Procedure: parse "<<setfile);
  if(!thesettings) {thesettings= new SetParser();}
  else { thesettings->clear();}
  sleep(1);
  string setf=string(setfile);
  thesettings->parseFile(setf.c_str());
  therecord->getheader()->setsettings(setf.c_str());
  thesetup->Settings(thesettings);
  thesetup->setTime(stime);
  return ;
}

/*
* clean the Record
*/
void ModuleTestSetup::ResetRecord() {
  DT_NOTE(" ResetRecord ");
  therecord->Reset(); 
}

void ModuleTestSetup::CreateTestProcedure(QObject* calparrent){

 thetestproc=new TestProcedure(calparrent,"Test");
 thetestproc->setTime(stime);
 thetestproc->Construct(thesetup,themodules,thedaqctrl,thesettings,therecord);
}

int ModuleTestSetup::StopTest(){
  ResetRecord();
  if(thetestproc) delete thetestproc;
  //else return -1 ;
  DT_DEBUG_LTAPPL("delete "<< themodules.size());
  for( vector<Module*>::const_iterator ci= themodules.begin();
       ci != themodules.end();ci++){
    if (*ci) { delete *ci;}
    else return -1;
  }
  DT_DEBUG_LTAPPL("Modules Deleted");
  themodules.clear();
  return 0;
}

int ModuleTestSetup::AddModules( vector<ModId*>* modid){
  theNmod=0;
  if(thesettings) {
    DT_NOTE(" start module testsetup ");
    //construct setup
    theNapvmax=1;
    int bpos=0;
    int napvspmod=1;
    Module* imod;
    if(modid){
    for ( vector<ModId*>::const_iterator ci =modid->begin();
	  ci != modid->end();ci++) {
      bpos=(*ci)->getBenchPosition(); 
      napvspmod=(*ci)->getNrApvs();
      DT_DEBUG_LTAPPL(" bpos "<<bpos <<" id "<<
		      (*ci)->getModId().c_str()  <<" napvpermodule " <<
		      napvspmod);
      //the bpos, id and napvspmod have to be replaced by ModId
      imod= new Module((*ci),therecord, thedaqctrl);
      thedaqctrl->registerGroup(imod->getGroupNr());
      themodules.push_back(imod);
      if(napvspmod>theNapvmax)  theNapvmax=napvspmod;
      imod->setTime(stime); 
      theNmod=themodules.size();      
    }
    // make sure the modules are switch on before doing the settings.
    int powerswitchondelay=4;
    if( thesettings->getProcedureDescriptor("default")){
      powerswitchondelay=thesettings->getProcedureDescriptor("default")
	->powerswitchondelay;
    }
    sleep(powerswitchondelay);
    for(vector<Module*>::iterator mi=themodules.begin();
	mi != themodules.end();mi++){
      (*mi)->Settings(thesettings);  
    }
    }                                         
    DT_NOTE(" ModuleTestSetup: constructed "<<theNmod<<" modules ");
    if(theNmod<=0)  {DT_ERROR("Procedure:define some detectors in the configuration file LtSetup !");;}
    else {
      DT_NOTE(" modules setup is constructed  ");
    }
   
  }
  return theNmod;
}


void ModuleTestSetup::powerOnNotRegisteredMod( vector<ModId*>* vmi){
  if(vmi){
    for (unsigned int  lc=0;lc<vmi->size();lc++){
      ModId* mi=(*vmi)[lc];
      if(mi) {
	BenchPosConnection* bi=
	  ltparser->getBenchPosConnections(mi->getBenchPosition());
	if (bi) {
	  DioChannelInterface* powerswitch = 
	    ltparser->getDioChannel(
	      bi->getPowerSwitch());
	  if( powerswitch) {
	    powerswitch->setBit(0);
	    fNotRegPowerSwitches.push_back(powerswitch);
	  }
	}
      }
    }
    
  }
}

void ModuleTestSetup::powerOffNotRegisteredMod( ){
 for (unsigned int  lc=0; lc< fNotRegPowerSwitches.size();lc++){
    fNotRegPowerSwitches[lc]->setBit(1);
    delete fNotRegPowerSwitches[lc];
  }
  fNotRegPowerSwitches.clear();
}

ColdBox* ModuleTestSetup::getColdBox(){ return thesetup->getTempCtrl();}
bool ModuleTestSetup::coldBoxValid() { return  thesetup->coldBoxValid();}
