#include "OnlineMonitor.h"
#include "daqerrormsg.h"
#include "RecHeader.h"
#include "RecSetup.h"
#include "RecModule.h"
#include "RecModHeader.h"
#include "RecModTest.h"
#include "RecModSctrl.h"
#include "Record.h"
#include "RecUtil.h"

#define NAPVCH 128
using namespace std;
using namespace RecUtil;

OnlineMonitor::OnlineMonitor(Record *rec)
{
   therec=rec;
   dtsleep=5;
   msg.SetWhat(kMESS_OBJECT); 
   SendingActive=false;
}

void OnlineMonitor::run()
{
  while (Sending)
    {
      TServerSocket *srv=new TServerSocket(9010,kTRUE);
      srv->SetOption(kNoBlock,1);
      DT_DEBUG_LTAPPL("# Online Monitor - waiting for client...");
      TInetAddress saddr;
      int scnt=1;
      do
	{
	  ss=srv->Accept();
	  scnt+=1;
	  sleep(1);
	}
      while ((int)ss==-1 && scnt<=2);
      if ((int)ss!=-1) 
	{
	  DT_DEBUG_LTAPPL( "# Thread - Server - connection established to: ");
	  saddr=ss->GetInetAddress();
	  saddr.Print();
	  ss->Send("OnlineMonitor");
	  srv->Close();
	  send();
	  ss->Send("Data_Is_Sent");
	  ss->Close();
	  delete ss;
	}
      else 
	{
	  srv->Close();
	}
      delete srv; 
      sleep(dtsleep);
    }
  SendingActive=false;
}

void OnlineMonitor::StartThread()
{
  Sending=true;
  if (!SendingActive) 
    {
      SendingActive=true;
      start();
    }
}

int OnlineMonitor::StopThread()
{
  Sending=false;
  if (SendingActive)
    {
      wait();
      return 0;
    }
  else return -1;
}

void OnlineMonitor::send()
{
     RecHeader *header=therec->getheader();
     RecSetup *setup=therec->getsetup();
     vector<RecModule*> modules=therec->getmodules();

     Send(totv(modules.end()-modules.begin()));

     for(vector<RecModule*>::iterator recmod=modules.begin();
      recmod != modules.end(); recmod++)
       {   
	 Send(totn(header->getoperator()));
	 Send(totn(header->getcenter()));
	 Send(totv(header->getnev()));
	 Send(totv(header->getnmod()));
         Send(totv(header->getrun()));
	 Send(totv(header->gettime()));
	 Send(totv(header->getapvmod()));
	 Send(totn(header->getsettings()));
	 Send(totn(header->getctime()));
//
	 Send(totv(setup->gettemptag()));
	 Send(totv(setup->gettempcold()));
	 Send(totv(setup->gettempbox()));
	 Send(totv(setup->gettempwarm())); 
	 Send(totv(setup->gettempext()));
	 Send(totv(setup->gethumext()));
	 Send(totv(setup->gethumbox()));
	 Send(totv(setup->gettemps()));
	 Send(totv(setup->gethums()));
 
	 vector<float>* ctime=setup->gettime();
	 TGSend(ctime,setup->gettemptagt(),"TGtemptag(t)");
	 TGSend(ctime,setup->gettempcoldt(),"TGtempcold(t)");
	 TGSend(ctime,setup->gettempwarmt(),"TGtempwarm(t)");
	 TGSend(ctime,setup->gettempboxt(),"TGtempbox(t)");
	 TGSend(ctime,setup->gettempextt(),"TGtempext(t)");
	 TGSend(ctime,setup->gethumboxt(),"TGhumbox(t)");
	 TGSend(ctime,setup->gethumextt(),"TGhumext(t)");

	 Send(totn((*recmod)->getmodheader()->getmodid()));
	 Send(totn((*recmod)->getmodheader()->getmodtype()));
	 Send(totn((*recmod)->getmodheader()->getdcuid()));
	 Send(totv((*recmod)->getmodheader()->getnapvs()));
	 Send(totv((*recmod)->getmodheader()->getnbench()));
	 Send(totv((*recmod)->getmodheader()->getnapverrors()));
 
	 Send(totv((*recmod)->getmodsctrl()->gettemphybdcu1()));
	 Send(totv((*recmod)->getmodsctrl()->gettemphybdcu2()));
	 Send(totv((*recmod)->getmodsctrl()->gettempsenntsc1()));
	 Send(totv((*recmod)->getmodsctrl()->gettempsenntsc2()));
	 Send(totv((*recmod)->getmodsctrl()->gethvbias()));
	 Send(totv((*recmod)->getmodsctrl()->getileakdcu()));
	 Send(totv((*recmod)->getmodsctrl()->getitest()));
	 Send(totv((*recmod)->getmodsctrl()->getihyb25()));
	 Send(totv((*recmod)->getmodsctrl()->getihyb125()));
	 Send(totv((*recmod)->getmodsctrl()->getvhyb25()));
	 Send(totv((*recmod)->getmodsctrl()->getvhyb125()));
	 Send(totv((*recmod)->getmodheader()->getnbadch()));
	 Send(totv((*recmod)->getmodtest()->connectivity()));

	 Send(totv((*recmod)->getmodtest()->TVivv()));
	 Send(totv((*recmod)->getmodtest()->TVivi()));
	 ctime=(*recmod)->getmodtest()->vtimet();
	 Send(totv((*recmod)->getmodtest()->vtimet()));
	 Send(totv((*recmod)->getmodsctrl()->it()));

	 TGSend(ctime,(*recmod)->getmodtest()->avrmst(),"TGavrms(t)");
	 TGSend(ctime,(*recmod)->getmodtest()->avcalat(),"TGavcala(t)");
	 TGSend(ctime,(*recmod)->getmodheader()->nbadcht(),"TGnbadcht(t)");
	 TGSend(ctime,(*recmod)->getmodsctrl()->temphybdcu1t(),"TGtemphybdcu1(t)");
	 TGSend(ctime,(*recmod)->getmodsctrl()->temphybdcu2t(),"TGtemphybdcu2(t)");
	 TGSend(ctime,(*recmod)->getmodsctrl()->temphybdcu3t(),"TGtemphybdcu3(t)");
	 TGSend(ctime,(*recmod)->getmodsctrl()->ihyb125t(),"TGihyb125(t)");
	 TGSend(ctime,(*recmod)->getmodsctrl()->ihyb25t(),"TGihyb25(t)");

	 TGSend(ctime,(*recmod)->getmodsctrl()->vt(),"TGv(t)");
	 TGSend(ctime,(*recmod)->getmodsctrl()->it(),"TGi(t)");
	 TGSend(ctime,(*recmod)->getmodsctrl()->tempsen1t(),"TGtempsen1(t)");
	 TGSend(ctime,(*recmod)->getmodsctrl()->tempsen2t(),"TGtempsen2(t)");

	 vector<float>* TVivv=(*recmod)->getmodtest()->TVivv();
	 TGSend(TVivv,(*recmod)->getmodtest()->TVivi(),"TGiv");

	 Send((*recmod)->getmodtest()->THpedestal());
	 Send((*recmod)->getmodtest()->THnoise());
	 Send((*recmod)->getmodtest()->THrawnoise());
	 Send((*recmod)->getmodheader()->THchflag());
	 Send((*recmod)->getmodtest()->THcalA());
	 Send((*recmod)->getmodtest()->THcalPApeak());
	 Send((*recmod)->getmodtest()->THcalSbackpl());
	 Send((*recmod)->getmodtest()->THcalSled());
	 Send((*recmod)->getmodtest()->TPbp());
	 Send((*recmod)->getmodtest()->TPled());
         
	 int napvs=(*recmod)->getmodheader()->getnapvs();
	 for (int k=0;k<napvs;k++)
	   {
	     Send((*recmod)->getmodtest()->hcmn(k));
	     Send((*recmod)->getmodtest()->hpipe(k));
	     for (int j=0;j<NAPVCH;j++)
	       {
		 Send((*recmod)->getmodtest()->hcalprof(k*NAPVCH+j)); 
		 Send((*recmod)->getmodtest()->hplatency(k*NAPVCH+j));
              }
         }
     }
}

void OnlineMonitor::TGSend(vector<float>* x,vector<float>* y,const char* title)
{
 TVector x1=totv(x);
 TVector y1=totv(y);
 TGraph* G=new TGraph(x1,y1);
 G->SetTitle(title);
 msg.Reset();
 msg.WriteObject(G);
 ss->Send(msg);
 delete G;
}


