#include "PedestalRun.h" 
#include "Module.h"
#include "daqerrormsg.h"
#include "TestBase.h"
#include "ModuleDescriptor.h"
#include "HybridI2C.h"
#include "RecModule.h"
#include "RecAnalyzer.h"
#include "RecModTest.h"
#include "RecModHeader.h"
#include "RecAnalyzer.h"
#include "RecModTest.h"
#include "RecModHeader.h"
#include "Spill_Analysis.h"
#include "setparser.h"


PedestalRun::PedestalRun(Module* m)   { 
 thefeds=m->feds();
 theapvanals=m->apvanalysis();
 thei2c=m->hybridi2c();
 therec=m->getrecmod();


  Napvs=thefeds.size();
  DT_DEBUG_PROC("Create PedestalRun");
}
/**/
PedestalRun::~PedestalRun() {
}
/**/
void PedestalRun::InitTest(SetParser* s) {
  DT_DEBUG_PROC("PedRun : init ");
   done=false;
   Nev=0;
   ps=0;
  prevmode= thei2c->GetMode();

  if (thei2c->CheckDecMode(prevmode))
   ps=s->getPedRunDescriptor("deconv");
  else
   ps=s->getPedRunDescriptor("peak");

   Npedrun=ps->nped;
// default
   therec->getmodtest()->ResetPed();
   float pedmin=50;
   float pedmax=500;
   for(vector<Spill_Analysis*>::iterator iapv=theapvanals.begin();iapv!=theapvanals.end();iapv++) {
   (*iapv)->setNoiseEstimator(ps->nped,ps->npedraw,
			      ps->modped,ps->tskip,ps->tbad,
                              ps->rmsmin,ps->rmsmax,pedmin,pedmax,ps->pskip);
   (*iapv)->setCmnEstimator(ps->modcmn,ps->nstrcmn,ps->ncmniter);
   }
   DT_DEBUG_LTAPPL(" PedestalRun InitSetup: lat "<<ps->lat);
   thei2c->SetLatency(ps->lat);
   thei2c->UnsetCalMode(prevmode);
   int cm=int( thei2c->GetMode());
   therec->getmodtest()->setapvmped(cm);
// Clear analysis
   for(vector<Spill_Analysis*>::iterator iapv=theapvanals.begin();iapv!=theapvanals.end();iapv++) (*iapv)->Clear(); 
}
/**/
bool  PedestalRun::Update(int nevinspill) {
  DT_DEBUG_PROC("PedestalRun: Update  nevinspill "<<nevinspill);
  if( nevinspill>256 )  nevinspill=256;
  if(done) {thei2c->SetMode(prevmode); return true;}
  loadSpill(thefeds, theapvanals, nevinspill);

  int nbadev=0;
  for(int iev =0; iev < nevinspill; iev++) {
    Nev++;
    DT_DEBUG_PROC(" Nev "<<Nev);
    if(Nev>Npedrun) {
      DT_DEBUG_PROC(" PedestalRun: update ; test done ");
      done=true; return true;}
    for (int ich=0;ich<Napvs;ich++) { 
      TH1F* hcmn= therec->getmodtest()->hcmn(ich);
      theapvanals[ich]->loadEvent(iev);
      //       cout<<"PedestalRun: update; loadedevent "<<iev<<" ich "<<ich<<endl;
      if (theapvanals[ich]->FindFrame()) {
	theapvanals[ich]->AnalyzePed(1);
	valarray<double> cmn=*(theapvanals[ich]->getCmn());
	float x=cmn.sum()/cmn.size();
	DT_DEBUG_PROC(" cmn "<<x<<" "<<ich);
	if(x>-100.&&x!=0.) hcmn->Fill(x);
      } else {DT_WARN("PedestalRun::Update >>> Cannot find frame ");nbadev++;}
    }
  }
  if(nbadev>=nevinspill) return false;
  return true;
}
/**/
void PedestalRun::FillRecord() {  
  DT_DEBUG_LTAPPL(" PedestalRun: FillRecord ");
  thei2c->SetMode(prevmode);

  int nch=Napvs*NAPVCH;
  double pedestal[nch];
  double rawrms[nch];
  double rmsnoise[nch];
  int  badch[nch];
  float avrms=0.;
 float avrrms=0.;
  int nbad=0;
  map<int, int, less<int> > mbad;
  for (int ich=0;ich<Napvs;ich++)     { 
    valarray<double> ped=*(theapvanals[ich]->getPedestal());
    valarray<double> rraw2=*(theapvanals[ich]->getRawRms2());
    valarray<double> rms=*(theapvanals[ich]->getRms());
    vector<int>  bad=*(theapvanals[ich]->getChStatus());
    nbad+=theapvanals[ich]->getNBadChan();
    avrms+=theapvanals[ich]->getMeanRms();
    avrrms+=theapvanals[ich]->getMeanRawRms();
	  
    for (int is=0;is<NAPVCH;is++) {
      pedestal[ich*NAPVCH+is]=ped[is];
      rawrms[ich*NAPVCH+is]=sqrt(rraw2[is]);
      rmsnoise[ich*NAPVCH+is]=rms[is];
      badch[ich*NAPVCH+is]=bad[is];
//     DT_DEBUG_PROC(" ped "<<pedestal[ich*128+is]<<" rms "<<rmsnoise[ich*128+is]<<" rrms "<<rawrms[ich*128+is]<<" nbad "<<badch[ich*128+is]) ;                         
    }
    avrms/=Napvs;
    avrrms/=Napvs;

    TH1F* hped=therec->getmodtest()->THpedestal();
    if(hped!=0) { for(int i=0;i<nch;i++)
      hped->SetBinContent(i+1,pedestal[i]); }
    TH1F* hrrms =therec->getmodtest()->THrawnoise();
    if(hrrms!=0){ for(int i=0;i<nch;i++)
      hrrms->SetBinContent(i+1,rawrms[i]);  }
    TH1F* hrms =therec->getmodtest()->THnoise();
    if(hrms!=0){ for(int i=0;i<nch;i++)
      hrms->SetBinContent(i+1,rmsnoise[i]);  }
    TH1F* hbad = therec->getmodheader()->THchflag();
    if(hbad!=0) { for(int i=0;i<nch;i++) {
      hbad->SetBinContent(i+1,badch[i]);
      if(badch[i]!=0) mbad[i+1]=-badch[i]; }
               }      
  }
  DT_DEBUG_LTAPPL(" PedestalRun: analyze "<<mbad.size());   
 therec->getmodheader()->setbadchmap(thei2c->CheckMode(prevmode),mbad);
// therec->AnalyzePed(thei2c->CheckMode(prevmode),ps); 
therec->getrecanalyzer()->AnalyzePed(thei2c->CheckMode(prevmode),ps); 
  DT_DEBUG_LTAPPL(" PedestalRun: done");
}
