using namespace std;
#include "Setup.h"
#include "tscchannel.h"
#include "coldbox.h" 
#include "humidityctrlinterface.h"
#include "slowctrlconfig.h"
#include  "temperaturechannellt.h" 
#include  "humiditychannellt.h" 
#include "setparser.h"
#include "Record.h"
#include "RecSetup.h"
#include "daqerrormsg.h"
#include "ltparser.h"

extern LtParser* ltparser;  

Setup::Setup(Record* r) {
  therec=r;  
  thescc=ltparser->getSlowCtrl(1);
  theNtemp=thescc->getNrTempProbes(); 
  theNhum=thescc->getNrHumidityProbes(); 
  DT_DEBUG_LTAPPL(" Setup: Ntemp "<<theNtemp<<" Nhum "<<theNhum);
  // cold box has to be constructed before the 
  thetempctrl=new ColdBox(thescc->getColdBoxId(0));   

  theboxtemp=0; theambienttemp=0;thetempctrltemp=0;theheatsinktemp=0;
  for(int itemp=0;itemp<theNtemp;itemp++) {  
    TemperatureChannelLt* tempprobe= new TemperatureChannelLt(thescc->getTempProbeId(itemp)) ;
    DT_DEBUG_LTAPPL("Temp " <<  itemp <<" Id "<<  thescc->getTempProbeId(itemp)
		    <<" name: "<< tempprobe->getName()  );
    // strncmp returns 0 is equal !!
    if(!strncmp(tempprobe->getName(),"Tempctrlprobe",13)) {thetempctrltemp=tempprobe;continue;}
    if(!strncmp(tempprobe->getName(),"Box",3)) { theboxtemp=tempprobe;continue;}
    if(!strncmp(tempprobe->getName(),"Ambient",7)) {theambienttemp=tempprobe; continue;}
    if(!strncmp(tempprobe->getName(),"HeatSink",8)) {theheatsinktemp=tempprobe;continue;}  
    thetempch.push_back(tempprobe);
  }


  theboxhumidity=0;theambienthumidity=0;
  if( thetempctrl->activeHumidityCtrl()) {
    theboxhumidity=thetempctrl-> getHumidityCtrl();
    DT_DEBUG_LTAPPL("hummidity ctrl defined ");
  }
  for(int ihum=0;ihum<theNhum;ihum++) {
    DT_DEBUG_LTAPPL("Hhm " <<  ihum <<" Id "<<  thescc->getHumidityProbeId(ihum)) ;
    HumidityChannelLt* humidity=
      new HumidityChannelLt(thescc->getHumidityProbeId(ihum));
    if(!theboxhumidity) {
      if(!strncmp(humidity->getName(),"Box",3)){
	theboxhumidity=humidity;
	continue ; // don't store in general humidity sensors 
      }
    }
   if(!strncmp(humidity->getName(),"Ambient",7)){
       theambienthumidity=humidity; continue;
    }
     // generic hum channel
    thehumch.push_back(humidity);  
   
  }
  therec->ConstructSetup(thetempch.size(),thehumch.size());
  float ftime=float(stime.secsTo(QDateTime::currentDateTime()));
  DT_NOTE("Setup: is constructed  Ntemp "<<theNtemp<<" Nhum "<<theNhum);
}
/*
 */
Setup::~Setup() { 
 thetempctrl->stopAutoCtrl();
  for(int itemp=0;itemp<theNtemp;itemp++) if(thetempch[itemp]!=0) { delete thetempch[itemp];}
  for(int ihum=0;ihum<theNhum;ihum++) if(thetempch[ihum]!=0)  {delete thehumch[ihum];}
  if (thetempctrl) {  delete thetempctrl;}

  DT_NOTE("Setup: destruct setup ");
}

/*
 * define settings 
 */
void Setup::Settings(SetParser* set) {
thesettings=set;
 SetupDescriptor* s=thesettings->getSetupDescriptor("default");
 theCurTemp=s->startingtemp;
  ChangeTemp(theCurTemp);
//  thetempctrl->startAutoCtrl();  
}
/*
 * regulate cooling power according to the history
 */

bool   Setup::coldBoxValid(){ 
  return thetempctrl? thetempctrl->isInitialized():0;
}

void Setup::UpdateCool()
{
  thetempctrl->control();
}
float  Setup::getActualTemp(){
  return thetempctrl->getActualTemperature () ;
  
}
float Setup:: getBoxHumidity(){
  if(theboxhumidity->getHumidity()) { 
    return theboxhumidity->getHumidity();
  }
  else { return -1;}

}
bool  Setup::maxHumidityReached(){
  if( thetempctrl->activeHumidityCtrl()){
    return thetempctrl->maxHumidityReached();
  }
  else {
    return false;
  }

}


/*
 * change cooling
 */
void Setup::ChangeTemp(float temp) { 
  if( ! thetempctrl->getAutoCtrl()){thetempctrl->startAutoCtrl();}
     theCurTemp=temp;
     thetempctrl->setTargetTemperature(temp);
     therec->getsetup()->settemptag(temp);
   }                                                            
/* update Xt parameters
 */
void Setup::UpdateXt()
{
  DT_DEBUG_LTAPPL("Setup:updateXt");
  float ftime=float(stime.secsTo(QDateTime::currentDateTime()));
  RecSetup* rec=therec->getsetup();
 
  rec->settemptag(theCurTemp);  
  DT_DEBUG_LTAPPL(" Setup: theCurTmp "<<theCurTemp); 
  
  int tch=0;
  if(thetempctrltemp) {rec->settempcold(thetempctrltemp->getTemperature());}
  if(theboxtemp) { rec->settempbox(theboxtemp->getTemperature());}
  if(theambienttemp) { rec->settempext(theambienttemp->getTemperature()); }
  if(theheatsinktemp) {  rec->settempwarm(theheatsinktemp->getTemperature());}
  for(vector<TemperatureChannelLt*>::iterator it=thetempch.begin();it!=thetempch.end();it++) {
    DT_DEBUG_LTAPPL("Update temp: tdevicename "<<(*it)->getName());       
    rec->settemps(tch,(*it)->getTemperature()); tch++;
  }


    int hch=0;
    if(theboxhumidity) {rec->sethumbox(theboxhumidity->getHumidity());}
    if( theambienthumidity){  rec->sethumext(theambienthumidity->getHumidity());}
  for(vector<HumidityChannelLt*>::iterator ih=thehumch.begin();ih!=thehumch.end();ih++) {
    rec->sethums(hch,(*ih)->getHumidity()); hch++;
  }
  DT_DEBUG_LTAPPL("Setup:  add meas "<<ftime);


  rec->AddMeast(ftime);

  DT_DEBUG_LTAPPL(" tempbox  "<<rec->gettempbox());
  DT_DEBUG_LTAPPL(" tempcold  "<<rec->gettempcold());
#ifdef THISISNOTDEFIEND 
//#ifdef DT_DEBUG_ENABLE  
// comment out because to much output 
   DaqToolsErrorMessage de;
  if( de.getPrintingLevel() & DT_DEBUG_LTAPPL) {
    vector<float>* vtb=(rec->gettempboxt());
    vector<float>* vt=(rec->gettime());
    for(int i=0; i<vt->size();i++) {
      DT_DEBUG_LTAPPL(" time "<<(*vt)[i]<<" tembox "<<(*vtb)[i]);
    }
  }
#endif 
  DT_DEBUG_LTAPPL(" Setup: done updateXt");
}
