#include "TestProcedure.h"
#include "RecordReader.h"
#include "DbParser.h"
#include "ModId.h"
#include "Record.h"
#include "RecHeader.h"
#include "ProcedureDescriptor.h"
#include <qtimer.h>
#include <qdatetime.h> 
#include "qapplication.h" 
#include "plli2cchannellt.h"
#include "setparser.h"
#include "tscchannelinterface.h"
#include "tscchannel.h"
#include "daqerrormsg.h"
#include "ltparser.h"
#include "Setup.h"

#include "DaqCtrl.h"
#include "Module.h"
#include "TestStatus.h"
#include <fstream.h>
#include <cmath>

extern   LtParser *ltparser;

#define TD  100
#define SCENARIOSTEPS  1000
using namespace std;

TestProcedure::TestProcedure(QObject  *callingparent, const char *name ): 
QObject( callingparent, name ) 
{ 
  myburn=new Scenario[SCENARIOSTEPS];
   checkenvpending=false;
   theScType="default";
   thelastchtempstep=0; 
  thestatus=new TestStatus();
  fTime =new QTime();
  fTime->start();
  actionTimer=new QTimer(this);
  connect(actionTimer,SIGNAL(timeout()),this,SLOT(ScenarioAction()) );
  slctrlTimer=new QTimer(this);
  connect(slctrlTimer,SIGNAL(timeout()),this,SLOT(CheckEnv()) );
  //testtimer=0;//new QTimer(this);
  //connect(testtimer,SIGNAL(timeout()),this,SLOT(UpdateTest())); 
  theNrecord=0;
  theNacq=0;
  selectedrec.clear();
  DT_NOTE("TestProcedure is created");
  Tologfile("==StartNewModuleTest==");

}
/**/
void TestProcedure::Construct(Setup* s,vector<Module*> m,DaqCtrl* c,SetParser* p,Record* r) {
  themodules=m;
  thesetup=s;
  thedaqctrl=c;
  thedaqctrl->setI2CChannel(themodules[0]->hybridi2c()->plli2c());
  therecord=r;
  thesettings=p;
  // will be changed if start has an other procedure descriptor
  rfname=thesettings->getProcedureDescriptor(theScType)->resultsfile();
  DT_NOTE("TestProcedure results file "<<rfname); 
  sleep(1);
  CheckEnv();
  DT_NOTE("TestProcedure is constructed with modules "<<m.size());
  thestatus->setInit();
  SetI2c(thesettings->getI2cDescriptor("i2cinit")); // init i2c
  DT_DEBUG_LTAPPL("i2c i2cinit   done apv ptr ");
  SetI2c(thesettings->getI2cDescriptor("i2cpedpeak")); // need to set some sets
  //DT_DEBUG_LTAPPL("ApvReadoutctrl "<< c);
  fActiveGroup=thedaqctrl->getFirstGroup();
  DT_DEBUG_MUX("current Mux group="<<fActiveGroup );
  thedaqctrl->AdjustFedThresholds(); // only once for the moment
  DT_DEBUG_LTAPPL("fed threshold done");
} 
/*
 */
TestProcedure::~TestProcedure() {
   DT_NOTE("TestProcedure is destructed ");
   ScenarioSteering("Stop");
   Abort();
   wait(); // in case thread is running 
   thedaqctrl->setI2CChannel(0);
   actionTimer->stop();slctrlTimer->stop();
   //testtimer->stop();
   disconnect(actionTimer,0,this,0);
   disconnect(slctrlTimer,0,this,0);
   //disconnect(testtimer,0,this,0);
   delete fTime;delete actionTimer;delete slctrlTimer;
   //delete testtimer;
  delete thestatus;
  delete[]  myburn;
}
/* 
 * load scenario file 
 */
bool TestProcedure::LoadScenario(const QString& fname)  {
  DT_NOTE(" Procedure: LoadScenario "<<fname);
 return readscenario(fname);
}
/*
 * load scenario from default scenario file
 */
bool TestProcedure::LoadScenario() {
  const char* fname=thesettings->getProcedureDescriptor(theScType)->scenariofile();
  QString s(fname);
  return readscenario(s);
}
/* 
 * private readscenario 
 */
bool  TestProcedure::readscenario (const QString& fname) {
  DT_NOTE("TestProcedure: readscenario "<<fname);
  Tologfile("Readscenario",fname);
  std::ifstream in_file(fname);
  if(!in_file)  {
    DT_WARN("Procedure: cannot open scenario file "<<fname);
    return false;
  } 
  int t,a1,a2;
  string cr, carg1;
  int i=0;
  theNsteps=0;
  while(in_file>>t>>cr>>carg1) {
    myburn[i].dt=t; 
    myburn[i].rtype=cr;
    myburn[i].arg1=carg1;
    DT_NOTE("   "<<myburn[i].dt<<"   "<<myburn[i].rtype<<"    "<<myburn[i].arg1;);
    i++;
  }
  theNsteps=i;
  DT_NOTE("TestProcedure: nsteps= "<<theNsteps);
  in_file.close();
  return true;
}

/*
 * Start Scenario timers
 * the action timer is steered by scenario
 * the Slow cntrl timer updates defined parameters with dt
 * Stop, Pause, Continue functions for scenario
 */
bool TestProcedure::ScenarioSteering(const QString& s) {
  if(!thestatus->isInit() ) return false;
  DT_NOTE("TestProcedure: ScenarioSteering "<<s);
  Tologfile("ScenarioSteering ",s);  
  if(!thestatus->isRunning()) DT_DEBUG_LTAPPL("status is  not running ");

  if(s=="StartScenario") {
    DT_NOTE(" ScenarioSteering  start ");
   
    if(!thestatus->SetScenario()) 
      {DT_WARN(" scenario is started already");return false;} 
    thecstep=0;
    theScenStartTime=fTime->elapsed()/1000;
    theScenEndTime=abs(myburn[theNsteps-1].dt);
    dtpause=0;
    ScenarioAction();
    CheckEnv();
int dtslow=1000*(thesettings->getProcedureDescriptor(theScType)->dtslowctrl);
    // if(!slctrlTimer->isActive()) 
      slctrlTimer->start(dtslow,FALSE); 
    return true; 
}

  else if(s=="Stop" ) {
    DT_NOTE("ScenarioSteering stop");
    thestatus->ResetScenario();
    if(slctrlTimer->isActive())  slctrlTimer->stop();
    if(actionTimer->isActive())  actionTimer->stop();  
    Abort(); // in case test is not stopped. 
    return true;
  }

  else if(s=="Pause") {
    DT_NOTE("ScenarioSteering  pause ");
    if(thestatus->scenarioOn()) actionTimer->stop();
    thestatus->SetPause();
    dtpause=fTime->elapsed()/1000;
    return true;
  }

  else if(s=="Continue") {
    DT_NOTE(" ScenarioSteering continue was  paused for "<<dtpause);
    dtpause=fTime->elapsed()/1000-dtpause;
    theScenEndTime+=dtpause;
    thestatus->ResetPause();
    if(thestatus->scenarioOn()) ScenarioAction();
    return true;
  }
  else {DT_WARN(" ScenarioSteering "<<s<<" is not defined");return false;}
}
/*
 * change slowcntl check time
 */
void TestProcedure::setDt(int dt) { 
  if(thestatus->scenarioOn()) {
    slctrlTimer->changeInterval(dt*1000);
  DT_NOTE(" Procedure : set Dt "<<dt);
  Tologfile("Change Dt slowctrl ",float(dt));
  }
}

/*
 * actiontimer slot
 * scenario action menu
 * call test according to scenario file
 */
void TestProcedure::ScenarioAction() {
  bool  runtype=false; 
  if(!thestatus->isInit() ) return;
  if(!thestatus->SetAction())  {
    DT_WARN(" ScenarioAction: action is still active ");  return;
  }
  if(thestatus->pauseOn()) return;
  QString testtype(myburn[thecstep].rtype.c_str()); 
  DT_DEBUG_LTAPPL("Procedure: ScenarioAction "<<testtype<<
		  " step "<<thecstep<<" arg "<<myburn[thecstep].arg1);
  Tologfile("ScenarioAction ",testtype);
  if(testtype=="Start") { 
    if(thesettings-> getProcedureDescriptor(myburn[thecstep].arg1)){
      theScType= myburn[thecstep].arg1;    
      rfname=thesettings->getProcedureDescriptor(theScType)->resultsfile();
      DT_NOTE("TestProcedure results file "<<rfname); 
      setDt(1000*(thesettings->getProcedureDescriptor(theScType)->dtslowctrl));

    }// else not found keep "default" proceduredescriptor 
    DT_NOTE(" Procedure:started scenario with procedure descriptor "
	    <<theScType);
  }
   if(testtype=="Stop") {
     if(theNrecord==0) 
    SaveResults(thesettings->getProcedureDescriptor(theScType)->resultsfile()); 
    ScenarioSteering("Stop");
  }
  if(testtype=="ChangeHV"){  ChangeHV(atof(myburn[thecstep].arg1.c_str()));}
  if(testtype=="ChangeCool"){
    thelastchtempstep=thecstep;
    ChangeTemp(atof(myburn[thecstep].arg1.c_str()));
  }
  if(testtype=="TempReached"){ 
    float deltatemp= thesetup->getActualTemp()-therequestedtemp;
    // check if the temperature is inside the requested temperature range 
    if ( fabs(deltatemp) >thesettings->getProcedureDescriptor(theScType)->temperatureaccuracy ){
      // go back to the next line after the  changecool 
      // the increment is done at the end of Scenario action
      thecstep=thelastchtempstep;
      DT_DEBUG_LTAPPL( "delta" << deltatemp << "max delta ");
      DT_DEBUG_LTAPPL(  thesettings->getProcedureDescriptor(theScType)->temperatureaccuracy);
      }
    // else go one
    thestatus->ResetAction();
  }
  if(testtype=="CheckEnv"){ CheckEnv(); }
  if(testtype=="SaveRec")  {

    if(atoi(myburn[thecstep].arg1.c_str())!=0) { 
      selectedrec.push_back(theNrecord);
      therecord->getheader()->setindb(1);
    } else therecord->getheader()->setindb(0);

    DT_DEBUG_LTAPPL(" select "<<theNrecord<<" size "<<selectedrec.size());
    SaveResults(thesettings->getProcedureDescriptor(theScType)->resultsfile()); 
  }
 if(testtype=="SaveFile")  {
    if(myburn[thecstep].arg1.length()!=0) {
      DT_NOTE(" Save File "<<myburn[thecstep].arg1.c_str());
      SaveResults(myburn[thecstep].arg1.c_str()); } 
       theNrecord--;
  }
  if(testtype=="ResetRec")  {
    therecord->Reset(); 
  }
  if(testtype=="SaveXml")  {
    SaveDb((thesettings->getProcedureDescriptor(theScType))->xmlfile()); 
  }
  if(thestatus->Find(testtype)) {
    SetI2c(thesettings->getI2cDescriptor(myburn[thecstep].arg1.c_str())); 
    StartTest(testtype);
    runtype=true; 
  }
  else thestatus->ResetAction();

  if(thecstep+1<=theNsteps) {
    if(!actionTimer->isActive()) { 
      if (myburn[thecstep].dt > 0 ){
	dwait=1000*abs(myburn[thecstep+1].dt-myburn[thecstep].dt);//c
	if(dwait<0)  dwait=1000*(myburn[thecstep+2].dt-myburn[thecstep].dt);
	actionTimer->start(dwait,TRUE);
	thecstep++;    
	DT_DEBUG_LTAPPL(" start timer with dwait "<<dwait);
      }
      else { 
	if(!runtype){
	  dwait=- 1000*(myburn[thecstep+1].dt);      
	  DT_DEBUG_LTAPPL("start1 timer with dwait "<<dwait);
	  actionTimer->start(dwait,TRUE);
	  thecstep++;
	  DT_DEBUG_LTAPPL("dwait" << dwait <<"line nr " << thecstep <<
			  myburn[thecstep].rtype );
	}
	//else in case of runtype action the action timer will be restarted
	// at the end of the run. 
      }
    }
  }
}

QString    TestProcedure::getCurrentScenarioLineInfo(){
  QString s("hallo");
  s.sprintf("line: %03d/%03d %s",
	    thecstep,theNsteps,myburn[thecstep].rtype.c_str());
  return s;
}

void TestProcedure::setPll(I2cDescriptor* i2cset){
 if(!thestatus->isInit() ) return;
  currenti2c=i2cset;
  vector<Module*>::iterator imod;
  for(imod=themodules.begin();imod!=themodules.end();imod++) {
    (*imod)->hybridi2c()->setPll(i2cset);
  }
  DT_NOTE(" InitPll's ");
  Tologfile("Initpll");
  thedaqctrl-> sendApvReset();
} 


/*
 * send i2c to the module
 */
void TestProcedure::SetI2c(I2cDescriptor* i2cset)
{
  if(!thestatus->isInit() ) return;
  currenti2c=i2cset;
  vector<Module*>::iterator imod;

  for(imod=themodules.begin();imod!=themodules.end();imod++) {
    (*imod)->hybridi2c()->SetI2C(i2cset);
  }
   DT_NOTE(" SetI2C ");
  Tologfile("SetI2C");
}


void TestProcedure::setLld(int gain, int bias, int ch){
  if(!thestatus->isInit() ) return;
  vector<Module*>::iterator imod;
  for(imod=themodules.begin();imod!=themodules.end();imod++) {
    (*imod)->hybridi2c()->setLld(gain,bias,ch);
  }
  Tologfile("SetLld");
}


/* 
 * Start demanded test s with timer
*/
bool  TestProcedure::StartTest(const QString& s) {
  if(!thestatus->isInit() ) return false;
  if(thestatus->isRunning() )  { 
    DT_WARN(" cannot Start Test "<<thestatus->currentTest()<<"is active "); return false;}
 
  if(thestatus->SetRun(s))  {  DT_NOTE(" start test  "<<s); }
  else {DT_WARN("Start Test: test"<<s<<" cannot be started "); return false;} 
  // check if the daq is used (FED) set to false if not needed
  fDaqUse=true;
  if( s=="IVRun"){ fDaqUse=false;}
  if(thestatus->isRunning()) { DT_DEBUG_LTAPPL(" running now "<<thestatus->currentTest());}
  Tologfile("Start Test ",thestatus->currentTest());
  setapvctrl(s);
  for(vector<Module*>::const_iterator imod=themodules.begin();imod!=themodules.end();imod++) {

    if(!(*imod)->Init(s)) {DT_WARN("StartTest: can't init "<<s); return false;}   
  }
  theNacq=0;
  fActiveGroup=thedaqctrl->getFirstGroup();
  {
    DT_DEBUG_LTAPPL("StartTest Thread "); 
    //only for test with the daq correct this after merging with mux
    thedaqctrl->startDaqTest(); 
    // make sure no thread is running
    if(QThread::wait(2000))  {   QThread::start();    }
    else { 
      DT_WARN("can not restart due hanging update thread");
      return false ;
    }       
  }
  return true;
}

void TestProcedure::run(){
  DT_DEBUG_LTAPPL(" UpdateTest:start run  ");
  bool stayrunning=true;
  theUpdateTries=0;
   do { 
     qApp->lock();
     bool teststatus=thestatus->isRunning();
     qApp->unlock();
     if(!teststatus){  
       DT_DEBUG_LTAPPL(" UpdateTest: forced exit run  ");
       // theapvctrl->stopDaqTest();
       stayrunning=false;
     }
     else {     stayrunning = updateTestAction();
     }
   }while (stayrunning);
   //in case the update TestAction is terminated
   if( thestatus->isAbort()){
     DT_DEBUG_LTAPPL("Daq reset due abort request");
     thedaqctrl->stopDaqTest();
     // abort is nowhere else tested so can be reset here 
     thestatus->Reset();
   }
   if(checkenvpending){checkenvpending=false;  CheckEnv();}
   if(! actionTimer->isActive() && thestatus->scenarioOn()) { 
     if(myburn[thecstep+1].dt<0) {
       dwait=- 1000*(myburn[thecstep+1].dt);
      
       DT_DEBUG_LTAPPL("start2 timer with dwait "<<dwait);
       actionTimer->start(dwait,TRUE);
       thecstep++;
       DT_DEBUG_LTAPPL(" dwait" << dwait <<"line nr " << thecstep 
		       << myburn[thecstep].rtype);
     }
   }
   return; 
}

bool  TestProcedure::updateTestAction(){
  if ( thedaqctrl->GetInterlockLineStatus()) {
    InterLockAction();
    feedBackSignal("interlockactive");
    return false;
  }
  if(thesetup->maxHumidityReached()) {
    feedBackSignal("maxhumidityreached");
    InterLockAction();// same as for interlock
    return false;
  }
  DT_DEBUG_LTAPPL("Check spill  ");
  bool alldone=true; bool domeas=false;
  if( !fDaqUse) {theUpdateTries=0;domeas =true;}
  else {   
    if(thedaqctrl->NewSpill() ) {theUpdateTries=0;domeas=true;}
    else  {domeas=false;} 
  }
  
    
    qApp->lock();
    QString currentTest= thestatus->currentTest();
    qApp->unlock(); 
    if ( currentTest ==  "None") return false;
    for(vector<Module*>::const_iterator imod=themodules.begin();
	imod!=themodules.end();imod++) {
      DT_DEBUG_MUX( "mod"<<(*imod)->getModId()
		    << "is connected "<<(*imod)->ConntectToDAQ());
      // if needed to connect to DAQ and not connected pass 
      if(fDaqUse && !(*imod)->ConntectToDAQ()){	continue; }
 
      //check if this module has to be procees 
      if(!(*imod)->Test(currentTest)->isDone() ) {
	if((*imod)->getModuleErrStatus()){ 
	  DT_DEBUG_LTAPPL("Module in bechpos "<<(*imod)->getBenchPos()<<" skipped due errors ");
	  // error  so set this module to done 
	  (*imod)->Test(currentTest)->Abort();
	}
	DT_DEBUG_LTAPPL("UpdateTest: nevinspill== "<<thedaqctrl->getNrEvents());
	if( domeas) {
	  // this check will exit the module loop in case false (see else) 
	  theNacq++; // is multiplied by the nr of modules connected to the group  
	  if((*imod)->Test(currentTest)->
	     Update(thedaqctrl->getNrEvents())){
	  }
	  else { 
	    DT_WARN("UpdateTest:can't update"  <<currentTest);
	    (*imod)->fModuleErrStatus++;
	  }
	}   
	else { 
	  // new spill returns false  and daq in use 
	  int maxupdatetries= thesettings->getProcedureDescriptor(theScType)->updatetries;
	  if(theUpdateTries++ > maxupdatetries)  {
	    DT_ERROR(" Acquistion err");
	  // so this module is causing the fed is stopped  
	    DT_WARN(" module in benchpos " << (*imod)->getBenchPos() << "cause daq errors");
	    (*imod)->fModuleErrStatus+=10;	    
	  }
	  //simple retry 	  
	  usleep(1000*thesettings->getProcedureDescriptor(theScType)->updatewait); 
	  if( ! (theUpdateTries%8)) {
	    DT_WARN("Force fresh new spill after "<<theUpdateTries <<" tries/ "<<maxupdatetries);
	    thedaqctrl->NewSpill(true); 
	  }
	  return true ; 
	}
      }// end if module done 
      
      // after processing it could be that all modules are 
      // ready so check this to avoid an extra acquisition 
      qApp->lock();
      currentTest= thestatus->currentTest();   
      qApp->unlock();   
      if ( currentTest ==  "None") return false;
      if((*imod)->Test(currentTest)->isDone() ) {
	DT_DEBUG_LTAPPL(" UpdateTest : test  done");
	 
      }
      else { 
	alldone=false; //so if one failes 	
      }
    }
    if(alldone && fDaqUse) {
      if(thedaqctrl->lastGroupNotReached()){	
	alldone=false;
        // check if all mux groups are done 
	DT_DEBUG_MUX("change group from " <<fActiveGroup ); 
	fActiveGroup=thedaqctrl->getNextActiveGroup();
	DT_DEBUG_MUX("to " << fActiveGroup 
		     <<" of "<<   thedaqctrl->getNrGroupsRegistered());
	thedaqctrl->NewSpill(true);
	return true;
      }
    }
    if(alldone) {
      thedaqctrl->stopDaqTest();
      DT_DEBUG_LTAPPL(" alldone");
      qApp->lock();
      QString currentTest= thestatus->currentTest();
      qApp->unlock();   
      if ( currentTest ==  "None") return false;
      Tologfile("Stop Test",currentTest);      
      // set to no status
      thestatus->setErrorStatus(currentTest,0);
      for(vector<Module*>::const_iterator imod=themodules.begin();
	  imod!=themodules.end();imod++) {
	int testresultstatus=
	  (*imod)->Test(currentTest)-> getTestResult(); 
	if( testresultstatus != 0 ){
	  //keep the last result only if different from 0
	  if( testresultstatus < 0){
	    thestatus->setErrorStatus(currentTest, testresultstatus);
	  }
	// positive means successful only if no negative before
	  else { 
	    if(thestatus->getErrorStatus(currentTest) >= 0){
	     thestatus->setErrorStatus(currentTest, testresultstatus);
	    }
	  }
	} 
	if( ! (*imod)->Results(currentTest)) {
	  DT_DEBUG_LTAPPL(" Can't det results from"
			  <<currentTest);
	}
      }
      if(thestatus->getErrorStatus(currentTest) != 0){
	feedBackSignal("finishedrun");
      }
      qApp->lock();
      thestatus->SetDone();
      qApp->unlock();
      return false;
    } // end alldone 
    else{
      if(fDaqUse) {
	thedaqctrl->NextSpill(); 
	DT_DEBUG_LTAPPL("asked for a new spill "); 
      }
      return true;
    }
}
  
  


/*
 * Abort current action
*/
void TestProcedure::Abort() {
  thestatus->Abort();
  // wait() for the test to stop?
  DT_NOTE(" Abort test.. "<<thestatus->currentTest());
  Tologfile(" Abort Test",thestatus->currentTest());
}
/*
 * select TSC card settings according to the test
*/


void TestProcedure::InterLockAction(){
  ChangeHV(0);
  
  for(vector<Module*>::const_iterator imod=themodules.begin();imod!=themodules.end();imod++) {
       (*imod)->switchPowerOn(false);
  }
  ChangeTemp(22);
  Abort();    
}

void TestProcedure::setapvctrl(const QString& s) {
  // update of tsc is done if latency is done
  int latency=300; // 300 is default 
  if(!thestatus->apvctrlOn())  {
    DT_DEBUG_LTAPPL("Procedure: setapvctrl for "<<s);
    if(s=="PedRun") { 
      thedaqctrl->enableCalSeq(false,false);
      thedaqctrl->disableReset(false);
    }
    if(s=="CalRun"){
      thedaqctrl->enableCalSeq(true,false);
      thedaqctrl->enableReset(false);
    }
    if(s=="CalProfRun"){
      thedaqctrl->enableCalSeq(true,false);
      thedaqctrl->enableReset(false);
    } 
    if(s=="LatRun"){
      thedaqctrl->enableCalSeq(false,false);
      thedaqctrl->enableReset(false);
    }
    if(s=="LedRun"){ 
      thedaqctrl->enableCalSeq(false,false);
      thedaqctrl->disableReset(false);
    }
    if(s=="LightRun"){ 
      thedaqctrl->enableCalSeq(false,false);
      thedaqctrl->disableReset(false);
      //thedaqctrl->setLightLevel(thesettings->getLigRunDescriptor("default")->lighintens)
      cout<<"Light level will  set "<<endl;
      thedaqctrl->setLightLevel(100);
      cout<<"Light level will  set "<<endl;
    }
    else {
      thedaqctrl->setLightLevel(0);
    }
    if(s=="BPlaneRun") {
      thedaqctrl->enableCalSeq(false,false);
      thedaqctrl->disableReset(false);
      thedaqctrl->enableBackPlanePulse();
      latency=thesettings->getBPlaneRunDescriptor("default")->tsclat;
    }
    else { 
      thedaqctrl->disableBackPlanePulse();
    }
    if(s=="FbRun") {
      thedaqctrl->enableCalSeq(false,false);
      thedaqctrl->enableReset(false);
       cout<<"Light level will  set "<<endl;
       thedaqctrl->setLightLevel(100);
       cout<<"Light level will  set "<<endl;   
    } 
    else {
      thedaqctrl->setLightLevel(0);
    }

   
    thedaqctrl->setTscLatency(latency);
  }
}
/*
 * Change HV, can take a while
 * for rs232 communication 
*/
void TestProcedure::ChangeHV(float hv, bool block)
{
  if(!thestatus->isInit() ) return;
  DT_NOTE("ChangeHV to "<<hv);
  for(vector<Module*>::iterator imod=themodules.begin();imod!=themodules.end();imod++)
    {    (*imod)->modulehv()->ChangeHV(hv,false);
    }
  if( !block)  return;

  unsigned int itry=0;
  bool hvreached=false;
  for ( unsigned int itry=0;itry<10;itry++) {
    hvreached= HVReached();
    if(hvreached) break;
  }
  if (!hvreached) {    DT_WARN("ChangeHv:  fail to set HV ");}
  thestatus->ResetAction();
  Tologfile("Change HV ",hv);
}

bool TestProcedure::HVReached(bool wait ){
  bool   allreached = true; 
  bool   checkwait=wait;
  for(vector<Module*>::iterator imod=themodules.begin();imod!=themodules.end();imod++) {  
    if ( !((*imod)->modulehv()->HvReached(checkwait)) ) { 
      DT_WARN(" present hv "<<(*imod)->modulehv()->getVbias(false) 
	       << "module" <<(*imod)->getModId());
      allreached = false ;
    }
    checkwait=false; // only the first module waits 
  }
  return allreached;
}


/*
 * change cooling temperature
 */
void TestProcedure::ChangeTemp(float temp)
{
   if(!thestatus->isInit() ) return;
  DT_NOTE("Procedure:ChangeTemp to "<<temp);
  thesetup->ChangeTemp(temp);
  therequestedtemp=temp;
  thestatus->ResetAction();
 Tologfile("Set Temperature ",temp);
}
/*
 * slowctltimer slot
 * update all X-t parameters
 * Temp. hum, current, voltages, averageRMS, etc 
 */
void TestProcedure::CheckEnv() {

  if(!thestatus->isInit() ) return;
   DT_DEBUG_LTAPPL(" CheckEnv: ask for xt ");
  if(!thestatus->isRunning()) {
    checkenvpending=false;
    thesetup->UpdateXt(); 
    DT_DEBUG_LTAPPL(" Check module env  "<<themodules.size());
    if(thesetup->maxHumidityReached()) {
      feedBackSignal("maxhumidityreached");
      InterLockAction();// same as for interrupt 
    }
    else {
      feedBackSignal("humidity_ok");
    }
 for(vector<Module*>::const_iterator imod=themodules.begin();imod!=themodules.end();imod++) {
       (*imod)->UpdateXt();
    }
    DT_DEBUG_LTAPPL(" CheckEnv done ");
    // Tologfile("CheckEnv");
  }  
  else { checkenvpending=true;}
}
/**/

/*
 * save Record in Root file
 */
void TestProcedure::SaveResults(const QString& s) {
  DT_DEBUG_LTAPPL("SaveResults into "<<s);
  if(s=="default") {
    rfname=thesettings->getProcedureDescriptor(theScType)->resultsfile();
    selectedrec.push_back(theNrecord);
  } 
  else{
    rfname=s;
  }
  float ftime=float(stime.secsTo(QDateTime::currentDateTime()));
  therecord->getheader()->settime(ftime); 
  DT_DEBUG_LTAPPL("SaveResults into "<<rfname<<" at time "<<ftime);
  int apvm=0;
  if(themodules.size()>0) apvm=themodules[0]->hybridi2c()->GetMode();
  therecord->getheader()->setapvmode(apvm);
  DT_DEBUG_LTAPPL("SaveResults: analyze ");  
  therecord->Analyze(thesettings->getFlagsDescriptor("default"));

  therecord->Save(rfname);
  theNrecord++;
  DT_NOTE("Save record in "<<s<<"  done");
  QString ss=s;
  Tologfile("Save results to ",ss);
}
//
void TestProcedure::SaveResults(const QString& s,QWidget* fov) {
  DT_DEBUG_LTAPPL("SaveResults1 into"<<s);
  if(s=="default") {
    rfname=thesettings->getProcedureDescriptor(theScType)->resultsfile();
    selectedrec.push_back(theNrecord); 
  } 
  else{
    rfname=s;
  }
  float ftime=float(stime.secsTo(QDateTime::currentDateTime()));
  DT_DEBUG_LTAPPL("SaveResults2 into "<<rfname<<" at time "<<ftime);
  therecord->getheader()->settime(ftime);
  int apvm=0;
  if(themodules.size()>0) apvm=themodules[0]->hybridi2c()->GetMode();
  therecord->getheader()->setapvmode(apvm);
  therecord->Analyze(thesettings->getFlagsDescriptor("default"));
  therecord->Save(rfname,fov);
  theNrecord++;
  DT_NOTE("Save record "<<theNrecord<<" in "<<s<<"  done");
  QString ss=s;
  Tologfile("Save results to ",ss);
}
/*
 *  save data to the Db xml file
 */
void  TestProcedure::SaveDb(const QString& s) {
// create DbParser
  RecordReader* recreader=new RecordReader();

  for(int i=0;i<themodules.size();i++)  { 
    DbParser* db=new DbParser(s);
    if(recreader->Read(therecord->getFileName(i),selectedrec)) {
//   if(recreader->Read(thesettings->getProcedureDescriptor(theScType)->resultsfile(),selectedrec)) { 

      DT_DEBUG_LTAPPL(" populate xml from "<<therecord->getFileName(i)<<" selsize "<<selectedrec.size() );
    db->populate(recreader); 
    db->write();
    delete db;
    }
  }
  delete recreader;

  DT_NOTE("Save to xml template "<<s<<"  done");
  QString ss=s;
  Tologfile("Save to xml template ",ss);
}
/**/
void TestProcedure::Tologfile(const char* s) {
  FILE* file;
  file=fopen("logfile","a");
  fprintf(file,"%s %s \n",QDateTime::currentDateTime().toString().latin1(),s);
  fclose(file);
}
void TestProcedure::Tologfile(const char* s, float f) {
  FILE* file;
  file=fopen("logfile","a");
  fprintf(file,"%s %s %f \n",QDateTime::currentDateTime().toString().latin1(),s,f);
  fclose(file);
}
void TestProcedure::Tologfile(const char*  s, const QString& c) {
  FILE* file;
  file=fopen("logfile","a");
   fprintf(file,"%s %s %s \n",QDateTime::currentDateTime().toString().latin1(),s,c.latin1());
  fclose(file);
}
