//
//Online DB Browser v01.02
//
#include <iostream.h>
//
#include <TROOT.h>
#include <TApplication.h>
#include <TGFrame.h>
#include <TRootEmbeddedCanvas.h>
#include <TGListBox.h>
#include <TGComboBox.h>
#include <TGButton.h>
#include <TGLabel.h>
#include <TG3DLine.h>
#include <TCanvas.h>
#include <TGraph.h>
#include <TH1.h>
#include <TProfile.h>
#include <TVector.h>
#include <TNamed.h>
#include <TSocket.h>
#include <TServerSocket.h>
#include <TMessage.h>
#include <TFile.h>
#include <TGFileDialog.h>
#include <TSystem.h>
#include "hmouse.h"

/**
 \Online DB Browser for CMS Test  

<p>
        Database browsing for CMS Test local Root database files 
	created by main monitor or online monitor. 
	Usage - using Root GUI widgets.
	Requires mouse event handler hmouse.
	
           
     \author V.Hovhannisyan,UIA   
     \version 1.0 
     \date 2002 9/27
 
*/

//
//Data items and types
//
  const int CHANS=128;
  const int COLORS=7; 
  const int HEADERN=9;
  const int SETUPN=16;
  const int MODN=50;
  const int NFILES=7;
//
  const char* header_n[HEADERN]={"operator","testcenter","ev",
				 "mod","run","time","apvmode",
				 "settings","stime"};
  const char* header_m[HEADERN]={"TN","TN","TV","TV","TV","TV","TV","TN","TN"};
  const char* setup_n[SETUPN]={"temptag","tempcold","tempbox","tempwarm",
			   "tempext","humext","humbox","temps","hums",
			   "temptag(t)","tempcold(t)","tempwarm(t)",
			   "tempbox(t)","tempext(t)","humbox(t)","humext(t)"};
  const char* setup_m[SETUPN]={"TV","TV","TV","TV","TV","TV",
			       "TV","TV","TV","TG","TG","TG",
			       "TG","TG","TG","TG" };
  const char* module_n[MODN]={"modid","modtype","dcuid",
			      "napvs","nbench","napverrors",
			      "temphybdcu1","temphybdcu2",
			      "tempsenntsc1","tempsenntsc2","hvbias",
			      "tileakdcu","itest","ihyb25","ihyb125",
			      "vhyb25","vhyb125","nbadch","connectivity",
			      "ivv","ivi","timet","it","avrmst","avcalat",
			      "hbadcht","temphybdcu1t","temphybdcu2t",
			      "temphybdcu3t","ihyb125t","ihyb25t",
			      "vt","it","tempsen1t","tempsen2t",
			      "iv","pedestal","noise","rawnoise",
			      "chflag","cala","calPApeak",
			      "calSbackpl","calSled","bp","led",
			      "cmn","pipe","calprof","latency" };
  const char* module_m[MODN]={"TN","TN","TN","TV","TV","TV",
			      "TV","TV","TV","TV","TV",
			      "TV","TV","TV","TV","TV","TV","TV","TV","TV",
			      "TV","TV","TV","TG","TG","TG","TG","TG","TG",
			      "TG","TG","TG","TG","TG","TG","TG","TH","TH",
			      "TH","TH","TH","TH","TH","TH","TP","TP","TH",
			      "TH","TP","TH" };
  const char* filetypes[]={"CMS Test DB Files","*_M_*.root",0,0,0,0,0,0};
  const int color_m[COLORS]={ 18,5,4,25,41,38,2 };
  const char* wname="Online DB Browser v01.04/CMS.UIA ";
//
//Online Monitor Receive Data
//
class OnlineMonitorReceive {

  public:

  OnlineMonitorReceive(TGMainFrame* d,TGListBox* st);
  bool ReceiveData(int dn, int &nnmod, int &napvs);

  private:

  void SetStatus(const char* s);

  TGMainFrame* fd;
  TGListBox* fst;
  char fip[80];
  int ent;

};

OnlineMonitorReceive::OnlineMonitorReceive(TGMainFrame* d,TGListBox* st)
{
  char str0[80];
  char* str=getenv("LTONMONIP");
  if (str==0) str="127.0.0.1";
  fd=d;fst=st;strcpy(fip,str); 
  sprintf(str0,"Iaddress: %s",fip);
  SetStatus(str0);
  ent++;
}

bool OnlineMonitorReceive::ReceiveData(int dn, int &nnmod, int &napvs)
{
  char str[32],str0[32];
  TFile* tf;
  TVector* tvv=new TVector();
//
//Make connection
//
  SetStatus("waiting for response...");
  ent++;
  TSocket *s;
  int maxcnt=5000;
  int i,k;
  int cnt=0;
  do
    {
      gClient->ProcessEventsFor(fd);
      s=new TSocket(fip,9010);
      s->Recv(str,32);
      cnt++;
    } 
  while (strcmp(str,"OnlineMonitor")!=0 && cnt<=maxcnt);
  if (cnt>maxcnt)
    {
      SetStatus("No response");
      return false;
    }
  SetStatus("Connection established");
  ent++;
//
//Receive number of modules 
//
  TMessage *msg=new TMessage();
  s->Recv(msg);if (msg->What()==kMESS_OBJECT) 
    tvv=(TVector*)msg->ReadObject(msg->GetClass());
  nnmod=(int)(*tvv)(0);
  sprintf(str,"Number of modules -> %d",nnmod);
  SetStatus(str);
  ent++;
//
//Receive data for i-st module
// 
 for(i=0;i<nnmod;i++)
  { 
//
//Open file
//
    sprintf(str,"Module #%d",i);
    SetStatus(str);
    ent++;
    sprintf(str0,"Update_%d_Module_%d.root",dn,i);
    tf=new TFile(str0,"RECREATE","UIA, Antwerpen");
//
//Receive and save the header data
//
    tf->cd();
    tf->mkdir("Header");
    tf->cd("Header");
    for (k=0;k<HEADERN;k++)
      { 
	strcpy(str,header_m[k]);strcat(str,header_n[k]);
	s->Recv(msg);if (msg->What()==kMESS_OBJECT) 
	  msg->ReadObject(msg->GetClass())->Write(str);
      }
    SetStatus("Header received");
    ent++;
//
//Receive and save setup data
//
    tf->cd();
    tf->mkdir("Setup");
    tf->cd("Setup"); 
    for (k=0;k<SETUPN;k++)
      {
	strcpy(str,setup_m[k]);strcat(str,setup_n[k]);
	s->Recv(msg);if (msg->What()==kMESS_OBJECT) 
	  msg->ReadObject(msg->GetClass())->Write(str);
      }
    SetStatus("Setup received");
    ent++;
//
//Receive and save module data
//
    tf->cd();
    TDirectory* dmod=tf->mkdir("Module");
    dmod->cd();
    for (k=0;k<MODN-4;k++)
      {
	strcpy(str,module_m[k]);strcat(str,module_n[k]);
	if (k==3) 
	  {
	    s->Recv(msg);if (msg->What()==kMESS_OBJECT) 
		tvv=(TVector*)msg->ReadObject(msg->GetClass());
	    tvv->Write(str);
	    napvs=(int)(*tvv)(0);
	    sprintf(str,"Number of APVs -> %d",napvs); 
	    SetStatus(str);
	    ent++;
	  }
	else
	  {
	    s->Recv(msg);if (msg->What()==kMESS_OBJECT)
	      msg->ReadObject(msg->GetClass())->Write(str);
	  } 
      }   
     TDirectory* dcmn=dmod->mkdir("cmn");
     TDirectory* dpip=dmod->mkdir("pipe");
     TDirectory* dcal=dmod->mkdir("calprof");
     TDirectory* dlat=dmod->mkdir("latency");
     for (int k=0;k<napvs;k++)
	{
	  dcmn->cd();
	  sprintf(str,"%s%s_%d",module_m[MODN-4],module_n[MODN-4],k);
	  s->Recv(msg);if (msg->What()==kMESS_OBJECT) 
	    msg->ReadObject(msg->GetClass())->Write(str);
	  dpip->cd();
	  sprintf(str,"%s%s_%d",module_m[MODN-3],module_n[MODN-3],k);
	  s->Recv(msg);if (msg->What()==kMESS_OBJECT) 
	    msg->ReadObject(msg->GetClass())->Write(str);
	  for (int j=0;j<CHANS;j++)
	    {
	      int l=CHANS*k+j;
	      dcal->cd();
	      sprintf(str,"%s%s_%d",module_m[MODN-2],module_n[MODN-2],l);
	      s->Recv(msg);if (msg->What()==kMESS_OBJECT) 
		msg->ReadObject(msg->GetClass())->Write(str); 
	      dlat->cd();
	      sprintf(str,"%s%s_%d",module_m[MODN-1],module_n[MODN-1],l);
	      s->Recv(msg);if (msg->What()==kMESS_OBJECT) 
		msg->ReadObject(msg->GetClass())->Write(str);
	    }
	}      
     SetStatus("Module received");
     ent++;
     tf->Close();
     delete tf;
    }
  s->Recv(msg);if (msg->What()==kMESS_STRING) msg->ReadString(str,32);
  SetStatus(str);
  ent++;
  s->Close();
  delete msg; 
  delete s;
  return true;
}

void OnlineMonitorReceive::SetStatus(const char* s)
{
  fst->AddEntry(s,ent);
  fst->MapSubwindows();
  fst->Layout();
  if (ent>0) fst->SetTopEntry(ent-1);
  gClient->ProcessEventsFor(fd);
}
//
//Online Monitor Receive - end
//
//Client GUI
//
class OnlineDBBrowser : public TGMainFrame  {

  public:

  OnlineDBBrowser(const TGWindow *p,UInt_t w,UInt_t h);
  ~OnlineDBBrowser();
  virtual Bool_t ProcessMessage(Long_t msg,Long_t parm1, Long_t parm2);
  virtual void CloseWindow();

  private:

  TGCompositeFrame* f1;
  TGLayoutHints* l1;
  TGListBox* header,*setup,*module,*valuebox,*statusbox; 
  TGComboBox* nmodule,*napv,*nchan,*nupdate,*nrecord;
  TGCheckButton* redraw,*increment;
  TGRadioButton* pad11,*pad22;
  TGTextButton* gclear,*update,*quit,*dfile,*savegif;
  TGLabel* header_l,*setup_l,*module_l,*nmodule_l,*napv_l,*nchan_l;
  TGLabel* value_l,*status_l;
  TGHorizontal3DLine* h1,*h2,*h3,*h4,*h5;
  TGVertical3DLine* v1,*v2;
  TGFileInfo fi;
  TRootEmbeddedCanvas* canr;
  TCanvas* canv;
  TPad* pad1,*pad2;
  TGraph* gr;
  TFile* datafile[NFILES+1];
  TDirectory* topdir;
  OnlineMonitorReceive* omr;
  hmouse* hm;
 //
  int dn,nmods,napvs,nchans,nrecs;
  int cdn,cmod,capv,cchan,cnheader,cnsetup,cnmodule,cnfile,cnrec;
  int valentry,npad;
  int ccolor,ncolor;
  bool incrementing,redrawing,updateflag,dbfileflag;
  const char* volume,*citem;
  char citemt[32];
  int recs[10];
//
  void ShowData();
  void ResetValues();
  void SetValues(const char* val);
  void SetValues(float val);
  void ResumeValues();
  void GetUpdate();
  void SetUpdate();
  void ResetUpdate();
  void OpenUpdateFile();
  void GetDBFile();
  void ResetModApv();
  void SetModApv();
  void ClearPad();
  void Draw(TNamed* n);
  void Draw(TVector* v);
  void Draw(TGraph* g);
  void Draw(TH1F* h);
  void Draw(TProfile* p);
  void ChangeColor();
  void SaveGIF();
  void SetRecord();
  void ResetRecord();
};

OnlineDBBrowser::OnlineDBBrowser(const TGWindow *p,UInt_t w,UInt_t h) 
: TGMainFrame(p,w,h)
{
  int i;
  char str[32];
  dn=0;
  nmods=1;
  napvs=1;
  nchans=128;
  nrecs=1;
  cdn=1;
  cmod=0;
  capv=0;
  cchan=0;
  cnrec=0;
  redrawing=false;
  incrementing=false;
  updateflag=false;
  dbfileflag=false;
  valentry=1;
  npad=2;
  ncolor=0;
  ccolor=color_m[ncolor];
  cnfile=0;
//
//Main dialog window
//
  f1=new TGCompositeFrame(this,600,410);  
  SetWindowName(wname);
  SetIconPixmap("h1_t.xpm");
  l1=new TGLayoutHints(kLHintsTop | kLHintsLeft);
//
//Canvas
//
  canr=new TRootEmbeddedCanvas("CMS/Test",f1,370,400);
  f1->AddFrame(canr,l1);
  canv=canr->GetCanvas();
  canv->SetEditable(kFALSE);
  canv->SetBit(kNoContextMenu);
//
//Pads
//
  pad1=new TPad("pad1","data",0.01,0.01,0.99,0.495,0);
  pad2=new TPad("pad2","data",0.01,0.5,0.99,0.99,0);
  pad1->SetGrid();
  pad2->SetGrid();
  pad1->Draw();
  pad2->Draw();
  gr=new TGraph();
  gr->Set(5);
  gr->SetTitle("CMS Test / UIA Antwerpen");
  pad1->cd();
  gr->Draw("AL");
  pad2->cd();
  gr->Draw("AL");
  canv->Update();
//
//Data List boxes
//  
  header=new TGListBox(f1,1);
  setup=new TGListBox(f1,2);    
  module=new TGListBox(f1,3);
  header->Associate(this);
  setup->Associate(this);
  module->Associate(this);
  f1->AddFrame(header,l1);
  f1->AddFrame(setup,l1);
  f1->AddFrame(module,l1);
  header->Resize(110,67);
  setup->Resize(110,67);
  module->Resize(110,67);
  for (i=0;i<=HEADERN-1;i++) header->AddEntry(header_n[i],i);
  for (i=0;i<=SETUPN-1;i++) setup->AddEntry(setup_n[i],i);
  for (i=0;i<=MODN-1;i++) module->AddEntry(module_n[i],i);
//
//Output List Box
//
  valuebox=new TGListBox(f1,4);
  valuebox->Associate(this);
  f1->AddFrame(valuebox,l1);
  valuebox->Resize(110,50);
//
//Status List Box
//
  statusbox=new TGListBox(f1,5);
  statusbox->Associate(this);
  f1->AddFrame(statusbox,l1);
  statusbox->Resize(200,45);
//
//Combo boxes
//
  nmodule=new TGComboBox(f1,1);
  napv=new TGComboBox(f1,2);
  nchan=new TGComboBox(f1,3);
  nupdate=new TGComboBox(f1,4);
  nrecord=new TGComboBox(f1,5);
  nmodule->Associate(this);
  napv->Associate(this);
  nchan->Associate(this);
  nupdate->Associate(this);
  nrecord->Associate(this);
  f1->AddFrame(nmodule,l1);
  f1->AddFrame(napv,l1);
  f1->AddFrame(nchan,l1);
  f1->AddFrame(nupdate,l1);
  f1->AddFrame(nrecord,l1);
  nmodule->Resize(50,20);
  napv->Resize(50,20);
  nchan->Resize(50,20);
  nupdate->Resize(60,20);
  nrecord->Resize(60,20);
  nmodule->AddEntry("0",0);
  nrecord->AddEntry("no file",0);
  napv->AddEntry("0",0);
  for (i=0;i<nchans;i++) 
    { 
      sprintf(str,"%d",i);
      nchan->AddEntry(str,i);
    }
  nupdate->AddEntry("no update",0);
  nmodule->Select(0);
  napv->Select(0);
  nchan->Select(0);
  nupdate->Select(0);
  nrecord->Select(0);
//
//Check Buttons
//
  redraw=new TGCheckButton(f1,"Same",1);
  increment=new TGCheckButton(f1,"Increment",2);
  redraw->Associate(this);
  increment->Associate(this);
  f1->AddFrame(redraw,l1);
  f1->AddFrame(increment,l1);
//
//Radio Buttons
//
  pad11=new TGRadioButton(f1,"Upper",1);
  pad22=new TGRadioButton(f1,"Lower",2);
  pad11->Associate(this);
  pad22->Associate(this);
  f1->AddFrame(pad11,l1);
  f1->AddFrame(pad22,l1);
  pad11->SetState(kButtonDown);
//
//Text Buttons
//
  gclear=new TGTextButton(f1,"Clear",1);
  update=new TGTextButton(f1,"Update",2);
  quit=new TGTextButton(f1,"Quit",4);
  dfile=new TGTextButton(f1,"DB File",3);
  savegif=new TGTextButton(f1,"Save PS",5);
  gclear->Associate(this);
  update->Associate(this);
  quit->Associate(this);
  dfile->Associate(this);
  savegif->Associate(this);
  f1->AddFrame(gclear,l1);
  f1->AddFrame(update,l1);
  f1->AddFrame(quit,l1);
  f1->AddFrame(dfile,l1);
  f1->AddFrame(savegif,l1);
//
//Labels
//
  header_l=new TGLabel(f1,"Header");
  setup_l=new TGLabel(f1,"Setup");
  module_l=new TGLabel(f1,"Module");
  nmodule_l=new TGLabel(f1,"Module");
  napv_l=new TGLabel(f1,"APV");
  nchan_l=new TGLabel(f1,"Channel");
  value_l=new TGLabel(f1,"Value(s)");
  status_l=new TGLabel(f1,"Status");
  f1->AddFrame(header_l,l1);
  f1->AddFrame(setup_l,l1);
  f1->AddFrame(module_l,l1);
  f1->AddFrame(nmodule_l,l1);
  f1->AddFrame(napv_l,l1);
  f1->AddFrame(nchan_l,l1);
  f1->AddFrame(value_l,l1);
  f1->AddFrame(status_l,l1);
//
//Lines
//
  h1=new TGHorizontal3DLine(f1,95,2); 
  h2=new TGHorizontal3DLine(f1,95,2); 
  h3=new TGHorizontal3DLine(f1,95,2); 
  h4=new TGHorizontal3DLine(f1,95,2); 
  h5=new TGHorizontal3DLine(f1,95,2); 
  v1=new TGVertical3DLine(f1,2,297); 
  v2=new TGVertical3DLine(f1,2,297); 
//
//Layout 
//
  f1->Layout();
  f1->MapSubwindows();
  canr->MapSubwindows();
  canr->Layout();
  MapSubwindows();
  Layout();
//
//Place widgets
//
  canr->Move(225,5);
  header->Move(10,210);
  setup->Move(10,135);
  module->Move(10,60);
  valuebox->Move(10,285);
  statusbox->Move(10,360);
  nmodule->Move(10,20);
  napv->Move(65,20);
  nchan->Move(120,20);
  nupdate->Move(140,205);
  nrecord->Move(140,264);
  module_l->Move(10,44);
  setup_l->Move(10,119);
  header_l->Move(10,194);
  nmodule_l->Move(10,4);
  value_l->Move(10,269);
  status_l->Move(10,343);
  napv_l->Move(65,4);
  nchan_l->Move(120,4);
  h1->Move(124,55);
  h2->Move(124,180);
  h3->Move(124,259);
  h4->Move(124,315);
  h5->Move(124,350);
  v1->Move(124,55);
  v2->Move(218,55);
  redraw->Move(140,60);
  increment->Move(140,185);
  pad11->Move(140,80);
  pad22->Move(140,100);
  gclear->MoveResize(140,120,60,25);
  savegif->MoveResize(140,150,60,25);
  update->MoveResize(140,230,60,25);
  dfile->MoveResize(140,287,60,25);
  quit->MoveResize(140,320,60,25);
//
//Initialize
//
  hm=new hmouse(canv,this,valuebox);
  omr=new OnlineMonitorReceive(this,statusbox);
  for (int i=1;i<=NFILES;i++) datafile[i]=new TFile();
  valuebox->MapSubwindows();
  valuebox->Layout();
  statusbox->MapSubwindows();
  statusbox->Layout();
//
  MapWindow();
}

Bool_t OnlineDBBrowser::ProcessMessage(Long_t msg,Long_t parm1,Long_t parm2)
{
  Int_t m=GET_MSG(msg);
  if (m==kC_COMMAND)
    {
      Int_t n=GET_SUBMSG(msg);
//
//Buttons
//
        if (n==kCM_BUTTON)
	  {
	    if (parm1==2) GetUpdate();
	    if (parm1==4) CloseWindow();
	    if (parm1==1) ClearPad();
	    if (parm1==3) GetDBFile();
	    if (parm1==5) SaveGIF();
	  }
//
//Check Buttons
//
	if (n==kCM_CHECKBUTTON)
	  {
	    if (parm1==1) redrawing=!redrawing;
	    if (parm1==2) incrementing=!incrementing;
	  }
//
//Radio Buttons
//
	if (n==kCM_RADIOBUTTON)
	  {
	    if (redrawing)
	      {
		redrawing=false;
		redraw->SetState(kButtonUp);
	      }
	    if (parm1==1) { pad22->SetState(kButtonUp);pad2->cd();npad=2; }
	    if (parm1==2) { pad11->SetState(kButtonUp);pad1->cd();npad=1; }  
	  }
//
//List Boxes
//
	if (n==kCM_LISTBOX)
	  {
	    if (parm1==1) 
	      {
		volume="Header";
		cnheader=header->GetSelected();
		if (cnheader!=-1)
		  {
		    citem=header_n[cnheader];
		    strcpy(citemt,header_m[cnheader]);
		    ShowData();
		  }
	      }
	    if (parm1==2) 
	      {
		volume="Setup";
		cnsetup=setup->GetSelected();
		if (cnsetup!=-1)
		  {
		    citem=setup_n[cnsetup];
		    strcpy(citemt,setup_m[cnsetup]);
		    ShowData();
		  }
	      }
	    if (parm1==3) 
	      {
		volume="Module";
		cnmodule=module->GetSelected();
		if (cnmodule!=-1)
		  {
		    citem=module_n[cnmodule];
		    strcpy(citemt,module_m[cnmodule]);
		    ShowData();
		  }
	      }
	  }
//
//Combo Boxes
//
	  if (n==kCM_COMBOBOX)
	    {
	      if (parm1==1) 
		{ 
		  cmod=parm2;
		  if (dn!=0 && updateflag) OpenUpdateFile(); 
		}
	      if (parm1==2) capv=parm2;
	      if (parm1==3) cchan=parm2;
	      if (parm1==4) 
		{ 
		  cdn=parm2;
		  if (dn!=0 && updateflag) OpenUpdateFile(); 
		}
	      if (parm1==5) 
		{
		  cnrec=parm2;
		  if (dbfileflag) SetRecord(); 
		}
	    }
    }
  return true;
}

void OnlineDBBrowser::ShowData()
{
  char str[32]; 
  TDirectory* dir1,*dir2;
  int j,k,kk,mx;
  double m1,max;
  TH1F* h;
  if (dbfileflag || updateflag)
    {
      strcpy(str,citemt);
      strcat(str,citem);
      dir1=(TDirectory*)topdir->Get(volume);
      if (strcmp(citem,"cmn")!=0 &&
	  strcmp(citem,"pipe")!=0 &&
	  strcmp(citem,"calprof")!=0 &&
	  strcmp(citem,"latency")!=0)
	{
	  if (strcmp(citemt,"TN")==0) Draw((TNamed*)dir1->Get(str));
	  if (strcmp(citemt,"TV")==0) Draw((TVector*)dir1->Get(str));
	  if (strcmp(citemt,"TG")==0) Draw((TGraph*)dir1->Get(str)); 
	  if (strcmp(citemt,"TH")==0) Draw((TH1F*)dir1->Get(str)); 
	  if (strcmp(citemt,"TP")==0) Draw((TProfile*)dir1->Get(str));
	}
      else
	{
	  k=capv*CHANS;
	  if (strcmp(citem,"cmn")==0)
	    {
	      dir2=(TDirectory*)dir1->Get(module_n[MODN-4]);
	      sprintf(str,"%s%s_%d",module_m[MODN-4],module_n[MODN-4],capv);
	      Draw((TH1F*)dir2->Get(str));
	    }
	  if (strcmp(citem,"pipe")==0)
	    {
	      dir2=(TDirectory*)dir1->Get(module_n[MODN-3]);
	      sprintf(str,"%s%s_%d",module_m[MODN-3],module_n[MODN-3],capv);
	      Draw((TH1F*)dir2->Get(str));
	    }
	  if (strcmp(citem,"calprof")==0)
	    {
	    dir2=(TDirectory*)dir1->Get(module_n[MODN-2]);
	      if (redrawing) 
		{
		  sprintf(str,"%s%s_%d",module_m[MODN-2],module_n[MODN-2],k);
		  h=(TH1F*)dir2->Get(str);
		  max=h->GetMaximum();
		  mx=0;
		  for (j=k+1;j<k+CHANS-1;j++)
		    {
		      sprintf(str,"%s%s_%d",
			      module_m[MODN-2],module_n[MODN-2],j);
		      h=(TH1F*)dir2->Get(str);
		      m1=h->GetMaximum();
		      if (m1>=max) { mx=j;max=m1; }
		    }
		  sprintf(str,"%s%s_%d",
			  module_m[MODN-2],module_n[MODN-2],k+mx);
		  cout << "MX=" << mx << endl;
		  h=(TH1F*)dir2->Get(str);
		  h->Draw("c");
		  for (j=k;j<k+CHANS;j++)
		    {
		      sprintf(str,"%s%s_%d",
			      module_m[MODN-2],module_n[MODN-2],j);
		      h=(TH1F*)dir2->Get(str);
		      h->Draw("csame");  
		    }
		}
	      else
		{
		  sprintf(str,"%s%s_%d",
			  module_m[MODN-2],module_n[MODN-2],k+cchan);
		  Draw((TH1F*)dir2->Get(str));
		}
	    }
	  if (strcmp(citem,"latency")==0)
	    {
	      dir2=(TDirectory*)dir1->Get(module_n[MODN-1]);
	      sprintf(str,"%s%s_%d",module_m[MODN-1],module_n[MODN-1],k+cchan);
	      Draw((TProfile*)dir2->Get(str));
	    }
	}
      canv->Update();
    }
}

void OnlineDBBrowser::Draw(TNamed* n)
{ 
  ResetValues();
  SetValues(citem);
  SetValues(n->GetName());
  ResumeValues();
}

void OnlineDBBrowser::Draw(TVector* v)
{ 
  ResetValues();
  SetValues(citem);
  int n=v->GetUpb();
  for (int k=0;k<=n;k++) SetValues((*v)(k));
  ResumeValues();
}

void OnlineDBBrowser::Draw(TGraph* g)
{
  char str[32];
  if (redrawing)
    { 
      g->SetLineColor(ccolor);
      g->Draw("LP");
      ChangeColor(); 
    }
  else
    {
      if (npad==1) pad1->Clear();
      if (npad==2) pad2->Clear();
      g->SetLineColor(1);
      g->Draw("ALP");
    }
  ResetValues();
  SetValues(citem);
  double* x=g->GetX();
  double* y=g->GetY();
  int n=g->GetN();
  sprintf(str,"N=%d",n);
  SetValues(str);
  for (int i=1;i<=n;i++) 
    {
      sprintf(str,"point #%d",i);
      SetValues(str);
      sprintf(str,"X:%f",*x++);
      SetValues(str);
      sprintf(str,"Y:%f",*y++);
      SetValues(str);
    }
  ResumeValues();
}

void OnlineDBBrowser::Draw(TH1F* h)
{
  char str[32];
  if (redrawing) 
    { 
      h->SetLineColor(ccolor);
      h->Draw("same"); 
      ChangeColor();
    }
  else 
    { 
      h->SetLineColor(1);
      h->Draw(); 
    }
  ResetValues();
  SetValues(citem);
  int n=h->GetNbinsX();
  sprintf(str,"nbinsx=%d",n);
  SetValues(str);
  for (int i=1;i<n;i++)
    {
      sprintf(str,"#%d:%f",i,h->GetBinContent(i));
      SetValues(str);
    }
  ResumeValues();
}

void OnlineDBBrowser::Draw(TProfile* p)
{ 
  char str[32];
  if (redrawing) 
    { 
      p->SetLineColor(ccolor);
      p->Draw("same"); 
      ChangeColor();
    }
  else 
    { 
      p->SetLineColor(1);
      p->Draw(); 
    }
  ResetValues();
  SetValues(citem);
  int n=p->GetNbinsX();
  sprintf(str,"nbinsx=%d",n);
  SetValues(str);
  for (int i=1;i<n;i++)
    {
      sprintf(str,"#%d:%f",i,p->GetBinContent(i));
      SetValues(str);
    }
  ResumeValues();
}

void OnlineDBBrowser::ChangeColor()
{
  ncolor++;
  if (ncolor>=COLORS) ncolor=0;
  ccolor=color_m[ncolor];
}

void OnlineDBBrowser::GetDBFile()
{
  char str[300];
  char* dir=".";
  TDirectory* dd;
  int i;
  fi.fFileTypes=filetypes;
  fi.fIniDir=StrDup(dir);
  new TGFileDialog(gClient->GetRoot(),this,kFDOpen,&fi);
  nrecord->RemoveEntries(0,nrecord->GetListBox()->GetNumberOfEntries());
  ResetUpdate();
  dir=fi.fIniDir;
  if (fi.fFilename!=NULL)
    {
      strcpy(str,wname);
      strcat(str,fi.fFilename);
      SetWindowName(str);
      cnfile++;
      if (cnfile>NFILES) cnfile=1;
      if (datafile[cnfile]->IsOpen()) datafile[cnfile]->Close();
      datafile[cnfile]=new TFile(fi.fFilename,"READ");
      nrecs=0;
      for (i=0;i<10;i++) 
	{
	  sprintf(str,"Record%d",i);
	  dd=(TDirectory*)datafile[cnfile]->Get(str);
	  if (dd) 
	    { 
	      recs[nrecs]=i;
	      sprintf(str,"Rec%d",recs[nrecs]);
	      nrecord->AddEntry(str,nrecs); 
	      nrecs++;
	    }
	}
      cnrec=0;
      nrecord->Select(0);
      SetRecord();
      dbfileflag=true;      
    }
}

void OnlineDBBrowser::SetRecord()
{
  char str[80];
  sprintf(str,"Record%d",recs[cnrec]);
  topdir=(TDirectory*)datafile[cnfile]->Get(str); 
  TDirectory* dd=(TDirectory*)topdir->Get("Module");
  TVector* vv=(TVector*)dd->Get("TVnapvs");
  napvs=(int)(*vv)(0);
  nmods=1;
  SetModApv();
}

void OnlineDBBrowser::ResetRecord()
{
  nrecord->RemoveEntries(0,nrecord->GetListBox()->GetNumberOfEntries());
  nrecord->AddEntry("no file",0);
  nrecord->Select(0);
  SetWindowName(wname);
  ResetModApv();
  dbfileflag=false;
}

void OnlineDBBrowser::SetUpdate()
{
  char str[32];
  if (dn==0) nupdate->RemoveEntry(0);
  dn++;
  sprintf(str,"%d",dn);
  nupdate->AddEntry(str,dn);
  nupdate->Select(dn);
  cdn=dn;
}

void OnlineDBBrowser::ResetUpdate()
{
  nupdate->RemoveEntries(0,nupdate->GetListBox()->GetNumberOfEntries());
  nupdate->AddEntry("no update",0);
  nupdate->Select(0);
  SetWindowName(wname);
  ResetModApv();
  dn=0;
  cdn=dn;
  updateflag=false;
}

void OnlineDBBrowser::ResetModApv()
{
  nmodule->RemoveEntries(0,nmodule->GetListBox()->GetNumberOfEntries());
  napv->RemoveEntries(0,napv->GetListBox()->GetNumberOfEntries());
}

void OnlineDBBrowser::SetModApv()
{
  int i;
  char str[32];
  for (i=0;i<nmods;i++) 
    {
      sprintf(str,"%d",i);
      nmodule->AddEntry(str,i);
    }
  nmodule->MapSubwindows();
  nmodule->Layout();
  for (i=0;i<napvs;i++)
    {
      sprintf(str,"%d",i);
      napv->AddEntry(str,i);
    }
  napv->MapSubwindows();
  napv->Layout();
}

void OnlineDBBrowser::ResetValues()
{
  valuebox->RemoveEntries(1,valuebox->GetNumberOfEntries());
  valentry=1;
}
 
void OnlineDBBrowser::SetValues(float val)
{
  char str[32];
  sprintf(str,"%f",val);
  valuebox->AddEntry(str,valentry);
  valentry++;
}

void  OnlineDBBrowser::SetValues(const char* val)
{
  valuebox->AddEntry(val,valentry);
  valentry++;
}

void OnlineDBBrowser::ResumeValues()
{
  valuebox->SetTopEntry(1);
  valuebox->MapSubwindows();
  valuebox->Layout();
}

void OnlineDBBrowser::GetUpdate()
{
  ResetRecord();
  nupdate->RemoveEntries(0,nupdate->GetListBox()->GetNumberOfEntries());
  statusbox->RemoveEntries(0,statusbox->GetNumberOfEntries());
  statusbox->MapSubwindows();
  statusbox->Layout();
  if (incrementing || dn==0) SetUpdate();
  ResetValues();
  updateflag=omr->ReceiveData(dn,nmods,napvs);
  if (updateflag) 
    { 
      SetModApv(); 
      OpenUpdateFile();
    } 
  else { ResetUpdate(); }
}

void OnlineDBBrowser::OpenUpdateFile()
{
  char str[32],str1[80];
  cnfile++;
  if (cnfile>NFILES) cnfile=1;
  sprintf(str,"Update_%d_Module_%d.root",cdn,cmod);
  if (datafile[cnfile]->IsOpen()) datafile[cnfile]->Close();
  datafile[cnfile]=new TFile(str,"READ");
  topdir=(TDirectory*)datafile[cnfile];
  strcpy(str1,wname);
  strcat(str1,str);
  SetWindowName(str1);
  gClient->ProcessEventsFor(this);
}

void OnlineDBBrowser::ClearPad()
{
  if (redrawing)
    {
      redrawing=false;
      redraw->SetState(kButtonUp);
    }
  if (npad==1) pad1->Clear();
  if (npad==2) pad2->Clear(); 
  gr->SetLineColor(1);
  gr->Draw("AL"); 
  canv->Update();
}

void OnlineDBBrowser::SaveGIF()
{
  char str[32];
  if (npad==1) 
    { 
     pad1->SaveAs(); 
     strcpy(str,"PS file pad1.ps created");
    }
   else 
    { 
      pad2->SaveAs(); 
      strcpy(str,"PS file pad2.gif created");
    }
  statusbox->RemoveEntry(0);
  statusbox->AddEntry(str,0);
  statusbox->SetTopEntry(0);
  statusbox->MapSubwindows();
  statusbox->Layout();
}

void OnlineDBBrowser::CloseWindow()
{
  gApplication->Terminate(0);
}

OnlineDBBrowser::~OnlineDBBrowser() 
{
  delete gr;
  delete l1;
  delete header;
  delete setup;
  delete module;
  delete nmodule;
  delete napv;
  delete nchan;
  delete nrecord;
  delete valuebox;
  delete statusbox;
  delete module_l;
  delete setup_l;
  delete header_l;
  delete value_l;
  delete nmodule_l;
  delete napv_l;
  delete nchan_l;
  delete h1;
  delete h2;
  delete h3;
  delete h4;
  delete h5;
  delete v1;
  delete v2;
  delete nupdate;
  delete redraw;
  delete increment;
  delete pad11;
  delete pad22;
  delete gclear;
  delete update;
  delete quit;
  delete savegif;
  delete canr;
  delete canv;
  delete datafile;
  delete omr;
  delete hm;
}
//
//Online DB Browser Main 
//
int main(int argc,char **argv)
{
  TApplication theapp("App",&argc,argv);
  OnlineDBBrowser mainWindow(gClient->GetRoot(),600,410);
  theapp.Run();
  return 0;
}
//Online DB Browser - end 
